<?php
namespace Controller;

use Service\ConnectionService;
use Service\DeleteCascadeService;
use Util\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Util\Utilities;

class ApiUsageLog extends Crud  {

    protected $dataSourceName = 'apiUsageLog';
    /** @var \Service\Manager\Mongo\Manager  */
    protected $organizationMongoManager;
    /** @var Environment */
    protected $environment;

    public function __construct(\Service\Manager\Mongo\Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, Environment $environment) {
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->organizationMongoManager = $organizationMongoManager;
        $this->environment = $environment;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $sort = $this->mongoManager->processSort($filterData);
        if(empty($sort)){
            $sort = array("CREATED_AT" => -1 );
        }
        $rows = $this->mongoManager->findByPagination($filterData,$sort)->getRows();
        foreach ($rows as &$item) {
            if(!empty($item['ERROR'])){
                if(!empty(\Service\Exception::LABEL_ERROR_API[$item['ERROR']])) {
                    $item['STATUS'] = \Service\Exception::LABEL_ERROR_API[$item['ERROR']];
                } else {
                    $item['STATUS'] = "Erro desconhecido";
                }
                if (!empty($item['PARAM_ERROR'])){
                    foreach ($item['PARAM_ERROR'] as $key => $string){
                        $item['STATUS'] = str_replace("__" . $key . "__", $string, $item['STATUS']);
                    }
                }
            } else if(!empty($item['STATUS_API'])){
                $item['STATUS'] = $item['STATUS_API'];
            }
            if($item['CONCLUDED'] == 'N') {
                if(empty($item['IS_ERROR']) || $item['IS_ERROR'] == 'N') {
                    $item['STOP_READY'] = 'Y';
                }
            }
        }
        $dataSet = new DataSet($this->dataSourceName, $rows);
        Utilities::datasetMongoToZeedhi($response, $dataSet,$this->dataSourceName);
    }


    public function stopApi(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $idLog = $row['_id'];
            /** @var \Service\Api $apiService */
            $apiService = InstanceManager::getInstance()->getService('\Service\Api');
            $apiService->stopApi($this->organizationMongoManager,$this->environment, $idLog);
        } catch(\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}
