<?php
namespace Controller;

use Util\ApiDataset;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Response;
use function React\Promise\all;

class ExternalApi {

    public function findDataSet(DTO\Request $request, DTO\Response $response){
	    try{
            list($apiId, $page, $filter) = $this->getValuesFindDataset($request);
            /** @var \Service\Api $apiService */
            $apiService = InstanceManager::getInstance()->getService('\Service\Api');
            $data = $apiService->findDataSet($apiId,$filter,$page);
            $length = count($data);
            $response->addDataSet(new DataSet('result', $data));
            $response->addDataSet(new ApiDataset('length', $length));
            $response->addDataSet(new ApiDataset('haveMorePages', $length == \Service\Api::PAGE_SIZE));
        }catch (\Exception $e){
            throw  $e;
        }

    }

    public function downloadAPiDocument(){
        try{
            $baseUrlApi = Utilities::getUrlAPiDocument();
            $type = explode('.',$baseUrlApi);
            $type = $type[count($type)-1];
            $name = "ApiDocument.$type";
            header('Content-Disposition: attachment; filename="' . $name.'"');
            readfile($baseUrlApi);
        }catch (\Exception $e){
            echo $e->getMessage().' Code: '.$e->getCode();
            throw $e;
        }


    }

    public function findBigDataSetApi($conditions = array()){
        /** @var \Service\Api $apiService */
        $apiService = InstanceManager::getInstance()->getService('\Service\Api');
        try{
            list($apiId, $page, $filter, $download,$token) = $this->getValuesFindDataset($conditions,$apiService);
            /** @var \Service\Api $apiService */
            $apiService->findbigDataSet($apiId,$filter,$page,$token,$download);
            die;
        }catch (\Exception $e){
            $error = array(
                'Erro' => $e->getMessage(),
                'Code' => $e->getCode(),
            );
            echo json_encode($error);
            $apiService->saveLogErro($e);
            throw $e;
        }
    }

    public function getValuesFindDataset($conditions, \Service\Api $apiService): array
    {
        $token = '';
        $download = false;
        $allHeaders = getallheaders();
        if(!empty($allHeaders['WEB_TOKEN'])){
            $conditions['WEB_TOKEN'] = $allHeaders['WEB_TOKEN'];
        }
        if(!empty($allHeaders['Webtoken'])){
            $conditions['WEB_TOKEN'] = $allHeaders['Webtoken'];
        }
        if(!empty($conditions['Webtoken'])){
            $conditions['WEB_TOKEN'] = $conditions['Webtoken'];
        }

        if(!empty($conditions['TEMP_TOKEN'])){
            $conditions['WEB_TOKEN'] = \Service\Api::getConditionsByTempToken($conditions['TEMP_TOKEN']);
            $conditions['DOWNLOAD'] = '1';
        }

        if(!empty($conditions['DOWNLOAD'])){
            $download = $conditions['DOWNLOAD'];
        }

        if(!empty($conditions['WEB_TOKEN'])){
            $token = $conditions['WEB_TOKEN'];
            $conditions = $apiService->getConditionsByWebToken($conditions['WEB_TOKEN']);
        }else{
            \Service\Exception::webTokenInvalid();
        }
        $apiId = empty($conditions['API_ID']) ? \Service\Exception::apiIdNotFound() : $conditions['API_ID'];
        $page = empty($conditions['PAGE']) ? \Service\Exception::pageNotFound() : $conditions['PAGE'];
        if (empty($conditions['FILTER'])) {
            $filterName = empty($conditions['FIELD_DATE_NAME']) ? \Service\Exception::fieldDateNameNotFound() : $conditions['FIELD_DATE_NAME'];
            $filterStart = empty($conditions['FIELD_DATE_START']) ? \Service\Exception::fieldDateStartNotFound() : $conditions['FIELD_DATE_START'];
            $filterEnd = empty($conditions['FIELD_DATE_END']) ? \Service\Exception::fieldDateEndNotFound() : $conditions['FIELD_DATE_END'];
            $filter = array(
                'FIELD_DATE' => array(
                    'NAME' => $filterName,
                    'START' => $filterStart,
                    'END' => $filterEnd
                )
            );
        } else {
            if (empty($conditions['FILTER']['FIELD_DATE']['NAME'])) {
                \Service\Exception::filterNameNotFound();
            }
            if (empty($conditions['FILTER']['FIELD_DATE']['START'])) {
                \Service\Exception::filterStartNotFound();
            }
            if (empty($conditions['FILTER']['FIELD_DATE']['END'])) {
                \Service\Exception::filterEndNotFound();
            }
            $filter = $conditions['FILTER'];

            if (!Utilities::validDatePtBr($filter['FIELD_DATE']['START'])) {
                \Service\Exception::filterStartNotCorrectFormat();
            }
            if (!Utilities::validDatePtBr($filter['FIELD_DATE']['END'])) {
                \Service\Exception::filterEndNotCorrectFormat();
            }
        }
        return array($apiId, $page, $filter, $download, $token);
    }

}
