<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\PeriodMathExpression\Period;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class Periods extends Crud {
    
    protected $dataSourceName = 'periods';
    /** @var  \Service\PeriodsService */
    protected $periodsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, $periodsService) {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->periodsService = $periodsService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {

        $periods= $this->findAllPeriods();

        $response->addDataSet(new DataSet('periods', $periods));
    }

    public function getDateByDateExpression(DTO\Request\Row $request, DTO\Response $response) {
        $row =$request->getRow()->get('datesExpression');
        $row = $this->periodsService->getDateByDateExpression($row);

        $response->addDataSet(new DataSet('getDateByDateExpression', $row));
    }

    /**
     * @return array
     */
    public function findAllPeriods()
    {
        $basePeriods = $this->periodsService->getPeriodsFromBase();
        $orgPeriods = $this->periodsService->getPeriodsFromOrg();
        $periods = array_merge($basePeriods, $orgPeriods);
        foreach ($periods as $key => &$period) {
            $startDateExpression = !empty($period['START_DATE_EXPRESSION']) ? $period['START_DATE_EXPRESSION'] : null;
            $endDateExpression = !empty($period['END_DATE_EXPRESSION']) ? $period['END_DATE_EXPRESSION'] : null;
            if ($startDateExpression || $endDateExpression) {
                try {

                    $period['interval'] = [
                        'start' => Period::getDate($startDateExpression),
                        'end' => Period::getDate($endDateExpression),
                        '_id' => $period['_id'],
                        'customized' => true
                    ];

                } catch (\Exception $e) {
                    $period['Erro'] = $e->getMessage();
                }
            }else{
                unset($period['interval']);
            }
        }
        return $periods;
    }
}