<?php
namespace Controller;

use Util\InstanceProvider;
use Util\OpensslCrypt;
use Util\Utilities;
use Zeedhi\Framework\DTO;
use Service\Manager\Mongo\Manager;
use Service\DeleteCascadeService;
use Service\ConnectionService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Environment;

class QueryRunner extends Crud {
    protected $connectionService;
	protected $environment;
    public function __construct(ConnectionService $connectionService, Environment $environment) {
		$this->connectionService = $connectionService;
		$this->environment = $environment;
	}

    public function runQuery(DTO\Request\Row $request, DTO\Response $response){
        $connectionsParams = $this->setsQuieryRunner($request);

        try {
            if ($connectionsParams['oracleWalletUse'] == 'Y') {
                $wallet = Utilities::getWallet($connectionsParams);
                if ( Utilities::getOrganizationUseWallet($wallet)) {
                    $queryResult = Utilities::execQueryWallet($connectionsParams['query'],
                        'queryRunner', $wallet);
                } else {
                    $queryResult = $this->connectionService->executeQuery($connectionsParams, 120);
                }
            } else {
                $queryResult = $this->connectionService->executeQuery($connectionsParams, 120);
            }
			$response->addDataSet(new DataSet('queryResult', $queryResult));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 0));
        }
    }

    public function runQueryMining(DTO\Request\Row $request, DTO\Response $response){
        $connectionsParams = $this->setsQuieryRunner($request);
        try {
            if ($connectionsParams['oracleWalletUse'] == 'Y') {
                $wallet = Utilities::getWallet($connectionsParams);
                if ( Utilities::getOrganizationUseWallet($wallet)) {
                    $queryResult = Utilities::execQueryWallet($connectionsParams['query'],
                        'queryRunner', $wallet);
                } else {
                    $queryResult = InstanceProvider::getQueryRunnerMining()->executeQueryMining($connectionsParams, 120);
                }
            } else {
                $queryResult = InstanceProvider::getQueryRunnerMining()->executeQueryMining($connectionsParams, 120);
            }

			$response->addDataSet(new DataSet('queryResult', $queryResult));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 0));
        }
    }

    public function resultQueryRunner(DTO\Request\Filter $request, DTO\Response $response){
        $condition = $request->getFilterCriteria()->getConditions();
        $processQueryRunner = Utilities::getValueInFilterDataSourceByName($condition,'processQueryRunner',true);
        $result = InstanceProvider::getQueryRunnerMining()->getResult($condition,$processQueryRunner);
        Utilities::setDataSetDefault($request,$response,$result);
    }

    public function getColumnsMetaData(DTO\Request\row $request, DTO\Response $response) {
        $connectionsParams = $this->setsQuieryRunner($request);
        $result = InstanceProvider::getQueryRunnerMining()->getColumnsMetaData($connectionsParams);
        Utilities::setDataSetDefault($request,$response,$result);
    }

    public function dropCollectionQueryRunner(DTO\Request $request, DTO\Response $response) {
        $result = InstanceProvider::getQueryRunnerMining()->dropCollectionQueryRunner();
        Utilities::setDataSetDefault($request,$response,$result);
    }

    public function clearImportHistory(DTO\Request $request, DTO\Response $response) {
        InstanceProvider::getQueryRunnerMining()->clearImportHistory();
    }

    /**
     * @param DTO\Request\Row $request
     * @return array
     */
    private function setsQuieryRunner(DTO\Request\Row $request)
    {
        ini_set('max_execution_time', '600');
        set_time_limit(600);
        $row = $request->getRow();
        $organizationID = Utilities::getOrganizationIdForMongo();
        $clientRoom = $this->connectionService->getClientRoom($organizationID, $row);
        $dbUrl = '';
        if (isset($row['connectionDbUrl'])) {
            $dbUrl = $row['connectionDbUrl'];
        } else if (isset($connectionParams['connectionHost']) && isset($connectionParams['connectionPort']) && isset($connectionParams['connectionDbname'])) {
            $dbUrl = "jdbc:oracle:thin:@" . $row['connectionHost'] . ":" . $row['connectionPort'] . "/" . $row['connectionDbname'];
        }
        $driverClass = isset($row['driverClass']) ? $row['driverClass'] : null;

        if ($driverClass == "Spreadsheet") {
            $row['query'] = $row['query'] . "::::EXEC";
        }
        $connectionsParams = array(
            'dbUrlSsl' => OpensslCrypt::encrypt($dbUrl),
            'usernameSsl' => OpensslCrypt::encrypt($row['connectionUsername']),
            'password' => $row['connectionPassword'],
            'passwordSsl' => isset($row['connectionPasswordSsl']) ? $row['connectionPasswordSsl'] : null,
            'query' => $row['query'],
            'driverClass' => $driverClass,
            'keys' => isset($row['keys']) ? $row['keys'] : null,
            'dialect' => isset($row['dialect']) ? $row['dialect'] : null,
            'oracleWalletUse' => isset($row['oracleWalletUse']) ? $row['oracleWalletUse'] : null,
            'connectionId' => isset($row['connectionId']) ? $row['connectionId'] : null,
            "room" => $clientRoom
        );
        return $connectionsParams;
    }
}
