<?php
namespace Service;

use Util\Utilities;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Structure;
use \Service\Manager\Mongo\Manager;
use \Service\ViewProperty;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;
use Moment\Moment;

class ImportSchedule {

    protected $dataSourceName = 'importSchedule';
    /** @var \Service\Manager\Mongo\Manager $mongoManager */
    protected $mongoManager;
    /** @var Session $sessionManager */
    protected $sessionManager;

    public function __construct(Manager $mongoManager, Session $sessionManager) {
        $this->mongoManager = $mongoManager;
        $this->sessionManager = $sessionManager;
    }

    public function findBy(FilterCriteria $filterCriteria) {
        $filterCriteria->addCondition('organizationId', (string) Utilities::getOrganizationIdForMongo());
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function findByDataSourceId($dataSourceId) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('organizationId', (string) Utilities::getOrganizationIdForMongo());
        $filterCriteria->addCondition('dataSource', $dataSourceId);
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function findImportsInProgress($dataSourceId){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('dataSource', $dataSourceId);
        $filterCriteria->addCondition('scheduleExpression', '');
        $importSchedule = $this->findBy($filterCriteria);

        $newValue = 'pending';
        $this->updateImportSchedule($importSchedule, $newValue);

        return $importSchedule;
    }

    public function updateImportSchedule(&$importSchedules, $newValue){
        $schedules = [];
        foreach ($importSchedules as &$schedule) {
            $schedule['status'] = $newValue;
            $schedule['date'] = Type::getType('mongoDate')->convertToDatabaseValue($schedule['date']);
        }

        $dataSet = new DataSet('importSchedule', $importSchedules);
        $persistedRows = $this->mongoManager->persist($dataSet);
    }

    public function findAll() {
        return $this->findBy(new FilterCriteria($this->dataSourceName));
    }
    
    public function saveScheduleExpression($newDataSource, $persistedDataSourceId) {
        $row = array(
            "scheduleExpression" => $newDataSource["scheduleExpression"],
            "beforeImportScript" => $newDataSource["beforeImportScript"],
            "afterImportScript" => $newDataSource["afterImportScript"],
            "persistType" => $newDataSource["persistType"],
            "dataSource" => $persistedDataSourceId[0]["_id"],
            "period" => $newDataSource["period"],
            "type" => !empty($newDataSource["type"]) ? $newDataSource["type"] : null,
            "date" => new \MongoDB\BSON\UTCDateTime((new Moment(null, 'America/Sao_Paulo'))->startOf('day')->format('U').'000')
        );
        $this->saveOne($row);
    }

    public function saveOne($schedule) {
        $schedule['organizationId'] = (string) Utilities::getOrganizationIdForMongo();
        $persistedItems = $this->mongoManager->persist(new DataSet('importSchedule', array($schedule)));

        return $persistedItems;
    }

    public function delete($scheduleItems) {
        $dataSet = new DataSet($this->dataSourceName, $scheduleItems);

        return $this->mongoManager->delete($dataSet);
    }

}