<?php
namespace Service\Mail;

use Service\Mail\Core\SmtpMail;
use Zend\Mime\Mime as ZendMime;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;
use Zeedhi\DataExporter\Utility\ReportInfoManager;

class BuildMail {

    private $smtpMail;
    private $baseDir = '';
    private $reportInfoManager;

    public function __construct(SmtpMail $smtpMail, ReportInfoManager $reportInfoManager){
        $this->smtpMail = $smtpMail;
        $this->baseDir = __DIR__.DIRECTORY_SEPARATOR.'EmailTemplates'.DIRECTORY_SEPARATOR;
        $this->reportInfoManager = $reportInfoManager;
    }

    public function getMailTemplatePath($template) {
        $defaultFile = $this->baseDir . $template;
        return $defaultFile;
    }

    function fileGetContentUtf8($fileName) {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8',
            mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }

    public function setMessage($template){
        $fileName = $this->getMailTemplatePath($template);
        return $this->fileGetContentUtf8($fileName);
    }

    public function setMessageMail($message){
        $messageMail = utf8_decode($message);
        $mimePart = new ZendMimePart($messageMail);
        $mimePart->type = 'text/html';
        $mimeMessage = new ZendMimeMessage();
        $mimeMessage->setParts(array($mimePart));

        return $mimeMessage;
    }

    public function setReportAsAttachment($reportKey, ZendMimeMessage $mimeMessage){
        try{
            $file = $this->reportInfoManager->getReportInfo($reportKey);
            $attachmentFile = file_get_contents($file["fileName"]);

            $attachmentPart = new ZendMimePart($attachmentFile);
            $attachmentPart->type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
            $attachmentPart->disposition = ZendMime::DISPOSITION_ATTACHMENT;
            $attachmentPart->encoding = ZendMime::ENCODING_BASE64;
            $attachmentPart->filename = $file["title"].".xlsx";

            $mimeMessage->addPart($attachmentPart);

            return $mimeMessage;
        }catch (\Exception $e){
            throw $e;
        }
    }

    public function sendHtml($to, $subject, $message, $senderName, $attachment = null){
        $this->smtpMail->setFrom($senderName);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);

        foreach($to as $address){
            $mailAddress = isset($address['EMAIL']) ? $address['EMAIL'] : $address;
            $this->smtpMail->setTo($mailAddress);
            $this->smtpMail->sendHtml();
        }
    }
}