<?php

namespace Service;

use \Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Moment\Moment;

class ManualImportQueueService {

    protected $dataSourceName = 'manualImportQueue';

    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $orgMongoManager;

	public function __construct(Manager $orgMongoManager)  {
        $this->orgMongoManager = $orgMongoManager;
	}

	public function findByCriteria($filterCriteria) {
	    return $this->orgMongoManager->findBy($filterCriteria)->getRows();
	}

	public function findAllByDataSourceId($dataSourceId) {
		$filterCriteria = $this->buildCriteria('dataSource', '=', $dataSourceId);
		$filterCriteria->addOrderBy('order', FilterCriteria::ORDER_ASC);
		return $this->findByCriteria($filterCriteria);
	}

	protected function buildCriteria($columnName, $operator, $columnValue) {
	    $filterCriteria = new FilterCriteria($this->dataSourceName);
	    $filterCriteria->addCondition($columnName, $operator, $columnValue);
	    return $filterCriteria;
	}

	public function findByConditions($columnName, $columnValue, $operator = '=') {
	    $filterCriteria = $this->buildCriteria($columnName, $operator, $columnValue);
	    return $this->findByCriteria($filterCriteria);
	}

	public function saveQueue($items) {
        $dataSet = new DataSet($this->dataSourceName, $items);
        return $this->orgMongoManager->persist($dataSet);
    }

    public function removeItemFromQueue($itemId) {
    	$dataSet = new DataSet($this->dataSourceName, array($itemId));
        return $this->orgMongoManager->delete($dataSet);
    }

	public function getMaxOrder($dataSourceId) {
	    $filterCriteria = $this->buildCriteria('dataSource', '=', $dataSourceId);
	    $filterCriteria->addOrderBy('order', FilterCriteria::ORDER_DESC);

	    $processes = $this->findByCriteria($filterCriteria);

	    if(count($processes) > 0) {
            return $processes[0]['order'];
	    } else {
	        return 0;
	    }
	}

	public function updateProcessesOrders($param = null) {
		$itemsToUpdate = is_array($param) ? $param : $this->findAllByDataSourceId($param);
		if(!$itemsToUpdate) {
			$itemsToUpdate = array();
		}

		foreach ($itemsToUpdate as &$item) {
			$item['order'] = $item['order'] -1;
		}

		$this->saveQueue($itemsToUpdate);
	}

	public function getItemToSchedule($dataSourceId) {
		$queue = $this->findAllByDataSourceId($dataSourceId);
		$queueItem = !empty($queue) ? $queue[0] : null;
		if($queueItem) {
			if(isset($queueItem['order'])) {
                unset($queueItem['order']);
            }
            
            $queueItem['date'] = new \MongoDB\BSON\UTCDateTime((new Moment(null, 'America/Sao_Paulo'))->startOf('day')->format('U').'000');
		}

		return $queueItem;
	}
}