<?php

namespace Service\MigrationLogin;


use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class MigrationAllUsers
{
    /** @var EntityManager $entityManagerIdentity */
    protected $entityManagerIdentity;

    public $dir;

    /** @var EntityManager $entityManagerSaas */
    protected $entityManagerSaas;

    /** @var EntityManager $entityManagerZmart */
    protected $entityManagerZmart;
    /** @var \Util\MongoDataBaseAlterations $mongoScript  */
    protected $mongoScript;
    private $migrationOperador = 0;
    private $migrationPerfilacesso = 0;
    private $migrationPerfacessparc = 0;



    private function getOrnanizationByIdentity($orgIdentityId)
    {

        /** @var EntityManager $entityManagerIdentity */
        $entityManagerIdentity = InstanceManager::getInstance()->getService('entityManagerIdentity');
        $query = <<<SQL
            SELECT ID, NRORG
              FROM ZH_ORGANIZATION
              WHERE ID = $orgIdentityId
SQL;
        $integrationOrganizations =  $entityManagerIdentity->getConnection()->fetchAll($query);
        return empty($integrationOrganizations) ? null : $integrationOrganizations[0]['NRORG'];
    }

    public function importationAdjustments($orgIdentityId, $nrorg = null){
        if (!$nrorg) {
            $nrorg = $this->getOrnanizationByIdentity($orgIdentityId);
        }
        if (!$nrorg) {
            throw new \Exception('UNIDENTIFIED_DESTINATION_ORGANIZATION');
        }

        list($this->entityManagerIdentity, $this->entityManagerSaas,$this->entityManagerZmart,$this->mongoScript) =
            EntityManagers::getEntityManagers($nrorg,$orgIdentityId);

        try {
            $this->mongoScript->changeOrganizationIdsInImportShedule();
            $this->echoMessageEnd();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getPrimary($nrorgSecondary){
        $sql = <<<SQL
    		  SELECT ORG.NRORGPRINCIPAL AS ID
                FROM ORGANIZACAO ORG 
               WHERE ORG.NRORG = $nrorgSecondary
SQL;

        $nrorgs = $this->entityManagerZmart->getConnection()->fetchAll($sql);
        if(empty($nrorgs)){
            throw New \Exception(Exception::buildErrorJSON(
                "NO_PRIMARY_NRORG",
                array("nrOrg"=>$nrorgSecondary)
            ));
        }
        return $nrorgs[0]['ID'];

    }

    public function findSecondaryOrganizations(){
        $sql = <<<SQL
    		   SELECT ORG.NRORG AS ID
                FROM OM_PRODUTO OMP
                JOIN OM_PRODUTO_ORG OMO ON (OMO.NRPRODUTO_ID = OMP.ID)
                JOIN ORGANIZACAO ORG ON (OMO.NRORG_ID = ORG.NRORG)
                JOIN ORGANIZACAO ORG2 ON (ORG.NRORGPRINCIPAL = ORG2.NRORG)
               WHERE OMP.ID = 1384
                 AND ORG2.PREPSTATUS = 'Z'
                 AND ORG.PREPSTATUS = 'M'
                 AND ORG.IDATIVO = 'S'
                 AND ORG.NRORGPRINCIPAL IS NOT NULL
SQL;

        $organizations = $this->entityManagerZmart->getConnection()->fetchAll($sql);
        $nrorgs = Utilities::getArrayFromAssociativeArray($organizations);
        return $nrorgs;

    }

    public function findAllOrganizationsSAAS(){
        $sql = <<<SQL
    		  SELECT ORG.NRORG AS ID
                FROM OM_PRODUTO OMP
                JOIN OM_PRODUTO_ORG OMO ON (OMO.NRPRODUTO_ID = OMP.ID)
                JOIN ORGANIZACAO ORG ON (OMO.NRORG_ID = ORG.NRORG)
               WHERE OMP.ID = 1384
                 AND ORG.PREPSTATUS <> 'Z'
                 AND ORG.IDATIVO = 'S'
                 AND OMO.DSURLREDIRECT IS NOT NULL
                 AND ORG.IDORGSAAS = 'S'
SQL;

        $organizations = $this->entityManagerZmart->getConnection()->fetchAll($sql);
        $nrorgs = Utilities::getArrayFromAssociativeArray($organizations);

        $sql = <<<SQL
    		  SELECT ID,NRORG FROM ZH_ORGANIZATION WHERE NRORG IN (:NRORGS) ORDER BY NRORG
SQL;

        $param = array(
            'NRORGS' => $nrorgs
        );

        $paramType = array(
            'NRORGS' => Connection::PARAM_STR_ARRAY
        );
        $organizationIdentities = $this->entityManagerIdentity->getConnection()->fetchAll($sql,$param,$paramType);
        $organizationIds = Utilities::getArrayFromAssociativeArray($organizationIdentities);
        $nrorgs = Utilities::getArrayFromAssociativeArray($organizationIdentities,'NRORG');
        return array($nrorgs,$organizationIds);

    }
    public static function testConnection($nrorg){
        $conn = oci_connect("/", "", "USR_ORG_$nrorg","AL32UTF8", OCI_CRED_EXT);
        if (!$conn) {
            throw new \Exception(Exception::buildErrorJSON(
                "WALLET_CONNECTION_FAIL",
                array("nrorg" => "USR_ORG_".$nrorg)
            ));
        }
        $stmt = oci_parse($conn, "select SUBSTR(TRIM(UPPER(SYS_CONTEXT('USERENV', 'SESSION_USER'))), 1, 50) from dual");
        if (oci_execute($stmt)) {
            $row = oci_fetch_row($stmt);
            if($row[0] != "USR_ORG_$nrorg"){
                $user = $row[0];
                throw new \Exception(Exception::buildErrorJSON(
                    "WALLET_USER_ERROR",
                    array("nrorg" => "USR_ORG_".$nrorg, "user" => $user)
                ));
            }
        } else {
            throw new \Exception(Exception::buildErrorJSON(
                "WALLET_EXECUTE_FAIL",
                array("nrorg" => "USR_ORG_".$nrorg)
            ));
        }
    }

    public static function createWalletSAAS($nrorgs){
        try {
            foreach ($nrorgs as $nrorg) {
                $console = shell_exec("/teknisa/scripts/criaWalletUsrsaas.sh {$nrorg}");
                self::testConnection($nrorg);
                self::addLog("Criado Wallet pra Nrorg $nrorg e testado \n Saida do console: $console");
            }
        } catch (\Exception $e) {
            self::addLog($e->getMessage());
            throw $e;
        }

    }

    public static function addLog($log){
        $str =  file_get_contents('./log.txt');
        if(!$str){
            $str = '';
        }
        $date = new \Datetime();
        $dateString = $date->format('d/m/Y H:i:s');
        $str = $str."\n\n".$dateString."\n".$log;
        file_put_contents('./log.txt', $str);
    }

    public static function setWalletInServicesMigration($organizationId){
        $str =  file_get_contents('../servicesMigration.xml.migration');
        $str = str_replace("USR_ORG_MIGRATION", "USR_ORG_$organizationId",$str);
        file_put_contents('../servicesMigration.xml', $str);
        self::addLog("Modificado Wallet no servicesMigration.xml para USR_ORG_$organizationId");

    }

    public function migrationSAAS($organizationIds,$nrorgs){
        foreach ($organizationIds as $key => $organizationId) {
            self::setWalletInServicesMigration($nrorgs[$key]);
            $console = shell_exec("cd {$this->dir} && php migration.php {$organizationId}");
            self::addLog("A migaração foi feita para $organizationId com a saida $console");
            //tratar erros
        }

    }

    public function migrationSsl()
    {
        $this->mongoScript = EntityManagers::getMongo();
        $allDbnames = $this->mongoScript->getListDatabases();
        foreach ($allDbnames['databases'] as $dbname) {
            if (strpos($dbname['name'], 'Zmart__') !== false) {
                $this->echoMessage('AddSsl no bd ' . $dbname['name']);
                $this->mongoScript->addSslInConnection($dbname['name']);
                $this->echoMessage('Ssl no bd ' . $dbname['name'] ." OK");
            }
        }
    }

    public function migrationAllSaas()
    {

        $this->entityManagerZmart = InstanceManager::getInstance()->getService('entityManagerZmart');
        $this->entityManagerIdentity = InstanceManager::getInstance()->getService('entityManagerIdentity');

        list($nrorgs,$organizationIds) = $this->findAllOrganizationsSAAS();
        $nrorgString = Utilities::getStringForArray($nrorgs);
        self::addLog("Organizações a serem migradas $nrorgString");
        self::createWalletSAAS($nrorgs);
        $this->migrationSAAS($organizationIds,$nrorgs);

    }

    public function migrationAllSecondary()
    {

        $this->entityManagerZmart = InstanceManager::getInstance()->getService('entityManagerZmart');

        $nrorgs = $this->findSecondaryOrganizations();
        $nrorgString = Utilities::getStringForArray($nrorgs);
        self::addLog("Organizações secundária a serem migradas $nrorgString");
        foreach ($nrorgs as $key => $nrorg) {
            $console = shell_exec("cd {$this->dir} && php migrationSecondary.php {$nrorg}");
            self::addLog("A migaração foi feita para $nrorg com a saida $console");
            //tratar erros
        }

    }

    public function migrationSecondary($nrorg)
    {


        $this->entityManagerZmart = InstanceManager::getInstance()->getService('entityManagerZmart');
        $nrorgPrimary = $this->getPrimary($nrorg);
        $this->mongoScript = EntityManagers::getMongo();
        $this->mongoScript->setOrgs($nrorg);
        $this->mongoScript->setRelatedIds(array($nrorgPrimary => $nrorg));
        $dbName = $this->mongoScript->ORG_DB_PREFIX.$nrorg;
        $orgMongoManager = $this->mongoScript->buildManager($dbName);
        $allNameCollections = $orgMongoManager->getCollectionNames();

        echo "\n Criando usuario no mongo da organização...";
        $this->mongoScript->createDbUserPerNrorg();
        if(empty($allNameCollections)){
            //throw new \Exception('O banco do mongo de destino "'.$dbName.'" está vazio');


            echo "\n Clonando o banco da primaria pra o novo ...";
            $time = $this->mongoScript->cloneDBNamePrimary();
            echo "\n Clonado! tempo = $time";
            echo "\n Criando usuario no mongo ...";
            $this->mongoScript->createDbUser();

        }

        $connectionSaas = $this->entityManagerZmart->getConnection();
        try {
            $connectionSaas->beginTransaction();
            $this->echoMessage("Atualizando o id da organização ...");
            $this->mongoScript->changeOrganizationIdsPrimary();
            $this->echoMessage("Salvando indicador que a organização foi migrada ...");
            $this->saveMigrationSecondary($nrorg);
            $connectionSaas->commit();

        } catch (\Exception $e) {
            echo $e->getMessage();
            $connectionSaas->rollBack();
            throw $e;
        }
    }

    public function saveMigrationSecondary($nrOrg){
        $updateZmart = <<<SQL
    		UPDATE ORGANIZACAO
    		   SET PREPSTATUS = 'X',PREPSTATUSBKP = 'M'
    		 WHERE NRORG = '$nrOrg'
SQL;
        $this->entityManagerZmart->getConnection()->executeQuery($updateZmart);
    }

    public function migrationAllUsers($orgIdentityId, $nrorg = null)
    {
        if (!$nrorg) {
            $nrorg = $this->getOrnanizationByIdentity($orgIdentityId);
        }
        if (!$nrorg) {
            throw new \Exception('UNIDENTIFIED_DESTINATION_ORGANIZATION');
        }

        list($this->entityManagerIdentity, $this->entityManagerSaas,$this->entityManagerZmart,$this->mongoScript) =
            EntityManagers::getEntityManagers($nrorg,$orgIdentityId);

        $connectionSaas = $this->entityManagerSaas->getConnection();
        try {
            $this->validateNovoCodigo($nrorg);
            $connectionSaas->beginTransaction();
            $this->echoMessage("Analizando organização do Identity '$orgIdentityId' ...");
            $zhUsers = $this->setOperadores($orgIdentityId, $nrorg);
            $this->echoMessage("Comparando com a $nrorg da teknisa ...");
            $this->setPerfilacesso($orgIdentityId, $nrorg, $zhUsers);
            $this->echoMessage("Atualizando o id da organização ...");
            $this->mongoScript->changeOrganizationIds();
            $this->echoMessage("Atualizando o ids dos usuarios ...");
            $this->mongoScript->changeUsersIds();
            $this->echoMessage("Salvando indicador que a organização foi migrada ...");
            $this->saveMigrations($nrorg);
            $this->echoMessage("Removendo redirecionamento ...");
            $this->removeRedirection($nrorg);

            $this->echoMessageEnd();
            $connectionSaas->commit();

        } catch (\Exception $e) {
            echo $e->getMessage();
            $connectionSaas->rollBack();
            throw $e;
        }
    }

    public function saveMigrations($nrOrg){
        $updateZmart = <<<SQL
    		UPDATE ORGANIZACAO
    		   SET PREPSTATUS = 'Z'
    		 WHERE NRORG = '$nrOrg'
SQL;
        $this->entityManagerZmart->getConnection()->executeQuery($updateZmart);
    }

    public function removeRedirection($nrOrg){
        $updateZmart = <<<SQL
    		UPDATE OM_PRODUTO_ORG
    		   SET DSURLREDIRECT = NULL
    		 WHERE NRPRODUTO_ID = 1384 AND NRORG_ID = $nrOrg
SQL;
        $this->entityManagerZmart->getConnection()->executeQuery($updateZmart);
    }

    public function validateNovoCodigo($nrorg){
        $queryNovocodigo = <<<SQL
    	  SELECT TAB_DUAL.TABELA, NOG.NRSEQUENCIAL ,NOG.CDCONTADOR, TAB.MAX,TAB.TAB ,NOG.NRORG
            FROM ( SELECT 'PARCNEGOCIO' AS TABELA FROM DUAL
                    UNION
                   SELECT 'PERFILACESSO' AS TABELA FROM DUAL
                    UNION
                   SELECT 'PERFACESSPARC' AS TABELA FROM DUAL
                    UNION
                   SELECT 'OPERADOR' AS TABELA FROM DUAL) TAB_DUAL
       LEFT JOIN NOVOCODIGO NOG ON TAB_DUAL.TABELA = NOG.CDCONTADOR
       LEFT JOIN (
                    SELECT MAX(NRPARCNEGOCIO) AS MAX, 'PARCNEGOCIO' AS TAB FROM PARCNEGOCIO
                    UNION
                    SELECT MAX(NRPERFILACESSO) AS MAX, 'PERFILACESSO' AS TAB FROM PERFILACESSO
                    UNION
                    SELECT MAX(NRPERFACESSPARC) AS MAX, 'PERFACESSPARC' AS TAB FROM PERFACESSPARC
                    ) TAB ON (TAB.TAB = TAB_DUAL.TABELA )
            WHERE TAB_DUAL.TABELA IN ('PARCNEGOCIO','OPERADOR','PERFILACESSO','PERFACESSPARC')
SQL;
        $novocodigos = $this->entityManagerSaas->getConnection()->fetchAll($queryNovocodigo);
        if ($novocodigos[0]['NRORG'] != '5'){
            $connectionSaas = $this->entityManagerSaas->getConnection();
            $connectionSaas->beginTransaction();
            foreach ($novocodigos as &$novocodigo) {
                $setNrorg = '';
                $setMax = '';
                $updateNovocodigo = '';
                $cdcontador = $novocodigo['TABELA'];

                if(!$novocodigo['CDCONTADOR']){
                    $newCode = EntityManagers::getNewCode($nrorg);
                    $newCode->getCodeByTableName($novocodigo['TABELA'],$nrorg,1,12);
                    $novocodigo['NRSEQUENCIAL'] = '0';
                    $novocodigo['NRORG'] = $nrorg;
                }

                if ($novocodigo['NRORG'] != $nrorg) {
                    $setNrorg = "NRORG = $nrorg";
                }

                if($novocodigo['MAX'] > $novocodigo['NRSEQUENCIAL'] ){
                    $max = $novocodigo['MAX'];
                    $setMax = "NRSEQUENCIAL = '".str_pad($max, 20, "0", STR_PAD_LEFT)."'";
                }

                if($setNrorg ||$setMax){
                    $comma = '';
                    if($setNrorg && $setMax){
                        $comma = ',';
                    }
                    $updateNovocodigo = <<<SQL
    		UPDATE NOVOCODIGO
    		SET
    		   $setNrorg $comma $setMax
    		WHERE
    		CDCONTADOR = '$cdcontador'
SQL;
                    $this->entityManagerSaas->getConnection()->executeQuery($updateNovocodigo);
                }
            }
            $this->entityManagerSaas->commit();
        }
    }

    public function insertOperador($zhUserName,$zhUserEmail,$nrorg,$password)
    {
        $newCode = EntityManagers::getNewCode($nrorg);
        $nrparcnegocio = $newCode->getCodeByTableName('PARCNEGOCIO',$nrorg,1,10,true);
        $insertParcnegocio = <<<SQL
    		INSERT INTO PARCNEGOCIO 
    		(NRPARCNEGOCIO,NRORG,NMPRINCIPALPARC,NMSECUNDARIPARC,IDATIVO,DTINCLUSAO,CDTIPOINSCRICAO,
    		IDPESSOAFISICA,IDINSTITUICAO,IDPARCFUNDIDO) 
    		VALUES 
    		(:NRPARCNEGOCIO,:NRORG,:NMPRINCIPALPARC,:NMSECUNDARIPARC,'S',SYSDATE,'LIVRE','S','N','N') 
SQL;
        $paramParcnegocio = array(
            ':NRPARCNEGOCIO' => $nrparcnegocio,
            ':NRORG' => $nrorg,
            ':NMPRINCIPALPARC' => explode(' ',$zhUserName)[0],
            ':NMSECUNDARIPARC' => end(explode(' ',$zhUserName)),
        );
        $this->entityManagerSaas->getConnection()->executeQuery($insertParcnegocio, $paramParcnegocio);
        $this->entityManagerSaas->flush();
        $cdOperador = 'ZMART'.$newCode->getCodeByTableName('OPERADOR',$nrorg,1,7);
        $insertOperador = <<<SQL
    		INSERT INTO OPERADOR 
    		(CDOPERADOR,IDLIGEXTOPER,IDATUOPERADOR,NRORG,DTINCLUSAO,IDCADPRAPROVORE,IDCADPRPROCEORE,
    		IDCHEFERECEITAM,DSEMAILOPER,CDSENHAOPERWEB,IDATIVO,NRPARCNEGOCIO,NMOPERADOR,IDOPERSAAS)
    		 VALUES 
    		 (:CDOPERADOR,'N','S',:NRORG,SYSDATE,'N','N','N',:DSEMAILOPER,
    		 :PASSWORD,'S',:NRPARCNEGOCIO,:NMOPERADOR,'S')
SQL;
        $paramOperador = array(
            ':CDOPERADOR' => $cdOperador,
            ':NRORG' => $nrorg,
            ':NMOPERADOR' => $zhUserName,
            ':DSEMAILOPER' => $zhUserEmail,
            ':NRPARCNEGOCIO' => $nrparcnegocio,
            ':PASSWORD' => $password,
        );
        $this->entityManagerSaas->getConnection()->executeQuery($insertOperador, $paramOperador);
        $this->entityManagerSaas->flush();
        $this->migrationOperador ++;
        return array($nrparcnegocio,$cdOperador);
    }

    public function insertPerfilacesso($zhProfileName,$zhProfileIsAdmin,$nrorg)
    {
        $newCode = EntityManagers::getNewCode($nrorg);
        $nrperfilacesso = $newCode->getCodeByTableName('PERFILACESSO',$nrorg,1,10,true);
        $insertPerfilacesso = <<<SQL
    		INSERT INTO PERFILACESSO 
    		(NRPERFILACESSO,NMPERFILACESSO,NRORG,IDATIVO,IDPERFILACESSOADM,DTINCLUSAO,NRORGINCLUSAO)
    		VALUES 
    		(:NRPERFILACESSO,:NMPERFILACESSO,:NRORG,'S',:IDPERFILACESSOADM,SYSDATE,:NRORG)
SQL;
        $paramPerfilacesso = array(
            ':NRPERFILACESSO' => $nrperfilacesso,
            ':NRORG' => $nrorg,
            ':NMPERFILACESSO' => $zhProfileName,
            ':IDPERFILACESSOADM' => $zhProfileIsAdmin,
        );
        $this->entityManagerSaas->getConnection()->executeQuery($insertPerfilacesso, $paramPerfilacesso);
        $this->entityManagerSaas->flush();
        $this->migrationPerfilacesso ++;
        return $nrperfilacesso;
    }

    public function insertPerfacessparc($nrparcnegocio,$nrperfilacesso,$nrorg)
    {
        $newCode = EntityManagers::getNewCode($nrorg);
        $nrperfacessparc = $newCode->getCodeByTableName('PERFACESSPARC',$nrorg,1,10,true);
        $insertPerfacessparc = <<<SQL
    		INSERT INTO PERFACESSPARC
    		(NRPERFACESSPARC,NRORG,NRPARCNEGOCIO,NRPERFILACESSO,IDATIVO,DTINCLUSAO,NRORGINCLUSAO)
    		VALUES
    		(:NRPERFACESSPARC,:NRORG,:NRPARCNEGOCIO,:NRPERFILACESSO,'S',SYSDATE,:NRORG)
SQL;
        $paramPerfacessparc = array(
            ':NRPERFACESSPARC' => $nrperfacessparc,
            ':NRORG' => $nrorg,
            ':NRPARCNEGOCIO' => $nrparcnegocio,
            ':NRPERFILACESSO' => $nrperfilacesso,
            ':NRORGINCLUSAO' => $nrorg,
        );
        $this->entityManagerSaas->getConnection()->executeQuery($insertPerfacessparc, $paramPerfacessparc);
        $this->entityManagerSaas->flush();
        $this->migrationPerfacessparc ++;
        return $nrperfacessparc;
    }

    public function getZhUsers($orgIdentityId)
    {
        $query = <<<SQL
    		SELECT ZHU.EMAIL, ZHU.ID,ZHU.NAME,ZHU.PASSWORD
    		 FROM ZH_USER ZHU
              WHERE ZHU.ORGANIZATION_ID = $orgIdentityId
              AND ZHU.STATUS = 'A'
SQL;


        $zhUsers = $this->entityManagerIdentity->getConnection()->fetchAll($query);
        return $zhUsers;
    }

    public function getPerfilacesso($nrorg)
    {
        $query = <<<SQL
    		SELECT NRPERFILACESSO,NMPERFILACESSO
    		 FROM PERFILACESSO
              WHERE NRORG = $nrorg
SQL;


        $perfilacesso = $this->entityManagerSaas->getConnection()->fetchAll($query);
        return $perfilacesso;
    }
    public function getOperadores($emails)
    {
        $query = <<<SQL
    	   SELECT OPE.DSEMAILOPER,OPE.NRPARCNEGOCIO,OPE.CDOPERADOR
    		 FROM OPERADOR OPE
            WHERE OPE.DSEMAILOPER IN (:emails)
SQL;


        $param = array(
            'emails' => $emails

        );
        $paramTypes = array(
            'emails' => Connection::PARAM_STR_ARRAY
        );

        $operadores = $this->entityManagerSaas->getConnection()->fetchAll($query, $param,$paramTypes);
        return $operadores;
    }

    public function setNrparceiro($nrorg, $operadores, $zhUsers)
    {
        foreach ($operadores as $operador) {
            foreach ($zhUsers as &$zhUser) {
                if ($zhUser['EMAIL'] == $operador['DSEMAILOPER']) {
                    $zhUser['NRPARCNEGOCIO'] = $operador['NRPARCNEGOCIO'];
                    $zhUser['CDOPERADOR'] = $operador['CDOPERADOR'];
                }
            }

        }

        foreach ($zhUsers as $key => &$zhUser) {
            if (empty($zhUser['NRPARCNEGOCIO'])) {
                if($nrorg == 20){
                    unset($zhUsers[$key]);
                }else{
                    try{
                        list($zhUser['NRPARCNEGOCIO'],$zhUser['CDOPERADOR']) = $this->insertOperador($zhUser['NAME'], $zhUser['EMAIL']
                            , $nrorg,$zhUser['PASSWORD']);
                    } catch (\Exception $e) {
                        try{
                            list($zhUser['NRPARCNEGOCIO'],$zhUser['CDOPERADOR']) = $this->insertOperador($zhUser['NAME'], $zhUser['EMAIL'] . '1'
                                , $nrorg,$zhUser['PASSWORD']);
                        } catch (\Exception $e) {
                            echo "\n".$e->getMessage()."\n";
                            unset($zhUsers[$key]);
                        }
                    }
                }
            }
        }
        return $zhUsers;
    }

    public function getOperador($email){
        $operadores = $this->getOperadores(array($email));
        return empty($operadores)? null : $operadores[0];
    }

    public function getZhProfile($orgIdentityId)
    {
        $query = <<<SQL
    	  SELECT ZPR.ID,ZPR.LABEL,CASE WHEN ZPR.IS_ADMINISTRATOR = 'Y' THEN 'S' ELSE 'N' END AS IS_ADMINISTRATOR
            FROM ZH_USER ZHU
       LEFT JOIN ZH_USER_PROFILE ZUP ON (ZUP.USER_ID = ZHU.ID)
            JOIN ZH_PROFILE ZPR ON (ZUP.PROFILE_ID = ZPR.ID)
           WHERE ZHU.ORGANIZATION_ID = :orgIdentityId
             AND ZHU.STATUS = 'A'
             AND ZPR.IS_ACTIVE = 'Y'
        GROUP BY ZPR.ID,ZPR.LABEL,ZPR.IS_ADMINISTRATOR
SQL;


        $param = array(
            ':orgIdentityId' => $orgIdentityId

        );
        return $this->entityManagerIdentity->getConnection()->fetchAll($query, $param);
    }

    public function getZhUserProfile($orgIdentityId)
    {
        $query = <<<SQL
    	  SELECT ZUP.USER_ID,ZUP.PROFILE_ID
            FROM ZH_USER ZHU
       LEFT JOIN ZH_USER_PROFILE ZUP ON (ZUP.USER_ID = ZHU.ID)
            JOIN ZH_PROFILE ZPR ON (ZUP.PROFILE_ID = ZPR.ID)
           WHERE ZHU.ORGANIZATION_ID = :orgIdentityId
             AND ZHU.STATUS = 'A'
             AND ZPR.IS_ACTIVE = 'Y'
        GROUP BY ZUP.USER_ID,ZUP.PROFILE_ID
SQL;


        $param = array(
            ':orgIdentityId' => $orgIdentityId

        );
        return $this->entityManagerIdentity->getConnection()->fetchAll($query, $param);
    }

    public function setNrPerfilasso($zhProfiles, $zhUserProfile)
    {
        foreach ($zhProfiles as $zhProfile) {
            if ($zhUserProfile['PROFILE_ID'] == $zhProfile['ID']) {
                return $zhProfile['NRPERFILACESSO'];
            }
        }
        return null;
    }

    public function setNrparc($zhUsers, $zhUserProfile)
    {
        foreach ($zhUsers as $zhUser) {
            if ($zhUser['ID'] == $zhUserProfile['USER_ID']) {
                return $zhUser['NRPARCNEGOCIO'];
            }
        }
        return null;
    }

    public function setPerfils($orgIdentityId, $nrorg, $zhUsers, $perfilacesso)
    {
        $zhProfiles = $this->getZhProfile($orgIdentityId);
        $zhUserProfiles = $this->getZhUserProfile($orgIdentityId);

        foreach ($zhProfiles as $key => &$zhProfile) {
            foreach ($perfilacesso as $perfil) {
                if($perfil['NMPERFILACESSO'] == $zhProfile['LABEL']){
                    $zhProfile['NRPERFILACESSO'] = $perfil['NRPERFILACESSO'];
                    $this->migrationPerfilacesso ++;
                }
            }
        }

        foreach ($zhProfiles as &$zhProfile) {
            if(empty($zhProfile['NRPERFILACESSO'])){
                $zhProfile['NRPERFILACESSO'] = $this->insertPerfilacesso($zhProfile['LABEL'], $zhProfile['IS_ADMINISTRATOR'], $nrorg);
            }
        }

        foreach ($zhUserProfiles as $zhUserProfile) {
            $zhUserProfile['NRPERFILACESSO'] = $this->setNrPerfilasso($zhProfiles, $zhUserProfile);
            $zhUserProfile['NRPARCNEGOCIO'] = $this->setNrparc($zhUsers, $zhUserProfile);
            $perfacessparc = $this->findPerfacessparc($zhUserProfile['NRPERFILACESSO'],$zhUserProfile['NRPARCNEGOCIO']);
            if (empty($perfacessparc) && $zhUserProfile['NRPERFILACESSO'] && $zhUserProfile['NRPARCNEGOCIO']) {
                $this->insertPerfacessparc($zhUserProfile['NRPARCNEGOCIO'], $zhUserProfile['NRPERFILACESSO'], $nrorg);
            }
        }
        return $zhProfiles;
    }

    public function findPerfacessparc($nrperfilacesso,$nrparcnegocio){

        $query = <<<SQL
    	  SELECT NRPERFILACESSO,NRPARCNEGOCIO
            FROM PERFACESSPARC
           WHERE NRPERFILACESSO = :NRPERFILACESSO
             AND NRPARCNEGOCIO  = :NRPARCNEGOCIO
SQL;


        $param = array(
            'NRPERFILACESSO' => $nrperfilacesso,
            'NRPARCNEGOCIO' => $nrparcnegocio,

        );
        return $this->entityManagerSaas->getConnection()->fetchAll($query, $param);
    }

    public function setRelationZhUser($zhUsers)
    {
        $relatedIdUsers = array();
        foreach ($zhUsers as $zhUser) {
            $relatedIdUsers[$zhUser['ID']] = $zhUser['CDOPERADOR'];
        }

        $this->mongoScript->setRelatedIdUsers($relatedIdUsers);
    }

    public function setRelationZhProfile($zhProfiles)
    {
        $relatedAccessProfiles = array();
        foreach ($zhProfiles as $zhProfile) {
            $relatedAccessProfiles[$zhProfile['ID']] = ''.$zhProfile['NRPERFILACESSO'];
        }

        $this->mongoScript->setRelatedAccessProfiles($relatedAccessProfiles);
    }

    public function getProfileTeknisa($orgIdentityId, $perfilacesso)
    {
        $zhProfiles = array();
        if($perfilacesso){
            $zhProfiles = $this->getZhProfile($orgIdentityId);

            foreach ($zhProfiles as $key => &$zhProfile) {
                foreach ($perfilacesso as $perfil) {
                    if($perfil['NMPERFILACESSO'] == $zhProfile['LABEL']){
                        $zhProfile['NRPERFILACESSO'] = $perfil['NRPERFILACESSO'];
                        $this->migrationPerfilacesso ++;
                    }
                }
                if(empty($zhProfile['NRPERFILACESSO'] )){
                    unset($zhProfiles[$key]);
                }
            }
        }
        return $zhProfiles;
    }

    public function echoMessageEnd()
    {
        $migrationOperador = $this->migrationOperador;
        $migrationPerfilacesso = $this->migrationPerfilacesso;
        $migrationPerfacessparc = $this->migrationPerfacessparc;

        echo "\n Foram migrados $migrationOperador operadores e $migrationPerfilacesso perfis de acesso com suas  $migrationPerfacessparc ligações com os operadores ";
        echo "\n ";
    }

    public function echoMessage($message)
    {

        echo "\n $message";
    }

    public function setPerfilacesso($orgIdentityId, $nrorg, $zhUsers)
    {
        $perfilacesso = $this->getPerfilacesso($nrorg);
        $zhProfiles = $this->setPerfils($orgIdentityId, $nrorg, $zhUsers,$perfilacesso);

        if (empty($zhProfiles)) {
            echo "Perfis não migrado por ja existir alguns criados na base de destino\n";
        } else {
            $this->setRelationZhProfile($zhProfiles);
        }
    }

    public function setOperadores($orgIdentityId, $nrorg)
    {
        $zhUsers = $this->getZhUsers($orgIdentityId);

        $emails = array();
        foreach ($zhUsers as $zhUser) {
            $emails[] = $zhUser['EMAIL'];
        }

        $operadores = $this->getOperadores($emails);
        $zhUsers = $this->setNrparceiro($nrorg, $operadores, $zhUsers);
        $this->setRelationZhUser($zhUsers);
        return $zhUsers;
    }
}
