<?php

namespace Service;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;

class PeriodsService {
    
    protected $dataSourceName = 'periods';
    protected $orgMongoManager;
    protected $baseMongoManager;
    
    public function __construct (Manager $orgMongoManager, Manager $baseMongoManager){
        $this->orgMongoManager = $orgMongoManager;
        $this->baseMongoManager = $baseMongoManager;
    }
    
    public function getPeriodsFromBase(){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $response = $this->baseMongoManager->findBy($filterCriteria)->getRows();
        
        foreach ($response as &$row) {
            $row['base'] = true;
        }
        
        return $response;
    }
    
    public function getPeriodsFromOrg(){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $response = $this->orgMongoManager->findBy($filterCriteria)->getRows();
        return $response;
    }



    public function getDateByDateExpression($dateExpression){
        $result = array();
        foreach ($dateExpression as $expression) {
            $result[] = PeriodMathExpression\Period::getDate($expression);
        }
        return $result;

    }

    public function findByCriteria($manager,$columnName, $value, $operator = '=') {
        $criteria = array( array(
            "columnName" => $columnName,
            "value" => $value,
            "operator" => $operator
        ));

        $filterCriteria = new FilterCriteria($this->dataSourceName, $criteria);
        return $manager->findBy($filterCriteria)->getRows();
    }

    public function findById($idPeriod) {
        $period = $this->findByCriteria($this->orgMongoManager,'_id', $idPeriod);
        if(empty($period)){
            $period = $this->findByCriteria($this->baseMongoManager,'_id', $idPeriod);
        }
        return count($period) > 0 ? $period[0] : null;
    }
}