/*
@C:\WEB_645\SQL\END_CONV\ORACLE\5289_ESTSTPO_GERA_EST_ARMAZ.SQL

 Descrio:
 Procedure para gerao de estoque na filial de armazenagem

 Criao:
 10/2024 - Jonathan Emmanuel Nunes Ferreira

 RAISE_APPLICATION_ERROR(-20501, '' , TRUE);
*/


CREATE OR REPLACE PROCEDURE GERA_EST_ARMAZ(
        P_FILIAL   IN NOTAFISCAL.CDFILIAL%TYPE,    --FILIAL OL
        P_LANCTONF IN NOTAFISCAL.NRLANCTONF%TYPE,  --N LANCAMENTO
        P_LANCESTQ IN LANCTOESTOQ.NRLANCESTQ%TYPE, --N LANCAMENTO DE ESTOQUE. SE NULO
        P_FILIARM  IN NOTAFISCAL.CDFILIAL%TYPE,    --FILIAL DE ARMAZENAGEM
        P_TELA     IN VARCHAR2,
        P_RETORNO OUT VARCHAR2)
IS
   /*ENTRADA*/
   V_NRLANCESTQ      VARCHAR2(20);
   V_DATAINIENTR     DATE;
   V_DATAFINENTR     DATE;
   V_CDFILIMOVI      VARCHAR2(04);
   V_CDOPERADOR      VARCHAR2(20);
   V_DTLANCESTQ      DATE;
   V_NRSEQITEM       NUMBER;
   V_ERRO            VARCHAR2(4000);
BEGIN
   V_NRLANCESTQ := NULL;
   V_NRSEQITEM := 1;
   P_RETORNO := 'OK';
   V_ERRO := NULL;

   BEGIN
      IF (P_TELA = 'EST11000') THEN /*Entrada de Estoque */
         /*Excluir lanamento replicado em caso de alterao*/
         DELETE
           FROM ITLANCTOEST
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         DELETE
           FROM LANCTOESTOQ
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         BEGIN
            SELECT L.DTLANCESTQ, L.CDFILIMOVI, L.CDOPERADOR
              INTO V_DTLANCESTQ, V_CDFILIMOVI, V_CDOPERADOR
              FROM LANCTOESTOQ L
             WHERE L.CDFILIAL   = P_FILIAL
               AND L.NRLANCESTQ = P_LANCESTQ;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_DTLANCESTQ := NULL; V_CDFILIMOVI := NULL; V_CDOPERADOR := NULL;
         END;

         IF (V_CDFILIMOVI IS NOT NULL) THEN
            NOVO_CODIGO_SC('LANCTOEST'    ||P_FILIARM, V_NRLANCESTQ);
            V_NRLANCESTQ := SUBSTR(V_NRLANCESTQ, 11, 10);

            INSERT INTO LANCTOESTOQ
            (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
            NRLANCTONF, CDFILIMOVI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ, CDFILIALOL, NRLANCESTQOL)
            VALUES
            (P_FILIARM, V_NRLANCESTQ, '2', TRUNC(V_DTLANCESTQ), 'Entrada Ref. Lan. Manual na Filial OL: '||V_CDFILIMOVI||' - '|| P_LANCESTQ, 'S',
            NULL, P_FILIARM, V_CDOPERADOR, NULL, NULL, P_FILIAL, P_LANCESTQ);

            FOR C_EST IN (SELECT IT.CDFILIAL,    IT.NRLANCESTQ,   IT.NRSEQUITEM, IT.IDTIPOMOVI, IT.CDPRODUTO,      IT.QTTOTLANCTO,
                                 IT.VRUNILANCTO, IT.VRTOTLANCTO,  IT.DTLANCMOVI, IT.IDORDEMCLC, IT.CDPRODMOVI,     IT.QTLANCTOEST,
                                 IT.VRLANCTOBRUT, IT.VRLANCTOEST, IT.CDFILIMOVI, IT.CDALMOXARIFE, IT.CDLOCALESTOQ, IT.NRLOTEESTQ,
                                 IT.NRSUBLOTE
                            FROM LANCTOESTOQ L, ITLANCTOEST IT
                           WHERE L.CDFILIAL   = P_FILIAL
                             AND L.NRLANCESTQ = P_LANCESTQ
                             AND L.CDFILIAL   = IT.CDFILIAL
                             AND L.NRLANCESTQ = IT.NRLANCESTQ)
               LOOP
                  INSERT
                    INTO ITLANCTOEST
                         (CDFILIAL, NRLANCESTQ, NRSEQUITEM, IDTIPOMOVI, CDPRODUTO, QTTOTLANCTO, VRUNILANCTO,
                          VRTOTLANCTO, DTLANCMOVI, IDORDEMCLC, CDPRODMOVI, QTLANCTOEST, VRLANCTOBRUT, VRLANCTOEST,
                          CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE)
                  VALUES
                         (P_FILIARM, V_NRLANCESTQ, TO_CHAR(LPAD(V_NRSEQITEM, 4, '0')), '2', C_EST.CDPRODUTO, C_EST.QTTOTLANCTO, C_EST.VRUNILANCTO,
                         C_EST.VRTOTLANCTO, TRUNC(V_DTLANCESTQ), '2', C_EST.CDPRODMOVI, C_EST.QTLANCTOEST,
                         C_EST.VRLANCTOBRUT, C_EST.VRLANCTOEST, P_FILIARM, ' ', ' ', ' ', ' ');

                  V_NRSEQITEM := V_NRSEQITEM + 1;
            END LOOP;
            IF TRUNC(V_DTLANCESTQ) < TRUNC(SYSDATE) THEN
               V_DATAINIENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAINIENTR := TRUNC(SYSDATE);
            END IF;

            IF TRUNC(V_DTLANCESTQ) > TRUNC(SYSDATE) THEN
               V_DATAFINENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAFINENTR := TRUNC(SYSDATE);
            END IF;
            ATUALIZA_ESTOQUE(P_FILIARM, V_DATAINIENTR, V_DATAFINENTR, P_FILIARM, V_NRLANCESTQ);
            COMMIT;
            P_RETORNO := 'OK';
         END IF;
      END IF;

      IF (P_TELA = 'EST21000') THEN  /* Saida de Estoque */
         /*Excluir lanamento replicado em caso de alterao*/
         DELETE
           FROM ITLANCTOEST
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         DELETE
           FROM LANCTOESTOQ
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         BEGIN
            SELECT L.DTLANCESTQ, L.CDFILIMOVI, L.CDOPERADOR
              INTO V_DTLANCESTQ, V_CDFILIMOVI, V_CDOPERADOR
              FROM LANCTOESTOQ L
             WHERE L.CDFILIAL   = P_FILIAL
               AND L.NRLANCESTQ = P_LANCESTQ;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_DTLANCESTQ := NULL; V_CDFILIMOVI := NULL; V_CDOPERADOR := NULL;
         END;

         IF (V_CDFILIMOVI IS NOT NULL) THEN
            NOVO_CODIGO_SC('LANCTOEST'    ||P_FILIARM, V_NRLANCESTQ);
            V_NRLANCESTQ := SUBSTR(V_NRLANCESTQ, 11, 10);

            INSERT INTO LANCTOESTOQ
            (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
            NRLANCTONF, CDFILIMOVI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ, CDFILIALOL, NRLANCESTQOL)
            VALUES
            (P_FILIARM, V_NRLANCESTQ, '4', TRUNC(V_DTLANCESTQ), 'Saida Ref. Lan. Manual na Filial OL: '||V_CDFILIMOVI||' - '|| P_LANCESTQ, 'S',
            NULL, P_FILIARM, V_CDOPERADOR, NULL, NULL, P_FILIAL, P_LANCESTQ);

            FOR C_EST IN (SELECT IT.CDFILIAL,    IT.NRLANCESTQ,   IT.NRSEQUITEM, IT.IDTIPOMOVI, IT.CDPRODUTO,      IT.QTTOTLANCTO,
                                 IT.VRUNILANCTO, IT.VRTOTLANCTO,  IT.DTLANCMOVI, IT.IDORDEMCLC, IT.CDPRODMOVI,     IT.QTLANCTOEST,
                                 IT.VRLANCTOBRUT, IT.VRLANCTOEST, IT.CDFILIMOVI, IT.CDALMOXARIFE, IT.CDLOCALESTOQ, IT.NRLOTEESTQ,
                                 IT.NRSUBLOTE
                            FROM LANCTOESTOQ L, ITLANCTOEST IT
                           WHERE L.CDFILIAL   = P_FILIAL
                             AND L.NRLANCESTQ = P_LANCESTQ
                             AND L.CDFILIAL   = IT.CDFILIAL
                             AND L.NRLANCESTQ = IT.NRLANCESTQ)
               LOOP
                  INSERT
                    INTO ITLANCTOEST
                         (CDFILIAL, NRLANCESTQ, NRSEQUITEM, IDTIPOMOVI, CDPRODUTO, QTTOTLANCTO, VRUNILANCTO,
                          VRTOTLANCTO, DTLANCMOVI, IDORDEMCLC, CDPRODMOVI, QTLANCTOEST, VRLANCTOBRUT, VRLANCTOEST,
                          CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE)
                  VALUES
                         (P_FILIARM, V_NRLANCESTQ, TO_CHAR(LPAD(V_NRSEQITEM, 4, '0')), '4', C_EST.CDPRODUTO, C_EST.QTTOTLANCTO, C_EST.VRUNILANCTO,
                         C_EST.VRTOTLANCTO, TRUNC(V_DTLANCESTQ), '2', C_EST.CDPRODMOVI, C_EST.QTLANCTOEST,
                         C_EST.VRLANCTOBRUT, C_EST.VRLANCTOEST, P_FILIARM, ' ', ' ', ' ', ' ');

                  V_NRSEQITEM := V_NRSEQITEM + 1;
            END LOOP;
            IF TRUNC(V_DTLANCESTQ) < TRUNC(SYSDATE) THEN
               V_DATAINIENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAINIENTR := TRUNC(SYSDATE);
            END IF;

            IF TRUNC(V_DTLANCESTQ) > TRUNC(SYSDATE) THEN
               V_DATAFINENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAFINENTR := TRUNC(SYSDATE);
            END IF;
            ATUALIZA_ESTOQUE(P_FILIARM, V_DATAINIENTR, V_DATAFINENTR, P_FILIARM, V_NRLANCESTQ);
            COMMIT;
            P_RETORNO := 'OK';
         END IF;
      END IF;

      IF (P_TELA = 'EST58000') THEN /* Retirada de Estoque */
         /*Excluir lanamento replicado em caso de alterao*/
         DELETE
           FROM ITLANCTOEST
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         DELETE
           FROM LANCTOESTOQ
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         BEGIN
            SELECT L.DTLANCESTQ, L.CDFILIMOVI, L.CDOPERADOR
              INTO V_DTLANCESTQ, V_CDFILIMOVI, V_CDOPERADOR
              FROM LANCTOESTOQ L
             WHERE L.CDFILIAL   = P_FILIAL
               AND L.NRLANCESTQ = P_LANCESTQ;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_DTLANCESTQ := NULL; V_CDFILIMOVI := NULL; V_CDOPERADOR := NULL;
         END;

         IF (V_CDFILIMOVI IS NOT NULL) THEN
            NOVO_CODIGO_SC('LANCTOEST'    ||P_FILIARM, V_NRLANCESTQ);
            V_NRLANCESTQ := SUBSTR(V_NRLANCESTQ, 11, 10);

            INSERT INTO LANCTOESTOQ
            (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
            NRLANCTONF, CDFILIMOVI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ, CDFILIALOL, NRLANCESTQOL)
            VALUES
            (P_FILIARM, V_NRLANCESTQ, '3', TRUNC(V_DTLANCESTQ), 'Retirada Ref. Lan. Manual na Filial OL: '||V_CDFILIMOVI||' - '|| P_LANCESTQ, 'S',
            NULL, P_FILIARM, V_CDOPERADOR, NULL, NULL, P_FILIAL, P_LANCESTQ);

            FOR C_EST IN (SELECT IT.CDFILIAL,    IT.NRLANCESTQ,   IT.NRSEQUITEM, IT.IDTIPOMOVI, IT.CDPRODUTO,      IT.QTTOTLANCTO,
                                 IT.VRUNILANCTO, IT.VRTOTLANCTO,  IT.DTLANCMOVI, IT.IDORDEMCLC, IT.CDPRODMOVI,     IT.QTLANCTOEST,
                                 IT.VRLANCTOBRUT, IT.VRLANCTOEST, IT.CDFILIMOVI, IT.CDALMOXARIFE, IT.CDLOCALESTOQ, IT.NRLOTEESTQ,
                                 IT.NRSUBLOTE
                            FROM LANCTOESTOQ L, ITLANCTOEST IT
                           WHERE L.CDFILIAL   = P_FILIAL
                             AND L.NRLANCESTQ = P_LANCESTQ
                             AND L.CDFILIAL   = IT.CDFILIAL
                             AND L.NRLANCESTQ = IT.NRLANCESTQ)
               LOOP
                  INSERT
                    INTO ITLANCTOEST
                         (CDFILIAL, NRLANCESTQ, NRSEQUITEM, IDTIPOMOVI, CDPRODUTO, QTTOTLANCTO, VRUNILANCTO,
                          VRTOTLANCTO, DTLANCMOVI, IDORDEMCLC, CDPRODMOVI, QTLANCTOEST, VRLANCTOBRUT, VRLANCTOEST,
                          CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE)
                  VALUES
                         (P_FILIARM, V_NRLANCESTQ, TO_CHAR(LPAD(V_NRSEQITEM, 4, '0')), '3', C_EST.CDPRODUTO, C_EST.QTTOTLANCTO, C_EST.VRUNILANCTO,
                         C_EST.VRTOTLANCTO, TRUNC(V_DTLANCESTQ), '2', C_EST.CDPRODMOVI, C_EST.QTLANCTOEST,
                         C_EST.VRLANCTOBRUT, C_EST.VRLANCTOEST, P_FILIARM, ' ', ' ', ' ', ' ');

                  V_NRSEQITEM := V_NRSEQITEM + 1;
            END LOOP;
            IF TRUNC(V_DTLANCESTQ) < TRUNC(SYSDATE) THEN
               V_DATAINIENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAINIENTR := TRUNC(SYSDATE);
            END IF;

            IF TRUNC(V_DTLANCESTQ) > TRUNC(SYSDATE) THEN
               V_DATAFINENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAFINENTR := TRUNC(SYSDATE);
            END IF;
            ATUALIZA_ESTOQUE(P_FILIARM, V_DATAINIENTR, V_DATAFINENTR, P_FILIARM, V_NRLANCESTQ);
            COMMIT;
            P_RETORNO := 'OK';
         END IF;
      END IF;

      IF (P_TELA = 'EST42000') THEN  /* Implantacao de Estoque */
         /*Excluir lanamento replicado em caso de alterao*/
         DELETE
           FROM ITLANCTOEST
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         DELETE
           FROM LANCTOESTOQ
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         BEGIN
            SELECT L.DTLANCESTQ, L.CDFILIMOVI, L.CDOPERADOR
              INTO V_DTLANCESTQ, V_CDFILIMOVI, V_CDOPERADOR
              FROM LANCTOESTOQ L
             WHERE L.CDFILIAL   = P_FILIAL
               AND L.NRLANCESTQ = P_LANCESTQ;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_DTLANCESTQ := NULL; V_CDFILIMOVI := NULL; V_CDOPERADOR := NULL;
         END;

         IF (V_CDFILIMOVI IS NOT NULL) THEN
            NOVO_CODIGO_SC('LANCTOEST'    ||P_FILIARM, V_NRLANCESTQ);
            V_NRLANCESTQ := SUBSTR(V_NRLANCESTQ, 11, 10);

            INSERT INTO LANCTOESTOQ
            (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
            NRLANCTONF, CDFILIMOVI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ, CDFILIALOL, NRLANCESTQOL)
            VALUES
            (P_FILIARM, V_NRLANCESTQ, '0', TRUNC(V_DTLANCESTQ), 'Implantacao Ref. Lan. Manual na Filial OL: '||V_CDFILIMOVI||' - '|| P_LANCESTQ, 'S',
            NULL, P_FILIARM, V_CDOPERADOR, NULL, NULL, P_FILIAL, P_LANCESTQ);

            FOR C_EST IN (SELECT IT.CDFILIAL,    IT.NRLANCESTQ,   IT.NRSEQUITEM, IT.IDTIPOMOVI, IT.CDPRODUTO,      IT.QTTOTLANCTO,
                                 IT.VRUNILANCTO, IT.VRTOTLANCTO,  IT.DTLANCMOVI, IT.IDORDEMCLC, IT.CDPRODMOVI,     IT.QTLANCTOEST,
                                 IT.VRLANCTOBRUT, IT.VRLANCTOEST, IT.CDFILIMOVI, IT.CDALMOXARIFE, IT.CDLOCALESTOQ, IT.NRLOTEESTQ,
                                 IT.NRSUBLOTE
                            FROM LANCTOESTOQ L, ITLANCTOEST IT
                           WHERE L.CDFILIAL   = P_FILIAL
                             AND L.NRLANCESTQ = P_LANCESTQ
                             AND L.CDFILIAL   = IT.CDFILIAL
                             AND L.NRLANCESTQ = IT.NRLANCESTQ)
               LOOP
                  INSERT
                    INTO ITLANCTOEST
                         (CDFILIAL, NRLANCESTQ, NRSEQUITEM, IDTIPOMOVI, CDPRODUTO, QTTOTLANCTO, VRUNILANCTO,
                          VRTOTLANCTO, DTLANCMOVI, IDORDEMCLC, CDPRODMOVI, QTLANCTOEST, VRLANCTOBRUT, VRLANCTOEST,
                          CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE)
                  VALUES
                         (P_FILIARM, V_NRLANCESTQ, TO_CHAR(LPAD(V_NRSEQITEM, 4, '0')), '0', C_EST.CDPRODUTO, C_EST.QTTOTLANCTO, C_EST.VRUNILANCTO,
                         C_EST.VRTOTLANCTO, TRUNC(V_DTLANCESTQ), '2', C_EST.CDPRODMOVI, C_EST.QTLANCTOEST,
                         C_EST.VRLANCTOBRUT, C_EST.VRLANCTOEST, P_FILIARM, ' ', ' ', ' ', ' ');

                  V_NRSEQITEM := V_NRSEQITEM + 1;
            END LOOP;
            IF TRUNC(V_DTLANCESTQ) < TRUNC(SYSDATE) THEN
               V_DATAINIENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAINIENTR := TRUNC(SYSDATE);
            END IF;

            IF TRUNC(V_DTLANCESTQ) > TRUNC(SYSDATE) THEN
               V_DATAFINENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAFINENTR := TRUNC(SYSDATE);
            END IF;
            ATUALIZA_ESTOQUE(P_FILIARM, V_DATAINIENTR, V_DATAFINENTR, P_FILIARM, V_NRLANCESTQ);
            COMMIT;
            P_RETORNO := 'OK';
         END IF;
      END IF;

      IF (P_TELA = 'EST41000') THEN  /* Ajuste de Inventario */
         /*Excluir lanamento replicado em caso de alterao*/
         DELETE
           FROM ITLANCTOEST
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         DELETE
           FROM LANCTOESTOQ
          WHERE CDFILIALOL   = P_FILIAL
            AND NRLANCESTQOL = P_LANCESTQ;

         BEGIN
            SELECT L.DTLANCESTQ, L.CDFILIMOVI, L.CDOPERADOR
              INTO V_DTLANCESTQ, V_CDFILIMOVI, V_CDOPERADOR
              FROM LANCTOESTOQ L
             WHERE L.CDFILIAL   = P_FILIAL
               AND L.NRLANCESTQ = P_LANCESTQ;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_DTLANCESTQ := NULL; V_CDFILIMOVI := NULL; V_CDOPERADOR := NULL;
         END;

         IF (V_CDFILIMOVI IS NOT NULL) THEN
            NOVO_CODIGO_SC('LANCTOEST'    ||P_FILIARM, V_NRLANCESTQ);
            V_NRLANCESTQ := SUBSTR(V_NRLANCESTQ, 11, 10);

            INSERT INTO LANCTOESTOQ
            (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
            NRLANCTONF, CDFILIMOVI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ, CDFILIALOL, NRLANCESTQOL)
            VALUES
            (P_FILIARM, V_NRLANCESTQ, '1', TRUNC(V_DTLANCESTQ), 'Ajuste Ref. Lan. Manual na Filial OL: '||V_CDFILIMOVI||' - '|| P_LANCESTQ, 'S',
            NULL, P_FILIARM, V_CDOPERADOR, NULL, NULL, P_FILIAL, P_LANCESTQ);

            FOR C_EST IN (SELECT IT.CDFILIAL,    IT.NRLANCESTQ,   IT.NRSEQUITEM, IT.IDTIPOMOVI, IT.CDPRODUTO,      IT.QTTOTLANCTO,
                                 IT.VRUNILANCTO, IT.VRTOTLANCTO,  IT.DTLANCMOVI, IT.IDORDEMCLC, IT.CDPRODMOVI,     IT.QTLANCTOEST,
                                 IT.VRLANCTOBRUT, IT.VRLANCTOEST, IT.CDFILIMOVI, IT.CDALMOXARIFE, IT.CDLOCALESTOQ, IT.NRLOTEESTQ,
                                 IT.NRSUBLOTE
                            FROM LANCTOESTOQ L, ITLANCTOEST IT
                           WHERE L.CDFILIAL   = P_FILIAL
                             AND L.NRLANCESTQ = P_LANCESTQ
                             AND L.CDFILIAL   = IT.CDFILIAL
                             AND L.NRLANCESTQ = IT.NRLANCESTQ)
               LOOP
                  INSERT
                    INTO ITLANCTOEST
                         (CDFILIAL, NRLANCESTQ, NRSEQUITEM, IDTIPOMOVI, CDPRODUTO, QTTOTLANCTO, VRUNILANCTO,
                          VRTOTLANCTO, DTLANCMOVI, IDORDEMCLC, CDPRODMOVI, QTLANCTOEST, VRLANCTOBRUT, VRLANCTOEST,
                          CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE)
                  VALUES
                         (P_FILIARM, V_NRLANCESTQ, TO_CHAR(LPAD(V_NRSEQITEM, 4, '0')), '1', C_EST.CDPRODUTO, C_EST.QTTOTLANCTO, C_EST.VRUNILANCTO,
                         C_EST.VRTOTLANCTO, TRUNC(V_DTLANCESTQ), '2', C_EST.CDPRODMOVI, C_EST.QTLANCTOEST,
                         C_EST.VRLANCTOBRUT, C_EST.VRLANCTOEST, P_FILIARM, ' ', ' ', ' ', ' ');

                  V_NRSEQITEM := V_NRSEQITEM + 1;
            END LOOP;
            IF TRUNC(V_DTLANCESTQ) < TRUNC(SYSDATE) THEN
               V_DATAINIENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAINIENTR := TRUNC(SYSDATE);
            END IF;

            IF TRUNC(V_DTLANCESTQ) > TRUNC(SYSDATE) THEN
               V_DATAFINENTR := TRUNC(V_DTLANCESTQ);
            ELSE
               V_DATAFINENTR := TRUNC(SYSDATE);
            END IF;
            ATUALIZA_ESTOQUE(P_FILIARM, V_DATAINIENTR, V_DATAFINENTR, P_FILIARM, V_NRLANCESTQ);
            COMMIT;
            P_RETORNO := 'OK';
         END IF;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
         ROLLBACK;
         V_ERRO := SQLERRM(SQLCODE);
         P_RETORNO := 'E';

         INSERT INTO HMA_MONITORAINT(DSTABELA, CDDADOS, DSERRO, IDPROCESSO)
         VALUES('Erro na replicao(Armazenagem) - '||P_TELA, P_FILIAL||P_LANCESTQ, V_ERRO, '1');

         COMMIT;
   END;
   /*
         INSERT INTO LANCTOESTOQ
      (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
      NRLANCTONF, CDFILIMOVI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ)
      VALUES
      ('3969', V_NRLANCESTQ, '4', TRUNC(V_DTLANCESTQ), 'Saida Ref. Remessa incorreta da NF: '||C_NOTAS.CDFILIAL||' - '|| C_NOTAS.NRLANCTONF, 'S',
      NULL, '3969', '000018454658', NULL, NULL);

   */
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('USR_GERA_EST_ARMAZ','6.49.076'); END;
/


