var TemplateManager = (function(){

	var SLASH = "/";
	var DEFAULT_SERVICE_URL = "../backend/service/index.php";
	var DEFAULT_META_DATA_URL = "./";
	var ZEEDHI_META_DATA_URL = "./bower_components/zeedhi-frontend/assets/";

	var _zhPromise, _window, _zhFeatures, _http;
	var self, mappedPackages, mappedServicePackages, commonFilesSourceMap, dataSourcesDefer, lastFolder;

	function TemplateManager($window, zhPromise, ZhFeatures, $http) {
		self = this;
		_window = $window;
		_zhPromise = zhPromise;
		_zhFeatures = ZhFeatures;
		_http = $http;
		setWindowParameters(_window);
		buildBaseUrlMap();
		buildDocsUrlMap();
		this.project = {
			aclEnable: false
		};
		this.projectPromise = _zhPromise.defer();
		dataSourcesDefer = _zhPromise.defer();

		_window.metadataUrls = addSlashToUrls(_window.metadataUrls);
		_window.metadataUrl = addSlashToUrl(_window.metadataUrl || DEFAULT_META_DATA_URL);

		checkMetadata(_window.metadataUrl, _window.metadataUrls, "metadataUrl");
		mappedPackages = buildPackagesMap(_window.metadataUrl, _window.metadataUrls);
		mappedPackages[this.ZEEDHI_PACKAGE] = mappedPackages[this.ZEEDHI_PACKAGE] || ZEEDHI_META_DATA_URL;

		this.buildMapOfServicePackages(_window.serviceUrl, _window.serviceUrls);

		commonFilesSourceMap = buildCommonJsonFilesSourceMap(_window.metadataUrl, _window.metadataUrls);

		lastFolder = getLastFolder();
	}

	function setWindowParameters(_window) {
		self.docsUrl = _window.docsUrl;
		self.docsUrls = _window.docsUrls;
		self.showMenu = _window.showMenu;
		self.defaultUserImageUrl = _window.defaultUserImageUrl;
		self.templateUrl = _window.templateUrl;
		self.templateUrls = _window.templateUrls;
		self.inDeveloperMode = _window.inDeveloperMode;
		self.developerWebSocketUrl = _window.developerWebSocketUrl;
		self.zhFeaturesWebSocketUrl = _window.zhFeaturesWebSocketUrl;
		self.versionFile = _window.versionFile;
		self.dependenciesFolder = _window.dependenciesFolder;
		self.backendDependenciesFolder = _window.backendDependenciesFolder;
	}

	function addSlashToUrls(urls) {
		return Util.parseToArray(urls).map(function(url){
			if (Util.isString(url)) {
				url = addSlashToUrl(url);
			} else if (url.baseUrl) {
				url.baseUrl = addSlashToUrl(url.baseUrl);
			}
			return url;
		});
	}

	function addSlashToUrl(url) {
		if (url && url[url.length - 1] !== SLASH) {
			url += SLASH;
		}
		return url;
	}

	function getLastFolder() {
		var zeedhiUrl = self.templateUrls
			.filter(function(item) {
				return item.zeedhi === true;
			})
			.map(function(item) {
				return item.baseUrl;
			}).shift();

		var paths = zeedhiUrl.split('/');
		return paths[paths.length - 2]; //the last value is always empty
	}

	function checkMetadata(metadataUrl, metadataUrls, errorData) {
		checkPackageData(metadataUrls, errorData);
	}

	function checkServiceUrl(serviceUrl, serviceUrls, errorData) {
		checkPackageData(serviceUrls, errorData);
	}

	function checkPackageData(urls, errorData) {
		if(urls.length){
			var hasSomeObject = urls.some(function(pckg){
				return Util.isObject(pckg);
			});
			for (var i = 0; i < urls.length; i++) {
				var pckg = urls[i];
				if ((Util.isString(pckg) && hasSomeObject) ||
					(Util.isObject(pckg) && (!pckg.baseUrl || !pckg.package))) {
					//throw new Error("Invalid "+errorData+" packages");
					_zhFeatures.useMappedPackagesUrlWithStringUrl();
				}
			}
		}
	}

	function buildPackagesMap(defaultPackage, packagesUrl) {
		var packages = { default: [] };

		if (defaultPackage) {
			packages.default.push(defaultPackage);
		}

		if(packagesUrl.length){
			packages = packagesUrl.reduce(function(current, pckg){
				if (Util.isObject(pckg)) {
					current[pckg.package] = pckg.baseUrl;
				} else if (Util.isString(pckg)) {
					current.default.push(pckg);
				}
				return current;
			}, packages);
		}

		return packages;
	}

	function buildCommonJsonFilesSourceMap(metadataUrl, metadataUrls) {
		var commonFilesSourceMap = {};
		commonFilesSourceMap.acl = [metadataUrl];
		commonFilesSourceMap.words = [metadataUrl];
		commonFilesSourceMap.data = [metadataUrl];

		if (metadataUrls.length) {
			metadataUrls.forEach(function(pckg){
				if (Util.isObject(pckg)) {
					if (pckg.hasAcl) {
						commonFilesSourceMap.acl.push(pckg.baseUrl);
					}
					if (pckg.hasWords) {
						commonFilesSourceMap.words.push(pckg.baseUrl);
					}
					if (pckg.hasSharedData) {
						commonFilesSourceMap.data.push(pckg.baseUrl);
					}
				} else if (Util.isString(pckg)) {
					commonFilesSourceMap.acl.push(pckg);
					commonFilesSourceMap.words.push(pckg);
					commonFilesSourceMap.data.push(pckg);
				}
			});
		}

		return commonFilesSourceMap;
	}

	TemplateManager.prototype.getMappedPackages = function() {
		return Util.clone(mappedPackages);
	};

	TemplateManager.prototype.getMappedServicePackages = function() {
		return Util.clone(mappedServicePackages);
	};

	TemplateManager.prototype.buildMapOfServicePackages = function(serviceUrl, serviceUrls) {
		serviceUrls = serviceUrls || [];
		checkServiceUrl(serviceUrl, serviceUrls, "serviceUrl");
		mappedServicePackages = buildPackagesMap(serviceUrl, serviceUrls);
	};

	TemplateManager.prototype.getCommonFilesSource = function() {
		return Util.clone(commonFilesSourceMap);
	};

	TemplateManager.prototype.getTemplateDefaultFolder = getZeedhiUrl;

	function buildBaseUrlMap() {
		var templateManager = self;
		var hasMultipleTemplateUrls = templateManager.templateUrls && templateManager.templateUrls.length;

		var hasDefaultUrl = hasMultipleTemplateUrls && templateManager.templateUrls.filter(function(templateUrl) {
			return templateUrl.package === "default";
		}).length;

		var hasFrameworkUrl = hasMultipleTemplateUrls && templateManager.templateUrls.filter(function(pckg) {
			return pckg.zeedhi;
		}).length;

		if(!hasMultipleTemplateUrls) {
			window.templateUrls = [];
			templateManager.templateUrls = [];
		}

		if(templateManager.templateUrl && !hasDefaultUrl) {
			templateManager.templateUrls.unshift({
				"package": "default",
				"baseUrl": templateManager.templateUrl
			});
		}

		if(!hasFrameworkUrl) {
			var defaultPackage = templateManager.templateUrls.filter(function(templateUrl) {
				return templateUrl.package === "default";
			}).shift();

			if (!defaultPackage) {
				throw new Error("The templateUrls must have a default template when there is not templateUrl");
			}

			defaultPackage.zeedhi = true;
		}
	}

	function buildDocsUrlMap(){
		var templateManager = self;
		var hasDocsUrl = templateManager.docsUrl;
		var hasMultipleDocsUrls = templateManager.docsUrls && templateManager.docsUrls.length;
		var hasDefaultUrl = hasMultipleDocsUrls && templateManager.docsUrls.filter(function(docsUrl) {
			return docsUrl.package === "default";
		}).length;

		if(!hasDocsUrl){
			var urlDefault = getBaseUrl("default", templateManager.templateUrls);
			urlDefault = urlDefault.substring(0, urlDefault.indexOf("/templates/"));
			templateManager.docsUrl = urlDefault + "/docs/dyman/";
		}
		if(!hasMultipleDocsUrls){
			window.docsUrls = [];
			templateManager.docsUrls = [];
		}

		if(!hasDefaultUrl) {
			templateManager.docsUrls.unshift({
				"package": "default",
				"baseUrl": templateManager.docsUrl
			});
		}
	}

	function getBaseUrl(packageName, urls) {
		urls = urls ? urls : self.templateUrls;
		var baseUrlArray = urls.filter(function(item) {
				return item.package === packageName;
			})
			.map(function(item) {
				return item.baseUrl;
			});
		return baseUrlArray[0];
	}

	function getZeedhiUrl() {
		var zeedhiUrl = self.templateUrls
			.filter(function(item) {
				return item.zeedhi === true;
			})
			.map(function(item) {
				return item.baseUrl;
			});

		if (lastFolder === 'templates') {
			zeedhiUrl[0] += 'dyman/';
		}

		return zeedhiUrl[0];
	}

	function getRealUrl(abstractURL, urls) {
		var realUrl;

		if (abstractURL) {
			var parts = abstractURL.split("#");
			var baseUrl;

			if (parts.length > 1) {
				baseUrl = getBaseUrl(parts[0], urls);
				if (!baseUrl) {
					throw new Error("There is no templateUrl with package " + parts[0]);
				}
				realUrl = baseUrl + parts[1];
			} else {
				baseUrl = getBaseUrl("default", urls);
				realUrl = baseUrl + parts[0];
			}
		}

		return realUrl;
	}

	function getRealDocsUrl(abstractURL){
		var docsUrl = self.docsUrl;
		var urls = self.docsUrls;
		if(urls || urls.length > 0){
			docsUrl = getRealUrl(abstractURL, urls);
		}
		return docsUrl;
	}

	TemplateManager.prototype.getFrameworkTemplateURL = function(abstractURL) {
		return getZeedhiUrl() + abstractURL;
	};

	TemplateManager.prototype.getDocsUrl = function(abstractURL) {
		return getRealDocsUrl(abstractURL);
	};

	TemplateManager.prototype.getTemplateURL = function(abstractURL) {
		return getRealUrl(abstractURL, self.templateUrls);
	};

	TemplateManager.prototype.updateTemplate = function() {
		if (this.onUpdate) {
			return this.onUpdate().data;
		} else {
			var deferred = _zhPromise.defer();
			deferred.reject();
			return deferred.promise;
		}
	};

	TemplateManager.prototype.menuVisible = function(visible) {
		this.project.menuVisible = visible;
		this.updateTemplate();
	};

	TemplateManager.prototype.getDataSources = function() {
		return dataSourcesDefer.promise;
	};

	TemplateManager.prototype.updateURL = function(uri) {
		window.__changeURI(uri);
	};

	function getDependency(libName) {
		var project = self.project;
		var versionFileName = project.versionFile.replace(/\.\.\//g, '').replace(/\.\//g, '');
		var libVersionFile = [project.dependenciesFolder, libName, versionFileName].join('/');

		return _http.get(libVersionFile).then(function(response) {
			var version = response.data.version || "";
			var name = response.data.name || libName;
			self.versionData.dependencies.frontend.push({name: name, version: version});
		});
	}

	function getBackendDependencies(project) {
		return _zhPromise.all([_http.get(project.backendVersionFile), _http.get([project.backendDependenciesFolder, "composer.lock"].join('/'))])
			.then(function(response) {
				if(response[0].data.require){
					var keys = Object.keys(response[0].data.require);
					response[1].data.packages.forEach(function(lib) {
						if(keys.indexOf(lib.name) !== -1){
							self.versionData.dependencies.backend.push({name: lib.name, version: lib.version});
						}
					});
				}
		});
	}

	function getFrontendDependencies(project) {
		return _http.get(project.versionFile).then(function(response) {
			var promises = [];
			var version = response.data.version || "";
			var productName = response.data.name;

			self.versionData.name = productName;
			self.versionData.version = version;
			if (response.data.dependencies) {
				promises = Object.keys(response.data.dependencies).map(function(libName) {
					return getDependency(libName);
				});
			}

			self.versionData.dependencies.current = self.versionData.dependencies.frontend;

			return _zhPromise.all(promises);
		});
	}

	TemplateManager.prototype.updateVersionData = function() {
		var project = self.project;

		self.versionData = {
			name: "",
			version: "",
			dependencies: {
				frontend: [],
				backend: [],
				current: [],
				modules: [],
				changeCurrentVersion: function(newVersion){
					switch (newVersion) {
						case 'frontend':
							self.versionData.dependencies.current = self.versionData.dependencies.frontend;
							break;
						case 'backend':
							self.versionData.dependencies.current = self.versionData.dependencies.backend;
							break;
						case 'modules':
							self.versionData.dependencies.current = self.versionData.dependencies.modules;
							break;
						default:
							break;
					}
				}
			}
		};

		return getBackendDependencies(project).then(function() {
			getFrontendDependencies(project);
		});
	};

	TemplateManager.prototype.setDataSources = function(data) {
		this.data = data;
		dataSourcesDefer.resolve(data);
	};

	TemplateManager.prototype.updateTitle = function(title) {
		document.title = title;
	};

	TemplateManager.prototype.ZEEDHI_PACKAGE = 'zeedhi';

	return TemplateManager;
})();