var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhNewGrid', function($timeout, templateManager) {

	function calcContentWidth(element) {
		element[0]._gridColumns = [];
		element.find('.header .tr .th').each(function(colIndex, col) {
			var i, fieldName = col.getAttribute('data-zh-field-name');

			// reset previous resizing computation
			var colElements = element[0].querySelectorAll('[data-zh-field-name="' + fieldName + '"]');
			for (i = 0; i < colElements.length; i++) {
				colElements[i].style.width = 'auto';
				colElements[i].style.marginLeft = 0;
			}

			if (element[0].__scope) {
				var widget = element[0].__scope.widget;
				var field = widget.getField(fieldName);
				var colWidth = 0;
				var fixed = (' '+col.className+' ').indexOf(' fixed ') > -1;
				var minWidth = field ? parseInt(field.minWidth || 0) : (widget.columnProperties ? parseInt(widget.columnProperties.minWidth || 0) : 0);
				var maxWidth = Math.max((field ? parseInt(field.maxWidth || 99999) : (widget.columnProperties ? parseInt(widget.columnProperties.maxWidth ||
					99999) : 99999)), minWidth);
				var actionWidth = screen.width >= 1200 ? 40 : 50;
				var cellPadding = screen.width >= 1200 ? 12 : 16;

				if (fieldName !== "__CHECKBOX") {
					minWidth = Math.max(minWidth, 40);
				}

				if (fieldName == "__LEFT_ACTIONS") {
					colWidth = (element[0].__scope.visibleLeftActions * actionWidth) + cellPadding;
				} else if (fieldName == "__RIGHT_ACTIONS") {
					colWidth = (element[0].__scope.visibleRightActions * actionWidth) + cellPadding;
				} else {
					for (i = 0; i < colElements.length; i++) {
						colWidth = Math.min(Math.max(colWidth, colElements[i].clientWidth + 5), maxWidth);
					}
					
					colWidth = Math.min(Math.max(colWidth, minWidth), maxWidth) + cellPadding;
					
				}

				if (field && !Util.isUndefined(field.inlineEditable)) {
					colWidth += 10;
				}

				if (field && field.inlineEditable === 'checkbox'){
					minWidth = maxWidth = colWidth;
				}

				element[0]._gridColumns.push({
					"name": fieldName,
					"width": colWidth,
					"fixed": fixed,
					"minWidth": minWidth,
					"maxWidth": maxWidth,
					"colElements": colElements
				});
			}
		});
	}

	function adjustWidths(element) {
		var fixedWidth = 0;
		var leftActionsWidth = 0;
		var rightActionsWidth = 0;
		var dynamicCols = [];
		var dynamicWidth = 0;
		var newDynamicWidth = 0;

		element[0]._gridColumns.forEach(function(col) {
			if (col.name == '__LEFT_ACTIONS') {
				if (Util.isPhone()) {
					leftActionsWidth += col.width;
				} else {
					fixedWidth += col.width;
				}
			} else if (col.name == '__RIGHT_ACTIONS') {
				rightActionsWidth += col.width;
			} else if (col.fixed) {
				fixedWidth += col.width;
			} else {
				dynamicWidth += col.width;
				dynamicCols.push(col);
			}
		});

		var gridWidth = element.innerWidth() - 10; // space to add scrollbar
		if (element.parent().css('position') !== "absolute") {
			gridWidth += parseInt(element.parent().css('padding-left')) * 2;
		}
		var columnsWidth = fixedWidth + leftActionsWidth + rightActionsWidth + dynamicWidth;
		if (columnsWidth < gridWidth && dynamicCols.length > 0) {
			var maxWidthAdjust = 0;
			dynamicCols.forEach(function(col, idx) {
				var newWidth = col.width * (gridWidth - fixedWidth - leftActionsWidth - rightActionsWidth) / dynamicWidth;
				if (maxWidthAdjust > 0) {
					newWidth += maxWidthAdjust;
				}
				if (newWidth > col.maxWidth) {
					maxWidthAdjust += (newWidth - col.maxWidth) / (dynamicCols.length - idx - 1);
					col.width = col.maxWidth;
				} else {
					col.width = newWidth;
				}
				newDynamicWidth += col.width;
			});
		} else {
			newDynamicWidth = dynamicWidth;
		}

		var rightActionsMargin = gridWidth - (fixedWidth + leftActionsWidth + rightActionsWidth + newDynamicWidth);
		if (rightActionsMargin > 0) {
			element.find('.right-grid-actions').css('margin-left', rightActionsMargin);
		}

		var scrollControl = element.find('.body-scroll-control');
		scrollControl.css('left', fixedWidth);
		scrollControl.find('div').css('width', newDynamicWidth + leftActionsWidth + rightActionsWidth);
		scrollControl.scrollLeft(0);

		if (Util.isUndefined(element[0]._gridScrollEvents)) {
			var body = element.find('.body');
			var gridId = element.attr('id');

			scrollControl.on('scroll.' + gridId, function(e) {
				var left = $(e.target).scrollLeft();
				element.find('.tr').each(function(index, tr) {
					if (index === 0) {
						$(tr).find('.th:not(.fixed)').eq(0).css('margin-left', -left);
					} else {
						$(tr).find('.td.zh-standard-column:not(.fixed):not(.ng-hide)').eq(0).css('margin-left', -left);
					}
				});
			});

			body.on('DOMMouseScroll.' + gridId + ' mousewheel.' + gridId, function(e) {
				var deltaX = e.originalEvent.wheelDeltaX;
				if (deltaX !== 0) {
					scrollControl[0].scrollLeft -= deltaX;
					e.preventDefault();
				}
			});

			body.on('touchstart.' + gridId, function(e) {
				element[0]._touchstartX = e.originalEvent.touches[0].pageX;
				element[0]._touchstartY = e.originalEvent.touches[0].pageY;
			});

			body.on('touchmove.' + gridId, function(e) {
				var deltaX = e.originalEvent.touches[0].pageX - element[0]._touchstartX;
				if (deltaX !== 0) {
					element[0]._touchstartX = e.originalEvent.touches[0].pageX;
					scrollControl[0].scrollLeft -= deltaX;
				}

				var deltaY = e.originalEvent.touches[0].pageY - element[0]._touchstartY;
				if (deltaY !== 0) {
					element[0]._touchstartY = e.originalEvent.touches[0].pageY;
					var topScrollElement = element.find('.grid-vertical-scroll-control')[0];
					if(topScrollElement){
						topScrollElement.scrollTop -= deltaY;
					}
					if (element[0].__scope.widget._calcTopRow) {
						element[0].__scope.widget._calcTopRow();
					}
				}
			});

			element[0].__scope.$on('$destroy', function() {
				scrollControl.off('scroll.' + gridId);
				body.off('DOMMouseScroll.' + gridId + ' mousewheel.' + gridId);
				body.off('touchstart.' + gridId);
				body.off('touchmove.' + gridId);
			});

			element[0]._gridScrollEvents = true;
		}
	}

	function resizeColumns(element) {
		element[0]._gridColumns.forEach(function(col) {
			for (var i = 0; i < col.colElements.length; i++) {
				col.colElements[i].style.width = col.width+'px';
			}
		});
	}

	function resizeGrid(element, forceRecalc) {
		if ((element[0].__scope && element[0].__scope.widget && element[0].__scope.widget._gridResize) || forceRecalc) {
			element[0]._gridColumns = undefined;
		}
		if (Util.isUndefined(element[0]._gridColumns)) {
			if (element[0].__scope && element[0].__scope.widget) element[0].__scope.widget._gridResize = false;
			calcContentWidth(element);
			adjustWidths(element);
		}
		resizeColumns(element);
		showData(element,forceRecalc);
	}

	function animateRedrawing(element) {
		hideData(element);
	}

	function hideData(element) {
		element.find('.th, .tr').css('opacity', 0);
	}

	function showData(element, animate) {
		if (animate) {
			element.find('.th, .tr').animate({
				'opacity': 1
			}, 300);
		} else {
			element.find('.th, .tr').css('opacity', 1);
		}
	}

	$(window).on('resize.grid', function(event, element) {
		var grids;
		if (Util.isUndefined(element)) {
			grids = $('.zh-new-grid');
		} else if (Util.isString(element)) {
			grids = $('#grid-' + element);
		} else {
			grids = $(element).find('.zh-new-grid');
		}
		grids.each(function(index, grid) {
			$timeout(function() {
				resizeGrid($(grid), true);
				$(window).trigger('resize.grid.after');
			}, 300, false);
		});
	});

	return {
		link: function($scope, element, attrs, controller) {
			element[0].__scope = $scope;
			
			element.on('$destroy', function() {
				element[0].__scope = undefined;
			});
			
			var _resizeGrid = Util.buildDebounceMethod(function(forceRecalc) {
				$timeout(function() {
					resizeGrid(element, forceRecalc);
				}, 0, false);
			}, 300);

			$scope.resizeGridColumns = function(forceRecalc) {
				_resizeGrid(forceRecalc);
			};
			$scope.widget.resizeGridColumns = $scope.resizeGridColumns;

			$scope.widget.redraw = function gridRedraw(force) {
				animateRedrawing(element);
				$scope.widget.updateGroups();
				$scope.widget.updateFilteredFields();
				$scope.widget.resizeGridColumns(force);
			};

			if ($scope.widget.onRender) {
				$scope.widget.onRender(function() {
					$scope.buildGridActions();
					$scope.widget.reload();
					$scope.resizeGridColumns(true);
				});
			}

			$scope.widget._getVisibilityParams = function() {
				var rowHeight = screen.width > 1200 ? 36 : 47;
				var gridBodyArea = element.find('.body').outerHeight();
				// To fix the behavior of body height being 0 even if the body visually have a height, a fallback will be used
				if (!gridBodyArea) {
					var headerHeight = element.find('.header').height();
					var bodyScrollControlHeight = element.find('.body-scroll-control').height();
					gridBodyArea = element.height() - headerHeight - bodyScrollControlHeight;
					if (gridBodyArea <= 0) {
						gridBodyArea = 0;
					}
				}
				$scope.widget._visibilityParams = {
					rowHeight: rowHeight,
					maxVisibleRows: Math.floor(gridBodyArea / rowHeight)
				};
			};

			$scope.widget._calcScrollAreas = function _calcScrollAreas() {
				var totalHeight = 0;
				var body = element.find('.body');
				var visibilityParams = $scope.widget._visibilityParams;
				// var heightAdjust = body.outerHeight() - (visibilityParams ? visibilityParams.rowHeight * visibilityParams.maxVisibleRows : 0);

				if ($scope.widget._hiddenRows > 0) {
					totalHeight = ($scope.widget._hiddenRows * (visibilityParams ? visibilityParams.rowHeight : 0));
				}

				var scrollControl = element.find('.grid-vertical-scroll-control');
				scrollControl.find('div').height(totalHeight + scrollControl.height());
			};

			$scope.widget._calcTopRow = function _calcTopRow() {
				if ($(':focus').is('.inline-edit')) {
					$(':focus').blur();
				}

				var body = element.find('.body');
				var scrollControl = element.find('.grid-vertical-scroll-control');
				var visibilityParams = $scope.widget._visibilityParams;
				$scope.widget._topRow = Math.round(scrollControl.scrollTop() / (visibilityParams ? visibilityParams.rowHeight : 0));
				$scope.widget._updateVisibleRows();
				if (!Util.isMobile()) {
					$scope.widget._calcScrollAreas();
				}

				if (scrollControl[0] && (scrollControl[0].scrollTop + scrollControl[0].offsetHeight) >= (scrollControl[0].scrollHeight - 10)) {
					body.find('.body-content').trigger('scroll.infinity', $scope.widget.hasPagination);
				}

				templateManager.updateTemplate();
			};

			var _debounceCalcTopRow = Util.buildDebounceMethod($scope.widget._calcTopRow, 200);
			var _throttleCalcTopRow = Util.buildThrottleMethod($scope.widget._calcTopRow, 10, $scope.widget);

			if (!Util.isMobile()) {
				element.find('.grid-vertical-scroll-control').on('scroll', _throttleCalcTopRow);
			} else {
				element.find('.grid-vertical-scroll-control').on('scroll', _debounceCalcTopRow);
			}

			element.find('.body-content').on('mousewheel wheel DOMMouseScroll', function(e) {
				element.find('.grid-vertical-scroll-control')[0].scrollTop += e.originalEvent.deltaY;
			});
		}
	};
});