module.exports = function(grunt) {

	var controllersPath = 'src/interval/controllers/**/*.js';
	var configPath = 'src/interval/config/**/*.js';
	var registersPath = 'src/interval/registers/**/*.js';
	var assetsPath = ['src/interval/*/*.js'];

	grunt.initConfig({
		jshint: {
			all: ['GruntFile.js', assetsPath]
		},
		watch: {
			pivotal: {
				files: [assetsPath],
				tasks: ['jshint']
			}
		},
		bump: {
			options: {
				files: ['bower.json', 'package.json'],
				pushTo: 'origin',
				commitFiles: ["-a"],
				push: true
			}
		},
		concat: {
			zh: {
				options: {
					process: function(src, filepath) {
						return '\n' + '// FILE: ' + filepath + '\n' + src;
					}
				},
				src: [configPath, registersPath, controllersPath],
				dest: 'dist/zh-interval-component.js'
			}
		},
		uglify: {
			options: {
				compress: false,
				mangle: false
			},
			target: {
				files: {
					'dist/zh-interval-component.min.js': ['dist/zh-interval-component.js']
				}
			}
		},
		copy: {
			main: {
				expand: true,
				cwd: 'src/interval/',
				src: 'templates/**/*',
				dest: 'dist/'
			}
		},
		buildEnvironment: {
			development: {
				configFile: 'example/config/development.json'
			},
			deploy: {
				configFile: 'example/config/deploy.json'
			},
			srcFolders: ['example/frontend', 'tests/functional/']
		}
	});

	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-copy');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-bump');

	// Default task.
	grunt.registerTask('default', ['jshint', 'concat', 'uglify', 'copy']);
};