zh-interval-component
====
**zh-interval-component** is a component which allows user to select an interval of dates using an calendar widget.

## Installation
**1)** First of all, check if you have Private Bower's Registry registered in your **.bowerrc** file.
```
{
    "registry": {
        "search": [
            "http://192.168.122.56:5678/"
        ]
    }
}
```

**2)** Then install via Bower:
```
$ bower install --save zh-interval-component
```

**3)** After installed, you need to link the main component's JavaScript file in your **index.html**:
```
<script src="path_to_bower_components/zh-interval-component/dist/zh-interval-component.js">
```
Or the minified version:
```
<script src="path_to_bower_components/zh-interval-component/dist/zh-interval-component.min.js">
```


**4)** **Zh Interval** already registers itself as a package. The default `baseUrl` registered is:
```javascript
<script type="text/javascript">
    window.templateUrls = [
        {
            package: "zh-interval-component",
            baseUrl: "bower_components/zh-interval-component/dist/templates/"
        }
    ]
</script>
```

So, it considers you have `bower_components` folder in the same path that your `index.html`.

If your `bower_components` is in another folder, you can register your own package path, overwriting the default's one:


```javascript
<script type="text/javascript">
    window.templateUrls = [
        {
            package: "zh-interval-component",
            baseUrl: "path_to_bower_components/zh-interval-component/dist/templates/"
        }
    ]
</script>
```


Be careful: if you already have a `window.templateUrls` registered, just insert the package's information inside of that. **Do not** override with another instance of `window.templateUrls`.

## Using as a field in your project
After installed, the component can normally be used in a JSON file:
```
"fields": [
    {
        "label": "My field"
        "name": "MY_FIELD",
        "template": "zh-interval-component#field/interval.html"
    }
]
```
*Remember: zh-interval-component is the package's name registered in your **index.html** using `window.templateUrls`*

## Properties

### Date format
**Zh Interval** uses the format `YYYY-MM-DD` as default. But if your project works with another format (e.g. brazilian format, `DD/MM/YYYY`), you can configure it with `dateFormat` property.
```
fields: [{
	  "name": "interval",
	  "label": "Period",
	  "isVisible": true,
	  "template": "field/interval.html",
	  "dateFormat": "DD/MM/YYYY"
}]
```

### Display format
Display format is how the dates are shown on view. For example, your project may work with american dates in its backend but, on view, shows it as brazilian format. You can configure it with `displayFormat` property.
```
fields: [{
	  "name": "interval",
	  "label": "Period",
	  "isVisible": true,
	  "template": "field/interval.html",
	  "displayFormat": "DD/MM/YYYY"
}]
```

### Start label & End label
These properties are simple: they just define how labels are shown on interval widget's top content. The default are `Start on` and `End on`. But if you want to configure it, just use `startLabel` and `endLabel` properties.
```
fields: [{
	  "name": "interval",
	  "label": "Period",
	  "isVisible": true,
	  "template": "field/interval.html",
	  "startLabel": "Start on",
	  "endLabel": "End on"
}]
```

### Intervals
The component supports periods, which are a list of pre-determined intervals.  You can use them by just adding the `intervals` property in your field declaration:
```
fields: [{
	  "name": "interval",
	  "label": "Period",
	  "isVisible": true,
	  "template": "field/interval.html",
	  "events": [],
	  "intervals": [
	      {
	          "label": "Today",
	          "period": "today"
	      },
	      {
	          "label": "Next Month",
	          "period": "next-month"
	      }
	   ]
}]
```

**Zh Interval** already comes with some default intervals periods:
```
'today', 'yesterday', 'tomorrow'
'this-week', 'last-week', 'next-week'
'this-quarter', 'last-quarter', 'next-quarter'
'this-month', 'last-month', 'next-month'
'this-year', 'last-year', 'next-year'
```

### How to create a custom interval
To create a custom interval, you need to create a object inside `"intervals":` property:

- `"label":` "Title that will be displayed in periods",
- `"perido":` "@customFunctionToRun()@"

**Example:**

```json
"intervals": [{
	"label": "My Custom Period",
	"period": "@SampleController.getCustomPeriod()@"
}]
```

To set `"period"` you need to create a function using `@` to delimit  your custom function.

In your controller you will need to create a function that will return a object that contain your custom period:

**And set the new period properties:**
- `"start":` "YYYY-MM-DD",
- `"end":`  "YYYY-MM-DD"

**Example**

```javascript
this.getCustomPeriod = function(period) {
		return {
			"start":"2017-07-16",
			"end":"2017-09-22"
		};
	};
```

### Default Value
You can set `"defaultValue":` property to make the field been filled by default. To set default values, you need to follow the date format as **YYYY/MM/DD**.

Using `"start":` and `"end":` properties.

* `start:` Initial date format as YYYY/MM/DD.
* `end:` End date format as YYYY/MM/DD.

```
"defaultValue": {
	"start": "2017/05/10",
	"end": "2017/05/20"
},
```
### Validations

To validate the field you need to use `"validations":` property.
Inside Zh Interval validations you have 3 properties that can be used:

* `"requiredInterval":` General validation, just validate the field;
* `"requiredIntervalStart":` Validate the Start Field Interval property;
* `"requiredIntervalEnd":`  Validate the End Field Interval property.

If you want to create a custom message, you will need to set the property `"message":` inside the "required" properties as below.

Validations using custom `"message":`

Single validation:

```
"validations": {
 	"requiredInterval": {
       	"message": "Start and End date is Required!"
 	}
 }
```
Or validate both fields independently:

```
"validations": {
 	"requiredIntervalStart": {
 		"message": "Start Date Required!"
 	}, 	
    "requiredIntervalEnd": {
 		"message": "End Date Required!"
 	}
 }
```
Default Validations without `message` property will show **Required field** as default message:

Single validation:

```
"validations": {
 	"requiredIntervalStart": {} 	
 }
```

Or validate both fields independently:

```
"validations": {
	"requiredIntervalStart": {},
	"requiredIntervalEnd": {}
}
```
### Create a help message using Tooltip
Like Zeedhi fields, zh-interval supports the property `"help":`.
This property create a tooltip message for the users.
To add support a tooltip you just need to set the property inside your Widget.
**Sample:** 
```
"help": "JUST A HELP MESSAGE"
```

## Values on Widget Row
As all other Zeedhi Components, **Zh Interval** applies values to the `currentRow`. The component creates a property which has its name. This property is an object and stores some other properties:
* field_name.**start**: *returns the selected start date*
* field_name.**end**: *returns the selected end date*
* field_name.period.**name**: *returns the selected period name*
* field_name.period.**label**: *returns the selected period label*.