module.exports = function(grunt) {

	var packageName = 'zh-select-autocomplete';

	var servicesPath = 'src/services/**/*.js';
	var controllersPath = 'src/controllers/**/*.js';
	var directivesPath = 'src/directives/**/*.js';
	var registersPath = 'src/registers/**/*.js';
	var configPath = 'src/config/**/*.js';
	var assetsPath = ['src/**/*.js'];
	var exceptPath = ['!src/utilities/**/'];

	var exampleControllers = ['example/frontend/js/**/*.js'];

	grunt.initConfig({
		jshint: {
			all: ['GruntFile.js', assetsPath, exceptPath]
		},
		watch: {
			pivotal: {
				files: [assetsPath],
				tasks: ['jshint']
			}
		},
		bump: {
		    options: {
		        files: ['bower.json', 'package.json'],
		        pushTo: 'origin',
		        commitFiles: ["-a"],
		        push: true
		    }
		},
		concat: {
			zh: {
				options: {
					process: function(src, filepath) {
						return '\n' + '// FILE: ' + filepath + '\n' + src;
					}
				},
				src: [configPath, servicesPath, controllersPath, directivesPath, registersPath, exceptPath],
				dest: 'dist/' + packageName + '.js'
			},
			example: {
				options: {
					process: function(src, filepath) {
						return '\n' + '// FILE: ' + filepath + '\n' + src;
					}
				},
				src: [exampleControllers],
				dest: 'example/frontend/dist/' + packageName + '-examples.js'
			}
		},
		zhstyles: {
			options: {
				stylesPath: "dist/style/colors",
				cssPath: "src/styles/zh-select-autocomplete.css"
			}
		},
		uglify: {
			options: {
				mangle: false
			},
			target: {
				files: {
					'dist/zh-select-autocomplete.min.js': ['dist/' + packageName + '.js']
				}
			}
		},
		copy: {
			main: {
				expand: true,
				cwd: 'src/',
				src: 'templates/**/*',
				dest: 'dist/'
			}
		},
		buildEnvironment: {
			development: {
				configFile: 'config/development.json'
			},
			deploy: {
				configFile: 'config/deploy.json'
			},
			srcFolders: ['example/frontend', 'tests/functional']
		},
		jsdoc: {
			dist: {
				options: {
					configure: 'jsdoc.conf.json',
					recurse: true
				}
			}
		}
	});

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-copy');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-bump');
	grunt.loadNpmTasks('grunt-zh-styles');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-jsdoc');

	// Default task.
	grunt.registerTask('default', ['jshint', 'concat']);
	grunt.registerTask('buildDev', ['jshint', 'concat', 'copy', 'uglify', 'zhstyles', 'buildEnvironment', 'jsdoc']);
	grunt.registerTask('deploy', ['jshint', 'concat', 'copy', 'uglify', 'zhstyles', 'buildEnvironment:deploy', 'jsdoc']);
	grunt.registerTask('doc', ['jsdoc']);
};