(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zmartDashboardCard', zmartDashboardCard);

    function zmartDashboardCard() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            function setColorByValue() {
                var color;
                var dataSourceRow = widget.dataSource.data[0];
                var conditions = widget.cardConditions || [];
                var colors = widget.cardColors || [];
                var value = widget.value;
                if(value !== undefined){
                    if(conditions && conditions[0]){
                        for(var i=0;i<conditions.length;i++){
                            var condition=parseFloat(conditions[i]);
                            if(isNaN(condition) || condition === null){
                                condition = dataSourceRow[conditions[i]] || null;
                            }
                            if(condition !== null && value >= condition){
                                color = colors[i%colors.length];
                            }
                        }
                    }else{
                        color = colors[0];
                    }
                }
                widget.color = color;
                element.css('background-color', color);
            }

            setColorByValue();

            scope.$watch(
                function() {
                    return widget.value;
                },
                function(newColor) {
                    setColorByValue();
                    applyWidgetColorOnElement();
                }
            );

            function fontShouldBeWhite(color) {
                var c = color.substring(1); // strip #
                var rgb = parseInt(c, 16); // convert rrggbb to decimal
                var r = (rgb >> 16) & 0xff; // extract red
                var g = (rgb >> 8) & 0xff; // extract green
                var b = (rgb >> 0) & 0xff; // extract blue

                var luma = 0.2126 * r + 0.7152 * g + 0.0722 * b;
                return luma < 155;
            }

            function applyWidgetColorOnElement() {
                if (widget.color) {
                    var color = widget.color || '#e2a53b';
                    if (fontShouldBeWhite(widget.color)) {
                        element.css('color', '#fff');
                    } else {
                        element.css('color', '#000');
                    }
                    element.css('background-color', color);
                }
            }
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCardGrid', zmartDashboardCardGrid);

    function zmartDashboardCardGrid() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var gridHeight = element.height();
                var gridRows = element.find('.card-grid-row, .card-grid-row-group');
                gridRows.css({
                    height: gridHeight/gridRows.length+'px',
                    lineHeight: gridHeight/gridRows.length+'px'
                });
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCard3', zmartDashboardCard3);

    function zmartDashboardCard3() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var gridHeight = element.height();
                element.find("li > span").css({
                    height: gridHeight/2+'px',
                    lineHeight: gridHeight/2+'px'
                });
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCard4Chart', zmartDashboardCard4Chart);

    function zmartDashboardCard4Chart() {

        var linkFunction = function(theScope, theElement, attrs) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var widgetItem = widget.widgets[parseInt(attrs.zmartDashboardCard4Chart)];
                var options = {
                    title: false,
                    subtitle: false,
                    fontName: "open_sanslight, Helvetica, Arial, sans-serif",
                    series: [{
                        type: "progress",
                        colors: widgetItem.progressParameters.color,
                        value: widgetItem.progressParameters.value,
                        maxValue: widgetItem.progressParameters.maxValue,
                        showValueLabels: false,
                        progressWidth: 0.2
                    }]
                };

                var chart = zhChart(options);
                chart(element[0]);
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCard4Grid', zmartDashboardCard4Grid);

    function zmartDashboardCard4Grid() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var gridHeight = element.height();
                var gridRows = element.find('li');
                element.find("li").css({
                    height: gridHeight/gridRows.length+'px'
                });
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();