function AclAccessProfileController(ScreenService, templateManager, localStorageEngine, ZhuAclController,
                                    AclMessages, AccessProfileService, ACLUtilities, ZHPromise, ZhuAclExportRulesController){

	var self = this;
	var ALE_FILL_REQ_FIELD = AclMessages.ALE_FILL_REQ_FIELD;
	var ERR_DEL_PROFILE = AclMessages.ERR_DEL_PROFILE;
	var ERR_SAV_PROFILE = AclMessages.ERR_SAV_PROFILE;
	var ERR_SAV_PROFILE_OPERATORS = AclMessages.ERR_SAV_PROFILE_OPERATORS;
	var CON_DEL_PROFILE = AclMessages.CON_DEL_PROFILE;
	var CON_DEL_PROFILE_OPERATORS = AclMessages.CON_DEL_PROFILE_OPERATORS;
	var SUC_DEL_OPERATOR = AclMessages.SUC_DEL_OPERATOR;
	var CON_CHEN_ACCESS_PROFILE = AclMessages.CON_CHEN_ACCESS_PROFILE;

	self.profileWidgetAfterMoveRow = function() {
		return self.setChildFilters(self.getWidgetProfile());
	};

	self.profileActionEventNew = function() {
		return self.addProfile(self.getWidgetProfile());
	};

	self.profileActionEventEnableAccessControll = function() {
		return self.enableAccessControll();
	};

	self.profileActionEventBack = function() {
		return self.masterDetailBackToList(self.getWidgetProfile());
	};

	self.profileActionEventCancel = function() {
		return self.profileRollback(self.getWidgetProfile());
	};

	self.profileActionEventEdit = function() {
		return self.editProfile(self.getWidgetProfile());
	};

	self.profileActionEventDelete = function() {
		return self.deleteProfileConfirmation(self.getWidgetProfile());
	};

	self.profileActionEventExportRules = function() {
		var profileWidget = self.getWidgetProfile();
		ZhuAclExportRulesController.profileActionEventExportRules(profileWidget.currentRow.NRPERFILACESSO);
	};

	self.profileActionEventSave = function() {
		return self.saveProfile(self.getWidgetProfile());
	};

	self.operatorsWidgetOnEnter = function() {
		return self.getWidgetOperators().reload();
	};

	self.operatorsActionEventNew = function() {
		return self.loadOperators(self.getWidgetOperators());
	};

	self.operatorsActionEventBack = function() {
		return self.masterDetailBackToList(self.getWidgetOperators().parent);
	};

	self.addOperatorActionEventAdd = function(currentWidget) {
		return self.addPerfAcessParc(self.getWidgetAddOperator(), currentWidget);
	};

	self.addOperatorActionEventCancel = function(currentWidget) {
		if(currentWidget && currentWidget.unSelectAll){
			currentWidget.unSelectAll();
		}
		self.getWidgetTabbedChildrenAddOperators().currentRow.DTVALIDADE = '';
		ScreenService.closeSwipe();
	};

	self.getContainerAccessProfile = function(){
		return templateManager.container;
	};

	self.getWidgetProfile = function() {
		return templateManager.container.getWidget('profile');
	};

	self.getWidgetOperators = function() {
		return templateManager.container.getWidget('operators');
	};

	self.getWidgetTabbedChildrenAddOperators = function() {
		return templateManager.container.getWidget('TABBED_CHILDREN_ADD_OPERATOR');
	};

	self.getWidgetAddOperator = function() {
		return templateManager.container.getWidget('addOperator');
	};

	self.getWidgetElements = function() {
		return templateManager.container.getWidget('elements');
	};

	self.getActionViewFromElements = function() {
		return self.getWidgetElements().getAction('view');
	};

	self.getActionEnableAccessProfile = function() {
		return self.getWidgetProfile().getAction('enableAccessProfile');
	};

	self.getActionEditFromElements = function() {
		return self.getWidgetElements().getAction('edit');
	};

	self.setWidgetsVisibility = function (arrayWidgets, visibility) {
		arrayWidgets.forEach(
			function (element) {
				element.isVisible = visibility;
			}
		);
	};

	self.addProfile = function(accessProfileWidget) {
		self.setWidgetsVisibility(accessProfileWidget.widgets, false);
		accessProfileWidget.newRow();
	};

	self.editProfile = function (accessProfileWidget) {
		self.setWidgetsVisibility(accessProfileWidget.widgets, false);
		accessProfileWidget.backupRow = Util.clone(accessProfileWidget.currentRow);
		accessProfileWidget.edit();
	};

	self.profileRollback = function (accessProfileWidget) {
		if (accessProfileWidget.isNewRow()) {
			if (accessProfileWidget.dataSource.data.indexOf(accessProfileWidget.currentRow) !== -1) {
				accessProfileWidget.dataSource.data.pop();
			}
			self.masterDetailBackToList(accessProfileWidget);
		}
		else {
			Util.extend(accessProfileWidget.currentRow, accessProfileWidget.backupRow);
			accessProfileWidget.view();
		}
		self.setWidgetsVisibility(accessProfileWidget.widgets, true);
	};

	self.masterDetailBackToList = function(accessProfileWidget) {
		ScreenService.closeSwipe().then(function() {
			accessProfileWidget.list();
			accessProfileWidget.activate();
		});
	};

	self.deleteProfileConfirmation = function(){
		ScreenService.confirmMessage(
			CON_DEL_PROFILE,
			'ALERT',
			removeProfile
		);
	};

	var removeProfile = function(){
		var accessProfileWidget = self.getWidgetProfile();
		accessProfileWidget.currentRow.__is_new = false;
		accessProfileWidget.removeRow().then(function(){
			ScreenService.closeSwipe();
		}).catch(function (error) {
			ScreenService.errorNotification(ERR_DEL_PROFILE + error.error);
		});
	};

	self.setChildFilters = function (accessProfileWidget) {
		var operatorsWidget = self.getWidgetOperators();
		var groupId = accessProfileWidget.currentRow.NRPERFILACESSO;
		operatorsWidget.dataSourceFilter[0].value = groupId;
        operatorsWidget.reload();
        ZhuAclController.setElementsListFilter(accessProfileWidget.container, groupId);
		accessProfileWidget.activate();
	};

	self.saveProfile = function(accessProfileWidget) {
		if(accessProfileWidget.isValid()) {
			accessProfileWidget.saveRow().then(
				afterSave,
				function(response){
					ScreenService.errorNotification(ERR_SAV_PROFILE + response.data.error);
				}
			);
		}
		else{
			ScreenService.showMessage(ALE_FILL_REQ_FIELD, "", "exclamation");
		}
	};

	function afterSave(response) {
		var accessProfileWidget = self.getWidgetProfile();
		var id = response.dataset.accessProfile[0].NRPERFILACESSO;
		var nrOrg = response.dataset.accessProfile[0].NRORG;
		var operator = response.dataset.accessProfileRowsDefaultValues[0].CDOPERINCLUSAO;
		var createDate = response.dataset.accessProfileRowsDefaultValues[0].DTINCLUSAO;
		self.setWidgetsVisibility(accessProfileWidget.widgets, true);
		accessProfileWidget.currentRow.NRPERFILACESSO = id;
		accessProfileWidget.currentRow.NRORG = nrOrg;
		accessProfileWidget.currentRow.NRORGINCLUSAO = nrOrg;
		accessProfileWidget.currentRow.CDOPERINCLUSAO = operator;
		accessProfileWidget.currentRow.DTINCLUSAO = createDate;
		accessProfileWidget.currentRow.__is_new = false;
		accessProfileWidget.currentRow.changed = false;
		accessProfileWidget.view();
		self.setChildFilters(accessProfileWidget);
	}

	self.loadOperators = function(operatorsWidget) {
		var addOperatorTabbedWidget = operatorsWidget.container.getWidget("TABBED_ADD_OPERATOR");
		var addOperatorTabbedChildrenWidget = operatorsWidget.container.getWidget("TABBED_CHILDREN_ADD_OPERATOR");
		var addOperatorWidget = operatorsWidget.container.getWidget("addOperator");
		var nrParcNegocio = [];
		operatorsWidget.dataSource.data.forEach(function(data){
			nrParcNegocio.push(data.NRPARCNEGOCIO);
		});

		if(nrParcNegocio.length > 0) {
			addOperatorWidget.dataSourceFilter = [{
				"name": "NRPARCNEGOCIO",
				"operator": "NOT IN",
				"value": nrParcNegocio
			}];
		}
		addOperatorWidget.reload();
		ScreenService.openSwipe(addOperatorTabbedWidget).then(function(){
			addOperatorTabbedWidget.edit();
			addOperatorWidget.activate();
		});
	};

	self.addPerfAcessParc = function (widget, currentWidget) {
		var profileWidget = self.getWidgetProfile();
		var operators     = widget.getCheckedRows();
		var addOperatorsTabbedChildrenWidget = self.getWidgetTabbedChildrenAddOperators();

        operators = operators.map(function (element) {
			element.CDTIPOINSCRICAO = getInscriptionType(element.NRINSCRICAOPARC);
            element.NRPERFILACESSO = profileWidget.currentRow.NRPERFILACESSO;
            element.DTVALIDADE = addOperatorsTabbedChildrenWidget.currentRow.DTVALIDADE || null;
            element.__is_new = true;
            element.changed = true;
            return element;
        });

		AccessProfileService.savePerfAccessParc(operators).then(
			finishAddOperators,
			function () {
				ScreenService.errorNotification(ERR_SAV_PROFILE_OPERATORS);
			}
		);

		if(currentWidget && currentWidget.unSelectAll){
			currentWidget.unSelectAll();
		}
	};

	var finishAddOperators = function() {
		var operatorsWidget = self.getWidgetOperators();
		delete(self.getWidgetTabbedChildrenAddOperators().currentRow.DTVALIDADE);
		operatorsWidget.reload();
		ScreenService.closeSwipe();
		operatorsWidget.list();
		operatorsWidget.activate();
	};

	var getInscriptionType = function (inscription) {
		if (inscription) {
			if (inscription.length == 11) {
				return "CPF";
			} else if (inscription.length == 14) {
				return "CNPJ";
			}
		} else {
			return "LIVRE";
		}
	};

	self.getProductId = function() {
		localStorageEngine.getLocalVar("productIdLoginUrl");
	};

    this.accessProfileContainerAfterinit = function() {
        ZhuAclController.insertElementsListWidget(templateManager.container, self.getWidgetProfile(), window.aclExtraElementsFile);

        var loginPromise = localStorageEngine.getLocalVar('loginPromise') || ZHPromise.when();
        ScreenService.showLoader();
        loginPromise.then(function() {
            ScreenService.hideLoader();
            findAccessControllUtilization();
        }).catch(ScreenService.hideLoader);
    };

    this.profileWidgetOnEnter = function(){
        var loginPromise = localStorageEngine.getLocalVar('loginPromise') || ZHPromise.when();
        ScreenService.showLoader();
        loginPromise.then(function() {
            ScreenService.hideLoader();
            var widget = self.getWidgetProfile();
            widget.currentRow = { IDATIVO: "S" };
            ACLUtilities.applyDefaultFilterOnEnter(widget);
        }).catch(ScreenService.hideLoader);
    };

    this.profileFilterAfterApply = function() {
        ACLUtilities.setFilterInit(self.getWidgetProfile());
    };

    this.enableAccessControll = function() {
    	ScreenService.confirmMessage(
			CON_CHEN_ACCESS_PROFILE,
			'question',
			changeAccessControll
		);
	};

    var changeAccessControll = function(){
		var row = {};
		row.IDUTCTRLACESWEB = localStorageEngine.getLocalVar("idutctrlacesweb");
		row.changed = true;
    	AccessProfileService.changeAccessProfile(row).then(function(response){
    		var idutctrlacesweb = response.dataset.changeAccessProfile[0].idutctrlacesweb;
    		var msg = (idutctrlacesweb == 'S') ? AclMessages.SUC_ACCESS_PROFILE_ACTIVATED : AclMessages.SUC_ACCESS_PROFILE_DISABLED ;
    		changeLabelEnableAccessProfile(idutctrlacesweb);
        	localStorageEngine.setLocalVar("idutctrlacesweb", idutctrlacesweb);
        	ScreenService.successNotification(msg);
    	});
    };



    var findAccessControllUtilization = function(){
    	AccessProfileService.findAccessControllUtilization().then(function(response){
    		var idutctrlacesweb;
    		var AccessControllUtilization = response.dataset.findAccessControllUtilization;
    		if (AccessControllUtilization && AccessControllUtilization.length > 0) {
        		idutctrlacesweb = AccessControllUtilization[0].IDUTCTRLACESWEB;
    		} else {
    			idutctrlacesweb = "N";
    		}
        	localStorageEngine.setLocalVar("idutctrlacesweb", idutctrlacesweb);
        	changeLabelEnableAccessProfile(idutctrlacesweb);
        });
    };

    var changeLabelEnableAccessProfile = function (idutctrlacesweb){
    	var actionEnableAccessProfile = self.getActionEnableAccessProfile();
    	var container = self.getContainerAccessProfile();
    	if (idutctrlacesweb == 'N') {
    		container.label = 'Access Profile (Access Control Disabled)';
    		actionEnableAccessProfile.label = 'Enable Access Control';
    	}else{
    		container.label = 'Access Profile';
    		actionEnableAccessProfile.label = 'Disable Access Control';
    	}
    };

    this.operatorsActionEventDelete = function() {
    	ScreenService.confirmMessage(
			CON_DEL_PROFILE_OPERATORS,
			'ALERT',
			deleteOperator
		);
	};

	var deleteOperator = function(){
		var widget = self.getWidgetOperators();
		var row = {};
		row.NRPERFACESSPARC = widget.currentRow.NRPERFACESSPARC;
		row.NRORG = widget.currentRow.NRORG;
		row.__is_new = false;
		row.changed = true;
		widget.dataSource.remove([row]).then(function(data) {
            ScreenService.closeSwipe();
            widget.reload();
            widget.list();
            ScreenService.successNotification(SUC_DEL_OPERATOR);
        }).then(null, function(message) {
            ScreenService.errorNotification(message);
        });
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('AclAccessProfileController', AclAccessProfileController);
});
