<?php
namespace Teknisa\Model\Teknisa\Repositories;

use Teknisa\Model\Teknisa\Entities;

class OmProdutoOrg extends \Doctrine\ORM\EntityRepository {

    public function obtemTodos($nrOrgId, $conditions){
        $query = <<<SQL
		SELECT *
          FROM (SELECT PO.ID, PO.NRORG_ID, PO.IDATIVO, PO.NRPRODUTO_ID, PO.CDAPPNOTIF, PO.DSCHAVENOTIF,
		               PR.NMPRODUTO, PR.DSURL_JSON_MENU, PO.DSURLREDIRECT, PO.IDPRODUTOEXCLUSIVO
                  FROM OM_PRODUTO_ORG PO
            INNER JOIN OM_PRODUTO PR ON PR.ID = PO.NRPRODUTO_ID
                 WHERE PO.NRORG_ID = :NRORG_ID
              ORDER BY PO.ID)
         WHERE 1 = 1
        $conditions
SQL;

        $params = array( 'NRORG_ID' => $nrOrgId );
        $produtoOrgs = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        foreach($produtoOrgs as &$produtoOrg){
            $produtoOrg['ID'] = intval($produtoOrg['ID']);
        }
        return $produtoOrgs;
    }
    
    public function findProductOrg($id,$product_org){
        $query = <<<SQL
        SELECT PRORG.ID AS PRODUCT_ORG, PRORG.DSCHAVENOTIF, PRORG.CDAPPNOTIF, PROD.ID, 
               PROD.NMPRODUTO, PROD.DSURL_JSON_MENU, PROD.IDSTATUS, PRORG.DSURLREDIRECT,
		       PROD.DTINCLUSAO, PROD.DTULTATU, OPERINC.NMOPERADOR AS NMOPERINCLUSAO,
		       OPERATU.NMOPERADOR AS NMOPERULTATU, PROD.NREXTERNAL_PRODUCT_ID, PRORG.IDPRODUTOSAAS, PRORG.IDPRODUTOEXCLUSIVO
            FROM OM_PRODUTO_ORG  PRORG
            INNER JOIN OM_PRODUTO PROD ON PROD.ID = PRORG.NRPRODUTO_ID
            LEFT JOIN OPERADOR OPERINC ON (OPERINC.CDOPERADOR = PROD.CDOPERINCLUSAO)
            LEFT JOIN OPERADOR OPERATU ON (OPERATU.CDOPERADOR = PROD.CDOPERULTATU)
         WHERE PRORG.ID = :product_org AND PROD.ID = :productid
SQL;
        $params = array( 
                'product_org' => $product_org,
                'productid' => $id
            );
        
        return $produtoOrgs = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function obtemTodosNaoIncluidos($organizationId, $conditions){
        $query = <<<SQL
        SELECT *
          FROM (SELECT OP.ID, OP.NMPRODUTO, OP.IDPRODUTOSAAS, 
                       CASE OP.IDPRODUTOSAAS
                       WHEN :SAAS        THEN 'Exclusivamente SAAS'
                       WHEN :NAOSAAS     THEN 'Exclusivamente NÃO SAAS'
                       WHEN :ORGANIZACAO THEN 'Assume identificador da Organização'
                        END AS PRODSTATUS
                  FROM OM_PRODUTO OP
	        INNER JOIN PRODUCT EXPROD ON EXPROD.ID = OP.NREXTERNAL_PRODUCT_ID
	        INNER JOIN PROJECT EXPROJ ON EXPROJ.ID = EXPROD.PROJECT_ID
                 WHERE OP.ID NOT IN (SELECT OPG.NRPRODUTO_ID
                                       FROM OM_PRODUTO_ORG OPG
                                      WHERE OPG.NRORG_ID = :NRORG_ID)
                   AND OP.IDSTATUS = 'A'
                   AND EXPROJ.PRODUCT_TYPE = 'C'
                 ORDER BY OP.ID)
         WHERE 1 = 1
        $conditions
SQL;
        $param = array( 'NRORG_ID' => $organizationId,
                        'SAAS' => Entities\OmProduto::PRODUTO_SAAS,    
                        'NAOSAAS' => Entities\OmProduto::PRODUTO_NAO_SAAS,    
                        'ORGANIZACAO' => Entities\OmProduto::HERDA_ORGANIZACAO,    
                    );
        $produtoOrgs = $this->getEntityManager()->getConnection()->fetchAll($query, $param);

        foreach($produtoOrgs as &$produtoOrg){
            $produtoOrg['ID'] = intval($produtoOrg['ID']);
        }
        return $produtoOrgs;
    }

    public function getOrgProductByExternalProductIdAndProductNrOrgId($externalProductId, $nrOrg){
        $query = <<<SQL
		SELECT OMP.ID
		  FROM OM_PRODUTO OMP
	INNER JOIN OM_PRODUTO_ORG OPO ON OPO.NRPRODUTO_ID = OMP.ID
		 WHERE OMP.NREXTERNAL_PRODUCT_ID = :EXTERNAL_PRODUCT_ID
		   AND OPO.NRORG_ID = :NRORG
SQL;
        $params = array(
            'EXTERNAL_PRODUCT_ID' => $externalProductId,
            'NRORG' => $nrOrg
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getOrgProductByNrOrgId($nrParcNegocio, $filterQuery) {
        $query = <<<SQL
        SELECT *
          FROM (SELECT OMP.ID AS NRPRODUTO, OMP.NMPRODUTO,
                       OMP.NREXTERNAL_PRODUCT_ID, OPO.ID AS NRPRODUTO_ORG,
                       PRO.VERSION_CHECK_TYPE, PRO.URL_WEBSERVICE,
                       OMP.IDPRODUTOSAAS, OPO.IDPRODUTOSAAS AS IDPRODUTOORGSAAS, ORG.IDORGSAAS
                  FROM OM_PRODUTO_ORG OPO
                 INNER JOIN OM_PRODUTO OMP ON OPO.NRPRODUTO_ID = OMP.ID
                 INNER JOIN ORGANIZACAO ORG ON ORG.NRORG = OPO.NRORG_ID
                 INNER JOIN PRODUCT PRO ON PRO.ID = OMP.NREXTERNAL_PRODUCT_ID
                 WHERE ORG.NRPARCNEGOCIO = :NRPARCNEGOCIO)
         WHERE 1 = 1
		       $filterQuery
	  ORDER BY NMPRODUTO
SQL;
        $params = array(
            'NRPARCNEGOCIO' => $nrParcNegocio
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $nrOrg
     * @param $productId
     * @return array
     */
    public function getURLRedirectByNrorgAndProductId($nrOrg, $productId){
        $query = "SELECT OO.DSURLREDIRECT
                    FROM OM_PRODUTO_ORG OO
                    JOIN OM_PRODUTO OP ON OP.ID = OO.NRPRODUTO_ID
                   WHERE OP.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
                     AND OO.NRORG_ID = :NRORG";

        $params = array( 'NRORG' => $nrOrg, 'PRODUCT_ID' => $productId );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    /**
     * @param $productId
     * @param $productOrgId
     * @param $filters
     * @return array
     */
    public function findProductElement($productId, $productOrgId, $filters)
    {
        $query = "
             SELECT *
              FROM (SELECT PE.ID, PE.LABEL, PE.UNIQUE_ID, PE.NRORDEM,
                           PE.NRPRODUTO_ID, PE.PARENT_ID, EP.LABEL AS PARENT_LABEL,
                           TO_CHAR(OE.DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE
                      FROM OM_PRODUTO_ELEMENTO PE
                      JOIN OM_PRODUTO_ORG_ELEMENTO OE ON OE.NRPRODUTO_ELEMENTO_ID = PE.ID
                                                     AND OE.NRPRODUTO_ORG_ID = :NRPRODUTO_ORG_ID
                      JOIN OM_PRODUTO OP ON OP.ID = PE.NRPRODUTO_ID
                 LEFT JOIN OM_PRODUTO_ELEMENTO EP ON EP.ID = PE.PARENT_ID                                
                     WHERE OP.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
                  ORDER BY PE.NRORDEM)
             WHERE 1 = 1
             $filters";

        $params = array(
            'PRODUCT_ID' => $productId,
            'NRPRODUTO_ORG_ID' => $productOrgId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}