function LSLoginController(ScreenService, templateManager, MessageService, LSLoginService, LSUtilities, LibUtilities, eventAggregator, ZhKeyboardService) {
    var TYPE_EXPIRED_PASSWORD      = 'EXPIRED_PASSWORD';
    var TYPE_PASSWORD_WILL_EXPIRE  = 'PASSWORD_WILL_EXPIRE';
    var TYPE_PASSWORD_EXPIRE_TODAY = 'PASSWORD_EXPIRE_TODAY';

    //Container
    var getContainer = function () {
        return templateManager.container;
    };

    //Widgets
    var getLoginWidget = function () {
        return getContainer().getWidget('LOGIN');
    };
    var getForgotPasswordWidget = function () {
        return getContainer().getWidget('FORGOT_PASSWORD');
    };
    var getExpiredPasswordWidget = function () {
        return getContainer().getWidget('EXPIRED_PASSWORD');
    };

    //Fields
    var getImageFieldFromLoginWidget = function () {
        return getLoginWidget().getField('IMAGE');
    };
    var getUserFieldFromLoginWidget = function () {
        return getLoginWidget().getField('USER');
    };
    var getKeepConnectedFieldFromLoginWidget = function () {
        return getLoginWidget().getField('KEEP_CONNECTED');
    };
    var getLanguageFieldFromLoginWidget = function () {
        return getLoginWidget().getField('LANGUAGE');
    };

    //LOGIN Widget Methods
    var checkLoginResponse = function (data) {
        if (data && data.dataset && data.dataset.userData.LOGGED) {
            var userData = data.dataset.userData;
            if (userData.EXPIRED_PASSWORD) {
                expiredPasswordValidation(userData);
            } else {
                redirectAfterLogin(userData);
            }
        } else {
            LSUtilities.defaultError(data);
        }
    };

    var expiredPasswordValidation = function(userData) {
        var loginWidget = getLoginWidget();
        var message = 'ALE_RESET_PASSWORD';
        var params  = {};
        if (userData.EXPIRED_PASSWORD.includes(TYPE_PASSWORD_WILL_EXPIRE)) {
            var daysToExpire = userData.EXPIRED_PASSWORD.split('_').pop();
            params = { days: daysToExpire};
            message = 'ALE_PASSWORD_WILL_EXPIRE';
            showDialogExpiredPassword(userData, loginWidget, message, params);
        } else if(userData.EXPIRED_PASSWORD.includes(TYPE_PASSWORD_EXPIRE_TODAY)) {
            message = 'ALE_PASSWORD_EXPIRE_TODAY';
            showDialogExpiredPassword(userData, loginWidget, message, params);
        } else {
            if (userData.EXPIRED_PASSWORD == TYPE_EXPIRED_PASSWORD) {
                message = 'ALE_EXPIRED_PASSWORD';
            }
            openExpiredPasswordWidget(userData, loginWidget, message);
        }
    };

    var showDialogExpiredPassword = function(userData, loginWidget, message, params) {
        MessageService.showCustomDialogByCode('login#' + message, params,
            {
                type: 'information',
                actions: [ {
                    label: ScreenService.i18n("Sim"),
                    code: function () {
                        openExpiredPasswordWidget(userData, loginWidget, null);
                    }
                }, {
                    label: ScreenService.i18n("Não"),
                    code: function () {
                        redirectAfterLogin(userData);
                    }
                } ]
            }
        );
    };

    var openExpiredPasswordWidget = function(userData, loginWidget, message) {
        var expiredPasswordWidget = getExpiredPasswordWidget();
        expiredPasswordWidget.newRow();
        expiredPasswordWidget.currentRow.OPERATOR = userData.USER_NAME;
        expiredPasswordWidget.currentRow.ACTUAL_PASSWORD = loginWidget.currentRow.PASSWORD;
        expiredPasswordWidget.currentRow.userData = userData;
        if(message) {
            MessageService.showMessageByCode('login#' + message, null, { type: 'ALERT' });
        }
        ScreenService.openPopup(expiredPasswordWidget);
    };

    var validateRcUrl = function(redirectUrl) {
        var url = window.location.href;
        if(url.includes('rc-')) {
            var redirectHost = angular.copy(redirectUrl);
            if(redirectHost.includes('://')) {
                redirectHost = redirectHost.split('://')[1];
            }
            var protocol = '';
            if(redirectUrl.includes('https')) {
                protocol = 'https://';
            } else if(redirectUrl.includes('http')) {
                protocol = 'http://';
            }
            redirectUrl = protocol + 'rc-' + redirectHost;
        }
        return redirectUrl;
    };

    var redirectAfterLogin = function(userData) {
        localStorage.setItem('LOGIN_USER', userData.USER);
        localStorage.setItem('LOGIN_TOKEN', userData.TOKEN);

        if(userData.REDIRECT_URL) {
            userData.REDIRECT_URL = validateRcUrl(userData.REDIRECT_URL);
            localStorage.setItem('LOGIN_REDIRECT_URL', userData.REDIRECT_URL);
            userData.REDIRECT_URL += '?authToken=' + userData.REDIRECTION_TOKEN;
            if (window.ZhNativeInterface) {
                window.ZhNativeInterface.redirect(userData.REDIRECT_URL);
            } else {
                window.location = userData.REDIRECT_URL;
            }
        } else {
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            redirectToProduct();
        }
    };

    var redirectToProduct = function() {
        var containerName = window.defaultContainer || templateManager.project.mainWindow;
        if(window.openLoginContainerWithoutRedirect) {
            eventAggregator.publish('loginSuccessWithoutRedirect');
            ScreenService.openWindow(containerName);
        } else {
            LibUtilities.openWindow(containerName);
        }
    };

    var setCurrentLanguage = function(language) {
        localStorage.setItem('LOGIN_LANGUAGE', language);
        ScreenService.setLanguage(language).then(function() {
            getLanguageFieldFromLoginWidget().value(language);
            getLoginWidget().redraw();
        });
    };

    var updateLanguage = function() {
        var languageField = getLanguageFieldFromLoginWidget();
        var language = window.lang;
        ScreenService.showLoader();
        LSLoginService.getProductLanguages().then(function(response) {
            ScreenService.hideLoader();
            var productLanguages = response.dataset.languages.LANGUAGES;
            var defaultLanguage  = response.dataset.languages.DEFAULT_LANGUAGE;
            if(defaultLanguage && defaultLanguage.SHORT_NAME) {
                var defaultLangValidation = productLanguages.filter(function(lang) {
                    return lang.SHORT_NAME == defaultLanguage.SHORT_NAME;
                });
                if(Util.isEmptyOrBlank(defaultLangValidation)) {
                    productLanguages.push(defaultLanguage);
                }
                language = localStorage.getItem('LOGIN_LANGUAGE') || defaultLanguage.SHORT_NAME;
            } else {
                language = localStorage.getItem('LOGIN_LANGUAGE') || language;
            }
            if(!Util.isEmptyOrBlank(productLanguages)) {
                languageField.dataSource.data = productLanguages;
                languageField.readOnly = productLanguages.length == 1;
            } else {
                languageField.readOnly = true;
            }
            setCurrentLanguage(language);
        }).catch(function() {
            ScreenService.hideLoader();
            languageField.readOnly = true;
            setCurrentLanguage(language);
        });
    };

    var testLicenseConnection = function() {
        LSLoginService.testLicenseConnection().then(function (response) {
            var result = response.dataset.testLicenseConnection;
            var params = {
                URL: result.URL
            };
            if(result.ERROR) {
                params.ERROR = result.ERROR;
                MessageService.showMessageByCode('login#ERR_LICENSE_CONNECTION', params, {type: 'ERROR'});
            } else {
                MessageService.showMessageByCode('login#SUC_LICENSE_CONNECTION', params, {type: 'SUCCESS'});
            }
        }).catch(LSUtilities.defaultError);
    };

    this.loginWidgetOnEnter = function () {
        if(localStorage.getItem('LOGIN_USER')) {
            redirectToProduct();
        } else {
            if(ScreenService.isSideMenuOpen()) {
                templateManager.container.showMenu = true;
                ScreenService.toggleSideMenu();
                templateManager.container.showMenu = false;
            }
            getLoginWidget().newRow();
            getLoginWidget().attempts = {};
            getImageFieldFromLoginWidget().source = window.logoImage;
            getKeepConnectedFieldFromLoginWidget().value(localStorage.getItem('LOGIN_KEEP_CONNECTED'));
            updateLanguage();
            ZhKeyboardService.bind('ctrl+shift+alt+,', testLicenseConnection);
        }
    };

    this.loginFieldOnClickSubmit = function () {
        var loginWidget = getLoginWidget();
        var user        = getUserFieldFromLoginWidget().element.val();
        var password    = loginWidget.currentRow.PASSWORD;

        if (!user) {
            MessageService.showMessageByCode('login#ALE_NOT_EMAIL', null, { type: 'ALERT' });
        } else if (!password) {
            MessageService.showMessageByCode('login#ALE_NOT_PASSWORD', null, { type: 'ALERT' });
        } else {
            if (!loginWidget.attempts.hasOwnProperty(user)) {
                loginWidget.attempts[user] = 1;
            }
            LSLoginService.login(user, password, loginWidget.attempts[user])
                .then(checkLoginResponse)
                .catch(function(error) {
                    if(error && error.data && error.data.errorCode) {
                        /** Error code 29: If miss the password again, the user will be blocked */
                        /** Error code 31: Access blocked by attempts */
                        /** Error code 32: Invalid user and password */
                        var errorCode = error.data.errorCode;
                        if(errorCode == 29 || errorCode == 32) {
                            loginWidget.attempts[user]++;
                        } else if(errorCode == 31) {
                            loginWidget.attempts[user] = 1;
                        }
                    }
                    LSUtilities.defaultError(error);
                });
        }
    };

    this.loginFieldOnClickForgotPassword = function () {
        ScreenService.openPopup(getForgotPasswordWidget());
    };

    this.loginFieldOnClickKeepConnected = function () {
        localStorage.setItem('LOGIN_KEEP_CONNECTED', getLoginWidget().currentRow.KEEP_CONNECTED);
    };

    this.loginFieldOnChangeLanguage = function () {
        var actualLanguage = localStorage.getItem('LOGIN_LANGUAGE');
        var language = getLoginWidget().currentRow.LANGUAGE;
        if(actualLanguage != language) {
            localStorage.setItem('LOGIN_LANGUAGE', language);
            ScreenService.setLanguage(language).then(function() {
                var updatedTitle = ScreenService.i18n(templateManager.container.label);
                if(document.title.includes(' - ')) {
                    var productSufixTitle = document.title.split(' - ').pop();
                    updatedTitle = updatedTitle + ' - ' + productSufixTitle;
                }
                templateManager.updateTitle(updatedTitle);
            });
        }
    };

    //FORGOT_PASSWORD Widget Methods
    this.forgotPasswordWidgetOnEnter = function () {
        var forgotPasswordWidget = getForgotPasswordWidget();
        forgotPasswordWidget.newRow();
        forgotPasswordWidget.currentRow.EMAIL = getUserFieldFromLoginWidget().element.val();
    };

    this.forgotPasswordActionEventConfirm = function () {
        var widget = getForgotPasswordWidget();
        if (widget.isValid()) {
            var email = widget.currentRow.EMAIL;
            LSLoginService.requestNewPassword(email, window.cssStyle).then(function () {
                MessageService.showMessageByCode('login#SUC_RESET_PASSWORD', { email: email });
                ScreenService.closePopup();
            }).catch(LSUtilities.defaultError);
        }
    };

    this.forgotPasswordActionEventCancel = function () {
        ScreenService.closePopup();
    };

    //EXPIRED_PASSWORD Widget Methods
    this.expiredPasswordActionEventConfirm = function () {
        var widget = getExpiredPasswordWidget();
        if (widget.isValid()) {
            var email = getUserFieldFromLoginWidget().element.val();
            var actualPassword = widget.currentRow.ACTUAL_PASSWORD;
            var newPassword = widget.currentRow.NEW_PASSWORD;
            var newPasswordConfirmation = widget.currentRow.CONFIRM_NEW_PASSWORD;

            LSLoginService.updatePassword(actualPassword, newPassword, newPasswordConfirmation, email).then(function () {
                MessageService.successNotificationByCode('login#SUC_PASSWORD_UPDATED');
                MessageService.getMessageByCode('login#LOA_REDIRECTING').then(function(message) {
                    ScreenService.changeLoadingMessage(ScreenService.i18n(message));
                    ScreenService.showLoader();
                    setTimeout(function () {
                        ScreenService.hideLoader();
                        ScreenService.restoreDefaultLoadingMessage();
                        redirectAfterLogin(widget.currentRow.userData);
                    }, 3000);
                }).catch(LSUtilities.defaultError);
            }).catch(LSUtilities.defaultError);
        }
    };

    this.expiredPasswordActionEventCancel = function () {
        ScreenService.closePopup();
    };

    this.authenticationContainerBeforeinit = function() {
        templateManager.project.aclDefaultRule = 'A';
        templateManager.aclRules = {};
        var containers = templateManager.containers;
        for (var key in containers) {
            if (!containers.hasOwnProperty(key)) continue;
            if(key.includes('#') && !key.includes('login#authentication')) {
                delete containers[key];
            }
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('LSLoginController', LSLoginController);
});