<?php

namespace Teknisa\ACL\Controllers;

use Teknisa\ACL\Services\AccessProfile as AccessProfileService;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\Environment;

class AccessProfile extends Crud{

    const IS_NEW = "__is_new";

    protected $dataSourceName = "accessProfile";

    /** @var AccessProfileService */
    protected $accessProfileService;
    
    /** @var Teknisa\Libs\Util\Environment */
    protected $environment;

    public function __construct(Manager $manager, AccessProfileService $accessProfileService, Environment $environment) {
        $this->accessProfileService = $accessProfileService;
        parent::__construct($manager);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $request->getFilterCriteria()->addCondition(AccessProfileService::NRORG, $this->accessProfileService->getNrOrgFilter());
        parent::find($request, $response);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $index = 10;
        $maxNrPerfilAcesso = $this->accessProfileService->getMaxNrPerfilAcesso();
        $rowsWithDefaultValues = array();

        foreach($request->getDataSet()->getRows() as &$row) {
            if ($row[self::IS_NEW]) {
                $row[AccessProfileService::NRPERFILACESSO] = $maxNrPerfilAcesso+$index;
                $index += 10;
            }

            $this->accessProfileService->setDefaultValues($row);
            $rowsWithDefaultValues[] = $row;
        }

        parent::save($request, $response);
        $defaultValuesDataSet = new DataSet("accessProfileRowsDefaultValues", $rowsWithDefaultValues);
        $response->addDataSet($defaultValuesDataSet);
    }



    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        $profilesToDelete = array();
        foreach ($request->getDataSet()->getRows() as &$row) {
            $profilesToDelete[] = $row[AccessProfileService::NRPERFILACESSO];
        }
        $this->accessProfileService->deleteDependencies($profilesToDelete);
        parent::delete($request, $response);
    }

    public function deleteRow(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $profileToDelete = array($row[AccessProfileService::NRPERFILACESSO]);
        $this->accessProfileService->deleteDependencies($profileToDelete);
        $dataSetToDelete = new DataSet($this->dataSourceName, array($row));
        $deletedRows = $this->dataSourceManager->delete($dataSetToDelete);
        $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        $nrDeletedRows = count($deletedRows);
        $response->addNotification(new DTO\Response\Notification("{$nrDeletedRows} row(s) deleted.", DTO\Response\Notification::TYPE_SUCCESS));
    }
    
    
    public function findAccessControllUtilization(Request $request, Response $response){
        $return = $this->accessProfileService->findAccessControllUtilization();
        $response->addDataSet(new DataSet('findAccessControllUtilization', $return));
    }
    
    public function changeAccessProfile(Request $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $idutctrlacesweb = $row['IDUTCTRLACESWEB'];
        $return = $this->accessProfileService->changeAccessProfile($idutctrlacesweb);
        $response->addDataSet(new DataSet('changeAccessProfile', [$return]));
    }
}