<?php

namespace Teknisa\ACL\Repositories;

use Teknisa\ACL\Util\InstanceProvider;
use Teknisa\ACL\Repositories\BaseCrud;

class Operator extends BaseCrud {

    protected $table = 'OPERADOR';

    public function __construct()
    {
        parent::__construct($this->table);
    }
    
    public function findOperators($organizationId, $filters, $limit, $offset, $filterParcNegocio = '')
    {
        $query = <<<SQL
            SELECT * 
              FROM (SELECT DISTINCT OPER.CDOPERADOR, PARC.NRINSCRICAOPARC, PARC.NMPRINCIPALPARC AS NMPARCNEGOCIO, OPER.NMACESSO, 
                           OPER.CDPAIS, OPER.NMOPERADOR, OPER.DTULTATUOPER, OPER.CDCLIENTE, OPER.CDEMPRESA, OPER.DSEMAILOPER, OPER.IDOPERMASTER,
                           OPER.IDOPERACESSO, OPER.IDOPERATIVO, OPER.NRORG, OPER.DTINCLUSAO, OPER.DTULTATU,
                           OPER.NRORGINCLUSAO, OPER.CDOPERINCLUSAO, OPER.CDOPERULTATU, OPER.NRORGULTATU,
                           OPER.NRPARCNEGOCIO, OPER.NRORGSELECIONADA, OPER.CDFILIALSELECIONADA, OPER.NRPERFILSELECIONADO,
                           OPER.NRORGTRAB, OPER.NRESTRUTTRAB, OPER.IDATIVO, OPER.IDTPOPERADOR, ORG.NMORGANIZACAO, EST.NMESTRUTURAM, 
                           CLI.NMRAZSOCCLIE, PA.NMPAIS, OPER.DTULTACESOPER, V.CDVENDEDOR, V.CDFILIAL, V.CDCAIXA, F.NMFILIAL, C.NMCAIXA,
                           DECODE(OPER.NRORGOPER, NULL,  OPER.NRORG, OPER.NRORGOPER) AS NRORGOPER,
                           DECODE(OPER.NRORGOPER, NULL,  ORG.NMORGANIZACAO, ORGOPER.NMORGANIZACAO) AS NMORGANIZACAOOPER,
                           (CASE NVL(V.CDVENDEDOR,'NULL') WHEN 'NULL' THEN 0 ELSE 1 END ) AS ADD_SELLER
                      FROM OPERADOR OPER
                 LEFT JOIN ORGANIZACAO ORG ON ORG.NRORG = OPER.NRORG
                 LEFT JOIN ESTRUTURAM EST ON EST.NRORG = OPER.NRORG 
                                         AND EST.NRESTRUTURAM = OPER.NRESTRUTTRAB
                 LEFT JOIN CLIENTE CLI ON CLI.NRORG = OPER.NRORG
                                      AND CLI.CDCLIENTE = OPER.CDCLIENTE
                 LEFT JOIN PAIS PA ON PA.CDPAIS = OPER.CDPAIS
                 LEFT JOIN PARCNEGOCIO PARC ON PARC.NRORG = OPER.NRORG
                                           AND PARC.NRPARCNEGOCIO = OPER.NRPARCNEGOCIO
                 LEFT JOIN VENDEDOR V ON OPER.CDOPERADOR = V.CDOPERADOR AND V.NRORG  = OPER.NRORG
                 LEFT JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL AND V.NRORG  = F.NRORG
                 LEFT JOIN CAIXA C ON V.CDCAIXA = C.CDCAIXA AND V.NRORG  = C.NRORG AND C.CDFILIAL = F.CDFILIAL
                 LEFT JOIN ORGANIZACAO ORGOPER ON ORGOPER.NRORG = OPER.NRORGOPER
                                           )
             WHERE NRORG = $organizationId
             $filterParcNegocio
             $filters
          ORDER BY NMOPERADOR ASC
            
SQL;
        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($queryWithPagination);
    }
    
    private function getPaginatedQuery($query, $limit, $offset) 
    {
        if ($limit > 0) {
            $max = $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }
        return $query;
    }
    
    public function saveOperator(
            $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
            $countryCode, $businessPartnership, $clientCode, $organizationId, $accessOperatorId,
            $selectedProfile, $organizationInclusion, $operatorInclusionCode, $password, $idopermaster, $nrestruttrab,
            $idtpoperador, $nrOrgOper)
    {
        $query = <<<SQL
            INSERT INTO OPERADOR
                        (CDOPERADOR, IDOPERATIVO, IDATIVO, NMOPERADOR, NMACESSO, DSEMAILOPER, CDPAIS, CDSENHAOPERWEB, NRPARCNEGOCIO, IDOPERMASTER,
                        CDCLIENTE, NRORG, NRORGTRAB, IDOPERACESSO, NRPERFILSELECIONADO, NRORGINCLUSAO,
                        CDOPERINCLUSAO, DTINCLUSAO, DTULTATUOPER, NRESTRUTTRAB, IDTPOPERADOR, NRORGOPER)
                 VALUES (:CDOPERADOR, :IDOPERATIVO, :IDATIVO, :NMOPERADOR, :NMACESSO, :DSEMAILOPER, :CDPAIS, :CDSENHAOPERWEB,
                         :NRPARCNEGOCIO, :IDOPERMASTER, :CDCLIENTE, :NRORG, :NRORGTRAB, :IDOPERACESSO, :NRPERFILSELECIONADO,
                         :NRORGINCLUSAO, :CDOPERINCLUSAO, SYSDATE, SYSDATE, :NRESTRUTTRAB, :IDTPOPERADOR, :NRORGOPER)
SQL;
        
        $params = array (
            "CDOPERADOR"          => $operatorCode,
            "IDOPERATIVO"         => $activeOperatorId,
            "IDATIVO"             => $activeId,
            "NMOPERADOR"          => $operatorName,
            "NMACESSO"            => $accessName,
            "DSEMAILOPER"         => $operatorEmail,
            "CDPAIS"              => $countryCode,
            "NRPARCNEGOCIO"       => $businessPartnership,
            "CDCLIENTE"           => $clientCode,
            "NRORG"               => $organizationId,
            "NRORGTRAB"           => $organizationId,
            "IDOPERACESSO"        => $accessOperatorId,
            "NRPERFILSELECIONADO" => $selectedProfile,
            "NRORGINCLUSAO"       => $organizationInclusion,
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDSENHAOPERWEB"      => $password,
            "IDOPERMASTER"        => $idopermaster,
            "NRESTRUTTRAB"        => $nrestruttrab,
            "IDTPOPERADOR"        => $idtpoperador,
            "NRORGOPER"           => $nrOrgOper
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    
    }
    
    public function editOperator(
        $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
        $countryCode, $clientCode, $organizationId, $workOrganizationId, $userId, $businessPartnership, $idopermaster, $nrestruttrab, $idtpoperador, $nrOrgOper)
    {
        $query = "UPDATE OPERADOR
                     SET IDOPERATIVO     = :IDOPERATIVO,
                         IDATIVO         = :IDATIVO,
                         NMOPERADOR      = :NMOPERADOR,
                         NMACESSO        = :NMACESSO,
                         DSEMAILOPER     = :DSEMAILOPER,
                         CDPAIS          = :CDPAIS,
                         CDCLIENTE       = :CDCLIENTE,
                         CDOPERULTATU    = :CDOPERULTATU,
                         NRORGULTATU     = :NRORG,
                         DTULTATUOPER    = SYSDATE,
                         NRPARCNEGOCIO   = :NRPARCNEGOCIO,
                         IDOPERMASTER    = :IDOPERMASTER,
                         NRESTRUTTRAB    = :NRESTRUTTRAB,
                         IDTPOPERADOR    = :IDTPOPERADOR,
                         NRORGOPER       = :NRORGOPER
                   WHERE CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";
        
        $params = array (
            "IDOPERATIVO"   => $activeOperatorId,
            "IDATIVO"       => $activeId,
            "NMOPERADOR"    => $operatorName,
            "NMACESSO"      => $accessName,
            "DSEMAILOPER"   => $operatorEmail,
            "CDPAIS"        => $countryCode,
            "CDCLIENTE"     => $clientCode,
            "NRORG"         => $organizationId,
            "CDOPERULTATU"  => $userId,
            "CDOPERADOR"    => $operatorCode,
            "NRPARCNEGOCIO" => $businessPartnership,
            "IDOPERMASTER"  => $idopermaster,
            "NRESTRUTTRAB"  => $nrestruttrab,
            "IDTPOPERADOR"  => $idtpoperador,
            "NRORGOPER"     => $nrOrgOper
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteOperator($operatorCode, $organizationId)
    {
        $query = "DELETE 
                    FROM OPERADOR 
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";
        
        $params = array(
            "CDOPERADOR"    => $operatorCode,
            "NRORG"         => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };      
    }
    
    public function findAccessProfileByOperator($organizationId, $businessPartnership)
    {
        $query = "SELECT NRPERFILACESSO, NMPERFILACESSO, IDPERFILACESSOADM 
                    FROM PERFILACESSO 
                   WHERE NRORG = :NRORG 
                     AND IDATIVO != 'N' 
                     AND NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSPARC WHERE :NRPARCNEGOCIO = NRPARCNEGOCIO)
                ORDER BY NMPERFILACESSO ASC";
        
        $params = array(
            "NRORG"         => $organizationId,
            "NRPARCNEGOCIO" => $businessPartnership
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };   
    }
    
    public function findIdAdmAccessProfile($organizationId, $accessProfiles)
    {
        $query = "SELECT IDPERFILACESSOADM
                    FROM PERFILACESSO 
                   WHERE NRORG = :NRORG 
                     AND IDATIVO != 'N' 
                     AND NRPERFILACESSO IN ($accessProfiles)";
        
        $params = array(
            "NRORG"         => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };   
    }
    
    
    public function createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode)
    {
        $query = "INSERT INTO PARCNEGOCIO
                              (NRPARCNEGOCIO, NRORG, NMPRINCIPALPARC, NMSECUNDARIPARC, NRINSCRICAOPARC, IDATIVO,
                              DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO,
                              IDPESSOAFISICA, IDINSTITUICAO, IDPARCFUNDIDO)
                      VALUES (:NRPARCNEGOCIO, :NRORG, :NMOPERADOR, :NMOPERADOR, :NRINSCRICAOPARC, :IDATIVO,
                                SYSDATE, :NRORG, :CDOPERINCLUSAO, :CDTIPOPARCPRINCIPAL, :CDTIPOINSCRICAO,
                              :IDPESSOAFISICA, :IDINSTITUICAO, :IDPARCFUNDIDO)";
        
        $params = array (
            "NRPARCNEGOCIO"       => $businessPartnership,
            "NRORG"               => $organizationId,
            "NMOPERADOR"          => $operatorName,
            "NRINSCRICAOPARC"     => $cpf,
            "IDATIVO"             => 'S',
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDTIPOPARCPRINCIPAL" => 'OPERADOR',
            "CDTIPOINSCRICAO"     => $inscriptionTypeCode,
            "IDPESSOAFISICA"      => 'S',
            "IDINSTITUICAO"       => 'N',
            "IDPARCFUNDIDO"       => 'N'
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, $partnerTipeCode, $partnerNumber, $activeId)
    {
        $query = "INSERT INTO PARCERIA
                               (NRPARCERIA, NRPARCNEGOCIO, CDTIPOPARCEIRO, NRORG, IDATIVO,
                               DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES (:NRPARCERIA, :NRPARCNEGOCIO, :CDTIPOPARCEIRO, :NRORG, :IDATIVO,
                               SYSDATE, :NRORG, :CDOPERINCLUSAO)";
        
        $params = array(
            "NRPARCERIA"     => $partnerNumber,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "NRORG"          => $organizationId,
            "CDTIPOPARCEIRO" => $partnerTipeCode,
            "IDATIVO"        => $activeId,
            "CDOPERINCLUSAO" => $operatorInclusionCode
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function createRelationshipOragnizationOperator($organizationOperator, $workOrganizationId,
        $operatorCode, $organizationId, $operatorInclusionCode, $activeId)
    {
        $query = "INSERT INTO ORGANIZACAOOPER
                              (NRORGANIZACAOOPER, NRORGREL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO,
                               CDOPERINCLUSAO, NRORG, IDATIVO)
                       VALUES (:NRORGANIZACAOOPER, :NRORGTRAB, :CDOPERADOR, SYSDATE, :NRORG,
                               :CDOPERINCLUSAO, :NRORG, :IDATIVO)";

        $params = array (
            "NRORGANIZACAOOPER" => $organizationOperator,
            "NRORGTRAB"         => $workOrganizationId,
            "CDOPERADOR"        => $operatorCode,
            "CDOPERINCLUSAO"    => $operatorInclusionCode,
            "NRORG"             => $organizationId,
            "IDATIVO"           => $activeId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function checkIfOperatorCode($operatorCode, $organizationId)
    {
        $query = "SELECT COUNT(*) AS COUNT
                    FROM OPERADOR
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG = :NRORG";
        
        $params = array(
            'CDOPERADOR' => $operatorCode,
            'NRORG'      => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function findOrganization($organizationId)
    {
        $query = "SELECT NRORG, NMORGANIZACAO
                    FROM ORGANIZACAO
                   WHERE NRORG = :NRORG";
                   
        $params = array(
            "NRORG" => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function findOrganizationZhuLog()
    {
        $query = "SELECT NRORG AS ID, NMORGANIZACAO as NAME
                    FROM ORGANIZACAO
                   WHERE IDATIVO = 'S'";

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute();
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function findPartnerAccessProfile($filters, $organizationId, $order = '')
    {
        $query = "SELECT CDOPERULTATU, NRORGULTATU, CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, DTINCLUSAO, IDATIVO, NRORG,
                         DTVALPERFILPARC, NRPERFILACESSO, NRPARCNEGOCIO, NRPERFACESSPARC, NMPERFILACESSO, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE
                    FROM (SELECT PEPA.CDOPERULTATU,  PEPA.NRORGULTATU,  PEPA.CDOPERINCLUSAO,  PEPA.NRORGINCLUSAO,  PEPA.DTULTATU,
                                 PEPA.DTINCLUSAO,  PEPA.IDATIVO, PEPA.NRORG,  PEPA.DTVALPERFILPARC,  PEPA.NRPERFILACESSO, 
                                 PEPA.NRPARCNEGOCIO,  PEPA.NRPERFACESSPARC,  PERF.NMPERFILACESSO, PEPA.DTVALIDADE
                            FROM PERFACESSPARC PEPA
                            JOIN PERFILACESSO PERF ON PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG)
                   WHERE NRORG = :NRORG
                $filters
                $order";
        $params = array(
            'NRORG'   => $organizationId
        );
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deletePartnerAccessProfile($partnerAccessProfileNumber, $organizationId)
    {
        $query = "DELETE 
                    FROM PERFACESSPARC
                   WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                     AND NRORG           = :NRORG";
        
        $params = array(
            'NRPERFACESSPARC'   => $partnerAccessProfileNumber,
            'NRORG'             => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function saveAccessProfileByOperator($accessProfile)
    {
        $query = "INSERT INTO PERFACESSPARC
                              (NRPERFACESSPARC, NRORG, NRPARCNEGOCIO, NRPERFILACESSO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTVALIDADE)
                       VALUES (:NRPERFACESSPARC, :NRORG, :NRPARCNEGOCIO, :NRPERFILACESSO, SYSDATE, :NRORG, :CDOPERINCLUSAO, :DTVALIDADE)";
                               
        $params = array(
            "NRPERFACESSPARC"   => $accessProfile['NRPERFACESSPARC'],
            "NRORG"             => $accessProfile['NRORG'],
            "NRPARCNEGOCIO"     => $accessProfile['NRPARCNEGOCIO'],
            "NRPERFILACESSO"    => $accessProfile['NRPERFILACESSO'],
            "CDOPERINCLUSAO"    => $accessProfile['CDOPERINCLUSAO'],
            "DTVALIDADE"        => $accessProfile['DTVALIDADE'],
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function editAccessProfileByOperator($partnerAccessProfileNumber, $activeId, $organizationId, $dtValidade)
    {
        $query = "UPDATE PERFACESSPARC
                     SET IDATIVO         = :IDATIVO,
                         DTVALIDADE      = :DTVALIDADE
                   WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                     AND NRORG           = :NRORG";
        
        $params = array(
            "IDATIVO"           => $activeId,
            "NRPERFACESSPARC"   => $partnerAccessProfileNumber,
            "NRORG"             => $organizationId,
            "DTVALIDADE"        => $dtValidade
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function updateBusinessPartner($cpf, $inscriptionTypeCode, $operatorName, $organizationId, $userId, $businessPartnership)
    {
        $query = "UPDATE PARCNEGOCIO
                      SET NRINSCRICAOPARC = :NRINSCRICAOPARC,
                          CDTIPOINSCRICAO = :CDTIPOINSCRICAO,
                          NMPRINCIPALPARC = :NMOPERADOR,
                          NMSECUNDARIPARC = :NMOPERADOR,
                          DTULTATU        =  SYSDATE,
                          NRORGULTATU     = :NRORG,
                          CDOPERULTATU    = :NRCDOPERADOR
                    WHERE NRPARCNEGOCIO   = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG";

        $params = array(
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORG"           => $organizationId,
            "NMOPERADOR"      => $operatorName,
            "NRINSCRICAOPARC" => $cpf,
            "NRCDOPERADOR"    => $userId,
            "CDTIPOINSCRICAO" => $inscriptionTypeCode,
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function inactiveAllPartnershipRelationships($userId, $organizationId, $businessPartnership, $activeId)
    {
        $query = "UPDATE RELACIONAPARC
                      SET IDATIVO         = :IDATIVO,
                          CDOPERULTATU    = :CDOPERULTATU,
                          NRORGULTATU     = :NRORGULTATU,
                          DTULTATU        = SYSDATE
                    WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG
                      AND NRTIPORELACIONA = :NRTIPORELACIONA";

        $params = array(
            "NRORG"           => $organizationId,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $userId,
            "IDATIVO"         => $activeId,
            "NRTIPORELACIONA" => 25
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }
        catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function inactiveAllClientPartners($userId, $organizationId, $businessPartnership)
    {
        $query = "UPDATE INTECLIE
                     SET IDATIVO        = :IDATIVO,
                         CDOPERULTATU   = :CDOPERULTATU,
                         NRORGULTATU    = :NRORGULTATU,
                         DTULTATU       = SYSDATE
                   WHERE NRRELACIONAPARC IN (SELECT NRRELACIONAPARC
                                               FROM RELACIONAPARC
                                              WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                                                AND NRORG           = :NRORG
                                                AND NRTIPORELACIONA = :NRTIPORELACIONA)
                     AND NRORG = :NRORG";

        $params = array(
            "NRORG"           => $organizationId,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $userId,
            "IDATIVO"         => "N",
            "NRTIPORELACIONA" => 25
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function setActiveInactivePartnership($userId, $organizationId, $businessPartnership, $activeId)
    {
        $query = "UPDATE PARCERIA
                      SET IDATIVO        = :IDATIVO,
                          CDOPERULTATU   = :CDOPERULTATU,
                          NRORGULTATU    = :NRORGULTATU,
                          DTULTATU       = SYSDATE
                    WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                      AND NRORG          = :NRORG
                      AND CDTIPOPARCEIRO = :CDTIPOPARCEIRO";

        $params = array(
            "NRORG"          => $organizationId,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "NRORGULTATU"    => $organizationId,
            "CDOPERULTATU"   => $userId,
            "IDATIVO"        => $activeId,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function isThereInterlocutorPartnership($organizationId, $businessPartnership)
    {
        $query = "SELECT COUNT(*) AS COUNT
                     FROM PARCERIA
                    WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                      AND NRORG          = :NRORG
                      AND CDTIPOPARCEIRO = :CDTIPOPARCEIRO";

        $params = array(
            "NRORG"          => $organizationId,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]["COUNT"];
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function isTherePartnerRelationship($organizationId, $businessPartnership, $clientCode)
    {
        $query = "SELECT COUNT(*) AS COUNT
                     FROM RELACIONAPARC
                    WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG
                      AND NRTIPORELACIONA = :NRTIPORELACIONA
                      AND NRPARCNEGOCIO  IN (SELECT NRPARCNEGOCIO
                                               FROM CLIENTE
                                              WHERE CDCLIENTE = :CDCLIENTE
                                                AND NRORG     = :NRORG)";

        $params = array(
            "NRORG"           => $organizationId,
            "NRTIPORELACIONA" => 25,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "CDCLIENTE"       => $clientCode
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]["COUNT"];
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function activateClientPartner($organizationId, $businessPartnership, $clientCode, $userId, $activeId)
    {
        $query =  "UPDATE INTECLIE
                      SET IDATIVO         = :IDATIVO,
                          CDOPERULTATU    = :CDOPERULTATU,
                          NRORGULTATU     = :NRORGULTATU,
                          DTULTATU        = SYSDATE
                    WHERE NRRELACIONAPARC IN (SELECT NRRELACIONAPARC
                                                FROM RELACIONAPARC
                                               WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                                              AND NRORG           = :NRORG
                                              AND NRTIPORELACIONA = :NRTIPORELACIONA)
                      AND NRORG           = :NRORG
                      AND CDCLIENTE       = :CDCLIENTE";

        $params = array(
            "NRORG"           => $organizationId,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $userId,
            "IDATIVO"         => $activeId,
            "NRTIPORELACIONA" => 25,
            "CDCLIENTE"       => $clientCode
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function resetPassword($passwordWeb, $passwordDelphi, $cdOperador, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET CDSENHAOPERWEB = :CDSENHAOPERWEB,
                         CDSENHOPER     = :CDSENHOPER,
                         IDSTATUSENHA   = 'A'
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG       = :NRORG";
        
        $params = array(
            'CDSENHAOPERWEB' => $passwordWeb,
            'CDSENHOPER'     => $passwordDelphi,
            'CDOPERADOR'     => $cdOperador,
            'NRORG'          => $organizationId
        );
        
        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function updatePassword($passwordWeb, $passwordDelphi, $userId, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET CDSENHAOPERWEB = :CDSENHAOPERWEB,
                         CDSENHOPER     = :CDSENHOPER,
                         IDSTATUSENHA   = 'A'
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";
        
        $params = array(
            'CDSENHAOPERWEB' => $passwordWeb,
            'CDSENHOPER'     => $passwordDelphi,
            'CDOPERADOR'     => $userId,
            'NRORG'          => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch(\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function uploadProfilePicture($imagePath, $userId, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET DSIMAGEMOPER = :DSIMAGEMOPER
                   WHERE CDOPERADOR    = :CDOPERADOR
                     AND NRORG          = :NRORG"; 
        
        $params = array(
            'DSIMAGEMOPER'  => $imagePath,
            'CDOPERADOR'    => $userId,
            'NRORG'         => $organizationId
        );
        
        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteRelatedBranch($cdfilial, $cdoperador, $organizationId)
    {
        $query = "DELETE 
                    FROM FILIOPER
                   WHERE CDFILIAL   = :CDFILIAL
                     AND CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";
        
        $params = array(
            'CDOPERADOR' => $cdoperador,
            'CDFILIAL'   => $cdfilial,
            'NRORG'      => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function saveRelatedBranchByOperator($relatedBranch)
    {
        $query = "INSERT INTO FILIOPER
                              (NRORG, CDFILIAL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORGFILIAL, IDATIVO, DTVALIDADE)
                       VALUES (:NRORG, :CDFILIAL, :CDOPERADOR, SYSDATE, :NRORG, :CDOPERINCLUSAO, :NRORG, :CDOPERINCLUSAO, :NRORG, 'S', :DTVALIDADE)";
                               
        $params = array(
            "CDFILIAL"          => $relatedBranch['CDFILIAL'],
            "NRORG"             => $relatedBranch['NRORG'],
            "CDOPERADOR"        => $relatedBranch['CDOPERADOR'],
            "CDOPERINCLUSAO"    => $relatedBranch['CDOPERINCLUSAO'],
            "DTVALIDADE"        => $relatedBranch['DTVALIDADE'],
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function editRelatedBranchByOperator($cdfilial, $cdoperador, $activeId, $organizationId, $dtValidade)
    {
        $query = "UPDATE FILIOPER
                     SET IDATIVO         = :IDATIVO,
                         DTVALIDADE      = :DTVALIDADE
                   WHERE CDFILIAL        = :CDFILIAL
                     AND NRORG           = :NRORG
                     AND CDOPERADOR      = :CDOPERADOR";
        
        $params = array(
            "IDATIVO"           => $activeId,
            "CDFILIAL"          => $cdfilial,
            "CDOPERADOR"        => $cdoperador,
            "NRORG"             => $organizationId,
            "DTVALIDADE"        => $dtValidade
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteRelatedOrganization($nrorgrel, $cdoperador, $organizationId)
    {
        $query = "DELETE 
                    FROM ORGANIZACAOOPER
                   WHERE NRORGREL   = :NRORGREL
                     AND CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";
        
        $params = array(
            'CDOPERADOR' => $cdoperador,
            'NRORGREL'   => $nrorgrel,
            'NRORG'      => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function saveRelatedOrganizationByOperator($relatedOrganization)
    {
        $query = "INSERT INTO ORGANIZACAOOPER
                              (NRORGANIZACAOOPER, NRORG, NRORGREL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO)
                       VALUES (:NRORGANIZACAOOPER, :NRORG, :NRORGREL, :CDOPERADOR, SYSDATE, :NRORG, :CDOPERINCLUSAO, :NRORG, :CDOPERINCLUSAO, 'S')";
                               
        $params = array(
            "NRORGANIZACAOOPER" => $relatedOrganization['NRORGANIZACAOOPER'],
            "NRORGREL"          => $relatedOrganization['NRORGREL'],
            "NRORG"             => $relatedOrganization['NRORG'],
            "CDOPERADOR"        => $relatedOrganization['CDOPERADOR'],
            "CDOPERINCLUSAO"    => $relatedOrganization['CDOPERINCLUSAO']
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function editRelatedOrganizationByOperator($nrorgrel, $cdoperador, $activeId, $organizationId)
    {
        $query = "UPDATE ORGANIZACAOOPER
                     SET IDATIVO         = :IDATIVO
                   WHERE NRORGREL        = :NRORGREL
                     AND CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";
        
        $params = array(
            "IDATIVO"           => $activeId,
            "NRORGREL"          => $nrorgrel,
            "CDOPERADOR"        => $cdoperador,
            "NRORG"             => $organizationId
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function importBranches($operatorCode, $organizationId, $branch, $operatorInclusionCode)
    {
        $query = "INSERT INTO FILIOPER
                              (CDOPERADOR, CDFILIAL, IDEXIBAVISO, DTULTATUFOPE, NRORG, DTINCLUSAO,
                              DTULTATU, NRORGINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORGFILIAL,
                              IDATIVO)
                      VALUES (:CDOPERADOR, :CDFILIAL, 'S', SYSDATE, :NRORG, SYSDATE,
                              SYSDATE, :NRORG, :NRORG, :CDOPERINCLUSAO, :NRORG,
                              'S')";
        
        $params = array (
            "CDOPERADOR"          => $operatorCode,
            "NRORG"               => $organizationId,
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDFILIAL"            => $branch
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function getParamGeralByNrorg($nrOrg)
    {
        $query = "SELECT CDPICTPROD, CDPICTPRAT, IDUTCTRLACESWEB
                    FROM PARAMGERAL
                   WHERE NRORG = :NRORG";

        $params = array( 'NRORG' => $nrOrg );
        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        $result = $std->fetchAll();
        return !empty($result) ? $result[0] : $result;
    }

    public function findFilioper($filters, $order)
    {
        $query = <<<SQL
            SELECT CDOPERADOR, CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE, NRORG
                   DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU,
                   CDOPERULTATU, IDATIVO, NMFILIAL, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE
              FROM (SELECT CDOPERADOR, FIOP.CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE,
                           FIOP.NRORG, FIOP.DTINCLUSAO, FIOP.DTULTATU, FIOP.NRORGINCLUSAO,
                           FIOP.CDOPERINCLUSAO, FIOP.NRORGULTATU, FIOP.CDOPERULTATU, FIOP.IDATIVO,
                           FILI.NMFILIAL, FIOP.DTVALIDADE
                      FROM FILIOPER FIOP
                      JOIN FILIAL FILI ON FILI.NRORG = FIOP.NRORG
                                      AND FILI.CDFILIAL = FIOP.CDFILIAL)
              WHERE 1 = 1
              $filters
              $order
SQL;
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query);
    }

    public function deleteFilioper($cdOperador, $cdFilial, $nrOrg)
    {
        $query = "DELETE
                    FROM FILIOPER
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND CDFILIAL   = :CDFILIAL
                     AND NRORG      = :NRORG";

        $params = array(
            "CDOPERADOR" => $cdOperador,
            "CDFILIAL"   => $cdFilial,
            "NRORG"      => $nrOrg
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findParamGeral($organizationId)
    {
        $query = "SELECT IDGERACDOPERADOR
                    FROM PARAMGERAL
                   WHERE NRORG = :NRORG";

        $params = array(
            "NRORG" => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('IDGERACDOPERADOR' => "N"));
        };
    }
}