
// FILE: assets/js/util/MasterDetailUtilities.js
function MasterDetailUtilities(ScreenService, templateManager, Query, ZHPromise) {

	var self = this;

	this.masterDetailRollback = function(widget) {
		if (widget.currentRow.__is_new) {
			if (widget.dataSource.data.indexOf(widget.currentRow) !== -1) {
				widget.dataSource.data.pop();
			}
			this.masterDetailBackToList(widget);
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
	};

	this.masterDetailEdit = function(widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
	};

	this.masterDetailSave = function(widget, dataSetName) {
		if (widget.isValid()) {
			widget.currentRow.changed = true;
			return widget.saveRow().then(function(response) {

				widget.currentRow.changed = false;
				widget.currentRow.__is_new = false;

				if (response.dataset[dataSetName][0]._id) {
					widget.currentRow._id = response.dataset[dataSetName][0]._id;
				}
				widget.view();
			});
		}
		return ZHPromise.reject();
	};

	this.masterDetailBackToList = function(widget) {
		ScreenService.closeSwipe().then(function() {
			widget.list();
			widget.activate();
		});
	};

	self.materDetailBackToParent = function(widget) {
		ScreenService.closeSwipe().then(function() {
			widget.parent.list();
			widget.parent.activate();
		});
	};

	this.formBackToView = function (widget) {
		widget.view();
		widget.template = widget.templateToView;
	};

	this.formCancelToView = function (widget) {
		widget.currentRow = widget.backupRow || widget.currentRow;
		widget.view();
		widget.template = widget.templateToView;
	};

	this.formEdit = function (widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		widget.template = widget.templateToForm;
	};

	this.afterMoveRow = function(widget, swipeLevel){
		widget.activate();
		ScreenService.closeSwipe(swipeLevel);
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('MasterDetailUtilities', MasterDetailUtilities);
});

// FILE: assets/js/util/PeriodsUtilities.js
/*jshint -W054 */
function PeriodsUtilities(ScreenService, restEngine) {
    var INVALID_PERIOD_FORMAT_MESSAGE = "Used period has an incorrect format.";

    var PREFIX_THIS = "this",
		PREFIX_NEXT = "next",
		PREFIX_LAST = "last",
		PREFIXES	= [PREFIX_THIS, PREFIX_NEXT, PREFIX_LAST];

	var SUFFIX_INCLUSIVE = "inclusive",
		SUFFIXES		 = [SUFFIX_INCLUSIVE];

	var UNIT_DAY	= "day",
		UNIT_WEEK	= "week",
		UNIT_MONTH	= "month",
		UNIT_YEAR	= "year",
		UNITS		= [UNIT_DAY, UNIT_WEEK, UNIT_MONTH, UNIT_YEAR];

	var PREPOSITION_IN    = "in",
		PREPOSITION_UNTIL = "until",
		PREPOSITIONS	  = [PREPOSITION_IN, PREPOSITION_UNTIL];

	var EXCEPTION_UNITS = {
        "days"   : UNIT_DAY,
        "weeks"  : UNIT_WEEK,
        "months" : UNIT_MONTH,
        "years"  : UNIT_YEAR
    };

    var ONE_WORD_EXPRESSIONS = {
        "today" 	: "this-day",
        "yesterday"	: "last-day",
        "tomorrow"	: "next-day"
    };

    var DATE_FORMAT = "DD/MM/YYYY";

	var dataSourcePeriods = [];

    var self = this;

    // Inserting

    this.setIntervalsOnPeriodField = function(periodField){
	    self.getFormatedIntervals().then(function(formatedIntervals){
	    	periodField.intervals = formatedIntervals;
	    });
	};

    this.setDataSourcePeriods = function (intervals) {
			dataSourcePeriods = intervals;
	};

	this.setDataSourceData = function(widget){
		widget.dataSource.data = widget.parent.currentRow.indicators;
	};

	this.getFormatedIntervals = function(){
		return self.getIntervals().then(function(response){
		    var intervals = self.formatPeriods(response.dataset.periods);
		    return intervals;
		});
	};

	this.getIntervals = function(){
		return restEngine.post("/periods",
			{
				"requestType": "FilterData",
				"filter": []
			}
		);
	};
	self.getPeriodMathExpression = function(_id){
		var interval = {};
		dataSourcePeriods.some(function (period) {
			if(period._id === _id && !period.Erro){
				interval = period.interval;
				interval.label = period.name;
				return true;
			}
		});
		return interval;
	};

	self.getFunctionPeriodMathExpression = function(_id){
		return "@PeriodsUtilities.getPeriodMathExpression('" + _id + "')@";
	};

	this.formatPeriods = function(periods){
		var intervals = [];
		dataSourcePeriods = periods;
		periods.forEach(function(period) {
			var periodCode = period.periodsCode;
			if(period.custom){
				if(period.END_DATE_EXPRESSION || period.START_DATE_EXPRESSION){
					periodCode = self.getFunctionPeriodMathExpression(period._id);
				}else if(period.custom  && period.periodsCode){
					periodCode = self.getCustomPeriodCode(period.periodsCode);
				}else{
					periodCode = 'today';
				}
			}
			intervals.push({
				label: period.base ? ScreenService.i18n(period.name) : period.name,
				period: periodCode,
				custom: period.custom
			});
		});

		return intervals;
	};

	// Decoding

	this.decodePeriod = function(periodCode){
		var now = moment();
		var start = moment();
		var end = moment();

		if(periodCode){

            if(periodCode == "last-month|until-last-day|in-last-month" && now.format('DD') == '01'){
                periodCode = "until-last-day|in-last-month";
            }
			var periodParams = getPeriodParams(periodCode);

			periodParams.forEach(function(params) {
				var period = buildInterval(params, start, end);
				start = period.start;
				end = period.end;
			});
		}

		var interval = factoryInterval(start, end);
		return interval;
	};

	function getPeriodParams(periodCode){
		var periodParams = [];

		var expressions = periodCode.split("|");
		expressions.forEach(function(expression){
			var param = decodeExpression(expression);
			periodParams.push(param);
		});

		return periodParams;
	}

	function decodeExpression(expression) {
		var splitedExpression = [];
		expression.split("-").forEach(function(expressionPart){
			expressionPart = expressionPart in ONE_WORD_EXPRESSIONS ? ONE_WORD_EXPRESSIONS[expressionPart] : expressionPart;
			expressionPart = expressionPart.split("-");

			expressionPart.forEach(function(part){
				splitedExpression.push(part);
			});
		});

		var decodedExpression = recognizeExpressionTerms(splitedExpression);
		return decodedExpression;
	}

	function recognizeExpressionTerms(splitedExpression) {
		var decodedExpression = {};
		splitedExpression.forEach(function(term){
			if(EXCEPTION_UNITS[term]){
				term = EXCEPTION_UNITS[term];
			}

			if(PREPOSITIONS.indexOf(term) !== -1) {
				decodedExpression.preposition = term;
			} else if (PREFIXES.indexOf(term) !== -1){
				decodedExpression.prefix = term;
			} else if (UNITS.indexOf(term) !== -1) {
				decodedExpression.unit = term;
			} else if (!isNaN(parseInt(term))) {
				decodedExpression.amount = term;
			} else if (SUFFIXES.indexOf(term) !== -1) {
				decodedExpression.suffix = term;
			} else {
				throw INVALID_PERIOD_FORMAT_MESSAGE;
			}
		});

		return decodedExpression;
	}

	function buildInterval(params, start, end) {
		var preposition = params.preposition || null;
		var prefix		= params.prefix 	 || null;
		var amount		= params.amount 	 || 1;
		var unit		= params.unit		 || null;
		var suffix		= params.suffix		 || null;


		switch(prefix){
			case PREFIX_LAST:
				return buildIntervalToLastPrefix(preposition, amount, unit, suffix, start, end);
			case PREFIX_NEXT:
				return buildIntervalToNextPrefix(preposition, amount, unit, suffix, start, end);
			case PREFIX_THIS:
				return buildIntervalToThisPrefix(preposition, unit, suffix, start, end);
			case null:
				return buildIntervalToNoPrefix(preposition, amount, unit, suffix, start, end);
		}
	}

	function buildIntervalToLastPrefix(preposition, amount, unit, suffix, periodStart, periodEnd) {
		var endAmount = suffix === SUFFIX_INCLUSIVE ? 0 : 1;
		amount = suffix === SUFFIX_INCLUSIVE ? amount-1 : amount;

		var start = moment().subtract(amount, unit);
		var end = moment().subtract(endAmount, unit);

		if (preposition === PREPOSITION_IN) {
			periodStart = getMomentWhenInClause(periodStart, unit, start);
			periodEnd = getMomentWhenInClause(periodEnd, unit, end);
		} else {
			periodEnd = end.endOf(unit);

			if(!preposition){
				periodStart = start.startOf(unit);
			}
		}
        if(periodStart > periodEnd){
            periodEnd = copyDate(periodStart);
        }

		return {
			start: periodStart,
			end: periodEnd
		};
	}

    var copyDate = function(data){
        return moment(data.format("YYYY-MM-DD"), "YYYY-MM-DD");
    };

	function buildIntervalToNextPrefix(preposition, amount, unit, suffix, periodStart, periodEnd) {
		var startAmount = suffix === SUFFIX_INCLUSIVE ? 0 : 1;
		var endAmount = suffix === SUFFIX_INCLUSIVE ? amount - 1 : amount;

		var start = moment().add(startAmount, unit);
		var end = moment().add(endAmount, unit);

		if (!preposition) {
			periodStart = start.startOf(unit);
			periodEnd = end.endOf(unit);
		}
		else if (preposition === PREPOSITION_UNTIL) {
			periodEnd = end.startOf(unit);
		}
		else if (preposition === PREPOSITION_IN) {
			periodStart = getMomentWhenInClause(periodStart, unit, start);
			periodEnd = getMomentWhenInClause(periodEnd, unit, end);
		}

		return {
			start : periodStart,
			end : periodEnd
		};
	}

	function buildIntervalToThisPrefix(preposition, unit, suffix, periodStart, periodEnd) {
		var start = moment();
		var end = moment();

		if (!preposition) {
			periodStart = start.startOf(unit);
			periodEnd = end.endOf(unit);
		} else if (preposition === PREPOSITION_UNTIL) {
			if(suffix !== SUFFIX_INCLUSIVE){
				end.subtract(1, unit);
			}
			periodEnd =  end.endOf(unit);
		} else if (preposition === PREPOSITION_IN) {
			periodStart = getMomentWhenInClause(periodStart, unit, start);
			periodEnd = getMomentWhenInClause(periodEnd, unit, end);
		}

        if(periodStart > periodEnd){
            periodEnd = copyDate(periodStart);
        }

		return {
			start: periodStart,
			end: periodEnd
		};
	}

	function buildIntervalToNoPrefix(preposition, amount, unit, suffix, periodStart, periodEnd){
		if(!preposition){
			periodStart = setDateParam(unit, amount, periodStart);
		} else if(preposition === PREPOSITION_UNTIL){
			periodEnd = setDateParam(unit, amount, periodEnd);
		}

		return {
			start : periodStart,
			end : periodEnd
		};
	}

	function setDateParam(unit, amount, moment) {
		switch(unit){
			case UNIT_DAY:
				return moment.date(amount);
			case UNIT_WEEK:
				return moment.week(amount);
			case UNIT_MONTH:
				return moment.month(amount);
			case UNIT_YEAR:
				return moment.year(amount);
		}
	}

	function getMomentWhenInClause(initialMoment, unit, newMoment) {
		var periodDate = initialMoment._d;
		var newDate = newMoment._d;
		getNewDateFromParams(unit, periodDate, newDate);
		return moment(periodDate);
	}

	function getNewDateFromParams(unit, periodDate, newDate) {
		if (unit === UNIT_DAY) {
			periodDate = newDate;
		}
		else if (unit === UNIT_MONTH) {
			periodDate.setMonth(newDate.getMonth());
			periodDate.setFullYear(newDate.getFullYear());
		}
		else if (unit === UNIT_YEAR) {
			periodDate.setFullYear(newDate.getFullYear());
		}
	}

	function factoryInterval(start, end) {
		if(!start){
			start = moment();
		}
		if(!end){
			end = moment();
		}
		return {
			"start": start.format(DATE_FORMAT),
			"end": end.format(DATE_FORMAT),
			"customized": true
		};
	}

	//Getting Periods Informations

	this.getPeriodValueByCodedName = function(periodCode){
		return periodCode.split("'")[1] || periodCode;
	};

	this.getCustomPeriodCode = function(periodString){
		var periodCode = '';
		if(periodString){
			var interval = self.getPeriodMathExpression(periodString);
			var condition = this.conditionPeriod(periodString,true);
			if(interval && interval._id){
				periodCode = self.getFunctionPeriodMathExpression(periodString);
			}else if(periodString.split("if (",1)[0] != ""){
				periodCode = "@PeriodsUtilities.decodePeriod('" + periodString + "')@";
            }
            else {
				periodCode = "@PeriodsUtilities.decodeCondition('" + condition + "')@";
            }
		}
		return periodCode;
	};

	this.isHexadecimal = function (str)
	{
		var regexp = /^[0-9a-fA-F]+$/;

		if (regexp.test(str))
		{
			return true;
		}
		else
		{
			return false;
		}
	};

	this.getPeriodFromCodeAndPeriods = function(periodCode, intervals){
		var period;
		if(intervals && intervals.length){
			intervals.forEach(function(interval){
				if(interval.periodsCode.split("if (",1)[0] === "") {
					if(self.conditionPeriod(interval.periodsCode ,true) === periodCode){
						period = interval;
					}
				}
				else if(interval.periodsCode === periodCode){
					period = interval;
				}
				if(periodCode === interval._id){
					period = interval;
				}
			});
		}
		return period;
	};

	this.validatePeriodIntervals = function(period){
		if(period && period.start && period.end){
			return true;
		}
	};

	this.getPeriodLabelFromIntervals = function(intervals, periodCode){
		var period = intervals.filter(function(interval){
			return interval.periodsCode === periodCode;
		}).shift();

		return period ? period.name : null;
	};

	this.removeCustomLabel = function(periodField) {
		periodField.label = periodField.label.split(' ')[0];
	};

    this.conditionPeriod = function(periodString,isCompare){
        if(periodString.split("if (",1)[0] === "")
        {
			var retorno = '{';
			var dot = ';';
			if( !periodString.split("return")[1])
			{
				retorno = '{ return "';
				dot = '";';
			}
            var periodTest = periodString.split('{');
            var period = "";
            for(var i=0;i<periodTest.length - 1;i++){
                period = period+periodTest[i]+retorno;
            }
            period = period + periodTest[periodTest.length - 1];
            periodTest = period.split(';');
            period = "";
            for(i=0;i<periodTest.length -1;i++){
                period = period+periodTest[i]+dot;
            }
            period = period + periodTest[periodTest.length - 1];
            if(isCompare === true){
                return period;
            }else{
                var condition  = new Function(period);
                return condition();
            }
        }
        else{
            return periodString;
        }
    };

    this.decodeCondition = function(period){
        if(period.split("if (",1)[0] === "") {
            var condition  = new Function(period);
            return this.decodePeriod(condition());
        }
        else {
            return null;
        }
    } ;

	//New Date expression
	this.getDateByExpressions = function (expressions) {
		return restEngine.post({
			service: "/getDateByDateExpression",
			typeRequest: "POST",
			params: { "requestType": "Row", "row": { "datesExpression": expressions}  },
			timeout: 150000
		}).then(function (response) {
			return response.dataset.getDateByDateExpression;
		});
	};

}





Configuration(function(ContextRegister) {
	ContextRegister.register('PeriodsUtilities', PeriodsUtilities);
});

// FILE: assets/js/util/UtilitiesController.js
function UtilitiesController($rootScope, ScreenService, templateManager, Query, ZHPromise, $location,
	PeriodsUtilities, MessageService,requestDataSourceEngine, restEngine) {

	/* ****** FRONT VERSION ****** */
	this.frontVersion = "1.0.0";
	/* *************************** */

	var self = this;

	self.itemClick = function(widget) {
		console.log(widget);
	};

	self.copiedToClipboard = function (texto,textarea) {
		var inputTest = null;
		if (textarea) {
			inputTest = document.createElement("textarea");
		} else {
			inputTest = document.createElement("input");
		}
		inputTest.value = texto;
		document.body.appendChild(inputTest);
		inputTest.select();
		document.execCommand('copy');
		document.body.removeChild(inputTest);
		MessageService.successNotificationByCode('COPIED');
	};

	this.getUrlZmartNext = function(route) {
        var token = localStorage.getItem('LOGIN_TOKEN');
        var hash = localStorage.getItem('LOGIN_HASH') || '0';
        var user = localStorage.getItem('LOGIN_USER');
        var languange = localStorage.getItem('LOGIN_LANGUAGE');
		var productId = window.projectId;

		// Clear Zmart session timeout to use only Zmart Next session timeout.
		clearTimeout(self.expiredSessionMethod);

		return Config.zmartNextUrl + "/" + route + "?header=0&menu=0" +
			"&token=" + token +
			"&hash=" + hash +
			"&user=" + user +
			"&languange=" + languange +
			"&productId=" + productId;
    };

	this.removeMessage = function(message) {
		var messageContainer = $('.zh-notification-messages');
		var item = messageContainer.find('#' + message.id);
		if(item){
			var index = $rootScope.notificationMessages.indexOf(message);
			if (index > -1) {
				$rootScope.notificationMessages.splice(index, 1);
			}
		}
	};

	this.cancelEditRow = function(widget) {
		if (widget.isNewRow()) {
			widget.dataSource.data.pop();
			widget.currentRow = {};
			ScreenService.closeSwipe();
		}
		else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
	};

	this.goBackWidget = function(widget) {
		ScreenService.closeSwipe();
		widget.activate();
	};

	this.showErrorMessageTrue = function(response) {
		var message = '';
		if(response && response.data && response.data.error){
			message = response.data.error;
		}else if(response && response.error){
			message = response.error;
		}else{
			message = response;
		}

		ScreenService.showMessage(message);
	};

	this.saveRow = function(widget) {
		if (widget.isValid()) {
			widget.saveRow().then(function(response) {
				widget.view();
			}, function(response) {
				ScreenService.showMessage(response.data.error);
			});
		}
		else {
			MessageService.showMessageByCode("REQUIRED_FIELDS");
		}
	};

	var propertiesToClear = [
		'__createdLocal',
		'__is_new',
		'__isSelected',
		'changed',
		'__index',
		'__notSelectable'
	];

	this.setServerIp = function(row) {

		if (this.checkIpdata(row)) {
			if (row.port !== undefined) {
				serverIp = row.ip + ':' + row.port;
			}
			else {
				serverIp = row.ip;
			}
		}
		templateManager.updateURL(serverIp);
	};

	this.checkIpdata = function(row) {
		if (row.ip === undefined) {
			ScreenService.showMessage('Informe o IP');
			return false;
		}
		return true;
	};

	this.checkEmail = function(email) {
		usuario = email.substring(0, email.indexOf("@"));
		dominio = email.substring(email.indexOf("@") + 1, email.length);

		if ((usuario.length >= 1) && (dominio.length >= 3) && (usuario.search("@") == -1) && (dominio.search("@") == -1) && (usuario.search(" ") == -1) &&
			(dominio.search(" ") == -1) && (dominio.search(".") != -1) && (dominio.indexOf(".") >= 1) && (dominio.lastIndexOf(".") < dominio.length - 1)) {
			return true;
		}
		else {
			return false;
		}
	};

	this.generateId = function() {
		return new Date().getTime().toString();
	};

	this.daysAgo = function(number) {
		var response = {
			"start": moment().subtract(number, 'day').format("DD/MM/YYYY"),
			"end": moment().format("DD/MM/YYYY")
		};
		return response;
	};

	this.currentDate = function(decrement) {
		var date = new Date();
		date.setDate(date.getDate() - decrement);
		var day = date.getDate();
		if (day.toString().length == 1)
			day = "0" + day;
		var month = date.getMonth() + 1;
		if (month.toString().length == 1)
			month = "0" + month;
		var ano = date.getFullYear();
		return day + "/" + month + "/" + year;
	};

	this.getWeekDay = function(dateString) {
		var date = new Date();
		day = dateString.substr(0, 2);
		month = dateString.substr(3, 2);
		year = dateString.substr(6, 9);
		date.setDate(day);
		date.setMonth(month - 1);
		date.setYear(year);
		weekDay = date.getDay();
		switch (weekDay) {
			case 0:
				weekDay = "Sunday";
				break;
			case 1:
				weekDay = "Monday";
				break;
			case 2:
				weekDay = "Tuesday";
				break;
			case 3:
				weekDay = "Wednesday";
				break;
			case 4:
				weekDay = "Thursday";
				break;
			case 5:
				weekDay = "Friday";
				break;
			case 6:
				weekDay = "Saturday";
				break;
		}
		return weekDay;
	};

	this.searchMultiSelectDatasource = function(widget, nameField) {
		if (widget.container.getWidget('indicators').currentRow._id !== undefined) {
			var row = this.getRowByFieldName(widget, nameField);

			if (row !== undefined) {
				var data = this.stringToObjectArray(row);
				widget.getField(nameField).dataSource.data = data;
				templateManager.updateTemplate();
			}
		}
	};

	this.copyObject = function(origin) {
		var copy = (origin instanceof Array) ? [] : {};
		for (var i in origin) {
			if (origin[i] && typeof origin[i] == 'object') {
				copy[i] = copyObject(origin[i]);
			}
			else {
				copy[i] = origin[i];
			}
		}
		return copy;
	};

	this.addGenericData = function(array, properties) {
		var currentPosition;
		var genericData;
		var resultArray = [];
		for (var idx in array) {
			currentPosition = array[idx];
			currentPosition = this.setProperties(currentPosition, properties);
			resultArray.push(currentPosition);
		}
		return resultArray;
	};

	this.setProperties = function(currentPosition, properties) {
		currentPosition.genericData = {};
		properties.forEach(function(property) {
			if (currentPosition[property] !== undefined) {
				currentPosition.genericData[property] = currentPosition[property];
			}
		});
		return currentPosition;
	};

	this.switchAddgenericData = function(array, nameField) {
		switch (nameField) {
			case 'gaugeLevels':
				array = this.addGenericData(array, gaugeLevels);
				break;
		}
		return array;
	};

	this.fillObjectsToMultiSelect = function(nameField, requireData, widget) {
		var field = widget.getField(nameField);
		field.dataSource.data = [];
		widget.currentRow[nameField] = [];
		if (requireData !== undefined) {
			requireData[nameField] = this.switchAddgenericData(requireData[nameField], nameField);
			requireData[nameField].forEach(function(item) {
				widget.currentRow[nameField].push(item.genericData);
			});
			field.dataSource.data = requireData[nameField];

			templateManager.updateTemplate();
		}
	};

	this.openPopUpGenericArray = function(widget, field) {
		widget.ownerField = field;
		widget.label = 'Add ' + field.label;
		ScreenService.openPopup(widget);
	};

	this.activeWidget = function(widget) {
		widget.activate();
	};

	this.closeSwipeAndActivateParent = function(widget) {
		ScreenService.closeSwipe().then(function() {
			widget.parent.currentWidget = widget.parent;
			widget.parent.activate();
		});
	};

	this.stringToObjectArray = function(array) {
		var newArray = [];
		for (i = 0; i < array.length; i++) {
			var obj = {};
			obj.genericData = array[i];
			newArray.push(obj);
		}
		return newArray;
	};

	this.getRowByFieldName = function(widget, nameField) {
		return widget.currentRow[nameField];
	};

	this.setVisibleWidget = function(widget) {
		widget.isVisible = true;
	};

	this.activateAndList = function(widget) {
		widget.activate();
		widget.list();
	};

	this.callRepositoryMethod = function(method, scope, dataSetName, args) {
		return method.apply(scope, args).then(function(response) {
			if(dataSetName){
				return response.dataset && response.dataset[dataSetName] ? response.dataset[dataSetName] : [];
			} else {
				return response.dataset && response.dataset ? response.dataset : [];
			}
		});
	};

    this.getRowInDataSource = function (dataSource, primaryProperty, primaryValue) {
        var currentRow = null;
        dataSource.data.some(function (row) {
            if (row[primaryProperty] == primaryValue) {
                currentRow = row;
                return true;
            }
        });
        return currentRow;
    };

	// Master detail general functions
	this.masterDetailRollback = function(widget) {
		if (widget.currentRow.__is_new) {
			if (widget.dataSource.data.indexOf(widget.currentRow) !== -1) {
				widget.dataSource.data.pop();
			}
			this.masterDetailBackToList(widget);
		}
		else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
	};

	this.masterDetailEdit = function(widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
	};

	this.masterDetailSave = function(widget, dataSetName) {
		if (widget.isValid()) {
			widget.currentRow.changed = true;
			return widget.saveRow().then(function(response) {

				widget.currentRow.changed = false;
				widget.currentRow.__is_new = false;

				widget.currentRow._id = response.dataset[dataSetName][0]._id;
				widget.reload();
				widget.view();
			});
		}
		return ZHPromise.reject();
	};

	this.masterDetailBackToList = function(widget) {
		return ScreenService.closeSwipe().then(function() {
			widget.list();
			self.activeWidget(widget);
		});
	};

	this.setSelectValueNullWhenEmpty = function(field, valueField) {
		if (!field.value()) {
			field.widget.currentRow[valueField] = null;
		}
	};

	var _conditionsMap = {
		string: function(query, property, source) {
			query.where(property).equals(source[property]);
		},
		object: function(query, property, source) {
			query.where(property.name)[property.method](source[property.name]);
		},
		orderBy_string: function(query, property, source) {
			query.orderBy(source[property], Query.ORDER_ASC);
		},
		orderBy_object: function(query, property, source) {
			query.orderBy(source[property.name], property.direction || Query.ORDER_ASC);
		}
	};

	this.buildQueryFilters = function(source, properties, orderByProperties, queryObj) {
		var query = Query.build();
		if (source) {
			Util.parseToArray(properties).forEach(function(property) {
				_conditionsMap[(typeof property)](query, property, source);
			});

			Util.parseToArray(orderByProperties).forEach(function(property) {
				_conditionsMap["orderBy_" + (typeof property)](query, property, source);
			});
		}
		return queryObj ? query : query.where();
	};

	self.transformArrayInData = function(array) {
		return array.map(function(element) {
			return {
				"name": element
			};
		});
	};

	self.findByMongoId = function(data, mongoId) {
		return data.filter(function(element) {
			return element._id === mongoId;
		}).shift();
	};

	self.changeWidgetVisibility = function(widgets, isVisible) {
		widgets.forEach(function(widget) {
			widget.isVisible = isVisible;
		});
	};

	self.sortByOrder = function(menuA, menuB) {
		if (menuA.order !== null && (menuB.order === null || menuA.order < menuB.order)) {
			return -1;
		}
		else if (menuB.order !== null && (menuA.order === null || menuB.order < menuA.order)) {
			return 1;
		}
		else if (menuA.order === menuB.order) {
			return 0;
		}
	};

	self.isColumnGenerated = function(column) {
		return (column.isGenerated === true || (Util.isUndefinedOrNull(column.isGenerated) && !column.isExternal));
	};

	var factoryExpiredSessionMethods = function() {
		if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
			self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
		}
	};

	var validateExpiredSession = function() {
		var timeout = templateManager.project.onRequestIdleTimeout;
		var filter = [ {
			name: 'TIMEOUT',
			value: timeout,
			operator: '='
		} ];

		requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
			if(response && response.dataset && response.dataset.expiredSessionInfo) {
				var expiredSessionInfo = response.dataset.expiredSessionInfo;
				if(expiredSessionInfo.IS_EXPIRED) {
					eventAggregator.publish('LibExpiredSession');
				} else {
					if(self.expiredSessionMethod) {
						clearTimeout(self.expiredSessionMethod);
					}
					Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
				}
			} else {
				self.showErrorMessage(response);
			}
		}).catch(self.showErrorMessage);
	};

	self.killAllAsyncProcesses = function(){
		var id = window.setTimeout(function() {}, 0);
    	for(var i=id;i>=0;i--){
        	window.clearTimeout(i);
    	}
    	if(ScreenService.isSwipeOpen()){
    		ScreenService.closeSwipe(0);
		}
		factoryExpiredSessionMethods();
	};

	self.removeFrameworkProperties = function(row) {
		propertiesToClear.forEach(function(property) {
			delete row[property];
		});
		return row;
	};

	this.getProductBaseUrl = function(){
		return requestModulesMetadata().then(function (modules) {
			return modules.product.baseUrl;
		});
	};

	self.openWindow = function(windowName, newWindow) {
        return requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module.replace("/","")].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
	};

	var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

	self.cleanInterval = function(currentRow, fieldName) {
		fieldName = fieldName || 'period';
		var field = currentRow[fieldName];

		if (field) {
			if (field.period && field.period.name) {
				if (field.period.name[0] !== ('@')) {
					field = field.period.name;
				}
				else {
					var periodValue = PeriodsUtilities.getPeriodValueByCodedName(field.period.name);
					if (periodValue) {
						field = periodValue;
					}
				}
			}
			else if (!field.end && !field.start) {
				field = "";
			}

			currentRow[fieldName] = field;
		}
	};

	self.showMessages = function(messages) {
		if (messages) {
			ScreenService.showMessage(messages[0].message, messages[0].type, undefined, messages[0].variables);
		}
	};

	this.setSearchParameterOrPush = function(search, key, value, subscribe) {
		if ((key == 'drV' || key == 'drP') && Util.isArray(search[key])) {
            search[key].push(value);
        } else if (key == 'sId' || key == 'sName') {
            search[key] = value;
		} else if ((search[key] || search[key] === '') && (!subscribe)) {
			if (search[key] === '') {
				if (Util.isArray(value)) {
					search[key] = [];
					search[key][search.vL] = value;
				}
				else {
					search[key] = value;
				}
			}
			else if (Util.isArray(search[key])) {
				if (search[key][search.vL]) {
					if (Util.isArray(search[key][search.vL])) {
						search[key][search.vL].push(value);
					}
					else {
						search[key][search.vL] = [search[key][search.vL], value];
					}
				}
				else {
					search[key][search.vL] = value;
				}
			}
			else {
				if (Util.isArray(value)) {
					search[key] = [search[key]];
					search[key][search.vL] = value;
				}
				else {
					search[key] = [search[key], value];
				}
			}
		}
		else {
			search[key] = typeof(search.drV) === "undefined" && key === "drP" ? undefined : value;
		}
	};

	 self.downloadFileInJs = function(filename, text) {
		var element = document.createElement('a');
		element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(text));
		element.setAttribute('download', filename);

		element.style.display = 'none';
		document.body.appendChild(element);

		element.click();

		document.body.removeChild(element);
	};

    this.mergeDataSetPagnation = function (field, filterMerge, valueFieldIndex, notFilter) {
        if(filterMerge){
            var filter = [];
            if(field.dataSourceFilter && !notFilter){
                filter = filter.concat(field.dataSourceFilter,filterMerge);
            }else{
                filter = filterMerge;
            }
            return requestDataSourceEngine.getDataSource(field.dataSource.name, filter).then(function (response) {
                var fieldValues = response.dataset[field.dataSource.name.split('/')[1]];
                fieldValues.forEach(function (fieldvalue) {
                    var validate = field.dataSource.data.filter(function (data) {
                        return data[valueFieldIndex] == fieldvalue[valueFieldIndex];
                    });
                    if (!validate.length) {
                        field.dataSource.data.push(fieldvalue);
                    }
                });
                templateManager.updateTemplate();
            });
        }
    };

    this.bugPaginationOnSelectMultiplyOnModeView = function(field){
        var valueFieldIndex = field.valueField;
        var filter = [
            {
                "operator": "IN",
                "name": valueFieldIndex,
                "value": field.value() || [0]
            }
        ];
        return this.mergeDataSetPagnation(field, filter, valueFieldIndex);
    };

	self.getUrlParamAtIndex = function(name, index) {
        var param;
        if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined){
            param = $location.$$search[name];
        }
		else{
            param = getLocalVar('indicatorNames')[getLocalVar('indicatorName')][name];
        }
        if (index && Util.isArray(param) && name != 'sId' && name != 'sName') {
			param = (param[index] && param[index] !== "undefined") ? param[index] : param[param.length-1];
		}
		return param;
	};

	this.isTherePeriodOnCurrentRow = function(currentRow) {
		return currentRow.period && currentRow.period.period;
	};

	this.isGrid = function(widget) {
		if (widget.template.indexOf("grid.html") > 0) {
			return true;
		}
		else {
			return false;
		}
	};

	this.setFieldsVisibility = function(widget, fields, visibility) {
		fields.forEach(function(fieldName) {
			var field = widget.getField(fieldName);
			if (field) {
				field.isVisible = visibility;
			}
		});
	};

	this.fixBugTabbedNavigation = function (timeout) {
        timeout = timeout || 300;
        var scrollContainer = $('.zh-tab-scroller');
        setTimeout(function () {
            if (scrollContainer[0].scrollLeft > 0) {
                $("span.icon-scroll-left").show();
            } else {
                $("span.icon-scroll-left").hide();
            }

            if (scrollContainer[0].scrollWidth - scrollContainer[0].scrollLeft > scrollContainer[0].clientWidth) {
                $("span.icon-scroll-right").show();
            } else {
                $("span.icon-scroll-right").hide();
            }
        }, 300, false);
    };

	this.deleteChangedFlag = function(widget) {
		widget.dataSource.data.forEach(function(row) {
			delete row.changed;
		});
	};

	this.setFieldsReadOnly = function(widget, fields, readOnly) {
		fields.forEach(function(fieldName) {
			var field = widget.getField(fieldName);
			if (field) {
				field.readOnly = readOnly;
			}
		});
	};

	this.setChangedField = function(field) {
		field.changed = true;
	};

	this.removeKeyFromLocalStorage = function(keyName) {
		localStorage.removeItem(keyName);
	};

	this.isStringInArray = function(string, array) {
		var isStringInArray = array.indexOf(string);
		return isStringInArray === -1 ? false : true;
	};

	var parseIfJson = function(string){
		try {
			var parse=JSON.parse(string);
			return parse;
		} catch (e) {

		}
		return string;
	};

	this.showMessage = function(messageCode, type, icon, params){
		messageCode = parseIfJson(messageCode);
		if(typeof messageCode == "object"){
			params = messageCode.params ? messageCode.params : messageCode.PARAMS ? messageCode.PARAMS : params;
			messageCode = messageCode.message ? messageCode.message : messageCode.CODE? messageCode.CODE : "";
		}
		return MessageService.getMessageByCode(messageCode).then(function(message) {
			return ScreenService.showMessage(message, type, icon, params);
		}).catch(function() {
			return ScreenService.showMessage(messageCode, type, icon, params);
		});
	};

	this.confirmMessage = function(message, type, success, fail, icon, variables) {
		icon = icon ? icon : getIconFromType(type, 'question');
		return ScreenService.showCustomDialog(message, type, icon, [{
			label: ScreenService.i18n("Sim"),
			"default": true,
			code: function(e, deferred) {
				if (Util.isFunction(success)) {
					success(e);
				}
				deferred.resolve(e);
			}
		}, {
			label: ScreenService.i18n("Não"),
			code: function(e, deferred) {
				if (Util.isFunction(fail)) {
					fail(e);
				}
				deferred.reject(e);
			}
		}], variables);
	};

	var getIconFromType = function(type, defaultIcon) {
		var icon = defaultIcon;
		if (type) {
			switch (type.toUpperCase()) {
				case 'ERROR':
				case 'E':
					icon = 'error';
					break;
				case 'ALERT':
				case 'W':
					icon = 'exclamation';
					break;
				case 'INFORMATION':
					icon = 'information';
					break;
				case 'CONFIRMATION':
					icon = 'question';
					break;
				case 'SUCCESS':
				case 'M':
					icon = 'checked';
					break;
			}
		}
		return icon;
	};

	this.confirmMessageByCode = function(messageCode, messageParameters, messageConfig) {
		var type = messageConfig && messageConfig.type;
		var icon = messageConfig && messageConfig.icon;
		var success = messageConfig && messageConfig.success;
		var fail = messageConfig && messageConfig.fail;
		return MessageService.getMessageByCode(messageCode).then(function(message) {
			return self.confirmMessage(message, type, success, fail, icon, messageParameters);
		}).catch(function(e) {
			if(e.name && e.name == "Error"){
				return self.confirmMessage(messageCode, type, success, fail, icon, messageParameters);
			} else {
				return e;
			}
		});
	};

	this.notificationMessage = function(messageCode, type, time, params) {
		messageCode = parseIfJson(messageCode);
		if(typeof messageCode == "object"){
			params = messageCode.params ? messageCode.params : params;
			messageCode = messageCode.message ? messageCode.message : null;
		}
		return MessageService.getMessageByCode(messageCode).then(function(message) {
			return ScreenService.notificationMessage(message, type, time, params);
		}).catch(function() {
			return ScreenService.notificationMessage(messageCode, type, time, params);
		});
	};

	this.errorNotification = function(message, variables) {
		self.notificationMessage(message, 'error', undefined, variables);
	};

	this.successNotification = function(message, time, variables) {
		self.notificationMessage(message, 'success', time, variables);
	};

	this.validateEmailFormat = function(email) {
		var regex = new RegExp("^[_a-z0-9-]+(\\.[_a-z0-9-]+)*@[a-z0-9-]+(\\.[a-z0-9-]+)*(\\.[a-z]{2,4})$");
		return regex.exec(email);
	};

	this.isDate = function(value) {
		return (new Date(value) !== "Invalid Date") && !isNaN(new Date(value));
	};

	this.validatecolorField = function(widget, fieldName) {
		if (widget.currentRow[fieldName] && typeof widget.currentRow[fieldName] != "string") {
			widget.currentRow[fieldName] = formatColor(widget.currentRow[fieldName]);
		}
	};

	var formatColor = function(color) {
		var r = Math.round(color._r),
			g = Math.round(color._g),
			b = Math.round(color._b);
		return '#' + ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1);
	};

	this.clearChangedRows = function(widget) {
		widget.dataSource.data.forEach(function(row) {
			row.changed = false;
		});
	};

    this.transliterateString = function(text) {
        text = text.replace(/[áàãâä´`~^]/g, 'a');
        text = text.replace(/[ÁÀÃÂÄ]/g, 'A');
        text = text.replace(/[éèêẽë]/g, 'e');
        text = text.replace(/[ÉÈÊẼË]/g, 'E');
        text = text.replace(/[íìîĩï]/g, 'i');
        text = text.replace(/[ÍÌĨÎÏ]/g, 'I');
        text = text.replace(/[óòõôö]/g, 'o');
        text = text.replace(/[ÓÒÕÔÖ]/g, 'O');
        text = text.replace(/[úùûũü]/g, 'u');
        text = text.replace(/[ÚÙŨÛÜ]/g, 'U');
        text = text.replace(/[Ç]/g, 'C');
        text = text.replace(/[ç]/g, 'c');
        text = text.replace(/[^a-z0-9]/gi, '_');
        return text;
    };

	this.validateContainerAndGetFilterId = function(widgetName) {
		var container = templateManager.container;
		if (container && container.getWidget) {
			var widget = container.getWidget(widgetName);
			if (widget) {
				var currentRow = widget.currentRow;
				return currentRow ? currentRow._id : null;
			}
		}
	};

	 this.removeItemArray = function(arr, item){
		for (var i = arr.length; i--;) {
			if (arr[i] === item) {
				arr.splice(i, 1);
			}
		}
	};

	this.checkWidgetModeAndUncheckRow = function(selectedRow){
		if(selectedRow.mode == "view"){
			selectedRow.unSelectAll();
		}
	};
	this.updateCheckedRows = function (widget) {
		widget.updateCheckedRows();
	};

	this.windowResizeKanban = function(time){
        setTimeout(function(){
            window.dispatchEvent(new Event('resize'));
        }, time);
    };

	self.fixBugScrollChartsLine = function(idView,min){
		if(isNaN(min)){
			min = 3;
		}
		setTimeout(function(){
			try{
				var htmlScrollBug =  $("#masonry-item-"+idView)[0].children[0].children[2].children[0].children[1].children[4];
				if(min > 0 ){
					if(htmlScrollBug){
						htmlScrollBug.remove();
					}
					min--;
					self.fixBugScrollChartsLine(idView,min);
				}
			}catch (e) {
				console.log(e);
			}
		}, 1000);
	};

    this.bugPaginationOnSelectMultiplyOnModeList = function(field){
        var data = field.widget.dataSource.data;
        var ids = data.reduce(function(actual, row){
            return actual.concat(row[field.name]);
        }, []);
        var valueFieldIndex = field.valueField;
        var filter = [
            {
                "operator": "IN",
                "name": valueFieldIndex,
                "value": this.uniqueArray(ids)
            }
        ];
        requestDataSourceEngine.getDataSource(field.dataSource.name, filter).then(function(response){
            var fieldValues = response.dataset[field.dataSource.name.split('/').pop()];
            fieldValues.forEach(function(fieldvalue){
                var validate = field.dataSource.data.filter(function(data){
                    return data[valueFieldIndex] == fieldvalue[valueFieldIndex];
                });
                if(!validate.length){
                    field.dataSource.data.push(fieldvalue);
                }
            });
            templateManager.updateTemplate();
        });
    };

    this.setValueInSelectField = function(field,value){
        field.widget.currentRow[field.outData[field.valueField]] = value;
        if(field.hasPagination !== false){
            this.bugPaginationOnSelectOnModeView(field,value).then(function(){
                self.setOutDataSelect(field, value);
            });
        }else {
            this.setOutDataSelect(field, value);
        }
    };

    this.setOutDataSelect = function (field, value) {
        var vField = field.valueField;
        if (value) {
            var matchRows = field.dataSource.data.filter(function (item) {
                return item[vField] == value;
            });
            if (matchRows.length > 0) {
                Object.keys(field.outData).forEach(function (key) {
                    field.widget.currentRow[field.outData[key]] = matchRows[0][key];
                });
            } else {
                field.widget.currentRow[field.outData[field.valueField]] = '';
            }
        }
    };

	this.setOutData = function(field,value){
		var vField = field.valueField;
		if (value) {
			this.bugPaginationOnSelectOnModeView(field,value).then(function () {
				var matchRows = field.dataSource.data.filter(function(item) {
					return item[vField] == value;
				});
				if (matchRows.length > 0) {
					Object.keys(field.outData).forEach(function(key) {
						field.widget.currentRow[field.outData[key]] = matchRows[0][key];
					});
				}
			});
		}
	};

	this.clearValueSelect = function(field){
		field.widget.currentRow[field.name] = "";
		for (var prop in field.outData) {
			field.widget.currentRow[field.outData[prop]] = "";
		}
	};


	this.bugPaginationOnSelectOnModeView = function(field,value){
        var valueFieldIndex = field.valueField;
        if(!value){
            value = field.widget.currentRow[field.outData[field.valueField]];
        }
        var filter = [
            {
                "operator": "LIKE",
                "name": valueFieldIndex,
                "value": value || 0
            }
        ];
        return this.mergeDataSetPagnation(field, filter, valueFieldIndex);
    };

    this.uniqueArray = function(array){
        return array.reduce(function(prev, cur) {
            return (prev.indexOf(cur) < 0) ? prev.concat([cur]) : prev;
        }, []);
    };

    this.hashToSearch = function(){
        var hash = $location.$$hash.split('?')[1];
        if(hash) {
            hash.split('&').map(function (semiData) {
                var aux = semiData.split('=');
                $location.$$search[aux[0]] = aux[1];
            });
        }
    };

	this.isJson = function(str) {
		var result = null;

		try {
			result = JSON.parse(str);
		} catch (e) { }

		return result;
	};

	this.showMessageErrorOfBackEnd = function(response) {
		return MessageService.getMessageByCode('UNEXPECTED_ERROR').then(function(defaultMessage) {
			if (response && response.data && response.data.error) {
				var error = response.data.error;
				if (self.isJson(error)) {
					error = JSON.parse(error);
					var params = error.PARAMS || null;
					var messageCode = error.CODE || 'UNEXPECTED_ERROR';
					return MessageService.getMessageByCode(messageCode).then(function(message) {
						return ScreenService.showMessage(message, 'ERROR', null, params);
					}).catch(function() {
						return ScreenService.showMessage(messageCode, 'ERROR', null, params);
					});
				} else {
					var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception: ").pop();
					return ScreenService.showMessage(message, 'ERROR');
				}
			} else if (response && response.error) {
				return ScreenService.showMessage(response.error, 'ERROR');
			} else {
				return ScreenService.showMessage(defaultMessage, 'ERROR');
			}
		});
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('UtilitiesController', UtilitiesController);
});


// FILE: assets/js/util/jsons.js
function JsonsUtil() {
    this.factColorWidget = function(){
        var factColorWidget = {
            "name": "factColor",
            "label": "Fact Color",
            "isVisible": true,
            "template": "widget/grid.html",
            "hasPagination": false,
            "floatingControl": {
                "refreshAction": false,
                "customizationAction": {
                    "layout": false,
                    "filter": false,
                    "view": false
                },
                "pdfAction": false,
                "xlsAction":false,
                "csvAction":false
            },
            "searchWithPagination": false,
            "mobile": {
                "template": "widget/list.html"
            },
            "dataSource":[
                {
                    "rest": false,
                    "localStorage": false,
                    "name": ""
                }
            ],
            "events": [
                {
                    "name": "WidgetOnEnter",
                    "code": "IndicatorController.factColorOnEnter(args.owner)"
                },
                {
                    "name": "WidgetAfterMoveRow",
                    "code": "IndicatorController.factColorAfterMoveRow(args.owner)"
                }
            ],
            "actions": [
                {
                    "name": "save",
                    "label": "Save",
                    "isVisible": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "IndicatorController.factColorWidgetActionEventSave(args.owner.widget);"
                        }
                    ]
                },
                {
                    "label": "Cancel",
                    "isVisible": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "IndicatorController.cancelFactColor(args.owner.widget);"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "Dimesion",
                    "label": "Dimension",
                    "showOnList": true
                },
                {
                    "name": "color",
                    "label": "Color",
                    "descriptionOrder": 1,
                    "showOnList": true,
                    "template": "field/text-edit.html",
                    "styles": [
                        {
                            "condition": true,
                            "stylingFunction": "@IndicatorController.colorCss(row,element)@"
                        }
                    ],
                    "showOnForm": true
                }
            ],
            "widgets":[
                {
                    "name": "selectFactColor",
                    "label": "Select Fact Color",
                    "isVisible": true,
                    "template": "widget/form.html",
                    "dataSource":[
                        {
                            "rest": false,
                            "localStorage": false,
                            "name": ""
                        }
                    ],
                    "actions": [
                        {
                            "name": "save",
                            "label": "Save",
                            "isVisible": true,
                            "showAsAction": "never",
                            "events": [
                                {
                                    "name": "ActionEvent",
                                    "code": "IndicatorController.selectFactColorWidgetActionEventSave(args.owner.widget);"
                                }
                            ]
                        },
                        {
                            "label": "Cancel",
                            "isVisible": true,
                            "showAsAction": "back",
                            "events": [
                                {
                                    "name": "ActionEvent",
                                    "code": "IndicatorController.cancelSelectFactColor();"
                                }
                            ]
                        },
                        {
                            "name": "DELETE",
                            "isVisible": true,
                            "showAsAction": "always|view",
                            "icon": "trash",
                            "events": [
                                {
                                    "name": "ActionEvent",
                                    "code": "IndicatorController.selectFactColorWidgetActionEventDelete(args.owner.widget);"
                                }]
                        }
                    ],
                    "fields": [
                        {
                            "name": "name",
                            "label": "Column",
                            "isVisible": false,
                            "showOnList": true
                        },
                        {
                            "name": "label",
                            "label": "Label",
                            "isVisible": false,
                            "showOnList": true
                        },
                        {
                            "name": "color",
                            "label": "Color",
                            "descriptionOrder": 1,
                            "showOnList": true,
                            "showOnForm": true,
                            "template": "field/color.html"
                        }
                    ],
                    "widgets":[]
                }
            ]
        };
        return factColorWidget;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('JsonsUtil', JsonsUtil);
});

// FILE: assets/js/models/repositories/DataSourceRepository.js
Configuration(function(ContextRegister, RepositoryFactory) {
	var DataSourceRepository = RepositoryFactory.factory('/dataSource', 'ONLINE');
	DataSourceRepository.name = "dataSource";
	ContextRegister.register('DataSourceRepository', DataSourceRepository);
});

// FILE: assets/js/models/repositories/StructureRepository.js
Configuration(function(ContextRegister, RepositoryFactory) {	
	var StructureRepository = RepositoryFactory.factory('/StructureRepository', 'LOCAL', 1, 60000);
	ContextRegister.register('StructureRepository', StructureRepository);
});

// FILE: assets/js/models/repositories/ViewPropertyRepository.js
Configuration(function(ContextRegister, RepositoryFactory) {
	var ViewPropertyRepository = RepositoryFactory.factory('/viewProperty', 'ONLINE');
	ViewPropertyRepository.name = 'viewProperty';
	ContextRegister.register('ViewPropertyRepository', ViewPropertyRepository);
});

// FILE: assets/js/models/repositories/ViewTypeRepository.js
Configuration(function(ContextRegister, RepositoryFactory, restEngine) {

	var ViewTypeRepository = RepositoryFactory.factory('/viewType', 'ONLINE');

	ViewTypeRepository.name = "viewType";

	ViewTypeRepository.findAllWithProperties = function () {
		return restEngine.post('/viewType/findAllWithProperties', {
				"requestType": "FilterData",
				"filter": []
			}
		);
	};

	ContextRegister.register('ViewTypeRepository', ViewTypeRepository);
});

// FILE: assets/js/models/repositories/WidgetRepository.js
Configuration(function(ContextRegister, RepositoryFactory) {
	var WidgetRepository = RepositoryFactory.factory('/widget', 'ONLINE');
	WidgetRepository.name = "widget";
	ContextRegister.register('WidgetRepository', WidgetRepository);
});

// FILE: zmart_components/controllers/compare-periods.js
function ComparePeriodsController($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query,
                               CrosstabService, $filter, maskEngine, ZhUtil, $window, ApplicationContext) {
    // BEGIN: Initialize
    var self = this;
    self.periods = {};
    var moment = $window.moment;
    var allowMultiSelect = null;
    $scope.filtered = [];
    self.orderByCustomListAsc = [];
    self.orderByCustomListDesc = [];

    this.__defineGetter__('widget', function() {
        return $scope.widget || $scope._currentPopup;
    });
    // END: Initialize

    // BEGIN: Utilities
    var CROSSTAB_TEMPLATE = 'zh-crosstab#widget/crosstab.html';
    var TREE_TEMPLATE = 'tree/grid.html';
    var TREE_LIST_TEMPLATE = 'tree/list.html';
    var GANTT_TEMPLATE = 'tree/gantt.html';

    var _isTemplate = function(template) {
        return self.widget.template && self.widget.template.indexOf(template) !== -1;
    };

    this.isTree = function() {
        return _isTemplate(TREE_TEMPLATE) || _isTemplate(TREE_LIST_TEMPLATE);
    };

    this.isGantt = function() {
        return _isTemplate(GANTT_TEMPLATE);
    };

    this.isCrosstab = function() {
        return _isTemplate(CROSSTAB_TEMPLATE);
    };
    // END: Utilities

    // BEGIN: Search
    $scope.searchList = this.widget.searchList;

    this.search = function(item) {
        self.widget.searchList = $scope.searchList;
        item.__notSelectable = false;
        if ($scope.searchList) {
            var returnValue = false;
            $.each(item, function(key, value) {
                var field = $.grep(self.widget.fields, function(field) {
                    return field.name === key;
                })[0];
                if(field && field.translate){
                    value = ScreenService.i18n(value);
                }
                returnValue = (!Util.isUndefinedOrNull(field) && field.isVisible && field.showOnList && !field.skipSearch &&
                    !Util.isUndefinedOrNull(value) && value.toString().indexOfLatin($scope.searchList) !== -1);
                return !returnValue;
            });
            item.__notSelectable = !returnValue;
            return returnValue;
        } else {
            return true;
        }
    };

    function getLikeAllFilter(widget){
        var filter = [];

        if (!Util.isEmptyOrBlank($scope.searchList)) {
            var columns = widget.fields
                .filter(function(field){
                    return field.isVisible && field.aclIsVisible && field.showOnList && !field.skipSearch;
                })
                .map(function(field){
                    return field.name;
                })
                .join('|');
            filter = Query.build().where(columns).likeAll("%" + $scope.searchList + "%").where();
        }

        return filter;
    }

    var _searchWithPaginationReload = Util.buildDebounceMethod(function (widget, callback) {
        var deferred = ZHPromise.defer();
        if(Util.isArray(self.widget.fields) && self.widget.fields.length) {
            widget.reload(getLikeAllFilter(self.widget)).then(function(data) {
                if (callback) callback();
                deferred.resolve(data);
            }, function() {
                deferred.reject();
            });
        } else {
            deferred.reject();
        }
        return deferred;
    }, 1000);

    var _resizeGrid = Util.buildDebounceMethod(function() {
        self.widget.resizeGridColumns();
    }, 1000);

    if (!self.isGantt() && !self.isCrosstab()) {
        $scope.$watch('searchList', function(current, old) {
            if (self.widget.searchWithPagination && !Util.isUndefined(current)) {
                _searchWithPaginationReload(self.widget, function() {
                    updateGroupedData();
                });
            }
            if (!Util.isUndefined(current) && current != old && self.widget.resizeGridColumns) { // new grid
                _resizeGrid();
            }
        });
    }
    // END: Search

    // BEGIN: Toggle
    function deselectAll(widget) {
        var event = widget.unSelectAll();
        if (!event || !event.isDefaultPrevented()) {
            widget.dataSource.clearCheckedRows();
            if (!$scope._currentPopup) {
                $rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
            }
        }
    }

    function selectAll(widget) {
        widget.dataSource.data.filter(self.search).forEach(function(row) {
            row.__isSelected = true;
            widget.dataSource.addCheckedRows(row);
        });
        if (!$scope._currentPopup) {
            $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
        }
    }

    $rootScope.$on("ui.deselectAll", function(event, widget) {
        deselectAll(widget);
    });

    this.onLongTap = function(row) {
        if (row.__group) return;

        if (this.widget.allowSelectOnLongTap) {
            this.toggle(row);
        }
    };

    this.toggle = function(row) {
        var widget = self.widget;
        widget.selectedRow = row;

        var event;
        if (widget.beforeSelectRow && !row.__isSelected) {
            event = widget.beforeSelectRow();
        } else if (widget.beforeUnselectRow && row.__isSelected) {
            event = widget.beforeUnselectRow();
        }

        if (!event || !event.isDefaultPrevented()) {

            var index = getIndex(row, $scope);
            if (index === -1) {
                row.__isSelected = true;
                if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
            } else {
                delete row.__isSelected;
                widget.dataSource.removeCheckedRows(row);
            }

            if (self.isTree()) {
                self.handleDependentRows(row, widget);
            }

            if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
                $rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
            }

            widget.updateCheckedRows();

            if (widget.afterSelectRow && row.__isSelected) {
                widget.afterSelectRow();
            } else if (widget.afterUnselectRow && !row.__isSelected) {
                widget.afterUnselectRow();
            }

        }
    };

    $scope.widget.toggle = self.toggle;

    this.toggleAll = function() {
        if (this.widget.hasCheckedRows()) {
            this.widget.unSelectAll();
        } else {
            this.widget.selectAll();
        }
        this.widget.updateCheckedRows();
    };

    this.widget.toggle = this.toggle;
    // END: Toggle

    // BEGIN: Others functions
    $scope.openRow = function(row, widget) {
        if (row.__group) return;

        if (widget.dataSource.getCheckedRows().length || allowMultiSelect === widget || widget.allowMultiSelect) {
            self.toggle(row);
        } else {
            widget.setCurrentRow(row);
        }
        widget.dataSource.updateCheckedRows();
    };

    $scope.openRowInnerList = function(row, widget) {
        if (typeof(widget.currentRow) !== "undefined" &&
            widget.currentRow.__index === widget.dataSource.data.indexOf(row)) {
            delete widget.currentRow;
        } else {
            $scope.openRow(row, widget);
        }
    };

    function getIndex(row, scope) {
        return self.widget.dataSource.checkedRows.indexOf(row);
    }

    $scope.scrollTo = function(evt) {
        $timeout(function() {
            var item = $(evt.target).closest('li');
            var itemTop = item.position().top;
            var itemBottom = itemTop + item[0].clientHeight;
            var scroller = item.closest("[zh-scroller]");
            var top = 0;
            if (itemTop < 0 || itemBottom > scroller.height()) {
                if (itemTop < 0) {
                    top = scroller.scrollTop() + itemTop - 10;
                } else {
                    top = scroller.scrollTop() + itemBottom - scroller.height() + 10;
                }
                scroller.animate({
                    scrollTop: top
                }, 500);
            }
        }, 100, true);
    };

    $rootScope.$on("ui.allowMultiSelect", function(event, widget) {
        if ($scope._currentPopup) {
            return;
        }
        allowMultiSelect = widget;
    });

    $rootScope.$on("ui.disableMultiSelect", function() {
        if ($scope._currentPopup) {
            return;
        }
        allowMultiSelect = null;
    });
    // END: Others functions

    // BEGIN: Conditional actions control

    var hasConditionalActions = false;
    var leftActions = [];
    var rightActionsGrid = [];
    var rightActionsList = [];
    var rightActions = [];
    var defaultActionsValue = function(){
        $scope.visibleLeftActions = leftActions.length;
        $scope.visibleRightActions = rightActions.length;
        $scope.leftActions = leftActions;
        $scope.rightActions = rightActions;
    };
    var buildGridActions = function(){
        hasConditionalActions = false;
        leftActions = $scope.widget.getActionsByVisibility('grid-left');
        rightActionsGrid = $scope.widget.getActionsByVisibility('grid-right');
        rightActionsList = $scope.widget.getActionsByVisibility('list-right');
        rightActions = rightActionsGrid.concat(rightActionsList);

        $scope.widget.actions.forEach(function(item) {
            if (item.condition) {
                hasConditionalActions = true;
            }
        });

        if(!hasConditionalActions){
            defaultActionsValue();
        }
    };
    $scope.buildGridActions = buildGridActions;
    buildGridActions();

    $scope.$watch("widget.dataSource.data", function(data) {

        if(hasConditionalActions){

            $scope.visibleLeftActions = 0;
            $scope.visibleRightActions = 0;

            if ($scope.widget && $scope.widget.dataSource && $scope.widget.dataSource.data) {
                $scope.widget.dataSource.data.forEach(function(row) {
                    ApplicationContext.registerInstance('row', row || {});
                    row.__rightActions = rightActions.filter(function(action) {
                        return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
                    });

                    row.__leftActions = leftActions.filter(function(action) {
                        return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
                    });
                    $scope.visibleRightActions = Math.max($scope.visibleRightActions, row.__rightActions.length);
                    $scope.visibleLeftActions = Math.max($scope.visibleLeftActions, row.__leftActions.length);
                    ApplicationContext.removeInstance('row');
                });
            }
        }
    });


    // END: Conditional actions control

    // BEGIN: Order by

    this.changeOrder = function(column) {
        if (!!column.disableOrdering) return;

        if (self.widget.order == column.name && self.widget.reverse) {
            self.widget.order = undefined;
            self.widget.reverse = false;
        } else {
            self.widget.reverse = (self.widget.order == column.name) ? !self.widget.reverse : false;
            self.widget.order = column.name;
        }
        updateGroupedData();
    };

    var orderByColumns;
    var orderByFieldFlow = {
        "datetime.html": getDatetimeProxyToOrderBy,
        "calendar.html": getDateProxyToOrderBy,
        "select-month.html": getMonthProxyToOrderBy
    };

    function getDateTimeFormat(field) {
        var format;
        var dateFormat = field['datetime-dateFormat'] || Util.DATE_FORMAT;
        var timeFormat = field['datetime-timeFormat'] || Util.TIME_FORMAT;
        switch (field['datetime-preset']) {
            case 'date':
            case 'month':
                format = dateFormat;
                break;
            case 'time':
                format = timeFormat;
                break;
            case 'datetime':
            case 'datetime-local':
                format = dateFormat + ' ' + timeFormat;
                break;
            default:
                format = dateFormat + ' ' + timeFormat;
                break;
        }
        return format;
    }

    function getDatetimeProxyToOrderBy(columnName, field) {
        var format = getDateTimeFormat(field);
        return getProxyToOrderBy(columnName, format);
    }

    function getDateProxyToOrderBy(columnName, field) {
        return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
    }

    function getMonthProxyToOrderBy(columnName, field) {
        return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
    }

    function getProxyToOrderBy(columnName, format) {
        var proxyProperty = getProxyProperty(columnName);
        buildDateProxyColumn(proxyProperty, columnName, format);
        return proxyProperty;
    }

    function getProxyProperty(columnName) {
        return "__proxy_" + columnName;
    }

    function buildDateProxyColumn(proxyProperty, sourceColumn, format) {
        if ($scope.widget.dataSource && Util.isArray($scope.widget.dataSource.data)) {
            $scope.widget.dataSource.data.forEach(function(item){
                item[proxyProperty] = moment(item[sourceColumn], format)._d;
            });
        }
    }

    function addColumnToOrder(columnName, reverse) {
        var field = $scope.widget.fields.filter(function(field){
            return field.name === columnName;
        }).shift();

        if (field) {
            var template = field.getTemplateSimpleName();
            if (orderByFieldFlow[template]) {
                columnName = orderByFieldFlow[template](columnName, field);
            }
        }

        orderByColumns.push((reverse ? '-' : '') + columnName);
    }

    function getOrderByColumns() {
        orderByColumns = [];

        if ($scope.widget.groups) {
            $scope.widget.groups.forEach(function(group, index) {
                addColumnToOrder(group.name);
            });
        }

        if ($scope.widget.order && !Util.isFunction($scope.widget.order)) {
            Util.parseToArray($scope.widget.order).forEach(function(collum){
                addColumnToOrder(collum, $scope.widget.reverse);
            });
        }
    }
    // END: Order by

    // BEGIN: Group and summarization
    $scope.fieldOrder = function(field) {
        return (field.fixed ? '0' : '1') + ('000'+field.sequenceInList).substr(-4);
    };

    $scope.widget.groupedData = [];

    // initialize widget.groups
    updateWidgetGroups();
    $scope.$on("updateWidgetGroups", function(){
        updateWidgetGroups();
    });

    function updateWidgetGroups() {
        $scope.widget.groups = $scope.widget.fields.filter(function(item) {
            if (item.grouped) item.isVisible = false;
            return item.aclIsVisible && item.grouped;
        }).sort(function(a, b) { return a.sequenceInList - b.sequenceInList; });
    }

    var oldItemsPerPage = $scope.widget.itemsPerPage;
    var oldHasPagination = $scope.widget.hasPagination;

    $scope.$watchCollection('widget.groups', function(newValue, oldValue) {
        if (newValue != oldValue) {
            if (newValue && newValue.length >= 1) {
                $scope.widget.itemsPerPage = 0;
                $scope.widget.hasPagination = false;
            } else {
                $scope.widget.itemsPerPage = oldItemsPerPage;
                $scope.widget.hasPagination = oldHasPagination;
            }
            $scope.widget._gridResize = true; // forces the grid element to recalculate column widths
            $scope.widget.reload();
        }
    });

    /** Watch ComparePeriods Controller */
    $scope.$watchCollection('widget.dataSource.data', function() {
        updateGroupedData();
        self.periods = {};
        updatePeriods();
    });

    var groups = [];
    var summaryFields = [];
    var summary = {};

    function resetFooterVariables(group) {
        group.footer.count = 0;
        summaryFields.forEach(function(field) {
            group.footer[field.name] = {};
            group.footer[field.name].sum = 0;
            group.footer[field.name].count = 0;
        });
    }

    function addGroupHeader(group, index) {
        var groupHeaderRow = {
            __group: true,
            __groupHeader: true,
            __groupIndex: index,
            __groupLabel: ScreenService.i18n(group.label),
            __groupValue: !group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil),
            __groupOpened: true,
            __groupHeaders: []
        };
        // add header for outer groups to the added row
        for (var i = 0; i < index; i++) {
            groupHeaderRow.__groupHeaders.push(groups[i].lastGroupHeaderRow);
        }
        group.lastGroupHeaderRow = groupHeaderRow;
        $scope.widget.groupedData.push(groupHeaderRow);
    }

    function summaryData(group, row, isGroup) {
        var field, value, lastValue, _group;
        for (var i = 0; i < summaryFields.length; i++) {
            field = summaryFields[i];
            if (isGroup) {
                lastValue = ' (' + (!group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil)) + ')';
                _group = group.footer;
            } else {
                lastValue = ' (' + ScreenService.i18n('TOTAL') + ')';
                _group = group;
            }

            if (field.expression == 'COUNT') {
                row[field.name] = ScreenService.i18n('Count') + lastValue + ': ' + _group[field.name].count;
            } else if (field.expression == 'SUM') {
                value = _group[field.name].sum;
                row[field.name] = ScreenService.i18n('Sum') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
            } else if (field.expression == 'AVG') {
                value = _group[field.name].sum / _group[field.name].count;
                row[field.name] = ScreenService.i18n('Average') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
            }
        }
    }

    function addGroupFooters(groupIndex) {
        if (summaryFields.length === 0) return;

        for (var i = groups.length - 1; i >= groupIndex; i--) {
            if (groups[i].footer.count > 0) {
                var groupFooterRow = {
                    __group: true,
                    __groupFooter: true,
                    __groupIndex: i,
                    __groupHeaders: []
                };

                // add header for outer groups
                for (var g = 0; g < i; g++) {
                    groupFooterRow.__groupHeaders.push(groups[g].lastGroupHeaderRow);
                }

                summaryData(groups[i], groupFooterRow, true);

                $scope.widget.groupedData.push(groupFooterRow);
            }
        }
    }

    function addSummary() {
        if (summaryFields.length === 0 || groups.length === 0) return;

        var summaryRow = {
            __group: true,
            __groupFooter: true,
            __groupSummary: true
        };

        summaryData(summary, summaryRow);

        $scope.widget.groupedData.push(summaryRow);
    }

    function calcSummary(row) {
        groups.forEach(function(group) {
            group.footer.count += 1;
            if (summaryFields.length > 0) {
                summaryFields.forEach(function(field) {
                    if (!Util.isUndefined(row[field.name])) {
                        group.footer[field.name].count += 1;
                    }
                    if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
                        group.footer[field.name].sum += parseFloat(row[field.name]);
                    }
                });
            }
        });

        summaryFields.forEach(function(field) {
            if (Util.isUndefined(summary[field.name])) {
                summary[field.name] = {
                    count: 0,
                    sum: 0
                };
            }

            if (!Util.isUndefined(row[field.name])) {
                summary[field.name].count += 1;
            }
            if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
                summary[field.name].sum += parseFloat(row[field.name]);
            }
        });
    }

    function initSummaryFields() {
        summaryFields = $scope.widget.fields.filter(function(field) {
            return !!field.expression && !field.grouped;
        });

        // reset summary variable
        summary = {};
    }

    function initGroups() {
        groups = [];
        $scope.widget.groups.forEach(function(group, index) {
            var newGroup = {
                field: group,
                name: group.name,
                label: group.label,
                lastValue: undefined,
                footer: {}
            };
            resetFooterVariables(newGroup);

            groups.push(newGroup);
        });
    }

    function getOrderedDataByColumns(dataSet, orderByColumns) {
        return orderByColumns.length ? $filter('orderBy')(dataSet, orderByColumns) : dataSet;
    }

    function updateGroupedData() {
        if (Util.isUndefined($scope.widget.dataSource) || Util.isEmptyOrBlank($scope.widget.dataSource.data)) {
            $scope.widget.groupedData = [];
        } else {
            getOrderByColumns();

            if (Util.isUndefined($scope.widget.groups) || $scope.widget.groups.length === 0) {
                $scope.widget.groupedData = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
            } else {
                $scope.widget.groupedData = [];

                initSummaryFields();
                initGroups();

                var dataAux = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
                dataAux.forEach(function(row, index) {
                    var groupBreak = false;
                    groups.forEach(function(group, index) {
                        if (group.lastValue != row[group.name] || groupBreak) {
                            if (!groupBreak) addGroupFooters(index);

                            group.lastValue = row[group.name];
                            resetFooterVariables(group);

                            addGroupHeader(group, index);
                            groupBreak = true;
                        }
                    });

                    row.__groupHeaders = groups.map(function(group) { return group.lastGroupHeaderRow; });
                    $scope.widget.groupedData.push(row);

                    calcSummary(row);
                });

                if ($scope.widget.groupedData.length > 0) {
                    addGroupFooters(0);
                    addSummary();
                }
            }
        }

        if (self._updateVisibleRows) {
            self._updateVisibleRows();
        }
    }

    this.toggleGroup = function(row) {
        row.__groupOpened = !row.__groupOpened;

        if (self._updateVisibleRows) {
            self._updateVisibleRows();
        }
    };

    this.getClosedStatus = function(row) {
        var status = false;
        if (row.__groupHeaders && row.__groupHeaders.length > 0) {
            status = row.__groupHeaders.some(function(group) {
                return group.__groupOpened === false;
            });
        }
        return status;
    };

    /**  compare-periods */
    function updatePeriods() {
        if(!$scope.widget.groupedData.length){
            if(!Util.isEmptyOrBlank($scope.widget.parameters.drillDownFilters)){
                $scope.widget.parameters.period = $scope.widget.period;
            }
        }
        if(Array.isArray($scope.widget.parameters.period) && !$scope.widget.isReversePeriods){
            $scope.widget.isReversePeriods = true;
            $scope.widget.parameters.periodCompare = Util.clone($scope.widget.parameters.period);
            $scope.widget.parameters.periodCompare.reverse();
        }
        $scope.widget.fields.forEach(function (field) {
            $scope.widget.fieldGroups.forEach(function (group) {
                if($scope.widget.groupedData.length){
                    setOrderByCustomList();
                }
                $scope.widget.groupedData.forEach(function (row) {
                    self.getPeriodsFromField(field, row, group);
                });
                if(!$scope.widget.groupedData.length || (Array.isArray($scope.widget.parameters.period) && $scope.widget.groupedData.length === 1 && $scope.widget.groupedData[0].NMFILIAL === 'Total')){
                    $scope.widget.visbleData = 'NOT_DATA';
                } else if(Array.isArray($scope.widget.parameters.period)){
                    $scope.widget.visbleData = 'DEFAULT';
                } else {
                    $scope.widget.visbleData = 'ONE_PERIOD';
                }
            });
        });

        $scope.widget.groupedData.sort(function (firstRow, secondRow) {
            if (firstRow.__isTotalRow) {
                return 1;
            } else if(secondRow.__isTotalRow) {
                return -1;
            } else {
                return secondRow.__periodPercentRawValue - firstRow.__periodPercentRawValue;
            }
        });
    }

    $scope.orderByCustom = function() {
        if($scope.widget.zmartOrdenationComparePeriods === 'DESC'){
            return self.orderByCustomListDesc;
        } else {
            return self.orderByCustomListAsc;
        }
    };

    /**  compare-periods */
    self.doDrillDown = function(row, periodLabel) {
        var period = periodLabel.split(' - ');
        $scope.widget.parameters.period = [{start: period[0], end: period[1]}];
        DrillDownController = ApplicationContext.DrillDownController;
        DrillDownController.openNextIndicatorView($scope.widget, row);
    };

    var setOrderByCustomList = function(){
        $scope.widget.parameters.periodCompare.forEach(function(per, index){
            if (index) {
                self.orderByCustomListAsc.push('order' + index);
                self.orderByCustomListDesc.push('-order' + index);
            }
        });
    };

    /**  compare-periods */
    this.getPeriodsFromField = function (field, row, group) {
        var periods = $scope.widget.parameters.periodCompare;
        var filteredKeys;
        if (Array.isArray(periods)) {
            var aux = [];
            periods.forEach(function (p) {
                var a = $scope.widget.periodsFact + '_' + p.start + '_' + p.end;
                aux.push(a);
            });
            filteredKeys = aux;

            filteredKeys = filteredKeys.map(function (key) {
                var keySplit = key.split('_');
                var toDate = keySplit.slice(-1).shift();
                var fromDate = keySplit.slice(-2, -1).shift();
                var per = keySplit[keySplit.length - 2] + ' - ' + keySplit[keySplit.length - 1];
                toDate = moment(toDate, 'L');
                fromDate = moment(fromDate, 'L');
                return {
                    name: keySplit[0],
                    untouchedName: key,
                    from: fromDate,
                    to: toDate,
                    per: per
                };
            });

            filteredKeys.forEach(function (key) {
                setPeriodLabelAndOrder(key);
            });

            filteredKeys.sort(function (firstValue, secondValue) {
                return firstValue.order - secondValue.order;
            });

            var periods = [];

            for (var i = 0; i < filteredKeys.length; i++) {
                var key = filteredKeys[i];
                var rawPercent;

                var period = getFieldCopy(field);

                period.name = key.untouchedName;
                period.label = key.label;
                period.per = key.per;

                var lastPeriod;
                var lastPeriodValue;
                var thisPeriodValue;
                var percent;

                if (i === 0) {
                    row[period.name + "_showBadge"] = false;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                } else if (i === 1) {
                    lastPeriod = periods[0];
                    lastPeriodValue = row[lastPeriod.name] ? row[lastPeriod.name] : 0;
                    thisPeriodValue = row[period.name] ? row[period.name] : 0;
                    percent = calculatePercentage(lastPeriodValue, thisPeriodValue);
                    rawPercent = percent;

                    if($scope.widget.zmartOrdenationComparePeriods){

                        row['order1'] = parseFloat(percent);
                        if (row.__isTotalRow && $scope.widget.zmartOrdenationComparePeriods === 'ASC') {
                            row['order1'] = 1/0;
                        }
                        if (row.__isTotalRow && $scope.widget.zmartOrdenationComparePeriods === 'DESC') {
                            row['order1'] = -1/0;
                        }
                    } else {
                        if (row.__isTotalRow) {
                            row['order1'] = 1/0;
                        } else {
                            $scope.widget.dataSource.data.some(function(data, index){
                                if(data[group.name] === row[group.name]){
                                    row['order1'] = index;
                                }
                            });
                        }
                    }

                    row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
                    row[period.name + "_isLowerThanBaseValue"] = percent < 0;
                    row[period.name + "_equalsBaseValue"] = percent === 0;
                    row[period.name + "_showBadge"] = true;

                    percent = Math.abs(percent);
                    percent = percent.toFixed(1);
                    row[period.name + "_percent"] = percent;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                } else {
                    lastPeriod = periods[0];
                    lastPeriodValue = row[lastPeriod.name] ? row[lastPeriod.name] : 0;
                    thisPeriodValue = row[period.name] ? row[period.name] : 0;
                    percent = calculatePercentage(lastPeriodValue, thisPeriodValue);
                    rawPercent = percent;

                    if($scope.widget.zmartOrdenationComparePeriods){
                        row[self.orderByCustomListAsc[i-1]] = parseFloat(percent);
                    }

                    row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
                    row[period.name + "_isLowerThanBaseValue"] = percent < 0;
                    row[period.name + "_equalsBaseValue"] = percent === 0;
                    row[period.name + "_showBadge"] = true;

                    percent = Math.abs(percent);

                    percent = percent.toFixed(1);
                    row[period.name + "_percent"] = percent;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                }

                periods.push(period);
                if (row[group.name] === "Total") {
                    row.__isTotalRow = true;
                }

                if (i === filteredKeys.length - 1) {
                    row.__periodPercentRawValue = rawPercent;
                }
            }

            self.periods[row[group.name]] = periods;
            return periods;
        }
    };

    function calculatePercentage(lastPeriodValue, thisPeriodValue){
        if(Math.floor(lastPeriodValue) === 0 && Math.floor(thisPeriodValue) === 0){
            return 0;
        } else if(Math.floor(lastPeriodValue) === 0 && Math.floor(thisPeriodValue) !== 0) {
            return -100;
        } else {
            return (((lastPeriodValue * 100) / thisPeriodValue) - 100);
        }
    }

    function setPeriodLabelAndOrder(key) {
        var periods = $scope.widget.parameters.periodCompare;
        periods.some(function (p) {
            if(key.from._i === p.start && key.to._i === p.end){
                key.label = p.periodLabel ? p.periodLabel : p.start + ' - ' + p.end;
                return key;
            }
        })
    }

    function getFieldCopy(field) {
        return {
            name: field.name,
            label: field.label,
            aclIsVisible: field.aclIsVisible,
            aclReadOnly: field.aclReadOnly,
            align: field.align,
            class: field.class,
            direction: field.direction,
            events: field.events,
            function: field.function,
            idGenerated: field.idGenerated,
            isDate: field.isDate,
            isFact: field.isFact,
            isGenerated: field.isGenerated,
            isKey: field.isKey,
            isSortable: field.isSortable,
            isVisible: field.isVisible,
            mask: field.mask,
            readOnly: field.readOnly,
            readOnlyOnEdit: field.readOnlyOnEdit,
            sequenceInList: field.sequenceInList,
            showOnForm: field.showOnForm,
            showOnList: field.showOnList,
            showOnView: field.showOnView,
            showSeparatorButton: field.showSeparatorButton,
            showSignalButton: field.showSignalButton,
            translate: field.translate
        };
    }
}

// FILE: zmart_components/controllers/dashboard-card.js
/**
 * Controller
 **/
function ZmartDashboardCardController($scope, maskEngine, ZhUtil){
    
    this.getTitle = function(titleNumber){
        var result;
        if (titleNumber) {
            $scope.widget['cardTitle' + titleNumber]; 
        } else {
            result = $scope.widget.cardTitle; 
        }
        return result;
    };
    
    this.getValue = function(valueNumber){
        var widget = $scope.widget;
        var field, cardMask;
        var value;
        
        var dataSourceOnWidget = isThereDataSourceDataOnWidget(widget);
        
        field = valueNumber ? widget['cardValue' + valueNumber] : widget.cardValue;
        cardMask = valueNumber ? widget['cardMask' + valueNumber] : widget.cardMask;
        value = (dataSourceOnWidget && dataSourceOnWidget.length > 0) ? widget.dataSource.data[0][field] : widget.chartSeries[0].cardValue;
        
        widget.value = value;
        
        if (cardMask) {
            value = applyMask(cardMask, value);
        }
        
        return value;
    };
    
    function isThereDataSourceDataOnWidget(widget){
        return widget.dataSource && widget.dataSource.data;
    }
    
    this.onCardClick = function(){
        if($scope.widget.afterMoveRow){
            if($scope.widget.dataSource.data && $scope.widget.dataSource.data[0]){
                $scope.widget.currentRow = $scope.widget.dataSource.data[0];
            }
            $scope.widget.afterMoveRow();
        }
    };
    
    function applyMask(cardMask, value) {
        var mask = {
            "params": {}
        };
        mask.params.precision = 2;
        mask.type = cardMask;
        return Util.applyMask(value, mask, maskEngine, ZhUtil);
    }
}

// FILE: zmart_components/controllers/dashboard-compare-periods.js
function DashboardComparePeriodsController($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query,
                                  CrosstabService, $filter, maskEngine, ZhUtil, $window, ApplicationContext) {
    // BEGIN: Initialize
    var self = this;
    self.periods = {};
    var moment = $window.moment;
    var allowMultiSelect = null;
    $scope.filtered = [];
    self.orderByCustomListAsc = [];
    self.orderByCustomListDesc = [];

    this.__defineGetter__('widget', function() {
        return $scope.widget || $scope._currentPopup;
    });
    // END: Initialize

    // BEGIN: Utilities
    var CROSSTAB_TEMPLATE = 'zh-crosstab#widget/crosstab.html';
    var TREE_TEMPLATE = 'tree/grid.html';
    var TREE_LIST_TEMPLATE = 'tree/list.html';
    var GANTT_TEMPLATE = 'tree/gantt.html';

    var _isTemplate = function(template) {
        return self.widget.template && self.widget.template.indexOf(template) !== -1;
    };

    this.isTree = function() {
        return _isTemplate(TREE_TEMPLATE) || _isTemplate(TREE_LIST_TEMPLATE);
    };

    this.isGantt = function() {
        return _isTemplate(GANTT_TEMPLATE);
    };

    this.isCrosstab = function() {
        return _isTemplate(CROSSTAB_TEMPLATE);
    };
    // END: Utilities

    // BEGIN: Search
    $scope.searchList = this.widget.searchList;

    this.search = function(item) {
        self.widget.searchList = $scope.searchList;
        item.__notSelectable = false;
        if ($scope.searchList) {
            var returnValue = false;
            $.each(item, function(key, value) {
                var field = $.grep(self.widget.fields, function(field) {
                    return field.name === key;
                })[0];
                if(field && field.translate){
                    value = ScreenService.i18n(value);
                }
                returnValue = (!Util.isUndefinedOrNull(field) && field.isVisible && field.showOnList && !field.skipSearch &&
                    !Util.isUndefinedOrNull(value) && value.toString().indexOfLatin($scope.searchList) !== -1);
                return !returnValue;
            });
            item.__notSelectable = !returnValue;
            return returnValue;
        } else {
            return true;
        }
    };

    function getLikeAllFilter(widget){
        var filter = [];

        if (!Util.isEmptyOrBlank($scope.searchList)) {
            var columns = widget.fields
                .filter(function(field){
                    return field.isVisible && field.aclIsVisible && field.showOnList && !field.skipSearch;
                })
                .map(function(field){
                    return field.name;
                })
                .join('|');
            filter = Query.build().where(columns).likeAll("%" + $scope.searchList + "%").where();
        }

        return filter;
    }

    var _searchWithPaginationReload = Util.buildDebounceMethod(function(widget, callback) {
        var deferred = ZHPromise.defer();
        if(Util.isArray(self.widget.fields) && self.widget.fields.length) {
            widget.reload(getLikeAllFilter(self.widget)).then(function(data) {
                if (callback) callback();
                deferred.resolve(data);
            }, function() {
                deferred.reject();
            });
        } else {
            deferred.reject();
        }
        return deferred;
    }, 1000);

    var _resizeGrid = Util.buildDebounceMethod(function() {
        self.widget.resizeGridColumns();
    }, 1000);

    if (!self.isGantt() && !self.isCrosstab()) {
        $scope.$watch('searchList', function(current, old) {
            if (self.widget.searchWithPagination && !Util.isUndefined(current)) {
                _searchWithPaginationReload(self.widget, function() {
                    updateGroupedData();
                });
            }
            if (!Util.isUndefined(current) && current != old && self.widget.resizeGridColumns) { // new grid
                _resizeGrid();
            }
        });
    }
    // END: Search

    // BEGIN: Toggle
    function deselectAll(widget) {
        var event = widget.unSelectAll();
        if (!event || !event.isDefaultPrevented()) {
            widget.dataSource.clearCheckedRows();
            if (!$scope._currentPopup) {
                $rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
            }
        }
    }

    function selectAll(widget) {
        widget.dataSource.data.filter(self.search).forEach(function(row) {
            row.__isSelected = true;
            widget.dataSource.addCheckedRows(row);
        });
        if (!$scope._currentPopup) {
            $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
        }
    }

    $rootScope.$on("ui.deselectAll", function(event, widget) {
        deselectAll(widget);
    });

    this.onLongTap = function(row) {
        if (row.__group) return;

        if (this.widget.allowSelectOnLongTap) {
            this.toggle(row);
        }
    };

    this.toggle = function(row) {
        var widget = self.widget;
        widget.selectedRow = row;

        var event;
        if (widget.beforeSelectRow && !row.__isSelected) {
            event = widget.beforeSelectRow();
        } else if (widget.beforeUnselectRow && row.__isSelected) {
            event = widget.beforeUnselectRow();
        }

        if (!event || !event.isDefaultPrevented()) {

            var index = getIndex(row, $scope);
            if (index === -1) {
                row.__isSelected = true;
                if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
            } else {
                delete row.__isSelected;
                widget.dataSource.removeCheckedRows(row);
            }

            if (self.isTree()) {
                self.handleDependentRows(row, widget);
            }

            if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
                $rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
            }

            widget.updateCheckedRows();

            if (widget.afterSelectRow && row.__isSelected) {
                widget.afterSelectRow();
            } else if (widget.afterUnselectRow && !row.__isSelected) {
                widget.afterUnselectRow();
            }

        }
    };

    $scope.widget.toggle = self.toggle;

    this.toggleAll = function() {
        if (this.widget.hasCheckedRows()) {
            this.widget.unSelectAll();
        } else {
            this.widget.selectAll();
        }
        this.widget.updateCheckedRows();
    };

    this.widget.toggle = this.toggle;
    // END: Toggle

    // BEGIN: Others functions
    $scope.openRow = function(row, widget) {
        if (row.__group) return;

        if (widget.dataSource.getCheckedRows().length || allowMultiSelect === widget || widget.allowMultiSelect) {
            self.toggle(row);
        } else {
            widget.setCurrentRow(row);
        }
        widget.dataSource.updateCheckedRows();
    };

    $scope.openRowInnerList = function(row, widget) {
        if (typeof(widget.currentRow) !== "undefined" &&
            widget.currentRow.__index === widget.dataSource.data.indexOf(row)) {
            delete widget.currentRow;
        } else {
            $scope.openRow(row, widget);
        }
    };

    function getIndex(row, scope) {
        return self.widget.dataSource.checkedRows.indexOf(row);
    }

    $scope.scrollTo = function(evt) {
        $timeout(function() {
            var item = $(evt.target).closest('li');
            var itemTop = item.position().top;
            var itemBottom = itemTop + item[0].clientHeight;
            var scroller = item.closest("[zh-scroller]");
            var top = 0;
            if (itemTop < 0 || itemBottom > scroller.height()) {
                if (itemTop < 0) {
                    top = scroller.scrollTop() + itemTop - 10;
                } else {
                    top = scroller.scrollTop() + itemBottom - scroller.height() + 10;
                }
                scroller.animate({
                    scrollTop: top
                }, 500);
            }
        }, 100, true);
    };

    $rootScope.$on("ui.allowMultiSelect", function(event, widget) {
        if ($scope._currentPopup) {
            return;
        }
        allowMultiSelect = widget;
    });

    $rootScope.$on("ui.disableMultiSelect", function() {
        if ($scope._currentPopup) {
            return;
        }
        allowMultiSelect = null;
    });
    // END: Others functions

    // BEGIN: Conditional actions control

    var hasConditionalActions = false;
    var leftActions = [];
    var rightActionsGrid = [];
    var rightActionsList = [];
    var rightActions = [];
    var defaultActionsValue = function(){
        $scope.visibleLeftActions = leftActions.length;
        $scope.visibleRightActions = rightActions.length;
        $scope.leftActions = leftActions;
        $scope.rightActions = rightActions;
    };
    var buildGridActions = function(){
        hasConditionalActions = false;
        leftActions = $scope.widget.getActionsByVisibility('grid-left');
        rightActionsGrid = $scope.widget.getActionsByVisibility('grid-right');
        rightActionsList = $scope.widget.getActionsByVisibility('list-right');
        rightActions = rightActionsGrid.concat(rightActionsList);

        $scope.widget.actions.forEach(function(item) {
            if (item.condition) {
                hasConditionalActions = true;
            }
        });

        if(!hasConditionalActions){
            defaultActionsValue();
        }
    };
    $scope.buildGridActions = buildGridActions;
    buildGridActions();
    //wands
    $scope.$watch("widget.dataSource.data", function(data) {

        if(hasConditionalActions){

            $scope.visibleLeftActions = 0;
            $scope.visibleRightActions = 0;

            if ($scope.widget && $scope.widget.dataSource && $scope.widget.dataSource.data) {
                $scope.widget.dataSource.data.forEach(function(row) {
                    ApplicationContext.registerInstance('row', row || {});
                    row.__rightActions = rightActions.filter(function(action) {
                        return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
                    });

                    row.__leftActions = leftActions.filter(function(action) {
                        return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
                    });
                    $scope.visibleRightActions = Math.max($scope.visibleRightActions, row.__rightActions.length);
                    $scope.visibleLeftActions = Math.max($scope.visibleLeftActions, row.__leftActions.length);
                    ApplicationContext.removeInstance('row');
                });
            }
        }
    });


    // END: Conditional actions control

    // BEGIN: Order by
    //wands
    this.changeOrder = function(column) {
        if (!!column.disableOrdering) return;

        if (self.widget.order == column.name && self.widget.reverse) {
            self.widget.order = undefined;
            self.widget.reverse = false;
        } else {
            self.widget.reverse = (self.widget.order == column.name) ? !self.widget.reverse : false;
            self.widget.order = column.name;
        }
        updateGroupedData();
    };

    var orderByColumns;
    var orderByFieldFlow = {
        "datetime.html": getDatetimeProxyToOrderBy,
        "calendar.html": getDateProxyToOrderBy,
        "select-month.html": getMonthProxyToOrderBy
    };

    function getDateTimeFormat(field) {
        var format;
        var dateFormat = field['datetime-dateFormat'] || Util.DATE_FORMAT;
        var timeFormat = field['datetime-timeFormat'] || Util.TIME_FORMAT;
        switch (field['datetime-preset']) {
            case 'date':
            case 'month':
                format = dateFormat;
                break;
            case 'time':
                format = timeFormat;
                break;
            case 'datetime':
            case 'datetime-local':
                format = dateFormat + ' ' + timeFormat;
                break;
            default:
                format = dateFormat + ' ' + timeFormat;
                break;
        }
        return format;
    }

    function getDatetimeProxyToOrderBy(columnName, field) {
        var format = getDateTimeFormat(field);
        return getProxyToOrderBy(columnName, format);
    }

    function getDateProxyToOrderBy(columnName, field) {
        return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
    }

    function getMonthProxyToOrderBy(columnName, field) {
        return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
    }

    function getProxyToOrderBy(columnName, format) {
        var proxyProperty = getProxyProperty(columnName);
        buildDateProxyColumn(proxyProperty, columnName, format);
        return proxyProperty;
    }

    function getProxyProperty(columnName) {
        return "__proxy_" + columnName;
    }

    function buildDateProxyColumn(proxyProperty, sourceColumn, format) {
        if ($scope.widget.dataSource && Util.isArray($scope.widget.dataSource.data)) {
            $scope.widget.dataSource.data.forEach(function(item){
                item[proxyProperty] = moment(item[sourceColumn], format)._d;
            });
        }
    }

    function addColumnToOrder(columnName, reverse) {
        var field = $scope.widget.fields.filter(function(field){
            return field.name === columnName;
        }).shift();

        if (field) {
            var template = field.getTemplateSimpleName();
            if (orderByFieldFlow[template]) {
                columnName = orderByFieldFlow[template](columnName, field);
            }
        }

        orderByColumns.push((reverse ? '-' : '') + columnName);
    }

    function getOrderByColumns() {
        orderByColumns = [];

        if ($scope.widget.groups) {
            $scope.widget.groups.forEach(function(group, index) {
                addColumnToOrder(group.name);
            });
        }

        if ($scope.widget.order && !Util.isFunction($scope.widget.order)) {
            Util.parseToArray($scope.widget.order).forEach(function(collum){
                addColumnToOrder(collum, $scope.widget.reverse);
            });
        }
    }
    // END: Order by

    // BEGIN: Group and summarization
    $scope.fieldOrder = function(field) {
        return (field.fixed ? '0' : '1') + ('000'+field.sequenceInList).substr(-4);
    };

    $scope.widget.groupedData = [];

    // initialize widget.groups
    updateWidgetGroups();
    $scope.$on("updateWidgetGroups", function(){
        updateWidgetGroups();
    });

    function updateWidgetGroups() {
        $scope.widget.groups = $scope.widget.fields.filter(function(item) {
            if (item.grouped) item.isVisible = false;
            return item.aclIsVisible && item.grouped;
        }).sort(function(a, b) { return a.sequenceInList - b.sequenceInList; });
    }

    var oldItemsPerPage = $scope.widget.itemsPerPage;
    var oldHasPagination = $scope.widget.hasPagination;

    $scope.$watchCollection('widget.groups', function(newValue, oldValue) {
        if (newValue != oldValue) {
            if (newValue && newValue.length >= 1) {
                $scope.widget.itemsPerPage = 0;
                $scope.widget.hasPagination = false;
            } else {
                $scope.widget.itemsPerPage = oldItemsPerPage;
                $scope.widget.hasPagination = oldHasPagination;
            }
            $scope.widget._gridResize = true; // forces the grid element to recalculate column widths
            $scope.widget.reload();
        }
    });

    /** Watch Dashboard ComparePeriods Controller */
    $scope.$watchCollection('widget.dataSource.data', function() {
        updateGroupedData();
        self.periods = {};
        updatePeriods();
    });

    var groups = [];
    var summaryFields = [];
    var summary = {};

    function resetFooterVariables(group) {
        group.footer.count = 0;
        summaryFields.forEach(function(field) {
            group.footer[field.name] = {};
            group.footer[field.name].sum = 0;
            group.footer[field.name].count = 0;
        });
    }

    function addGroupHeader(group, index) {
        var groupHeaderRow = {
            __group: true,
            __groupHeader: true,
            __groupIndex: index,
            __groupLabel: ScreenService.i18n(group.label),
            __groupValue: !group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil),
            __groupOpened: true,
            __groupHeaders: []
        };
        // add header for outer groups to the added row
        for (var i = 0; i < index; i++) {
            groupHeaderRow.__groupHeaders.push(groups[i].lastGroupHeaderRow);
        }
        group.lastGroupHeaderRow = groupHeaderRow;
        $scope.widget.groupedData.push(groupHeaderRow);
    }

    function summaryData(group, row, isGroup) {
        var field, value, lastValue, _group;
        for (var i = 0; i < summaryFields.length; i++) {
            field = summaryFields[i];
            if (isGroup) {
                lastValue = ' (' + (!group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil)) + ')';
                _group = group.footer;
            } else {
                lastValue = ' (' + ScreenService.i18n('TOTAL') + ')';
                _group = group;
            }

            if (field.expression == 'COUNT') {
                row[field.name] = ScreenService.i18n('Count') + lastValue + ': ' + _group[field.name].count;
            } else if (field.expression == 'SUM') {
                value = _group[field.name].sum;
                row[field.name] = ScreenService.i18n('Sum') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
            } else if (field.expression == 'AVG') {
                value = _group[field.name].sum / _group[field.name].count;
                row[field.name] = ScreenService.i18n('Average') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
            }
        }
    }

    function addGroupFooters(groupIndex) {
        if (summaryFields.length === 0) return;

        for (var i = groups.length - 1; i >= groupIndex; i--) {
            if (groups[i].footer.count > 0) {
                var groupFooterRow = {
                    __group: true,
                    __groupFooter: true,
                    __groupIndex: i,
                    __groupHeaders: []
                };

                // add header for outer groups
                for (var g = 0; g < i; g++) {
                    groupFooterRow.__groupHeaders.push(groups[g].lastGroupHeaderRow);
                }

                summaryData(groups[i], groupFooterRow, true);

                $scope.widget.groupedData.push(groupFooterRow);
            }
        }
    }

    function addSummary() {
        if (summaryFields.length === 0 || groups.length === 0) return;

        var summaryRow = {
            __group: true,
            __groupFooter: true,
            __groupSummary: true
        };

        summaryData(summary, summaryRow);

        $scope.widget.groupedData.push(summaryRow);
    }

    function calcSummary(row) {
        groups.forEach(function(group) {
            group.footer.count += 1;
            if (summaryFields.length > 0) {
                summaryFields.forEach(function(field) {
                    if (!Util.isUndefined(row[field.name])) {
                        group.footer[field.name].count += 1;
                    }
                    if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
                        group.footer[field.name].sum += parseFloat(row[field.name]);
                    }
                });
            }
        });

        summaryFields.forEach(function(field) {
            if (Util.isUndefined(summary[field.name])) {
                summary[field.name] = {
                    count: 0,
                    sum: 0
                };
            }

            if (!Util.isUndefined(row[field.name])) {
                summary[field.name].count += 1;
            }
            if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
                summary[field.name].sum += parseFloat(row[field.name]);
            }
        });
    }

    function initSummaryFields() {
        summaryFields = $scope.widget.fields.filter(function(field) {
            return !!field.expression && !field.grouped;
        });

        // reset summary variable
        summary = {};
    }

    function initGroups() {
        groups = [];
        $scope.widget.groups.forEach(function(group, index) {
            var newGroup = {
                field: group,
                name: group.name,
                label: group.label,
                lastValue: undefined,
                footer: {}
            };
            resetFooterVariables(newGroup);

            groups.push(newGroup);
        });
    }

    function getOrderedDataByColumns(dataSet, orderByColumns) {
        return orderByColumns.length ? $filter('orderBy')(dataSet, orderByColumns) : dataSet;
    }

    function updateGroupedData() {
        if (Util.isUndefined($scope.widget.dataSource) || Util.isEmptyOrBlank($scope.widget.dataSource.data)) {
            $scope.widget.groupedData = [];
        } else {
            getOrderByColumns();

            if (Util.isUndefined($scope.widget.groups) || $scope.widget.groups.length === 0) {
                $scope.widget.groupedData = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
            } else {
                $scope.widget.groupedData = [];

                initSummaryFields();
                initGroups();

                var dataAux = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
                dataAux.forEach(function(row, index) {
                    var groupBreak = false;
                    groups.forEach(function(group, index) {
                        if (group.lastValue != row[group.name] || groupBreak) {
                            if (!groupBreak) addGroupFooters(index);

                            group.lastValue = row[group.name];
                            resetFooterVariables(group);

                            addGroupHeader(group, index);
                            groupBreak = true;
                        }
                    });

                    row.__groupHeaders = groups.map(function(group) { return group.lastGroupHeaderRow; });
                    $scope.widget.groupedData.push(row);

                    calcSummary(row);
                });

                if ($scope.widget.groupedData.length > 0) {
                    addGroupFooters(0);
                    addSummary();
                }
            }
        }

        if (self._updateVisibleRows) {
            self._updateVisibleRows();
        }
    }

    this.toggleGroup = function(row) {
        row.__groupOpened = !row.__groupOpened;

        if (self._updateVisibleRows) {
            self._updateVisibleRows();
        }
    };

    this.getClosedStatus = function(row) {
        var status = false;
        if (row.__groupHeaders && row.__groupHeaders.length > 0) {
            status = row.__groupHeaders.some(function(group) {
                return group.__groupOpened === false;
            });
        }
        return status;
    };

    /** Dashboard-compare-periods */
    function updatePeriods() {
        if(!$scope.widget.groupedData.length){
            if(!Util.isEmptyOrBlank($scope.widget.parameters.drillDownFilters)){
                $scope.widget.parameters.period = $scope.widget.period;
            }
        }
        if(Array.isArray($scope.widget.parameters.period) && !$scope.widget.isReversePeriods){
            $scope.widget.isReversePeriods = true;
            $scope.widget.parameters.periodCompare = Util.clone($scope.widget.parameters.period);
            $scope.widget.parameters.periodCompare.reverse();
        }
        $scope.widget.fields.forEach(function (field) {
            $scope.widget.fieldGroups.forEach(function (group) {
                if($scope.widget.groupedData.length){
                    setOrderByCustomList();
                }
                $scope.widget.groupedData.forEach(function (row) {
                    self.getPeriodsFromField(field, row, group);
                });
                if(!$scope.widget.groupedData.length || (Array.isArray($scope.widget.parameters.period) && $scope.widget.groupedData.length === 1 && $scope.widget.groupedData[0].NMFILIAL === 'Total')){
                    $scope.widget.visbleData = 'NOT_DATA';
                } else if(Array.isArray($scope.widget.parameters.period)){
                    $scope.widget.visbleData = 'DEFAULT';
                } else {
                    $scope.widget.visbleData = 'ONE_PERIOD';
                }
            });
        });

        $scope.widget.groupedData.sort(function (firstRow, secondRow) {
            if (firstRow.__isTotalRow) {
                return 1;
            } else if(secondRow.__isTotalRow) {
                return -1;
            } else if ($scope.widget.zmartOrdenationComparePeriods === 'ASC') {
                return firstRow.ordenationValue - secondRow.ordenationValue;
            } else {
                return secondRow.ordenationValue - firstRow.ordenationValue;
            }
        });

        if($scope.widget.parameters.limitRows && $scope.widget.groupedData.length) {
            $scope.widget.groupedData = $scope.widget.groupedData.slice(0, $scope.widget.parameters.limitRows);
        }
    }

    $scope.orderByCustom = function() {
        if($scope.widget.zmartOrdenationComparePeriods === 'DESC'){
            return self.orderByCustomListDesc;
        } else {
            return self.orderByCustomListAsc;
        }
    };

    /** Dashboard-compare-periods */
    self.doDrillDown = function(row, periodLabel) {
        var period = periodLabel.split(' - ');
        if($scope.widget.actionOnClick === 'filter') {
            row.isDashboardComparePeriod = true;
            row.periodClicked = {start: period[0], end: period[1]};
            ViewController = ApplicationContext.ViewController;
            ViewController.filterDashboardByWidget($scope.widget, row);
        } else {
            $scope.widget.parameters.period = [{start: period[0], end: period[1]}];
            DrillDownController = ApplicationContext.DrillDownController;
            DrillDownController.openNextIndicatorView($scope.widget, row);
        }
    };

    var setOrderByCustomList = function(){
        $scope.widget.parameters.periodCompare.forEach(function(per, index){
            if (index) {
                self.orderByCustomListAsc.push('order' + index);
                self.orderByCustomListDesc.push('-order' + index);
            }
        });
    };

    this.getPeriodsFromField = function (field, row, group) {
        var periods = $scope.widget.parameters.periodCompare;
        var filteredKeys;
        if (Array.isArray(periods)) {

            var aux = [];
            periods.forEach(function (p) {
                var a = $scope.widget.periodsFact + '_' + p.start + '_' + p.end;
                aux.push(a);
            });
            filteredKeys = aux;

            filteredKeys = filteredKeys.map(function (key) {
                var keySplit = key.split('_');
                var toDate = keySplit.slice(-1).shift();
                var fromDate = keySplit.slice(-2, -1).shift();
                var per = keySplit[keySplit.length - 2] + ' - ' + keySplit[keySplit.length - 1];
                toDate = moment(toDate, 'L');
                fromDate = moment(fromDate, 'L');
                return {
                    name: keySplit[0],
                    untouchedName: key,
                    from: fromDate,
                    to: toDate,
                    per: per
                };
            });

            filteredKeys.forEach(function (key) {
                setPeriodLabelAndOrder(key);
            });

            filteredKeys.sort(function (firstValue, secondValue) {
                return firstValue.order - secondValue.order;
            });

            var periods = [];

            for (var i = 0; i < filteredKeys.length; i++) {
                var key = filteredKeys[i];
                var rawPercent;

                var period = getFieldCopy(field);

                period.name = key.untouchedName;
                period.label = key.label;
                period.per = key.per;

                var lastPeriod;
                var lastPeriodValue;
                var thisPeriodValue;
                var percent;

                if (i === 0) {
                    row[period.name + "_showBadge"] = false;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                } else if (i === 1) {
                    lastPeriod = periods[0];
                    lastPeriodValue = row[lastPeriod.name] ? row[lastPeriod.name] : 0;
                    thisPeriodValue = row[period.name] ? row[period.name] : 0;
                    percent = calculatePercentage(lastPeriodValue, thisPeriodValue);
                    rawPercent = percent;

                    if($scope.widget.zmartOrdenationComparePeriods){

                        row['order1'] = parseFloat(percent);
                        if (row.__isTotalRow && $scope.widget.zmartOrdenationComparePeriods === 'ASC') {
                            row['order1'] = 1/0;
                        }
                        if (row.__isTotalRow && $scope.widget.zmartOrdenationComparePeriods === 'DESC') {
                            row['order1'] = -1/0;
                        }
                    } else {
                        if (row.__isTotalRow) {
                            row['order1'] = 1/0;
                        } else {
                            $scope.widget.dataSource.data.some(function(data, index){
                                if(data[group.name] === row[group.name]){
                                    row['order1'] = index;
                                }
                            });
                        }
                    }


                    row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
                    row[period.name + "_isLowerThanBaseValue"] = percent < 0;
                    row[period.name + "_equalsBaseValue"] = percent === 0;
                    row[period.name + "_showBadge"] = true;

                    percent = Math.abs(percent);
                    percent = percent.toFixed(1);
                    row[period.name + "_percent"] = percent;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                } else {
                    lastPeriod = periods[0];
                    lastPeriodValue = row[lastPeriod.name] ? row[lastPeriod.name] : 0;
                    thisPeriodValue = row[period.name] ? row[period.name] : 0;
                    percent = calculatePercentage(lastPeriodValue, thisPeriodValue);
                    rawPercent = percent;

                    if($scope.widget.zmartOrdenationComparePeriods){
                        row[self.orderByCustomListAsc[i-1]] = parseFloat(percent);
                    }

                    row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
                    row[period.name + "_isLowerThanBaseValue"] = percent < 0;
                    row[period.name + "_equalsBaseValue"] = percent === 0;
                    row[period.name + "_showBadge"] = true;

                    percent = Math.abs(percent);

                    percent = percent.toFixed(1);
                    row[period.name + "_percent"] = percent;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                }

                periods.push(period);
                if (row[group.name] === "Total") {
                    row.__isTotalRow = true;
                }

                if (i === filteredKeys.length - 1) {
                    row.__periodPercentRawValue = rawPercent;
                }
            }

        }

        self.periods[row[group.name]] = periods;
        return periods;
    };

    function calculatePercentage(lastPeriodValue, thisPeriodValue){
        if(Math.floor(lastPeriodValue) === 0 && Math.floor(thisPeriodValue) === 0){
            return 0;
        } else if(Math.floor(lastPeriodValue) === 0 && Math.floor(thisPeriodValue) !== 0) {
            return -100;
        } else {
            return (((lastPeriodValue * 100) / thisPeriodValue) - 100);
        }
    }

    function setPeriodLabelAndOrder(key) {
        var periods = $scope.widget.parameters.periodCompare;
        periods.some(function (p) {
            if(key.from._i === p.start && key.to._i === p.end){
                key.label = p.periodLabel ? p.periodLabel : p.start + ' - ' + p.end;
                return key;
            }
        })
    }

    function getFieldCopy(field) {
        return {
            name: field.name,
            label: field.label,
            aclIsVisible: field.aclIsVisible,
            aclReadOnly: field.aclReadOnly,
            align: field.align,
            class: field.class,
            direction: field.direction,
            events: field.events,
            function: field.function,
            idGenerated: field.idGenerated,
            isDate: field.isDate,
            isFact: field.isFact,
            isGenerated: field.isGenerated,
            isKey: field.isKey,
            isSortable: field.isSortable,
            isVisible: field.isVisible,
            mask: field.mask,
            readOnly: field.readOnly,
            readOnlyOnEdit: field.readOnlyOnEdit,
            sequenceInList: field.sequenceInList,
            showOnForm: field.showOnForm,
            showOnList: field.showOnList,
            showOnView: field.showOnView,
            showSeparatorButton: field.showSeparatorButton,
            showSignalButton: field.showSignalButton,
            translate: field.translate
        };
    }
}

// FILE: zmart_components/controllers/zmart-accordion.js
/**
 * Controller
 **/
function ZmartAccordionController($scope, maskEngine, ZhUtil, ScreenService, ApplicationContext, templateManager){
    var self = this;
    var widget = $scope.widget;

    self.hasIndicators = function(item) {
        return item.dataSource && item.dataSource.data && item.dataSource.data.length > 0;
    };
    
    DrillDownController = ApplicationContext.DrillDownController;
    self.itemClick = function(widget) {
        DrillDownController.openNextIndicatorView(widget,{});
    };
    
    GenericDashboardFactory = ApplicationContext.GenericDashboardFactory;
    self.factoryDashboardAccordionActions = function(item, container){
        GenericDashboardFactory.factoryDashboardAccordionActions(item, container);
    };

    GenericDashboardService = ApplicationContext.GenericDashboardService;
    self.addDataAccordionWidget = function(item){
        GenericDashboardService.addDataAccordionWidget(item);
    };

    self.selectedItem = undefined;
    self.toggleItem = function(item) {
        var container = templateManager.container;
		var currentWidget = container.getActiveWidget();
		if(currentWidget.label !== "Dashboard" && item.label !== currentWidget.label){
            currentWidget.opened = false;
            currentWidget.calcElementHeight();
		}
		
        item.opened = !item.opened;
        item.activate();
        if (item.opened && widget.exclusive) {
            if (self.selectedItem && self.selectedItem != item) {
                self.selectedItem.opened = false;
                self.selectedItem.calcElementHeight();
            }
            self.selectedItem = item;
        }
        self.factoryDashboardAccordionActions(item, container);
        self.addDataAccordionWidget(item);
        item.calcElementHeight();
    };
    
    self.applyMask = function(cardMask, value) {
        var mask = {};
        mask.type = cardMask;
        return Util.applyMask(value, mask, maskEngine, ZhUtil);
    };

    self.getGradientCSS = function(gradient) {
        var css = "";
        if (gradient) {
            gradient = Util.parseToArray(gradient);
            if (gradient.length === 1)     {
                css = "background: "+gradient[0]+";";
            } else {
                css = "background: "+gradient[0]+"; background: -webkit-linear-gradient(left, "+gradient[0]+" 0%,"+gradient[1]+" 100%); background: linear-gradient(to right, "+gradient[0]+" 0%,"+gradient[1]+" 100%);";
            }
        }
        return css;
    };
}

// FILE: zmart_components/controllers/zmart-list.js
/**
 * Controller
 **/
function ZmartListController($scope, maskEngine, ZhUtil){
    var self = this;
    var widget = $scope.widget;

    self.applyMask = function(cardMask, value) {
        var mask = {};
        mask.type = cardMask;
        return Util.applyMask(value, mask, maskEngine, ZhUtil);
    };
}

// FILE: zmart_components/directives/dashboard-card.js
(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zmartDashboardCard', zmartDashboardCard);

    function zmartDashboardCard() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            function setColorByValue() {
                var color;
                var dataSourceRow = widget.dataSource.data[0];
                var conditions = widget.cardConditions || [];
                var colors = widget.cardColors || [];
                var value = widget.value;
                if(value !== undefined){
                    if(conditions && conditions[0]){
                        for(var i=0;i<conditions.length;i++){
                            var condition=parseFloat(conditions[i]);
                            if(isNaN(condition) || condition === null){
                                condition = dataSourceRow[conditions[i]] || null;
                            }
                            if(condition !== null && value >= condition){
                                color = colors[i%colors.length];
                            }
                        }
                    }else{
                        color = colors[0];
                    }
                }
                widget.color = color;
                element.css('background-color', color);
            }

            setColorByValue();

            scope.$watch(
                function() {
                    return widget.value;
                },
                function(newColor) {
                    setColorByValue();
                    applyWidgetColorOnElement();
                }
            );

            function fontShouldBeWhite(color) {
                var c = color.substring(1); // strip #
                var rgb = parseInt(c, 16); // convert rrggbb to decimal
                var r = (rgb >> 16) & 0xff; // extract red
                var g = (rgb >> 8) & 0xff; // extract green
                var b = (rgb >> 0) & 0xff; // extract blue

                var luma = 0.2126 * r + 0.7152 * g + 0.0722 * b;
                return luma < 155;
            }

            function applyWidgetColorOnElement() {
                if (widget.color) {
                    var color = widget.color || '#e2a53b';
                    if (fontShouldBeWhite(widget.color)) {
                        element.css('color', '#fff');
                    } else {
                        element.css('color', '#000');
                    }
                    element.css('background-color', color);
                }
            }
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCardGrid', zmartDashboardCardGrid);

    function zmartDashboardCardGrid() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var gridHeight = element.height();
                var gridRows = element.find('.card-grid-row, .card-grid-row-group');
                gridRows.css({
                    height: gridHeight/gridRows.length+'px',
                    lineHeight: gridHeight/gridRows.length+'px'
                });
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCard3', zmartDashboardCard3);

    function zmartDashboardCard3() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var gridHeight = element.height();
                element.find("li > span").css({
                    height: gridHeight/2+'px',
                    lineHeight: gridHeight/2+'px'
                });
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCard4Chart', zmartDashboardCard4Chart);

    function zmartDashboardCard4Chart() {

        var linkFunction = function(theScope, theElement, attrs) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var widgetItem = widget.widgets[parseInt(attrs.zmartDashboardCard4Chart)];
                var options = {
                    title: false,
                    subtitle: false,
                    fontName: "open_sanslight, Helvetica, Arial, sans-serif",
                    series: [{
                        type: "progress",
                        colors: widgetItem.progressParameters.color,
                        value: widgetItem.progressParameters.value,
                        maxValue: widgetItem.progressParameters.maxValue,
                        showValueLabels: false,
                        progressWidth: 0.2
                    }]
                };

                var chart = zhChart(options);
                chart(element[0]);
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }

    ZeedhiDirectives.directive('zmartDashboardCard4Grid', zmartDashboardCard4Grid);

    function zmartDashboardCard4Grid() {

        var linkFunction = function(theScope, theElement) {
            var scope = theScope;
            var element = theElement;
            var widget = theScope.widget;

            scope.$on('zh-masonry-grid-finished', function(e) {
                var gridHeight = element.height();
                var gridRows = element.find('li');
                element.find("li").css({
                    height: gridHeight/gridRows.length+'px'
                });
            });
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();

// FILE: zmart_components/directives/zmart-accordion.js
(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zmartAccordionItem', zmartAccordionItem);

    function zmartAccordionItem($timeout) {
        var widget;

        function calcElementHeight(elements) {
            elements.each(function(index, el) {
                el = $(el);
                var container = el.closest('.container');
                var accordionItemCount = widget.widgets.length;
                var openedHeight = height = container.height() / accordionItemCount;
                var item = el[0].item;

                if (item && item.opened) {
                    var dashboard = el.find('.zmart-accordion-item-dashboard');

                    openedHeight = height + dashboard.height(); // padding: 5px;
                }
                el.animate({height: openedHeight}, 300);

                if (item && item.opened && (!item.widgets || item.widgets.length === 0)) {
                    el.find('.zmart-accordion-item-header').animate({height: openedHeight}, 300);
                } else {
                    el.find('.zmart-accordion-item-header').animate({height: height}, 300);
                }
            });
        }

        $(window).on('resize.zmart-accordion', function() {
            $timeout(function() {
                calcElementHeight($('.zmart-accordion-item'));
            },100);
        });

        var linkFunction = function(theScope, theElement, attrs) {
            var scope = theScope;
            var element = theElement;
            widget = theScope.widget;

            scope.$on('zh-scrolled-finished', function(events, container) {
                if (container[0] == element.closest('.container')[0]) {
                    calcElementHeight(element);
                }
            });

            function initializeItem() {
                var item = widget.widgets[parseInt(attrs.itemIndex)];
                item.element = element;
                item.calcElementHeight = function() {
                    calcElementHeight(this.element);
                };
                element[0].item = item;
            }
            initializeItem();
        };

        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();

// FILE: zmart_components/esprima/esprima.js
/*
  Copyright (C) 2013 Ariya Hidayat <ariya.hidayat@gmail.com>
  Copyright (C) 2013 Thaddee Tyl <thaddee.tyl@gmail.com>
  Copyright (C) 2013 Mathias Bynens <mathias@qiwi.be>
  Copyright (C) 2012 Ariya Hidayat <ariya.hidayat@gmail.com>
  Copyright (C) 2012 Mathias Bynens <mathias@qiwi.be>
  Copyright (C) 2012 Joost-Wim Boekesteijn <joost-wim@boekesteijn.nl>
  Copyright (C) 2012 Kris Kowal <kris.kowal@cixar.com>
  Copyright (C) 2012 Yusuke Suzuki <utatane.tea@gmail.com>
  Copyright (C) 2012 Arpad Borsos <arpad.borsos@googlemail.com>
  Copyright (C) 2011 Ariya Hidayat <ariya.hidayat@gmail.com>

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

(function (root, factory) {
    'use strict';

    // Universal Module Definition (UMD) to support AMD, CommonJS/Node.js,
    // Rhino, and plain browser loading.

    /* istanbul ignore next */
    if (typeof define === 'function' && define.amd) {
        define(['exports'], factory);
    } else if (typeof exports !== 'undefined') {
        factory(exports);
    } else {
        factory((root.esprima = {}));
    }
}(this, function (exports) {
    'use strict';

    var Token,
        TokenName,
        FnExprTokens,
        Syntax,
        PlaceHolders,
        PropertyKind,
        Messages,
        Regex,
        source,
        strict,
        index,
        lineNumber,
        lineStart,
        length,
        lookahead,
        state,
        extra;

    Token = {
        BooleanLiteral: 1,
        EOF: 2,
        Identifier: 3,
        Keyword: 4,
        NullLiteral: 5,
        NumericLiteral: 6,
        Punctuator: 7,
        StringLiteral: 8,
        RegularExpression: 9
    };

    TokenName = {};
    TokenName[Token.BooleanLiteral] = 'Boolean';
    TokenName[Token.EOF] = '<end>';
    TokenName[Token.Identifier] = 'Identifier';
    TokenName[Token.Keyword] = 'Keyword';
    TokenName[Token.NullLiteral] = 'Null';
    TokenName[Token.NumericLiteral] = 'Numeric';
    TokenName[Token.Punctuator] = 'Punctuator';
    TokenName[Token.StringLiteral] = 'String';
    TokenName[Token.RegularExpression] = 'RegularExpression';

    // A function following one of those tokens is an expression.
    FnExprTokens = ['(', '{', '[', 'in', 'typeof', 'instanceof', 'new',
                    'return', 'case', 'delete', 'throw', 'void',
                    // assignment operators
                    '=', '+=', '-=', '*=', '/=', '%=', '<<=', '>>=', '>>>=',
                    '&=', '|=', '^=', ',',
                    // binary/unary operators
                    '+', '-', '*', '/', '%', '++', '--', '<<', '>>', '>>>', '&',
                    '|', '^', '!', '~', '&&', '||', '?', ':', '===', '==', '>=',
                    '<=', '<', '>', '!=', '!=='];

    Syntax = {
        AssignmentExpression: 'AssignmentExpression',
        ArrayExpression: 'ArrayExpression',
        ArrowFunctionExpression: 'ArrowFunctionExpression',
        BlockStatement: 'BlockStatement',
        BinaryExpression: 'BinaryExpression',
        BreakStatement: 'BreakStatement',
        CallExpression: 'CallExpression',
        CatchClause: 'CatchClause',
        ConditionalExpression: 'ConditionalExpression',
        ContinueStatement: 'ContinueStatement',
        DoWhileStatement: 'DoWhileStatement',
        DebuggerStatement: 'DebuggerStatement',
        EmptyStatement: 'EmptyStatement',
        ExpressionStatement: 'ExpressionStatement',
        ForStatement: 'ForStatement',
        ForInStatement: 'ForInStatement',
        FunctionDeclaration: 'FunctionDeclaration',
        FunctionExpression: 'FunctionExpression',
        Identifier: 'Identifier',
        IfStatement: 'IfStatement',
        Literal: 'Literal',
        LabeledStatement: 'LabeledStatement',
        LogicalExpression: 'LogicalExpression',
        MemberExpression: 'MemberExpression',
        NewExpression: 'NewExpression',
        ObjectExpression: 'ObjectExpression',
        Program: 'Program',
        Property: 'Property',
        ReturnStatement: 'ReturnStatement',
        SequenceExpression: 'SequenceExpression',
        SwitchStatement: 'SwitchStatement',
        SwitchCase: 'SwitchCase',
        ThisExpression: 'ThisExpression',
        ThrowStatement: 'ThrowStatement',
        TryStatement: 'TryStatement',
        UnaryExpression: 'UnaryExpression',
        UpdateExpression: 'UpdateExpression',
        VariableDeclaration: 'VariableDeclaration',
        VariableDeclarator: 'VariableDeclarator',
        WhileStatement: 'WhileStatement',
        WithStatement: 'WithStatement'
    };

    PlaceHolders = {
        ArrowParameterPlaceHolder: {
            type: 'ArrowParameterPlaceHolder'
        }
    };

    PropertyKind = {
        Data: 1,
        Get: 2,
        Set: 4
    };

    // Error messages should be identical to V8.
    Messages = {
        UnexpectedToken: 'Unexpected token %0',
        UnexpectedNumber: 'Unexpected number',
        UnexpectedString: 'Unexpected string',
        UnexpectedIdentifier: 'Unexpected identifier',
        UnexpectedReserved: 'Unexpected reserved word',
        UnexpectedEOS: 'Unexpected end of input',
        NewlineAfterThrow: 'Illegal newline after throw',
        InvalidRegExp: 'Invalid regular expression',
        UnterminatedRegExp: 'Invalid regular expression: missing /',
        InvalidLHSInAssignment: 'Invalid left-hand side in assignment',
        InvalidLHSInForIn: 'Invalid left-hand side in for-in',
        MultipleDefaultsInSwitch: 'More than one default clause in switch statement',
        NoCatchOrFinally: 'Missing catch or finally after try',
        UnknownLabel: 'Undefined label \'%0\'',
        Redeclaration: '%0 \'%1\' has already been declared',
        IllegalContinue: 'Illegal continue statement',
        IllegalBreak: 'Illegal break statement',
        IllegalReturn: 'Illegal return statement',
        StrictModeWith: 'Strict mode code may not include a with statement',
        StrictCatchVariable: 'Catch variable may not be eval or arguments in strict mode',
        StrictVarName: 'Variable name may not be eval or arguments in strict mode',
        StrictParamName: 'Parameter name eval or arguments is not allowed in strict mode',
        StrictParamDupe: 'Strict mode function may not have duplicate parameter names',
        StrictFunctionName: 'Function name may not be eval or arguments in strict mode',
        StrictOctalLiteral: 'Octal literals are not allowed in strict mode.',
        StrictDelete: 'Delete of an unqualified identifier in strict mode.',
        StrictDuplicateProperty: 'Duplicate data property in object literal not allowed in strict mode',
        AccessorDataProperty: 'Object literal may not have data and accessor property with the same name',
        AccessorGetSet: 'Object literal may not have multiple get/set accessors with the same name',
        StrictLHSAssignment: 'Assignment to eval or arguments is not allowed in strict mode',
        StrictLHSPostfix: 'Postfix increment/decrement may not have eval or arguments operand in strict mode',
        StrictLHSPrefix: 'Prefix increment/decrement may not have eval or arguments operand in strict mode',
        StrictReservedWord: 'Use of future reserved word in strict mode'
    };

    // See also tools/generate-unicode-regex.py.
    Regex = {
        NonAsciiIdentifierStart: new RegExp('[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0-\u08B2\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58\u0C59\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D60\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F4\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19C1-\u19C7\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCC\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA78E\uA790-\uA7AD\uA7B0\uA7B1\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB5F\uAB64\uAB65\uABC0-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]'),
        NonAsciiIdentifierPart: new RegExp('[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u08A0-\u08B2\u08E4-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58\u0C59\u0C60-\u0C63\u0C66-\u0C6F\u0C81-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D01-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D57\u0D60-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1380-\u138F\u13A0-\u13F4\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19D9\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1CD0-\u1CD2\u1CD4-\u1CF6\u1CF8\u1CF9\u1D00-\u1DF5\u1DFC-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u200C\u200D\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCC\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA69D\uA69F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA78E\uA790-\uA7AD\uA7B0\uA7B1\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C4\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB5F\uAB64\uAB65\uABC0-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2D\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]')
    };

    // Ensure the condition is true, otherwise throw an error.
    // This is only to have a better contract semantic, i.e. another safety net
    // to catch a logic error. The condition shall be fulfilled in normal case.
    // Do NOT use this to enforce a certain condition on any user input.

    function assert(condition, message) {
        /* istanbul ignore if */
        if (!condition) {
            throw new Error('ASSERT: ' + message);
        }
    }

    function isDecimalDigit(ch) {
        return (ch >= 0x30 && ch <= 0x39);   // 0..9
    }

    function isHexDigit(ch) {
        return '0123456789abcdefABCDEF'.indexOf(ch) >= 0;
    }

    function isOctalDigit(ch) {
        return '01234567'.indexOf(ch) >= 0;
    }


    // 7.2 White Space

    function isWhiteSpace(ch) {
        return (ch === 0x20) || (ch === 0x09) || (ch === 0x0B) || (ch === 0x0C) || (ch === 0xA0) ||
            (ch >= 0x1680 && [0x1680, 0x180E, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2007, 0x2008, 0x2009, 0x200A, 0x202F, 0x205F, 0x3000, 0xFEFF].indexOf(ch) >= 0);
    }

    // 7.3 Line Terminators

    function isLineTerminator(ch) {
        return (ch === 0x0A) || (ch === 0x0D) || (ch === 0x2028) || (ch === 0x2029);
    }

    // 7.6 Identifier Names and Identifiers

    function isIdentifierStart(ch) {
        return (ch === 0x24) || (ch === 0x5F) ||  // $ (dollar) and _ (underscore)
            (ch >= 0x41 && ch <= 0x5A) ||         // A..Z
            (ch >= 0x61 && ch <= 0x7A) ||         // a..z
            (ch === 0x5C) ||                      // \ (backslash)
            ((ch >= 0x80) && Regex.NonAsciiIdentifierStart.test(String.fromCharCode(ch)));
    }

    function isIdentifierPart(ch) {
        return (ch === 0x24) || (ch === 0x5F) ||  // $ (dollar) and _ (underscore)
            (ch >= 0x41 && ch <= 0x5A) ||         // A..Z
            (ch >= 0x61 && ch <= 0x7A) ||         // a..z
            (ch >= 0x30 && ch <= 0x39) ||         // 0..9
            (ch === 0x5C) ||                      // \ (backslash)
            ((ch >= 0x80) && Regex.NonAsciiIdentifierPart.test(String.fromCharCode(ch)));
    }

    // 7.6.1.2 Future Reserved Words

    function isFutureReservedWord(id) {
        switch (id) {
        case 'class':
        case 'enum':
        case 'export':
        case 'extends':
        case 'import':
        case 'super':
            return true;
        default:
            return false;
        }
    }

    function isStrictModeReservedWord(id) {
        switch (id) {
        case 'implements':
        case 'interface':
        case 'package':
        case 'private':
        case 'protected':
        case 'public':
        case 'static':
        case 'yield':
        case 'let':
            return true;
        default:
            return false;
        }
    }

    function isRestrictedWord(id) {
        return id === 'eval' || id === 'arguments';
    }

    // 7.6.1.1 Keywords

    function isKeyword(id) {
        if (strict && isStrictModeReservedWord(id)) {
            return true;
        }

        // 'const' is specialized as Keyword in V8.
        // 'yield' and 'let' are for compatibility with SpiderMonkey and ES.next.
        // Some others are from future reserved words.

        switch (id.length) {
        case 2:
            return (id === 'if') || (id === 'in') || (id === 'do');
        case 3:
            return (id === 'var') || (id === 'for') || (id === 'new') ||
                (id === 'try') || (id === 'let');
        case 4:
            return (id === 'this') || (id === 'else') || (id === 'case') ||
                (id === 'void') || (id === 'with') || (id === 'enum');
        case 5:
            return (id === 'while') || (id === 'break') || (id === 'catch') ||
                (id === 'throw') || (id === 'const') || (id === 'yield') ||
                (id === 'class') || (id === 'super');
        case 6:
            return (id === 'return') || (id === 'typeof') || (id === 'delete') ||
                (id === 'switch') || (id === 'export') || (id === 'import');
        case 7:
            return (id === 'default') || (id === 'finally') || (id === 'extends');
        case 8:
            return (id === 'function') || (id === 'continue') || (id === 'debugger');
        case 10:
            return (id === 'instanceof');
        default:
            return false;
        }
    }

    // 7.4 Comments

    function addComment(type, value, start, end, loc) {
        var comment;

        assert(typeof start === 'number', 'Comment must have valid position');

        // Because the way the actual token is scanned, often the comments
        // (if any) are skipped twice during the lexical analysis.
        // Thus, we need to skip adding a comment if the comment array already
        // handled it.
        if (state.lastCommentStart >= start) {
            return;
        }
        state.lastCommentStart = start;

        comment = {
            type: type,
            value: value
        };
        if (extra.range) {
            comment.range = [start, end];
        }
        if (extra.loc) {
            comment.loc = loc;
        }
        extra.comments.push(comment);
        if (extra.attachComment) {
            extra.leadingComments.push(comment);
            extra.trailingComments.push(comment);
        }
    }

    function skipSingleLineComment(offset) {
        var start, loc, ch, comment;

        start = index - offset;
        loc = {
            start: {
                line: lineNumber,
                column: index - lineStart - offset
            }
        };

        while (index < length) {
            ch = source.charCodeAt(index);
            ++index;
            if (isLineTerminator(ch)) {
                if (extra.comments) {
                    comment = source.slice(start + offset, index - 1);
                    loc.end = {
                        line: lineNumber,
                        column: index - lineStart - 1
                    };
                    addComment('Line', comment, start, index - 1, loc);
                }
                if (ch === 13 && source.charCodeAt(index) === 10) {
                    ++index;
                }
                ++lineNumber;
                lineStart = index;
                return;
            }
        }

        if (extra.comments) {
            comment = source.slice(start + offset, index);
            loc.end = {
                line: lineNumber,
                column: index - lineStart
            };
            addComment('Line', comment, start, index, loc);
        }
    }

    function skipMultiLineComment() {
        var start, loc, ch, comment;

        if (extra.comments) {
            start = index - 2;
            loc = {
                start: {
                    line: lineNumber,
                    column: index - lineStart - 2
                }
            };
        }

        while (index < length) {
            ch = source.charCodeAt(index);
            if (isLineTerminator(ch)) {
                if (ch === 0x0D && source.charCodeAt(index + 1) === 0x0A) {
                    ++index;
                }
                ++lineNumber;
                ++index;
                lineStart = index;
                if (index >= length) {
                    throwUnexpectedToken();
                }
            } else if (ch === 0x2A) {
                // Block comment ends with '*/'.
                if (source.charCodeAt(index + 1) === 0x2F) {
                    ++index;
                    ++index;
                    if (extra.comments) {
                        comment = source.slice(start + 2, index - 2);
                        loc.end = {
                            line: lineNumber,
                            column: index - lineStart
                        };
                        addComment('Block', comment, start, index, loc);
                    }
                    return;
                }
                ++index;
            } else {
                ++index;
            }
        }

        throwUnexpectedToken();
    }

    function skipComment() {
        var ch, start;

        start = (index === 0);
        while (index < length) {
            ch = source.charCodeAt(index);

            if (isWhiteSpace(ch)) {
                ++index;
            } else if (isLineTerminator(ch)) {
                ++index;
                if (ch === 0x0D && source.charCodeAt(index) === 0x0A) {
                    ++index;
                }
                ++lineNumber;
                lineStart = index;
                start = true;
            } else if (ch === 0x2F) { // U+002F is '/'
                ch = source.charCodeAt(index + 1);
                if (ch === 0x2F) {
                    ++index;
                    ++index;
                    skipSingleLineComment(2);
                    start = true;
                } else if (ch === 0x2A) {  // U+002A is '*'
                    ++index;
                    ++index;
                    skipMultiLineComment();
                } else {
                    break;
                }
            } else if (start && ch === 0x2D) { // U+002D is '-'
                // U+003E is '>'
                if ((source.charCodeAt(index + 1) === 0x2D) && (source.charCodeAt(index + 2) === 0x3E)) {
                    // '-->' is a single-line comment
                    index += 3;
                    skipSingleLineComment(3);
                } else {
                    break;
                }
            } else if (ch === 0x3C) { // U+003C is '<'
                if (source.slice(index + 1, index + 4) === '!--') {
                    ++index; // `<`
                    ++index; // `!`
                    ++index; // `-`
                    ++index; // `-`
                    skipSingleLineComment(4);
                } else {
                    break;
                }
            } else {
                break;
            }
        }
    }

    function scanHexEscape(prefix) {
        var i, len, ch, code = 0;

        len = (prefix === 'u') ? 4 : 2;
        for (i = 0; i < len; ++i) {
            if (index < length && isHexDigit(source[index])) {
                ch = source[index++];
                code = code * 16 + '0123456789abcdef'.indexOf(ch.toLowerCase());
            } else {
                return '';
            }
        }
        return String.fromCharCode(code);
    }

    function scanUnicodeCodePointEscape() {
        var ch, code, cu1, cu2;

        ch = source[index];
        code = 0;

        // At least, one hex digit is required.
        if (ch === '}') {
            throwUnexpectedToken();
        }

        while (index < length) {
            ch = source[index++];
            if (!isHexDigit(ch)) {
                break;
            }
            code = code * 16 + '0123456789abcdef'.indexOf(ch.toLowerCase());
        }

        if (code > 0x10FFFF || ch !== '}') {
            throwUnexpectedToken();
        }

        // UTF-16 Encoding
        if (code <= 0xFFFF) {
            return String.fromCharCode(code);
        }
        cu1 = ((code - 0x10000) >> 10) + 0xD800;
        cu2 = ((code - 0x10000) & 1023) + 0xDC00;
        return String.fromCharCode(cu1, cu2);
    }

    function getEscapedIdentifier() {
        var ch, id;

        ch = source.charCodeAt(index++);
        id = String.fromCharCode(ch);

        // '\u' (U+005C, U+0075) denotes an escaped character.
        if (ch === 0x5C) {
            if (source.charCodeAt(index) !== 0x75) {
                throwUnexpectedToken();
            }
            ++index;
            ch = scanHexEscape('u');
            if (!ch || ch === '\\' || !isIdentifierStart(ch.charCodeAt(0))) {
                throwUnexpectedToken();
            }
            id = ch;
        }

        while (index < length) {
            ch = source.charCodeAt(index);
            if (!isIdentifierPart(ch)) {
                break;
            }
            ++index;
            id += String.fromCharCode(ch);

            // '\u' (U+005C, U+0075) denotes an escaped character.
            if (ch === 0x5C) {
                id = id.substr(0, id.length - 1);
                if (source.charCodeAt(index) !== 0x75) {
                    throwUnexpectedToken();
                }
                ++index;
                ch = scanHexEscape('u');
                if (!ch || ch === '\\' || !isIdentifierPart(ch.charCodeAt(0))) {
                    throwUnexpectedToken();
                }
                id += ch;
            }
        }

        return id;
    }

    function getIdentifier() {
        var start, ch;

        start = index++;
        while (index < length) {
            ch = source.charCodeAt(index);
            if (ch === 0x5C) {
                // Blackslash (U+005C) marks Unicode escape sequence.
                index = start;
                return getEscapedIdentifier();
            }
            if (isIdentifierPart(ch)) {
                ++index;
            } else {
                break;
            }
        }

        return source.slice(start, index);
    }

    function scanIdentifier() {
        var start, id, type;

        start = index;

        // Backslash (U+005C) starts an escaped character.
        id = (source.charCodeAt(index) === 0x5C) ? getEscapedIdentifier() : getIdentifier();

        // There is no keyword or literal with only one character.
        // Thus, it must be an identifier.
        if (id.length === 1) {
            type = Token.Identifier;
        } else if (isKeyword(id)) {
            type = Token.Keyword;
        } else if (id === 'null') {
            type = Token.NullLiteral;
        } else if (id === 'true' || id === 'false') {
            type = Token.BooleanLiteral;
        } else {
            type = Token.Identifier;
        }

        return {
            type: type,
            value: id,
            lineNumber: lineNumber,
            lineStart: lineStart,
            start: start,
            end: index
        };
    }


    // 7.7 Punctuators

    function scanPunctuator() {
        var start = index,
            code = source.charCodeAt(index),
            code2,
            ch1 = source[index],
            ch2,
            ch3,
            ch4;

        switch (code) {

        // Check for most common single-character punctuators.
        case 0x2E:  // . dot
        case 0x28:  // ( open bracket
        case 0x29:  // ) close bracket
        case 0x3B:  // ; semicolon
        case 0x2C:  // , comma
        case 0x7B:  // { open curly brace
        case 0x7D:  // } close curly brace
        case 0x5B:  // [
        case 0x5D:  // ]
        case 0x3A:  // :
        case 0x3F:  // ?
        case 0x7E:  // ~
            ++index;
            if (extra.tokenize) {
                if (code === 0x28) {
                    extra.openParenToken = extra.tokens.length;
                } else if (code === 0x7B) {
                    extra.openCurlyToken = extra.tokens.length;
                }
            }
            return {
                type: Token.Punctuator,
                value: String.fromCharCode(code),
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: start,
                end: index
            };

        default:
            code2 = source.charCodeAt(index + 1);

            // '=' (U+003D) marks an assignment or comparison operator.
            if (code2 === 0x3D) {
                switch (code) {
                case 0x2B:  // +
                case 0x2D:  // -
                case 0x2F:  // /
                case 0x3C:  // <
                case 0x3E:  // >
                case 0x5E:  // ^
                case 0x7C:  // |
                case 0x25:  // %
                case 0x26:  // &
                case 0x2A:  // *
                    index += 2;
                    return {
                        type: Token.Punctuator,
                        value: String.fromCharCode(code) + String.fromCharCode(code2),
                        lineNumber: lineNumber,
                        lineStart: lineStart,
                        start: start,
                        end: index
                    };

                case 0x21: // !
                case 0x3D: // =
                    index += 2;

                    // !== and ===
                    if (source.charCodeAt(index) === 0x3D) {
                        ++index;
                    }
                    return {
                        type: Token.Punctuator,
                        value: source.slice(start, index),
                        lineNumber: lineNumber,
                        lineStart: lineStart,
                        start: start,
                        end: index
                    };
                }
            }
        }

        // 4-character punctuator: >>>=

        ch4 = source.substr(index, 4);

        if (ch4 === '>>>=') {
            index += 4;
            return {
                type: Token.Punctuator,
                value: ch4,
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: start,
                end: index
            };
        }

        // 3-character punctuators: === !== >>> <<= >>=

        ch3 = ch4.substr(0, 3);

        if (ch3 === '>>>' || ch3 === '<<=' || ch3 === '>>=') {
            index += 3;
            return {
                type: Token.Punctuator,
                value: ch3,
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: start,
                end: index
            };
        }

        // Other 2-character punctuators: ++ -- << >> && ||
        ch2 = ch3.substr(0, 2);

        if ((ch1 === ch2[1] && ('+-<>&|'.indexOf(ch1) >= 0)) || ch2 === '=>') {
            index += 2;
            return {
                type: Token.Punctuator,
                value: ch2,
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: start,
                end: index
            };
        }

        // 1-character punctuators: < > = ! + - * % & | ^ /

        if ('<>=!+-*%&|^/'.indexOf(ch1) >= 0) {
            ++index;
            return {
                type: Token.Punctuator,
                value: ch1,
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: start,
                end: index
            };
        }

        throwUnexpectedToken();
    }

    // 7.8.3 Numeric Literals

    function scanHexLiteral(start) {
        var number = '';

        while (index < length) {
            if (!isHexDigit(source[index])) {
                break;
            }
            number += source[index++];
        }

        if (number.length === 0) {
            throwUnexpectedToken();
        }

        if (isIdentifierStart(source.charCodeAt(index))) {
            throwUnexpectedToken();
        }

        return {
            type: Token.NumericLiteral,
            value: parseInt('0x' + number, 16),
            lineNumber: lineNumber,
            lineStart: lineStart,
            start: start,
            end: index
        };
    }

    function scanBinaryLiteral(start) {
        var ch, number;

        number = '';

        while (index < length) {
            ch = source[index];
            if (ch !== '0' && ch !== '1') {
                break;
            }
            number += source[index++];
        }

        if (number.length === 0) {
            // only 0b or 0B
            throwUnexpectedToken();
        }

        if (index < length) {
            ch = source.charCodeAt(index);
            /* istanbul ignore else */
            if (isIdentifierStart(ch) || isDecimalDigit(ch)) {
                throwUnexpectedToken();
            }
        }

        return {
            type: Token.NumericLiteral,
            value: parseInt(number, 2),
            lineNumber: lineNumber,
            lineStart: lineStart,
            start: start,
            end: index
        };
    }

    function scanOctalLiteral(prefix, start) {
        var number, octal;

        if (isOctalDigit(prefix)) {
            octal = true;
            number = '0' + source[index++];
        } else {
            octal = false;
            ++index;
            number = '';
        }

        while (index < length) {
            if (!isOctalDigit(source[index])) {
                break;
            }
            number += source[index++];
        }

        if (!octal && number.length === 0) {
            // only 0o or 0O
            throwUnexpectedToken();
        }

        if (isIdentifierStart(source.charCodeAt(index)) || isDecimalDigit(source.charCodeAt(index))) {
            throwUnexpectedToken();
        }

        return {
            type: Token.NumericLiteral,
            value: parseInt(number, 8),
            octal: octal,
            lineNumber: lineNumber,
            lineStart: lineStart,
            start: start,
            end: index
        };
    }

    function isImplicitOctalLiteral() {
        var i, ch;

        // Implicit octal, unless there is a non-octal digit.
        // (Annex B.1.1 on Numeric Literals)
        for (i = index + 1; i < length; ++i) {
            ch = source[i];
            if (ch === '8' || ch === '9') {
                return false;
            }
            if (!isOctalDigit(ch)) {
                return true;
            }
        }

        return true;
    }

    function scanNumericLiteral() {
        var number, start, ch;

        ch = source[index];
        assert(isDecimalDigit(ch.charCodeAt(0)) || (ch === '.'),
            'Numeric literal must start with a decimal digit or a decimal point');

        start = index;
        number = '';
        if (ch !== '.') {
            number = source[index++];
            ch = source[index];

            // Hex number starts with '0x'.
            // Octal number starts with '0'.
            // Octal number in ES6 starts with '0o'.
            // Binary number in ES6 starts with '0b'.
            if (number === '0') {
                if (ch === 'x' || ch === 'X') {
                    ++index;
                    return scanHexLiteral(start);
                }
                if (ch === 'b' || ch === 'B') {
                    ++index;
                    return scanBinaryLiteral(start);
                }
                if (ch === 'o' || ch === 'O') {
                    return scanOctalLiteral(ch, start);
                }

                if (isOctalDigit(ch)) {
                    if (isImplicitOctalLiteral()) {
                        return scanOctalLiteral(ch, start);
                    }
                }
            }

            while (isDecimalDigit(source.charCodeAt(index))) {
                number += source[index++];
            }
            ch = source[index];
        }

        if (ch === '.') {
            number += source[index++];
            while (isDecimalDigit(source.charCodeAt(index))) {
                number += source[index++];
            }
            ch = source[index];
        }

        if (ch === 'e' || ch === 'E') {
            number += source[index++];

            ch = source[index];
            if (ch === '+' || ch === '-') {
                number += source[index++];
            }
            if (isDecimalDigit(source.charCodeAt(index))) {
                while (isDecimalDigit(source.charCodeAt(index))) {
                    number += source[index++];
                }
            } else {
                throwUnexpectedToken();
            }
        }

        if (isIdentifierStart(source.charCodeAt(index))) {
            throwUnexpectedToken();
        }

        return {
            type: Token.NumericLiteral,
            value: parseFloat(number),
            lineNumber: lineNumber,
            lineStart: lineStart,
            start: start,
            end: index
        };
    }

    // 7.8.4 String Literals

    function scanStringLiteral() {
        var str = '', quote, start, ch, code, unescaped, restore, octal = false, startLineNumber, startLineStart;
        startLineNumber = lineNumber;
        startLineStart = lineStart;

        quote = source[index];
        assert((quote === '\'' || quote === '"'),
            'String literal must starts with a quote');

        start = index;
        ++index;

        while (index < length) {
            ch = source[index++];

            if (ch === quote) {
                quote = '';
                break;
            } else if (ch === '\\') {
                ch = source[index++];
                if (!ch || !isLineTerminator(ch.charCodeAt(0))) {
                    switch (ch) {
                    case 'u':
                    case 'x':
                        if (source[index] === '{') {
                            ++index;
                            str += scanUnicodeCodePointEscape();
                        } else {
                            restore = index;
                            unescaped = scanHexEscape(ch);
                            if (unescaped) {
                                str += unescaped;
                            } else {
                                index = restore;
                                str += ch;
                            }
                        }
                        break;
                    case 'n':
                        str += '\n';
                        break;
                    case 'r':
                        str += '\r';
                        break;
                    case 't':
                        str += '\t';
                        break;
                    case 'b':
                        str += '\b';
                        break;
                    case 'f':
                        str += '\f';
                        break;
                    case 'v':
                        str += '\x0B';
                        break;

                    default:
                        if (isOctalDigit(ch)) {
                            code = '01234567'.indexOf(ch);

                            // \0 is not octal escape sequence
                            if (code !== 0) {
                                octal = true;
                            }

                            if (index < length && isOctalDigit(source[index])) {
                                octal = true;
                                code = code * 8 + '01234567'.indexOf(source[index++]);

                                // 3 digits are only allowed when string starts
                                // with 0, 1, 2, 3
                                if ('0123'.indexOf(ch) >= 0 &&
                                        index < length &&
                                        isOctalDigit(source[index])) {
                                    code = code * 8 + '01234567'.indexOf(source[index++]);
                                }
                            }
                            str += String.fromCharCode(code);
                        } else {
                            str += ch;
                        }
                        break;
                    }
                } else {
                    ++lineNumber;
                    if (ch === '\r' && source[index] === '\n') {
                        ++index;
                    }
                    lineStart = index;
                }
            } else if (isLineTerminator(ch.charCodeAt(0))) {
                break;
            } else {
                str += ch;
            }
        }

        if (quote !== '') {
            throwUnexpectedToken();
        }

        return {
            type: Token.StringLiteral,
            value: str,
            octal: octal,
            startLineNumber: startLineNumber,
            startLineStart: startLineStart,
            lineNumber: lineNumber,
            lineStart: lineStart,
            start: start,
            end: index
        };
    }

    function testRegExp(pattern, flags) {
        var tmp = pattern,
            value;

        if (flags.indexOf('u') >= 0) {
            // Replace each astral symbol and every Unicode code point
            // escape sequence with a single ASCII symbol to avoid throwing on
            // regular expressions that are only valid in combination with the
            // `/u` flag.
            // Note: replacing with the ASCII symbol `x` might cause false
            // negatives in unlikely scenarios. For example, `[\u{61}-b]` is a
            // perfectly valid pattern that is equivalent to `[a-b]`, but it
            // would be replaced by `[x-b]` which throws an error.
            tmp = tmp
                .replace(/\\u\{([0-9a-fA-F]+)\}/g, function ($0, $1) {
                    if (parseInt($1, 16) <= 0x10FFFF) {
                        return 'x';
                    }
                    throwError(Messages.InvalidRegExp);
                })
                .replace(/[\uD800-\uDBFF][\uDC00-\uDFFF]/g, 'x');
        }

        // First, detect invalid regular expressions.
        try {
            value = new RegExp(tmp);
        } catch (e) {
            throwError(Messages.InvalidRegExp);
        }

        // Return a regular expression object for this pattern-flag pair, or
        // `null` in case the current environment doesn't support the flags it
        // uses.
        try {
            return new RegExp(pattern, flags);
        } catch (exception) {
            return null;
        }
    }

    function scanRegExpBody() {
        var ch, str, classMarker, terminated, body;

        ch = source[index];
        assert(ch === '/', 'Regular expression literal must start with a slash');
        str = source[index++];

        classMarker = false;
        terminated = false;
        while (index < length) {
            ch = source[index++];
            str += ch;
            if (ch === '\\') {
                ch = source[index++];
                // ECMA-262 7.8.5
                if (isLineTerminator(ch.charCodeAt(0))) {
                    throwError(Messages.UnterminatedRegExp);
                }
                str += ch;
            } else if (isLineTerminator(ch.charCodeAt(0))) {
                throwError(Messages.UnterminatedRegExp);
            } else if (classMarker) {
                if (ch === ']') {
                    classMarker = false;
                }
            } else {
                if (ch === '/') {
                    terminated = true;
                    break;
                } else if (ch === '[') {
                    classMarker = true;
                }
            }
        }

        if (!terminated) {
            throwError(Messages.UnterminatedRegExp);
        }

        // Exclude leading and trailing slash.
        body = str.substr(1, str.length - 2);
        return {
            value: body,
            literal: str
        };
    }

    function scanRegExpFlags() {
        var ch, str, flags, restore;

        str = '';
        flags = '';
        while (index < length) {
            ch = source[index];
            if (!isIdentifierPart(ch.charCodeAt(0))) {
                break;
            }

            ++index;
            if (ch === '\\' && index < length) {
                ch = source[index];
                if (ch === 'u') {
                    ++index;
                    restore = index;
                    ch = scanHexEscape('u');
                    if (ch) {
                        flags += ch;
                        for (str += '\\u'; restore < index; ++restore) {
                            str += source[restore];
                        }
                    } else {
                        index = restore;
                        flags += 'u';
                        str += '\\u';
                    }
                    tolerateUnexpectedToken();
                } else {
                    str += '\\';
                    tolerateUnexpectedToken();
                }
            } else {
                flags += ch;
                str += ch;
            }
        }

        return {
            value: flags,
            literal: str
        };
    }

    function scanRegExp() {
        var start, body, flags, value;

        lookahead = null;
        skipComment();
        start = index;

        body = scanRegExpBody();
        flags = scanRegExpFlags();
        value = testRegExp(body.value, flags.value);

        if (extra.tokenize) {
            return {
                type: Token.RegularExpression,
                value: value,
                regex: {
                    pattern: body.value,
                    flags: flags.value
                },
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: start,
                end: index
            };
        }

        return {
            literal: body.literal + flags.literal,
            value: value,
            regex: {
                pattern: body.value,
                flags: flags.value
            },
            start: start,
            end: index
        };
    }

    function collectRegex() {
        var pos, loc, regex, token;

        skipComment();

        pos = index;
        loc = {
            start: {
                line: lineNumber,
                column: index - lineStart
            }
        };

        regex = scanRegExp();

        loc.end = {
            line: lineNumber,
            column: index - lineStart
        };

        /* istanbul ignore next */
        if (!extra.tokenize) {
            // Pop the previous token, which is likely '/' or '/='
            if (extra.tokens.length > 0) {
                token = extra.tokens[extra.tokens.length - 1];
                if (token.range[0] === pos && token.type === 'Punctuator') {
                    if (token.value === '/' || token.value === '/=') {
                        extra.tokens.pop();
                    }
                }
            }

            extra.tokens.push({
                type: 'RegularExpression',
                value: regex.literal,
                regex: regex.regex,
                range: [pos, index],
                loc: loc
            });
        }

        return regex;
    }

    function isIdentifierName(token) {
        return token.type === Token.Identifier ||
            token.type === Token.Keyword ||
            token.type === Token.BooleanLiteral ||
            token.type === Token.NullLiteral;
    }

    function advanceSlash() {
        var prevToken,
            checkToken;
        // Using the following algorithm:
        // https://github.com/mozilla/sweet.js/wiki/design
        prevToken = extra.tokens[extra.tokens.length - 1];
        if (!prevToken) {
            // Nothing before that: it cannot be a division.
            return collectRegex();
        }
        if (prevToken.type === 'Punctuator') {
            if (prevToken.value === ']') {
                return scanPunctuator();
            }
            if (prevToken.value === ')') {
                checkToken = extra.tokens[extra.openParenToken - 1];
                if (checkToken &&
                        checkToken.type === 'Keyword' &&
                        (checkToken.value === 'if' ||
                         checkToken.value === 'while' ||
                         checkToken.value === 'for' ||
                         checkToken.value === 'with')) {
                    return collectRegex();
                }
                return scanPunctuator();
            }
            if (prevToken.value === '}') {
                // Dividing a function by anything makes little sense,
                // but we have to check for that.
                if (extra.tokens[extra.openCurlyToken - 3] &&
                        extra.tokens[extra.openCurlyToken - 3].type === 'Keyword') {
                    // Anonymous function.
                    checkToken = extra.tokens[extra.openCurlyToken - 4];
                    if (!checkToken) {
                        return scanPunctuator();
                    }
                } else if (extra.tokens[extra.openCurlyToken - 4] &&
                        extra.tokens[extra.openCurlyToken - 4].type === 'Keyword') {
                    // Named function.
                    checkToken = extra.tokens[extra.openCurlyToken - 5];
                    if (!checkToken) {
                        return collectRegex();
                    }
                } else {
                    return scanPunctuator();
                }
                // checkToken determines whether the function is
                // a declaration or an expression.
                if (FnExprTokens.indexOf(checkToken.value) >= 0) {
                    // It is an expression.
                    return scanPunctuator();
                }
                // It is a declaration.
                return collectRegex();
            }
            return collectRegex();
        }
        if (prevToken.type === 'Keyword' && prevToken.value !== 'this') {
            return collectRegex();
        }
        return scanPunctuator();
    }

    function advance() {
        var ch;

        skipComment();

        if (index >= length) {
            return {
                type: Token.EOF,
                lineNumber: lineNumber,
                lineStart: lineStart,
                start: index,
                end: index
            };
        }

        ch = source.charCodeAt(index);

        if (isIdentifierStart(ch)) {
            return scanIdentifier();
        }

        // Very common: ( and ) and ;
        if (ch === 0x28 || ch === 0x29 || ch === 0x3B) {
            return scanPunctuator();
        }

        // String literal starts with single quote (U+0027) or double quote (U+0022).
        if (ch === 0x27 || ch === 0x22) {
            return scanStringLiteral();
        }


        // Dot (.) U+002E can also start a floating-point number, hence the need
        // to check the next character.
        if (ch === 0x2E) {
            if (isDecimalDigit(source.charCodeAt(index + 1))) {
                return scanNumericLiteral();
            }
            return scanPunctuator();
        }

        if (isDecimalDigit(ch)) {
            return scanNumericLiteral();
        }

        // Slash (/) U+002F can also start a regex.
        if (extra.tokenize && ch === 0x2F) {
            return advanceSlash();
        }

        return scanPunctuator();
    }

    function collectToken() {
        var loc, token, value, entry;

        skipComment();
        loc = {
            start: {
                line: lineNumber,
                column: index - lineStart
            }
        };

        token = advance();
        loc.end = {
            line: lineNumber,
            column: index - lineStart
        };

        if (token.type !== Token.EOF) {
            value = source.slice(token.start, token.end);
            entry = {
                type: TokenName[token.type],
                value: value,
                range: [token.start, token.end],
                loc: loc
            };
            if (token.regex) {
                entry.regex = {
                    pattern: token.regex.pattern,
                    flags: token.regex.flags
                };
            }
            extra.tokens.push(entry);
        }

        return token;
    }

    function lex() {
        var token;

        token = lookahead;
        index = token.end;
        lineNumber = token.lineNumber;
        lineStart = token.lineStart;

        lookahead = (typeof extra.tokens !== 'undefined') ? collectToken() : advance();

        index = token.end;
        lineNumber = token.lineNumber;
        lineStart = token.lineStart;

        return token;
    }

    function peek() {
        var pos, line, start;

        pos = index;
        line = lineNumber;
        start = lineStart;
        lookahead = (typeof extra.tokens !== 'undefined') ? collectToken() : advance();
        index = pos;
        lineNumber = line;
        lineStart = start;
    }

    function Position() {
        this.line = lineNumber;
        this.column = index - lineStart;
    }

    function SourceLocation() {
        this.start = new Position();
        this.end = null;
    }

    function WrappingSourceLocation(startToken) {
        if (startToken.type === Token.StringLiteral) {
            this.start = {
                line: startToken.startLineNumber,
                column: startToken.start - startToken.startLineStart
            };
        } else {
            this.start = {
                line: startToken.lineNumber,
                column: startToken.start - startToken.lineStart
            };
        }
        this.end = null;
    }

    function Node() {
        // Skip comment.
        index = lookahead.start;
        if (lookahead.type === Token.StringLiteral) {
            lineNumber = lookahead.startLineNumber;
            lineStart = lookahead.startLineStart;
        } else {
            lineNumber = lookahead.lineNumber;
            lineStart = lookahead.lineStart;
        }
        if (extra.range) {
            this.range = [index, 0];
        }
        if (extra.loc) {
            this.loc = new SourceLocation();
        }
    }

    function WrappingNode(startToken) {
        if (extra.range) {
            this.range = [startToken.start, 0];
        }
        if (extra.loc) {
            this.loc = new WrappingSourceLocation(startToken);
        }
    }

    WrappingNode.prototype = Node.prototype = {

        processComment: function () {
            var lastChild,
                leadingComments,
                trailingComments,
                bottomRight = extra.bottomRightStack,
                i,
                comment,
                last = bottomRight[bottomRight.length - 1];

            if (this.type === Syntax.Program) {
                if (this.body.length > 0) {
                    return;
                }
            }

            if (extra.trailingComments.length > 0) {
                trailingComments = [];
                for (i = extra.trailingComments.length - 1; i >= 0; --i) {
                    comment = extra.trailingComments[i];
                    if (comment.range[0] >= this.range[1]) {
                        trailingComments.unshift(comment);
                        extra.trailingComments.splice(i, 1);
                    }
                }
                extra.trailingComments = [];
            } else {
                if (last && last.trailingComments && last.trailingComments[0].range[0] >= this.range[1]) {
                    trailingComments = last.trailingComments;
                    delete last.trailingComments;
                }
            }

            // Eating the stack.
            if (last) {
                while (last && last.range[0] >= this.range[0]) {
                    lastChild = last;
                    last = bottomRight.pop();
                }
            }

            if (lastChild) {
                if (lastChild.leadingComments && lastChild.leadingComments[lastChild.leadingComments.length - 1].range[1] <= this.range[0]) {
                    this.leadingComments = lastChild.leadingComments;
                    lastChild.leadingComments = undefined;
                }
            } else if (extra.leadingComments.length > 0) {
                leadingComments = [];
                for (i = extra.leadingComments.length - 1; i >= 0; --i) {
                    comment = extra.leadingComments[i];
                    if (comment.range[1] <= this.range[0]) {
                        leadingComments.unshift(comment);
                        extra.leadingComments.splice(i, 1);
                    }
                }
            }


            if (leadingComments && leadingComments.length > 0) {
                this.leadingComments = leadingComments;
            }
            if (trailingComments && trailingComments.length > 0) {
                this.trailingComments = trailingComments;
            }

            bottomRight.push(this);
        },

        finish: function () {
            if (extra.range) {
                this.range[1] = index;
            }
            if (extra.loc) {
                this.loc.end = new Position();
                if (extra.source) {
                    this.loc.source = extra.source;
                }
            }

            if (extra.attachComment) {
                this.processComment();
            }
        },

        finishArrayExpression: function (elements) {
            this.type = Syntax.ArrayExpression;
            this.elements = elements;
            this.finish();
            return this;
        },

        finishArrowFunctionExpression: function (params, defaults, body, expression) {
            this.type = Syntax.ArrowFunctionExpression;
            this.id = null;
            this.params = params;
            this.defaults = defaults;
            this.body = body;
            this.rest = null;
            this.generator = false;
            this.expression = expression;
            this.finish();
            return this;
        },

        finishAssignmentExpression: function (operator, left, right) {
            this.type = Syntax.AssignmentExpression;
            this.operator = operator;
            this.left = left;
            this.right = right;
            this.finish();
            return this;
        },

        finishBinaryExpression: function (operator, left, right) {
            this.type = (operator === '||' || operator === '&&') ? Syntax.LogicalExpression : Syntax.BinaryExpression;
            this.operator = operator;
            this.left = left;
            this.right = right;
            this.finish();
            return this;
        },

        finishBlockStatement: function (body) {
            this.type = Syntax.BlockStatement;
            this.body = body;
            this.finish();
            return this;
        },

        finishBreakStatement: function (label) {
            this.type = Syntax.BreakStatement;
            this.label = label;
            this.finish();
            return this;
        },

        finishCallExpression: function (callee, args) {
            this.type = Syntax.CallExpression;
            this.callee = callee;
            this.arguments = args;
            this.finish();
            return this;
        },

        finishCatchClause: function (param, body) {
            this.type = Syntax.CatchClause;
            this.param = param;
            this.body = body;
            this.finish();
            return this;
        },

        finishConditionalExpression: function (test, consequent, alternate) {
            this.type = Syntax.ConditionalExpression;
            this.test = test;
            this.consequent = consequent;
            this.alternate = alternate;
            this.finish();
            return this;
        },

        finishContinueStatement: function (label) {
            this.type = Syntax.ContinueStatement;
            this.label = label;
            this.finish();
            return this;
        },

        finishDebuggerStatement: function () {
            this.type = Syntax.DebuggerStatement;
            this.finish();
            return this;
        },

        finishDoWhileStatement: function (body, test) {
            this.type = Syntax.DoWhileStatement;
            this.body = body;
            this.test = test;
            this.finish();
            return this;
        },

        finishEmptyStatement: function () {
            this.type = Syntax.EmptyStatement;
            this.finish();
            return this;
        },

        finishExpressionStatement: function (expression) {
            this.type = Syntax.ExpressionStatement;
            this.expression = expression;
            this.finish();
            return this;
        },

        finishForStatement: function (init, test, update, body) {
            this.type = Syntax.ForStatement;
            this.init = init;
            this.test = test;
            this.update = update;
            this.body = body;
            this.finish();
            return this;
        },

        finishForInStatement: function (left, right, body) {
            this.type = Syntax.ForInStatement;
            this.left = left;
            this.right = right;
            this.body = body;
            this.each = false;
            this.finish();
            return this;
        },

        finishFunctionDeclaration: function (id, params, defaults, body) {
            this.type = Syntax.FunctionDeclaration;
            this.id = id;
            this.params = params;
            this.defaults = defaults;
            this.body = body;
            this.rest = null;
            this.generator = false;
            this.expression = false;
            this.finish();
            return this;
        },

        finishFunctionExpression: function (id, params, defaults, body) {
            this.type = Syntax.FunctionExpression;
            this.id = id;
            this.params = params;
            this.defaults = defaults;
            this.body = body;
            this.rest = null;
            this.generator = false;
            this.expression = false;
            this.finish();
            return this;
        },

        finishIdentifier: function (name) {
            this.type = Syntax.Identifier;
            this.name = name;
            this.finish();
            return this;
        },

        finishIfStatement: function (test, consequent, alternate) {
            this.type = Syntax.IfStatement;
            this.test = test;
            this.consequent = consequent;
            this.alternate = alternate;
            this.finish();
            return this;
        },

        finishLabeledStatement: function (label, body) {
            this.type = Syntax.LabeledStatement;
            this.label = label;
            this.body = body;
            this.finish();
            return this;
        },

        finishLiteral: function (token) {
            this.type = Syntax.Literal;
            this.value = token.value;
            this.raw = source.slice(token.start, token.end);
            if (token.regex) {
                this.regex = token.regex;
            }
            this.finish();
            return this;
        },

        finishMemberExpression: function (accessor, object, property) {
            this.type = Syntax.MemberExpression;
            this.computed = accessor === '[';
            this.object = object;
            this.property = property;
            this.finish();
            return this;
        },

        finishNewExpression: function (callee, args) {
            this.type = Syntax.NewExpression;
            this.callee = callee;
            this.arguments = args;
            this.finish();
            return this;
        },

        finishObjectExpression: function (properties) {
            this.type = Syntax.ObjectExpression;
            this.properties = properties;
            this.finish();
            return this;
        },

        finishPostfixExpression: function (operator, argument) {
            this.type = Syntax.UpdateExpression;
            this.operator = operator;
            this.argument = argument;
            this.prefix = false;
            this.finish();
            return this;
        },

        finishProgram: function (body) {
            this.type = Syntax.Program;
            this.body = body;
            this.finish();
            return this;
        },

        finishProperty: function (kind, key, value, method, shorthand) {
            this.type = Syntax.Property;
            this.key = key;
            this.value = value;
            this.kind = kind;
            this.method = method;
            this.shorthand = shorthand;
            this.finish();
            return this;
        },

        finishReturnStatement: function (argument) {
            this.type = Syntax.ReturnStatement;
            this.argument = argument;
            this.finish();
            return this;
        },

        finishSequenceExpression: function (expressions) {
            this.type = Syntax.SequenceExpression;
            this.expressions = expressions;
            this.finish();
            return this;
        },

        finishSwitchCase: function (test, consequent) {
            this.type = Syntax.SwitchCase;
            this.test = test;
            this.consequent = consequent;
            this.finish();
            return this;
        },

        finishSwitchStatement: function (discriminant, cases) {
            this.type = Syntax.SwitchStatement;
            this.discriminant = discriminant;
            this.cases = cases;
            this.finish();
            return this;
        },

        finishThisExpression: function () {
            this.type = Syntax.ThisExpression;
            this.finish();
            return this;
        },

        finishThrowStatement: function (argument) {
            this.type = Syntax.ThrowStatement;
            this.argument = argument;
            this.finish();
            return this;
        },

        finishTryStatement: function (block, guardedHandlers, handlers, finalizer) {
            this.type = Syntax.TryStatement;
            this.block = block;
            this.guardedHandlers = guardedHandlers;
            this.handlers = handlers;
            this.finalizer = finalizer;
            this.finish();
            return this;
        },

        finishUnaryExpression: function (operator, argument) {
            this.type = (operator === '++' || operator === '--') ? Syntax.UpdateExpression : Syntax.UnaryExpression;
            this.operator = operator;
            this.argument = argument;
            this.prefix = true;
            this.finish();
            return this;
        },

        finishVariableDeclaration: function (declarations, kind) {
            this.type = Syntax.VariableDeclaration;
            this.declarations = declarations;
            this.kind = kind;
            this.finish();
            return this;
        },

        finishVariableDeclarator: function (id, init) {
            this.type = Syntax.VariableDeclarator;
            this.id = id;
            this.init = init;
            this.finish();
            return this;
        },

        finishWhileStatement: function (test, body) {
            this.type = Syntax.WhileStatement;
            this.test = test;
            this.body = body;
            this.finish();
            return this;
        },

        finishWithStatement: function (object, body) {
            this.type = Syntax.WithStatement;
            this.object = object;
            this.body = body;
            this.finish();
            return this;
        }
    };

    // Return true if there is a line terminator before the next token.

    function peekLineTerminator() {
        var pos, line, start, found;

        pos = index;
        line = lineNumber;
        start = lineStart;
        skipComment();
        found = lineNumber !== line;
        index = pos;
        lineNumber = line;
        lineStart = start;

        return found;
    }

    function createError(line, pos, description) {
        var error = new Error('Line ' + line + ': ' + description);
        error.index = pos;
        error.lineNumber = line;
        error.column = pos - lineStart + 1;
        error.description = description;
        return error;
    }

    // Throw an exception

    function throwError(messageFormat) {
        var args, msg;

        args = Array.prototype.slice.call(arguments, 1);
        msg = messageFormat.replace(/%(\d)/g,
            function (whole, idx) {
                assert(idx < args.length, 'Message reference must be in range');
                return args[idx];
            }
        );

        throw createError(lineNumber, index, msg);
    }

    function tolerateError(messageFormat) {
        var args, msg, error;

        args = Array.prototype.slice.call(arguments, 1);
        /* istanbul ignore next */
        msg = messageFormat.replace(/%(\d)/g,
            function (whole, idx) {
                assert(idx < args.length, 'Message reference must be in range');
                return args[idx];
            }
        );

        error = createError(lineNumber, index, msg);
        if (extra.errors) {
            extra.errors.push(error);
        } else {
            throw error;
        }
    }

    // Throw an exception because of the token.

    function unexpectedTokenError(token, message) {
        var msg = Messages.UnexpectedToken;

        if (token) {
            msg = message ? message :
                (token.type === Token.EOF) ? Messages.UnexpectedEOS :
                (token.type === Token.Identifier) ? Messages.UnexpectedIdentifier :
                (token.type === Token.NumericLiteral) ? Messages.UnexpectedNumber :
                (token.type === Token.StringLiteral) ? Messages.UnexpectedString :
                Messages.UnexpectedToken;

            if (token.type === Token.Keyword) {
                if (isFutureReservedWord(token.value)) {
                    msg = Messages.UnexpectedReserved;
                } else if (strict && isStrictModeReservedWord(token.value)) {
                    msg = Messages.StrictReservedWord;
                }
            }
        }

        msg = msg.replace('%0', token ? token.value : 'ILLEGAL');

        return (token && typeof token.lineNumber === 'number') ?
            createError(token.lineNumber, token.start, msg) :
            createError(lineNumber, index, msg);
    }

    function throwUnexpectedToken(token, message) {
        throw unexpectedTokenError(token, message);
    }

    function tolerateUnexpectedToken(token, message) {
        var error = unexpectedTokenError(token, message);
        if (extra.errors) {
            extra.errors.push(error);
        } else {
            throw error;
        }
    }

    // Expect the next token to match the specified punctuator.
    // If not, an exception will be thrown.

    function expect(value) {
        var token = lex();
        if (token.type !== Token.Punctuator || token.value !== value) {
            throwUnexpectedToken(token);
        }
    }

    /**
     * @name expectCommaSeparator
     * @description Quietly expect a comma when in tolerant mode, otherwise delegates
     * to <code>expect(value)</code>
     * @since 2.0
     */
    function expectCommaSeparator() {
        var token;

        if (extra.errors) {
            token = lookahead;
            if (token.type === Token.Punctuator && token.value === ',') {
                lex();
            } else if (token.type === Token.Punctuator && token.value === ';') {
                lex();
                tolerateUnexpectedToken(token);
            } else {
                tolerateUnexpectedToken(token, Messages.UnexpectedToken);
            }
        } else {
            expect(',');
        }
    }

    // Expect the next token to match the specified keyword.
    // If not, an exception will be thrown.

    function expectKeyword(keyword) {
        var token = lex();
        if (token.type !== Token.Keyword || token.value !== keyword) {
            throwUnexpectedToken(token);
        }
    }

    // Return true if the next token matches the specified punctuator.

    function match(value) {
        return lookahead.type === Token.Punctuator && lookahead.value === value;
    }

    // Return true if the next token matches the specified keyword

    function matchKeyword(keyword) {
        return lookahead.type === Token.Keyword && lookahead.value === keyword;
    }

    // Return true if the next token is an assignment operator

    function matchAssign() {
        var op;

        if (lookahead.type !== Token.Punctuator) {
            return false;
        }
        op = lookahead.value;
        return op === '=' ||
            op === '*=' ||
            op === '/=' ||
            op === '%=' ||
            op === '+=' ||
            op === '-=' ||
            op === '<<=' ||
            op === '>>=' ||
            op === '>>>=' ||
            op === '&=' ||
            op === '^=' ||
            op === '|=';
    }

    function consumeSemicolon() {
        var line, oldIndex = index, oldLineNumber = lineNumber,
            oldLineStart = lineStart, oldLookahead = lookahead;

        // Catch the very common case first: immediately a semicolon (U+003B).
        if (source.charCodeAt(index) === 0x3B || match(';')) {
            lex();
            return;
        }

        line = lineNumber;
        skipComment();
        if (lineNumber !== line) {
            index = oldIndex;
            lineNumber = oldLineNumber;
            lineStart = oldLineStart;
            lookahead = oldLookahead;
            return;
        }

        if (lookahead.type !== Token.EOF && !match('}')) {
            throwUnexpectedToken(lookahead);
        }
    }

    // Return true if provided expression is LeftHandSideExpression

    function isLeftHandSide(expr) {
        return expr.type === Syntax.Identifier || expr.type === Syntax.MemberExpression;
    }

    // 11.1.4 Array Initialiser

    function parseArrayInitialiser() {
        var elements = [], node = new Node();

        expect('[');

        while (!match(']')) {
            if (match(',')) {
                lex();
                elements.push(null);
            } else {
                elements.push(parseAssignmentExpression());

                if (!match(']')) {
                    expect(',');
                }
            }
        }

        lex();

        return node.finishArrayExpression(elements);
    }

    // 11.1.5 Object Initialiser

    function parsePropertyFunction(param, first) {
        var previousStrict, body, node = new Node();

        previousStrict = strict;
        body = parseFunctionSourceElements();
        if (first && strict && isRestrictedWord(param[0].name)) {
            tolerateUnexpectedToken(first, Messages.StrictParamName);
        }
        strict = previousStrict;
        return node.finishFunctionExpression(null, param, [], body);
    }

    function parsePropertyMethodFunction() {
        var previousStrict, param, method;

        previousStrict = strict;
        strict = true;
        param = parseParams();
        method = parsePropertyFunction(param.params);
        strict = previousStrict;

        return method;
    }

    function parseObjectPropertyKey() {
        var token, node = new Node();

        token = lex();

        // Note: This function is called only from parseObjectProperty(), where
        // EOF and Punctuator tokens are already filtered out.

        if (token.type === Token.StringLiteral || token.type === Token.NumericLiteral) {
            if (strict && token.octal) {
                tolerateUnexpectedToken(token, Messages.StrictOctalLiteral);
            }
            return node.finishLiteral(token);
        }

        return node.finishIdentifier(token.value);
    }

    function parseObjectProperty() {
        var token, key, id, value, param, node = new Node();

        token = lookahead;

        if (token.type === Token.Identifier) {

            id = parseObjectPropertyKey();

            // Property Assignment: Getter and Setter.

            if (token.value === 'get' && !(match(':') || match('('))) {
                key = parseObjectPropertyKey();
                expect('(');
                expect(')');
                value = parsePropertyFunction([]);
                return node.finishProperty('get', key, value, false, false);
            }
            if (token.value === 'set' && !(match(':') || match('('))) {
                key = parseObjectPropertyKey();
                expect('(');
                token = lookahead;
                if (token.type !== Token.Identifier) {
                    expect(')');
                    tolerateUnexpectedToken(token);
                    value = parsePropertyFunction([]);
                } else {
                    param = [ parseVariableIdentifier() ];
                    expect(')');
                    value = parsePropertyFunction(param, token);
                }
                return node.finishProperty('set', key, value, false, false);
            }
            if (match(':')) {
                lex();
                value = parseAssignmentExpression();
                return node.finishProperty('init', id, value, false, false);
            }
            if (match('(')) {
                value = parsePropertyMethodFunction();
                return node.finishProperty('init', id, value, true, false);
            }

            value = id;
            return node.finishProperty('init', id, value, false, true);
        }
        if (token.type === Token.EOF || token.type === Token.Punctuator) {
            throwUnexpectedToken(token);
        } else {
            key = parseObjectPropertyKey();
            if (match(':')) {
                lex();
                value = parseAssignmentExpression();
                return node.finishProperty('init', key, value, false, false);
            }
            if (match('(')) {
                value = parsePropertyMethodFunction();
                return node.finishProperty('init', key, value, true, false);
            }
            throwUnexpectedToken(lex());
        }
    }

    function parseObjectInitialiser() {
        var properties = [], property, name, key, kind, map = {}, toString = String, node = new Node();

        expect('{');

        while (!match('}')) {
            property = parseObjectProperty();

            if (property.key.type === Syntax.Identifier) {
                name = property.key.name;
            } else {
                name = toString(property.key.value);
            }
            kind = (property.kind === 'init') ? PropertyKind.Data : (property.kind === 'get') ? PropertyKind.Get : PropertyKind.Set;

            key = '$' + name;
            if (Object.prototype.hasOwnProperty.call(map, key)) {
                if (map[key] === PropertyKind.Data) {
                    if (strict && kind === PropertyKind.Data) {
                        tolerateError(Messages.StrictDuplicateProperty);
                    } else if (kind !== PropertyKind.Data) {
                        tolerateError(Messages.AccessorDataProperty);
                    }
                } else {
                    if (kind === PropertyKind.Data) {
                        tolerateError(Messages.AccessorDataProperty);
                    } else if (map[key] & kind) {
                        tolerateError(Messages.AccessorGetSet);
                    }
                }
                map[key] |= kind;
            } else {
                map[key] = kind;
            }

            properties.push(property);

            if (!match('}')) {
                expectCommaSeparator();
            }
        }

        expect('}');

        return node.finishObjectExpression(properties);
    }

    // 11.1.6 The Grouping Operator

    function parseGroupExpression() {
        var expr;

        expect('(');

        if (match(')')) {
            lex();
            return PlaceHolders.ArrowParameterPlaceHolder;
        }

        ++state.parenthesisCount;

        expr = parseExpression();

        expect(')');

        return expr;
    }


    // 11.1 Primary Expressions

    function parsePrimaryExpression() {
        var type, token, expr, node;

        if (match('(')) {
            return parseGroupExpression();
        }

        if (match('[')) {
            return parseArrayInitialiser();
        }

        if (match('{')) {
            return parseObjectInitialiser();
        }

        type = lookahead.type;
        node = new Node();

        if (type === Token.Identifier) {
            expr = node.finishIdentifier(lex().value);
        } else if (type === Token.StringLiteral || type === Token.NumericLiteral) {
            if (strict && lookahead.octal) {
                tolerateUnexpectedToken(lookahead, Messages.StrictOctalLiteral);
            }
            expr = node.finishLiteral(lex());
        } else if (type === Token.Keyword) {
            if (matchKeyword('function')) {
                return parseFunctionExpression();
            }
            if (matchKeyword('this')) {
                lex();
                expr = node.finishThisExpression();
            } else {
                throwUnexpectedToken(lex());
            }
        } else if (type === Token.BooleanLiteral) {
            token = lex();
            token.value = (token.value === 'true');
            expr = node.finishLiteral(token);
        } else if (type === Token.NullLiteral) {
            token = lex();
            token.value = null;
            expr = node.finishLiteral(token);
        } else if (match('/') || match('/=')) {
            if (typeof extra.tokens !== 'undefined') {
                expr = node.finishLiteral(collectRegex());
            } else {
                expr = node.finishLiteral(scanRegExp());
            }
            peek();
        } else {
            throwUnexpectedToken(lex());
        }

        return expr;
    }

    // 11.2 Left-Hand-Side Expressions

    function parseArguments() {
        var args = [];

        expect('(');

        if (!match(')')) {
            while (index < length) {
                args.push(parseAssignmentExpression());
                if (match(')')) {
                    break;
                }
                expectCommaSeparator();
            }
        }

        expect(')');

        return args;
    }

    function parseNonComputedProperty() {
        var token, node = new Node();

        token = lex();

        if (!isIdentifierName(token)) {
            throwUnexpectedToken(token);
        }

        return node.finishIdentifier(token.value);
    }

    function parseNonComputedMember() {
        expect('.');

        return parseNonComputedProperty();
    }

    function parseComputedMember() {
        var expr;

        expect('[');

        expr = parseExpression();

        expect(']');

        return expr;
    }

    function parseNewExpression() {
        var callee, args, node = new Node();

        expectKeyword('new');
        callee = parseLeftHandSideExpression();
        args = match('(') ? parseArguments() : [];

        return node.finishNewExpression(callee, args);
    }

    function parseLeftHandSideExpressionAllowCall() {
        var expr, args, property, startToken, previousAllowIn = state.allowIn;

        startToken = lookahead;
        state.allowIn = true;
        expr = matchKeyword('new') ? parseNewExpression() : parsePrimaryExpression();

        for (;;) {
            if (match('.')) {
                property = parseNonComputedMember();
                expr = new WrappingNode(startToken).finishMemberExpression('.', expr, property);
            } else if (match('(')) {
                args = parseArguments();
                expr = new WrappingNode(startToken).finishCallExpression(expr, args);
            } else if (match('[')) {
                property = parseComputedMember();
                expr = new WrappingNode(startToken).finishMemberExpression('[', expr, property);
            } else {
                break;
            }
        }
        state.allowIn = previousAllowIn;

        return expr;
    }

    function parseLeftHandSideExpression() {
        var expr, property, startToken;
        assert(state.allowIn, 'callee of new expression always allow in keyword.');

        startToken = lookahead;

        expr = matchKeyword('new') ? parseNewExpression() : parsePrimaryExpression();

        for (;;) {
            if (match('[')) {
                property = parseComputedMember();
                expr = new WrappingNode(startToken).finishMemberExpression('[', expr, property);
            } else if (match('.')) {
                property = parseNonComputedMember();
                expr = new WrappingNode(startToken).finishMemberExpression('.', expr, property);
            } else {
                break;
            }
        }
        return expr;
    }

    // 11.3 Postfix Expressions

    function parsePostfixExpression() {
        var expr, token, startToken = lookahead;

        expr = parseLeftHandSideExpressionAllowCall();

        if (lookahead.type === Token.Punctuator) {
            if ((match('++') || match('--')) && !peekLineTerminator()) {
                // 11.3.1, 11.3.2
                if (strict && expr.type === Syntax.Identifier && isRestrictedWord(expr.name)) {
                    tolerateError(Messages.StrictLHSPostfix);
                }

                if (!isLeftHandSide(expr)) {
                    tolerateError(Messages.InvalidLHSInAssignment);
                }

                token = lex();
                expr = new WrappingNode(startToken).finishPostfixExpression(token.value, expr);
            }
        }

        return expr;
    }

    // 11.4 Unary Operators

    function parseUnaryExpression() {
        var token, expr, startToken;

        if (lookahead.type !== Token.Punctuator && lookahead.type !== Token.Keyword) {
            expr = parsePostfixExpression();
        } else if (match('++') || match('--')) {
            startToken = lookahead;
            token = lex();
            expr = parseUnaryExpression();
            // 11.4.4, 11.4.5
            if (strict && expr.type === Syntax.Identifier && isRestrictedWord(expr.name)) {
                tolerateError(Messages.StrictLHSPrefix);
            }

            if (!isLeftHandSide(expr)) {
                tolerateError(Messages.InvalidLHSInAssignment);
            }

            expr = new WrappingNode(startToken).finishUnaryExpression(token.value, expr);
        } else if (match('+') || match('-') || match('~') || match('!')) {
            startToken = lookahead;
            token = lex();
            expr = parseUnaryExpression();
            expr = new WrappingNode(startToken).finishUnaryExpression(token.value, expr);
        } else if (matchKeyword('delete') || matchKeyword('void') || matchKeyword('typeof')) {
            startToken = lookahead;
            token = lex();
            expr = parseUnaryExpression();
            expr = new WrappingNode(startToken).finishUnaryExpression(token.value, expr);
            if (strict && expr.operator === 'delete' && expr.argument.type === Syntax.Identifier) {
                tolerateError(Messages.StrictDelete);
            }
        } else {
            expr = parsePostfixExpression();
        }

        return expr;
    }

    function binaryPrecedence(token, allowIn) {
        var prec = 0;

        if (token.type !== Token.Punctuator && token.type !== Token.Keyword) {
            return 0;
        }

        switch (token.value) {
        case '||':
            prec = 1;
            break;

        case '&&':
            prec = 2;
            break;

        case '|':
            prec = 3;
            break;

        case '^':
            prec = 4;
            break;

        case '&':
            prec = 5;
            break;

        case '==':
        case '!=':
        case '===':
        case '!==':
            prec = 6;
            break;

        case '<':
        case '>':
        case '<=':
        case '>=':
        case 'instanceof':
            prec = 7;
            break;

        case 'in':
            prec = allowIn ? 7 : 0;
            break;

        case '<<':
        case '>>':
        case '>>>':
            prec = 8;
            break;

        case '+':
        case '-':
            prec = 9;
            break;

        case '*':
        case '/':
        case '%':
            prec = 11;
            break;

        default:
            break;
        }

        return prec;
    }

    // 11.5 Multiplicative Operators
    // 11.6 Additive Operators
    // 11.7 Bitwise Shift Operators
    // 11.8 Relational Operators
    // 11.9 Equality Operators
    // 11.10 Binary Bitwise Operators
    // 11.11 Binary Logical Operators

    function parseBinaryExpression() {
        var marker, markers, expr, token, prec, stack, right, operator, left, i;

        marker = lookahead;
        left = parseUnaryExpression();
        if (left === PlaceHolders.ArrowParameterPlaceHolder) {
            return left;
        }

        token = lookahead;
        prec = binaryPrecedence(token, state.allowIn);
        if (prec === 0) {
            return left;
        }
        token.prec = prec;
        lex();

        markers = [marker, lookahead];
        right = parseUnaryExpression();

        stack = [left, token, right];

        while ((prec = binaryPrecedence(lookahead, state.allowIn)) > 0) {

            // Reduce: make a binary expression from the three topmost entries.
            while ((stack.length > 2) && (prec <= stack[stack.length - 2].prec)) {
                right = stack.pop();
                operator = stack.pop().value;
                left = stack.pop();
                markers.pop();
                expr = new WrappingNode(markers[markers.length - 1]).finishBinaryExpression(operator, left, right);
                stack.push(expr);
            }

            // Shift.
            token = lex();
            token.prec = prec;
            stack.push(token);
            markers.push(lookahead);
            expr = parseUnaryExpression();
            stack.push(expr);
        }

        // Final reduce to clean-up the stack.
        i = stack.length - 1;
        expr = stack[i];
        markers.pop();
        while (i > 1) {
            expr = new WrappingNode(markers.pop()).finishBinaryExpression(stack[i - 1].value, stack[i - 2], expr);
            i -= 2;
        }

        return expr;
    }


    // 11.12 Conditional Operator

    function parseConditionalExpression() {
        var expr, previousAllowIn, consequent, alternate, startToken;

        startToken = lookahead;

        expr = parseBinaryExpression();
        if (expr === PlaceHolders.ArrowParameterPlaceHolder) {
            return expr;
        }
        if (match('?')) {
            lex();
            previousAllowIn = state.allowIn;
            state.allowIn = true;
            consequent = parseAssignmentExpression();
            state.allowIn = previousAllowIn;
            expect(':');
            alternate = parseAssignmentExpression();

            expr = new WrappingNode(startToken).finishConditionalExpression(expr, consequent, alternate);
        }

        return expr;
    }

    // [ES6] 14.2 Arrow Function

    function parseConciseBody() {
        if (match('{')) {
            return parseFunctionSourceElements();
        }
        return parseAssignmentExpression();
    }

    function reinterpretAsCoverFormalsList(expressions) {
        var i, len, param, params, defaults, defaultCount, options, rest, token;

        params = [];
        defaults = [];
        defaultCount = 0;
        rest = null;
        options = {
            paramSet: {}
        };

        for (i = 0, len = expressions.length; i < len; i += 1) {
            param = expressions[i];
            if (param.type === Syntax.Identifier) {
                params.push(param);
                defaults.push(null);
                validateParam(options, param, param.name);
            } else if (param.type === Syntax.AssignmentExpression) {
                params.push(param.left);
                defaults.push(param.right);
                ++defaultCount;
                validateParam(options, param.left, param.left.name);
            } else {
                return null;
            }
        }

        if (options.message === Messages.StrictParamDupe) {
            token = strict ? options.stricted : options.firstRestricted;
            throwUnexpectedToken(token, options.message);
        }

        if (defaultCount === 0) {
            defaults = [];
        }

        return {
            params: params,
            defaults: defaults,
            rest: rest,
            stricted: options.stricted,
            firstRestricted: options.firstRestricted,
            message: options.message
        };
    }

    function parseArrowFunctionExpression(options, node) {
        var previousStrict, body;

        expect('=>');
        previousStrict = strict;

        body = parseConciseBody();

        if (strict && options.firstRestricted) {
            throwUnexpectedToken(options.firstRestricted, options.message);
        }
        if (strict && options.stricted) {
            tolerateUnexpectedToken(options.stricted, options.message);
        }

        strict = previousStrict;

        return node.finishArrowFunctionExpression(options.params, options.defaults, body, body.type !== Syntax.BlockStatement);
    }

    // 11.13 Assignment Operators

    function parseAssignmentExpression() {
        var oldParenthesisCount, token, expr, right, list, startToken;

        oldParenthesisCount = state.parenthesisCount;

        startToken = lookahead;
        token = lookahead;

        expr = parseConditionalExpression();

        if (expr === PlaceHolders.ArrowParameterPlaceHolder || match('=>')) {
            if (state.parenthesisCount === oldParenthesisCount ||
                    state.parenthesisCount === (oldParenthesisCount + 1)) {
                if (expr.type === Syntax.Identifier) {
                    list = reinterpretAsCoverFormalsList([ expr ]);
                } else if (expr.type === Syntax.AssignmentExpression) {
                    list = reinterpretAsCoverFormalsList([ expr ]);
                } else if (expr.type === Syntax.SequenceExpression) {
                    list = reinterpretAsCoverFormalsList(expr.expressions);
                } else if (expr === PlaceHolders.ArrowParameterPlaceHolder) {
                    list = reinterpretAsCoverFormalsList([]);
                }
                if (list) {
                    return parseArrowFunctionExpression(list, new WrappingNode(startToken));
                }
            }
        }

        if (matchAssign()) {
            // LeftHandSideExpression
            if (!isLeftHandSide(expr)) {
                tolerateError(Messages.InvalidLHSInAssignment);
            }

            // 11.13.1
            if (strict && expr.type === Syntax.Identifier && isRestrictedWord(expr.name)) {
                tolerateUnexpectedToken(token, Messages.StrictLHSAssignment);
            }

            token = lex();
            right = parseAssignmentExpression();
            expr = new WrappingNode(startToken).finishAssignmentExpression(token.value, expr, right);
        }

        return expr;
    }

    // 11.14 Comma Operator

    function parseExpression() {
        var expr, startToken = lookahead, expressions;

        expr = parseAssignmentExpression();

        if (match(',')) {
            expressions = [expr];

            while (index < length) {
                if (!match(',')) {
                    break;
                }
                lex();
                expressions.push(parseAssignmentExpression());
            }

            expr = new WrappingNode(startToken).finishSequenceExpression(expressions);
        }

        return expr;
    }

    // 12.1 Block

    function parseStatementList() {
        var list = [],
            statement;

        while (index < length) {
            if (match('}')) {
                break;
            }
            statement = parseSourceElement();
            if (typeof statement === 'undefined') {
                break;
            }
            list.push(statement);
        }

        return list;
    }

    function parseBlock() {
        var block, node = new Node();

        expect('{');

        block = parseStatementList();

        expect('}');

        return node.finishBlockStatement(block);
    }

    // 12.2 Variable Statement

    function parseVariableIdentifier() {
        var token, node = new Node();

        token = lex();

        if (token.type !== Token.Identifier) {
            if (strict && token.type === Token.Keyword && isStrictModeReservedWord(token.value)) {
                tolerateUnexpectedToken(token, Messages.StrictReservedWord);
            } else {
                throwUnexpectedToken(token);
            }
        }

        return node.finishIdentifier(token.value);
    }

    function parseVariableDeclaration(kind) {
        var init = null, id, node = new Node();

        id = parseVariableIdentifier();

        // 12.2.1
        if (strict && isRestrictedWord(id.name)) {
            tolerateError(Messages.StrictVarName);
        }

        if (kind === 'const') {
            expect('=');
            init = parseAssignmentExpression();
        } else if (match('=')) {
            lex();
            init = parseAssignmentExpression();
        }

        return node.finishVariableDeclarator(id, init);
    }

    function parseVariableDeclarationList(kind) {
        var list = [];

        do {
            list.push(parseVariableDeclaration(kind));
            if (!match(',')) {
                break;
            }
            lex();
        } while (index < length);

        return list;
    }

    function parseVariableStatement(node) {
        var declarations;

        expectKeyword('var');

        declarations = parseVariableDeclarationList();

        consumeSemicolon();

        return node.finishVariableDeclaration(declarations, 'var');
    }

    // kind may be `const` or `let`
    // Both are experimental and not in the specification yet.
    // see http://wiki.ecmascript.org/doku.php?id=harmony:const
    // and http://wiki.ecmascript.org/doku.php?id=harmony:let
    function parseConstLetDeclaration(kind) {
        var declarations, node = new Node();

        expectKeyword(kind);

        declarations = parseVariableDeclarationList(kind);

        consumeSemicolon();

        return node.finishVariableDeclaration(declarations, kind);
    }

    // 12.3 Empty Statement

    function parseEmptyStatement() {
        var node = new Node();
        expect(';');
        return node.finishEmptyStatement();
    }

    // 12.4 Expression Statement

    function parseExpressionStatement(node) {
        var expr = parseExpression();
        consumeSemicolon();
        return node.finishExpressionStatement(expr);
    }

    // 12.5 If statement

    function parseIfStatement(node) {
        var test, consequent, alternate;

        expectKeyword('if');

        expect('(');

        test = parseExpression();

        expect(')');

        consequent = parseStatement();

        if (matchKeyword('else')) {
            lex();
            alternate = parseStatement();
        } else {
            alternate = null;
        }

        return node.finishIfStatement(test, consequent, alternate);
    }

    // 12.6 Iteration Statements

    function parseDoWhileStatement(node) {
        var body, test, oldInIteration;

        expectKeyword('do');

        oldInIteration = state.inIteration;
        state.inIteration = true;

        body = parseStatement();

        state.inIteration = oldInIteration;

        expectKeyword('while');

        expect('(');

        test = parseExpression();

        expect(')');

        if (match(';')) {
            lex();
        }

        return node.finishDoWhileStatement(body, test);
    }

    function parseWhileStatement(node) {
        var test, body, oldInIteration;

        expectKeyword('while');

        expect('(');

        test = parseExpression();

        expect(')');

        oldInIteration = state.inIteration;
        state.inIteration = true;

        body = parseStatement();

        state.inIteration = oldInIteration;

        return node.finishWhileStatement(test, body);
    }

    function parseForVariableDeclaration() {
        var token, declarations, node = new Node();

        token = lex();
        declarations = parseVariableDeclarationList();

        return node.finishVariableDeclaration(declarations, token.value);
    }

    function parseForStatement(node) {
        var init, test, update, left, right, body, oldInIteration, previousAllowIn = state.allowIn;

        init = test = update = null;

        expectKeyword('for');

        expect('(');

        if (match(';')) {
            lex();
        } else {
            if (matchKeyword('var') || matchKeyword('let')) {
                state.allowIn = false;
                init = parseForVariableDeclaration();
                state.allowIn = previousAllowIn;

                if (init.declarations.length === 1 && matchKeyword('in')) {
                    lex();
                    left = init;
                    right = parseExpression();
                    init = null;
                }
            } else {
                state.allowIn = false;
                init = parseExpression();
                state.allowIn = previousAllowIn;

                if (matchKeyword('in')) {
                    // LeftHandSideExpression
                    if (!isLeftHandSide(init)) {
                        tolerateError(Messages.InvalidLHSInForIn);
                    }

                    lex();
                    left = init;
                    right = parseExpression();
                    init = null;
                }
            }

            if (typeof left === 'undefined') {
                expect(';');
            }
        }

        if (typeof left === 'undefined') {

            if (!match(';')) {
                test = parseExpression();
            }
            expect(';');

            if (!match(')')) {
                update = parseExpression();
            }
        }

        expect(')');

        oldInIteration = state.inIteration;
        state.inIteration = true;

        body = parseStatement();

        state.inIteration = oldInIteration;

        return (typeof left === 'undefined') ?
                node.finishForStatement(init, test, update, body) :
                node.finishForInStatement(left, right, body);
    }

    // 12.7 The continue statement

    function parseContinueStatement(node) {
        var label = null, key;

        expectKeyword('continue');

        // Optimize the most common form: 'continue;'.
        if (source.charCodeAt(index) === 0x3B) {
            lex();

            if (!state.inIteration) {
                throwError(Messages.IllegalContinue);
            }

            return node.finishContinueStatement(null);
        }

        if (peekLineTerminator()) {
            if (!state.inIteration) {
                throwError(Messages.IllegalContinue);
            }

            return node.finishContinueStatement(null);
        }

        if (lookahead.type === Token.Identifier) {
            label = parseVariableIdentifier();

            key = '$' + label.name;
            if (!Object.prototype.hasOwnProperty.call(state.labelSet, key)) {
                throwError(Messages.UnknownLabel, label.name);
            }
        }

        consumeSemicolon();

        if (label === null && !state.inIteration) {
            throwError(Messages.IllegalContinue);
        }

        return node.finishContinueStatement(label);
    }

    // 12.8 The break statement

    function parseBreakStatement(node) {
        var label = null, key;

        expectKeyword('break');

        // Catch the very common case first: immediately a semicolon (U+003B).
        if (source.charCodeAt(index) === 0x3B) {
            lex();

            if (!(state.inIteration || state.inSwitch)) {
                throwError(Messages.IllegalBreak);
            }

            return node.finishBreakStatement(null);
        }

        if (peekLineTerminator()) {
            if (!(state.inIteration || state.inSwitch)) {
                throwError(Messages.IllegalBreak);
            }

            return node.finishBreakStatement(null);
        }

        if (lookahead.type === Token.Identifier) {
            label = parseVariableIdentifier();

            key = '$' + label.name;
            if (!Object.prototype.hasOwnProperty.call(state.labelSet, key)) {
                throwError(Messages.UnknownLabel, label.name);
            }
        }

        consumeSemicolon();

        if (label === null && !(state.inIteration || state.inSwitch)) {
            throwError(Messages.IllegalBreak);
        }

        return node.finishBreakStatement(label);
    }

    // 12.9 The return statement

    function parseReturnStatement(node) {
        var argument = null;

        expectKeyword('return');

        if (!state.inFunctionBody) {
            tolerateError(Messages.IllegalReturn);
        }

        // 'return' followed by a space and an identifier is very common.
        if (source.charCodeAt(index) === 0x20) {
            if (isIdentifierStart(source.charCodeAt(index + 1))) {
                argument = parseExpression();
                consumeSemicolon();
                return node.finishReturnStatement(argument);
            }
        }

        if (peekLineTerminator()) {
            return node.finishReturnStatement(null);
        }

        if (!match(';')) {
            if (!match('}') && lookahead.type !== Token.EOF) {
                argument = parseExpression();
            }
        }

        consumeSemicolon();

        return node.finishReturnStatement(argument);
    }

    // 12.10 The with statement

    function parseWithStatement(node) {
        var object, body;

        if (strict) {
            // TODO(ikarienator): Should we update the test cases instead?
            skipComment();
            tolerateError(Messages.StrictModeWith);
        }

        expectKeyword('with');

        expect('(');

        object = parseExpression();

        expect(')');

        body = parseStatement();

        return node.finishWithStatement(object, body);
    }

    // 12.10 The swith statement

    function parseSwitchCase() {
        var test, consequent = [], statement, node = new Node();

        if (matchKeyword('default')) {
            lex();
            test = null;
        } else {
            expectKeyword('case');
            test = parseExpression();
        }
        expect(':');

        while (index < length) {
            if (match('}') || matchKeyword('default') || matchKeyword('case')) {
                break;
            }
            statement = parseStatement();
            consequent.push(statement);
        }

        return node.finishSwitchCase(test, consequent);
    }

    function parseSwitchStatement(node) {
        var discriminant, cases, clause, oldInSwitch, defaultFound;

        expectKeyword('switch');

        expect('(');

        discriminant = parseExpression();

        expect(')');

        expect('{');

        cases = [];

        if (match('}')) {
            lex();
            return node.finishSwitchStatement(discriminant, cases);
        }

        oldInSwitch = state.inSwitch;
        state.inSwitch = true;
        defaultFound = false;

        while (index < length) {
            if (match('}')) {
                break;
            }
            clause = parseSwitchCase();
            if (clause.test === null) {
                if (defaultFound) {
                    throwError(Messages.MultipleDefaultsInSwitch);
                }
                defaultFound = true;
            }
            cases.push(clause);
        }

        state.inSwitch = oldInSwitch;

        expect('}');

        return node.finishSwitchStatement(discriminant, cases);
    }

    // 12.13 The throw statement

    function parseThrowStatement(node) {
        var argument;

        expectKeyword('throw');

        if (peekLineTerminator()) {
            throwError(Messages.NewlineAfterThrow);
        }

        argument = parseExpression();

        consumeSemicolon();

        return node.finishThrowStatement(argument);
    }

    // 12.14 The try statement

    function parseCatchClause() {
        var param, body, node = new Node();

        expectKeyword('catch');

        expect('(');
        if (match(')')) {
            throwUnexpectedToken(lookahead);
        }

        param = parseVariableIdentifier();
        // 12.14.1
        if (strict && isRestrictedWord(param.name)) {
            tolerateError(Messages.StrictCatchVariable);
        }

        expect(')');
        body = parseBlock();
        return node.finishCatchClause(param, body);
    }

    function parseTryStatement(node) {
        var block, handlers = [], finalizer = null;

        expectKeyword('try');

        block = parseBlock();

        if (matchKeyword('catch')) {
            handlers.push(parseCatchClause());
        }

        if (matchKeyword('finally')) {
            lex();
            finalizer = parseBlock();
        }

        if (handlers.length === 0 && !finalizer) {
            throwError(Messages.NoCatchOrFinally);
        }

        return node.finishTryStatement(block, [], handlers, finalizer);
    }

    // 12.15 The debugger statement

    function parseDebuggerStatement(node) {
        expectKeyword('debugger');

        consumeSemicolon();

        return node.finishDebuggerStatement();
    }

    // 12 Statements

    function parseStatement() {
        var type = lookahead.type,
            expr,
            labeledBody,
            key,
            node;

        if (type === Token.EOF) {
            throwUnexpectedToken(lookahead);
        }

        if (type === Token.Punctuator && lookahead.value === '{') {
            return parseBlock();
        }

        node = new Node();

        if (type === Token.Punctuator) {
            switch (lookahead.value) {
            case ';':
                return parseEmptyStatement(node);
            case '(':
                return parseExpressionStatement(node);
            default:
                break;
            }
        } else if (type === Token.Keyword) {
            switch (lookahead.value) {
            case 'break':
                return parseBreakStatement(node);
            case 'continue':
                return parseContinueStatement(node);
            case 'debugger':
                return parseDebuggerStatement(node);
            case 'do':
                return parseDoWhileStatement(node);
            case 'for':
                return parseForStatement(node);
            case 'function':
                return parseFunctionDeclaration(node);
            case 'if':
                return parseIfStatement(node);
            case 'return':
                return parseReturnStatement(node);
            case 'switch':
                return parseSwitchStatement(node);
            case 'throw':
                return parseThrowStatement(node);
            case 'try':
                return parseTryStatement(node);
            case 'var':
                return parseVariableStatement(node);
            case 'while':
                return parseWhileStatement(node);
            case 'with':
                return parseWithStatement(node);
            default:
                break;
            }
        }

        expr = parseExpression();

        // 12.12 Labelled Statements
        if ((expr.type === Syntax.Identifier) && match(':')) {
            lex();

            key = '$' + expr.name;
            if (Object.prototype.hasOwnProperty.call(state.labelSet, key)) {
                throwError(Messages.Redeclaration, 'Label', expr.name);
            }

            state.labelSet[key] = true;
            labeledBody = parseStatement();
            delete state.labelSet[key];
            return node.finishLabeledStatement(expr, labeledBody);
        }

        consumeSemicolon();

        return node.finishExpressionStatement(expr);
    }

    // 13 Function Definition

    function parseFunctionSourceElements() {
        var sourceElement, sourceElements = [], token, directive, firstRestricted,
            oldLabelSet, oldInIteration, oldInSwitch, oldInFunctionBody, oldParenthesisCount,
            node = new Node();

        expect('{');

        while (index < length) {
            if (lookahead.type !== Token.StringLiteral) {
                break;
            }
            token = lookahead;

            sourceElement = parseSourceElement();
            sourceElements.push(sourceElement);
            if (sourceElement.expression.type !== Syntax.Literal) {
                // this is not directive
                break;
            }
            directive = source.slice(token.start + 1, token.end - 1);
            if (directive === 'use strict') {
                strict = true;
                if (firstRestricted) {
                    tolerateUnexpectedToken(firstRestricted, Messages.StrictOctalLiteral);
                }
            } else {
                if (!firstRestricted && token.octal) {
                    firstRestricted = token;
                }
            }
        }

        oldLabelSet = state.labelSet;
        oldInIteration = state.inIteration;
        oldInSwitch = state.inSwitch;
        oldInFunctionBody = state.inFunctionBody;
        oldParenthesisCount = state.parenthesizedCount;

        state.labelSet = {};
        state.inIteration = false;
        state.inSwitch = false;
        state.inFunctionBody = true;
        state.parenthesizedCount = 0;

        while (index < length) {
            if (match('}')) {
                break;
            }
            sourceElement = parseSourceElement();
            if (typeof sourceElement === 'undefined') {
                break;
            }
            sourceElements.push(sourceElement);
        }

        expect('}');

        state.labelSet = oldLabelSet;
        state.inIteration = oldInIteration;
        state.inSwitch = oldInSwitch;
        state.inFunctionBody = oldInFunctionBody;
        state.parenthesizedCount = oldParenthesisCount;

        return node.finishBlockStatement(sourceElements);
    }

    function validateParam(options, param, name) {
        var key = '$' + name;
        if (strict) {
            if (isRestrictedWord(name)) {
                options.stricted = param;
                options.message = Messages.StrictParamName;
            }
            if (Object.prototype.hasOwnProperty.call(options.paramSet, key)) {
                options.stricted = param;
                options.message = Messages.StrictParamDupe;
            }
        } else if (!options.firstRestricted) {
            if (isRestrictedWord(name)) {
                options.firstRestricted = param;
                options.message = Messages.StrictParamName;
            } else if (isStrictModeReservedWord(name)) {
                options.firstRestricted = param;
                options.message = Messages.StrictReservedWord;
            } else if (Object.prototype.hasOwnProperty.call(options.paramSet, key)) {
                options.firstRestricted = param;
                options.message = Messages.StrictParamDupe;
            }
        }
        options.paramSet[key] = true;
    }

    function parseParam(options) {
        var token, param, def;

        token = lookahead;
        param = parseVariableIdentifier();
        validateParam(options, token, token.value);
        if (match('=')) {
            lex();
            def = parseAssignmentExpression();
            ++options.defaultCount;
        }

        options.params.push(param);
        options.defaults.push(def);

        return !match(')');
    }

    function parseParams(firstRestricted) {
        var options;

        options = {
            params: [],
            defaultCount: 0,
            defaults: [],
            firstRestricted: firstRestricted
        };

        expect('(');

        if (!match(')')) {
            options.paramSet = {};
            while (index < length) {
                if (!parseParam(options)) {
                    break;
                }
                expect(',');
            }
        }

        expect(')');

        if (options.defaultCount === 0) {
            options.defaults = [];
        }

        return {
            params: options.params,
            defaults: options.defaults,
            stricted: options.stricted,
            firstRestricted: options.firstRestricted,
            message: options.message
        };
    }

    function parseFunctionDeclaration() {
        var id, params = [], defaults = [], body, token, stricted, tmp, firstRestricted, message, previousStrict, node = new Node();

        expectKeyword('function');
        token = lookahead;
        id = parseVariableIdentifier();
        if (strict) {
            if (isRestrictedWord(token.value)) {
                tolerateUnexpectedToken(token, Messages.StrictFunctionName);
            }
        } else {
            if (isRestrictedWord(token.value)) {
                firstRestricted = token;
                message = Messages.StrictFunctionName;
            } else if (isStrictModeReservedWord(token.value)) {
                firstRestricted = token;
                message = Messages.StrictReservedWord;
            }
        }

        tmp = parseParams(firstRestricted);
        params = tmp.params;
        defaults = tmp.defaults;
        stricted = tmp.stricted;
        firstRestricted = tmp.firstRestricted;
        if (tmp.message) {
            message = tmp.message;
        }

        previousStrict = strict;
        body = parseFunctionSourceElements();
        if (strict && firstRestricted) {
            throwUnexpectedToken(firstRestricted, message);
        }
        if (strict && stricted) {
            tolerateUnexpectedToken(stricted, message);
        }
        strict = previousStrict;

        return node.finishFunctionDeclaration(id, params, defaults, body);
    }

    function parseFunctionExpression() {
        var token, id = null, stricted, firstRestricted, message, tmp,
            params = [], defaults = [], body, previousStrict, node = new Node();

        expectKeyword('function');

        if (!match('(')) {
            token = lookahead;
            id = parseVariableIdentifier();
            if (strict) {
                if (isRestrictedWord(token.value)) {
                    tolerateUnexpectedToken(token, Messages.StrictFunctionName);
                }
            } else {
                if (isRestrictedWord(token.value)) {
                    firstRestricted = token;
                    message = Messages.StrictFunctionName;
                } else if (isStrictModeReservedWord(token.value)) {
                    firstRestricted = token;
                    message = Messages.StrictReservedWord;
                }
            }
        }

        tmp = parseParams(firstRestricted);
        params = tmp.params;
        defaults = tmp.defaults;
        stricted = tmp.stricted;
        firstRestricted = tmp.firstRestricted;
        if (tmp.message) {
            message = tmp.message;
        }

        previousStrict = strict;
        body = parseFunctionSourceElements();
        if (strict && firstRestricted) {
            throwUnexpectedToken(firstRestricted, message);
        }
        if (strict && stricted) {
            tolerateUnexpectedToken(stricted, message);
        }
        strict = previousStrict;

        return node.finishFunctionExpression(id, params, defaults, body);
    }

    // 14 Program

    function parseSourceElement() {
        if (lookahead.type === Token.Keyword) {
            switch (lookahead.value) {
            case 'const':
            case 'let':
                return parseConstLetDeclaration(lookahead.value);
            case 'function':
                return parseFunctionDeclaration();
            default:
                return parseStatement();
            }
        }

        if (lookahead.type !== Token.EOF) {
            return parseStatement();
        }
    }

    function parseSourceElements() {
        var sourceElement, sourceElements = [], token, directive, firstRestricted;

        while (index < length) {
            token = lookahead;
            if (token.type !== Token.StringLiteral) {
                break;
            }

            sourceElement = parseSourceElement();
            sourceElements.push(sourceElement);
            if (sourceElement.expression.type !== Syntax.Literal) {
                // this is not directive
                break;
            }
            directive = source.slice(token.start + 1, token.end - 1);
            if (directive === 'use strict') {
                strict = true;
                if (firstRestricted) {
                    tolerateUnexpectedToken(firstRestricted, Messages.StrictOctalLiteral);
                }
            } else {
                if (!firstRestricted && token.octal) {
                    firstRestricted = token;
                }
            }
        }

        while (index < length) {
            sourceElement = parseSourceElement();
            /* istanbul ignore if */
            if (typeof sourceElement === 'undefined') {
                break;
            }
            sourceElements.push(sourceElement);
        }
        return sourceElements;
    }

    function parseProgram() {
        var body, node;

        skipComment();
        peek();
        node = new Node();
        strict = false;

        body = parseSourceElements();
        return node.finishProgram(body);
    }

    function filterTokenLocation() {
        var i, entry, token, tokens = [];

        for (i = 0; i < extra.tokens.length; ++i) {
            entry = extra.tokens[i];
            token = {
                type: entry.type,
                value: entry.value
            };
            if (entry.regex) {
                token.regex = {
                    pattern: entry.regex.pattern,
                    flags: entry.regex.flags
                };
            }
            if (extra.range) {
                token.range = entry.range;
            }
            if (extra.loc) {
                token.loc = entry.loc;
            }
            tokens.push(token);
        }

        extra.tokens = tokens;
    }

    function tokenize(code, options) {
        var toString,
            tokens;

        toString = String;
        if (typeof code !== 'string' && !(code instanceof String)) {
            code = toString(code);
        }

        source = code;
        index = 0;
        lineNumber = (source.length > 0) ? 1 : 0;
        lineStart = 0;
        length = source.length;
        lookahead = null;
        state = {
            allowIn: true,
            labelSet: {},
            inFunctionBody: false,
            inIteration: false,
            inSwitch: false,
            lastCommentStart: -1
        };

        extra = {};

        // Options matching.
        options = options || {};

        // Of course we collect tokens here.
        options.tokens = true;
        extra.tokens = [];
        extra.tokenize = true;
        // The following two fields are necessary to compute the Regex tokens.
        extra.openParenToken = -1;
        extra.openCurlyToken = -1;

        extra.range = (typeof options.range === 'boolean') && options.range;
        extra.loc = (typeof options.loc === 'boolean') && options.loc;

        if (typeof options.comment === 'boolean' && options.comment) {
            extra.comments = [];
        }
        if (typeof options.tolerant === 'boolean' && options.tolerant) {
            extra.errors = [];
        }

        try {
            peek();
            if (lookahead.type === Token.EOF) {
                return extra.tokens;
            }

            lex();
            while (lookahead.type !== Token.EOF) {
                try {
                    lex();
                } catch (lexError) {
                    if (extra.errors) {
                        extra.errors.push(lexError);
                        // We have to break on the first error
                        // to avoid infinite loops.
                        break;
                    } else {
                        throw lexError;
                    }
                }
            }

            filterTokenLocation();
            tokens = extra.tokens;
            if (typeof extra.comments !== 'undefined') {
                tokens.comments = extra.comments;
            }
            if (typeof extra.errors !== 'undefined') {
                tokens.errors = extra.errors;
            }
        } catch (e) {
            throw e;
        } finally {
            extra = {};
        }
        return tokens;
    }

    function parse(code, options) {
        var program, toString;

        toString = String;
        if (typeof code !== 'string' && !(code instanceof String)) {
            code = toString(code);
        }

        source = code;
        index = 0;
        lineNumber = (source.length > 0) ? 1 : 0;
        lineStart = 0;
        length = source.length;
        lookahead = null;
        state = {
            allowIn: true,
            labelSet: {},
            parenthesisCount: 0,
            inFunctionBody: false,
            inIteration: false,
            inSwitch: false,
            lastCommentStart: -1
        };

        extra = {};
        if (typeof options !== 'undefined') {
            extra.range = (typeof options.range === 'boolean') && options.range;
            extra.loc = (typeof options.loc === 'boolean') && options.loc;
            extra.attachComment = (typeof options.attachComment === 'boolean') && options.attachComment;

            if (extra.loc && options.source !== null && options.source !== undefined) {
                extra.source = toString(options.source);
            }

            if (typeof options.tokens === 'boolean' && options.tokens) {
                extra.tokens = [];
            }
            if (typeof options.comment === 'boolean' && options.comment) {
                extra.comments = [];
            }
            if (typeof options.tolerant === 'boolean' && options.tolerant) {
                extra.errors = [];
            }
            if (extra.attachComment) {
                extra.range = true;
                extra.comments = [];
                extra.bottomRightStack = [];
                extra.trailingComments = [];
                extra.leadingComments = [];
            }
        }

        try {
            program = parseProgram();
            if (typeof extra.comments !== 'undefined') {
                program.comments = extra.comments;
            }
            if (typeof extra.tokens !== 'undefined') {
                filterTokenLocation();
                program.tokens = extra.tokens;
            }
            if (typeof extra.errors !== 'undefined') {
                program.errors = extra.errors;
            }
        } catch (e) {
            throw e;
        } finally {
            extra = {};
        }

        return program;
    }

    // Sync with *.json manifests.
    exports.version = '2.0.0';

    exports.tokenize = tokenize;

    exports.parse = parse;

    // Deep copy.
   /* istanbul ignore next */
    exports.Syntax = (function () {
        var name, types = {};

        if (typeof Object.create === 'function') {
            types = Object.create(null);
        }

        for (name in Syntax) {
            if (Syntax.hasOwnProperty(name)) {
                types[name] = Syntax[name];
            }
        }

        if (typeof Object.freeze === 'function') {
            Object.freeze(types);
        }

        return types;
    }());

}));
/* vim: set sw=4 ts=4 et tw=80 : */


// FILE: assets/js/factories/DataSourceManagerFactory.js
function DataSourceManagerFactory(metaDataFactory, ScreenService, templateManager, PeriodsUtilities) {
	var MODEL_LOG_WIDGET = {
		"name": "importLog",
		"label": "Log de importação",
		"template": "widget/form.html",
		"isVisible": false,
		"actions": [
			{
				"name": "back",
				"label": "Back",
				"icon": "back",
				"isVisible": true,
				"showOnForm": true,
				"showAsAction": "back",
				"events": [
					{
						"name": "ActionEvent",
						"code": "DataSourceManagerController.closeImportLog(args.owner.widget);"
					}
				]
			}
		],
		"fields": [
			{
				"name": "logField",
				"isVisible": true,
				"readOnly": true,
				"template": "field/textarea.html",
				"textRows": 8,
				"class": "12"
			}
		],
		"events": [],
		"dataSource": {},
		"widgets": []
	};

	var MODEL_PERIOD_WIDGET = {
		"class": 8,
		"name": "period",
		"label": "Período",
		"showOnForm": true,
		"isVisible": true,
		"template": "zh-interval-component#field/interval.html",
		"dateFormat": "DD/MM/YYYY",
		"displayFormat": "DD/MM/YYYY",
		"intervals": []
	};

	var MODEL_FILTER_WIDGET = {
		"name": "importFilterWidget",
		"label": "Filtro",
		"isVisible": true,
		"template": "widget/form.html",
		"events": [
			{
				"name": "WidgetOnEnter",
				"code": "DataSourceManagerController.setFilterWidgetLastActiveWidget(args.owner);"
			}
		],
		"actions": [
			{
				"label": "Filter",
				"name": "Filter",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"showAsAction": "never",
				"events": [
					{
						"name": "ActionEvent",
						"code": "DataSourceManagerController.filterDataSourceImportHistory(args.owner.widget); ScreenService.closePopup(); args.owner.parent.activate();"
					}
				]
			},
			{
				"name": "cancel",
				"label": "Cancel",
				"showAsAction": "back",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [
					{
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}
				]
			}
		],
		"fields": []
	};

	var MODEL_IMPORT_NOW_WIDGET = {
		"name": "importNow",
		"label": "Importar agora",
		"isVisible": true,
		"template": "widget/form.html",
		"events": [],
		"actions": [
			{
				"label": "Import",
				"name": "import",
				"isVisible": true,
				"showOnForm": true,
				"showAsAction": "never",
				"events": [
					{
						"name": "ActionEvent",
						"code": "ImportSchedulingController.checkImportExistence(args.owner.widget);"
					}
				]
			},
			{
				"name": "cancel",
				"label": "Cancel",
				"showAsAction": "back",
				"isVisible": true,
				"showOnForm": true,
				"events": [
					{
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}
				]
			}
		],
		"fields": [
			{
				"name": "persistType",
				"label": "Type",
				"class": 8,
				"isVisible": true,
				"showOnForm": true,
				"template": "field/select-native.html",
				"validations": {
					"required": {}
				},
				"hideEmptyValue": true,
				"breakline": true,
				"descriptionField": "name",
				"valueField": "value",
				"dataSource": {
					"data": [
						{
							"name": "Drop By Period",
							"value": "dropByPeriod"
						},
						{
							"name": "Concat",
							"value": "concat"
						},
						{
							"name": "Drop",
							"value": "drop"
						}
					]
				},
				"events": []
			}
		]
	};

	this.factoryLogWidget = function (parent) {
		return metaDataFactory.widgetFactory(MODEL_LOG_WIDGET, parent);
	};

	this.factoryFilterWidget = function (parent) {
		var widget = metaDataFactory.widgetFactory(MODEL_FILTER_WIDGET, parent);
		metaDataFactory.fieldFactory(MODEL_PERIOD_WIDGET, widget, true);
		return widget;
	};

	this.factoryImportNowWidget = function (parent) {
		var widget = metaDataFactory.widgetFactory(MODEL_IMPORT_NOW_WIDGET, parent);
		var period = metaDataFactory.fieldFactory(MODEL_PERIOD_WIDGET, widget, false);
		PeriodsUtilities.setIntervalsOnPeriodField(period);

		widget.fields.unshift(period);
		return widget;
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('DataSourceManagerFactory', DataSourceManagerFactory);
});


// FILE: assets/js/factories/GenericDashboardFactory.js
function GenericDashboardFactory(UtilitiesController, metaDataFactory) {
    var self = this;

    this.ACCORDION_TEMPLATE = "zmart-components#zmart-accordion.html";
    this.TAB_DASHBOARD_TEMPLATE = "widget/tabbedWidget.html";

    var DEFAULT_HEADER_COLOR = "orange";

    this.factoryDashboardTabWidget = function(name, label, order, isPreview) {
        isPreview = isPreview ? true : false;
        return {
            "id": name,
            "name": name,
            "label": label,
            "order": order,
            "floatingControl": false,
            "isVisible": true,
            "template": "widget/dashboard.html",
            "widgets": [],
            "events": [
                {
                    "name": "WidgetOnEnterTab",
                    "code": "GenericDashboardController.activeDashboardSearch(" + isPreview + ");"
                }
            ],
            "actions": isPreview ? [] : [
                {
                    "name": "buildIndicators",
                    "tooltip": "Reload",
                    "icon": "loading",
                    "isVisible": true,
                    "showAsAction": "always_list",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "GenericDashboardController.activeDashboardSearch(null,true);"
                        }
                    ]
                },
                {
                    "name": "filter",
                    "icon": "filter",
                    "tooltip": "Filter",
                    "showAsAction": "always",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "GenericDashboardController.openFilterWidget();"
                        }
                    ]
                },
                {
                    "name": "back",
                    "label": "Back",
                    "showAsAction": "back",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "GenericDashboardController.actionEventBack();"
                        }
                    ]
                }
            ]
        };
    };

    this.factoryDashboardAccordionHeader = function(id, name, label, color, dataSource, image, icon, gradient) {
        return {
            "id": id,
            "name": name,
            "label": label,
            "color": color,
            "active": false,
            "showForm": true,
            "opened": false,
            "isVisible": true,
            "widgets": [],
            "headerWidgets": [],
            "dataSource": dataSource,
            "image": image,
            "icon": icon,
            "gradient": gradient
        };
    };

    this.buildAccordion = function(dashboard){
        var accordion = this.buildAccordionHeader(dashboard);
        
        if(dashboard.indicators){
            dashboard.indicators.forEach(function(indicator){
                if(indicator.displayMode === 'header'){
                    accordion.headerWidgets.push(indicator);
                } else {
                    accordion.widgets.push(indicator);
                }
            });
        }

        accordion.widgets = accordion.widgets.sort(UtilitiesController.sortByOrder);
        return accordion;
    };

    this.buildAccordionHeader = function(dashboard){
        var id = dashboard._id || null;
        var label = dashboard.name || null;
        var color = DEFAULT_HEADER_COLOR;
        var image = null;
        var icon = null;
        var gradients = null;
        var dataSource = {
            "data" : []
        };

        if (dashboard.accordionHeaderProperties) {
            color = dashboard.accordionHeaderProperties.color || color;
            image = dashboard.accordionHeaderProperties.image || image;
            icon = dashboard.accordionHeaderProperties.icon || icon;
            dataSource = dashboard.accordionHeaderProperties.dataSource || dataSource;
            gradients = dashboard.accordionHeaderProperties.gradient ? [color, dashboard.accordionHeaderProperties.gradient] : gradients;
        }

        var accordionHeader = self.factoryDashboardAccordionHeader(id, id, label, color, dataSource, image, icon, gradients);

        return accordionHeader;
    };


    this.factoryDashboardAccordionActions = function(tabbedWidget, container) {
       if(tabbedWidget.opened){
            if(tabbedWidget.actions.length === 0 && tabbedWidget.opened){
                metaDataFactory.actionFactory({
                    "tooltip": "Reload",
                    "name": "reload",
                    "isVisible": true,
                    "icon": "loading",
                    "showAsAction": "always_list",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "GenericDashboardController.reloadWidgetAccordion(args.owner);"
                        }
                    ]
                }, tabbedWidget, true);

                metaDataFactory.actionFactory({
                    "tooltip": "Filter",
                    "name": "filter",
                    "isVisible": true,
                    "icon": "filter",
                    "showAsAction": "always_list",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "GenericDashboardController.addFilterAction();"
                        },
                        {
                            "name": "ActionEvent",
                            "code": "GenericDashboardController.openFilterWidget();"
                        }

                    ]
                }, tabbedWidget, true);
                container.showFooter = true;
            } else if(tabbedWidget.opened){
                container.showFooter = true;
            } else {
                container.showFooter = false;
            }
       }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GenericDashboardFactory', GenericDashboardFactory);
});

// FILE: assets/js/factories/GenericFieldFactory.js
function GenericFieldFactory(metaDataFactory) {
    var metaDataFieldText = function(name,label,fixed,isDimension,mask) {
        return {
            "name": name,
            "label": label,
            "template": "field/text-edit.html",
            "isGenerated": true,
            "fixed": fixed,
            "showOnList": true,
            "mask": isDimension?null:mask,
            "align": isDimension?"left":"right",
            "showOnForm": true,
            "showOnView": true,
            "events": [
                {
                    "name": "FieldOnClick",
                    "code": "DrillDownController.fieldFieldOnClickCrosstab(args);",
                    "id": "1644186869794193203136"
                }
            ],
            "isVisible": true
        };
    };

	this.factoryFieldText = function (name,label,widget,fixed,isDimension,mask) {
		return metaDataFactory.fieldFactory(metaDataFieldText(name,label,fixed,isDimension,mask), widget);
	};


}

Configuration(function(ContextRegister){
	ContextRegister.register('GenericFieldFactory', GenericFieldFactory);
});


// FILE: assets/js/factories/IndicatorFactory.js
function IndicatorFactory(metaDataFactory, ScreenService, ZHPromise, Query, WidgetRepository) {

	var self = this;

	this.getWidget = function(name) {
		if (name) {
			var query = Query.build().where('name').equals(name);
			return WidgetRepository.find(query);
		} else {
			return ZHPromise.reject({
				message: ScreenService.i18n("Widget name missing.")
			});
		}
	};

	this.addToContainer = function(widget, container, factory) {
		if (widget && container && container instanceof Container) {
			var deferred = ZHPromise.defer();
			Util.doAsync(function() {
				if (!container.widgets) {
					container.widgets = [];
				}
				container.widgets.push(factory ? self.factoryWidget(widget, container) : widget);
				deferred.resolve(container);
			});
			return deferred.promise;
		} else {
			return ZHPromise.reject({
				message: ScreenService.i18n('Invalid parameters. Should be passed a widget and a instance of Container.')
			});
		}
	};

	this.factoryWidget = function(widget, container) {
		if (widget) {
			if (container) {
				widget.container = container;
			}
			return metaDataFactory.widgetFactory(widget, container);
		} else {
			throw "Invalid widget.";
		}
	};

	function getSetBaseOrgModel(action) {
		var MODEL_SETBASE_ORGANIZATION = {
			"name": "configImportIndicator",
			"label": "Set origin organization",
			"isVisible": true,
			"template": "widget/form.html",
			"events": [],
			"actions": [{
				"label": "Import",
				"name": "import",
				"isVisible": true,
				"showOnForm": true,
				"showAsAction": "never",
				"events": [{
					"name": "ActionEvent",
					"code": action
				}]
			}, {
				"name": "cancel",
				"label": "Cancel",
				"showAsAction": "back",
				"isVisible": true,
				"showOnForm": true,
				"events": [{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
				}]
			}],
			"fields": [{
				"name": "baseConnection",
				"label": "Base Connections",
				"class": 12,
				"isVisible": true,
				"showOnForm": true,
				"template": "field/text-edit.html",
				"events": []
			}]
		};

		return MODEL_SETBASE_ORGANIZATION;
	}

	this.factoryImportNowWidget = function (parent, object) {
		var action;
		if(object === 'indicator'){
			action = "IndicatorController.openImportIndicators(args.owner.widget);";
		} else if(object === 'dashboard') {
			action = "DashboardController.openImportDashboard(args.owner.widget);";
		}

		var modelWidget = getSetBaseOrgModel(action);
		var widget = metaDataFactory.widgetFactory(modelWidget, parent);
		return widget;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('IndicatorFactory', IndicatorFactory);
});

// FILE: assets/js/factories/NotificationFactory.js
var NotificationFactory = function(metaDataFactory) {
    
    this.createNotificationPopup = function(container) {
        var notificationPopup = getNotificationPopup();
        return metaDataFactory.widgetFactory(notificationPopup, container);
    };

    var getNotificationPopup = function() {
        return {
            'name': 'notification',
            'label': 'Send Notification',
            'isVisible': true,
            'template': 'widget/form.html',
            'fields': [
                {
                    "name": "to",
                    "label": "To",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "template": "field/select-multiple.html",
                    "mobile": {
                        "showSelectAs": "list"
                    },
                    "class": 12,
                    "descriptionField": "NMOPERADOR",
                    "valueField": "DSEMAILOPER",
                    "dataSource": {
                        "name": "/zmartOperator",
                        "rest": true,
                        "data": []
                    },
                    "validations": {
                        "required": {}
                    }
                },
                {
                    'id': '16441830241971350668690',
                    'name': 'message',
                    "template": "field/textarea.html",
                    'label': 'Message',
                    'isVisible': true,
                    'class': '12',
                    'textRows': 5
                }
            ],
            'actions': [
                {
                    'id': '16441868489112692846124',
                    'label': 'Send',
                    'name': 'sendnotification',
                    'isVisible': true,
                    'showAsAction': 'never',
                    'events': [
                        {
                            'id': '16441868682308117471259',
                            'name': 'ActionEvent',
                            'code': 'ChartViewController.sendNotificationIndicator(args.owner.widget);'
                        }
                    ]
                },
                {
                    'id': '16441868212692846489114',
                    'label': 'Cancel',
                    'name': 'cancelnotification',
                    'isVisible': true,
                    'showAsAction': 'back',
                    'events': [
                        {
                            'id': '16441862308687478111259',
                            'name': 'ActionEvent',
                            'code': 'ScreenService.closePopup()'
                        }
                    ]
                }
            ]
        };
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('NotificationFactory', NotificationFactory);
});

// FILE: assets/js/factories/ViewPropertyFactory.js
var CUSTOM_FIELD_FUNCTIONS = {
	convertStringToArray: function(widget, field) {
			var fieldValue = widget.currentRow[field.name];
			if(typeof fieldValue === "string") {
				fieldValue = fieldValue.replace(" ", "");
				fieldValue = fieldValue.split(",");
			}
			widget.currentRow[field.name] = fieldValue;
	},
	convertArrayToString: function(widget, field) {
		var fieldValue = widget.currentRow[field.name];
		if(typeof fieldValue === "object") {
			fieldValue = fieldValue.join(",");
		}
		widget.currentRow[field.name] = fieldValue;
	}
};

function ViewPropertyFactory(metaDataFactory, ScreenService, templateManager, PeriodsUtilities) {

	var MODEL_FIELD_OBJECT = {
		label: "Field property",
		template: "field/text-edit.html",
		showOnForm: true,
		showOnList: false,
		isVisible: false,
		class: 6,
		dataSource: {
			data: []
		}
	};

	var MODEL_WIDGET_OBJECT = {
		"isVisible": true,
		"template": "widget/form.html",
		"events": [],
		"actions": [
			{
				"label": "Save",
				"name": "save",
				"isVisible": true,
				"showAsAction": "never",
				"events": [
					{
						"name": "ActionEvent",
						"code": "args.owner.widget.addItemAndClearForm(args.row, args.owner.widget.parentField);"
					}
				]
			},
			{
				"label": "Cancel",
				"showAsAction": "back",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [
					{
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}
				]
			}
		],
		"fields": []
	};

	var CUSTOM_FACTORIES_FLOW = {
		"field/select.html": factorySelect,
		"field/select-multiple.html": factorySelectMultiple,
		"field/color.html": factoryColor,
		"field/color-multiple.html": factoryColor
	};

	var DEFAULT_NAME = "fieldProperty";
	var LAST_ID = 0;

	var SELECT_SUFIX = "_select";

	function setEvents(field) {
		field.events = [{
			"name": "FieldOnAdd",
			"code": "ScreenService.openPopup(args.owner.field.formWidget)"
		}];
	}

	function setEventsFieldColor(field) {
		field.events = [{
			"name": "FieldOnChange",
			"code": "args.owner.field.convertColors(args.owner.field, args.owner.field.value());"
		}];
	}

	function convertColors(field, colors) {
		if (Util.isArray(colors)) {
			colors = colors.map(function(color) {
				return color.toHexString();
			});
			field.setValue(colors);
		} else if (colors) {
			colors = tinycolor(colors).toHexString();
			field.setValue(colors);
		}
	}

	function factoryColor(field) {
		setEventsFieldColor(field);
		field.convertColors = convertColors;
	}

	function factoryWidgetToAdd(field, parentWidget) {
		if (field.showAddButtonAs == 'action') {
			var form = Util.clone(MODEL_WIDGET_OBJECT);
			form.name = field.name + '_form';
			form.label = ScreenService.i18n('Add') + ScreenService.i18n(field.label);
			form.fields = field.fields;
			form.addItemAndClearForm = addItemAndClearForm;
			form.parentField = field;

			setEvents(field);

			if (!parentWidget.selectMultipleFieldsWithCRUD) parentWidget.selectMultipleFieldsWithCRUD = [];
			if (field.isSelectMultipleObject) parentWidget.selectMultipleFieldsWithCRUD.push(field.name);

			if (field.fields) {
				field.fields.forEach(function(childField) {
					if (CUSTOM_FACTORIES_FLOW[childField.template]) {
						CUSTOM_FACTORIES_FLOW[childField.template](childField);
					}
				});
			}

			field.formWidget = metaDataFactory.widgetFactory(form, parentWidget.container);
		}
	}

	function addItemAndClearForm(row, field) {
		addItemToDataSource(row, field);
		clearFormData(row);
	}

	function clearFormData(row) {
		for (var idx in row) {
			row[idx] = "";
		}
	}

	function addItemToDataSource(row, field) {
		delete row.changed;
		var dataSource = field.dataSource;
		if (!Util.isArray(dataSource.data)) {
			dataSource.data = [];
		}

		var values = Util.clone(row);
		if (field.isSelectMultipleObject) {
			values[field.valueField] = Util.clone(row);
		}

		dataSource.data.push(values);
		dataSource.addCheckedRows(values);

		templateManager.updateTemplate();
		ScreenService.closePopup();
	}

	function setColumnsData(field, dimensions, facts, allColumns) {
		if (field.isColumn) {
			if (field.isDimension) {
				field.dataSource.data = Util.clone(dimensions);
			} else if (field.isFact) {
				field.dataSource.data = Util.clone(facts);
			} else {
				field.dataSource.data = Util.clone(allColumns);
			}
		}
	}

	function factorySelect(field, parentWidget, dimensions, facts, allColumns) {
		setColumnsData(field, dimensions, facts, allColumns);
		factoryWidgetToAdd(field, parentWidget);
		field.outData = {};
		field.outData[field.valueField] = field.name;
		field.name += SELECT_SUFIX;
		field.outData[field.descriptionField] = field.name;
	}

	function factorySelectMultiple(field, parentWidget, dimensions, facts, allColumns) {
		setColumnsData(field, dimensions, facts, allColumns);
		factoryWidgetToAdd(field, parentWidget);
	}

	function factoryFieldFromProperty(property, parentWidget, dimensions, facts, allColumns) {
		var field = metaDataFactory.factoryField(Util.extend(MODEL_FIELD_OBJECT, property, true), parentWidget, true);

		if (CUSTOM_FACTORIES_FLOW[field.template]) {
			CUSTOM_FACTORIES_FLOW[field.template](field, parentWidget, dimensions, facts, allColumns);
		}

		field.id = field.id || ++LAST_ID;

		return field;
	}

	this.factory = function(property, parentWidget, dataSourceColumns) {
		property.help = ScreenService.i18n(property.help);

		var dimensions = [],
			facts = [];
		dataSourceColumns.forEach(function(column) {
			if (column.isDimension) {
				dimensions.push(column);
			} else if (column.isFact) {
				facts.push(column);
			}
		});

		return factoryFieldFromProperty(property, parentWidget, dimensions, facts, dataSourceColumns);
	};

	this.loadSelectMultipleEditableValues = function(row, widget) {
		if(widget.selectMultipleFieldsWithCRUD && Util.isArray(widget.selectMultipleFieldsWithCRUD)) {
			widget.selectMultipleFieldsWithCRUD.forEach(function(fieldName) {
				var field = widget.getField(fieldName);
				var values = row[fieldName];
				if (values) {
					var dataSource = field.dataSource;
					dataSource.data = values.map(function(value) {
						var clone = Util.clone(value);
						clone[field.valueField] = value;
						return clone;
					});
					dataSource.addCheckedRows(dataSource.data);
					templateManager.updateTemplate();
				}
			});
		}
	};

	this.getIntervalField = function(periodNumber, viewWidget, appendToWidget) {
		return this.getIntervalFieldObj(periodNumber).then(function(intervalField){
			return metaDataFactory.factoryField(intervalField, viewWidget, appendToWidget);
		});
	};

	this.buildIntervalField = function(periodNumber, intervals, viewWidget, appendToWidget){
		return metaDataFactory.factoryField(intervalField(periodNumber, intervals), viewWidget, appendToWidget);
	};

	this.getTrashLabelWithIconField = function(periodNumber, viewWidget, appendToWidget, controllerCall) {
		return metaDataFactory.factoryField(this.getTrashLabelWithIconObj(periodNumber, controllerCall), viewWidget, appendToWidget);
	};

	this.getTrashLabelWithIconObj = function(periodNumber, controllerCall) {
		return {
			"name": "remove_" + periodNumber,
			"label": null,
			"isVisible": true,
			"showOnForm": true,
			"showOnList": false,
			"showOnView": false,
			"template": "field/label-with-icon.html",
			"icon": "trash",
			"fieldGroup": "period",
			"class": "4 zmart-pull-right",
			"relatedPeriod": periodNumber,
			"events": [
				{
					"name": "FieldOnClick",
					"code": controllerCall + ".removeInterval(args.owner.field.widget, args.owner.field.relatedPeriod);"
				}
			]
		};
	};

	this.getIntervalFieldObj = function(periodNumber) {
		return PeriodsUtilities.getFormatedIntervals().then(function(formatedIntervals){
			return intervalField(periodNumber, formatedIntervals);
		});
	};

	function intervalField(periodNumber, intervals){
		intervals = intervals || [];
		return {
			"name": "period_" + periodNumber,
			"label": "Período",
			"class": 8,
			"showOnForm": true,
			"showOnView": true,
			"showOnList": false,
			"isVisible": true,
			"dateFormat": "DD/MM/YYYY",
			"displayFormat": "DD/MM/YYYY",
			"template": "zh-interval-component#field/interval.html",
			"fieldGroup": "period",
			"events": [],
			"intervals": intervals
		};
	}
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ViewPropertyFactory', ViewPropertyFactory);
});


// FILE: assets/js/factories/ZmartLoginFactory.js
function ZmartLoginFactory(metaDataFactory){
    
    var MODEL_PASSWORD_RECOVER_WIDGET = {
		"name": "PASSWORD_RECOVER",
		"label": "Forgot your password?",
		"isVisible": true,
		"template": "widget/form.html",
		"events": [],
		"actions": [
			{
				"label": "Recover password",
				"name": "RECOVER_PASSWORD",
				"isVisible": true,
				"showOnForm": true,
				"showAsAction": "never",
				"events": [
					{
						"name": "ActionEvent",
						"code": "ZmartLoginController.recoverPassword(args.owner.widget);"
					}
				]
			},
			{
				"name": "CANCEL",
				"label": "Cancel",
				"showAsAction": "back",
				"isVisible": true,
				"showOnForm": true,
				"events": [
					{
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}
				]
			}
		],
		"fields": [
			{
                "name": "EMAIL",
                "label": "Email",
                "class": 12,
                "isReadOnly": false,
                "isVisible": true,
                "isVisibleForm": true,
                "isVisibleGrid": false,
                "newLine": false,
                "defaultValue": null,
                "template": "field/text.html",
                "events": []
            }
		]
	};
	
	this.getForgotPasswordWidget = function(container){
	    return metaDataFactory.widgetFactory(MODEL_PASSWORD_RECOVER_WIDGET, container);
	};
}
Configuration(function (ContextRegister) {
	ContextRegister.register('ZmartLoginFactory', ZmartLoginFactory);
});

// FILE: assets/js/services/ChartService.js
var ChartService = function(templateManager) {
    templateManager.zhFeaturesWebSocketUrl = window.zhFeaturesWebSocketUrl;
    var dataSourcesDisplayed = [];
    var pendingDataSources = [];

    this.updateDataSourcesDisplayed = function(widgets) {
        this.clearDataSourcesDisplayed();
        this.registerDashboardWidgets(widgets);
    };

    this.addDataSourceDisplayed = function(widget) {
        if (!dataSourcesDisplayed[widget.dataSourceId]) {
            dataSourcesDisplayed[widget.dataSourceId] = [];
        }

        dataSourcesDisplayed[widget.dataSourceId].push(widget);
    };

    this.registerDashboardWidgets = function(widgets) {
        widgets.forEach((function(widget) {
            this.addDataSourceDisplayed(widget);
        }).bind(this));
    };

    this.clearDataSourcesDisplayed = function() {
        dataSourcesDisplayed = {};
    };

    this.getWidgetsByDataSourceId = function(dataSourceId) {
        return dataSourcesDisplayed[dataSourceId] || [];
    };

    this.clearPendingDataSources = function() {
        pendingDataSources = [];
    };

    this.addPendingDataSources = function(dataSourceId) {
        pendingDataSources.push(dataSourceId);
    };

    this.getPendingDataSources = function() {
        return pendingDataSources;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ChartService', ChartService);
});

// FILE: assets/js/services/AccessControlService.js
function AccessControlService(restEngine) {

    var self = this;

    self.sync = function () {
        return restEngine.post("/zhu_acl_element_rules/sync", undefined, undefined, undefined, undefined, undefined, 90000);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('AccessControlService', AccessControlService);
});

// FILE: assets/js/services/DataSourceService.js
function DataSourceService(Query, DataSourceRepository, UtilitiesController, restEngine) {

	this.findOneByIndicator = function(indicator) {
		return restEngine.post("/dataSource/findByIndicator", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "indicatorId",
					"operator": "=",
					"value": indicator
					}
				]
		});
	};

	this.find = function(_id) {
		return UtilitiesController.callRepositoryMethod(
			DataSourceRepository.find,
			DataSourceRepository,
			DataSourceRepository.name, [Query.build().where('_id').equals(_id)]
		).then(function(dataSource) {
			return Util.isArray(dataSource) ? dataSource[0] : undefined;
		});
	};

}
Configuration(function(ContextRegister) {
	ContextRegister.register('DataSourceService', DataSourceService);
});

// FILE: assets/js/services/FilterService.js
function FilterService(templateManager, localStorageEngine, Query) {
	
	this.filterIndicators = function(){
		var filterWidget = templateManager.container.getWidget('filterWidget');
		var indicatorField = filterWidget.getField('indicator_label');
		var dataSourceFilter = Query.build().where('isActive').equals(true);
		var parentMenuId = localStorageEngine.getLocalVar('parentMenuId');
		if (parentMenuId) {
			dataSourceFilter.where('menu').equals(parentMenuId);
		}
		indicatorField.dataSourceFilter = dataSourceFilter.where();
		return indicatorField.reload();
	};	
}

Configuration(function(ContextRegister){
	ContextRegister.register('FilterService', FilterService);
});

// FILE: assets/js/services/StructureService.js
function StructureService(restEngine) {

	this.getFieldKeysFromQuery = function (row) {
		return restEngine.post("\/structureType\/getFieldKeysFromQuery",
			{
				"requestType": "Row",
				"row": row
			}
		);
	};

	this.saveTooManyStructures = function (rows) {
		return restEngine.post("\/structure\/save",
			{
				"requestType": "DataSet",
				"dataset": rows
			}
		);
	};

	this.generateHierarchies = function (row, relations) {
		row.fieldKeysRelation = relations;
		return restEngine.post("\/hierarchy\/generateHierarchies",
			{
				"requestType": "Row",
				"row": row
			}
		);
	};

	this.getStructuresFromQuery = function (structureTypeRow, structureData) {
		return restEngine.post("\/structure\/getStructuresFromQuery",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "type",
						"operator": "=",
						"value": structureTypeRow
					},
					{
						"name": "structures",
						"operator": "IN",
						"value": structureData
					}
				]
			}
		);
	};

	this.getManyStructuresFromQuery = function(types){
		return restEngine.post("\/structure\/getManyStructuresFromQuery",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "types",
						"operator": "=",
						"value": types
					}
				]
			}
		);
	};

    this.getStructDrillInfoByName = function(structureName, indicatorId) {
        return restEngine.post("\/structure\/getStructDrillInfoByName",
			{
				"requestType": "Row",
				"row": {
                    structureName: structureName,
                    indicatorId: indicatorId
                }
			}
		);
	};

	this.getStructureByUser = function(fullAccessProfiles) {
    	return restEngine.post("/getStructureByUser",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "fullAccessProfiles",
						"operator": "=",
						"value": fullAccessProfiles
					}
				]
			}
		);
    };
    
    this.getUserStructuresByIndicator = function(indicatorId) {
    	return restEngine.post("/getUserStructuresByIndicator",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "indicator",
						"operator": "=",
						"value": indicatorId
					}
				]
			}
		);
    };

}
Configuration(function (ContextRegister) {
	ContextRegister.register('StructureService', StructureService);
});

// FILE: assets/js/services/ViewPropertyService.js
function ViewPropertyService(ViewPropertyRepository, UtilitiesController) {

	this.findAll = function() {
		return UtilitiesController.callRepositoryMethod(
			ViewPropertyRepository.findAll,
			ViewPropertyRepository,
			ViewPropertyRepository.name
		).then(function(viewProperties){
			var hasOrderProperties = [];
			var hasNoOrderProperties = [];
			viewProperties.forEach(function(viewProperty){
				if(viewProperty.order){
					hasOrderProperties.push(viewProperty);
				} else {
					hasNoOrderProperties.push(viewProperty);
				}
			});
			
			hasOrderProperties.sort(function(a, b) {
				return a.order - b.order;
			});
			
			var newViewProperties = hasOrderProperties.concat(hasNoOrderProperties);
			return newViewProperties;
		});
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ViewPropertyService', ViewPropertyService);
});

// FILE: assets/js/services/ViewTypeService.js
function ViewTypeService(ViewTypeRepository, UtilitiesController) {

	this.findAllWithProperties = function() {
		return UtilitiesController.callRepositoryMethod(
			ViewTypeRepository.findAllWithProperties,
			ViewTypeRepository,
			ViewTypeRepository.name
		);
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ViewTypeService', ViewTypeService);
});

// FILE: assets/js/services/WidgetService.js
function WidgetService(ZHPromise, Query, WidgetRepository, UtilitiesController, ViewPropertyFactory, ViewPropertyService, templateManager,
	DataSourceService, restEngine, ScreenService, $rootScope) {

	var viewPropertiesField = {};
	var idSerieTypeBar = "58826350788e6171fd911eb2";
	var idSerieTypeLine = "588265de788e6171fd911ec3";
	var observationNotificationList = [];
	var self=this;

	function clearFieldPropertiesHistory() {
		viewPropertiesField = {};
		var viewWidget = templateManager.container.getWidget('view');
		removeDinamicFields(viewWidget);
	}
	
	function removeDinamicFields(viewWidget){
		var lastField = viewWidget.getField('dateFilterColumnLabel');
		var lastPosition = viewWidget.fields.indexOf(lastField) + 1;
		viewWidget.fields.splice(lastPosition);
	}

	this.findViewsByIndicator = function(indicator) {
		return UtilitiesController.callRepositoryMethod(
			WidgetRepository.find,
			WidgetRepository,
			WidgetRepository.name, [Query.build().where('indicator').equals(indicator)]
		);
	};

	// find chartType and chartProperties and factory all fields
	this.buildViewWidgetProperties = function(chartTypeField, viewWidget, dataSourceId, clearProperties) {
		if(clearProperties){
			clearFieldPropertiesHistory();
		}
	
		return ZHPromise.all([
			DataSourceService.find(dataSourceId),
			ViewPropertyService.findAll()
		]).then(function(responses) {
			var chartProperties = responses[1];
			if (Util.isArray(chartProperties)) {
				var dataSourceColumns = responses[0] && responses[0].columns ? responses[0].columns : [];
				chartProperties.forEach(function(property) {
					viewPropertiesField[property._id] = ViewPropertyFactory.factory(property, viewWidget, dataSourceColumns);
				});
				templateManager.updateTemplate();
			}
			return viewPropertiesField;
		});
	};
	
	this.validateSerieType = function (propertyId){
		viewPropertiesField[propertyId].isVisible = false;
	};
	
	this.buildFieldstProperties = function (propertiesId, fieldGroup) {
		Util.parseToArray(propertiesId).forEach(function(propertyId) {
			viewPropertiesField[propertyId].isVisible = true;
			viewPropertiesField[propertyId].showOnForm = true;
			viewPropertiesField[propertyId].class = 6;
			viewPropertiesField[propertyId].fieldGroup = fieldGroup;
			viewPropertiesField[propertyId].invisible = false;
		});
		
		templateManager.updateTemplate();
	};

	this.enableFields = function(propertiesId) {
		var precision = false;
		Util.parseToArray(propertiesId).forEach(function(propertyId) {
			if (typeof viewPropertiesField[propertyId] != "undefined"){
				viewPropertiesField[propertyId].isVisible = Util.isUndefined(viewPropertiesField[propertyId].invisible) ? true : !viewPropertiesField[propertyId].invisible;
				if (viewPropertiesField[propertyId].name == "serieYAxisFormat" || viewPropertiesField[propertyId].name == "serieValueFormat")
						if (viewPropertiesField[propertyId].value() == "currency" || viewPropertiesField[propertyId].value() == "float" || viewPropertiesField[propertyId].value() == "percent")
								precision = true;
			}
		});
		
		if (precision)
			this.enablePrecisionField(propertiesId,precision);
		
		templateManager.updateTemplate();
	};
	
	this.enablePrecisionField = function(propertiesId, isVisible) {
		Util.parseToArray(propertiesId).forEach(function(propertyId) {
			if (viewPropertiesField[propertyId].name == "precision"){
				viewPropertiesField[propertyId].isVisible = isVisible;
			}
		});
	};
	
	this.checkFieldToEnable = function(propertiesId){
		var isVisible = false;
		Util.parseToArray(propertiesId).forEach(
			function(propertyId) {
			if (viewPropertiesField[propertyId].name == "serieYAxisFormat" || viewPropertiesField[propertyId].name == "serieValueFormat"){
				if (viewPropertiesField[propertyId].value() == "currency" || viewPropertiesField[propertyId].value() == "float" || viewPropertiesField[propertyId].value() == "percent"){
					isVisible = true;
				}
			}
		});
		return isVisible;
	};
	
	this.disableFields = function(propertiesId) {
		propertiesId = Util.parseToArray(propertiesId);
		if (!propertiesId.length) {
			propertiesId = Object.keys(viewPropertiesField);
		}
		propertiesId.forEach(function(propertyId) {
			viewPropertiesField[propertyId].isVisible = false;
		});
		
		templateManager.updateTemplate();
	};
	
	this.changeVisibilityFieldGroup = function(fieldGroups, groups, visible) {
		groups.forEach(function(group){
			fieldGroups.forEach(function(fieldGroup){
				if(fieldGroup.name === group){
					fieldGroup.isVisible = visible;
				}
			});
		});
	};
	
	this.saveInserver = function(data) {
		return WidgetRepository.saveInServer(Util.parseToArray(data));
	};

	this.buildWidgetByViewId = function(row) {
		self.removeObservationNotifications();
		return UtilitiesController.callRepositoryMethod(
			restEngine.post,
			restEngine,
			null, [{
				service: "/widget/buildWidgetByViewId",
				typeRequest: "POST",
				params: { "requestType": "Row", "row": row }
			}]
		).then(function(widgets){
			if(widgets && widgets.periodObservationNotification && widgets.periodObservationNotification.length > 0){
				self.showObservationNotifications(widgets.periodObservationNotification);
			}
			if (widgets && Util.isArray(widgets.widget)) {
				return widgets.widget.shift();
			}
		});
	};

	this.showObservationNotifications = function(observations){
		observations.forEach(function(obs){
			UtilitiesController.notificationMessage(obs,"alert",1000000);
			$rootScope.notificationMessages.forEach(function(message){
				if(message.text == obs){
					observationNotificationList.push(message);
				}
			});
		});
	};

	this.removeObservationNotifications = function(){
		if(observationNotificationList && observationNotificationList.length > 0){
			while(observationNotificationList.length > 0){
				UtilitiesController.removeMessage(observationNotificationList.pop());
			}
		}
	};

	this.fetchWidgetData = function(row,widget) {
        if(widget && widget.reload){
            widget.dataSourceFilter =  [
                {
                    "name": "row",
                    "operator": "=",
                    "value": (Util.clone(row) || {})
                }
            ];
            return widget.reload();
        }else{
            return restEngine.post({
                service: "/view/fetchWidgetData",
                typeRequest: "POST",
                params: {
                    "requestType": "FilterData",
                    "filter": [
                        {
                            "name": "row",
                            "operator": "=",
                            "value": (Util.clone(row) || {})
                        }
                    ]
                },
                timeout: 120000
            });
        }
	};
	
	this.findWidgetById = function(viewId) {
		return restEngine.post("/widget", 
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "_id",
						"operator": "=",
						"value": viewId
					}
				]
			}
		);
	};
}
Configuration(function(ContextRegister) {
	ContextRegister.register('WidgetService', WidgetService);
});

// FILE: assets/js/services/StyleService.js
function StyleService(restEngine) {

	var self = this;
	var orgCssStyles;

	function getCssStyles(){
		return restEngine.post("/style",
			{
				"requestType": "FilterData",
				"filter": []
			}
		);
	}

    this.loadCustomCssStyles = function(){
        self.removeAppliedCustomStyles();
        return getCssStyles().then(function(response){
            var cssStyles = {};
            response.dataset.style.forEach(function(style){
                cssStyles[style.formatedName] = style.cssCode;
            });

            orgCssStyles = cssStyles;
        });
    };

	this.getGridStyles = function(indicatorId) {
        return restEngine.post("/gridStyle",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "indicator",
						"operator": "=",
						"value": indicatorId
					}
				]
			}
		);
    };

    function checkAppliedCustomCss(styleName){
        var styles = $('head style[type="text/css"]');
        styles.each(function(index, style) {
            if (style.id.indexOf(styleName) === 0) {
                return true;
            }
        });
    }

    this.applyCustomStyle = function(styleName, custom) {
        if(!checkAppliedCustomCss(styleName) && orgCssStyles && orgCssStyles[styleName]) {
            var style = document.createElement('style');
            style.type = 'text/css';
            style.innerHTML = orgCssStyles[styleName];
            style.id = custom ? "custom_" + styleName : styleName;
            document.getElementsByTagName('head')[0].appendChild(style);
        }
    };

    this.removeAppliedCustomStyles = function(styleName) {
        var styles = $('head style[type="text/css"]');
        var styleToRemove = styleName ? styleName : "custom_";

        styles.each(function(index, style) {
            if (style.id.indexOf(styleToRemove) === 0) {
                styleToRemove = style;
                styleToRemove.parentNode.removeChild(styleToRemove);
            }
        });
    };

    //gridStyle
    this.getGridsRelatedToAlteredStyle = function(styleId){
        return restEngine.post("/gridStyle",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "styleId",
						"operator": "=",
						"value": styleId
					}
				]
			}
		);
    };

    this.persistStyleUpdatesToGrids = function(rows){
        restEngine.post("/gridStyle/save",
			{
				"requestType": "DataSet",
				"dataset": rows
			}
		);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('StyleService', StyleService);
});

// FILE: assets/js/services/ChartViewService.js
function ChartViewService(ZHPromise, WidgetService, templateManager, IndicatorFactory, ScreenService,
restEngine, StyleService, UtilitiesController, $location, localStorageEngine, DataSourceService) {

	var self = this;
	var ZMART_LIST_TEMPLATE = "zmart-components#zmart-list.html";
	var DEFAULT_GRID_LIST_TEMPLATE = "zmart-list";
	var reloadTimeout = null;

	var reloadAfterSchedule = function(widget){
		clearTimeout(reloadTimeout);
		restEngine.post("/scheduleMiliseconds", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "_id",
					"operator": "=",
					"value": widget.dataSourceId
					}
				]
		}).then(function(miliseconds){
			if(miliseconds.dataset.dataset[0] > 0){
				reloadTimeout = Util.doAsync(function(){
					if(templateManager.container.widgets && templateManager.container.widgets[1] && templateManager.container.widgets[1]._id && templateManager.container.widgets[1]._id == widget._id){
						reloadAfterSchedule(widget);
						widget.reload();
					}
				},miliseconds.dataset.dataset[0]);
			}
		});
	};

	this.buildIndicatorView = function(viewId, filterRow) {
		if(viewId) {
			if(getLocalVar('previousConditions')){
				Object.assign(filterRow.drillDownFilters,getPreviousOrPostConditionsForDrillDown(getLocalVar('previousConditions')));
				setLocalVar("previousConditions",null);
			}
			if(getLocalVar('postConditions')){
				Object.assign(filterRow.drillDownFilters,getPreviousOrPostConditionsForDrillDown(getLocalVar('postConditions')));
				setLocalVar("postConditions",null);
			}
			return WidgetService.buildWidgetByViewId(filterRow).then(function(widget) {
				if (widget) {
					widget = self.buildWidget(widget);
					reloadAfterSchedule(widget);
					return StyleService.loadCustomCssStyles().then(function(){
						if(widget.serieType === 'grid'){
							applyCssStyle(widget);

							if(widget.template === ZMART_LIST_TEMPLATE){
								var widgetClass = widget.class && typeof widget.class === 'string' ? widget.class : DEFAULT_GRID_LIST_TEMPLATE;
								StyleService.applyCustomStyle(widgetClass);
							}
						}

						var flag = true;
						for(var key in filterRow) {
							if(key !== "viewId") {
								flag = false;
							}
						}
						if(flag) {
							getDefaultFilterRow(widget, filterRow);
						}
						
					   	if(filterRow.period === null || filterRow.period === ""){
							widget.parameters.period = null;
						}
                        self.setWidgetConditions(widget.previousConditions, filterRow, 'previousConditions');
						self.setWidgetConditions(widget.postConditions, filterRow, 'postConditions');
						self.setWidgetConditions(widget.postGroupConditions, filterRow, 'postGroupConditions');

						return self.fetchWidgetData(widget, filterRow, filterRow.isViewPreview).then(function(response) {
							if(response && response.dataset && response.dataset.fetchWidgetData && response.dataset.fetchWidgetData.length > 0 && !!response.dataset.fetchWidgetData[0].PROCESS){
								widget.dataSource.name = "/resultQueryRunner";
								widget.dataSourceFilter = [
									{
										"name": "processQueryRunner",
										"operator": "=",
										"value": response.dataset.fetchWidgetData[0].PROCESS
									}
								];
								widget.dataSource.data = [];
							} else {
								widget.filterRow = filterRow;
								UtilitiesController.windowResizeKanban();
							}
							return widget;
						});
					});
				} else {
					return ZHPromise.reject({ data: { error: ScreenService.i18n("View not found!") }});
				}
			});
		}
	};

	var getPreviousOrPostConditionsForDrillDown = function(conditions){
		var filter = {};
		if(conditions){
			if(conditions.operator == "&&" || conditions.operator == "and"){
				var first = getPreviousOrPostConditionsForDrillDown(conditions.firstValue);
				var second = getPreviousOrPostConditionsForDrillDown(conditions.secondValue);
				Object.assign(filter,first,second);
			}else{
				filter[conditions.firstValue] = conditions.secondValue;
			}
		}
		return filter;
	};

	this.getImportStatusText = function(importProcess){
        var status =  'c';
        var subprocesses = null;
        var hasError = null;
        if(importProcess){
            status =  importProcess.status;
            subprocesses =  importProcess.subprocesses;
            hasError = importProcess.hasError;
        }

        if (hasError) {
            return "error";
        } else if (status === 's') {
            return 'Saving documents...';
        } else if (status === 100) {
            return "100%";
        } else if (status === 'c') {
            return "0%";
        } else if ((status > 0 && status < 100) || (status >= 0 && status < 100 && subprocesses > 0)) {
           return status + '%';
        } else if(status === 0 && subprocesses === 0) {
            return 'Starting Processes...';
        }
		return "--";
    };

    this.setColorToPie = function (widget){
        var color = 0;
        var colors = widget.chart.updateSerieOptions(0).colors;
        var pieData = widget.chart.updateSerieOptions(0).pieData;
        if(widget.specificColor && pieData){
            widget.chartSeries[0].serieColors = colors;
            widget.serieColors = colors;
            pieData.forEach(function(data){
                for(var j=0;j<widget.specificColor.length;j++){
                    var e = widget.specificColor[j];
                    if(((data[widget.chartSeries[0].serieFieldLabel] == null && e[widget.chartSeries[0].serieFieldLabel] == "(sem valor)")|| data[widget.chartSeries[0].serieFieldLabel] == e[widget.chartSeries[0].serieFieldLabel]) && e.color){
                        widget.chartSeries[0].serieColors[color] ="#"+e.color;
                        widget.serieColors[color] ="#"+e.color;
                    }
                }
                color++;
            });
            widget.updateChart();
        }
    };

	this.buildWidget = function(widgetObj) {
		templateManager.container.label = widgetObj.name || '';
		widgetObj.isVisible = true;

		var widget = IndicatorFactory.factoryWidget(widgetObj, templateManager.container);

		IndicatorFactory.addToContainer(widget, templateManager.container).then(function() {
			templateManager.updateTemplate();
		});

		return widget;
	};

    this.applyCssStyleCrossTab = function(widget){
        return applyCssStyle(widget);
    };

	function applyCssStyle(widget) {
		return StyleService.getGridStyles(widget.indicator).then(function(response) {
			var gridStyles = response.dataset.gridStyle;
            if(widget.isCrossTab){
                var fact = widget.parameters.factValue;
                gridStyles.forEach(function(gridStyle){
                    if(Array.isArray(gridStyle.styleFields) &&
                        -1 != gridStyle.styleFields.indexOf(fact)){
                        gridStyle.styleFields = gridStyle.styleFields.concat(widget.namefieldsFact);
                    }
                    if(-1 != gridStyle.condition.indexOf(fact)){
                        var condition = '('+gridStyle.condition+')';
                        var addCondition = condition;
                        gridStyle.conditionCrossTabFields = {};
                        widget.namefieldsFact.forEach(function(nameFact){
                            var conditionFact = condition.replace(fact,nameFact);
                            addCondition += '||' + conditionFact;
                            gridStyle.conditionCrossTabFields[nameFact] = conditionFact;
                        });
                        gridStyle.condition = addCondition;
                    }
                });
            }
			gridStyles.forEach(function(gridStyle) {
				var styleName = gridStyle.styleName.replace( /\s/g, '_' );
				StyleService.applyCustomStyle(styleName, true);
			});

			var fieldStyles = gridStyles.filter(function(style) {
				return style.styleFields && style.styleFields.length;
			});
			var rowStyles = gridStyles.filter(function(style) {
				return !style.styleFields || style.styleFields.length === 0;
			});

			widget.styles = rowStyles;

			widget.fields.forEach(function(field) {
				field.styles = [];
			});

            fieldStyles.forEach(function (styles) {
                styles.styleFields.forEach(function (fieldName) {
                    var fieldToApplyStyle = widget.getField(fieldName);
                    if (fieldToApplyStyle) {
                        if (widget.isCrossTab && styles.conditionCrossTabFields[fieldName]) {
                            var fieldStyle = angular.copy(styles);
                            fieldStyle.condition = fieldStyle.conditionCrossTabFields[fieldName];
                            fieldToApplyStyle.styles.push(fieldStyle);
                        } else {
                            fieldToApplyStyle.styles.push(styles);
                        }
                    }
                });
            });
		});
	}

	this.setWidgetConditions = function(conditions, filterRow, previousOrPost) {
		if (conditions) {
			if (filterRow && filterRow[previousOrPost]) {
				if (!Util.isEmptyObject(filterRow[previousOrPost])) {
					var newConditions = {};
					newConditions.firstValue = filterRow[previousOrPost];
					newConditions.operator = "&&";
					newConditions.secondValue = conditions;

					filterRow[previousOrPost] = newConditions;
				}
				else {
					filterRow[previousOrPost] = [filterRow[previousOrPost], conditions];
				}
			}
			else {
				filterRow[previousOrPost] = conditions;
			}
		}
	};

    this.processQueryRunnerAfterLoad = function(widget){
        if(widget.dataSource.name == "/resultQueryRunner"){
            afterFetchWidgetData(widget.dataSource.data, widget, null);
        }
    };

	this.fetchWidgetData = function(widget, row, isViewPreview, isStructureFromFilter) {
		row.structureId = row.structureId || widget.parameters.structureId;
		row.isStructureFromFilter = isStructureFromFilter;
		row.period = (isViewPreview && widget.mockPeriod) ? widget.mockPeriod : widget.parameters.period;
		row.zmartTotalized = !!widget.zmartTotalized ? widget.zmartTotalized : false;
		return WidgetService.fetchWidgetData(row,widget).then(function(response){
            var data = response && response.dataset && response.dataset.fetchWidgetData ? response.dataset.fetchWidgetData : [];
            return afterFetchWidgetData(data, widget, response);
        });
	};

    var afterFetchWidgetData = function(data, widget, response){
        self.setTitleWidgetOnEnter();
        if (Util.isArray(data)) {
            if(widget.convertToNumber) {
                if(widget.chart && data[0] && !data[0].PROCESS) {
                    var chartValue = data.length > 0 ? data[0][widget[widget.factProperties[0]]] : '';
                    widget.chart.value(chartValue).updateSeries();
                    widget.chartSeries[0].serieValue = chartValue;
                } else if(widget.chartSeries && widget.chartSeries[0] && widget.chartSeries[0].serieType) {
                    widget.chartSeries[0][widget.factProperties[0]] = data.length > 0 ? data[0][widget[widget.factProperties[0]]] : 0;
                } else {
                    widget[widget.factProperties[0]] = data.length > 0 ? data[0][widget[widget.factProperties[0]]] : 0;
                }
            } else if(!widget.isCrossTab){
                widget.dataSource.data = data;
                fetchSpecialWidgetData(widget, data);
            }

            if (widget.serieType == "gauge") {
                var chartSeries = widget.chartSeries[0];
                if (data.length == 0 || !!data[0].PROCESS) {
                    if (widget.chart) {
                        widget.chart.updateSerieOptions(0, {ranges: []});
                    }
                    chartSeries.serieRanges = [];
                }else {
                    chartSeries.serieMaxValue = chartSeries.zmartSerieMaxValue;
                    chartSeries.serieMinValue = chartSeries.zmartSerieMinValue;
                    if (typeof chartSeries.serieMaxValue == "string") {
                        var maxValue = data.length > 0 ? data[0][chartSeries.serieMaxValue] : '';
                        chartSeries.serieMaxValue = maxValue;
                    }
                    if (chartSeries.zmartSerieRanges && Array.isArray(chartSeries.zmartSerieRanges) && typeof chartSeries.zmartSerieRanges[0] == "string") {
                        var rangeValue;
                        for (var i = 0; i < chartSeries.zmartSerieRanges.length; i++) {
                            rangeValue = data.length > 0 ? data[0][chartSeries.zmartSerieRanges[i]] : '';
                            chartSeries.zmartSerieRanges[i] = rangeValue;
                        }
                        chartSeries.zmartSerieRanges.push(chartSeries.serieMaxValue);
                        chartSeries.zmartSerieRanges.sort(function (a, b) {
                            return a - b;
                        });
                        chartSeries.serieRanges = chartSeries.zmartSerieRanges;
                    }
                    chartSeries.serieGaugeShowLegend = true;
                    if(widget.chart){
                        widget.chart.updateSerieOptions(0, {value: chartSeries.serieValue});
                        widget.chart.updateSerieOptions(0, {ranges: chartSeries.serieRanges});
                        widget.chart.updateSerieOptions(0, {maxValue: chartSeries.serieMaxValue});
                        widget.chart.updateSerieOptions(0, {minValue: chartSeries.serieMinValue});
                        widget.chart.updateSerieOptions(0, {gaugeShowLegend: true});
                        widget.updateChart();

                    }
                }
            }
        }

        if (response && response.messages) {
            var messages = response.messages;
            UtilitiesController.showMessage(messages[0].message, messages[0].type, undefined, messages[0].variables);
        }
        return response;
    };

	function getDefaultFilterRow(widget, filterRow) {
		filterRow.viewId = widget._id;
		filterRow.structureId = widget.parameters.structureId;
		filterRow.period = widget.parameters.period;
	}

	this.sendNotification = function(row){
		return restEngine.post('/notification/sendNotificationIndicatorMail', {
			"requestType": "Row",
			"row":row
		});
	};

	function fetchSpecialWidgetData(widget, dataSource){
		var PROGRESS_DETAIL_TEMPLATE = "zmart-components#dashboard-card-4.html";

		switch (widget.template) {
			case PROGRESS_DETAIL_TEMPLATE:
				fetchProgressDetailData(widget, dataSource);
				break;
		}
	}

	function fetchProgressDetailData(widget, dataSource) {
		widget.widgets.forEach(function(detailWidget){
			detailWidget.dataSource.data = [];
			dataSource.forEach(function(data){
				detailWidget.progressParameters.maxValue = data[detailWidget.progressParameters.maxValue];
				detailWidget.progressParameters.value = data[detailWidget.progressParameters.value];

				for (var value in data){
					var pushData = {};
					if(detailWidget.fieldValue.indexOf(value) !== -1){
						pushData.label = getLabelFromValue(detailWidget.fieldValue, detailWidget.fieldValueLabel, value);
						pushData.value = data[value];
						detailWidget.dataSource.data.push(pushData);
					}
				}
			});
		});
	}

	function getLabelFromValue(values, labels, searchValue) {
		var index = values.indexOf(searchValue);
		return (index !== -1) ? labels[index] : searchValue;
	}

	this.getLastImportationDates = function(dataSourcesIds){
		return restEngine.post('/importHistory/getLastImportationDates', {
			"requestType": "Row",
			"row": {
				indicatorIds: dataSourcesIds
			}
		}).then(function(response){
			return response.dataset.lastImportDates;
		});
	};

	this.setHeaderInfoFromWidgetParameters = function (widget) {
    	var parameters = widget.parameters;
        var headerInfo = '';
        var indicatorName = parameters.indicatorName;

        if (indicatorName) {
        	headerInfo = ScreenService.i18n('Indicator') + ': ' + indicatorName;

        	if(widget.lastImportation){
        		headerInfo += ' | ' + ScreenService.i18n('Updated in') + ': ' + widget.lastImportation;
        	}
        }

        ScreenService.setHeaderInfo(headerInfo);
    };

    this.setTitleWidgetOnEnter = function(){
        var titleName = templateManager.container.label;
        var updatedTitle = ScreenService.i18n(titleName) + ' - Zmart';
        templateManager.updateTitle(updatedTitle);
    };

    self.setIndicatorNamePreview = function () {
		UtilitiesController.hashToSearch();
		if(getLocalVar('indicatorNames')===undefined){
			localStorageEngine.setLocalVar('indicatorNames',{});
		}
		localStorageEngine.setLocalVar('indicatorName',$location.$$search.Name);
	};

    this.setIndicatorName = function(){
        ScreenService.showLoader();
        return loginPromise.then(function(){
            ScreenService.hideLoader();
            if(getLocalVar('indicatorNames')===undefined){
                localStorageEngine.setLocalVar('indicatorNames',{});
            }
            if($location.$$search.Name === undefined || getLocalVar('indicatorNames')[$location.$$search.Name] === undefined)
            {
                UtilitiesController.hashToSearch();
                var menu = {
                    name:$location.$$search.Name
				};
				if(!templateManager.container.preview) {
					templateManager.interations.openIndicatorView(menu);
				}
			}
            localStorageEngine.setLocalVar('indicatorName',$location.$$search.Name);

        });

    };

    this.loadFail = function(response) {
        if (response && response.data && response.data.error) {
        	if(!(response.data.errorCode &&
				response.data.errorCode === 20 &&
				response.data.errorCode === 100 &&
				response.data.errorCode === 101)){
				UtilitiesController.showMessage(ScreenService.i18n(response.data.error));
			}
        }
    };

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ChartViewService', ChartViewService);
});

// FILE: assets/js/services/ViewService.js
var constants = {
	INDICATOR_VIEW_ID: "indicatorView",
	MENU_ID: "menuId",
	DRILLDOWN_PARAMETERS: "drilldownParameters",
	INDICATOR_VIEW_WINDOW: "chartView",
	DASHBORAD_WINDOW: "genericDashboard"
};

var urlConstants = {
	STRUCTURE_ID: "sId",
	STRUCTURE_NAME: "sName",
    DRILL_STRUCTURE_ID: "drillSId",
    DRILL_STRUCTURE_NAME: "drillSName",
	PERIOD_START: "pStart",
	PERIOD_END: "pEnd",
	DRILL_PROPERTY: "drP",
	DRILL_VALUE: "drV",
	IS_STRUCTURE_DRILL: "isSDr",
	GROUP_BY_STRUCTURE: "gByS",
	STRUCTURE_TYPE_ID: "sTId",
	VIEW_LEVEL: "vL",
	POST_CONDITIONS: "posCon",
	PREVIOUS_CONDITIONS: "preCon",
	FILTERS: "f"
};

var max = 999999999999999999999;

function ViewService(templateManager, restEngine, UtilitiesController, localStorageEngine,
                     ScreenService, ViewPropertyFactory, $location, PeriodsUtilities, ZHPromise) {

	var INDICATOR_GROUP_MENU = "indicators";
	var INDICATOR_GROUP_MENU_SUFIX = "_indicators";
	var DASHBOARD_MENU_SUFIX = "_dashboard";

	var INTERATIONS = {
		DASHBOARD: "openDashboard",
		INDICATOR_GROUP: "openIndicator",
		INDICATOR_VIEW: "openIndicatorView"
	};

	var self = this;

	var INTERVALS;

	this.findView = function(id) {
		return restEngine.post('/view', {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "_id",
					"operator": "=",
					"value": id
					}
				]
		});
	};

	this.loadAndBuildViewMenuHierarchy = function() {
		return loadViewMenuHierarchy().then(buildViewMenuHierarchy);
	};

	this.updateMenu = function() {
        if(templateManager.project.getMenu('indicators')) {
            templateManager.project.getMenu('indicators').menus = [];
        }
		this.loadAndBuildViewMenuHierarchy();
	};

	function loadViewMenuHierarchy() {
		return UtilitiesController.callRepositoryMethod(
			postRequest, null, "menuHierarchy", ["/view/menuHierarchy", {
				"requestType": "FilterData",
				"filter": []
			}]
		);
	}

	function postRequest(route, params) {
		return restEngine.post(route, params);
	}

	function genereteId(menus) {
		menus.forEach(function(menu){
			if(menu.menus.length > 0){
				genereteId(menu.menus);
			}
			menu.id = Math.floor(Math.random() * max);
		});
	}

	function buildViewMenuHierarchy(menus) {
		var menuTree = [];
		if (Util.isArray(menus) && menus.length) {
			menus = factoryMenus(menus);
			genereteId(menus);
			menuTree = buildMenuTree(menus);
		}
		templateManager.project.addMenus(menuTree);
	}

	function factoryMenus(menus) {
		return menus.map(function(menu) {
			var menuItem = factoryMenu(menu._id, menu.name, (menu.parent || INDICATOR_GROUP_MENU), menu.order);
			buildDashboardSubmenu(menuItem, menu.hasDashboards);
			buildIndicatorGroupSubmenu(menuItem, menu.indicators);

			return menuItem;
		});
	}

	function buildMenuTree(allMenus, parents, menusTree) {
		if ((Util.isArray(parents) && parents.length)) {
			for (var i in parents) {
				var parent = parents[i];
				var menus = parent.menus.concat(getLevelMenus(allMenus, parent));

				parent.menus = orderMenus(menus);
				buildMenuTree(allMenus, parent.menus, menusTree);
			}
		} else if (!menusTree || menusTree.length === 0) {
			menusTree = orderMenus(getLevelMenus(allMenus, {
				name: INDICATOR_GROUP_MENU
			}));
			buildMenuTree(allMenus, menusTree, menusTree);
		}
		return menusTree;
	}

	function orderMenus(menus) {
		return menus.sort(UtilitiesController.sortByOrder);
	}

	function getLevelMenus(allMenus, parent) {
		return allMenus.filter(function(menu) {
			return parent.name === menu.parent;
		});
	}

	function buildDashboardSubmenu(menu, hasDashboards) {
		if (hasDashboards) {
			var dashboardMenuItem = factoryMenu(menu.name + DASHBOARD_MENU_SUFIX, "Dashboards", menu.name);
			addEvent(dashboardMenuItem, INTERATIONS.DASHBOARD);

			menu.menus.push(dashboardMenuItem);
		}
	}

	function buildIndicatorGroupSubmenu(menu, indicators) {
		if (Util.isArray(indicators) && indicators.length) {
			indicators = setIndicatorsWhenItIsNotMenu(indicators);
			var menus = menu.menus.concat(indicators.map(function(indicator) {
				var menuItem = factoryMenu(indicator._id, indicator.name, menu.name, indicator.order);

				if (indicator.isView){
					addEvent(menuItem, INTERATIONS.INDICATOR_VIEW);
					menuItem.menus = [];
				} else if (indicator.views.length === 1) {
					var view = indicator.views[0];
					menuItem.firstIndicator = factoryMenu(view._id, view.name);
					addEvent(menuItem, INTERATIONS.INDICATOR_GROUP);
				} else {
					menuItem.menus = indicator.views.map(function(view) {
						var viewMenuItem = factoryMenu(view._id, view.name, menuItem.name, view.order);
						addEvent(viewMenuItem, INTERATIONS.INDICATOR_VIEW);
						return viewMenuItem;
					}).sort(UtilitiesController.sortByOrder);
				}

				return menuItem;
			}));

			menu.menus = menus.sort(UtilitiesController.sortByOrder);
		}
	}

	function setIndicatorsWhenItIsNotMenu(indicators) {
		var menuIndicators = indicators;
		if (indicators.length === 1 && indicators[0].ignoreFirstMenuLevel) {
			menuIndicators = indicators[0].views.map(function(indicator) {
				indicator.views = [];
				indicator.isView = true;
				return indicator;
			});
		}

		return menuIndicators;
	}

	function factoryMenu(name, label, parent, order) {
		return {
			name: name,
			label: ScreenService.i18n(label),
			parent: parent,
			isVisible: true,
			icon: "project",
			aclIsVisible: true,
			order: (order || null),
			menus: []
		};
	}

	function addEvent(menu, method) {
		menu.events = [{
			name: "MenuOnClick",
			code: "templateManager.interations." + method + "(args.owner)"
		}];
		return menu;
	}

	templateManager.interations = {};

	templateManager.interations[INTERATIONS.DASHBOARD] = function(menu) {
		localStorageEngine.setLocalVar(ViewService.MENU_ID, menu.parent.name);

		if (templateManager.container.name === ViewService.DASHBORAD_WINDOW) {
			templateManager.container.onRefreshDashboard();
		} else {
			ScreenService.openWindow(ViewService.DASHBORAD_WINDOW);
		}

		ScreenService.toggleSideMenu();
	};

	templateManager.interations[INTERATIONS.INDICATOR_GROUP] = function(menu) {
		if (menu.firstIndicator) {
			templateManager.interations[INTERATIONS.INDICATOR_VIEW](menu.firstIndicator);
		}
	};

	templateManager.interations[INTERATIONS.INDICATOR_VIEW] = function(menu) {
		UtilitiesController.killAllAsyncProcesses();
		if(getLocalVar("filterWidget") && getLocalVar("filterWidget")[getLocalVar('indicatorName')]){
			var filterWidget = getLocalVar("filterWidget");
			delete filterWidget[getLocalVar('indicatorName')];
			setLocalVar("filterWidget",filterWidget);
		}
		localStorageEngine.setLocalVar(ViewService.INDICATOR_VIEW_ID, menu.name);
		var defaultParameters = self.mapDefaultParameters();
		var blankParameters = defaultParameters.reduce(function(prev, cur) {
			return prev + (cur ? '=' + '&' + cur : '');
		}) + '=';
		var search = urlConstants.VIEW_LEVEL + '=0&' + constants.INDICATOR_VIEW_ID + '=' + menu.name;
		search = search + '&' + blankParameters;

        var indicatorNames = getLocalVar("indicatorNames");
        if(indicatorNames === undefined){
            indicatorNames = {};
        }
        indicatorNames[menu.name] = $location.search(search).$$search;
        localStorageEngine.setLocalVar("indicatorNames",indicatorNames);
        $location.path( '/product#' + constants.INDICATOR_VIEW_WINDOW).search('Name=' + menu.name).hash('');
        if(ScreenService.isSideMenuOpen()){
            ScreenService.toggleSideMenu();
        }
	};

	this.mapDefaultParameters = function() {
		return Object.keys(urlConstants).map(function(constant) {
			if (urlConstants[constant] !== urlConstants.VIEW_LEVEL) {
				return urlConstants[constant];
			}
		});
	};

	this.findGeneratedMenu = function(widgetId){
		return restEngine.post('/view', {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "type",
					"operator": "=",
					"value": "G"
				},
				{
					"name": "widgetId",
					"operator": "=",
					"value": widgetId
				}
			]
		});
	};

	this.findDashboardPreview = function(id){
		return restEngine.post("/view/getGenericDashboardsViews", {
			"requestType": "Row",
			"row": {
				"name": id,
				"isViewPreview": true
			}
		});
	};


	this.saveView = function(row){
		return restEngine.post("/view/save",
			{
				"requestType": "DataSet",
				"dataset": row
			}
		);
	};

	this.deleteView = function(row){
		restEngine.post("/view/remove",
			{
				"requestType": "DataSet",
				"dataset": [row]
			}
		);
	};

	Util.extend(this, constants);
	Util.extend(this, urlConstants);
}

Util.extend(ViewService, constants);

Configuration(function(ContextRegister) {
	ContextRegister.register('ViewService', ViewService);
});

// FILE: assets/js/services/DrillDownService.js
function DrillDownService(UtilitiesController, ViewService, $location){
    this.getDrillDownParameters = function(viewLevel){
        var structureId = UtilitiesController.getUrlParamAtIndex(ViewService.STRUCTURE_ID, viewLevel);
        var structureName = UtilitiesController.getUrlParamAtIndex(ViewService.STRUCTURE_NAME, viewLevel);
        var drillstructureId = UtilitiesController.getUrlParamAtIndex(ViewService.DRILL_STRUCTURE_ID, viewLevel);
        var drillstructureName = UtilitiesController.getUrlParamAtIndex(ViewService.DRILL_STRUCTURE_NAME, viewLevel);
        var periodStart = UtilitiesController.getUrlParamAtIndex(ViewService.PERIOD_START, viewLevel);
        var periodEnd = UtilitiesController.getUrlParamAtIndex(ViewService.PERIOD_END, viewLevel);
	    var previousConditions = UtilitiesController.getUrlParamAtIndex(ViewService.PREVIOUS_CONDITIONS, viewLevel);
	    var postConditions = UtilitiesController.getUrlParamAtIndex(ViewService.POST_CONDITIONS, viewLevel);
        var drillDownFilters = {};
        var drillDownParameters = {};

		if ((getLocalVar('indicatorName') &&
            getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.DRILL_PROPERTY] &&
            getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.DRILL_VALUE] ) ||
            ($location.$$search[ViewService.DRILL_PROPERTY] && $location.$$search[ViewService.DRILL_VALUE])) {
			drillDownFilters = buildDrillDownFilters(viewLevel);
        }
        var searchParameters = getLocalVar('indicatorNames')[getLocalVar('indicatorName')];
        if(searchParameters && searchParameters[ViewService.FILTERS]){
            Object.assign(drillDownFilters,searchParameters[ViewService.FILTERS]);
        }

        if (drillDownFilters) {

	        drillDownParameters = {
                drillDownFilters: drillDownFilters,
		        structureId: drillstructureId|| structureId,
                structureName: drillstructureName || structureName

            };
	        if (periodStart && periodEnd) {
                drillDownParameters.period = {
                    start: periodStart,
                    end: periodEnd
                };
            }
	        if(previousConditions) {
		        drillDownParameters.previousConditions = previousConditions;
	        }
	        if(postConditions) {
		        drillDownParameters.postConditions = postConditions;
	        }
        }

		return drillDownParameters;
    };

    function buildDrillDownFilters (viewLevel) {
		var drillDownFilters = {};

        var drillProp;
        var searchParameters = getLocalVar('indicatorNames')[getLocalVar('indicatorName')];
        if (searchParameters === undefined){
            drillProp = $location.$$search[ViewService.DRILL_PROPERTY];
        }
        else{
            drillProp = searchParameters[ViewService.DRILL_PROPERTY];
        }

		var drillValue;
        if (searchParameters === undefined){
            drillValue = $location.$$search[ViewService.DRILL_VALUE];
        }
        else{
            drillValue = searchParameters[ViewService.DRILL_VALUE];
        }

		if (Util.isArray(drillProp) && Util.isArray(drillValue)) {
			for (var i in (drillProp)) {
				if (drillProp[i] !== "undefined" && drillValue[i] !== "undefined") {
					drillDownFilters[drillProp[i]] = drillValue[i];
				}
			}
		} else if (drillProp && drillValue) {
			drillDownFilters[drillProp] = drillValue;
        }

		return drillDownFilters;
	}
}

Configuration(function(ContextRegister) {
	ContextRegister.register('DrillDownService', DrillDownService);
});

// FILE: assets/js/services/ExpressionInterpreter.js
function ExpressionInterpreter() {

	var aggFunctions = [];
	var functionsExp = [];
	var logicalExp = [];

	var functions = {
		"SUM": {
			"name": "",
			"function": "SUM",
			"content": "",
			"id": 0
		},
		"AVG": {
			"name": "",
			"function": "AVG",
			"content": "",
			"id": 0
		},
		"MIN": {
			"name": "",
			"function": "MIN",
			"content": "",
			"id": 0
		},
		"MAX": {
			"name": "",
			"function": "MAX",
			"content": "",
			"id": 0
		},
		"COUNT": {
			"name": "",
			"function": "COUNT",
			"content": "",
			"id": 0
		}
	};

	var binaryExpressionObj = {
		"firstValue": "",
		"operator": "",
		"secondValue": "",
		"id": 0
	};

	var logicalExpressionObj = {
		"operator": "",
		"id": 0
	};

	var operators = ["+", "-", "*", "/"];
	var logicalOperators = ["==", ">", "<", "!=", ">=", "<="];

	var facts = [];
	var dimensions = [];
	var nonFactsColumns = [];
	var columns = [];
	var usedColumns = [];
	var fieldName;

	var functionToValidateColumn = validateAndReturnIsColumn;

	var hasFunctionAdded = false; // Flag: Prevents "function inside a function" error; On finish a function build, resetFlag;
	var hasAggFunctionAdded = false;

	function cleanObjects() {
		aggFunctions = [];
		functionsExp = [];
		logicalExp = [];

		functions = {
			"SUM": {
				"name": "",
				"function": "SUM",
				"content": "",
				"id": 0
			},
			"AVG": {
				"name": "",
				"function": "AVG",
				"content": "",
				"id": 0
			},
			"MIN": {
				"name": "",
				"function": "MIN",
				"content": "",
				"id": 0
			},
			"MAX": {
				"name": "",
				"function": "MAX",
				"content": "",
				"id": 0
			},
			"COUNT": {
				"name": "",
				"function": "COUNT",
				"content": "",
				"id": 0
			}
		};

		binaryExpressionObj = {
			"firstValue": "",
			"operator": "",
			"secondValue": "",
			"id": 0
		};

		logicalExpressionObj = {
			"operator": "",
			"id": 0
		};

		facts = [];
		dimensions = [];
		nonFactsColumns = [];
		columns = [];
		usedColumns = [];
		functionToValidateColumn = validateAndReturnIsColumn;
		fieldName = "";

		hasFunctionAdded = false; // Flag: Prevents "function inside a function" error; On finish a function build, resetFlag;
		hasAggFunctionAdded = false;
	}

	this.readExpression = function (expressionString, paramFacts, paramNonColumnFacts, paramColumnName) {
		cleanObjects();
		facts = paramFacts;
		nonFactsColumns = paramNonColumnFacts;
		fieldName = paramColumnName;
		var decodedExpression = esprima.parse(expressionString);
		var expression = decodedExpression.body[0].expression;
		buildExpression(expression);
		return {
			"functions": aggFunctions,
			"mathExpressions": functionsExp
		};
	};

	this.readComparisonExpression = function (expressionString, previousOrPost, paramColumns, paramUsedColumns) {
		cleanObjects();
		columns = paramColumns;
		usedColumns = paramUsedColumns;
		expressionString = expressionString.replace(/ AND | and /g, " && ");
		expressionString = expressionString.replace(/ OR  | or /g, " || ");
		if(previousOrPost === "post") {
			functionToValidateColumn = validateAndReturnIsColumnUsed;
		}

		var decodedExpression = esprima.parse(expressionString);
		var expression = decodedExpression.body[0].expression;
		return buildComparisonExpression(expression);
	};

	function buildExpression(expression) {
		var objectToReturn;
		if (expression.type === "BinaryExpression") {
			binaryExpressionObj.id++;
			var buildedObj = Util.clone(binaryExpressionObj);

			if(!hasFunctionAdded && !hasAggFunctionAdded) {
				hasAggFunctionAdded = true;
				buildedObj.name = fieldName;
				functionsExp.push(buildedObj);
			}

			getBinaryExpression(buildedObj, expression, buildExpression, validateAndReturnFact);
			objectToReturn = buildedObj;
		} else if (expression.type === "CallExpression") {
			if (!hasFunctionAdded) {
				objectToReturn = getFunctionExpression(expression);
			} else {
				throwFunctionInsideFunction();
			}
		} else {
			throwUnreconizedExpressionType(expression.type);
		}
		return objectToReturn;
	}

	function buildComparisonExpression(expression) {
		var objectToReturn;
		var buildedObj;

		var rightSide = getExpressionSide(expression, 'right', buildComparisonExpression, functionToValidateColumn);
		var leftSide = getExpressionSide(expression, 'left', buildComparisonExpression, functionToValidateColumn);

		if (expression.type === "LogicalExpression") {
			logicalExpressionObj.id++;
			buildedObj = Util.clone(logicalExpressionObj);
			if (expression.operator === "&&") {
				buildedObj.operator = expression.operator;
				buildedObj.firstValue = leftSide;
				buildedObj.secondValue = rightSide;
			} else if (expression.operator === "||") {
				buildedObj.operator = expression.operator;
				buildedObj.expressions = [];
				logicalOrConcatOrPush(buildedObj, leftSide);
				logicalOrConcatOrPush(buildedObj, rightSide);
			}
			objectToReturn = buildedObj;
		} else if (expression.type === "BinaryExpression") {
			binaryExpressionObj.id++;
			buildedObj = Util.clone(binaryExpressionObj);

			if (isValidLogicalOperator(expression.operator) || isValidOperator(expression.operator)) {
				buildedObj.operator = expression.operator;
			} else {
				throwInvalidLogicalOperator(expression.operator);
			}

			buildedObj.firstValue = leftSide;
			buildedObj.secondValue = rightSide;

			objectToReturn = buildedObj;
		} else {
			throwShouldBeALogicalExpression();
		}
		return objectToReturn;
	}

	function getBinaryExpression(buildedObj, expression, functionToDeep, functionToValidate) {
		buildedObj.operator = validateAndReturnOperator(expression.operator);
		buildedObj.firstValue = getExpressionSide(expression, 'left', functionToDeep, functionToValidate);
		buildedObj.secondValue = getExpressionSide(expression, 'right', functionToDeep, functionToValidate);
	}

	function getFunctionExpression(expression) {
		var functionName = expression.callee.name.toUpperCase();
		if (functions[functionName]) {
			hasFunctionAdded = true;
			functions[functionName].id++;
			var functionToAdd = Util.clone(functions[functionName]);
			if (!hasAggFunctionAdded) {
				functionToAdd.name = fieldName;
			} else {
				functionToAdd.name = functionToAdd.function + '_' + fieldName + '_' + functionToAdd.id;
			}
			functionToAdd.content = getFunctionContent(expression.arguments[0]);
			hasFunctionAdded = false;
			aggFunctions.push(functionToAdd);
			return functionToAdd;
		}
		else {
			throwUnmappedFunction(functionName);
		}
	}

	function getExpressionSide(expression, side, functionToDeep, functionToValidate) {
		var value;
		if (expression[side].type === "BinaryExpression" || expression[side].type === "LogicalExpression") {
			value = functionToDeep(expression[side]);
		} else if (expression[side].type === "CallExpression") {
			value = functionToDeep(expression[side]).name;
		} else if (expression[side].type === "Identifier") {
			value = functionToValidate(expression[side].name);
		} else if (expression[side].type === "Literal") {
			value = expression[side].value;
		} else {
			throwUnreconizedExpressionType(expression[side].type);
		}
		return value;
	}

	function getFunctionContent(argument) {
		var value;
		if (argument.type === "BinaryExpression") {
			value = buildExpression(argument);
		} else if (argument.type === "CallExpression") {
			value = buildExpression(argument).name;
		} else if (argument.type === "Identifier") {
			value = validateAndReturnFact(argument.name);
		} else if (argument.type === "Literal") {
			value = argument.value;
		}
		return value;
	}

	function logicalOrConcatOrPush(buildedObj, side) {
		if (side.operator == "||") {
			buildedObj.expressions = buildedObj.expressions.concat(side.expressions);
		} else {
			buildedObj.expressions.push(side);
		}
	}

	function validateAndReturnFact(value) {
		if (isFact(value)) {
			return value;
		}
		else {
			throwExpColumnError(value);
		}
	}

	function isFact(value) {
		return isInArray(facts, value);
	}

	function validateAndReturnOperator(operator) {
		if (isValidOperator(operator)) {
			return operator;
		}
		else {
			throwInvalidOperator(operator);
		}
	}

	function isValidOperator(operator) {
		return isInArray(operators, operator);
	}

	function validateAndReturnIsColumn(column) {
		if(isInArray(columns, column)) {
			return column;
		}
		else {
			throwShouldBeColumn(column);
		}
	}

	function validateAndReturnIsColumnUsed(column) {
		if(isInArray(usedColumns, column)) {
			return column;
		} else {
			throwShouldBeUsedColumn(column);
		}
	}

	function isValidLogicalOperator(operator) {
		return isInArray(logicalOperators, operator);
	}

	function isInArray(arr, element) {
		return !!(~arr.indexOf(element));
	}

	function throwExpColumnError(column) {
		if (nonFactsColumns.indexOf(column)) {
			throwExpressionWithNonFact(column);
		} else if (functions[column]) {
			throwFunctionInsideFunction(column);
		} else {
			throwUnreconizedWord(column);
		}
	}

	function throwUnreconizedWord(word) {
		throw "Unreconized Word: " + word;
	}

	function throwFunctionInsideFunction() {
		throw "Isn't possible build a function inside a function";
	}

	function throwExpressionWithNonFact(column) {
		throw "Column " + column + " should be a fact or a integer. Only operations with facts or integer are allowed.";
	}

	function throwUnmappedFunction(functionName) {
		throw "Unmapped function: " + functionName;
	}

	function throwUnreconizedExpressionType(expressionType) {
		throw "Unreconized Expression Type: " + expressionType;
	}

	function throwInvalidLogicalOperator(operator) {
		throw "Invalid Logical Operator: " + operator;
	}

	function throwInvalidOperator(operator) {
		throw "Invalid Operator: " + operator;
	}

	function throwShouldBeALogicalExpression() {
		throw "This expression should be a Logical Expression";
	}

	function throwShouldBeColumn(nonColumn) {
		throw "Only columns are allowed to comparise. Non column name: " + nonColumn;
	}

	function throwShouldBeUsedColumn(nonColumn) {
		throw "Only used columns in query are allowed to comparise. Column name: " + nonColumn;
	}

}

Configuration(function (ContextRegister) {
	ContextRegister.register('ExpressionInterpreter', ExpressionInterpreter);
});

// FILE: assets/js/services/GenericDashboardService.js
function GenericDashboardService(UtilitiesController, restEngine, ZHPromise, templateManager,
	IndicatorFactory, ScreenService, ChartViewService, localStorageEngine, $location,
	ViewService, DrillDownService, metaDataFactory, ExpressionInterpreter,
    GenericDashboardFactory, WidgetService, DataSourceService, MessageService) {

        var self = this;
        var reloadTimeout = [];

        this.LAST_ACTIVE_DASHBOARD = 'lastActiveDashboard';

        this.setLastActiveDashboard = function(dashboardName) {
            localStorageEngine.setLocalVar(this.LAST_ACTIVE_DASHBOARD, dashboardName);
        };

        this.getLastActiveDashboard = function() {
            var dashboardName = localStorageEngine.getLocalVar(this.LAST_ACTIVE_DASHBOARD, dashboardName);
            this.cleanLastActiveDashboard();
            return dashboardName;
        };

        this.cleanLastActiveDashboard = function() {
            localStorageEngine.setLocalVar(this.LAST_ACTIVE_DASHBOARD, '');
        };

        this.buildDashboards = function(menuId) {
            var viewLevel;
            if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined){
                viewLevel = +$location.$$search[ViewService.VIEW_LEVEL];
            }
            else{
                viewLevel = + getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.VIEW_LEVEL];
            }
            var filterParams = {
                menuId: menuId,
                drillDownParameters: {}
            };

            if (viewLevel >= 0){
                filterParams.drillDownParameters = DrillDownService.getDrillDownParameters(viewLevel) || filterParams.drillDownParameters;
            }

            if(filterParams.menuId) {
                return getDashboards(filterParams).then(treatBuildDashboardResponse);
            }
        };

        this.buildMainDashboards = function() {
            return getMainDashboards().then(treatBuildDashboardResponse);
        };

        this.buildGenericDashboards = function() {
            return getGenericDashboards().then(treatBuildDashboardResponse);
        };

        this.buildPreviewDashboards = function(id) {
            return getPreviewDashboards(id).then(treatBuildDashboardResponse);
        };

        this.removeNullField = function(fields){
            return fields.filter(function (el) {
                return el != null;
            });
        };

        var reloadAfterSchedule = function(widget, dashboardId){
            if(reloadTimeout[widget._id]){
                clearTimeout(reloadTimeout[widget._id]);
            }
            restEngine.post("/scheduleMiliseconds", {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "_id",
                        "operator": "=",
                        "value": widget.dataSourceId
                        }
                    ]
            }).then(function(miliseconds){
                if(miliseconds.dataset.dataset[0] > 0){
                    reloadTimeout[widget._id] = Util.doAsync(function(){
                       if(templateManager.container.getActiveWidget().id == dashboardId){
                            reloadAfterSchedule(widget, dashboardId);
                            widget.reload();
                        }
                    },miliseconds.dataset.dataset[0]);
                }
            });
        };

        function treatBuildDashboardResponse(dashboards) {
            if (Util.isArray(dashboards)) {
                _buildDashboards(dashboards);
                return dashboards;
            } else {
                return ZHPromise.reject({ data: { error: ScreenService.i18n("Dashboards not found!") }});
            }
        }

        function setInitialFilterFields(){
            var filterWidget = templateManager.container.getWidget('filterWidget');
            filterWidget.fields = self.removeNullField(filterWidget.fields);
            var periodField = filterWidget.getField("period");
            var structureField = filterWidget.getField("structure_label");
            var filtersField = filterWidget.getField("otherFilters");
            filterWidget.fields = [structureField, periodField, filtersField];
        }

        function filterDashboardsByTemplate(dashboards, template) {
            return dashboards.filter(function(dashboard) {
                return dashboard.template === template;
            });
        }

        function _buildDashboards(dashboards) {
            var tabbedWidget = templateManager.container.getWidget('dashboard_tabbed');
            tabbedWidget.widgets.splice(0, tabbedWidget.widgets.length);
            tabbedWidget.currentWidget = null;

            setInitialFilterFields();
            if(filterDashboardsByTemplate(dashboards, 'accordion').length > 0){
                buildAccordionWidget(dashboards, tabbedWidget);
            } else {
                buildTabWidgets(dashboards, tabbedWidget);
            }
            tabbedWidget.fistTime = false;
        }

        function clearWidget(dashboards){
            var tabbedWidget = templateManager.container.getWidget('dashboard_tabbed');
            buildAccordionWidget(dashboards, tabbedWidget);
        }

        this.addDataAccordionWidget = function(item){
            var tabbedWidget = templateManager.container.getWidget('dashboard_tabbed');
            if(item.opened){
                tabbedWidget.widgets.forEach(function(widget){
                    widget.widgets.forEach(function(wid){
                        wid.parameters.period = wid.parameters.fixedPeriod;
                        wid.chartSerieField = wid.chartSerieFieldFixed;
                    });
                    self.fetchAccordionWidgetsData(widget);
                });
            } else {
                getMainDashboards().then(clearWidget);
            }
        };

        this.windowResizeKanban = function(time){
            setTimeout(function(){
                window.dispatchEvent(new Event('resize'));
            }, time);
        };

        function buildAccordionWidget(dashboards, tabbedWidget, item) {
            tabbedWidget.template = GenericDashboardFactory.ACCORDION_TEMPLATE;
            tabbedWidget.widgets = [];
            dashboards = dashboards.sort(UtilitiesController.sortByOrder);
            dashboards.forEach(function(dashboard){
                var accordion = GenericDashboardFactory.buildAccordion(dashboard);
                tabbedWidget.widgets.push(metaDataFactory.widgetFactory(accordion, templateManager.container));
			});
            tabbedWidget.widgets.forEach(function(widget){
                widget.widgets.forEach(function(wid){
                    wid.parameters.fixedPeriod = wid.parameters.period;
                    wid.chartSerieFieldFixed = wid.chartSerieField ? wid.chartSerieField : null;
                });
            });
			UtilitiesController.windowResizeKanban();
        }

        function buildTabWidgets(dashboards, tabbedWidget) {
            if(dashboards[0] && dashboards[0].name){
                var currentDashboard = dashboards[0].name;
                dashboards.forEach(function(dashboard) {
                    var dashboardWidget = GenericDashboardFactory.factoryDashboardTabWidget(dashboard._id, dashboard.name, dashboard.order, tabbedWidget.preview);
                    dashboardWidget = IndicatorFactory.factoryWidget(dashboardWidget, templateManager.container);
                    dashboardWidget.parent = tabbedWidget;
                    tabbedWidget.widgets.push(dashboardWidget);
                    dashboardWidget.currentDashboard = false;
                    dashboardWidget.loadedCharts = false;
                    if(currentDashboard === dashboardWidget.label){
                        dashboardWidget.currentDashboard = true;
                        dashboardWidget.loadedCharts = true;
                    }
                });
            }else{
                ScreenService.hideLoader();
            }
            prepareActions();
            UtilitiesController.windowResizeKanban();
        }

        function getMainDashboardViews(dashboardName, period) {
            return UtilitiesController.callRepositoryMethod(
                restEngine.post,
                restEngine,
                null, [{
                    service: "/view/getMainDashboardViews",
                    typeRequest: "POST",
                    params: {
                        "requestType": "Row",
                        "row": {
                            name: dashboardName,
                            period: period
                        }
                    }
                }]
            ).then(getViewResolve);
        }

        var getViewResolve = function(widgets){
            if(widgets){
                if(widgets.periodObservationNotification && widgets.periodObservationNotification.length > 0){
                    WidgetService.removeObservationNotifications();
                    WidgetService.showObservationNotifications(widgets.periodObservationNotification);
                }
                if(widgets.error && widgets.error.length > 0){
                    widgets.error.forEach(function(error){
                        UtilitiesController.notificationMessage(error,"error");
                    });
                }
            }
            return widgets.mainDashboard;
        };

        function getGenericDashboardsViews(dashboardName, isViewPreview) {
            return UtilitiesController.callRepositoryMethod(
                restEngine.post,
                restEngine,
                null, [{
                    service: "/view/getGenericDashboardsViews",
                    typeRequest: "POST",
                    params: {
                        "requestType": "Row",
                        "row": {
                            name: dashboardName,
                            isViewPreview: isViewPreview
                        }
                    }
                }]
            ).then(getViewResolve);
        }

        function getPreviewDashboards(id) {
            return UtilitiesController.callRepositoryMethod(
                restEngine.post,
                restEngine,
                "dashboards", [{
                    service: "/dashboard/buildPreviewDashboards",
                    typeRequest: "POST",
                    params: {
                        "requestType": "Row",
                        "row": {
                            dashboards: id
                        }
                    },
                    timeout: 50000
                }]
            );
        }

        this.fetchAccordionWidgetsData = function(accordion, filterRow){
            var accordionWidgets = accordion.widgets.concat(accordion.headerWidgets);

            accordionWidgets.forEach(function(widget) {
                var filter = self.getDefaultFilterRow(widget);
                self.setWidgetSubtitle(widget);
                ChartViewService.fetchWidgetData(widget, filter).then(function() {
                    var dimensionFilterInfo = self.getDimensionFilterInfo(widget);
                    if (!Util.isEmptyObject(dimensionFilterInfo)) {
                        buildFilterObject(dimensionFilterInfo);
                    }
                    if(widget.displayMode === "header"){
                        var widgetData = widget.dataSource.data[0];
                        var cardValue = widgetData ? widgetData[widget.cardValue] : widget.chartSeries[0][widget.factProperties[0]];
                        accordion.dataSource.data.push({
                            "label": widget.cardTitle,
                            "mask": widget.cardMask,
                            "value": cardValue,
                            "order": widget.order
                        });
                    }
                });
            });
        };

        function buildDashboardWidget(dashboard, indicators){
            var dashboardWidget = dashboard;
            if (Util.isArray(indicators)) {
                dashboardWidget.widgets = indicators.map(function(indicator) {
                    indicator.id = indicator._id;
                    indicator.name = indicator._id;
                    indicator.isVisible = true;
                    indicator.label = ScreenService.i18n(indicator.label || indicator.name || "");
                    self.setWidgetSubtitle(indicator);

                    return IndicatorFactory.factoryWidget(indicator, templateManager.container);
                }).sort(UtilitiesController.sortByOrder);
                dashboardWidget.widgets.forEach(function(wid){
                    wid.parent = dashboard;
                });
            }
        }

        function prepareActions(dashboard) {
            var container = templateManager.container;
            container.actions = undefined;
            container.prepareActions();
            templateManager.updateTemplate();
        }

        this.setWidgetSubtitle = function(indicator) {
            ScreenService.setHeaderInfo(indicator.parameters.structureName);
            var labelInfo = '';
            var periodParameter = indicator.parameters.period && indicator.parameters.period.period ? indicator.parameters.period.period : indicator.parameters.period;

            if (periodParameter && Util.isString(periodParameter)) {
                labelInfo = ScreenService.i18n(periodParameter);
            } else if (Util.isObject(periodParameter)){
                labelInfo = periodParameter.periodLabel ? periodParameter.periodLabel : periodParameter.start + " - " + periodParameter.end;
            } else {
                labelInfo = ScreenService.i18n("Todo o período");
            }
            if(periodParameter && periodParameter.base){
                labelInfo = ScreenService.i18n(labelInfo);
            }
            if(indicator.lastImportation){
                labelInfo = labelInfo ? labelInfo += " • " : labelInfo;
                labelInfo += ScreenService.i18n("Updated in") + " : " + indicator.lastImportation;
            }
            indicator.labelInfo = labelInfo;
        };

        this.reloadWidgets = function(dashboard, filterRow, fixedWidget){
            var period = filterRow ? filterRow.period : null;
            var containerName = dashboard.container.name;
            if(containerName == "mainDashboard"){
                getMainDashboardViews(dashboard.label,period).then(function(mainDashboard){
                    reloadWidgets(dashboard, mainDashboard, filterRow, fixedWidget);
                });
            }else{
                getGenericDashboardsViews(dashboard.label,period).then(function(genericDashboard){
                    reloadWidgets(dashboard, genericDashboard, filterRow, fixedWidget);
                });
            }
        };

        this.getDimensionFilters = function(filterRow){
            var dimensionFiltersFields = getDimensionFiltersFields();
            var dimensionFilters = [];

            for(var filterProperty in filterRow){
                if(dimensionFiltersFields.indexOf(filterProperty) > -1 && filterRow[filterProperty].length){
                    dimensionFilters.push(filterProperty);
                }
            }

            return dimensionFilters;
        };

        var getDimensionFiltersFields = function() {
            var filterFields = templateManager.container.getWidget('filterWidget').fields;
            filterFields = filterFields.filter(function(field) {
                return field && field.fieldGroup === 'dimensionsFilters';
            }).map(function(field) {
                return field.name;
            });

            return filterFields;
        };

        this.reloadWidget = function(dashboard, mainDashboard, filterRow, fixedWidget, isViewPreview){
            reloadWidgets(dashboard, mainDashboard, filterRow, fixedWidget, isViewPreview);
        };

        function reloadWidgets(dashboard, mainDashboard, filterRow, fixedWidget, isViewPreview) {
            var filterWidget = templateManager.container.getWidget('filterWidget');
            filterWidget.fields = [filterWidget.fields[0], filterWidget.fields[1], filterWidget.fields[2]];
            filterWidget.fields = self.removeNullField(filterWidget.fields);
            if(isViewPreview){
                templateManager.container.label = "Dashboard";
            } else {
                templateManager.container.label = "Dashboard principal";
            }
            if(filterRow && filterRow.filterName && filterRow.filterName.length > 0){
                templateManager.container.label += " - " + filterRow.filterName[0];
                filterWidget.getField('otherFilters').dataSource.data = [{"value":filterRow.filterName[0]}];
                filterWidget.currentRow.otherFilters = [filterRow.filterName[0]];
            } else {
                if(filterWidget.getField('otherFilters')){
                    filterWidget.getField('otherFilters').dataSource.data = [];
                }
                filterWidget.currentRow.otherFilters = [];
                filterWidget.dashboardFilters = null;
            }
            if(mainDashboard[0] && mainDashboard[0].fullAccessProfiles){
                dashboard.fullAccessProfiles = mainDashboard[0].fullAccessProfiles;
            }
            buildDashboardWidget(dashboard, mainDashboard.indicators);

            var structureId = null;
            if(filterRow){
                structureId = filterRow.structureId;
            }

            self.fetchWidgetData(dashboard, filterRow, fixedWidget, isViewPreview, !!structureId);
        }

        this.reloadGenericWidgets = function(dashboard, filterRow, fixedWidget, isViewPreview){
            var filterWidget = templateManager.container.getWidget('filterWidget');
		        if(filterWidget.currentRow.filterId != dashboard.id){
                    delete filterWidget.currentRow.otherFilters;
                    var dimension = self.getDimensionFilters(filterWidget.currentRow);
                    if(dimension){
                        delete filterWidget.currentRow[dimension];
                    }
		        }
            getGenericDashboardsViews(dashboard.label, isViewPreview).then(function(mainDashboard){
                reloadWidgets(dashboard, mainDashboard, filterRow, fixedWidget, isViewPreview);
            });
        };

        this.fetchWidgetData = function(dashboard, filterRow, fixedWidget, isViewPreview, isStructureFromFilter){
            self.updateWidgetImportation(dashboard.widgets);
            var arrayPromise = [];
            var arrayFixBugScrollChartsLine = [];
            dashboard.widgets.forEach(function(widget) {
                var filter = self.getDefaultFilterRow(widget, dashboard.fullAccessProfiles);
                filter.isViewPreview = isViewPreview;
                var viewLevel = 0;
                if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined){
                    viewLevel = +$location.$$search[ViewService.VIEW_LEVEL];
                }
                else{
                    viewLevel = + getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.VIEW_LEVEL];
                }
                reloadAfterSchedule(widget, dashboard.id);
                if(filterRow && widget._id !== fixedWidget){
                    buildParametersByFilterRow(filterRow, widget, filter);
                } else if (viewLevel >= 0) {
                    filter = updateFilterParams(filter, viewLevel, isViewPreview);
                    var labelInfoDetail = filter.drillDownFilters[Object.keys(filter.drillDownFilters)[0]];
                    widget.labelInfo = widget.parameters.structureName + " - " + labelInfoDetail;
                }

                self.setWidgetSubtitle(widget);
                arrayPromise.push(ChartViewService.fetchWidgetData(widget, filter, isViewPreview, isStructureFromFilter).then(function(response){
                    if(response && response.dataset && response.dataset.fetchWidgetData && response.dataset.fetchWidgetData.length > 0 && !!response.dataset.fetchWidgetData[0].PROCESS){
                        widget.dataSource.name = "/resultQueryRunner";
                        widget.dataSourceFilter = [
                            {
                                "name": "processQueryRunner",
                                "operator": "=",
                                "value": response.dataset.fetchWidgetData[0].PROCESS
                            }
                        ];
                        widget.dataSource.data = [];
                    } else {
                        if(widget.updateChart && widget.chartType == 'gauge'){
                            widget.updateChart();
                        }
                        var dimensionFilterInfo = self.getDimensionFilterInfo(widget);
                        if (!Util.isEmptyObject(dimensionFilterInfo)) {
                            buildFilterObject(dimensionFilterInfo);
                        }
                        if(widget.chartType === 'line' || widget.chartType === 'area'){
                            arrayFixBugScrollChartsLine.push(widget.id);
                        }
                    }
                }));
            });

            ZHPromise.all(arrayPromise).then(function(){
                UtilitiesController.windowResizeKanban(500);
                fixBugScrollChartsLine(arrayFixBugScrollChartsLine);
            }).catch(function(){
                UtilitiesController.windowResizeKanban(500);
                fixBugScrollChartsLine(arrayFixBugScrollChartsLine);
            });
        };
        var fixBugScrollChartsLine = function (arrayFixBugScrollChartsLine) {
            arrayFixBugScrollChartsLine.forEach(function (id) {
                UtilitiesController.fixBugScrollChartsLine(id);
            });
        };

        function buildParametersByFilterRow(filterRow, widget, filter) {
            if (filterRow.period && !Util.isEmptyObject(filterRow.period)) {
                if (!Util.isArray(filterRow.period) && widget.chartSerieField) {
                    var serieFields = [widget.chartSerieField.name];
                    var serieLabels = [widget.chartSerieField.label];
                    buildWidgetSeriesAccordingPeriod(widget, serieFields, serieLabels);
                }
                filter.period = widget.parameters.period = filterRow.period;
                widget.parameters.period.start = widget.parameters.period.start;
                widget.parameters.period.end = widget.parameters.period.end;
            }

            if (filterRow.filterWidgetDimensions) {
                filter.dashboardPreConditions = getDimensionFiltersCondition(widget, filterRow.filterWidgetDimensions);
                ChartViewService.setWidgetConditions(widget.dashboardPreConditions, filter, 'dashboardPreConditions');
            }
            if (filterRow.structureId) {
                filter.structureId = [filterRow.structureId];
            }
        }

        function buildWidgetSeriesAccordingPeriod(widget, serieFields, serieLabels){
            widget.serieFields = serieFields;
            widget.serieLabels = serieLabels;
            widget.chartSeries[0].serieFields = serieFields;
            widget.chartSeries[0].serieLabels = serieLabels;
        }

        function getDimensionFiltersCondition(widget, dimensionFilters){
            var dimensionConditions = buildDimensionFilter(widget, dimensionFilters);
            var dimensionExpression = dimensionConditions ? ExpressionInterpreter.readComparisonExpression(dimensionConditions, "previous", dimensionFilters, dimensionFilters) : null;
            return dimensionExpression;
        }

        function buildDimensionFilter(widget, dimensions){
            var widgetDimensions = widget.indicatorDimensions;
            var filterRow = templateManager.container.getWidget('filterWidget').currentRow;
            var dimensionConditions = "";

            dimensions.forEach(function(dimension){
                if(widgetDimensions.indexOf(dimension) !== -1){
                    var condition = "";

                    var dimensionField = filterRow[dimension];
                    if(dimensionField && dimensionField.length) {
                        dimensionField.forEach(function(dimensionValue){
                            if(typeof dimensionValue === "string"){
                                condition += dimension + " == '" + dimensionValue + "' || ";
                            } else {
                                condition += dimension + " == " + dimensionValue + " || ";
                            }
                        });

                        condition = removeSpecialCaractersFromString(condition);

                        dimensionConditions += "(" + condition + ")" + " && ";
                    }
                }
            });

            dimensionConditions = removeSpecialCaractersFromString(dimensionConditions);
            return dimensionConditions;
        }

        function removeSpecialCaractersFromString(string){
            if ((string.substr(-4) === " && ") || (string.substr(-4) === " || ")) {
                string = string.slice(0, -4);
            }
            return string;
        }

        this.getDimensionFilterInfo = function(widget) {
            var dimensionName = widget.parameters.dimensionName;
            var dimensionFilterInfo = {};

            if(dimensionName){
                var fieldData = [];
                widget.dataSource.data.forEach(function(data) {
                    var dimensionData = {};
                    if (data[dimensionName]) {
                        dimensionData[dimensionName] = data[dimensionName];
                        fieldData.push(dimensionData);
                    }
                });

                dimensionFilterInfo.fieldName = dimensionName;
                dimensionFilterInfo.fieldLabel = widget.parameters.dimensionLabel;
                dimensionFilterInfo.fieldData = fieldData;
            }

            return dimensionFilterInfo;
        };

    self.buildFilterObject = function(fieldInfo){
        buildFilterObject(fieldInfo);
    };
        function buildFilterObject(fieldInfo){
            var filterWidget = templateManager.container.getWidget('filterWidget');
            filterWidget.fields = self.removeNullField(filterWidget.fields);
            filterWidget.dimensions = filterWidget.dimensions || [];

            var existentField = filterWidget.getField(fieldInfo.fieldName);
            if(existentField){
                addDimensionFilterData(existentField, fieldInfo);
            } else {
                if(filterWidget.dimensions.indexOf(fieldInfo.fieldName) != -1 ) {
                    filterWidget.dimensions.push(fieldInfo.fieldName);
                }

                var filterDimensionField = getFilterFieldFromParams(fieldInfo);
                metaDataFactory.factoryField(filterDimensionField, filterWidget, true);
            }
        }

        function addDimensionFilterData(field, fieldInfo){
            fieldInfo.fieldData.forEach(function(newData) {
                var dataAlreadySet = false;
                field.dataSource.data.forEach(function(fieldData){
                    if(fieldData[fieldInfo.fieldName] === newData[fieldInfo.fieldName]){
                        dataAlreadySet = true;
                    }
                });
                if(!dataAlreadySet){
                    field.dataSource.data.push(newData);
                }
            });
        }

        function getFilterFieldFromParams(fieldInfo){
            var filterDimensionField = {
                name: fieldInfo.fieldName,
                label: fieldInfo.fieldLabel,
                class: 12,
                readOnly: false,
                showOnForm: true,
                showInDevTools: true,
                valueField: fieldInfo.fieldName,
                descriptionField: fieldInfo.fieldName,
                template: "field/select-multiple.html",
                fieldGroup: "dimensionsFilters",
                generatedField: true,
                dataSource: {
                    data: fieldInfo.fieldData
                }
            };
            return filterDimensionField;
        }

        function updateFilterParams (filter, viewLevel, isViewPreview){
            var drillDownParameters = DrillDownService.getDrillDownParameters(viewLevel);
            if(!isViewPreview){
                if(drillDownParameters.structureId && drillDownParameters.structureId.length){
                    filter.structureId = drillDownParameters.structureId;
                }

                if(drillDownParameters.period && !Util.isEmptyObject(drillDownParameters.period)){
                    filter.period = drillDownParameters.period;
                }

                filter.drillDownFilters = drillDownParameters.drillDownFilters;
            } else {
                filter.drillDownFilters = {};
            }
            ChartViewService.setWidgetConditions(drillDownParameters.previousConditions, filter, 'previousConditions');
            ChartViewService.setWidgetConditions(drillDownParameters.postConditions, filter, 'postConditions');

            return filter;
        }

        function fetchWidgetsData(dashboards, filterRow) {
            dashboards.forEach(function(dashboard){
                if(dashboard.currentDashboard){
                    self.reloadWidgets(dashboard);
                }
            });
        }

        this.reloadDashboards = function(dashboards, filterRow, fixedWidget){
            dashboards.forEach(function(dashboard){
                self.reloadWidgets(dashboard, filterRow, fixedWidget);
            });
        };

        this.getDefaultFilterRow = function(widget, fullAccessProfiles) {
            var orderBy = {};
            var widgetParams = widget.parameters ? widget.parameters : {};

            if (widgetParams.orderByName) {
                orderBy[widgetParams.orderByName] = widgetParams.orderByDirection;
            }

            return {
                viewId: widget._id,
                structureId: widgetParams.structureId,
                fullAccessProfiles: fullAccessProfiles,
                period: widgetParams.period,
                limitRows: widgetParams.limitRows,
                previousConditions: widgetParams.previousConditions ? widgetParams.previousConditions : null,
                postConditions: widgetParams.postConditions ? widgetParams.postConditions : null,
                postGroupConditions: widget.postGroupConditions ? widget.postGroupConditions : null,
                orderBy: orderBy
            };
        };

        function getDashboards(filterParams) {
            return UtilitiesController.callRepositoryMethod(
                restEngine.post,
                restEngine,
                null, [{
                    service: "/dashboard/buildDashboardsByMenuId",
                    typeRequest: "POST",
                    params: { "requestType": "Row", "row": {
                        menuId: filterParams.menuId,
                        drillDownParameters: filterParams.drillDownParameters
                    }}
                }]
            ).then(function(response){
                if(response.errorMessages.error){
                    MessageService.showMessageByCode(response.errorMessages.error,{"d": response.errorMessages.d.substring(0,response.errorMessages.d.length-2)});
                }
                return response.dashboards;
            });
        }

        function getMainDashboards() {
            return UtilitiesController.callRepositoryMethod(
                restEngine.post,
                restEngine,
                "dashboards", [{
                    service: "/dashboard/buildMainDashboards",
                    typeRequest: "POST",
                    params: {
                        "requestType": "Row",
                        "row": {}
                    },
                    timeout: 50000
                }]
            );
        }

        function getGenericDashboards() {
            return UtilitiesController.callRepositoryMethod(
                restEngine.post,
                restEngine,
                "dashboards", [{
                    service: "/dashboard/buildGenericDashboards",
                    typeRequest: "POST",
                    params: {
                        "requestType": "Row",
                        "row": {}
                    },
                    timeout: 50000
                }]
            );
        }

        this.activateFirstOrLast = function() {
            var dashboardWidget = templateManager.container.getWidget(GenericDashboardService.getLastActiveDashboard());
            if(dashboardWidget) {
                activeTab(dashboardWidget);
            } else {
                self.activateFirstTab();
            }
            UtilitiesController.fixBugTabbedNavigation();
        };

        function activeTab(tabToActivate) {
            if (tabToActivate.parent) {
                tabToActivate.parent.currentWidget = tabToActivate;
            }
            tabToActivate.activate();
            templateManager.updateTemplate();
        }

        this.activateFirstTab = function(){
            var tabbedWidget = templateManager.container.getWidget('dashboard_tabbed');
            if(tabbedWidget.template === GenericDashboardFactory.TAB_DASHBOARD_TEMPLATE && tabbedWidget.widgets.length) {
                var firstWidget = tabbedWidget.widgets[0];
                tabbedWidget.currentWidget = firstWidget;
                firstWidget.activate();
                templateManager.updateTemplate();
            }
        };

        self.updateWidgetImportation = function(widgets) {
            var indicatorIds=[];
            widgets.forEach(function(widget){
                if(!indicatorIds.includes(widget.indicator)){
                    indicatorIds.push(widget.indicator);
                }
            });
            ChartViewService.getLastImportationDates(indicatorIds).then(function(lastImportDates){
                widgets.forEach(function(widget) {
                    var dataSourceId = widget.dataSourceId;

                    if(lastImportDates[dataSourceId]){
                        widget.lastImportation = lastImportDates[dataSourceId];

                        var activeWidget = templateManager.container.getActiveWidget();
                        var containerName = activeWidget.container.name;
                        if(containerName === 'chartView'){
                            ChartViewService.setHeaderInfoFromWidgetParameters(widget);
                        } else if (containerName === 'mainDashboard' || containerName === 'genericDashboard'){
                            self.setWidgetSubtitle(widget);
                        }
                    }
                });
            });
        };

        self.factoryDashboardAccordionActions = function(tabbedWidget, container){
            GenericDashboardFactory.factoryDashboardAccordionActions(tabbedWidget, container);
        };
    }

    Configuration(function(ContextRegister) {
        ContextRegister.register('GenericDashboardService', GenericDashboardService);
    });


// FILE: assets/js/services/AutoUpdateService.js
var AutoUpdateService = function(templateManager, GenericDashboardService, ChartViewService) {

    var dataSources = {};

    this.setDataSourceLastUpdateTime = function(dataSourceId) {
        dataSources[dataSourceId] = moment();
    };

    function _updateIndicators(widgets) {
        widgets.forEach(function(widget) {
            var dataSourceLastUpdate = dataSources[widget.dataSourceId];
            if ((dataSourceLastUpdate && widget._lastUpdated && dataSourceLastUpdate > widget._lastUpdated) || (dataSourceLastUpdate && !widget._lastUpdated)) {
                widget._lastUpdated = dataSourceLastUpdate;
                var filter = widget.filterRow || GenericDashboardService.getDefaultFilterRow(widget);
                ChartViewService.fetchWidgetData(widget, filter);
            } else {
                widget.reload().then(function(){
                    ChartViewService.processQueryRunnerAfterLoad(widget);
                });
            }
        });
        GenericDashboardService.updateWidgetImportation(widgets);
    }

    this.updateQueryRunner = function (process) {
        if(templateManager.container.name == 'queryRunner'){
            setDataset(process);
        }
    };

    var setDataset = function (processQuery) {
        var resultWidget = templateManager.container.getWidget('results');
        if(resultWidget && resultWidget.dataSourceFilter[0].value === processQuery){
            resultWidget.reload();
        }
    };

    this.updateIndicators = function(widgets) {
        if (widgets === undefined) {
            switch (templateManager.container.name) {
                case 'mainDashboard':
                case 'genericDashboard':
                    widgets = templateManager.container.getActiveWidget().widgets;
                    break;
                case 'chartView':
                    widgets = [templateManager.container.widgets[1]];
                    break;
            }
        }
        if(widgets){
            _updateIndicators(widgets);
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('AutoUpdateService', AutoUpdateService);
});


// FILE: assets/js/services/ConnectionService.js
function ConnectionService(restEngine, templateManager) {

    this.checkCreateNewConnection = function(row){
        return restEngine.post("/checkCreateNewConnection",
			{
				"requestType": "FilterData",
				"filter": []
			}
		);
    };

    this.testConnection = function(row){
        return restEngine.post({
            service: "/testConnection",
            params: {
                "requestType" : "Row",
                "row" : row
            },
            timeout: 40000,
            typeRequest: "POST"
        });
    };

    this.testConnectionById = function(id){
        return restEngine.post({
            service: "/testAllConnections",
            params: {
                "requestType" : "FilterData",
                "filter" : [{
                    'name' : '_id',
                    'operator' : '=',
                    'value' : id
                }]
            },
            timeout: 40000,
            typeRequest: "POST"
        });
    };

    this.testAllConnections = function() {
        return restEngine.post("/testAllConnections",
			{
				"requestType": "FilterData",
				"filter": []
			}
		);
    };

    this.getMaxCollectorVersion = function() {
        var fileName = "modules.json";
        return restEngine.requestMetaData(Util.getRightPath(fileName,"assets/json/"), {}).then(function(file){
            return file.data.zeedhiDataCollector;
        });
    };
}

Configuration(function(ContextRegister){
	ContextRegister.register('ConnectionService', ConnectionService);
});


// FILE: assets/js/services/DashboardService.js
function DashboardService(Query, DataSourceRepository, UtilitiesController, restEngine) {

	this.importDefaultOrgDashboards = function(dashboards, additionalIndicators) {
		return restEngine.post("/defaultOrgDashboard/import", {
			"requestType": "Row",
			"row": {
				dashboards: dashboards,
				additionalIndicators: additionalIndicators
			}
		});
	};

	this.checkDrillDownIndicators = function(dashboards) {
		return restEngine.post("/defaultOrgDashboard/checkDrillDownIndicators", {
			"requestType": "Row",
			"row": {
				dashboards: dashboards
			}
		});
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('DashboardService', DashboardService);
});

// FILE: assets/js/services/DataSourceManagerService.js
function DataSourceManagerService(restEngine, UtilitiesController) {

    this.CODE_FINALITY_SEND_MAIL = "E";
    this.CODE_FINALITY_IMPORTATION = "I";

	this.syncColumns = function (row) {
		return restEngine.post({
			service: "/dataSource/syncColumns",
			typeRequest: "POST",
			params: { "requestType": "Row", "row": (Util.clone(row) || {}) },
			timeout: 150000
		});
	};

	this.stopApi = function (row) {
		return restEngine.post({
			service: "/api/stopApi",
			typeRequest: "POST",
			params: { "requestType": "Row", "row": (Util.clone(row) || {}) },
			timeout: 150000
		});
	};

	this.deleteIndexes = function(row){
		return restEngine.post({
			service: "/dataSource/deleteIndexes",
			typeRequest: "POST",
			params: { "requestType": "Row", "row": (Util.clone(row) || {}) },
			timeout: 150000
		});
	};

	this.checkIndicatorsWithDataSource = function(rows){
		return restEngine.post("/checkIndicatorsWithDataSource",
			{
				"requestType": "DataSet",
				"dataset": rows
			}
		);
	};

	this.createTokenTemp = function(WEB_TOKEN){
		return restEngine.post({
			service: "/tokenTemp/save",
			typeRequest: "POST",
			params: { "requestType": "Row", "row": {"WEB_TOKEN": WEB_TOKEN} }
		});
	};

	this.saveIndexes = function(row){
		return restEngine.post({
			service: "/dataSource/saveIndexes",
			typeRequest: "POST",
			params: { "requestType": "Row", "row": (Util.clone(row) || {}) },
			timeout: 150000
		});
	};

	this.getLastImportHistoriesFromDataSourceIds = function(dataSourceIds) {
		return UtilitiesController.callRepositoryMethod(
			restEngine.post, restEngine,"importHistory", [{
				service: "/importHistory/findLastOnesByDataSourceIds",
				typeRequest: "POST",
				params: {
					"requestType": "Row",
					"row": ({
						dataSourceIds: dataSourceIds
					} || {})
				},
				timeout: 150000
			}]
		);
	};

	this.findByDataSourceIdAndPeriod = function (dataSourceId, startDate, endDate) {
		var params = {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "dataSourceId",
					"operator": "=",
					"value": dataSourceId
				}
			]
		};
		if (startDate) {
			startDate = moment(startDate, "DD/MM/YYYY");
			var dayStart = startDate.startOf('day').unix();
			var dayEnd;

			if (endDate) {
				endDate = moment(endDate, "DD/MM/YYYY");
				dayEnd = endDate.endOf('day').unix();
			} else {
				dayEnd = startDate.endOf('day').unix();
			}

			params.filter.push({
				"name": "createDate",
				"operator": "BETWEEN",
				"value": [dayStart, dayEnd]
			});
		}

		return UtilitiesController.callRepositoryMethod(
			restEngine.post, restEngine, "importHistory", [{
				service: "/importHistory",
				typeRequest: "POST",
				params: params,
				timeout: 150000
			}]
		);
	};

	this.saveImportSchedule = function(row) {
		return restEngine.post("/importSchedule/save", {
			"requestType": "Row",
			"row": (Util.clone(row) || {})
		});
	};

	this.deleteRelatedImportSchedule = function(row) {
		return restEngine.post("/importSchedule/remove",
		{
			"requestType" : "Row",
		     "row" : {
		    	dataSource: row._id
		    }
		});
	};

    this.deleteImportHistory = function(row) {
        return restEngine.post("/importHistory/remove",
            {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "_id",
                        "operator": "IN",
                        "value": row
                    }
                ]
            });
	};

	this.cleanAllRelations = function(dataSourceId) {
		return restEngine.post("/dataSource/clearAllRelations",
		{
			"requestType" : "Row",
		     "row" : {
		    	dataSource: dataSourceId
		    }
		});
	};

	this.getIndicatorsRelatedToDataSource = function(dataSourceId){
		return restEngine.post("/indicator",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "dataSource",
						"operator": "=",
						"value": dataSourceId
					}
				]
			}
		);
	};

	this.persistIndicatorsColumns = function(rows){
		restEngine.post("/indicator/save",
			{
				"requestType": "DataSet",
				"dataset": rows
			}
		);
	};

	this.persistWidgetsColumns = function(rows){
		restEngine.post("/widget/save",
			{
				"requestType": "DataSet",
				"dataset": rows
			}
		);
	};

	this.getDataSourceById = function(dataSourceId) {
		return restEngine.post("/dataSource", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "_id",
					"operator": "=",
					"value": dataSourceId
				}
			]
		});
	};

	this.findByDataSourceId = function(dataSourceId) {
		return restEngine.post("/dataSourceGeneratedData", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "dataSourceId",
					"operator": "=",
					"value": dataSourceId
				}
			]
		});
	};

	this.findImportsInProgress = function(dataSourceId) {
		return restEngine.post("/dataSource/findImports", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "dataSourceId",
					"operator": "=",
					"value": dataSourceId
				}
			]
		});
	};

	this.updateStatusSchedule = function(dataSourceId) {
		return restEngine.post("/dataSource/updateStatusSchedule", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "dataSourceId",
					"operator": "=",
					"value": dataSourceId
				}
			]
		});
	};

	this.cancelImportation = function(dataSourceId){
		return restEngine.post("/dataSource/cancelImportation", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "dataSourceId",
					"operator": "=",
					"value": dataSourceId
				}
			]
		});
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('DataSourceManagerService', DataSourceManagerService);
});


// FILE: assets/js/services/DefaultRelatedStructuresService.js
function DefaultRelatedStructuresService(restEngine) {

	this.findByStructureTypeId = function(structureTypeId) {
		return restEngine.post("/defaultRelations", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "structureType",
					"operator": "=",
					"value": structureTypeId
				}
			]
		});
	};

	this.saveRow = function(row) {
		return restEngine.post("/defaultRelations/save", {
			"requestType": "Row",
			"row": row
		});
	};

}
Configuration(function(ContextRegister) {
	ContextRegister.register('DefaultRelatedStructuresService', DefaultRelatedStructuresService);
});

// FILE: assets/js/services/GoalNotificationsService.js
function GoalNotificationsService(restEngine) {

    this.getWidgetsAndGoalsByDataSourceId = function(datasourceid) {
        return restEngine.post("/getGoalsInfo",
            {
                "requestType" : "Row",
                "row" : {
                    dataSourceId: datasourceid
                }
            }
        );
    };

    this.getGoalNotifications = function(goalId) {
        return restEngine.post("/goalNotifications",
            {
                "requestType" : "FilterData",
                "filter": [
					{
						"name": "goalId",
						"operator": "=",
						"value": goalId
					}
                ]
            }
        );
    };

    this.removeGoalNotification = function(goalNotificationId) {
        return restEngine.post("/goalNotifications/remove",
            {
                "requestType" : "Row",
                "row" : {
                    _id: goalNotificationId
                }
            }
        );
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GoalNotificationsService', GoalNotificationsService);
});

// FILE: assets/js/services/GoalsService.js
function GoalsService(restEngine) {

    this.getGoalByViewId = function(viewId) {
        return restEngine.post("/goals",
            {
                "requestType" : "FilterData",
                "filter": [
					{
						"name": "viewId",
						"operator": "=",
						"value": viewId
					}
                ]
            }
        );
    };

    this.removeGoal = function(goals) {
        return restEngine.post("/goals/removeGoal",
            {
                "requestType" : "Row",
                "row" : {
                    goals: goals
                }
            }
        );
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GoalsService', GoalsService);
});

// FILE: assets/js/services/ImportDefaultService.js
function ImportDefaultService(restEngine) {

	this.importRows = function(rows, accessProfiles){
		return restEngine.post("/importDefault", {
            "requestType": "Row",
            "row": {
                rows: rows,
                accessProfiles: accessProfiles
            }
        });
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ImportDefaultService', ImportDefaultService);
});

// FILE: assets/js/services/ImportQueueService.js
function ImportQueueService(restEngine) {

    this.getQueueByDataSourceId = function(dataSourceId) {
        return restEngine.post("/manualImportQueue",
            {
                "requestType" : "FilterData",
                "filter": [
					{
						"name": "dataSource",
						"operator": "=",
						"value": dataSourceId
					}
                ]
            }
        );
    };

    this.addItemToQueue = function(scheduleItem) {
        return restEngine.post("/manualImportQueue/save", {
			"requestType": "Row",
			"row": {
                'scheduleItem': scheduleItem
            }
		});
    };

    this.removeItemFromQueue = function(queueItem) {
        return restEngine.post("/manualImportQueue/remove", {
            "requestType" : "Row",
            "row" : {
                _id: queueItem._id,
                order: queueItem.order
            }
        });
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ImportQueueService', ImportQueueService);
});

// FILE: assets/js/services/IndicatorService.js
function IndicatorService(Query, DataSourceRepository, IndicatorFactory, JsonsUtil, templateManager, UtilitiesController, restEngine) {

	this.importDefaultOrgIndicators = function(indicators) {
		return restEngine.post("/defaultOrgIndicator/import", {
			"requestType": "Row",
			"row": {
                indicators: indicators
            }
		});
	};

	this.checkDrillDownIndicators = function(indicators) {
		return restEngine.post("/defaultOrgIndicator/checkDrillDownIndicators", {
			"requestType": "Row",
			"row": {
                indicators: indicators
            }
		});
	};

	this.setBaseConnectionToImport = function(baseConnections){
		return restEngine.post("/setBaseConnectionToImport", {
			"requestType": "Row",
			"row": {
                'baseConnections': baseConnections
            }
		});
	};

	this.getIndicatorById = function(indicatorId){
		return restEngine.post("/indicator",
			{
				"requestType": "FilterData",
				"filter": [
					{
						"name": "_id",
						"operator": "=",
						"value": indicatorId
					}
				]
			}
		);
	};

	this.getViewDependencies = function(views) {
		return restEngine.post("/indicator/getViewDependencies",
            {
                "requestType" : "Row",
                "row" : {
                    views: views
                }
            }
        );
	};

    this.buildWidget = function(widgetObj) {
        var widget = IndicatorFactory.factoryWidget(widgetObj, templateManager.container);
        templateManager.container.getWidget("view").widgets.push(widget);
        templateManager.updateTemplate();
    };

    this.factColorAddToContainer = function (){
        return this.buildWidget(JsonsUtil.factColorWidget());
    };
}
Configuration(function(ContextRegister) {
	ContextRegister.register('IndicatorService', IndicatorService);
});

// FILE: assets/js/services/JoinDataSourceService.js
var dataSourceConstants = {
    REGULAR_DATASOURCE_FIELDS: ['connectionName', 'hasDataMining', 'query', 'beforeImportScript', 'afterImportScript',
        'execAfterIfImportError', 'execAfterIfBeforeScriptError'],
    JOIN_DATASOURCE_FIELDS: ['originDataSources', 'columnOriginDataSources'] 
};

function JoinDataSourceService(restEngine, templateManager, UtilitiesController) {

    var JOIN_SETTINGS_WIDGET_NAME = 'joinSettings';
    var self = this;

	this.updateImportDataSourceDependencies = function(widget){
		var oldOriginDataSources = widget.backupRow ? widget.backupRow.originDataSources : [];
		return restEngine.post("/dataSourceDependencies/updateDependentDataSources", {
			"requestType": "Row",
			"row": {
				oldOriginDataSources: oldOriginDataSources,
				newOriginDataSources: widget.currentRow.originDataSources,
				joinDataSourceId: widget.currentRow._id
			}
		});
	};
	
	this.deleteJoinDataSourceDependencies = function(row){
		return restEngine.post("/dataSourceDependencies/updateDependentDataSources", {
			"requestType": "Row",
			"row": {
				oldOriginDataSources: row.originDataSources,
				newOriginDataSources: [],
				joinDataSourceId: row._id
			}
		});
	};
	
	this.getDependentJoinDataSources = function(dataSourceId){
		return restEngine.post("/dataSourceDependencies/getDataSourceDependencies", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "originDataSourceId",
					"operator": "=",
					"value": dataSourceId
				}
			]
		});
	};

    this.configWidgetWhenJoinDataSource = function(isJoinDataSource, widgetName) {
        isJoinDataSource = isJoinDataSource || false;
        var widget = templateManager.container.getWidget(widgetName);
        var joinSettingsWidget = templateManager.container.getWidget(JOIN_SETTINGS_WIDGET_NAME);

        UtilitiesController.setFieldsVisibility(widget, self.JOIN_DATASOURCE_FIELDS, isJoinDataSource);
        UtilitiesController.setFieldsVisibility(widget, self.REGULAR_DATASOURCE_FIELDS, !isJoinDataSource);

        if(widget.mode !== 'edit'){
            UtilitiesController.changeWidgetVisibility([joinSettingsWidget], isJoinDataSource);
        }

        if (isJoinDataSource) {
            widget.currentRow.hasDataMining = true;
        }
    };
    
    Util.extend(this, dataSourceConstants);
}

Util.extend(JoinDataSourceService, dataSourceConstants);

Configuration(function(ContextRegister) {
	ContextRegister.register('JoinDataSourceService', JoinDataSourceService);
});

// FILE: assets/js/services/PeriodsManagerService.js
function PeriodsManagerService(PeriodsUtilities, ScreenService, ViewPropertyFactory) {

    var self = this;
    var periodNumber = 0;

	this.setPeriodsLabelsAndValues = function(widget) {
		var periodFieldsNames = self.getPeriodFieldsNames(widget);
		periodFieldsNames.forEach(function(fieldName) {
			self.setPeriodFieldLabel(widget, fieldName);

			var periodValue = widget.currentRow[fieldName];
			if(periodValue && periodValue.period && periodValue.period.interval &&
				periodValue.period.interval._id){
				widget.currentRow[fieldName].start = periodValue.period.interval.start;
				widget.currentRow[fieldName].end = periodValue.period.interval.end;
			}else if(periodValue && periodValue.period && !(periodValue.start || periodValue.start)){
				var periodCode = PeriodsUtilities.getPeriodValueByCodedName(periodValue.period.name);
				var interval = PeriodsUtilities.decodeCondition(periodCode);
                if(!interval){
                    interval = PeriodsUtilities.decodePeriod(periodCode);
                }
				widget.currentRow[fieldName].start = interval.start;
				widget.currentRow[fieldName].end = interval.end;
			}
		});
	};

	this.setPeriodFieldValue = function(widget, fieldName){
		return PeriodsUtilities.getIntervals().then(function(response){
			var intervals = response.dataset.periods;
			var period = widget.currentRow[fieldName];
			var periodLabel;

			if (period && period.period) {
				var periodCode = period.period.name;
				PeriodsUtilities.setDataSourcePeriods(intervals);
				var interval = PeriodsUtilities.getPeriodMathExpression(periodCode);

				var decodedPeriod = PeriodsUtilities.decodeCondition(periodCode);
				if (interval && interval._id) {
					period.period.label = interval.label;
					period.start = interval.start;
					period.end = interval.end;
					period.customized = interval.customized;
				} else if (!decodedPeriod) {
					decodedPeriod = PeriodsUtilities.decodePeriod(periodCode);
					period.start = decodedPeriod.start;
					period.end = decodedPeriod.end;
				}
				if (!period.start && !period.end) {
					periodCode = 'today';
					decodedPeriod = PeriodsUtilities.decodePeriod(periodCode);
					period.start = decodedPeriod.start;
					period.end = decodedPeriod.end;
				}
				period.period.name = PeriodsUtilities.getCustomPeriodCode(periodCode);
				periodLabel = period.period.label ? period.period.label : PeriodsUtilities.getPeriodLabelFromIntervals(intervals, periodCode);

				period.period.label = periodLabel;
				widget.currentRow[fieldName] = period;
			}
			self.setPeriodFieldLabel(widget, fieldName, periodLabel);
		});
	};

	this.setPeriodFieldLabel = function(widget, fieldName, periodLabel) {
		widget.getField(fieldName).label = ScreenService.i18n("Period");
		periodLabel = periodLabel ? periodLabel : self.getPeriodLabel(widget, fieldName);

		if (periodLabel) {
			widget.getField(fieldName).label += " (" + periodLabel + ")";
		}
	};

	this.getPeriodLabel = function(widget, fieldName){
		var period = widget.currentRow[fieldName];
		var periodLabel;

		var intervalField = widget.getField(fieldName);
		if(intervalField.dataSource){
			var intervals = intervalField.dataSource.data || [];

			if(period && period.period){
				var periodCode = period.period.name;
				periodLabel = period.period.label ? period.period.label : PeriodsUtilities.getPeriodLabelFromIntervals(intervals, periodCode);
			}
		}

		return periodLabel;
	};

	this.resetAllPeriodFields = function(widget){
		var periodFieldsNames = self.getPeriodFieldsNames(widget);
		periodFieldsNames.forEach(function(fieldName){
			var field = widget.getField(fieldName);
			self.resetPeriodFieldLabel(field);
		});
	};

	this.resetPeriodFieldLabel = function(field){
		field.label = ScreenService.i18n("Period");
	};

	this.getPeriodFieldsNames = function(widget){
		return widget.fields.filter(function(field){
			return field.name && field.name.substr(0, 7) === "period_" || field.name === "period";
		}).map(function(field){
			return field.name;
		});
	};

	this.getPeriodsValues = function(widget, periodNumber) {
		var periodsValues = [widget.currentRow.period];

		for (var i = 1; i <= periodNumber; i++) {
			self.cleanInterval(widget.currentRow, "period_" + i);

			var periodValue = widget.currentRow["period_" + i];
			delete widget.currentRow["period_" + i];

			if (periodValue) {
				periodsValues.push(periodValue);
			}
		}
		return periodsValues;
	};

	this.cleanInterval = function(currentRow, fieldName, periodNumber) {
		fieldName = fieldName || 'period';
		var field = currentRow[fieldName];

		if (field) {
			if (field.period && field.period.name) {
				if (field.period.name[0] !== ('@')) {
					field = field.period.name;
				}
				else {
					var periodValue = PeriodsUtilities.getPeriodValueByCodedName(field.period.name);
					if (periodValue) {
						field = periodValue;
					}
				}
			}
			else if (!field.end && !field.start && !field[0]) {
				field = "";
			} else if(field[0]){
				field = field;
				periodNumber = 0;
			}

			currentRow[fieldName] = field;
		}
		return periodNumber;
	};

	this.setIntervalFieldsNumber = function(widget, periodNumber, controllerCall) {
		periodNumber = self.cleanIntervalFields(widget, periodNumber);
        if(periodNumber != 0){
            periodNumber = 0;
        }
		return PeriodsUtilities.getIntervals().then(function(response){
			var intervals = response.dataset.periods;
			var formatedIntervals = PeriodsUtilities.formatPeriods(intervals);
			if (Util.isArray(widget.currentRow.period)) {
				var periods = widget.currentRow.period;
				while (periodNumber < periods.length - 1) {
					periodNumber++;
					buildPeriodField(widget, periodNumber, [], controllerCall, formatedIntervals);
				}

				widget.currentRow.period = periods.shift();
				widget.currentRow.period = self.getCustomPeriodValue(widget.currentRow.period, intervals);

				setCustomPeriodsValues(widget, periods, intervals);
			} else if(widget.currentRow.period) {
				widget.currentRow.period = setWidgetPeriodWhenThereIsJustOne(widget, intervals);
			}
			widget.getField('period').intervals = formatedIntervals;
			return periodNumber;
		});
	};

	this.cleanIntervalFields = function(widget, periodNumber) {
        var relatedPeriods = widget.fields.filter(function (field) {
            return field.name && field.name.substr(0, 7) === "remove_";
        }).map(function (field) {
            return field.relatedPeriod;
        });

        relatedPeriods.forEach(function (relatedPeriod) {
            self.removeInterval(widget, relatedPeriod, periodNumber, []);
            periodNumber--;
        }, this);

		return periodNumber; //should be 0
	};

	this.removeInterval = function(widget, relatedPeriod, periodNumber, periodUnusedIds) {
		relatedPeriod = (relatedPeriod || periodNumber);
		var deletedField = widget.getField("period_" + relatedPeriod);
		var deletedButtonLabel = widget.getField("remove_" + relatedPeriod);

		self.cleanInterval(widget.currentRow, deletedField.name);
		delete widget.currentRow[deletedField.name];

		var fieldIndex = widget.fields.indexOf(deletedField);
		widget.fields.splice(fieldIndex, 1);

		var fieldLabelIndex = widget.fields.indexOf(deletedButtonLabel);

		widget.fields.splice(fieldLabelIndex, 1);

		if (relatedPeriod <= periodNumber) {
			periodUnusedIds.push(relatedPeriod);
		}

		return periodUnusedIds;

		//periodNumber--;
	};

	function buildPeriodField(widget, periodNumber, periodUnusedIds, controllerCall, intervals){
		var periodId = periodUnusedIds.length > 0 ? periodUnusedIds.shift() : periodNumber;
		var removeField = ViewPropertyFactory.getTrashLabelWithIconField(periodId, widget, false, controllerCall);
		var periodField = ViewPropertyFactory.buildIntervalField(periodId, intervals, widget, false);
		widget.fields.unshift(removeField);
		widget.fields.unshift(periodField);
		return periodUnusedIds;
	}

	this.getCustomPeriodValue = function(periodCode, intervals){
		if(Util.isString(periodCode)){
			var period = PeriodsUtilities.getPeriodFromCodeAndPeriods(periodCode, intervals);
			var periodName = periodCode;
			var periodLabel = "";
			var interval = "";

			if(periodCode && PeriodsUtilities.isHexadecimal(periodCode) && !period){
				period = {
					custom: true,
					description: "",
					name: "Hoje",
					periodsCode: "today"
				};
				periodCode = periodName = "today";
			}

			if (period) {
				periodName = period.custom ? PeriodsUtilities.getCustomPeriodCode(periodCode) : periodName;
				periodLabel = period.name || periodLabel;
				interval = period.interval;
			}

			var field = {
				"period": {
					"name": periodName,
					"interval": interval,
					"label": periodLabel
				}
			};

			return field;
		} else {
			return periodCode;
		}
	};

	function setCustomPeriodsValues(widget, periods, intervals){
		for (var i = 1; i <= periods.length; i++) {
			var index = i -1;
			var periodField = "period_" + i;
			widget.currentRow[periodField] = self.getCustomPeriodValue(periods[index], intervals);
		}
	}

	function setWidgetPeriodWhenThereIsJustOne(widget, intervals) {
		var period = self.getCustomPeriodValue(widget.currentRow.period, intervals);
		if (period && period.period) {
			if (period.period.interval) {
				period.start = period.period.interval.start;
				period.end = period.period.interval.end;
			} else {
				var decodedIntervals = PeriodsUtilities.decodeCondition(widget.currentRow.period);
				if (!decodedIntervals) {
					decodedIntervals = PeriodsUtilities.decodePeriod(widget.currentRow.period);
				}
				period.start = decodedIntervals.start;
				period.end = decodedIntervals.end;
			}
		}
		return period;
	}

	this.addNewInterval = function(widget, periodNumber, periodUnusedIds, controllerCall) {
		//periodNumber++;
		var periodId = periodUnusedIds.length > 0 ? periodUnusedIds.shift() : periodNumber;
		var removeField = ViewPropertyFactory.getTrashLabelWithIconField(periodId, widget, false, controllerCall);
		return ViewPropertyFactory.getIntervalField(periodId, widget, false).then(function(periodField){
			widget.fields.unshift(removeField);
			widget.fields.unshift(periodField);
			return periodUnusedIds;
		});
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('PeriodsManagerService', PeriodsManagerService);
});

// FILE: assets/js/services/QueryRunnerService.js
function QueryRunnerService(restEngine) {

    this.runQuery = function(row){

        return restEngine.post("/runQuery",
            {
                "requestType" : "Row",
                "row" : row
            }
        );
    };

    this.setStatusColumnValue = function(fieldStatus, importProcess){

        var status =  'c';
        var subprocesses = null;
        var hasError = null;
        if(importProcess){
            status =  importProcess.status;
            subprocesses =  importProcess.subprocesses;
            hasError = importProcess.hasError;
            hasError = importProcess.totalImported;
        }

        if (hasError) {
            fieldStatus.class = fieldStatus.classDefault + ' circle-status error bottom20 ';
            fieldStatus.label = ' ';
        } else if (status === 's') {
            fieldStatus.class = fieldStatus.classDefault;
            fieldStatus.label = 'Saving documents...';
        } else if (status === 100) {
            fieldStatus.class = fieldStatus.classDefault + ' circle-status success bottom20 ';
            fieldStatus.label = ' ';
        } else if (status === 'c') {
            fieldStatus.class = fieldStatus.classDefault + ' circle-status cancel bottom20 ';
            fieldStatus.label = ' ';
        } else if ((status > 0 && status < 100) || (status >= 0 && status < 100 && subprocesses > 0)) {
            fieldStatus.class = fieldStatus.classDefault;
            fieldStatus.label = status + '%';
        }  else if(totalImported > 0) {
            fieldStatus.class = fieldStatus.classDefault;
            fieldStatus.label = "In process (" + totalImported +" linhas)...";
        } else if (status === 0 && subprocesses === 0) {
            fieldStatus.class = fieldStatus.classDefault;
            fieldStatus.label = 'Starting Processes...';
        }
    };

    this.runQueryMining = function(row){

        return restEngine.post("/runQueryMining",
            {
                "requestType" : "Row",
                "row" : row
            }
        );
    };

    this.getColumnsMetaData = function(row){

        return restEngine.post("/getColumnsMetaData",
            {
                "requestType" : "Row",
                "row" : row
            }
        );
    };

    this.dropCollectionQueryRunner = function(){
        return restEngine.post("/dropCollectionQueryRunner",  {
            "requestType" : "Row",
            "row" : {}
        });
    };
}
Configuration(function(ContextRegister){
	ContextRegister.register('QueryRunnerService', QueryRunnerService);
});


// FILE: assets/js/services/ReleaseNotesService.js
function ReleaseNotesService(restEngine) {

    this.getZmartAdminUsers = function() {
        return restEngine.post("/zmartAdminUsers",
            {
                "requestType" : "FilterData",
                "filter": []
            }
        );
    };

    this.getReleaseNotes = function() {
        return restEngine.post("/releaseNotes",
            {
                "requestType" : "FilterData",
                "filter": []
            }
        );
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ReleaseNotesService', ReleaseNotesService);
});

// FILE: assets/js/services/SignUpService.js
function SignUpService(restEngine, ScreenService, UtilitiesController) {
    
    var USER_LOGIN_WIDGET = "identityLibraries#login";

    this.registerNewUser = function(row) {
        restEngine.post('IDENTITY#/zi_identitySignUp/save', {
            "requestType": "DataSet",
            "dataset": [row]
        }, function(response) {
            var organizationId = response.dataset.response[0];
            persistNewOrganization(organizationId, row.SAAS).then(function(){
                UtilitiesController.showMessage("SIGNUP_SUCCESS");
                ScreenService.openWindow(USER_LOGIN_WIDGET);
            }, function(error){
                UtilitiesController.showMessage(error.data.error);
            });
        }, undefined, function(error) {
            UtilitiesController.showMessage(error.data.error);
        });
    };
    
    function persistNewOrganization(organizationId, saasConnection){
        return restEngine.post("/signUpOrganization",
            {
                "requestType" : "Row",
                "row" : {
                    "ORGANIZATION_ID": organizationId,
                    "SAAS_CONNECTION": saasConnection
                }
            }
        );
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('SignUpService', SignUpService);
});

// FILE: assets/js/controllers/MainDashboardController.js
function MainDashboardController(templateManager,GenericDashboardService, ScreenService, localStorageEngine,
                                 $location, ChartViewService,StyleService, PeriodsUtilities) {

	var loaded = false;
	var self = this;
	var activatedTab = false;

	this.buildMainDashboards = function() {
        ScreenService.showLoader();
        loginPromise.then(function() {
            ScreenService.hideLoader();
            if (!loaded) {
                StyleService.loadCustomCssStyles().then(function () {
                    GenericDashboardService.buildMainDashboards().then(self.activateFirstOrLast);
                    loaded = true;
                });
            }
            if($location.$$search.Name === undefined && $location.$$hash.Name === undefined){
                if(getLocalVar('indicatorNames')===undefined){
                    localStorageEngine.setLocalVar('indicatorNames',{});
                }
                localStorageEngine.setLocalVar('indicatorName','');
            }else{
                ChartViewService.setIndicatorName();
            }
        });
	};

	this.forceDashboardReload = function() {
		loaded = false;
	};
	
	this.otherFiltersFieldOnChange = function() {
		var filterWidget = templateManager.container.getWidget('filterWidget');
		var dimension = GenericDashboardService.getDimensionFilters(filterWidget.currentRow);
		if(dimension){
			filterWidget.currentRow[dimension] = filterWidget.currentRow.otherFilters;
			filterWidget.currentRow.filterId = filterWidget.currentRow.otherFilters;
			if(filterWidget.currentRow.otherFilters == undefined){
				delete filterWidget.currentRow[dimension];
			}
		}
	};

	this.filterWidgetWidgetOnEnter = function(periodField) {
		PeriodsUtilities.setIntervalsOnPeriodField(periodField);
		var filterWidget = templateManager.container.getWidget('filterWidget');
		var dimension = GenericDashboardService.getDimensionFilters(filterWidget.currentRow);
		if(dimension){
			filterWidget.currentRow.otherFilters = filterWidget.currentRow[dimension];
		}
	};

    this.structureLabelFieldOnClick = function(args){
      	args.owner.field.reload();
	};

	this.deactivateTab = function(){
		activatedTab = false;
	};

	this.activateFirstOrLast = function() {
		if(templateManager.container){
			var dashboardWidget = templateManager.container.getWidget("dashboard_tabbed");
			if(dashboardWidget && dashboardWidget.widgets.length > 0){
				if(!getLocalVar('returningFromBackButton')){
					GenericDashboardService.cleanLastActiveDashboard();
				}
				var lastActiveDashboard = GenericDashboardService.getLastActiveDashboard();
				dashboardWidget = dashboardWidget.widgets.find(function(widget){
					return (widget.name && widget.name == lastActiveDashboard);
				});
				if(dashboardWidget) {
					localStorageEngine.setLocalVar('returningFromBackButton',false);
					activatedTab = true;
					activeTab(dashboardWidget);
				} else if(!activatedTab){
					GenericDashboardService.activateFirstTab();
				}
			}
		}
	};

	function activeTab(tabToActivate) {
		if (tabToActivate.parent) {
			tabToActivate.parent.currentWidget = tabToActivate;
		}
		tabToActivate.activate();
		templateManager.updateTemplate();
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('MainDashboardController', MainDashboardController);
});

// FILE: assets/js/controllers/FilterController.js
function FilterController(Query, ScreenService, DataSourceService, templateManager, UtilitiesController,
ZHPromise, localStorageEngine, FilterService, ViewTypeService) {

	var self = this;
	var temporaryRow = {};
	var viewTypesMap = {};

	var propertiesToFilter = ['dimension', 'indicator', 'view', 'period_start', 'period_end', 'structure', {
		name: 'fact',
		method: 'in'
	}, 'structure_label', 'indicator_label'];
	var orderByProperties = [{
		name: 'orderBy',
		direction: Query.ORDER_ASC
	}];

	var propertiesToClear = [
		'view',
		'view_label',
		'structure',
		'structure_label',
		'dimension',
		'dimension_label',
		'fact',
		'orderBy',
		'orderBy_label',
		'orderByDirection',
		'period'
	];

	var dataSourceFieldsToClear = [
		'dimension_label',
		'fact',
		'orderBy_label'
	];

	var _clearDataSource = function(field) {
		if (field && field.dataSource && field.dataSource instanceof DataSource) {
			field.value("");
			field.dataSource.clearCheckedRows();
			field.dataSource.data = [];
		}
	};

	function reloadField(widget, fieldName) {
		var field = widget.getField(fieldName);
		field.dataSource.data = [];
		return field.reload();
	}

	this.fillProperties = function(indicatorId, widget) {
		var dependentFields = ['view_label', 'structure_label', 'dimension_label', 'fact'];
		widget.columnsMap = {};
		this.resetFilter(widget);
		if (!indicatorId) {
			this.disableFields(widget.fields, dependentFields.concat('orderBy_label'));
			return ZHPromise.when();
		} else {
			this.enableFields(widget.fields, dependentFields);

			if (widget.currentRow.indicator) {
				localStorageEngine.setLocalVar('indicatorId', widget.currentRow.indicator);
			}

			reloadField(widget, 'view_label');
			reloadField(widget, 'structure_label');

			return DataSourceService.findOneByIndicator(indicatorId).then(function(dataSource) {
				var dimensionField = widget.getField('dimension_label');
				var factField = widget.getField('fact');
				var orderByField = widget.getField('orderBy_label');

				dimensionField.dataSource.data = [];
				factField.dataSource.data = [];
				orderByField.dataSource.data = [];

				if (dataSource && Util.isArray(dataSource.columns)) {
					dataSource.columns.forEach(function(column) {
						if (column.isDimension) {
							dimensionField.dataSource.data.push(column);
						} else if (column.isFact) {
							factField.dataSource.data.push(column);
						}
						widget.columnsMap[column.name] = column;
					});
				}

				templateManager.updateTemplate();
			});
		}
	};

	this.viewOnChange = function(viewId, viewField, filterWidget) {
		if(viewField.dataSource.data && viewField.dataSource.data.length > 0) {
			var view = viewField.dataSource.data.filter(function(view) {
				return view._id === viewId;
			}).shift();
			var viewType = viewTypesMap[view.viewType];

			hideFilterProperties(filterWidget, viewType);
			this.fillDefaultViewProperties(view, viewType, viewField, filterWidget);
		}
	};

	function hideFilterProperties(filterWidget, viewType) {
		viewType.filterPropertiesToHide = viewType.filterPropertiesToHide || [];
		filterWidget.fields.forEach(function(field) {
			field.isVisible = viewType.filterPropertiesToHide.indexOf(field.name) === -1;
		});
	}

	this.fillDefaultViewProperties = function(view, viewType, viewField, filterWidget) {
		this.fillFacts(filterWidget, view, viewType);
		this.fillDimension(filterWidget, view, viewType);
		this.fillOrderBy(filterWidget.currentRow, filterWidget.getField('orderBy_label'), filterWidget);
	};

	this.fillFacts = function(filterWidget, view, viewType) {
		var factField = filterWidget.getField('fact');
		if(factField.isVisible){
			var facts = [];
			if (Util.isArray(viewType.factProperties) && viewType.factProperties.length) {
				facts = Util.parseToArray(view[viewType.factProperties[0].name]) || [];
			}
			factField.value(facts);
		} else {
			factField.value([]);
		}
		templateManager.updateTemplate();
	};

	this.fillDimension = function(filterWidget, view, viewType) {
		var dimensionField = filterWidget.getField('dimension_label');
		var dimension = {};
		if(dimensionField.isVisible) {
			var columnName = "";
			if (viewType.dimensionProperty) {
				if (Util.isArray(view[viewType.dimensionProperty.name])) {
					columnName = view[viewType.dimensionProperty.name][0];
				} else {
					columnName = view[viewType.dimensionProperty.name];
				}
			}
			dimension = columnName ? filterWidget.columnsMap[columnName] : {};
		}
		filterWidget.currentRow.dimension_label = dimension.label || "";
		filterWidget.currentRow.dimension = dimension.name || "";
		templateManager.updateTemplate();
	};

	this.fillOrderBy = function(row, orderByField, filterWidget) {
		var orderByFieldNames = ['orderBy_label', 'orderByDirection'];
		var orderBy = [];
		var dimension = filterWidget.columnsMap[row.dimension];

		if(orderByField.isVisible) {
			if (Util.isArray(row.fact)) {
				row.fact.forEach(function(factName){
					if (filterWidget.columnsMap[factName]) {
						orderBy.push(filterWidget.columnsMap[factName]);
					}
				});
			}
			if (dimension) {
				orderBy.push(dimension);
			}
		}

		if (orderBy.length) {
			this.enableFields(orderByField.widget.fields, orderByFieldNames);
			filterWidget.currentRow.orderBy_label = orderBy[0].label;
			filterWidget.currentRow.orderBy = orderBy[0].name;
			filterWidget.currentRow.orderByDirection = "DESC";
		} else {
			this.disableFields(orderByField.widget.fields, orderByFieldNames);
		}
		orderByField.dataSource.data = Util.clone(orderBy);

		templateManager.updateTemplate();
	};

	this.resetFilter = function(widget) {
		propertiesToClear.forEach(function(properties) {
			widget.currentRow[properties] = '';
		});
		dataSourceFieldsToClear.forEach(function(field) {
			widget.getField(field).dataSource.data = [];
		});
		templateManager.updateTemplate();
	};

	this.refreshChart = function(widget, chartWidget) {
		localStorageEngine.setLocalVar('indicatorFilters', widget.currentRow);
		orderByProperties[0].direction = widget.currentRow.orderByDirection;
		var filters = UtilitiesController.buildQueryFilters(widget.currentRow, propertiesToFilter, orderByProperties);
		return chartWidget.reload(filters);
	};

	this.showChart = function(widget) {
		localStorageEngine.setLocalVar('indicatorFilters', widget.currentRow);
		ScreenService.openWindow('chartView');
	};

	this.openFilterFromDashboard = function(widget) {
		temporaryRow.indicator = widget.indicator;
		temporaryRow.indicator_label = widget.name;
		ScreenService.openWindow('filter');
	};

	this.loadTemporaryRow = function(widget) {
		if (temporaryRow.indicator) {
			this.fillProperties(temporaryRow.indicator, widget);
			widget.currentRow.indicator_label = temporaryRow.indicator_label;
			widget.currentRow.indicator = temporaryRow.indicator;
			temporaryRow = {};
			templateManager.updateTemplate();
		}
	};

	this.disableFields = function(fields, fieldsToDisable) {
		fields.forEach(function(field) {
			if (fieldsToDisable.indexOf(field.name) !== -1) {
				field.readOnly = true;
			}
		});
	};

	this.enableFields = function(fields, fieldsToEnable) {
		fields.forEach(function(field) {
			if (fieldsToEnable.indexOf(field.name) !== -1) {
				field.readOnly = false;
			}
		});
	};

	this.filterWidgetOnEnter = function(widget) {
		buildViewTypesMap();

		FilterService.filterIndicators().then(function() {
			self.loadTemporaryRow(widget);
		});
	};

	function buildViewTypesMap() {
		viewTypesMap = {};
		return ViewTypeService.findAllWithProperties().then(function(viewTypes) {
			if (Util.isArray(viewTypes)) {
				viewTypes.forEach(function(viewType) {
					viewTypesMap[viewType.name] = viewType;
				});
			}
		});
	}

	this.openGenericFilterWindow = function() {
		localStorageEngine.setLocalVar('parentMenuId', null);
		if (templateManager.container.name === 'filter') {
			FilterService.filterIndicators();
		} else {
			ScreenService.openWindow('filter');
		}
	};

}
Configuration(function(ContextRegister) {
	ContextRegister.register('FilterController', FilterController);
});


// FILE: assets/js/controllers/GenericDashboardController.js
function GenericDashboardController(templateManager, ZHPromise, localStorageEngine, 
    ScreenService,GenericDashboardService, $location, ViewService, ChartViewService, UtilitiesController, WidgetService)
{

    var self = this;
    var firstTime = false;
    var dashboardStack = [];

    this.dashboardBeforeInit = function(){
        firstTime = false;
    };
    this.buildDashboards = function() {
        if($location.$$search.Name === undefined && $location.$$hash.Name === undefined){
            if(getLocalVar('indicatorNames')===undefined){
                localStorageEngine.setLocalVar('indicatorNames',{});
            }
            localStorageEngine.setLocalVar('indicatorName','');
        }else{
            ChartViewService.setIndicatorName();
        }
        if(localStorageEngine.getLocalVar('isLogged')){
            clearDashboard();
            var menuId = localStorageEngine.getLocalVar(ViewService.MENU_ID);
            if (menuId) {
                return buildDashboardsByMenu(menuId).then(this.activateFirstOrLast);
            } else {
                return this.buildGenericDashboards().then(this.activateFirstOrLast);
            }
        }
    };

    this.activateFirstOrLast = function() {
        activateFirstTab();
    };

    this.activeTab = function(tabToActivate) {
        if (tabToActivate.parent) {
            tabToActivate.parent.currentWidget = tabToActivate;
        }
        tabToActivate.activate();
        templateManager.updateTemplate();
    };

    function activateFirstTab(){
        var tabbedWidget = templateManager.container.getWidget('dashboard_tabbed');
        if (tabbedWidget.widgets.length) {
            var firstWidget = tabbedWidget.widgets[0];
            tabbedWidget.currentWidget = firstWidget;
            templateManager.container.widgets.forEach(function(widget){
                widget.fields = widget.fields.filter(function(field){
                    return field !== undefined;
                });
            });
            firstWidget.activate();
            templateManager.updateTemplate();
        }
    }

    function buildDashboardsByMenu(menuId) {
        return GenericDashboardService.buildDashboards(menuId).then(null, treateBuildDashboardsFail);
    }

    this.buildGenericDashboards = function() {
        return GenericDashboardService.buildGenericDashboards().then(function(dashboards) {
            if (!Util.isArray(dashboards) || !dashboards.length) {
                ScreenService.goBack();
            }
        }, treateBuildDashboardsFail);
    };

    this.buildPreviewDashboards = function(id) {
        return GenericDashboardService.buildPreviewDashboards(id).then(function(dashboards) {
            if (!Util.isArray(dashboards) || !dashboards.length) {
                ScreenService.goBack();
            }
            return dashboards;
        }, treateBuildDashboardsFail);
    };

    function treateBuildDashboardsFail(response) {
        ChartViewService.loadFail(response);
        ScreenService.goBack();
    }

    function clearDashboard() {
        templateManager.container.getWidget("dashboard_tabbed").widgets.splice(0);
    }

    this.openFilterWidget = function (){
        var filterWidget = templateManager.container.getWidget('filterWidget');
        filterWidget.isVisible = true;
        ScreenService.openPopup(filterWidget);
    };

    this.filterDashboards = function(currentFilterRow) {
        WidgetService.removeObservationNotifications();
        var mainDashboardTabbed = templateManager.container.getWidget('dashboard_tabbed');
        var currentWidget = mainDashboardTabbed.currentWidget;
        var filterRow = self.buildFilterObject(currentFilterRow);
        GenericDashboardService.reloadWidgets(currentWidget, filterRow);
        this.closeFilterWidget();
        currentWidget.activate();
    };

    this.buildFilterObject = function(filterRow){
        var period = {};
        if(filterRow && filterRow.period){
            var startDate = filterRow.period.start;
            var endDate = filterRow.period.end;
            period = (startDate || endDate) ? {"period": {"start": startDate, "end": endDate}} : {};
        }

        var dimensions = GenericDashboardService.getDimensionFilters(filterRow);

        var ret = {
            'period': period,
            'filterWidgetDimensions': dimensions,
            'structureId': filterRow.structure,
            'structureName': filterRow.structure_label
        };
        if(dimensions){
            ret.filterName = filterRow[dimensions];
            ret.filterId = filterRow.filterId;
        }
        return ret;
    };

    this.closeFilterWidget = function(){
        var filterWidget = templateManager.container.getWidget('filterWidget');
        filterWidget.isVisible = false;
        ScreenService.closePopup();
    };

    this.reloadDashboards = function(dashboards, filterRow, fixedWidget){
        dashboards.forEach(function(widgets){
            GenericDashboardService.updateWidgetImportation(widgets.widgets);
        });
    };

    this.getCurrentWidget = function(widget, currentWidgetLabel){
        widget.widgets.forEach(function(wid){
            if(wid.label === currentWidgetLabel){
                wid.currentDashboard = true;
            } else {
                wid.currentDashboard = false;
            }
        });
    };

    this.actionEventBack = function(){
        if(templateManager.container.name == "mainDashboard"){
            dashboardStack.pop();
            var widget = dashboardStack.pop();
            if(widget){
                widget.activate();
            }
        }else{
            localStorageEngine.setLocalVar('returningFromBackButton', true);
            ScreenService.goBack();
        }
    };

    this.activeDashboardSearch = function(isViewPreview, clearFilterRow){
        WidgetService.removeObservationNotifications();
        var tabbedWidget = templateManager.container;
        if((tabbedWidget.widgets[0].fistTime && !isViewPreview)){
            UtilitiesController.killAllAsyncProcesses();
        }
        tabbedWidget.widgets[0].fistTime = true;
        firstTime = true;
        var currentWidget = tabbedWidget.getActiveWidget();
        if(isViewPreview && currentWidget.widgets && currentWidget.widgets[0]){
            currentWidget = currentWidget.widgets[0];
        }
        this.getCurrentWidget(tabbedWidget.widgets[0], currentWidget.label);
        var backAction = currentWidget.getAction("back");
        if(backAction){
            backAction.isVisible = dashboardStack.length > 0;
        }
        if(currentWidget.currentDashboard || !currentWidget.loadedCharts){
            if(dashboardStack.length == 0 || dashboardStack[dashboardStack.length-1].name !== currentWidget.name){
                dashboardStack.push(currentWidget);
            }
            var filterWidget = templateManager.container.getWidget('filterWidget');
            var filterRow;
            if(!clearFilterRow){
                if(filterWidget.currentRow){
                    filterRow = self.buildFilterObject(filterWidget.currentRow);
                }
            }else{
                filterWidget.currentRow = [];
            }
            GenericDashboardService.reloadGenericWidgets(currentWidget, filterRow, undefined, isViewPreview);
        }
    };

    this.getFullAccessProfilesFromDashboard = function(){
        var filterWidget = templateManager.container.getWidget('filterWidget');
        return filterWidget.lastActiveWidget.fullAccessProfiles;
    };
    
    this.reloadWidgetAccordion = function(widget){
        GenericDashboardService.fetchWidgetData(widget.widget);
    };

    this.addFilterAction = function(){
        var filterWidget = templateManager.container.getWidget('filterWidget');
        filterWidget.actions[0].events[0].code = "GenericDashboardController.filterAccordion();";
    };

    this.filterAccordion = function(){
        var currentFilterRow = templateManager.container.getWidget('filterWidget').currentRow;
        var filterRow = self.buildFilterObject(currentFilterRow);

        var currentWidget = templateManager.container.widgets[0];
        currentWidget = currentWidget.widgets.filter(function(widget){
            return widget.opened === true;
        })[0];

        GenericDashboardService.fetchWidgetData(currentWidget, filterRow);
        this.closeFilterWidget();
        currentWidget.activate();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GenericDashboardController', GenericDashboardController);
});


// FILE: assets/js/controllers/ChartViewController.js
function ChartViewController(templateManager, ScreenService, localStorageEngine, DataSourceService, ChartViewService, ViewService,
                             $location, eventAggregator, UtilitiesController, NotificationFactory, ZHPromise, DrillDownService,
                             metaDataFactory, ExpressionInterpreter, PeriodsManagerService, WidgetService, IndicatorService,
                             MessageService, GenericFieldFactory, ZhReports) {


    var self = this;

    var periodNumber = 0;
    var inSerInterval = false;
    var periodUnusedIds = [];
    var needClearFilter = true;
    var isReturningFromDrillDown = false;

    var defaultOrderBy = {
        "name": "",
        "label": "",
        "direction": ""
    };

    var updateInterval;
    var testCounter = 0;

    var getWidgetInformationStripe = function(){
        return templateManager.container.getWidget("informationStripe");
    };

    var getActionSendNotification = function(){
        return getWidgetInformationStripe().getAction("sendNotification");
    };

    this.openNotificationPopup = function (container) {
        var widget = container.getWidget('notification');
        if (widget === undefined) {
            widget = createNotificationPopup(container);
        }

        widget.newRow();
        ScreenService.openPopup(widget);
    };

    var createNotificationPopup = function (container) {
        return NotificationFactory.createNotificationPopup(container);
    };

    var notificate = function (widget) {
        if (widget.isValid()) {
            ChartViewService.sendNotification(widget.currentRow).then(function (response) {
                response.messages.forEach(function (message) {
                    if (message.type === "S") {
                        UtilitiesController.successNotification(message.message);
                    } else {
                        UtilitiesController.errorNotification(message.message);
                    }
                });
                ScreenService.closePopup();
            });

        } else {
            MessageService.showMessageByCode("REQUIRED_FIELDS");
        }
    };

    this.sendNotificationIndicator = function (widget) {
        var containerWidget = widget.container.widgets[1];
        var isChartTypeWidget = !!containerWidget.chart;
        var data = new Date();
        widget.currentRow.sender = localStorage.username;
        widget.currentRow.sendDate = data.getDate() + "/" + (data.getMonth() + 1) + "/" + data.getFullYear();
        widget.currentRow.nameIndicator = containerWidget.label;
        widget.currentRow.indicatorPath = window.location.href;
        widget.currentRow.messageType = isChartTypeWidget ? 'chart' : 'grid';

        if (isChartTypeWidget) {
            widget.currentRow.indicator = containerWidget.chart.getBase64();
            notificate(widget);
        } else {
            ZhReports.generateReport({}, containerWidget, function (response) {
                UtilitiesController.getProductBaseUrl().then(function(url){
                    widget.currentRow.indicator = response.path.split("[KEY]=")[1];
                    notificate(widget);
                });
            }, function(error){

            }, 'XLS');
        }
    };

    this.buildView = function () {
        ChartViewService.setTitleWidgetOnEnter();
        needClearFilter = true;
        UtilitiesController.hashToSearch();
        var viewLevel;
        if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined) {
            viewLevel = +$location.$$search[ViewService.VIEW_LEVEL];
        } else {
            viewLevel = +getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.VIEW_LEVEL];
        }
        var serieColors;
        if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined) {
            serieColors = $location.$$search.serieColors ? $location.$$search.serieColors : 0;
        } else {
            serieColors = getLocalVar('indicatorNames')[getLocalVar('indicatorName')].serieColors ? getLocalVar('indicatorNames')[getLocalVar('indicatorName')].serieColors : 0;
        }

        if (viewLevel >= 0) {
            var viewId = getViewId(viewLevel);
            var drillDownParameters = DrillDownService.getDrillDownParameters(viewLevel);

            if (!drillDownParameters.period && viewLevel > 0) {
                drillDownParameters.period = null;
            }

            if (templateManager.container.preview && viewLevel >= 1) {
                templateManager.container.getWidget("informationStripe").getAction("back").isVisible = true;
            }

            if (viewId) {
                self.buildIndicatorView(viewId, drillDownParameters, serieColors);
            }
        }
    };

    function getViewId(viewLevel) {
        var viewId = UtilitiesController.getUrlParamAtIndex(ViewService.INDICATOR_VIEW_ID, viewLevel);
        if (!viewId) {
            viewId = localStorageEngine.getLocalVar(ViewService.INDICATOR_VIEW_ID);
        }

        return viewId;
    }

    function getUrlParametersByStructure(viewLevel, filterRow) {
        var groupByStructure = UtilitiesController.getUrlParamAtIndex(ViewService.GROUP_BY_STRUCTURE, viewLevel);
        if (groupByStructure) {
            filterRow.structureId = UtilitiesController.getUrlParamAtIndex(ViewService.STRUCTURE_ID, viewLevel);
            filterRow.structureTypeId = UtilitiesController.getUrlParamAtIndex(ViewService.STRUCTURE_TYPE_ID, viewLevel);
            filterRow.isStructureDrillDown = UtilitiesController.getUrlParamAtIndex(ViewService.IS_STRUCTURE_DRILL, viewLevel);
            filterRow.groupByStructure = groupByStructure;
            filterRow.structureName = UtilitiesController.getUrlParamAtIndex(ViewService.STRUCTURE_NAME, viewLevel);
        }
        filterRow.isViewPreview = templateManager.container.preview;
        return filterRow;
    }

    this.buildIndicatorView = function (viewId, filterRow, serieColors) {
        this.removeChartWidgets();
        filterRow = filterRow || {};
        filterRow.viewId = viewId;
        filterRow.serieColors = serieColors;

        var viewLevel;
        if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined) {
            viewLevel = +$location.$$search[ViewService.VIEW_LEVEL];
        } else {
            viewLevel = +getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.VIEW_LEVEL];
        }
        filterRow = getUrlParametersByStructure(viewLevel, filterRow);

        return ChartViewService.buildIndicatorView(viewId, filterRow).then(function (widget) {
            getActionSendNotification().isVisible = !!widget.chart || widget.template == "widget/grid.html";
            var filterWidget = templateManager.container.getWidget('filterWidget');

            self.fillStripeWidgetProperties(widget.parameters, filterRow);
            ChartViewService.setHeaderInfoFromWidgetParameters(widget);

            filterWidget.indicatorWidget = widget;
            filterWidget.getField('orderByTotal').checked = widget.orderByTotal;
            if (needClearFilter) {
                self.fillFilterWidgetProperties(widget, filterRow).then(function () {
                    self.fillOrderBy(filterWidget);
                });
                needClearFilter = false;
            }
            if (widget.chartSeries && widget.chartSeries[0] && widget.chartSeries[0].serieType === "pie") {
                setTimeout(function () {
                    ChartViewService.setColorToPie(widget);
                }, 300);
            }
        }, function (response) {
            self.loadFail(response);
            ScreenService.goBack();
        });
    };

    this.fillStripeWidgetProperties = function (parameters, row) {
        var stripe = templateManager.container.getWidget('informationStripe');

        if (parameters) {
            stripe.getField('structure').value(row.structureName || parameters.structureName);

            var periodField = stripe.getField('period');
            var periodValue = ScreenService.i18n("All Period");

            if (parameters.period) {
                if (parameters.period[0] && parameters.period.length === 1) {
                    periodValue = parameters.period[0].start + ' ' + ScreenService.i18n("to") + ' ' + parameters.period[0].end;
                } else {
                    if (parameters.period.start && parameters.period.end) {
                        periodValue = parameters.period.start + ' ' + ScreenService.i18n("to") + ' ' + parameters.period.end;
                    }
                }
            }

            var drillDownField = stripe.getField('drillDownFilters');
            var drillDownValue = '';

            if (parameters.drillDownFilters && !Util.isEmptyObject(parameters.drillDownFilters)) {
                Object.keys(parameters.drillDownFilters).forEach(function (key, index, array) {
                    if (drillDownValue) {
                        drillDownValue += ' - ';
                    }
                    drillDownValue += parameters.drillDownFilters[key];
                });
            }

            periodField.value(periodValue);
            drillDownField.value(drillDownValue);
        }
    };

    this.fillFilterWidgetProperties = function (widget, row) {
        var parameters = widget.parameters;
        var filterWidget = templateManager.container.getWidget('filterWidget');
        this.clearFilterWidgetProperties();

        var indicatorId = parameters.indicatorId;

        if (indicatorId) {
            localStorageEngine.setLocalVar("indicatorId", indicatorId);

            setFilterWidgetReadOnlyFields(filterWidget, row);

            filterWidget.currentRow.structure = row.structureId || parameters.structureId;
            filterWidget.currentRow.structure_label = row.structureName || parameters.structureName;

            setFilterWidgetDimensionFields(widget, filterWidget, parameters);
            setDrillDownFilters(filterWidget, parameters.drillDownFilters);

            filterWidget.currentRow.fact = parameters.facts;

            self.setOrderByValues(parameters.orderByName, parameters.orderByLabel, (parameters.orderByDirection || "ASC"));
            setDefaultOrderByValues(parameters.orderByName, parameters.orderByLabel, (parameters.orderByDirection || "ASC"));

            filterWidget.currentRow.period = parameters.period;
            self.setIntervalFieldsNumber(filterWidget);

            setFilterWidgetFactFields(widget, filterWidget, parameters);
            return DataSourceService.findOneByIndicator(indicatorId).then(function (response) {
                var dataSource = response.dataset.dataSource[0];
                setFilterWidgetColumnsData(dataSource, filterWidget, widget);
                setFilterWidgetDimensionFilter(filterWidget);
                templateManager.updateTemplate();
            });
        }
    };

    function setFilterWidgetReadOnlyFields(filterWidget, row) {
        filterWidget.getField("structure_label").reload();
        filterWidget.getField("dimension_label").readOnly = !!row.postConditions;
        filterWidget.getField("fact").readOnly = !!row.postConditions;
    }

    function setDrillDownFilters(filterWidget, drillDownFilters) {
        var drillDownFiltersDataSource = [];
        var drillDownFilterValues = [];
        filterWidget.currentRow.drillDownFilters = [];
        var drillDownFilterData = {};

        for (var dimension in drillDownFilters) {
            drillDownFilterData = {
                'value': drillDownFilters[dimension],
                'dimension': dimension
            };
            drillDownFiltersDataSource.push(drillDownFilterData);
            drillDownFilterValues.push(drillDownFilters[dimension]);
        }

        filterWidget.currentRow.drillDownFilters = drillDownFilterValues;
        filterWidget.getField("drillDownFilters").dataSource.data = drillDownFiltersDataSource;
    }

    function setFilterWidgetDimensionFields(widget, filterWidget, parameters) {
        var dimensionLabelField = filterWidget.getField("dimension_label");

        if (widget.allowMoreThanOneDimension) {
            filterWidget.currentRow.dimension_label = parameters.dimensions;
            filterWidget.currentRow.dimension = parameters.dimensions;
            dimensionLabelField.template = "field/select-multiple.html";
            dimensionLabelField.valueField = 'name';
            delete (dimensionLabelField.outData);
        } else if (widget.isCrossTab) {
            var dimensionXField = filterWidget.getField("dimensionXLabel");
            var dimensionYField = filterWidget.getField("dimensionY");
            filterWidget.currentRow.dimension_label = parameters.dimensionName;
            filterWidget.currentRow.dimensionX = parameters.dimensionX;
            filterWidget.currentRow.dimensionY = parameters.dimensionY;
            filterWidget.currentRow.dimensionXLabel = parameters.dimensionXName;
            dimensionLabelField.isVisible = false;
            dimensionXField.isVisible = true;
            dimensionYField.isVisible = true;
            filterWidget.fieldGroups.forEach(function (fieldGroup) {
                if (fieldGroup.isDimensionsFilters) {
                    fieldGroup.isVisible = false;
                }
            });
        } else {
            filterWidget.currentRow.dimension = parameters.dimensionName;
            filterWidget.currentRow.dimension_label = parameters.dimensionName;
        }
    }

    function setFilterWidgetFactFields(widget, filterWidget, parameters) {
        var factField = filterWidget.getField("fact");

        if (widget.isCrossTab) {
            filterWidget.currentRow.fact = parameters.factLabel;
            filterWidget.currentRow.factValue = parameters.factValue;
            factField.template = "zh-select-autocomplete#field/select-autocomplete.html";
            factField.outData = {
                "name": "factValue",
                "label": "fact"
            };
        }
    }

    function setFilterWidgetDimensionFilter(filterWidget) {
        clearFilterWidgetDimensionFilter(filterWidget);

        var dimensionsField;
        var selectedDimensions = [];
        var widgetData = filterWidget.indicatorWidget.dataSource.data;
        var dimensions = filterWidget.getField('dimension_label').dataSource.data;

        var dimension = dimensions.map(function (elem) {
            return elem.name;
        });

        if (filterWidget.currentRow.dimension) {
            dimensionsField = filterWidget.currentRow.dimension;
        } else if (filterWidget.currentRow.dimension_label) {
            dimensionsField = filterWidget.currentRow.dimension_label;
        } else if (dimension.length > 0) {
            dimensionsField = dimension;
        }

        dimensionsField = Util.isArray(dimensionsField) ? dimensionsField : [dimensionsField];

        dimensions.forEach(function (dimension) {
            dimensionsField.forEach(function (selectedDimension) {
                if (dimension.name === selectedDimension) {
                    selectedDimensions.push(dimension);
                }
            });
        });

        buildDimensionFilterFields(selectedDimensions, widgetData, filterWidget);
    }

    function clearFilterWidgetDimensionFilter(filterWidget) {
        filterWidget.fields = filterWidget.fields.filter(function (field) {
            return field.fieldGroup !== "dimensionsFilters";
        });
    }

    function buildDimensionFilterFields(selectedDimensions, widgetData, filterWidget) {
        selectedDimensions.forEach(function (selectedDimension) {
            var dimension = selectedDimension.name;
            var fieldData = [];

            widgetData.forEach(function (data) {
                var dimensionValue = data[dimension];
                if (dimensionValue) {
                    var alreadyFetched = false;
                    fieldData.forEach(function (fetchedData) {
                        if (fetchedData[dimension] === dimensionValue) {
                            alreadyFetched = true;
                        }
                    });

                    if (!alreadyFetched) {
                        var dimensionData = {};
                        dimensionData[dimension] = dimensionValue;
                        fieldData.push(dimensionData);
                    }
                }
            });

            var filterDimensionField = constructFilterDimensionField(selectedDimension, fieldData);
            metaDataFactory.factoryField(filterDimensionField, filterWidget, true);
        });
    }

    function constructFilterDimensionField(fieldInfo, fieldData) {
        var filterField = {
            name: "DIMENSION_FILTER_" + fieldInfo.name,
            label: fieldInfo.label,
            class: 12,
            readOnly: false,
            showOnForm: true,
            valueField: fieldInfo.name,
            descriptionField: fieldInfo.name,
            template: "field/select-multiple.html",
            fieldGroup: "dimensionsFilters",
            dataSource: {
                data: fieldData
            }
        };

        return filterField;
    }

    function setFilterWidgetColumnsData(dataSource, filterWidget, widget) {
        var dimensionField = filterWidget.getField('dimension_label');
        var factField = filterWidget.getField('fact');

        dimensionField.dataSource.data = [];
        factField.dataSource.data = [];

        if (dataSource && Util.isArray(dataSource.columns)) {
            dataSource.columns.forEach(function (column) {
                if (column.isDimension) {
                    dimensionField.dataSource.data.push(column);
                } else if (column.isFact) {
                    factField.dataSource.data.push(column);
                }
                filterWidget.columnsMap[column.name] = column;
            });
            if (widget.isCrossTab) {
                filterWidget.getField("dimensionXLabel").dataSource.data = dimensionField.dataSource.data;
                filterWidget.getField("dimensionY").dataSource.data = dimensionField.dataSource.data;
            }
        }
    }

    this.clearFilterWidgetProperties = function () {
        var filterWidget = templateManager.container.getWidget('filterWidget');

        filterWidget.columnsMap = {};
        filterWidget.currentRow.structure = "";
        filterWidget.currentRow.structure_label = "";
        filterWidget.currentRow.dimension = "";
        filterWidget.currentRow.dimension_label = "";
        filterWidget.currentRow.fact = [];
        filterWidget.currentRow.orderBy = "";
        filterWidget.currentRow.orderByLabel = "";
        filterWidget.currentRow.orderByDirection = "";
        filterWidget.getField("period").clearValue();
    };

    function buildFilterObject(viewId, filterWidget) {
        var orderBy = {};
        if (filterWidget.currentRow.orderBy) {
            orderBy[filterWidget.currentRow.orderBy] = filterWidget.currentRow.orderByDirection;
        }

        builPeriodFilterObj(filterWidget);
        var period;
        if (!(filterWidget.currentRow.period === undefined || filterWidget.currentRow.period === null)) {
            period = filterWidget.currentRow.period.period;
            period = period && period.name ? period.name : filterWidget.currentRow.period;
        }
        var dimensionColumns = filterWidget.getField('dimension_label').dataSource.data.map(function (column) {
            return column.name;
        });

        var dimensions = filterWidget.currentRow.dimension;
        var dimensionsFilters = buildDimensionFilterObj(filterWidget, dimensions);
        var dimensionConditions = dimensionsFilters ? ExpressionInterpreter.readComparisonExpression(dimensionsFilters, "previous", dimensionColumns, dimensions) : null;
        var drillDownFilters = buildDrillDownFilters(filterWidget);
        var orderByTotal = filterWidget.getField('orderByTotal').checked;

        var fact = filterWidget.currentRow.fact;
        var widget = filterWidget.indicatorWidget;
        if (widget.isCrossTab) {
            fact = [filterWidget.currentRow.factValue];
            dimensions = Util.clone(filterWidget.currentRow.dimensionY);
            dimensions.push(filterWidget.currentRow.dimensionX);
        }
        return {
            'viewId': viewId,
            'structureId': filterWidget.currentRow.structure,
            'structureName': filterWidget.currentRow.structure_label,
            'period': period,
            'dimension': dimensions,
            'dimensionX': filterWidget.currentRow.dimensionX,
            'dimensionY': filterWidget.currentRow.dimensionY,
            'facts': fact,
            'orderBy': orderBy,
            'previousConditions': dimensionConditions,
            'drillDownFilters': drillDownFilters,
            'orderByTotal': orderByTotal
        };
    }

    function builPeriodFilterObj(filterWidget) {
        if (periodNumber > 0) {
            UtilitiesController.cleanInterval(filterWidget.currentRow);
            var periodsValues = [filterWidget.currentRow.period];
            var ip = 1;
            var periodValue = null;
            for (var i = 1; i <= periodNumber; i++) {
                periodValue = undefined;
                UtilitiesController.cleanInterval(filterWidget.currentRow, "period_" + ip);
                var stop = 0;
                while (periodValue === undefined && stop <= 10) {
                    periodValue = filterWidget.currentRow["period_" + ip];
                    delete filterWidget.currentRow["period_" + ip];
                    ip++;
                    stop++;
                }
                periodsValues.push(periodValue);
            }

            filterWidget.currentRow.period = periodsValues;
        } else {
            UtilitiesController.cleanInterval(filterWidget.currentRow);
        }
    }

    function buildDimensionFilterObj(filterWidget, dimensions) {
        dimensions = Util.isArray(dimensions) ? dimensions : [dimensions];
        var dimensionConditions = "";
        dimensions.forEach(function (dimension) {
            var condition = "";

            var dimensionField = filterWidget.currentRow['DIMENSION_FILTER_' + dimension];
            if (dimensionField && !Util.isEmptyObject(dimensionField)) {
                dimensionField.forEach(function (dimensionValue) {
                    if (typeof dimensionValue === "string") {
                        condition += dimension + " == '" + dimensionValue + "' || ";
                    } else {
                        condition += dimension + " == " + dimensionValue + " || ";
                    }
                });

                condition = removeSpecialCaractersFromString(condition);

                dimensionConditions += "(" + condition + ")" + " && ";
            }
        });

        dimensionConditions = removeSpecialCaractersFromString(dimensionConditions);
        return dimensionConditions;
    }

    function removeSpecialCaractersFromString(string) {
        if ((string.substr(-4) === " && ") || (string.substr(-4) === " || ")) {
            string = string.slice(0, -4);
        }
        return string;
    }

    function buildDrillDownFilters(filterWidget) {
        var drillDownFilters = {};
        var drillValues = filterWidget.currentRow.drillDownFilters;
        var drillFilters = filterWidget.getField("drillDownFilters").dataSource.data;

        if (drillValues && drillValues.length > 0) {
            drillValues.forEach(function (value) {
                var filter = drillFilters.filter(function (drillFilter) {
                    return drillFilter.value === value;
                })[0];
                if (filter && filter.dimension) {
                    drillDownFilters[filter.dimension] = filter.value;
                }
            });
        }

        return drillDownFilters;
    }

    this.filterWidgetView = function (filter) {
        var filterWidget = {};
        if (getLocalVar("filterWidget")) {
            filterWidget = getLocalVar("filterWidget");
        }
        filterWidget[getLocalVar('indicatorName')] = {};
        filterWidget[getLocalVar('indicatorName')].currentRow = filter.currentRow;
        filterWidget[getLocalVar('indicatorName')].periodNumber = periodNumber;
        localStorageEngine.setLocalVar("filterWidget", filterWidget);
        if (!("period" in filter.currentRow)) {
            filter.currentRow.period = [];
        }
        this.rebuildIndicatorView(true, filter);
        ScreenService.closePopup();
        templateManager.container.getWidget('informationStripe').activate();
    };

    this.returnBuildPreview = function () {
        ChartViewService.setIndicatorNamePreview();
        self.returnBuild();
    };

    this.returnBuild = function () {
        widget = templateManager.container.getWidget('filterWidget');
        if (getLocalVar("filterWidget") && getLocalVar("filterWidget")[getLocalVar('indicatorName')] != undefined && (widget.getField("drillDownFilters").dataSource.data.length === 0 || isReturningFromDrillDown)) {
            widget.currentRow = getLocalVar("filterWidget")[getLocalVar('indicatorName')].currentRow;
            periodNumber = getLocalVar("filterWidget")[getLocalVar('indicatorName')].periodNumber;
            periodUnusedIds = PeriodsManagerService.addNewInterval(templateManager.container.getWidget('filterWidget'), periodNumber, periodUnusedIds, "ChartViewController");
            widget.indicatorWidget = templateManager.container.getWidget('informationStripe');
            widget.currentRow.serieStacked = widget.indicatorWidget.serieStacked;
            widget.currentRow.orderByTotal = widget.indicatorWidget.orderByTotal;
            if (isReturningFromDrillDown) {
                isReturningFromDrillDown = false;
                needClearFilter = true;
            }
            self.rebuildIndicatorView(true, widget);
        } else {
            this.buildView();
        }
    };

    var getViewLevel = function () {
        var viewLevel;
        if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined) {
            viewLevel = +$location.$$search[ViewService.VIEW_LEVEL];
        } else {
            viewLevel = +getLocalVar('indicatorNames')[getLocalVar('indicatorName')][ViewService.VIEW_LEVEL];
        }
        return viewLevel;
    };

    this.rebuildIndicatorView = function (hasFilterRow, filter) {
        this.removeChartWidgets();
        var filterWidget = filter;

        var viewLevel = getViewLevel();
        var viewId = getViewId(viewLevel);
        if (viewId) {
            var filterRow;
            if (hasFilterRow) {
                filterRow = buildFilterObject(viewId, filterWidget);
            }
            this.buildIndicatorView(viewId, filterRow).then(function () {
                setFilterWidgetDimensionFilter(filterWidget);
                var viewWidget = templateManager.container.widgets[1];
                self.setIntervalFieldsNumber(filterWidget);
                setUrlToNewParams(filterRow);
                viewWidget.redraw();
            });
        }
    };

    function setUrlToNewParams(filterRow) {
        var search;
        if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined) {
            search = $location.$$search;
        } else {
            search = getLocalVar('indicatorNames')[getLocalVar('indicatorName')];
        }
        if (filterRow.period) {
            search[ViewService.PERIOD_START] = filterRow.period.start;
            search[ViewService.PERIOD_END] = filterRow.period.end;
        }
        var indicatorNames = getLocalVar('indicatorNames');
        indicatorNames[getLocalVar('indicatorName')] = $location.search(search).$$search;
        localStorageEngine.setLocalVar('indicatorNames', indicatorNames);
        var newLocation = $location.path('/product#' + ViewService.INDICATOR_VIEW_WINDOW).search('Name=' + getLocalVar('indicatorName')).hash('');
    }

    this.removeChartWidgets = function () {
        templateManager.container.widgets.splice(1);
        templateManager.container.label = '';

        var stripe = templateManager.container.getWidget('informationStripe');
        if (stripe) {
            delete stripe.params;
            stripe.getField('structure').value("");
            stripe.getField('period').value("");
            ScreenService.setHeaderInfo('');
        }
    };

    this.openFilterWidget = function (filterWidget) {
        ScreenService.openPopup(filterWidget);
    };

    this.loadFail = function (response) {
        ChartViewService.loadFail(response);
    };

    this.dimensionLabelOnChange = function (filterWidget) {
        self.cleanAndSetOrderByValues(filterWidget);

        var widget = filterWidget.indicatorWidget;
        var viewId = widget._id;

        var filterRow = buildFilterObject(viewId, filterWidget);
        ChartViewService.setWidgetConditions(widget.previousConditions, filterRow, 'previousConditions');
        ChartViewService.setWidgetConditions(widget.postConditions, filterRow, 'postConditions');

        ChartViewService.fetchWidgetData(widget, filterRow).then(function () {
            setFilterWidgetDimensionFilter(filterWidget);
        });
    };

    var getColumnLabelByName = function (widget, nameField) {
        var name = nameField;
        widget.columns.some(function (column) {
            if (column.name == nameField) {
                name = column.label;
                return true;
            }
        });
        return name;
    };

    this.setIsReturningFromDrillDown = function (value) {
        isReturningFromDrillDown = value;
    };

    this.setFieldsInCrossTab = function (widget) {
        if (widget && widget.isCrossTab) {
            widget.fields = [];
            if (widget.dataSource.data[0]) {
                var parameters = widget.parameters;
                var nameFields = parameters.dimensionY;
                var dimensionNames = angular.copy(nameFields);
                dimensionNames.push(parameters.dimensionName);
                var mask = widget.labelPropertyMask;
                nameFields = nameFields.concat(widget.dataSource.data[0].ZmartFields);
                var fixedColumns = widget.fixedColumns;
                if (fixedColumns < 1) {
                    fixedColumns = 1;
                }
                factoredFieldInCrossTab(nameFields, widget, fixedColumns, parameters.dimensionY.length, mask, dimensionNames);
                if (Array.isArray(widget.yAxisProperty)) {
                    widget.yAxisProperty.forEach(function (nameField, index) {
                        widget.fields[index].label = getColumnLabelByName(widget, nameField);

                    });
                } else {
                    widget.fields[0].label = getColumnLabelByName(widget, widget.yAxisProperty);
                }
            }
            ChartViewService.applyCssStyleCrossTab(widget).then(function () {
                UtilitiesController.windowResizeKanban(500);
                widget.redraw(true);
                widget.dataSource.data.forEach(function (row) {
                    row.zmartApplyCss = true;
                });
            });
            widget.fields.forEach(function (field) {
                widget.dataSource.data.forEach(function (data) {
                    if (data[field.name] == undefined) {
                        data[field.name] = widget.emptyLabel;
                    }
                });
            });
        }
    };

    var factoredFieldInCrossTab = function (namefields, widget, fixedColumns, dimensions, mask, dimensionNames) {
        widget.namefieldsFact = [];
        namefields.forEach(function (namefield, index) {
            var labelField = UtilitiesController.transliterateString(namefield);
            if (-1 == dimensionNames.indexOf(labelField)) {
                widget.namefieldsFact.push(labelField);
            }
            widget.fields.push(GenericFieldFactory.factoryFieldText(labelField, namefield, widget, index < fixedColumns, index < dimensions, mask));
        });
    };

    this.cleanAndSetOrderByValues = function (filterWidget) {
        self.setOrderByValues("", "", "");
        self.fillOrderBy(filterWidget);
    };

    this.fillOrderBy = function (filterWidget) {
        var row = filterWidget.currentRow;
        var currentFact = row.fact;

        var orderByField = filterWidget.getField('orderByLabel');

        var orderByFieldNames = ['orderByLabel', 'orderByDirection'];
        var indicatorWidget = filterWidget.indicatorWidget;
        var currentDimension = row.dimension;

        if (filterWidget.indicatorWidget.isCrossTab) {
            currentFact = [row.factValue];
            currentDimension = filterWidget.currentRow.dimensionX;
        }

        var dimension = filterWidget.columnsMap[currentDimension];
        var orderBy = [];
        orderBy.push(defaultOrderBy);

        if (Util.isArray(currentFact)) {

            if (currentFact.length === 1 && !filterWidget.currentRow.orderBy) {
                var fact = filterWidget.columnsMap[currentFact[0]];
                if (fact && fact.__isSelected) {
                    delete fact.__isSelected;
                }

                orderBy.push(buildOrderByColumn(fact));
                self.setOrderByValues(fact.name, fact.label, (fact.direction || "asc"));
            } else {
                currentFact.forEach(function (factName) {
                    var fact = filterWidget.columnsMap[factName];
                    if (fact) {
                        orderBy.push(buildOrderByColumn(fact));
                    }
                });
            }
        }

        if (!filterWidget.currentRow.orderBy) {
            self.setOrderByValues(defaultOrderBy.name, defaultOrderBy.label, defaultOrderBy.direction);
        }

        if (dimension && dimension.isSortable) {
            orderBy.push(buildOrderByColumn(dimension));
            if (!filterWidget.currentRow.orderBy) {
                self.setOrderByValues(dimension.name, dimension.label, dimension.direction);
            }
        }

        if (orderBy.length !== 0) {
            this.enableFields(filterWidget.fields, orderByFieldNames);
        } else {
            this.disableFields(filterWidget.fields, orderByFieldNames);
        }

        orderByField.dataSource.data = Util.clone(orderBy);
        templateManager.updateTemplate();
    };


    this.disableFields = function (fields, fieldsToDisable) {
        fields.forEach(function (field) {
            if (fieldsToDisable.indexOf(field.name) !== -1) {
                field.readOnly = true;
            }
        });
    };

    this.enableFields = function (fields, fieldsToEnable) {
        fields.forEach(function (field) {
            if (fieldsToEnable.indexOf(field.name) !== -1) {
                field.readOnly = false;
            }
        });
    };

    this.setOrderByValues = function (orderBy, orderByLabel, direction) {
        var widget = templateManager.container.getWidget('filterWidget');
        widget.currentRow.orderBy = orderBy;
        widget.currentRow.orderByLabel = orderByLabel;
        widget.currentRow.orderByDirection = direction.toUpperCase();
    };

    var setDefaultOrderByValues = function (name, label, direction) {
        defaultOrderBy.name = name;
        defaultOrderBy.label = label;
        defaultOrderBy.direction = direction.toUpperCase();
    };

    var buildOrderByColumn = function (column) {
        return {
            "name": column.name,
            "label": column.label,
            "direction": !!column.direction ? column.direction.toUpperCase() : "ASC"
        };
    };

    this.setIntervalFieldsNumber = function (viewWidget) {
        if (!inSerInterval) {
            inSerInterval = true;
            PeriodsManagerService.setIntervalFieldsNumber(viewWidget, periodNumber, "ChartViewController").then(function (response) {
                periodNumber = response;
                periodUnusedIds = [];
                inSerInterval = false;
            }).catch(function () {
                inSerInterval = false;
            });
        }
    };

    this.addNewInterval = function (viewWidget) {
        periodNumber++;
        periodUnusedIds = PeriodsManagerService.addNewInterval(viewWidget, periodNumber, periodUnusedIds, "ChartViewController");
    };

    this.removeInterval = function (viewWidget, relatedPeriod) {
        periodUnusedIds = PeriodsManagerService.removeInterval(viewWidget, relatedPeriod, periodNumber, periodUnusedIds);
        periodNumber--;
    };

    this.openChartViewConfig = function () {
        var viewWidget = templateManager.container.widgets[1];
        var viewId = viewWidget.id;
        sessionStorage.setItem('indicator', viewWidget.indicator);
        viewWidget.currentRow = {};
        delete viewWidget.changed;

        WidgetService.findWidgetById(viewId).then(function (response) {
            var viewSettingsWidget = templateManager.container.getWidget('view');
            var chartView = response.dataset.widget[0];
            viewSettingsWidget.currentRow = chartView;
            ScreenService.openSwipe(viewSettingsWidget);
            viewSettingsWidget.edit();
            self.onEnterViewSettingsWidget(viewSettingsWidget);
        });
    };

    this.onEnterViewSettingsWidget = function (viewSettingsWidget) {
        var chartView = viewSettingsWidget.currentRow;
        IndicatorService.getIndicatorById(chartView.indicator).then(function (response) {
            var indicator = response.dataset.indicator[0];
            var viewTypeId = viewSettingsWidget.currentRow.viewTypeId;
            WidgetService.buildViewWidgetProperties(viewTypeId, viewSettingsWidget, indicator.dataSource, true).then(function () {
                self.setDateFilterColumn(viewSettingsWidget, indicator);
                self.updateFieldsVisibility(chartView);
            });
        });
    };

    this.setDateFilterColumn = function (widget, indicator) {
        var dateColumns = indicator.dataSourceColumns;
        var newDateColuns = [];
        if (Util.isArray(dateColumns)) {
            newDateColuns = dateColumns.filter(function (column) {
                return column.isDate;
            });
        }
        widget.getField('dateFilterColumnLabel').dataSource.data = newDateColuns;
    };

    this.setProperties = function (viewTypeField, viewTypeId, property) {
        return viewTypeField.dataSource.data.filter(function (row) {
            return row._id === viewTypeId;
        }).map(function (viewType) {
            return viewType[property];
        }).shift();
    };

    this.updateFieldsVisibility = function (row) {
        var viewTypeId = row.viewTypeId;
        var viewWidget = templateManager.container.getWidget("view");
        var viewTypeField = viewWidget.getField("viewTypeId");

        disablePropertyFields(viewWidget);
        var properties = this.setProperties(viewTypeField, viewTypeId, 'properties');

        if (properties) {
            WidgetService.enableFields(properties);
        }
    };

    this.closeViewSettingsAfterSave = function (saveAsNew, viewWidget) {
        ScreenService.closePopup();
        ScreenService.closeSwipe();

        var widgetToActivate = templateManager.container.getWidget("informationStripe");
        widgetToActivate.activate();

        if (saveAsNew) {
            var indicatorFilter = {
                name: viewWidget.currentRow._id
            };

            openIndicator(indicatorFilter);
        } else {
            self.buildView();
        }
    };

    function openIndicator(indicatorFilter) {
        templateManager.interations.openIndicatorView(indicatorFilter);
        templateManager.updateTemplate();
        ScreenService.toggleSideMenu();
    }

    function disablePropertyFields(viewWidget) {
        var propertiesFieldGroups = ["dimensionProperties", "factProperties", "properties", "propertiesBar",
            "propertiesLine", "card1", "card2", "card3"];

        viewWidget.fields.forEach(function (field) {
            var fieldGroup = field.fieldGroup;
            if (fieldGroup && UtilitiesController.isStringInArray(fieldGroup, propertiesFieldGroups)) {
                field.isVisible = false;
            }
        });

        templateManager.updateTemplate();
    }

    this.setFieldPrecision = function () {
        var viewTypeId = templateManager.container.getWidget('view').currentRow.viewTypeId;
        var viewWidget = templateManager.container.getWidget("view");
        var viewTypeField = viewWidget.getField("viewTypeId");

        var properties = this.setProperties(viewTypeField, viewTypeId, 'properties');
        if (properties) {
            var isVisible = WidgetService.checkFieldToEnable(properties);
            WidgetService.enablePrecisionField(properties, isVisible);
        }
    };

    this.showViewDescription = function () {
        var widget = templateManager.container.widgets[1];
        var viewDescription = widget.viewDescription;
        UtilitiesController.showMessage(viewDescription);
    };

    this.fillSerieStacked = function (filterWidget, container) {
        filterWidget.indicatorWidget = container.widgets[1];
        filterWidget.currentRow.serieStacked = filterWidget.indicatorWidget.serieStacked;
        filterWidget.currentRow.orderByTotal = filterWidget.indicatorWidget.orderByTotal;
    };


    this.periodOnChange = function (widget) {
        widget.fields.forEach(function (field, index, widget) {
            if (field.fieldGroup === "period" && field.name === 'period') {
                if (field.label.indexOf('(all period)') != -1 && field.value() != undefined) {
                    field.label = 'Period';
                } else if (field.value() === undefined && field.label.indexOf('(all period)') == -1) {
                    widget[index].label = widget[index].label + " (all period)";
                }
            }
        });
    };
}

ChartViewController.containerName = "chartView";
Configuration(function (ContextRegister) {
    ContextRegister.register('ChartViewController', ChartViewController);
});


// FILE: assets/js/controllers/ViewController.js
function ViewController(ScreenService, templateManager, DataSourceService, ViewService, ViewTypeService,
UtilitiesController, MainDashboardController, ExpressionInterpreter, GenericDashboardController, GenericDashboardService,
PeriodsUtilities, MessageService, restEngine) {

	var DASHBOARD_WIDGET = 'dashboardConfiguration';
	var self = this;

	this.changeWidgetVisibility = function(widgets, isVisible) {
		widgets.forEach(function(widget) {
			widget.isVisible = isVisible;
		});
	};

	self.setParentMenuName = function(widget) {
		var parentMenuId = widget.currentRow.parent;
		if (parentMenuId) {
			ViewService.findView(parentMenuId).then(
				function(response) {
					widget.currentRow.parentName = response.dataset.view[0].name;
				}
			);
		}
	};

	self.setParentMenuFilter = function(widget) {
		var menu_id = widget.currentRow._id;
		var parentMenuField = widget.getField('parentName');
		parentMenuField.dataSourceFilter = [
			{
				"name": "type",
				"operator": "=",
				"value": "M"
			}
		];
		if (menu_id) {
			parentMenuField.dataSourceFilter.push({
				"name": "_id",
				"operator": "NOT_IN",
				"value": [menu_id]
			});
		}
		parentMenuField.reload();
	};

	self.setIndicatorData = function(widget) {
		if (widget.currentRow.indicators) {
			widget.widgets[0].dataSource.data = widget.currentRow.indicators;
		} else {
			widget.widgets[0].dataSource.data = [];
		}
	};

	self.newRow = function(widget) {
		this.changeWidgetVisibility(widget.widgets, false);
		widget.newRow();
	};

	self.edit = function(widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		this.changeWidgetVisibility(widget.widgets, false);
		widget.edit();
		self.setAccordionPropertiesVisibility(widget);
	};

	self.cancel = function(widget) {
		if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
			widget.list();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
		this.changeWidgetVisibility(widget.widgets, true);
	};

	self.back = function(widget) {
		UtilitiesController.masterDetailBackToList(widget).then(function() {
			UtilitiesController.deleteChangedFlag(widget);
		});
	};

	self.goBack = function(){
		ScreenService.openWindow(DASHBOARD_WIDGET);
		var widget = templateManager.container.getActiveWidget();
		UtilitiesController.deleteChangedFlag(widget);
	};

	self.backParentList = function(widget){
		var dashboardWidget = templateManager.container.getWidget(DASHBOARD_WIDGET);
		widget.parent.list();

		UtilitiesController.closeSwipeAndActivateParent(widget);
		UtilitiesController.deleteChangedFlag(dashboardWidget);
	};

	var save = function(widget, callback) {
		if (widget.isValid()) {
			var parentNameBkp = widget.currentRow.parentName;
			widget.currentRow.parentName = undefined;

			self.validatecolorField(widget, 'headerColor');
			self.validatecolorField(widget, 'headerGradient');

			return widget.saveRow().then(
				function(response) {
					var dataSourceName = widget.dataSource.name.substr(1);
					widget.currentRow._id = response.dataset[dataSourceName][0]._id;
					widget.currentRow.changed = false;
					widget.currentRow.__is_new = false;
					widget.currentRow.parentName = parentNameBkp;
					widget.view();
					widget.getField('parentName').reload();
					self.changeWidgetVisibility(widget.widgets, true);
					if (callback) {
						callback();
					}
				},
				function(response) {
					widget.currentRow.parentName = parentNameBkp;
				}
			);
		} else {
			MessageService.alertNotificationByCode('REQUIRED_FIELDS');
		}
	}; 

	this.validatecolorField = function(widget, fieldName) {
		if (widget.currentRow[fieldName] && typeof widget.currentRow[fieldName] != "string") {
			widget.currentRow[fieldName] = formatColor(widget.currentRow[fieldName]);
		}
	};

	self.saveAndUpdateMenu = function(widget) {
		return save(widget, function() {
			ViewService.updateMenu();
		});
	};

	self.saveAndUpdateDashboard = function(widget, dontReload) {
		if(widget.currentRow.dashboardTemplate === 'tab' && widget.currentRow.indicators){
			widget.currentRow.indicators.forEach(function(indicator){
				delete indicator.displayMode;
			});
		}
		delete widget.currentRow.changed;
		return save(widget, function() {
			MainDashboardController.forceDashboardReload();
			ScreenService.closePopup();
			ScreenService.closeSwipe();
			if(!dontReload){
				widget.reload();
			}
		});
	};
	
	self.dashboardSaveAs = function(widget) {
		widget.currentRow.__is_new = true;
		delete widget.currentRow._id;
		delete widget.currentRow.oldId;
		if(widget.currentRow.dashboardTemplate === 'tab' && widget.currentRow.indicators){
			widget.currentRow.indicators.forEach(function(indicator){
				delete indicator.displayMode;
			});
		}
		delete widget.currentRow.changed;
		return save(widget, function() {
			MainDashboardController.forceDashboardReload();
			ScreenService.closePopup();
			ScreenService.closeSwipe();
			widget.reload();
		});
	};

	var formatColor = function (color) {
		var r = Math.round(color._r),
			g = Math.round(color._g),
			b = Math.round(color._b);
		return '#' + ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1);
	};

	self.deleteRow = function(widget) {
		return widget.removeCheckedRows().then(function() {
			ViewService.updateMenu();
		});
	};

	self.deleteDashboard = function(widget) {
		return widget.removeCheckedRows().then(function() {
			MainDashboardController.forceDashboardReload();
		});
	};

	self.dashboardAfterMoveRow = function(widget){
		UtilitiesController.clearChangedRows(widget);
		this.setIndicatorData(widget);
		this.setParentMenuName(widget);
		this.setAccordionPropertiesVisibility();
		UtilitiesController.deleteChangedFlag(widget);
	};

	this.setAccordionPropertiesVisibility = function(){
		var widget = templateManager.container.getWidget(DASHBOARD_WIDGET);
		var visibility = widget.currentRow.dashboardTemplate === "accordion";
		widget.fieldGroups.forEach(function(fieldGroup){
			if(fieldGroup.name === "accordionProperties"){
				fieldGroup.isVisible = visibility;
			}
		});
	};

	var setFilterDashboard = function(dimensionValue, widget, dimensionProperty, row, filterWithPeriod) {
		UtilitiesController.killAllAsyncProcesses();
		var filter = {};
		if (dimensionValue == undefined &&
			widget.dataSource && widget.dataSource.data &&
			widget.dataSource.data[0] && widget.dataSource.data[0][dimensionProperty]) {
			dimensionValue = widget.dataSource.data[0][dimensionProperty];
		}
		filter[dimensionProperty] = [dimensionValue];
		var filterWidget = templateManager.container.getWidget('filterWidget');
		filterWidget.dashboardFilters = Util.clone(filter);
		if(filterWithPeriod && widget.parameters.period){
			filter.period = widget.parameters.period;
			if(!filter.period.start && filter.period.period){
				filter.period = filter.period.period;
			}
		}else if (row.isDashboardComparePeriod) {
			filter.period = {};
			filter.period.start = row.periodClicked.start;
			filter.period.end = row.periodClicked.end;
		}
		filter.filterName = dimensionValue;
		templateManager.container.getWidget('filterWidget').currentRow = filter;
		var parentDash = widget.parent.name;
		filter.filterId = parentDash;
		var dashboards = templateManager.container.getWidget('dashboard_tabbed').widgets;
		dashboards = dashboards.filter(function (dashboard) {
			return dashboard.name === parentDash;
		});
		var filterRow = GenericDashboardController.buildFilterObject(filter);
		var fixedWidget = widget._id;
		var isFilter = dashboards[0].isFilter = true;

		dashboards.forEach(function (dashboard) {
			if (dashboard.container.name == "mainDashboard") {
				dashboard.actions.forEach(function (action) {
					if (action.name == "back" || action.name == "BACK") {
						action.isVisible = false;
					}
				});
			}
		});


		GenericDashboardService.reloadDashboards(dashboards, filterRow, fixedWidget);
	};

	this.filterDashboardByWidget = function(widget, row, filterWithPeriod){
		var dimensionProperty = widget[widget.dimensionProperty];
		var dimensionValue = row[dimensionProperty];
		if(!filterWithPeriod){
			filterWithPeriod = widget.actionOnClick && widget.actionOnClick == "filterWithPeriod";
		}
		setFilterDashboard(dimensionValue, widget, dimensionProperty, row, filterWithPeriod);

	};

    var getFirstValue = function (conditions) {
        var value = null;
        if (conditions) {
            if(conditions.expressions && conditions.expressions[0]){
                conditions = conditions.expressions[0];
            }
            if (conditions.firstValue instanceof Object) {
                value = getFirstValue(conditions.firstValue);
            } else {
                value = conditions.firstValue;
            }
        }
        return value;
    };


    var getSecondValue = function (conditions) {
        var value = null;
        if (conditions) {
            if(conditions.expressions && conditions.expressions[0]){
                conditions = conditions.expressions[0];
            }
            if (conditions.firstValue instanceof Object) {
                value = getSecondValue(conditions.firstValue);
            } else {
                value = conditions.secondValue;
            }
        }
        return value;
    };

    var buildFilterObject = function(dimensionProperty, dimensionValue) {
        var dimensionFilterInfo = {};
        dimensionFilterInfo.fieldName = dimensionProperty;
        dimensionFilterInfo.fieldLabel = dimensionProperty;
        dimensionFilterInfo.fieldData = [dimensionValue];
        if (!Util.isEmptyObject(dimensionFilterInfo)) {
            GenericDashboardService.buildFilterObject(dimensionFilterInfo);
        }
    };

	this.filterPreDashboardByWidget = function(widget, row){
		var dimensionProperty = getFirstValue(widget.parameters.previousConditions);
		var dimensionValue = getSecondValue(widget.parameters.previousConditions);
		buildFilterObject(dimensionProperty, dimensionValue);
		setFilterDashboard(dimensionValue, widget, dimensionProperty, row);

	};


	this.filterPosDashboardByWidget = function(widget, row){
		var dimensionProperty = getFirstValue(widget.parameters.postConditions);
		var dimensionValue = getSecondValue(widget.parameters.postConditions);
		buildFilterObject(dimensionProperty, dimensionValue);
		setFilterDashboard(dimensionValue, widget, dimensionProperty,  row);

	};

	this.verifySerieStacked= function(field){
		var widget = field.widget;
		var fieldOrderbyTotal = widget.getField('orderByTotal');
		if(!field.checked){
			fieldOrderbyTotal.checked = false;
			widget.currentRow.orderByTotal = false;
		}
	};

	this.verifyOrderByTotal= function(){
		var viewWidget = templateManager.container.getWidget('view');
		var fieldOrderbyTotal = viewWidget.getField('orderByTotal');
		if(!viewWidget.currentRow.serieStacked){
			fieldOrderbyTotal.checked = false;
			viewWidget.currentRow.orderByTotal = false;
			ScreenService.alertNotification("To order by total the field serie stacked  must be selected!");
		}
	};

    self.beforeGenerateReportByWidget = function(args){
        var widget = args.owner;
        widget.fields = widget.fields.filter(function(field){
            return field.reload;
        });
        var orderBy = false;
        widget.fields.some(function (field) {
            if (field.name == widget.parameters.orderByName) {
                orderBy = true;
                return true;
            }
        });
        if (orderBy) {
            orderBy = {
                "name": widget.parameters.orderByName,
                "direction": widget.parameters.orderByDirection
            };

            var hasOrderby = false;
            widget.dataSourceFilter.forEach(function (filter) {
                if (filter.operator == "ORDER_BY") {
                    hasOrderby = true;
                    if (-1 == filter.name.indexOf(orderBy)) {
                        filter.name.push(orderBy);
                    }
                }
            });
            if (!hasOrderby) {
                widget.dataSourceFilter.push({
                    operator: "ORDER_BY",
                    value: null,
                    "name": [orderBy]
                });
            }
        }


    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ViewController', ViewController);
});

// FILE: assets/js/controllers/DashboardIndicatorController.js
function DashboardIndicatorController (ViewController, PeriodsManagerService, ScreenService, UtilitiesController,
ZHPromise, ExpressionInterpreter, templateManager, ViewTypeService, MessageService, PeriodsUtilities,
MasterDetailUtilities, DataSourceManagerService) {

    var self = this;
    var DASHBOARD_WIDGET = 'dashboardConfiguration';
    var periodNumber = 0;
	var periodUnusedIds = [];
	var viewTypes;

	var getWidgetIndicators = function(){
		return templateManager.container.getWidget("indicators");
	};

	var getWidgetDashboardConfiguration = function(){
		return templateManager.container.getWidget(DASHBOARD_WIDGET);
	};

	var getFieldIndicatorLabelFromIndicatorsWidget = function(){
		return getWidgetIndicators().getField("indicator_label");
	};
	
	var getFieldOrderByLabelFromIndicatorsWidget = function(){ 
		return getWidgetIndicators().getField("orderBy_label");
	};

	var getFieldAccessProfilesFromDashboardConfigurationWidget = function(){
		return getWidgetDashboardConfiguration().getField("accessProfiles");
	};

	var getFieldFullAccessProfilesFromDashboardConfigurationWidget = function(){
		return getWidgetDashboardConfiguration().getField("fullAccessProfiles");
	};

	this.afterMoveRow = function(widget){
		UtilitiesController.clearChangedRows(widget);
		MasterDetailUtilities.afterMoveRow(widget, 2);
		return self.fillPropretiesAndOrderBy(widget).then(function(){
			self.setVisibilityOfIndicatorsFields(widget);
			self.fillActionOnClick(widget);
			return self.setIntervalFieldsNumber(widget).then(function(){
				return PeriodsUtilities.setIntervalsOnPeriodField(widget.getField('period'));
			});
		});
	};

	this.deleteIndicator = function(widget){
		UtilitiesController.confirmMessageByCode("DELETE_CONFIRMATION",{},{
			"type": 'CONFIRMATION',
			"success": function (){
				var myIndex = [];
				widget.getCheckedRows().forEach(function(checkedRow, index) {
					widget.dataSource.data.filter(
						function(element, index) {
							if (element === checkedRow) {
								myIndex.push(index);
							}
							return element === checkedRow;
						}
					);
				});

				for (var i = myIndex.length - 1; i >= 0; i--) {
					widget.dataSource.data.splice(myIndex[i], 1);
				}

				widget.parent.currentRow.indicators = widget.dataSource.data;

				ViewController.saveAndUpdateDashboard(widget.parent).then(function() {
					widget.activate();
				});
			}
		});
	};
	
	this.accessProfilesFieldOnChange = function(){
		var fullAccessProfilesField = getFieldFullAccessProfilesFromDashboardConfigurationWidget();
		var fullAccessProfiles = fullAccessProfilesField.value();
		if(fullAccessProfiles){
			var accessProfilesField = getFieldAccessProfilesFromDashboardConfigurationWidget();
			var accessProfiles = accessProfilesField.value();
			var selectedValues = fullAccessProfiles.filter(function(value){
				return accessProfiles.includes(value);
			});
			fullAccessProfilesField.value(selectedValues);
			fullAccessProfilesField.dataSource.clearCheckedRows();
			fullAccessProfilesField.dataSource.addCheckedRows(selectedValues);
			templateManager.updateTemplate();
		}
	};

	this.fullAccessProfilesFieldOnChange = function(){
		var accessProfilesField = getFieldAccessProfilesFromDashboardConfigurationWidget();
		var fullAccessProfilesField = getFieldFullAccessProfilesFromDashboardConfigurationWidget();

		var accessProfiles = accessProfilesField.value();
		if(!accessProfiles){
			accessProfiles = [];
		}
		var fullAccessProfiles = fullAccessProfilesField.value();

		var unselectedValues = fullAccessProfiles.filter(function(value){
			return !accessProfiles.includes(value);
		});
		if(unselectedValues.length > 0){
			accessProfilesField.dataSource.addCheckedRows(unselectedValues);
			accessProfilesField.value(accessProfilesField.dataSource.getCheckedRows().map(function(value){
				if(typeof value == "object" && value.NRPERFILACESSO){
					return value.NRPERFILACESSO;
				}
				return value;
			}));
			templateManager.updateTemplate();
		}
	};

    this.edit = function(widget) {
    	widget.backupRow = Util.clone(widget.currentRow);
    	ViewController.changeWidgetVisibility(widget.widgets, false);
    	widget.edit();
		PeriodsManagerService.resetAllPeriodFields(widget);
		reloadDataSourceColumns(widget);
	};
	
	var reloadDataSourceColumns = function(widget){
		var indicatorLabelField = getFieldIndicatorLabelFromIndicatorsWidget();
		indicatorLabelField.reload().then(function(){
			indicatorLabelField.dataSource.data.some(function(dataSourceData){
				if(dataSourceData.dataSource == indicatorLabelField.widget.currentRow.dataSource){
					widget.currentRow.dataSourceColumns = dataSourceData.dataSourceColumns;
					getFieldOrderByLabelFromIndicatorsWidget().dataSource.data = dataSourceData.dataSourceColumns;
					return true;
				}
			});
		});
	};

	this.cancel = function(widget){
	    if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
			widget.list();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.currentRow.period = widget.currentRow.backupPeriod || widget.currentRow.period;
			widget.view();
		}

		self.setIntervalFieldsNumber(widget);
		ViewController.changeWidgetVisibility(widget.widgets, true);
	};

	this.setIntervalFieldsNumber = function(widget) {
		widget.currentRow.backupPeriod = Util.clone(widget.currentRow.period);
		return PeriodsManagerService.setIntervalFieldsNumber(widget, periodNumber, 'DashboardIndicatorController').then(function(response){
			periodNumber = response;
			periodUnusedIds = [];

			return PeriodsManagerService.setPeriodsLabelsAndValues(widget);
		});
	};

	this.saveIndicator = function(widget) {
		if (widget.isValid()) {
			widget.currentRow.previousConditions = self.readComparisonExpression(widget, "previous");
			widget.currentRow.postConditions = self.readComparisonExpression(widget, "post");
			widget.currentRow.postGroupConditions = self.readComparisonExpression(widget, "postGroup");
			widget.parent.currentRow.changed = true;
			widget.currentRow.__is_new = false;

			ViewController.validatecolorField(widget, 'color');
			periodNumber = PeriodsManagerService.cleanInterval(widget.currentRow, "", periodNumber);

			if(periodNumber > 0) {
				widget.currentRow.period = PeriodsManagerService.getPeriodsValues(widget, periodNumber);
			}

			if(widget.currentRow.backupPeriod){
				delete widget.currentRow.backupPeriod;
			}

			widget.parent.currentRow.indicators = widget.dataSource.data;
			ViewController.saveAndUpdateDashboard(widget.parent, true).then(function() {
				widget.activate();
				widget.list();
				self.setIntervalFieldsNumber(widget);
			});
		}
	};

    this.fillPropretiesAndOrderBy = function(widget) {
		return self.fillPropreties(widget).then(function() {
			return self.fillOrderBy(widget);
		});
	};

	this.fillPropreties = function(widget) {
		var indicatorId = widget.currentRow.indicator;
		var dependentFields = ['widget_label'];
		if (!indicatorId) {
			setReadOnlyField(widget.fields, dependentFields, true);
			return ZHPromise.when();
		} else {
			setReadOnlyField(widget.fields, dependentFields, false);
			var widgetField = widget.getField('widget_label');
			widgetField.dataSourceFilter[0].value = indicatorId;
			return widgetField.reload();
		}
	};

	function setReadOnlyField(fields, fieldsToEnable, readOnly){
	    fields.forEach(function(field) {
			if (fieldsToEnable.indexOf(field.name) !== -1) {
				field.readOnly = readOnly;
			}
		});
	}

	this.fillOrderBy = function(widget) {
		var indicatorId = widget.currentRow.indicator;
		var widgetId = widget.currentRow.widget;
		var dependentFields = ['orderBy_label', 'orderByDirection'];

		if (!indicatorId || !widgetId) {
			setReadOnlyField(widget.fields, dependentFields, true);
			return ZHPromise.when();
		} else {
			setReadOnlyField(widget.fields, dependentFields, false);
			return DataSourceManagerService.getDataSourceById(widget.currentRow.dataSource).then(function(response){
				var orderByField = widget.getField('orderBy_label');
				var dataSourceColumns = [];

				if(widget.currentRow.dataSourceColumns){
					dataSourceColumns = widget.currentRow.dataSourceColumns;
				} else if (response.dataset && response.dataset.dataSource.length){
					dataSourceColumns = response.dataset.dataSource[0].columns;
				}

				orderByField.dataSource.data = [];
				dataSourceColumns.forEach(function(column){
					orderByField.dataSource.data.push(buildOrderByColumn(column));
				});

				setDefaultOrderByValue(orderByField);
			});
		}
	};

	function buildOrderByColumn(column) {
		return {
			"name": column.name,
			"label": column.label,
			"direction": !!column.direction ? column.direction.toUpperCase() : "ASC"
		};
	}

	function setDefaultOrderByValue(orderByField) {
		if (orderByField.dataSource.data.length === 1) {
			var orderBy = orderByField.dataSource.getFirstRow();
			var currentRow = orderByField.widget.currentRow;
			currentRow.orderBy = orderBy.name;
			currentRow.orderBy_label = orderBy.label;
			currentRow.orderByDirection = orderBy.direction.toUpperCase();
		}

		self.setEditabilityOrderBy(orderByField);
	}

	this.setEditabilityOrderBy = function(orderByLabelField) {
		var orderByDirectionField = orderByLabelField.widget.getField('orderByDirection');
		orderByDirectionField.readOnly = orderByLabelField.value() ? false : true;
	};

	this.loadViewTypes = function() {
		if (!viewTypes) {
			viewTypes = {};
			ViewTypeService.findAllWithProperties().then(function(response) {
				response.forEach(function(element) {
					viewTypes[element._id] = element;
				});
			});
		}
	};

	this.resetFilter = function(widget) {
		if(widget.currentRow.backupPeriod){
			widget.currentRow.period = widget.currentRow.backupPeriod;
		}
	};

	this.setVisibilityOfIndicatorsFields = function(widget){
		var displayModeVisibility = widget.parent.currentRow.dashboardTemplate === "accordion" ? true : false;
		UtilitiesController.setFieldsVisibility(widget, ['displayMode'], displayModeVisibility);
	};

	this.fillActionOnClick = function(widget){
		var actionField = widget.getField("actionOnClick");
		if (widget.currentRow.widget) {
			actionField.readOnly = false;
			if(!widget.currentRow.actionOnClick){
				var indicator = getDashboardWidget(widget)[0];
				if (indicator && indicator.hasDrillDown) {
					actionField.value("drillDown");
				} else {
					actionField.value("none");
				}
			}
		} else {
			delete widget.currentRow.actionOnClick;
		}
	};

	function getDashboardWidget(widget) {
		return widget.getField('widget_label').dataSource.data.filter(function(wid) {
			return wid._id === widget.currentRow.widget;
		});
	}

	function validateIndicatorDisplayMode(widget){
		var viewField = widget.getField("widget_label");
		var displayModeField = widget.getField("displayMode");

		var view = viewField.dataSource.data.filter(function(view){
			return view.label === widget.currentRow.widget_label;
		}).shift();

		if(view.chartType !== "dashboardCard" && widget.currentRow.displayMode === "header"){
			if (viewField.changed) {
				widget.currentRow.widget_label = "";
			} else if(displayModeField.changed){
				widget.currentRow.displayMode = "detailing";
			}

			MessageService.errorNotificationByCode('ONLY_DASHBOARD_CARD_ON_HEADER');
		}
	}

	this.displayModeOnChange = function(widget){
		if(widget.currentRow.widget_label){
			var displayModeField = widget.getField('displayMode');
			displayModeField.changed = true;
			validateIndicatorDisplayMode(widget);
			delete displayModeField.changed;
		}
	};

	this.indicatorOnChange = function(widget){
	    self.fillPropreties(widget);
	    self.resetFilter(widget);
	};

	this.widgetLabelOnChange = function(widget){
		if(widget.currentRow.widget_label){
			var viewField = widget.getField('widget_label');
			viewField.changed = true;
			validateIndicatorDisplayMode(widget);
			delete viewField.changed;
		}

		self.resetFilter(widget);
		self.fillOrderBy(widget).then(function(){
			self.fillActionOnClick(widget);
		});
	};

	this.roundValue = function(field) {
		var valueToRound = field.value();
		var moduleResult = valueToRound % 0.25;
		var valueToSet;
		if (moduleResult !== 0) {
			if (moduleResult < 0.125) {
				valueToSet = valueToRound - moduleResult;
			} else {
				valueToSet = valueToRound + (0.25 - moduleResult);
			}
			field.setValue(valueToSet);
		}
	};

	this.checkIfIndicatorHasDrillDown = function(widget){
		var actionField = widget.getField("actionOnClick");
		if (widget.currentRow.widget) {
			if (widget.currentRow.actionOnClick === "drillDown") {
				var indicator = getDashboardWidget(widget)[0];

				if (!indicator.hasDrillDown) {
					actionField.value("none");
					MessageService.showMessageByCode('CANT_ADD_DRILL_IF_HAS_NO_DRILL');
				}
			}
		} else {
			actionField.readOnly = true;
		}
	};

	this.readComparisonExpression = function (indicatorsWidget, previousOrPost) {
		try {
			var expression = null;
			if (indicatorsWidget.currentRow[previousOrPost + "ConditionsString"]) {
				var columnsNames = indicatorsWidget.currentRow.dataSourceColumns.map(function (column) {
					return column.name;
				});
				expression = ExpressionInterpreter.readComparisonExpression(indicatorsWidget.currentRow[previousOrPost + "ConditionsString"], previousOrPost, columnsNames, columnsNames);
				MessageService.successNotificationByCode('VALID_EXPRESSION');
			}
			return expression;
		} catch (e) {
			UtilitiesController.showMessage(e, "error");
		}
	};

	this.addNewInterval = function(indicatorWidget) {
		periodNumber++;
		periodUnusedIds = PeriodsManagerService.addNewInterval(indicatorWidget, periodNumber, periodUnusedIds, "DashboardIndicatorController");
	};

	this.removeInterval = function(indicatorWidget, relatedPeriod) {
		periodUnusedIds = PeriodsManagerService.removeInterval(indicatorWidget, relatedPeriod, periodNumber, periodUnusedIds);
		periodNumber--;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('DashboardIndicatorController', DashboardIndicatorController);
});

// FILE: assets/js/controllers/GoalsController.js
function GoalsController(templateManager, ScreenService, GoalNotificationsService, GoalsService, UtilitiesController,
StructureService) {

    var self = this;

    function getViewWidget() {
        return templateManager.container.getWidget('view');
    }

    function getIndicatorsWidget() {
        return templateManager.container.getWidget('indicators');
    }

    function getGoalsSettingsWidget() {
        return templateManager.container.getWidget('goalSettings');
    }

    this.goalsSettingsWidgetBeforeMoveRow = function(){
        UtilitiesController.deleteChangedFlag(getGoalsSettingsWidget());
    };

    this.goalSettingsActionEventBackList = function(){
        ScreenService.closeSwipe();
    };

    this.goalSettingsActionEventBackView = function(){
        ScreenService.closeSwipe();
    };

    this.goalSettingsWidgetAfterMoveRow = function() {
        var widget = getGoalsSettingsWidget();
        UtilitiesController.clearChangedRows(widget);
        self.setFieldsVisibility(widget);
        setGoalFilterValues(widget);
        setStructureData();
    };

    function setStructureData() {
        var indicatorWidget = getIndicatorsWidget();
        var indicatorId = indicatorWidget.currentRow._id;
        StructureService.getUserStructuresByIndicator(indicatorId).then(function(data){
            var goalsWidget = getGoalsSettingsWidget();
            data = data.dataset.getUserStructuresByIndicator;
            goalsWidget.getField('filteredStructure').dataSource.data = data;
        });
    }

    function setGoalFilterValues(goalsWidget) {
        goalsWidget.widgets.forEach(function(widget) {
            widget.dataSourceFilter[0].value = goalsWidget.currentRow._id;
        });
    }

    this.goalSettingsActionEventAdd = function() {
        var viewWidget = getViewWidget();
        var widget = getGoalsSettingsWidget();

        widget.newRow();
        widget.currentRow.viewId = viewWidget.currentRow._id;
    };

    this.setFieldsVisibility = function(widget) {
        var structureField = widget.getField('filteredStructure');
        var isApplicabilityFiltered = widget.currentRow.applicability == 'byStructure';
        structureField.isVisible = isApplicabilityFiltered;
        resetStructureField(widget, !isApplicabilityFiltered);
    };

    function resetStructureField(widget, resetStructure) {
        if(resetStructure) {
            widget.currentRow.filteredStructure = null;
            widget.currentRow.filteredStructureId = null;
        }
    }

    this.goalSettingsActionEventSave = function() {
        var widget = prepareGoalsWidgetToSave();

        delete widget.backupRow;
        widget.saveRow().then(function(response){
            widget.currentRow._id = response.dataset.goals[0]._id;
            widget.view();
        });
    };

    function prepareGoalsWidgetToSave() {
        var goalsWidget = getGoalsSettingsWidget();
        var viewWidget = getViewWidget();
        var indicatorWidget = getIndicatorsWidget();

        goalsWidget.currentRow.viewPeriod = viewWidget.currentRow.period;
        goalsWidget.currentRow.viewLabel = viewWidget.currentRow.label;
        goalsWidget.currentRow.dataSourceId = indicatorWidget.currentRow.dataSource;
        goalsWidget.currentRow.dataSourceName = indicatorWidget.currentRow.dataSourceName;

        return goalsWidget;
    }

    this.goalSettingsActionEventEdit = function() {
        var widget = getGoalsSettingsWidget();
        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    this.goalSettingsActionEventCancel = function() {
        var widget = getGoalsSettingsWidget();
        if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
    };

    this.goalSettingsActionEventRemove = function(){
        var widget = getGoalsSettingsWidget();
        var goals = widget.getCheckedRows();
        GoalsService.removeGoal(goals).then(function(){
            widget.reload();
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GoalsController', GoalsController);
});

// FILE: assets/js/controllers/IndicatorController.js
function IndicatorController(UtilitiesController, ScreenService, templateManager, WidgetService, ViewService,
IndicatorService, ExpressionInterpreter, ViewPropertyFactory, ZHPromise, DashboardIndicatorController,
PeriodsManagerService, IndicatorFactory, ChartViewController, MainDashboardController, MessageService, GoalsService,
GoalsController, MasterDetailUtilities, PeriodsUtilities,localStorageEngine,DataSourceManagerService
) {

    var self = this;
    var periodNumber = 0;
    var periodUnusedIds = [];
    var reloadWidget = true;
    var activateParent = true;
    var currentIndicatorId;
    var BAR_LINE_TYPE = "barLine";
    var PROGRESS_DETAIL = "progressDetail";
    var CROSSTAB_TYPE = "crosstab";

    window.onload = function() {
        localStorageEngine.setLocalVar("colorWidgetDimension",null);
    };

    this.colorCss = function(row, element){
        element.children().css("display","none");
        if(row.color != ""){
            element.css("background-color","#"+row.color);
            element.css("border","solid 5px white");
            element.css("border-style","double");
        }else{
            element.css("background-color","white");
        }

    };

    this.setFactColor = function(){
        var dataSourceID;
        var setFactColorWidget;
        if(templateManager.container.getWidget('indicators')){
            dataSourceID = templateManager.container.getWidget('indicators').currentRow.dataSource;
            setFactColorWidget = templateManager.container.getWidget('factColor');
            openFactColorWidget(dataSourceID,setFactColorWidget);
        }else{
            dataSourceID = templateManager.container.widgets[1].dataSourceId;
            if(templateManager.container.getWidget('factColor')){
                setFactColorWidget = templateManager.container.getWidget('factColor');
                openFactColorWidget(dataSourceID,setFactColorWidget);
            }else{
                IndicatorService.factColorAddToContainer();
                setFactColorWidget = templateManager.container.getWidget('factColor');
                openFactColorWidget(dataSourceID,setFactColorWidget);
            }
        }
    };

    function openFactColorWidget(dataSourceID,setFactColorWidget){
        DataSourceManagerService.getDataSourceById(dataSourceID).then(function(response){
            if(response && response.dataset.dataSource.length >0 && !response.dataset.dataSource[0].hasDataMining){
                UtilitiesController.showMessage("ONLY_DATAMINING");
            }else{
                setFactColorWidget.isVisible = true;
                ScreenService.openPopup(setFactColorWidget);
            }
        });
    }

    this.factColorOnEnter = function (factColorWidget){
        var widget = templateManager.container.getWidget('view');
        var row = { viewId: widget.currentRow._id,
                    dimension: widget.currentRow.serieFieldLabel,
                    structureId: []};
        row.structureId.push(widget.currentRow.viewTypeId);
        factColorWidget.fields[0].name = widget.currentRow.serieFieldLabel;
        if(getLocalVar("colorWidgetDimension") == undefined || getLocalVar("colorWidgetDimension") == null){
            WidgetService.fetchWidgetData(row,null).then(function(response){
                factColorWidget.dataSource.data = response.dataset.fetchWidgetData;
                localStorageEngine.setLocalVar("colorWidgetDimension",widget.currentRow.serieFieldLabel);
                loadColor(factColorWidget,widget);

            });
        }else {
            loadColor(factColorWidget,widget);
        }
    };

    var removeColors = function(widget){
        widget.dataSource.data = widget.dataSource.data.map(function(element){
            element.color = null;
            return element;
        });
    };

    var loadColor = function(factColorWidget, widget){
        factColorWidget.dataSource.data = factColorWidget.dataSource.data.map(function(element){

            if(element[widget.currentRow.serieFieldLabel] == null){
                element[widget.currentRow.serieFieldLabel] = "(sem valor)";
            }

            if(!element.color && element.color != "" && widget.currentRow.specificColor != undefined && widget.currentRow.specificColor != null){
                widget.currentRow.specificColor.forEach(function(data){
                    if(element[widget.currentRow.serieFieldLabel] == data[widget.currentRow.serieFieldLabel]){
                        element.color = data.color;
                    }
                });
            }

            if(!element.color){
                element.color = "";
            }

            return element;
        });
    };

    this.selectFactColorWidgetActionEventDelete = function(widget){
        widget.currentRow.color = "";
        widget.redraw();
    };

    this.factColorAfterMoveRow = function (widget){
        if(widget != undefined && widget.currentRow != undefined && widget.currentRow[getLocalVar("colorWidgetDimension")] != undefined){
            var setSelectFactColorWidget = templateManager.container.getWidget('selectFactColor');
            setSelectFactColorWidget.isVisible = true;
            ScreenService.openPopup(setSelectFactColorWidget).then(function(){
                setSelectFactColorWidget.dataSource.data = [];
                setSelectFactColorWidget.dataSource.data.push(widget.currentRow);
                setSelectFactColorWidget.currentRow = Util.clone(widget.currentRow);
            });
        }
    };

    this.selectFactColorWidgetActionEventSave = function(widget){
        var setFactColorWidget = templateManager.container.getWidget('factColor');
        if(widget.currentRow.color && widget.currentRow.color != ""){
            widget.currentRow.color = widget.currentRow.color.toHex();
        }
        Util.extend(setFactColorWidget.currentRow,widget.currentRow);
        ScreenService.closePopup();
    };

    this.factColorWidgetActionEventSave = function(widget){
        var viewWidget = templateManager.container.getWidget('view');
        if(viewWidget.currentRow.specificColor == undefined || viewWidget.currentRow.specificColor == null){
            viewWidget.currentRow.specificColor = [];
        }
        var specificColor = [];
        var objColor;
        widget.dataSource.data.forEach(function(element){
            objColor = null;
            if(element.color && element.color != ""){
                objColor = {};
                objColor[getLocalVar("colorWidgetDimension")] = element[getLocalVar("colorWidgetDimension")];
                objColor.color = element.color;
            }
            if(objColor){
                specificColor.push(objColor);
            }
        });
        viewWidget.currentRow.specificColor = specificColor;
        removeColors(widget);
        ScreenService.closePopup();
    };

    this.cancelFactColor = function (widget){
        removeColors(widget);
        ScreenService.closePopup();
    };

    this.cancelSelectFactColor = function (){
        ScreenService.closePopup();
    };

    var getIndicatorsWidget = function(){
        return templateManager.container.getWidget('indicators');
    };

    var getColumnsSettingsWidget = function(){
        return templateManager.container.getWidget('columnsSettings');
    };

    var getGridStyleWidget = function(){
        return templateManager.container.getWidget('gridStyle');
    };

    var getViewWidget = function(){
        return templateManager.container.getWidget('view');
    };

    var getFieldPeriodFromViewWidget = function(){
        return getViewWidget().getField('period');
    };

    var getFieldViewTypeIdFromViewWidget = function(){
        return getViewWidget().getField('viewTypeId');
    };

    var getActionSaveAsFromViewWidget = function(){
        return getViewWidget().getAction('save_as');
    };

    var getActionimportFromIndicators = function(){
        return getIndicatorsWidget().getAction('import');
    };

    var getFieldStyleFieldsFromGridStyleWidget = function(){
        return getGridStyleWidget().getField("styleFields");
    };

    var getActiveWidget = function() {
        return templateManager.container.getActiveWidget();
    };

    //General
    this.setIndicatorFilterValues = function(indicatorWidget) {
        reloadWidget = true;
        indicatorWidget.widgets.forEach(function(widget) {
            widget.dataSourceFilter[0].value = indicatorWidget.currentRow._id;
        });
    };

    this.setCurrentIndicatorId = function(widget){
        currentIndicatorId = widget.currentRow._id;
    };

    this.getCurrentIndicatorId = function (){
        return currentIndicatorId;
    };

    this.reloadWidgetIfNeeded = function(widget) {
        if (reloadWidget) {
            widget.reload();
            reloadWidget = false;
        }
    };

    this.closeSwipeAndSetChangedFalse = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.currentRow.changed = false;
        });
    };

    this.indicatorActionEventBackView = function(){
        var widget = getIndicatorsWidget();
        ScreenService.closeSwipe();
        widget.currentRow.changed = false;
        widget.list();
        widget.activate();
    };

    this.goBackToWidgetIndicators = function(){
        var widget = getIndicatorsWidget();
        ScreenService.closeSwipe();
        widget.list();
        widget.activate();
    };

    this.isPositiveNumber = function (field) {
        var fieldValue = field.value();
        if(fieldValue < 0){
            UtilitiesController.showMessage("ONLY_POSITIVE_NUMBER");
            field.clearValue();
        }
    };

    this.indicatorActionEventAdd = function() {
        var widget = getIndicatorsWidget();
        UtilitiesController.changeWidgetVisibility(widget.widgets, false);
        widget.newRow();
    };

    this.indicatorActionEventEdit = function() {
        var widget = getIndicatorsWidget();
        editEvent(widget);
    };

    this.viewWidgetActionEventEdit = function() {
        var widget = getViewWidget();
        setSaveAsActionVisibility(true);
        editEvent(widget);
    };

    this.viewWidgetActionEventBackList = function() {
        self.goBackToWidgetIndicators();
    };

    this.gridStyleWidgetActionEventBackList = function() {
        self.goBackToWidgetIndicators();
    };

    this.viewWidgetActionEventBackView = function() {
        var widget = getViewWidget();
        self.backToList(widget);
    };

    this.viewWidgetActionEventCancel = function() {
        localStorageEngine.setLocalVar("colorWidgetDimension",null);
        var widget = getViewWidget();
        self.rollbackView(widget);
    };

    var editEvent = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
    	UtilitiesController.changeWidgetVisibility(widget.widgets, false);
    	widget.edit();
    	PeriodsManagerService.resetAllPeriodFields(widget);
        var drillDownViewField = widget.getField('drillDownViewField');
        if(drillDownViewField){
            drillDownViewField.reload();
        }

    };

    this.indicatorActionEventCancel = function() {
        var widget = getIndicatorsWidget();
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        }
        else {
            Util.extend(widget.currentRow, widget.backupRow);
            widget.view();
        }
        UtilitiesController.changeWidgetVisibility(widget.widgets, true);
    };

    this.dataSourceNameFieldOnChange = function(){
        var widget=getIndicatorsWidget();
        if(!widget.isNewRow() && widget.backupRow && widget.backupRow.dataSource && widget.currentRow.dataSource && widget.backupRow.dataSource != widget.currentRow.dataSource){
            UtilitiesController.confirmMessageByCode("INDICATOR_CHANGE_DATASOURCE_WARNING",
            {},{
                "type": 'CONFIRMATION',
                "success": function (){},
                "fail": function(){
                    widget.currentRow.dataSource = widget.backupRow.dataSource;
                    widget.currentRow.dataSourceName = widget.backupRow.dataSourceName;
                }
            });
        }
    };

    this.indicatorActionEventSave = function() {
        var widget = getIndicatorsWidget();
        if (widget.isValid()) {
            widget.currentRow.changed = true;
            widget.saveRow().then(function(response) {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;
                widget.currentRow._id = response.dataset.indicator[0]._id;
                self.setCurrentIndicatorId(widget);
                widget.view();
                UtilitiesController.changeWidgetVisibility(widget.widgets, true);
                self.setIndicatorFilterValues(widget);

                ViewService.updateMenu();
            });
        }
    };

    this.indicatorActionEventDelete = function() {
        var widget = getIndicatorsWidget();
        return UtilitiesController.confirmMessageByCode("DELETE_CONFIRMATION",{},{
            "type": 'CONFIRMATION',
            "success": function (){
                    return widget.removeCheckedRows().then(function() {
                    ViewService.updateMenu();
                }).catch(function(error){
                    UtilitiesController.showMessage(error.error,"error");
                });
            }
        });
        
    };

    this.drillDownIndicatorsMessage = function(drillDownIndicators,dataSource){
        var message = "";
        drillDownIndicators.forEach(function(drillDownIndicator){
            var neededIndicator = dataSource.filter(function(wid){
                return wid._id == drillDownIndicator;
            });
            if(neededIndicator.length > 0 && neededIndicator[0].name){
                message += "「"+neededIndicator[0].name+"」<br>";
            }
        });
        return message;
    };

    this.importDefaultOrgIndicators = function(widget) {
        var checkedRows = widget.dataSource.getCheckedRows();
        var indicatorIds = checkedRows.map(function(indicator) {
            return indicator._id;
        });
        IndicatorService.checkDrillDownIndicators(indicatorIds).then(function(drillDownIndicators){
            if(drillDownIndicators && drillDownIndicators.dataset && drillDownIndicators.dataset.checkDrillDownIndicators.length > 0){
                drillDownIndicators = drillDownIndicators.dataset.checkDrillDownIndicators;
                var message = self.drillDownIndicatorsMessage(drillDownIndicators,widget.dataSource.data);
                if(message){
                    UtilitiesController.confirmMessageByCode("DRILL_DOWN_INDICATORS",
                    {
                        "i": message
                    },{
                        "type": 'CONFIRMATION',
                        "success": function (){
                            importIndicators(indicatorIds.concat(drillDownIndicators));
                        },
                        "fail": function(){
                            importIndicators(indicatorIds);
                        }
                    });
                } else {
                    importIndicators(indicatorIds);
                }
            } else {
                importIndicators(indicatorIds);
            }
        });
    };

    var importIndicators = function(indicatorIds){
        IndicatorService.importDefaultOrgIndicators(indicatorIds).then(function(response) {
            MessageService.successNotificationByCode('INDICATOR_IMPORTED_WITH_SUCCESS');
            ScreenService.openWindow('indicator');
            ViewService.updateMenu();
        }, function(response) {
            MessageService.errorNotificationByCode('ERROR_ON_IMPORT_INDICATORS');
        });
    };

    this.setOrderValidation = function(widget) {
        var menu = widget.currentRow.menu;
        if (menu) {
            widget.getField('order').validations = {
                "required": {}
            };
        }
        else {
            widget.getField('order').validations = {};
        }
    };

    // View widget functions - start
    var setViewFilterValues = function(viewWidget) {
        viewWidget.widgets.forEach(function(widget) {
            if(widget.name === 'goalSettings'){
                widget.dataSourceFilter[0].value = viewWidget.currentRow._id;
            }
        });
    };

    this.gridStyleWidgetActionEventCancel = function() {
        var widget = getGridStyleWidget();
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        }
        else {
            Util.extend(widget.currentRow, widget.backupRow);
            widget.view();
        }
    };

    this.gridStyleWidgetActionBackView = function() {
        var widget = getGridStyleWidget();
        UtilitiesController.masterDetailBackToList(widget);
    };

    this.gridStyleWidgetActionEdit = function() {
        var widget = getGridStyleWidget();
        UtilitiesController.masterDetailEdit(widget);
    };


    this.buildViewWidgetProperties = function(viewWidget) {
        var dataSourceId = viewWidget.container.getWidget('indicators').currentRow.dataSource;
        if (viewWidget.inListMode()) {
            WidgetService.buildViewWidgetProperties(viewWidget.getField('viewTypeId'), viewWidget, dataSourceId, true);
        }
    };

    this.setProperties = function(viewTypeField, viewTypeId, property) {
        return viewTypeField.dataSource.data.filter(function(row) {
            return row._id === viewTypeId;
        }).map(function(viewType) {
            return viewType[property];
        }).shift();
    };

    this.updateFieldsVisibility = function() {
        var viewWidget = getViewWidget();
        var viewTypeId = viewWidget.currentRow.viewTypeId;
        var viewTypeField = getFieldViewTypeIdFromViewWidget();

        WidgetService.disableFields();
        var properties = this.setProperties(viewTypeField, viewTypeId, 'properties');
        if (properties) {
            WidgetService.enableFields(properties);
        }
        templateManager.updateTemplate();
    };

    this.setDrillDownPropertiesVisibility = function(hasDrillDown) {
        var viewWidget = getViewWidget();
        var drillDownGroup;
        viewWidget.fieldGroups.forEach(function(fieldGroup) {
            if (fieldGroup.name === 'drillDownProperties') {
                drillDownGroup = fieldGroup;
            }
        });

        drillDownGroup.isVisible = Boolean(hasDrillDown);

        if (hasDrillDown) {
            self.setDrillDownFieldVisibility(viewWidget);
        }
    };

    this.setDrillDownFieldVisibility = function(viewWidget) {
        var drillDownType = viewWidget.currentRow.drillDownType;
        var drillDownViewField = viewWidget.getField('drillDownViewField');
        var drillDownDashboardField = viewWidget.getField('drillDownDashboardField');

        switch (drillDownType) {
            case 'structure':
                drillDownViewField.isVisible = false;
                drillDownDashboardField.isVisible = false;
                break;
            case 'widget':
                drillDownViewField.isVisible = true;
                drillDownDashboardField.isVisible = false;
                break;
            case 'view':
                drillDownViewField.isVisible = false;
                drillDownDashboardField.isVisible = true;
                break;
            default:
                drillDownViewField.isVisible = true;
                drillDownDashboardField.isVisible = false;
                break;
        }
    };

    this.setViewWidgetMethods = function(viewWidget) {
        if (!viewWidget.zmartMethods) {
            viewWidget.zmartMethods = {
                "afterMoveRow": {},
                "widgetOnSave": {}
            };
            viewWidget.zmartMethods.afterMoveRow.convertArrayToString = CUSTOM_FIELD_FUNCTIONS.convertArrayToString;
            viewWidget.zmartMethods.widgetOnSave.convertStringToArray = CUSTOM_FIELD_FUNCTIONS.convertStringToArray;
        }
    };

    this.callAfterMoveRowMethods = function(viewWidget) {
        viewWidget.fields.forEach(function(field) {
            if (field.isVisible && field.functionsToAfterMoveRow && field.functionsToAfterMoveRow.length > 0) {
                field.functionsToAfterMoveRow.forEach(function(method) {
                    if (viewWidget.zmartMethods.afterMoveRow[method]) {
                        viewWidget.zmartMethods.afterMoveRow[method](viewWidget, field);
                    }
                });
            }
        });
    };

    this.callOnSaveMethods = function(viewWidget) {
        viewWidget.fields.forEach(function(field) {
            if (field.isVisible && field.functionsToOnSave && field.functionsToOnSave.length > 0) {
                field.functionsToOnSave.forEach(function(method) {
                    self.setViewWidgetMethods(viewWidget);
                    if (viewWidget.zmartMethods.widgetOnSave[method]) {
                        viewWidget.zmartMethods.widgetOnSave[method](viewWidget, field);
                    }
                });
            }
        });
    };

    this.viewWidgetActionEventAdd = function() {
        var widget = getViewWidget();
        setSaveAsActionVisibility(false);
        widget.newRow();
        widget.parent.reload().then(function(){
            widget.currentRow.columns = widget.parent.currentRow.dataSourceColumns;
        });
        self.setDrillDownPropertiesVisibility(widget.currentRow.hasDrillDown);
    };

    this.setIntervalFieldsNumber = function(widget, controllerName) {
		widget.currentRow.backupPeriod = Util.clone(widget.currentRow.period);
		PeriodsManagerService.setIntervalFieldsNumber(widget, periodNumber, controllerName).then(function(response){
			periodNumber = response;
			periodUnusedIds = [];

			PeriodsManagerService.setPeriodsLabelsAndValues(widget);
		});
	};

    var propertyBelongsToSerieType = function(propertyName, serieType, local) {
        var propertiePrefix = propertyName.substr(0, checkSignal(serieType, local));
        var propertySuffix = propertyName.substr(checkSignal(serieType, local));
        return propertySuffix === serieType || propertiePrefix === serieType;
    };

    var checkPropertyTypeAndAppend = function(propertyName, value, serieType, seriesToAppend, local) {
        var size = checkSignal(serieType, local);
        if (propertyBelongsToSerieType(propertyName, serieType, local)) {
            if (size > 0) {
                var propertyWithoutPrefix = propertyName.substr(size, propertyName.length);
                seriesToAppend[propertyWithoutPrefix] = value;
            }
            else {
                var propertyWithoutSuffix = propertyName.substr(0, propertyName.length + size);
                seriesToAppend[propertyWithoutSuffix] = value;
            }
        }
    };

    var copySerieProperties = function(viewWidget, widgetProperties, serieType, local) {
        var serie = {};
        for (var key in widgetProperties) {
            checkPropertyTypeAndAppend(key, widgetProperties[key], serieType, serie, local);
        }
        viewWidget.currentRow.chartSeries.push(serie);
    };

    this.separeteSeries = function(viewWidget, serie, suffix, position) {
        copySerieProperties(viewWidget, serie, suffix, position);
    };

    var checkSignal = function(serieType, position) {
        switch (position) {
            case "start":
                return serieType.length;
            case "end":
                return -serieType.length;
            default:
                return 0;
        }
    };

    this.clearSeries = function(viewWidget, suffix, position) {
        var size = checkSignal(suffix, position);
        for (var key in viewWidget.currentRow) {
            if (size > 0 && key.substr(0, size) === suffix) {
                delete viewWidget.currentRow[key];
            }
            else if (key.substr(key.length - suffix.length, suffix.length) === suffix) {
                delete viewWidget.currentRow[key];
            }
        }
    };

    this.validateSeriesType = function(serie) {
        serie.serieTypeBar = "bar";
        serie.serieTypeLine = "line";
    };

    this.setChartSeries = function(viewWidget, viewType) {
        viewWidget.currentRow.chartSeries = [];
        var serie = {};
        var bar = "Bar",
            position = "end",
            line = "Line";

        for (var key in viewWidget.currentRow) {
            if (key === "serieYAxisLinkedSerieLine" && viewType.name === "barLine") {
                serie[key] = viewWidget.currentRow[key];
            }

            if (viewWidget.currentRow[key] === null || viewWidget.currentRow[key] === "" || Number.isNaN(viewWidget.currentRow[key])) {
                delete viewWidget.currentRow[key];
            } else if (key.substr(0, 5) === "serie" && key in viewWidget.currentRow && viewType.useChartSeries) {
                serie[key] = viewWidget.currentRow[key];
            }
        }

        if (viewType.name === BAR_LINE_TYPE) {
            this.validateSeriesType(serie);
            this.separeteSeries(viewWidget, serie, bar, position);
            this.separeteSeries(viewWidget, serie, line, position);
        } else {
            viewWidget.currentRow.chartSeries.push(serie);
            this.clearSeries(viewWidget, bar, position);
            this.clearSeries(viewWidget, line, position);
        }
    };

    this.resetChartSeries = function(viewWidget) {
        viewWidget.currentRow.chartSeries = [];
    };

    this.setProgressDetailWidgetProperties = function(viewWidget, suffix, prefix) {
        var widgetProperties = {},
            widget = {};

        widgetProperties.currentRow = buildCardProperties(viewWidget, viewWidget.currentRow, suffix, "end");
        widgetProperties.progressParameters = buildCardProperties(viewWidget, widgetProperties.currentRow, prefix, "start");

        this.clearSeries(widgetProperties, prefix, "start");

        widget = widgetProperties.currentRow;
        widget.progressParameters = widgetProperties.progressParameters;

        viewWidget.currentRow.widgets.push(widget);
    };

    var buildCardProperties = function(viewWidget, row, subStringPart, position) {
        self.resetChartSeries(viewWidget);
        self.separeteSeries(viewWidget, row, subStringPart, position);
        return viewWidget.currentRow.chartSeries[0];
    };


    this.setMaskInGrid = function(row){
    var columns = row.columns;
    var type = row.viewType;

      if (type === "grid"){
          columns.forEach(function(colum){
               if(colum.maskField === "currency" || colum.maskField === "float" || colum.maskField === "percent"){
                  colum.mask = {
                        "type": colum.maskField,
                        "params" : {
                            "precision" : colum.precision
                        }
                    };
                } else{
                    if (!colum.maskField){
                        delete colum.mask;
                    } else{
                        colum.mask = {
                            "type": colum.maskField,
                            "params" : {
                                "precision" : 2
                            }
                        };
                    }
                }
            });
        }
    };


    this.setMaskInCharts = function(row){
        var serieValueFormat = row.serieValueFormat;
        var serieYAxisFormat = row.serieYAxisFormat;
        var precision = row.precision;

        if(precision){
            if (serieValueFormat){
               if (serieValueFormat === "currency" || serieValueFormat === "float" || serieValueFormat === "percent")
                    row.serieValueFormatParams = {"precision" : precision };
                else
                    row.serieValueFormatParams = {"precision" : 2 };
            }
            if (serieYAxisFormat){
                if (serieYAxisFormat === "currency" || serieYAxisFormat === "float" || serieYAxisFormat === "percent")
                    row.serieYAxisFormatParams = {"precision" : precision };
                else
                    row.serieYAxisFormatParams = {"precision" : 2 };
            }
        }
    };


    this.setCurrentRow = function(viewWidget, viewType) {
        var row =  viewWidget.currentRow;
        self.callOnSaveMethods(viewWidget);
        row.previousConditions = self.readComparisonExpression(viewWidget, "previous");
        row.postConditions = self.readComparisonExpression(viewWidget, "post");
        row.postGroupConditions = self.readComparisonExpression(viewWidget, "postGroup");
        row.isVisible = true;
        row.events = [];
        row.actions = [];
        row.fields = [];
        row.bringGroupsCollapsed = true;
        row.template = viewType.template;
        row.viewType = row.chartType = viewType.name;
        row.serieType = viewType.serieType || viewType.name;

        row.allowMoreThanOneDimension = viewType.allowMoreThanOneDimension || null;
        row.chartInvertedAxes = viewType.chartInvertedAxes || null;
        if(viewType.name === 'grid'){
            this.setMaskInGrid(row);
            row.class = row.class && !Util.isEmptyObject(row.class) ? row.class : viewType.defaultMobileStyle;
        }else{
            this.setMaskInCharts(row);
        }

        if (row.groupsGrid) {
            row.groupsGrid = row.groupsGrid.split(",");
        }
    };

    this.setDrillDownProperties = function(viewWidget) {
        if (!viewWidget.currentRow.hasDrillDown || viewWidget.currentRow.drillDownType === 'structure') {
            viewWidget.currentRow.drillDownViewField = null;
            viewWidget.currentRow.drillDownView = null;
            viewWidget.currentRow.drillDownDashboardField = null;
        }
        else if (viewWidget.currentRow.hasDrillDown) {
            if (viewWidget.currentRow.drillDownType === 'widget') {
                viewWidget.currentRow.drillDownDashboardField = null;
            }
            else if (viewWidget.currentRow.drillDownType === 'view') {
                viewWidget.currentRow.drillDownViewField = null;
                viewWidget.currentRow.drillDownView = null;
            }
        }
    };

    this.setSpecialChartsProperties = function (viewWidget, viewType) {
        viewWidget.currentRow.widgets = [];
        if (viewType.name === PROGRESS_DETAIL) {
            viewWidget.currentRow.fieldLabel = "label";
            viewWidget.currentRow.fieldValue = "value";
            var endOne = "_1",
                endTwo = "_2",
                start = "progress_";
            self.setProgressDetailWidgetProperties(viewWidget, endOne, start);
            self.setProgressDetailWidgetProperties(viewWidget, endTwo, start);
        }
    };

    this.viewWidgetActionEventRemove = function(){
        UtilitiesController.confirmMessageByCode("DELETE_CONFIRMATION",{},{
            "type": 'CONFIRMATION',
            "success": function (){
                var widget = getViewWidget();
                var views = widget.getCheckedRows();
                IndicatorService.getViewDependencies(views).then(function(response){
                    var goals = response.dataset.result[0];
                    widget.removeCheckedRows().then(function(){
                        GoalsService.removeGoal(goals);
                    }).catch(function(error){
                        UtilitiesController.showMessage(error.error,"error");
                    });
                });
            }
        });
    };

    this.removeCheckedRows = function(widget) {
        return widget.removeCheckedRows().then(function() {
            ViewService.updateMenu();
        });
    };

    self.getViewDependences = function(viewId){
        return GoalsService.getGoalByViewId(viewId).then(function(response){
            return response.dataset.goals;
        });
    };

    this.saveViewWidget = function() {
        var viewWidget = getViewWidget();
        if(!viewWidget.currentRow.columns){
            var indicatorWidget = getIndicatorsWidget();
            if(indicatorWidget){
                viewWidget.currentRow.columns = indicatorWidget.currentRow.dataSourceColumns;
            }
        }

        viewWidget.currentRow.indicator = viewWidget.parent.currentRow._id;
        prepareWidgetToSave(viewWidget);
        viewWidget.currentRow.changed = true;
        return viewWidget.saveRow().then(function(response) {
            setRowAfterSave(viewWidget, response);
            self.setDrillDownPropertiesVisibility(viewWidget.currentRow.hasDrillDown);
            ViewService.updateMenu();
            MainDashboardController.forceDashboardReload();

            return self.updateGeneratedDrillDownMenu(viewWidget, viewWidget.currentRow.drillDownType).then(function(){
                return self.setIntervalFieldsNumber(viewWidget, 'IndicatorController').then(function(){
                    ScreenService.closePopup();
                    viewWidget.reload();
                    viewWidget.view();
                    viewWidget.activate();
                    UtilitiesController.changeWidgetVisibility(viewWidget.widgets, true);
                });
            });
        });
    };

    this.viewWidgetActionEventSave = function() {
        localStorageEngine.setLocalVar("colorWidgetDimension",null);
        var viewWidget = getViewWidget();
        if (viewWidget.isValid()) {
            return self.saveViewWidget();
        } else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
		}
    };

    this.viewWidgetActionEventSaveAs = function(){
        localStorageEngine.setLocalVar("colorWidgetDimension",null);
        var viewWidget = getViewWidget();
        viewWidget.currentRow.__is_new = true;
        delete viewWidget.currentRow._id;
        if (viewWidget.isValid()) {
            self.saveViewWidget();
        } else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
        }
    };

    var setSaveAsActionVisibility = function(visibility){
        getActionSaveAsFromViewWidget().isVisible = visibility;
    };

    this.saveViewSettings = function(viewWidget, saveAsNew){

        if (saveAsNew) {
    		viewWidget.currentRow.__is_new = true;
    		delete viewWidget.currentRow._id;
        }

    	if (viewWidget.isValid()) {
            prepareWidgetToSave(viewWidget);
            return viewWidget.saveRow().then(function(response) {
                setRowAfterSave(viewWidget, response);
                ViewService.updateMenu();
                MainDashboardController.forceDashboardReload();
                self.updateGeneratedDrillDownMenu(viewWidget, viewWidget.currentRow.drillDownType).then(function() {
                    ChartViewController.closeViewSettingsAfterSave(saveAsNew, viewWidget);
                });
            });
        } else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
		}
    };

    this.clearCurrentRowFields = function(){
        var viewWidget = getViewWidget();
        viewWidget.fields.forEach(function(field){
            if(field.isVisible === false){
                viewWidget.currentRow[field.name] = "";
            }
        });
    };

    this.setsFixedValuesCrosstab =  function(viewWidget){
        var dimensionX = viewWidget.currentRow.xKeysX;
        var dimensionY = viewWidget.currentRow.xKeysY;

        viewWidget.currentRow.xAxisProperty = dimensionX;
        viewWidget.currentRow.yAxisProperty = dimensionY;

        if(Array.isArray(dimensionY)){
            var xKeys = Util.clone(dimensionY);
            xKeys.push(dimensionX);
            viewWidget.currentRow.xKeys = xKeys;
        }else{
            viewWidget.currentRow.xKeys = Array(dimensionX, dimensionY);
        }



        viewWidget.currentRow.labelProperty = viewWidget.currentRow.yKeys;
    };

    function prepareWidgetToSave(viewWidget) {
        var viewTypeField = viewWidget.getField('viewTypeId');
        var viewType = viewTypeField.dataSource.data.filter(function(item) {
            return item._id === viewWidget.currentRow.viewTypeId;
        }).shift();

        periodNumber = PeriodsManagerService.cleanInterval(viewWidget.currentRow, "", periodNumber);

        if (periodNumber > 0) {
            viewWidget.currentRow.period = PeriodsManagerService.getPeriodsValues(viewWidget, periodNumber);
        }

        if(viewType.name === CROSSTAB_TYPE){
            self.setsFixedValuesCrosstab(viewWidget);
        }

        if (viewWidget.currentRow.backupPeriod) {
            delete viewWidget.currentRow.backupPeriod;
        }

        self.setCurrentRow(viewWidget, viewType);
        self.setChartSeries(viewWidget, viewType);
        self.setDrillDownProperties(viewWidget);
        self.setSpecialChartsProperties(viewWidget, viewType);
        self.setAutoCompleteCssCard(viewWidget);
    }

    self.setAutoCompleteCssCard = function (widget) {
        var currentRow = widget.currentRow;
        if (currentRow.chartType === "dashboardCard") {
            if (widget.currentRow.CSS_TITLE && !isNaN(widget.currentRow.CSS_TITLE)) {
                widget.currentRow.CSS_TITLE = "font-size: " + widget.currentRow.CSS_TITLE + "em";
            }
            if (widget.currentRow.CSS_VALUE && !isNaN(widget.currentRow.CSS_VALUE)) {
                widget.currentRow.CSS_VALUE = "font-size: " + widget.currentRow.CSS_VALUE + "em";
            }
        }
    };

    function setRowAfterSave(viewWidget, response){
        viewWidget.currentRow.changed = false;
        viewWidget.currentRow.__is_new = false;
        viewWidget.currentRow._id = response.dataset.widget[0]._id;
    }

    this.updateGeneratedDrillDownMenu = function(viewWidget, viewType) {
        var viewWidgetId = viewWidget.currentRow._id;
        var menuDashboards = viewWidget.currentRow.drillDownDashboardField;

        return ViewService.findGeneratedMenu(viewWidgetId).then(function(response) {
            var drillDownMenu = response.dataset.view[0] || {};

            if (viewType === 'view') {
                drillDownMenu.type = 'G';
                drillDownMenu.widgetId = viewWidgetId;
                drillDownMenu.dashboards = menuDashboards;

                ViewService.saveView([drillDownMenu]).then(function(response) {
                    drillDownMenu._id = response.dataset.view[0]._id;
                    self.updateDashboardsRelatedToGeneratedMenu(drillDownMenu._id, drillDownMenu.dashboards, 'set');
                });
            }
            else if (!Util.isEmptyObject(drillDownMenu)) {
                ViewService.deleteView(drillDownMenu);
            }

            var backupDrillDownField = null;
            if (viewWidget.backupRow && viewWidget.backupRow.drillDownDashboardField){
                backupDrillDownField = viewWidget.backupRow.drillDownDashboardField;
            }

            return self.updateDashboardsRelatedToGeneratedMenu(drillDownMenu._id, backupDrillDownField, 'clean');
        });
    };

    this.cleanDashboardsRelatedToMenu = function(dashboard, menuId) {
        if (dashboard.generatedMenuAssociated) {
            var elementToRemove = dashboard.generatedMenuAssociated.indexOf(menuId);
            dashboard.generatedMenuAssociated.splice(elementToRemove, 1);
        }
        return dashboard;
    };

    this.setMenuIdOnRelatedDashboard = function(dashboard, menuId) {
        if (!dashboard.generatedMenuAssociated) {
            dashboard.generatedMenuAssociated = [];
        }
        dashboard.generatedMenuAssociated.push(menuId);
        return dashboard;
    };

    this.updateDashboardsRelatedToGeneratedMenu = function(generatedMenuId, dashboardsToUpdate, option) {
        var dashboardsToPersist = [];
        if (dashboardsToUpdate) {
            dashboardsToUpdate.forEach(function(dashboardId) {
                dashboardsToPersist.push(ViewService.findView(dashboardId).then(function(response) {
                    var dashboard = response.dataset.view[0];

                    if (option === 'clean') {
                        return self.cleanDashboardsRelatedToMenu(dashboard, generatedMenuId);
                    }
                    else if (option === 'set') {
                        return self.setMenuIdOnRelatedDashboard(dashboard, generatedMenuId);
                    }
                }));
            });
        }

        return ZHPromise.all(dashboardsToPersist).then(function(dashboards) {
            return ViewService.saveView(dashboards);
        });
    };

    this.backToList = function(viewWidget) {
        UtilitiesController.masterDetailBackToList(viewWidget).then(function() {
            var dataSource = viewWidget.dataSource.data;
            dataSource.forEach(function(data) {
                if (data.changed === true) {
                    data.changed = false;
                }
            });
        });
    };

    this.rollbackView = function(viewWidget) {
        if (viewWidget.currentRow.__is_new) {
			viewWidget.dataSource.data.pop();
			ScreenService.closeSwipe();
			viewWidget.list();
		} else {
			Util.extend(viewWidget.currentRow, viewWidget.backupRow);
            self.setDrillDownPropertiesVisibility(viewWidget.currentRow.hasDrillDown);
			viewWidget.currentRow.period = viewWidget.currentRow.backupPeriod || viewWidget.currentRow.period;
            if(!viewWidget.backupRow.postConditionsString){
                viewWidget.currentRow.postConditionsString = null;
            }
            if(!viewWidget.backupRow.previousConditionsString){
                viewWidget.currentRow.previousConditionsString = null;
            }
            if(!viewWidget.backupRow.postGroupConditionsString){
                viewWidget.currentRow.postGroupConditionsString = null;
            }
            viewWidget.view();
		}

        self.setIntervalFieldsNumber(viewWidget, "IndicatorController");
        UtilitiesController.changeWidgetVisibility(viewWidget.widgets, true);
    };

    this.setIntervalFieldsNumber = function(widget, controllerName) {
		widget.currentRow.backupPeriod = Util.clone(widget.currentRow.period);
		return PeriodsManagerService.setIntervalFieldsNumber(widget, periodNumber, controllerName).then(function(response){
			periodNumber = response;
			periodUnusedIds = [];

			return PeriodsManagerService.setPeriodsLabelsAndValues(widget);
		});
	};

    var resetPeriod = function(viewWidget){
        if(viewWidget.currentRow.backupPeriod){
			viewWidget.currentRow.period = viewWidget.currentRow.backupPeriod;
		}
    };

    this.addNewInterval = function(indicatorWidget) {
		periodNumber++;
		PeriodsManagerService.addNewInterval(indicatorWidget, periodNumber, periodUnusedIds, "IndicatorController").then(function(response){
		    periodUnusedIds = response;
		});
	};

	this.removeInterval = function(indicatorWidget, relatedPeriod) {
		periodUnusedIds = PeriodsManagerService.removeInterval(indicatorWidget, relatedPeriod, periodNumber, periodUnusedIds);
		periodNumber--;
	};

    self.readComparisonExpression = function(viewWidget, previousOrPost) {
        try {
            var expression = null;
            if (viewWidget.currentRow[previousOrPost + "ConditionsString"]) {
                var columns = viewWidget.currentRow.columns || [];
                var indicatorWidget = templateManager.container.getWidget('indicators');

                if(!columns.length && indicatorWidget){
                    columns = indicatorWidget.currentRow.dataSourceColumns;
                }

                if(columns.length){
                    var columnsNames = columns.map(function(column) {
                        return column.name;
                    });

                    expression = ExpressionInterpreter.readComparisonExpression(viewWidget.currentRow[previousOrPost + "ConditionsString"], previousOrPost, columnsNames, columnsNames);
                    UtilitiesController.successNotification("Valid Comparison Expression");
                } else {
                    MessageService.errorNotificationByCode('CANT_VALIDATE_EXPRESSION');
                }
            }
            return expression;
        }
        catch (e) {
            UtilitiesController.showMessage(e, "error");
        }
    };

    //GridStyleFunctions
    this.gridStyleActionEventSave = function() {
        var widget = getGridStyleWidget();
        if (widget.isValid()) {
            widget.currentRow.indicator = widget.parent.currentRow._id;
            widget.saveRow().then(function() {
                ScreenService.closeSwipe();
                widget.reload();
                widget.list();
            }, function(response) {
                UtilitiesController.showMessage(response.data.error);
            });
        }
        else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
        }
    };

    this.gridStyleWidgetOnEnter = function() {
        var widget = getGridStyleWidget();
        getFieldStyleFieldsFromGridStyleWidget().dataSource.data = widget.parent.currentRow.dataSourceColumns;
    };

    this.gridStyleWidgetActionEventEdit = function() {
        var widget = getGridStyleWidget();
        MasterDetailUtilities.masterDetailEdit(widget);
    };

    //Indicators importion functions
    this.indicatorActionEventImport = function(){
        var setOrganizationWidget = IndicatorFactory.factoryImportNowWidget(templateManager.container, 'indicator');
        setOrganizationWidget.isVisible = true;
		setOrganizationWidget.newRow();
		ScreenService.openPopup(setOrganizationWidget);
    };

    this.openImportIndicators = function(widget){
        var baseConnection = widget.currentRow.baseConnection || null;
        IndicatorService.setBaseConnectionToImport(baseConnection).then(function(){
            ScreenService.openWindow('indicatorsImport');
        });
    };

    self.indicatorImportOnEnter = function(widget) {
        UtilitiesController.clearChangedRows(widget);
        self.setIndicatorFilterValues(widget);
        self.setOrderValidation(widget);
    };

    self.setDateFilterColumn = function(widget) {
        var currentRow = widget.parent.currentRow;
        var dateColumns = currentRow.dataSourceColumns;
        var dateFilterDataSource = widget.getField('dateFilterColumnLabel').dataSource;
        var newDateColuns = [];
        if (Util.isArray(dateColumns)) {
            newDateColuns = dateColumns.filter(function(column) {
                return column.isDate;
            });
        }
        dateFilterDataSource.data = newDateColuns;
    };

    self.indicatorContainerAfterInit = function(){
        getActionimportFromIndicators().isVisible = getLocalVar('isTeknisa');
        UtilitiesController.activateAndList(getIndicatorsWidget());
        getIndicatorsWidget().reload();
    };

    this.indicatorsWidgetBeforeMoveRow = function(){
        UtilitiesController.deleteChangedFlag(getIndicatorsWidget());
    };

    this.gridStyleWidgetBeforeMoveRow = function(){
        UtilitiesController.deleteChangedFlag(getGridStyleWidget());
    };

    this.viewWidgetBeforeMoveRow = function(){
        var widget = getViewWidget();
        UtilitiesController.deleteChangedFlag(widget);
        resetPeriod(widget);
    };

    this.indicatorsWidgetAfterMoveRow = function(){
        var widget = getIndicatorsWidget();
        UtilitiesController.clearChangedRows(widget);
        widget.activate();
        self.setCurrentIndicatorId(widget);
        self.setIndicatorFilterValues(widget);
        self.setOrderValidation(widget);
        MasterDetailUtilities.afterMoveRow(widget, 1);
    };

    this.viewWidgetOnEnter = function(){
        var widget = getViewWidget();
        var indicator = widget.dataSourceFilter.find(function(column) {
            return column.name === "indicator";
        });
        var idIndicator = null;
        if(indicator)
        {
            idIndicator = indicator.value;
        }else if(widget.currentRow.indicator){
            idIndicator = widget.currentRow.indicator;
        }
        sessionStorage.setItem('indicator',idIndicator);
        localStorageEngine.setLocalVar('_id',widget.currentRow._id ||null);
        self.reloadWidgetIfNeeded(widget);
        self.setViewWidgetMethods(widget);
        PeriodsUtilities.setIntervalsOnPeriodField(getFieldPeriodFromViewWidget());
    };

    this.viewWidgetOnEnterTab = function(){
        var widget = getViewWidget();
        localStorageEngine.setLocalVar('_id',widget.currentRow._id);
        self.buildViewWidgetProperties(widget);
    };

    this.viewWidgetAfterMoveRow = function(){
        var widget = getViewWidget();
        UtilitiesController.clearChangedRows(widget);
        setViewFilterValues(widget);
        self.updateFieldsVisibility();
        self.callAfterMoveRowMethods(widget);
        self.setDrillDownPropertiesVisibility(widget.currentRow.hasDrillDown);
        ViewPropertyFactory.loadSelectMultipleEditableValues(widget.currentRow, widget);
        self.setDateFilterColumn(widget);
        self.setIntervalFieldsNumber(widget, 'IndicatorController');
        MasterDetailUtilities.afterMoveRow(widget, 2);
    };

    this.gridStyleWidgetActionEventAdd = function(){
        var widget = getGridStyleWidget();
        widget.newRow();
    };

    this.gridStyleWidgetActionEventDelete = function(){
        var widget = getGridStyleWidget();
        widget.removeCheckedRows();
    };

    this.columnsSettingsWidgetAfterMoveRow  = function() {
		var isFact = getColumnsSettingsWidget().currentRow.type === 'fact';
		widget.getField('function').isVisible = isFact;
	};
    var getWidgetIndicatorsImport = function() {
        return templateManager.container.getWidget('indicatorsImport');
    };

    this.indicatorsImportContainerAfterinit  = function() {
        var widgetIndicatorsImport = getWidgetIndicatorsImport();
        UtilitiesController.activateAndList(widgetIndicatorsImport);
        widgetIndicatorsImport.reload();
        widgetIndicatorsImport.dataSource.clearCheckedRows();
        widgetIndicatorsImport.updateCheckedRows();

	};

}

Configuration(function(ContextRegister) {
    ContextRegister.register('IndicatorController', IndicatorController);
});

// FILE: assets/js/controllers/AccessControlController.js
function AccessControlController(ScreenService, templateManager) {

	var self = this;
	var widgetsToSetVisibility = ["users", "elements"];

	self.addNewProfile = function (widget) {
		self.setWidgetsVisibility(false, widget.container, widgetsToSetVisibility);
		widget.newRow();
	};

	self.openEdit = function (widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		widget.edit();
		self.setWidgetsVisibility(false, widget.container, widgetsToSetVisibility);
	};

	self.cancelEdit = function (widget) {
		if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			widget.list();
			ScreenService.closeSwipe();
		}
		else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
		self.setWidgetsVisibility(true, widget.container, widgetsToSetVisibility);
	};

	self.closeSwipes = function (widget) {
		ScreenService.closeSwipe();
		widget.parent.activate();
	};

	self.setWidgetsVisibility = function (isVisible, container, widgetsNamesArray) {
		widgetsNamesArray.forEach(
			function (element) {
				 container.getWidget(element).isVisible = isVisible;
			}
		);
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('AccessControlController', AccessControlController);
});

// FILE: assets/js/controllers/ChartTypesController.js
function ChartTypesController(templateManager) {

    function getViewWidget() {
        return templateManager.container.getWidget('view');
    }

    this.setLabelMaskProperty = function(field) {
        var widget = getViewWidget();
        widget.currentRow.labelPropertyMask = {
            type: widget.currentRow[field.name]
        };
    };

    this.setColumnProperties = function(field) {
        var widget = getViewWidget();
        widget.currentRow.columnProperties = widget.currentRow.columnProperties || {};
        widget.currentRow.columnProperties.align = widget.currentRow[field.name];
    };

    this.buildSerieYAxisRange = function(field) {
        var widget = getViewWidget();
        var fieldValue = field.value();
        var range = fieldValue.split(',');
        var serieYAxisRange = {
            min: range[0],
            max: range[1]
        };

        widget.currentRow.serieYAxisRange = serieYAxisRange;
    };

    this.buildSerieYAxisRangeLine = function(args) {
        var widget = getViewWidget();
        var fieldValue = args.owner.field.value();
        var range = fieldValue.split(',');
        var serieYAxisRangeline = {
            min: range[0],
            max: range[1]
        };

        widget.currentRow.serieYAxisRangeLine = serieYAxisRangeline;
    };

    this.onLegendDisplayChange = function(field) {
        var widget = templateManager.container.getWidget('view');
        widget.currentRow.chartLegendVisible = field.value() !== 'removeLegend';
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ChartTypesController', ChartTypesController);
});

// FILE: assets/js/controllers/CollectorLogController.js
function CollectorLogController(restEngine,templateManager,UtilitiesController) {

    var getWidgetLogs = function(){
        return templateManager.container.getWidget('logs');
    };

    var getFieldLogContent = function(){
        return getWidgetLogs().getField('content');
    };


    this.loadLogContent = function(log) {
        return restEngine.post({
            'service': '/collector-log-content',
            'typeRequest': 'POST',
            'params': {
                'requestType': 'Row',
                'row': {
                    'logName': log.name
                }
            }
        }).then(function(response) {
            log.content = response.dataset.logFile[0].log;
            getWidgetLogs().edit();
        });
    };

    this.logFieldOnClickButtonCopyLog = function(){
        var text = getFieldLogContent().value();
        UtilitiesController.copiedToClipboard(text,true);
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('CollectorLogController', CollectorLogController);
});

// FILE: assets/js/controllers/ColorPaletteController.js
function ColorPaletteController(ScreenService, templateManager, restEngine) {
    var allowedCharacters = ["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"];

    var getWidgetColors = function(){
        return templateManager.container.getWidget("colors");
    };

    var getWidgetColorPopup = function(){
        return templateManager.container.getWidget("colorPopup");
    };

    this.colorsActionEventAdd = function(){
        var colorWidget = getWidgetColorPopup();
        colorWidget.currentRow.color = "";
        colorWidget.currentRow.colorHex = "";
        colorWidget.currentRow._id = null;
        colorWidget.currentRow.order = highestOrder()+10;
        ScreenService.openPopup(colorWidget);
    };

    var highestOrder = function(){
        var widget = getWidgetColors();
        var order = [];
        if(widget.dataSource.data.length > 0){
            widget.dataSource.data.forEach(function(data){
                order.push(data.order);
            });
            return Math.max.apply(Math, order);
        }else{
            return -10;
        }
    };

    this.colorsActionEventEdit = function(){
        var colorWidget = getWidgetColorPopup();
        var colorGrid = getWidgetColors();
        colorWidget.currentRow.color = colorGrid.currentRow.color;
        colorWidget.currentRow.colorHex = colorWidget.currentRow.color;
        colorWidget.currentRow._id = colorGrid.currentRow._id;
        colorWidget.currentRow.order = colorGrid.currentRow.order;
        ScreenService.openPopup(colorWidget);
    };

    this.colorHexFieldOnChange = function(){
        var colorWidgetRow = getWidgetColorPopup().currentRow;
        var colorHex = validateColorHex(colorWidgetRow.colorHex);
        colorWidgetRow.colorHex = colorHex;
        colorWidgetRow.color = colorHex;
    };

    var validateColorHex = function(colorHex){
        var allowedChars = allowedCharacters;
        var newColor = "";
        colorHex.split('').forEach(function(char, i){
            if(allowedChars.includes(char.toLowerCase())){
                newColor += char.toLowerCase();
            }
        });
        colorHex = newColor;
        if(!colorHex.startsWith("#")){
            if(colorHex.length >= 7){
                colorHex = colorHex.slice(0, colorHex.length-1);
            }
            colorHex = "#" + colorHex;
        }
        while(colorHex.length < 7){
            colorHex += "0";
        }
        return colorHex;
    };

    this.colorFieldOnChange = function(){
        var colorWidget = getWidgetColorPopup();
        var color = colorWidget.currentRow.color;
        colorWidget.currentRow.colorHex = color.toHexString();
    };

    this.colorsActionEventDelete = function(){
        var widget = getWidgetColors();
        return restEngine.post({
            'service': '/deleteColor',
            'typeRequest': 'POST',
            'params': {
                'requestType': 'Row',
                'row': widget.currentRow
            }
        }).then(function(response) {
            widget.reload();
        });
    };

    this.colorStyle = function(element, row){
        if(element.field && element.field.name == "colorBg"){
            element.css("background-color", row.color);
            element.css("border", "10px solid #ffffff");
        }
    };
    
    this.colorPopupActionEventSave = function(){
        var widget = getWidgetColorPopup();
        if(widget.isValid()){
            return restEngine.post({
                'service': '/saveColor',
                'typeRequest': 'POST',
                'params': {
                    'requestType': 'Row',
                    'row': widget.currentRow
                }
            }).then(function(response) {
                ScreenService.closePopup();
                getWidgetColors().reload();
            });
        }
    };
    
    this.colorPopupActionEventCancel = function(){
        ScreenService.closePopup();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ColorPaletteController', ColorPaletteController);
});

// FILE: assets/js/controllers/ColumnsSettingsController.js
function ColumnsSettingsController(templateManager, IndicatorController, UtilitiesController, ScreenService) {

    var self = this;

    var getColumnsSettingsWidget = function() {
        return templateManager.container.getWidget('columnsSettings');
    };

    var getViewWidget = function() {
        return templateManager.container.getWidget('view');
    };

    var getFieldMaskFromColumnsSettingsWidget = function() {
        return getColumnsSettingsWidget().getField('maskField');
    };

    var getFieldPrecisionFromColumnsSettingsWidget = function() {
        return getColumnsSettingsWidget().getField('precision');
    };

    var getFieldFunctionFromColumnsSettingsWidget = function() {
        return getColumnsSettingsWidget().getField('function');
    };

    this.columnsSettingsWidgetOnEnter = function() {
        var widget = getColumnsSettingsWidget();
        loadDataSource(widget);
        var helpToTranslate = widget.getField('ordirnationPriority').help;
        widget.getField('ordirnationPriority').help = ScreenService.i18n(helpToTranslate);
    };

    function loadDataSource(widget) {
        var viewWidget = getViewWidget();
        widget.dataSource.data = viewWidget.currentRow.columns;
    }

    this.saveColumnsSettings = function(widget) {
        var columns = widget.dataSource.data;
        var viewWidget = getViewWidget();
        viewWidget.currentRow.columns = columns;
        viewWidget.currentRow.orderBy = getMultipleOrderBy(widget);

        return IndicatorController.saveViewWidget().then(function() {
            widget.list();
            widget.activate();
        });
    };

    function getMultipleOrderBy(widget) {
        var orderBy = [];
        var columns = widget.dataSource.data;
        columns.forEach(function(column) {
            if (column.ordirnationPriority) {
                orderBy.push({
                    name: column.name,
                    direction: column.direction || 'asc',
                    priority: column.ordirnationPriority
                });
            }
        });

        orderBy.sort(function(a, b) {
            return a.priority - b.priority;
        });

        return orderBy;
    }

    this.columnsSettingsActionEventEdit = function() {
        var widget = getColumnsSettingsWidget();
        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    this.columnsSettingsWidgetAfterMoveRow = function() {
        var widget = getColumnsSettingsWidget();
        UtilitiesController.clearChangedRows(widget);
        widget.dataSource.clearCheckedRows();

        var isFact = widget.currentRow.type === 'fact';
        this.setFieldMaskVisibility(widget);
        this.setFieldPrecisionVisibily();
        getFieldFunctionFromColumnsSettingsWidget().isVisible = isFact;
    };


    this.columnsSettingsActionEventCancel = function() {
        var widget = getColumnsSettingsWidget();
        Util.extend(widget.currentRow, widget.backupRow);
        widget.view();
    };

    this.setFieldMaskVisibility = function(widget) {
        getFieldMaskFromColumnsSettingsWidget().isVisible = (widget.parent.currentRow.chartType === "grid");
    };

    this.setFieldPrecisionVisibily = function() {
        var valueField = getFieldMaskFromColumnsSettingsWidget().value();
        var chartType = getColumnsSettingsWidget().parent.currentRow.chartType;

        if (chartType === "grid")
            getFieldPrecisionFromColumnsSettingsWidget().isVisible = valueField === "currency" || valueField === "float" || valueField === "percent";
        else
            getFieldPrecisionFromColumnsSettingsWidget().isVisible = false;

    };

    this.columnsSettingsActionEventSave = function() {
        var widget = getColumnsSettingsWidget();
        self.saveColumnsSettings(widget).then(function() {
            widget.view();
        });
    };

    this.columnsSettingsWidgetOnSaveInlineEdit = function() {
        var widget = getColumnsSettingsWidget();
        var columns = widget.dataSource.data;
        var viewWidget = getViewWidget();
        viewWidget.currentRow.columns = columns;
        viewWidget.currentRow.orderBy = getMultipleOrderBy(widget);

        return IndicatorController.viewWidgetActionEventSave().then(function() {
            widget.list();
            widget.activate();
        });
    };

    this.columnsSettingsActionEventBackList = function() {
        ScreenService.closeSwipe();
        getViewWidget().activate();
    };

    this.columnsSettingsActionEventBackView = function() {
        ScreenService.closeSwipe();
    };



}

Configuration(function(ContextRegister) {
    ContextRegister.register('ColumnsSettingsController', ColumnsSettingsController);
});

// FILE: assets/js/controllers/ConnectionController.js
function ConnectionController(ScreenService, ConnectionService, MasterDetailUtilities, MessageService,
UtilitiesController,templateManager,engine) {
    var self = this;
    var DRIVER_SPREADSHEET = "Spreadsheet";
    var DRIVER_MYSQL = "com.mysql.cj.jdbc.Driver";
    var DRIVER_ORACLE= "oracle.jdbc.driver.OracleDriver";
	//Widget interactions
	this.newRow = function(widget){
		widget.newRow();
	};

	this.deleteRows = function(widget){
		widget.removeCheckedRows();
	};

	this.cancelEdit = function (widget) {
		UtilitiesController.cancelEditRow(widget);
        setFieldsConnection(widget);
	};

	this.goBack = function(widget){
		UtilitiesController.goBackWidget(widget);
	};

    var getWidgetConnection = function() {
        return templateManager.container.getWidget('Connection');
    };

    this.connectionContainerAfterinit = function(){
        getWidgetConnection().sourceUrl = UtilitiesController.getUrlZmartNext("conexaoBD");
        Util.doAsync(ScreenService.hideLoader, 2000);
    };

    var getFieldDatabaseFormConnection = function() {
        return getWidgetConnection().getField('database');
    };

    var getFieldDbUrlFormConnection = function() {
        return getWidgetConnection().getField('dbUrl');
    };

    var getFieldIsTeknisaFormConnection = function() {
        return getWidgetConnection().getField('IS_TEKNISA');
    };

    var setFieldsConnection = function (widget) {
        var isSpreadsheet = widget.currentRow.driverClass === DRIVER_SPREADSHEET;
        var isOracle = widget.currentRow.driverClass === DRIVER_ORACLE;
        if(!isOracle){
            widget.currentRow.oracleWalletUse = false;
        }
        var isWallet = widget.currentRow.oracleWalletUse === 'Y';
        getWidgetConnection().fields.forEach(function (field) {
            if (field.isVisibleControl) {
                if(isSpreadsheet){
                    field.isVisible = (field.isVisibleSpreadsheet === 'Y');
                }else if(isOracle){
                    if(isWallet){
                        field.isVisible = (field.isVisibleWallet === 'Y');
                    }else{
                        field.isVisible = (field.isVisibleOracle === 'Y');
                    }
                }else{
                    field.isVisible = field.isVisibleOriginal !== undefined ? field.isVisibleOriginal : true;
                }
            }

            if(field.oracleClass){
                field.class = isOracle ? field.oracleClass : field.originClass ;
            }

            if(field.labelSpreadsheet){
                field.label = isSpreadsheet ? field.labelSpreadsheet : field.labelOriginal ;
            }

            if(field.helpSpreadsheet || field.helpWallet){
                if(isSpreadsheet){
                    field.help = field.helpSpreadsheet;
                }else if(isWallet){
                    field.help = field.helpWallet;
                }else {
                    field.help = field.helpOriginal;
                }
            }
        });
        getFieldIsTeknisaFormConnection().isVisible = !!getLocalVar('isTeknisa');
    };

    var getNrorg = function() {
        var nrorg = getLocalVar('ORGANIZATION_ID');
        var clienteNRORG = getLocalVar('clienteNRORG');
        if (getLocalVar("clienteNRORG")) {
            nrorg = clienteNRORG;
        }
        return nrorg;
    };

    var setHelpWallet = function() {
        var nrorg = getNrorg();
        var fieldDbUrl = getFieldDbUrlFormConnection();
        fieldDbUrl.helpWallet = fieldDbUrl.helpWallet.replace('NRORG',nrorg);
    };

     var validWallet = function(currentRow){
         var valid = true;
         if(false && currentRow.oracleWalletUse){
             var nrorg = getNrorg();
             var wallet = 'ZMART_'+nrorg+'_';
             if(currentRow.dbUrl.indexOf(wallet) === -1){
                 valid = false;
                 MessageService.showMessageByCode('ERROR_WALLET_SECURITY',{'wallet':wallet},{type:'error'});
             }
         }
         return valid;
     };

    this.ConnectionWidgetAfterMoveRow = function(widget){
        setFieldsConnection(widget);
        UtilitiesController.clearChangedRows(widget);
    };

    var saveConnection = function (widget) {
        return widget.saveRow().then(function (response) {
            widget.currentRow._id = response.dataset.connection[0]._id;
            widget.currentRow.password = response.dataset.encryptedPassword[0];
            clearRowAfterSave(widget);
            widget.view();
        }, function (response) {
            if(response.data.errorCode === 1){
                var nrorg = getNrorg();
                var wallet = 'USR_ORG_'+nrorg;
                MessageService.showMessageByCode('ERROR_WALLET_SECURITY',{'wallet':wallet},{type:'error'});
            }else{
                UtilitiesController.showMessage(response.data.error);
            }
        });
    };

    var validDirSpreadsheet = function(currentRow){
        var valid = true;
        if(currentRow.dbUrl.split('.').pop() != 'xlsx'){
            valid = false;
        }
        return valid;

    };

    this.saveConnection = function (widget) {
        if (widget.isValid()) {
            addPrefixUrlWallet(widget);
            removeWallet(widget);
            var driverClass = widget.currentRow.driverClass;
            if (driverClass == DRIVER_SPREADSHEET || driverClass == DRIVER_MYSQL) {
                if (driverClass == DRIVER_SPREADSHEET) {
                    if (!validDirSpreadsheet(widget.currentRow)) {
                        MessageService.showMessageByCode('EXTENSION_ERRO_XLSX');
                        throw 'File invalid';
                    }
                    widget.currentRow.username = DRIVER_SPREADSHEET;
                    widget.currentRow.password = DRIVER_SPREADSHEET;
                    
                }
                return testConnection(widget, true).then(function (version) {
                    if (engine.hasErrorValidVersion(version, '1.3.0')) {
                        var codeMessage = 'NOT_VERSION_MIN_SPREADSHEET';
                        if (driverClass == DRIVER_MYSQL) {
                            codeMessage = 'NOT_VERSION_MIN_MYSQL';

                        }
                        MessageService.showMessageByCode(codeMessage, {'V': version});
                    } else {
                        return saveConnection(widget);
                    }
                });
            } else {
                return saveConnection(widget);
            }
        } else {
            MessageService.showMessageByCode("REQUIRED_FIELDS");
        }
    };

	this.edit = function(widget){
		MasterDetailUtilities.masterDetailEdit(widget);
	};

    this.connectionFieldOnChangeDatabase = function(args){
        setFieldsConnection(getWidgetConnection());
    };

    this.connectionFieldOnChangeOracleWalletUse = function(args){
        setFieldsConnection(getWidgetConnection());
    };
    var addPrefixUrlWallet = function (widget) {
        if(widget.currentRow.oracleWalletUse === 'Y' &&
            widget.currentRow.dbUrl.indexOf('jdbc:oracle:thin:@') === -1)
        {
            widget.currentRow.dbUrl = 'jdbc:oracle:thin:@' + widget.currentRow.dbUrl;
        }
    };
    var removeWallet = function (widget) {
        if(widget.currentRow.oracleWalletUse === 'Y' )
        {
            widget.currentRow.username = "";
            widget.currentRow.usernameSsl = "";
            widget.currentRow.password = "";
            widget.currentRow.passwordSsl = "";
        }
    };

    var testConnection = function (widget,notMessage) {
        addPrefixUrlWallet(widget);
        return ConnectionService.testConnection(widget.currentRow).then(function (response) {
            if (response && response.dataset && response.dataset.testResponse) {
                response = response.dataset.testResponse;
                var testResponse = response.response;
                var hasError = response.hasError;
                if (hasError) {
                    UtilitiesController.showMessage(testResponse, 'error');
                    throw 'Erro backEnd';
                } else {
                    var clientVersion = testResponse.version;
                    var succeededCon = ScreenService.i18n("Conectado com sucesso com o coletor na versão ");
                    if(!notMessage) {
                        UtilitiesController.showMessage(succeededCon + clientVersion);
                    }
                    return clientVersion;
                }
            }
        }, function (response) {
            if(response.errorCode === 1){
                var nrorg = getNrorg();
                var wallet = 'USR_ORG_'+nrorg;
                MessageService.showMessageByCode('ERROR_WALLET_SECURITY',{'wallet':wallet},{type:'error'});
            }else{
                UtilitiesController.showMessage(response.error);
            }
            throw 'Erro backEnd';
        });
    };

    this.testConnection = function (widget) {
        if (widget.isValid()) {
            testConnection(widget);
        } else {
            MessageService.showMessageByCode("REQUIRED_FIELDS");
        }
    };

	//Auxiliar functions
	function clearRowAfterSave(widget) {
		var attributesToClear = ["__is_new", "changed", "passwordNeedEncrypt"];
		attributesToClear.forEach(function(attribute) {
			delete widget.currentRow[attribute];
		});
	}

	this.setEncryptFlag = function(field) {
		var encryptPropertyName = field.name + "NeedEncrypt";
		field.widget.currentRow[encryptPropertyName] = true;
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('ConnectionController', ConnectionController);
});


// FILE: assets/js/controllers/DashboardController.js
function ZmartDashboardController(IndicatorFactory, ScreenService, IndicatorService, DashboardService,
MessageService, ViewController, MasterDetailUtilities, UtilitiesController, IndicatorController,templateManager) {

	var getDashboardConfigurationWidget = function () {
		return templateManager.container.getWidget('dashboardConfiguration');
	};

	var getFieldAccessProfilesFromDashboardConfigurationWidget = function () {
		return getDashboardConfigurationWidget().getField('accessProfiles');
	};

	//Dashboard import
	this.openSetOriginOrganization = function(widget){
        var setOrganizationWidget = IndicatorFactory.factoryImportNowWidget(widget.container, 'dashboard');
        setOrganizationWidget.isVisible = true;
		setOrganizationWidget.newRow();
		ScreenService.openPopup(setOrganizationWidget);
    };

    this.openImportDashboard = function(widget){
        var baseConnection = widget.currentRow.baseConnection || null;
        IndicatorService.setBaseConnectionToImport(baseConnection).then(function(){
            ScreenService.openWindow('dashboardsImport');
        });
	};
	
	this.dashboardGridOnClick = function(widget,row,event){
		if(Object.keys(row).length > 0){
			event(widget,row);
		} else if(widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0){
			event(widget,widget.dataSource.data[0]);
		}
	};

	this.importDefaultOrgDashboards = function (widget) {
		var checkedRows = widget.dataSource.getCheckedRows();
		var DashboardIds = checkedRows.map(function (indicator) {
			return indicator._id;
		});
		DashboardService.checkDrillDownIndicators(DashboardIds).then(function(drillDownIndicators){
			if(drillDownIndicators && drillDownIndicators.dataset && drillDownIndicators.dataset.checkDrillDownIndicators.length > 0){
				var names = drillDownIndicators.dataset.checkDrillDownIndicatorsNames;
				drillDownIndicators = drillDownIndicators.dataset.checkDrillDownIndicators;
				var message = IndicatorController.drillDownIndicatorsMessage(drillDownIndicators,names);
				if(message){
                    UtilitiesController.confirmMessageByCode("DRILL_DOWN_INDICATORS",
                    {
                        "i": message
                    },{
                        "type": 'CONFIRMATION',
                        "success": function (){
                            importDashboards(DashboardIds,widget,drillDownIndicators);
                        },
                        "fail": function(){
                            importDashboards(DashboardIds,widget);
                        }
                    });
                } else {
                    importDashboards(DashboardIds,widget);
                }
			} else {
				importDashboards(DashboardIds,widget);
			}
		});
	};

	var importDashboards = function(DashboardIds,widget,additionalIndicators){
		if(!additionalIndicators){
			additionalIndicators = false;
		}
		DashboardService.importDefaultOrgDashboards(DashboardIds,additionalIndicators).then(function (response) {
			MessageService.successNotificationByCode('INDICATOR_IMPORTED_WITH_SUCCESS');
			ScreenService.openWindow('dashboardConfiguration');
			widget.dataSource.clearCheckedRows();
		}, function (response) {
			MessageService.errorNotificationByCode('ERROR_ON_IMPORT_INDICATORS');
		});
	};

	this.dashboardConfigurationAfterMoveRow= function(widget){
		UtilitiesController.clearChangedRows(widget);
		ViewController.dashboardAfterMoveRow(widget);
		setLocalVar('view',widget.currentRow._id);
		MasterDetailUtilities.afterMoveRow(widget, 1);
	};

	this.dashboardConfigurationActionEventCancel= function(widget) {
		ViewController.cancel(widget);
		UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getFieldAccessProfilesFromDashboardConfigurationWidget());

	};
	this.dashboardConfigurationActionEventSave= function(widget) {
		ViewController.saveAndUpdateDashboard(widget);
		UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getFieldAccessProfilesFromDashboardConfigurationWidget());

	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('DashboardController', ZmartDashboardController);
});

// FILE: assets/js/controllers/DashboardsImport.js
function DashboardsImportController(ScreenService, templateManager, UtilitiesController) {

    var getWidgetDashboardsImport = function(){
        return templateManager.container.getWidget('dashboardsImport');
    };

    this.containerAfterinitDashboardImport = function() {
        var widget = getWidgetDashboardsImport();
        UtilitiesController.activateAndList(widget);
        widget.reload();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('DashboardsImportController', DashboardsImportController);
});

// FILE: assets/js/controllers/DataSourceManagerController.js
function DataSourceManagerController(ExpressionInterpreter, DataSourceManagerService, UtilitiesController,
MasterDetailUtilities, ScreenService, templateManager, DataSourceManagerFactory, metaDataFactory, ZHPromise,
JoinDataSourceService, PeriodsManagerService, MessageService, WidgetService, PeriodsUtilities,ConnectionService,
engine) {

	var self = this;
	var logWidget;
	var filterWidget;
    var dataVisualizationDataSource;
	var JOIN_DATASOURCE_FLAG = "joinDataSource";
	var dataSourceRefreshAlert = "You are about to overwrite your current datasource keys by the dimensions. This will guarantee your data consistency, but this may causa your dataSource to import slowly. Are you sure of that?";
	var DRIVER_SPREADSHEET = "Spreadsheet";
	var CODE_FINALITY_SEND_MAIL = DataSourceManagerService.CODE_FINALITY_SEND_MAIL;
	var CODE_FINALITY_IMPORTATION = DataSourceManagerService.CODE_FINALITY_IMPORTATION;
	var indexFieldNumber = 1;
	var indexIdList = {};

	var getDefaultColumnsWidget = function() {
		return templateManager.container.getWidget('defaultColumnsWidget');
	};

	this.defaultColumnsContainerAfterinit = function () {
		getDefaultColumnsWidget().sourceUrl = UtilitiesController.getUrlZmartNext('defaultColumns');
	};

    var getImportNowWidget = function () {
        return  templateManager.container.getWidget('importNow');
    };

    var getImportNowParentWidget = function () {
      return  templateManager.container.getWidget('importNow').parent;
	};

    var getWidgetLog = function () {
      return  templateManager.container.getWidget('log');
	};

	var getAnotherWidgetLog = function (name) {
		return  templateManager.container.getWidget('log'+name);
	  };

	var getIndexesWidget = function(){
		return templateManager.container.getWidget("indexes");
	};

	var getIndexAddWidget = function(){
		return templateManager.container.getWidget("INDEX_ADD");
	};

	var getApiWidget = function(){
		return templateManager.container.getWidget("API");
	};

	var getApiUsageLogWidget = function(){
		return templateManager.container.getWidget("API_USAGE_LOG");
	};

	var getWidgetDataEdit = function(){
		return templateManager.container.getWidget("DATA_EDIT");
	};

	var getWidgetDataVisualization = function(){
		return templateManager.container.getWidget("dataVisualization");
	};

	var getWidgetImportScheduling= function(){
		return templateManager.container.getWidget("importScheduling");
	};

	var getActionEditFromDataVisualization = function(){
		return getWidgetDataVisualization().getAction('EDIT');
	};

	var getActionEditFromDataSource = function(){
		return getWidgetDataSource().getAction('EDIT');
	};

	var getActionSaveFromDataSource = function(){
		return getWidgetDataSource().getAction('SAVE');
	};

	var getActionSaveFromColumns = function(){
		return getWidgetColumns().getAction('SAVE');
	};

	var getActionEditFromColumns = function(){
		return getWidgetColumns().getAction('EDIT');
	};

	var getActionAddFromColumns = function(){
		return getWidgetColumns().getAction('ADD');
	};

	var getActionImportFromColumns = function(){
		return getWidgetColumns().getAction('IMPORT');
	};

	var getActionDeleteFromColumns = function(){
		return getWidgetColumns().getAction('DELETE');
	};

	var getActionCleanRelationsAllFromRelations = function(){
		return getWidgetRelateStructures().getAction('CLEAN_RELATIONS_ALL');
	};

	var getActionEditFromImportScheduling = function(){
		return getWidgetImportScheduling().getAction('EDIT');
	};

	var getActionSaveFromImportScheduling= function(){
		return getWidgetImportScheduling().getAction('SAVE');
	};

	var getFieldIsTeknisaFormDataSource = function(){
		return getDataSourceWidget().getField('IS_TEKNISA');
	};

	var getFieldQueryFormDataSource = function(){
		return getDataSourceWidget().getField('query');
	};

	var getFieldColumnsFromIndexAddWidget = function(id){
		return getIndexAddWidget().getField("COLUMN_"+id);
	};

	var getFieldPersistTypeFromImportNow= function(){
		return getImportNowWidget().getField("persistType");
	};

	var getFieldNameIndexAddWidget = function(){
		return getIndexAddWidget().getField("NAME");
	};

	var getFieldFieldDatePeriodApiWidget = function(){
		return getApiWidget().getField("FIELD_DATE_PERIOD");
	};

	var getFieldFieldDateLabelApiWidget = function(){
		return getApiWidget().getField("FIELD_DATE_LABEL");
	};
	var getFieldApiLinkApiWidget = function(){
		return getApiWidget().getField("API_LINK");
	};
	var getFieldWebTokenApiWidget = function(){
		return getApiWidget().getField("WEB_TOKEN");
	};
	var getFieldApiPowerBiApiWidget = function(){
		return getApiWidget().getField("API_POWER_BI");
	};
	var getFieldNmOperadorApiWidget = function(){
		return getApiWidget().getField("NMOPERADOR");
	};

	var getActionDownloadJsonApiWidget = function(){
		return getApiWidget().getAction("DOWNLOAD_JSON");
	};



	self.dataSourceNewRow = function(widget) {
		getFieldIsTeknisaFormDataSource().isVisible = !!getLocalVar('isTeknisa');
		UtilitiesController.changeWidgetVisibility(widget.widgets, false);
		widget.newRow();
		UtilitiesController.setValueInSelectField(getFieldHasDataMiningForFinalityName(),CODE_FINALITY_IMPORTATION);
	};

	self.dataSourceEditRow = function(widget) {
		UtilitiesController.changeWidgetVisibility(widget.widgets, false);
		MasterDetailUtilities.masterDetailEdit(widget);
	};

	self.setReadOnlyToJoinDataSourceFlag = function(widget){
		var readOnly = widget.isNewRow() ? false : true;
		UtilitiesController.setFieldsReadOnly(widget, [JOIN_DATASOURCE_FLAG], readOnly);
	};


    var getWidgetDataSource = function() {
        return templateManager.container.getWidget('dataSource');
    };

    var getWidgetColumns = function() {
        return templateManager.container.getWidget('columns');
    };

    var getWidgetRelateStructures = function() {
        return templateManager.container.getWidget('relateStructures');
    };

    var getWidgetRelations = function() {
        return templateManager.container.getWidget('relations');
    };

    var getFieldHasDataMiningForDataSource= function() {
        return getWidgetDataSource().getField('hasDataMining');
    };
    var getFieldHasDataMiningForFinalityName= function() {
        return getWidgetDataSource().getField('finalityName');
    };

    var getFieldConnectionNameForDataSource= function() {
        return getWidgetDataSource().getField('connectionName');
    };

    this.logWidgetOnEnter = function(){
		getWidgetLog().reload();
	};

	this.anotherLogWidgetOnEnter = function(name){
		getAnotherWidgetLog(name).reload();
	};

    var setFieldsExcel = function (widget) {
        var idcon = widget.currentRow.connection;
        var fieldConnection = getFieldConnectionNameForDataSource();
        var connection = UtilitiesController.getRowInDataSource(fieldConnection.dataSource,'_id',idcon);
        var isSpreadsheet = connection && connection.driverClass == DRIVER_SPREADSHEET;
        var fieldHasDataMining = getFieldHasDataMiningForDataSource();
        fieldHasDataMining.readOnly = isSpreadsheet;
        if(isSpreadsheet){
            fieldHasDataMining.value(fieldHasDataMining.valueToTrue);
        }
        widget.fields.forEach(function (field) {
            if(field.labelSpreadsheet){
                field.label = isSpreadsheet ? field.labelSpreadsheet : field.labelOriginal ;
            }

            if(field.helpSpreadsheet){
                field.help = isSpreadsheet ? field.helpSpreadsheet : field.helpOriginal;
            }
        });
    };

    var setFieldsFinality = function (widget) {
        var codeFinality = widget.currentRow.finality;
        var isSendEmail = codeFinality === CODE_FINALITY_SEND_MAIL;
        var fieldHasDataMining = getFieldHasDataMiningForDataSource();
        if(isSendEmail){
            fieldHasDataMining.value(fieldHasDataMining.valueToTrue);
        }
        widget.fields.forEach(function (field) {
            if(field.isVisibleSendEmail){
                field.showOnForm = isSendEmail ? field.isVisibleSendEmail === 'Y' : field.isVisibleSendEmail !== 'Y';
                field.showOnView = isSendEmail ? field.isVisibleSendEmail === 'Y' : field.isVisibleSendEmail !== 'Y';
                if (field.isVisibleSendEmail === 'Y') {
                    field.validations = isSendEmail ? {"required": {}} : {};
                }
            }
        });
        getWidgetImportScheduling().fields.forEach(function (field) {
            if(field.isVisibleSendEmail){
                field.showOnForm = isSendEmail ? field.isVisibleSendEmail === 'Y' : field.isVisibleSendEmail !== 'Y';
                field.showOnView = isSendEmail ? field.isVisibleSendEmail === 'Y' : field.isVisibleSendEmail !== 'Y';
            }
        });
        getFieldPersistTypeFromImportNow().showOnForm = !isSendEmail;
    };

    self.dataSourceFieldOnChangeConnectionName = function(args){
        UtilitiesController.setSelectValueNullWhenEmpty(args.owner.field, 'connection');
        setFieldsExcel(getWidgetDataSource());
    };

    self.dataSourceFieldOnChangeFinalityName = function(args){
        UtilitiesController.setSelectValueNullWhenEmpty(args.owner.field, args.owner.field.name);
        setFieldsFinality(getWidgetDataSource());
    };

	self.dataSourceRollback = function(widget) {
		UtilitiesController.changeWidgetVisibility(widget.widgets, true);
		MasterDetailUtilities.masterDetailRollback(widget);
		setFieldsExcel(getWidgetDataSource());
        setFieldsFinality(getWidgetDataSource());
		JoinDataSourceService.configWidgetWhenJoinDataSource(widget.currentRow.joinDataSource, 'dataSource');
	};
	self.closeViewLog = function (widget) {
		MasterDetailUtilities.masterDetailBackToList(getWidgetLog());
	};

	self.deleteIndexes = function(widget){
		DataSourceManagerService.deleteIndexes(widget.getCheckedRows()).then(function(){
			getIndexesWidget().reload();
		});
	};

	self.addIndex = function(){
		var widget = getIndexAddWidget();
		widget.dataSource.data = [];
		widget.newRow();
		ScreenService.openSwipe(widget);
		widget.edit();
	};

	self.viewIndex = function(){
		var widget = getIndexAddWidget();
		var indexesCurrentRow = getIndexesWidget().currentRow;
		var currentRow = {};
		var keys = Object.keys(indexesCurrentRow.key);
		var values = Object.values(indexesCurrentRow.key);
		currentRow.NAME = indexesCurrentRow.name;
		resetFields();
		for(var i=0;i<keys.length;i++){
			currentRow["COLUMN_"+(i+1)] = keys[i];
			currentRow["ORDER_"+(i+1)] = values[i];
			createFields(i+1);
		}
		widget.currentRow = currentRow;
		widget.view();
		var fields = widget.fields;
		templateManager.updateTemplate();
		ScreenService.openSwipe(widget).then(function(){
			widget.fields = fields;
		});
	};

	self.createIndexFields = function(reset){
		if(reset){
			indexFieldNumber = 1;
			resetFields();
		}
		createFields(indexFieldNumber);
		indexIdList[indexFieldNumber] = indexFieldNumber;
		indexFieldNumber++;
	};

	var createFields = function(id){
		var widget = getIndexAddWidget();
		var fields = indexFields(id);
		fields.forEach(function(field){
			field.isVisible = true;
			widget.fields.push(metaDataFactory.factoryField(field,widget));
		});
		templateManager.updateTemplate();
	};

	var resetFields = function(){
		var widget = getIndexAddWidget();
		var fields = indexStaticFields();
		widget.fields = [];
		fields.forEach(function(field){
			widget.fields.push(metaDataFactory.factoryField(field,widget));
		});
		templateManager.updateTemplate();
	};

	self.removeIndexField = function(id){
		var fields = indexFields(id), fieldNames = [];
		var widgetFields = getIndexAddWidget().fields;
		if(widgetFields.length > fields.length + indexStaticFields().length){
			fields.forEach(function(field){
				fieldNames.push(field.name);
			});
			widgetFields.forEach(function(field, i){
				if(fieldNames.includes(field.name)){
					delete widgetFields[i];
				}
			});
			delete indexIdList[id];
			getIndexAddWidget().fields = widgetFields.filter(Boolean);
			templateManager.updateTemplate();
		}
	};

	self.setColumnIndexFieldDataSource = function(id){
		getFieldColumnsFromIndexAddWidget(id).dataSource.data = getColumnsWidget().dataSource.data;
	};

	self.apiFieldOnClickButtonCopyLink = function(){
		var text = getFieldApiLinkApiWidget().value();
		UtilitiesController.copiedToClipboard(text);
	};

	self.apiFieldOnClickButtonButtonCopyToken = function(){
		var text = getFieldWebTokenApiWidget().value();
		UtilitiesController.copiedToClipboard(text);
	};

	self.apiFieldOnClickButtonButtonCopyPowerBi = function(){
		var text = getFieldApiPowerBiApiWidget().value();
		UtilitiesController.copiedToClipboard(text);
	};


	self.apiFieldOnClickButtonDownloadDocument = function(){
		window.open(serviceUrl + "/api/downloadAPiDocument");
	};

	self.apiActionEventDownloadDocument = function(){
		window.open(serviceUrl + "/api/downloadAPiDocument");
	};

	self.apiActionEventApiUsageLog = function(){
		var widget = getApiUsageLogWidget();
		ScreenService.openSwipe(widget).then(function(){
			widget.list();
		});
	};

	function ajaxDownloadJsonApi(widget) {
		return $.ajax({
			url: widget.currentRow.API_LINK + '?DOWNLOAD=1',
			type: 'get',

			headers: {
				"Content-Type": 'application/json:charset=utf-8',
				"WEB_TOKEN": widget.currentRow.WEB_TOKEN,
			},
			dataType: 'json',
			success: function (response) {
				var filename = 'report.json';
				var text = JSON.stringify(response);
				UtilitiesController.downloadFileInJs(filename, text);

			},
			error: function (response) {
				console.log(response);
			}
		});
	}

	self.apiFieldOnClickButtonDownloadJson = function(){
		var widget = getApiWidget();
		if(getLocalVar('testeCors')){
			ajaxDownloadJsonApi(widget);
		}else{
			DataSourceManagerService.createTokenTemp(widget.currentRow.WEB_TOKEN).then(function (response) {
				if(response.dataset && response.dataset.apiDataSource[0]){
					var token = response.dataset.apiDataSource[0];
					var url = widget.currentRow.API_LINK + '?TEMP_TOKEN=' + token;
					window.open(url);
				}
			});
		}
	};

	self.apiActionEventBackParent = function(){
		var widget = getApiWidget();
		MasterDetailUtilities.materDetailBackToParent(widget);
	};
	self.apiUsageLogActionEventBack = function(){
		var widget = getApiUsageLogWidget();
		MasterDetailUtilities.masterDetailBackToList(widget);
	};
	self.apiUsageLogActionEventStop = function (event) {
		UtilitiesController.confirmMessageByCode("STOP_API_DATASOURCE_CONF", {}, {
			"type": 'CONFIRMATION',
			"success": function () {
				var widget = getApiUsageLogWidget();
				DataSourceManagerService.stopApi({_id: event.row._id}).then(function (response) {
					MessageService.showMessageByCode('STOP_API_DATASOURCE');
					widget.reload();
				}).catch(function (error) {
					UtilitiesController.showMessage(error.data.error);
				});
			}
		});
	};

	self.apiUsageLogActionEventBackParent = function(){
		var widget = getApiUsageLogWidget();
		MasterDetailUtilities.materDetailBackToParent(widget);
	};

	self.apiActionEventBack = function(){
		var widget = getApiWidget();
		MasterDetailUtilities.masterDetailBackToList(widget);
	};

	self.apiActionEventCancel = function(){
		var widget = getApiWidget();
		MasterDetailUtilities.masterDetailRollback(widget);
	};

	self.apiActionEventAdd = function(){
		var widget = getApiWidget();
		widget.newRow();
		widget.currentRow.API_ID = getLocalVar('DATA_SOURCE_ID');
		var userId = getLocalVar('userId');
		var fieldOperador = getFieldNmOperadorApiWidget();
		UtilitiesController.setOutData(fieldOperador,userId);

	};

	self.apiActionEventEdit = function(){
		var widget = getApiWidget();
		MasterDetailUtilities.masterDetailEdit(widget);
	};

	self.apiActionEventSave = function(){
		var widget = getApiWidget();
		if (widget.isValid()) {
            widget.currentRow.changed = true;
            if(widget.currentRow.FIELD_DATE_PERIOD.period && widget.currentRow.FIELD_DATE_PERIOD.period.name){
                widget.currentRow.PERIOD_CODE = PeriodsUtilities.getPeriodValueByCodedName(widget.currentRow.FIELD_DATE_PERIOD.period.name);
            }
            return widget.saveRow().then(function(response) {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;

                if (response.dataset.apiDataSource[0]._id) {
                    widget.currentRow._id = response.dataset.apiDataSource[0]._id;
                    widget.currentRow.API_LINK = response.dataset.apiDataSource[0].API_LINK;
                    widget.currentRow.WEB_TOKEN = response.dataset.apiDataSource[0].WEB_TOKEN;
                    widget.currentRow.API_POWER_BI = response.dataset.apiDataSource[0].API_POWER_BI;
                }
                widget.view();
            }).catch(function(error){
                UtilitiesController.showMessage(error.data.error);
            });
		}
	};

	self.apiActionEventDelete = function(){
		var widget = getApiWidget();
		widget.removeCheckedRows().then(function () {
			widget.dataSource.checkedRows = [];
		});
	};

	self.setDataSourceFieldColumnDateApi = function(field) {
		var dataSourceWidget = getDataSourceWidget();
		var dataSourceColumns = dataSourceWidget.currentRow.columns;

		var dateColumns = [];
		if(Util.isArray(dataSourceColumns)) {
			dateColumns = dataSourceColumns.filter(function(column) {
				return column.isDate;
			});
		}
		field.dataSource.data = dateColumns;
	};

	self.apiWidgetOnEnterTab = function(){
		PeriodsUtilities.setIntervalsOnPeriodField(getFieldFieldDatePeriodApiWidget());
		self.setDataSourceFieldColumnDateApi(getFieldFieldDateLabelApiWidget());
		getApiWidget().reload();
	};
	self.apiUsageLogWidgetOnEnterTab = function(){

		setLocalVar('WEB_TOKEN',getApiWidget().currentRow.WEB_TOKEN);
		getApiUsageLogWidget().reload();
	};

	self.setNameIndexFieldDataSource = function(id){
		var nameField = getFieldNameIndexAddWidget();
		var nameFieldValue = nameField.value();
		var columnFieldValue = getFieldColumnsFromIndexAddWidget(id).value();
		if(nameFieldValue === "" || nameFieldValue === undefined || nameFieldValue === columnFieldValue){
			nameField.value(columnFieldValue);
		}
	};

	self.saveIndexes = function(widget){
		if(widget.isValid()){
			UtilitiesController.confirmMessageByCode("INDEX_ADD_TIMEOUT_WARNING",{},{
                "type": 'CONFIRMATION',
                "success": function (){
                    var fieldNameList = ["COLUMN_", "ORDER_"];
					var idList = Object.values(indexIdList);
					var row = {};
					fieldNameList.forEach(function(name){
						row[name] = [];
						idList.forEach(function(id){
							row[name].push(widget.currentRow[name+id]);
						});
					});

					row.name = widget.currentRow.NAME;
					row.dataSource = getLocalVar('DATA_SOURCE_ID') ? getLocalVar('DATA_SOURCE_ID') : 0;
					DataSourceManagerService.saveIndexes(row).then(function(response){
						if(response && response.dataset && response.dataset.message){
							return UtilitiesController.confirmMessageByCode("CHANGE_DATASOURCES_FROM_OTHER_CLIENTS",{i: getChangeDatasourcesMessageParameters(response.dataset.message)},{
								"type": 'CONFIRMATION',
								"success": function (){
									row.confirm = true;
									return DataSourceManagerService.saveIndexes(row).then(function(){
										ScreenService.closeSwipe();
										getIndexesWidget().reload();
									});
								}
							});

						}else{
							ScreenService.closeSwipe();
							getIndexesWidget().reload();
						}
					}).catch(function(error){
						if(error){
							if(error.data === null && error.status === 0){
								ScreenService.closeSwipe();
								MessageService.showMessageByCode('INDEX_ADD_TIMEOUT');
							} else if(error.error && error.error.includes("Index must have unique name.")) {
								MessageService.showMessageByCode('INDEX_REPEATED_NAME');
							} else {
								ScreenService.closeSwipe();
								getIndexesWidget().reload();
							}
						}
					});
                }
            });
		}
	};

	var indexStaticFields = function(){
		return [
			{
				"name": "NAME",
				"label": "Name",
				"maxlength": 80,
				"template": "field/text-edit.html",
				"validations": {"required": true},
				"class": 9,
				"isVisible": true
			},
			{
				"name": "ADD",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": false,
				"showOnView": false,
				"label": "Add column",
				"template": "field/label-with-icon.html",
				"icon": "plus",
				"class": 3,
				"events": [
					{
						"name": "ActionEvent",
						"code": "DataSourceManagerController.createIndexFields();"
					}
				]
			}
		];
	};

	var indexFields = function(id){
		return [
			{
				"name": "COLUMN_"+id,
				"label": "Column",
				"indexWidgetId": id,
				"template": "zh-select-autocomplete#field/select-autocomplete.html",
				"class": 6,
				"events": [
					{
						"name": "FieldOnClick",
						"code": "DataSourceManagerController.setColumnIndexFieldDataSource("+id+");"
					},
					{
						"name": "FieldOnChange",
						"code": "DataSourceManagerController.setNameIndexFieldDataSource("+id+");"
					}
				],
				"valueField": "name",
				"descriptionField": "name",
				"validations": {
					"required": true
				},
				"dataSource": {
					"data": []
				}
			},
			{
				"name": "ORDER_"+id,
				"label": "Order",
				"template": "field/radio-group.html",
				"valueField": "value",
				"descriptionField": "description",
				"class": 5,
				"columns": 2,
				"defaultOption": 0,
				"dataSource": {
					"data": [
						{
							"value": "1",
							"description": "Crescente"
						},
						{
							"value": "-1",
							"description": "Decrescente"
						}
					]
				}
			},
			{
				"name": "REMOVE_"+id,
				"isVisible": true,
				"showOnForm": true,
				"showOnList": false,
				"showOnView": false,
				"template": "field/label-with-icon.html",
				"icon": "trash",
				"class": 1,
				"events": [
					{
						"name": "FieldOnClick",
						"code": "DataSourceManagerController.removeIndexField("+id+");"
					}
				]
			}
		];
	};

	var saveDataSourceReject = function(response) {
		if(response.data){
			if(response.data.errorCode && response.data.errorCode === 1){
				MessageService.showMessageByCode('DATASOURCE_REPEATED_COLUMN_NAME',{"C": response.data.error.replace("Exception: ","")});
			}else if(response.data.errorCode && response.data.errorCode === 2){
				MessageService.showMessageByCode('ERRO_START_END_QUERY_DATASOURCE');
			}else{
				UtilitiesController.showMessage(response.data.error, 'E');
			}
		} else {
			MessageService.errorNotificationByCode('ERROR_ON_SAVE_DATASOURCE');
		}
		throw response;
	};

	var saveDataSourceResolve = function(widget,showMessage,response){
		UtilitiesController.changeWidgetVisibility(widget.widgets, true);
		self.setValuesAfterSave(widget, response);
		self.updateIndicatorsOnChangeColumns(widget.currentRow);
		MasterDetailUtilities.formBackToView(getImportSchedulingWidget());
		widget.view();
		JoinDataSourceService.configWidgetWhenJoinDataSource(widget.currentRow.joinDataSource, 'dataSource');
		self.setVisibilityApi();
		if(showMessage) {
			var dataColumnsWidget = getColumnsWidget().dataSource.data;
			if(!widget.currentRow.hasDataMining){
				dataColumnsWidget.forEach(function(data) {
					if (data.specialFunction == "case"){
						if(widget.backupRow.hasDataMining !== widget.currentRow.hasDataMining) {
							UtilitiesController.showMessage("CASE_ADJUST_DATASOURCE");
						}
					}
				});
			} else {
					dataColumnsWidget.forEach(function(data){
						if(data.specialFunction == "case"){
							if(widget.backupRow.hasDataMining !== widget.currentRow.hasDataMining){
								UtilitiesController.showMessage("CASE_ADJUST_DATAMINING");
							}
						}
					});
			}
		}
	};

	var getChangeDatasourcesMessageParameters = function(messages){
		var messageParameters = "";
		Object.keys(messages).forEach(function(nrorg){
			messageParameters += "<br>• <b>["+nrorg+"]</b>: "+messages[nrorg];
		});
		return messageParameters;
	};

    self.validateVersionMinSendEmail = function (widget) {
        var returnPromise = ZHPromise.defer();
        if(widget.currentRow.nameAttachment){
            if(!widget.currentRow.options ){
                widget.currentRow.options = {};
            }
            widget.currentRow.options.nameAttachment = widget.currentRow.nameAttachment;

        }
        if (widget.currentRow.finality === CODE_FINALITY_SEND_MAIL) {
            ConnectionService.testConnectionById(widget.currentRow.connection).then(function (response) {
                var version = response.dataset.testConnections[0].testResult.version;
                var maxVersion = '1.7.0';
                if(engine.hasErrorValidVersion(version, maxVersion)){
                    var messageParameters = {
                        connName: widget.currentRow.connectionName,
                        version: version
                    };
                    UtilitiesController.showMessage('UPDATE_VERSION_CONNECTED', 'Erro', undefined, messageParameters);
                    returnPromise.reject();
                } else {
                    returnPromise.resolve();
                }

            }).catch(function (response) {
                UtilitiesController.showMessage(response.error);
                returnPromise.resolve();
            });
        }else{
            returnPromise.resolve();
        }
        return returnPromise;
    };

	self.saveDataSource = function(widget, showMessage) {
		getImportNowParentWidget().currentRow.isEdited = false;
		if (widget.isValid() && validAggregationFunctionWhenFact(widget)) {
			return checkForDataSourceDependentsJoin(widget).then(function(dependentDataSources){
				if (!dependentDataSources.length) {
					widget.currentRow.changed = true;
					widget.currentRow.columns   = widget.currentRow.columns || [];
					widget.currentRow.relations = widget.currentRow.columns || [];
					prepareJoinDataSourceRow(widget);
					return widget.saveRow().then(function(response) {
						if(response && response.dataset && response.dataset.message){
							return UtilitiesController.confirmMessageByCode("CHANGE_DATASOURCES_FROM_OTHER_CLIENTS",{i: getChangeDatasourcesMessageParameters(response.dataset.message)},{
								"type": 'CONFIRMATION',
								"success": function (){
									widget.currentRow.confirm = true;
									return widget.saveRow().then(function(response){
										widget.currentRow.confirm = false;
										return saveDataSourceResolve(widget,showMessage,response);
									},saveDataSourceReject);
								}
							});

						}else{
							return saveDataSourceResolve(widget,showMessage,response);
						}
					}, saveDataSourceReject);
				} else {
					MessageService.showMessageByCode('ERROR_MESSAGE_ON_DATASOURCE_CHANGE_TYPE');
				}
			});
		}
	};

	self.actionEventSave =  function () {
	    var widget = getDataSourceWidget();
        self.validateVersionMinSendEmail(widget).then(function () {
            self.saveDataSource(widget, true);
        });
	};

	function prepareJoinDataSourceRow(widget){
		var dataSourcesInformations = widget.currentRow.dataSourcesInformations;
		var dateFilterColumn = widget.currentRow.dateFilterColumn;
		for (var dtSrcInfo in dataSourcesInformations) {
			dataSourcesInformations[dtSrcInfo].dateFilterColumn = dateFilterColumn;
		}
	}

    self.importHistoryActionEventDelete = function(){
        var widget = getImportHistoryWidget();
        var idArray =getImportSchedulesIds(widget.getCheckedRows());
        DataSourceManagerService.deleteImportHistory(idArray).then(function(){
            self.setImportHistoryDataSource(widget);
            UtilitiesController.successNotification(widget.getCheckedRows().length + ' Row(s) deleted with sucess');
        });
    };

    var getImportSchedulesIds = function(checkedRows){
        var idArray = [];
        checkedRows.forEach( function(item){
           idArray.push(item._id);
        });
        return idArray;
    };

	self.deleteRows = function(widget) {
		var checkedRows = widget.getCheckedRows();
		DataSourceManagerService.checkIndicatorsWithDataSource(checkedRows).then(function(){
			UtilitiesController.confirmMessageByCode("DELETE_CONFIRMATION",{},{
                "type": 'CONFIRMATION',
				"success": function (){
					var existDependentDataSources = false;
					var verifyDependencies = [];

					checkedRows.forEach(function(row){
						if(row.joinDataSource){
							JoinDataSourceService.deleteJoinDataSourceDependencies(row);
						} else {
							verifyDependencies.push(getDependentJoinDataSources(row._id).then(function(dependentDataSources){
								if(dependentDataSources){
									existDependentDataSources = true;
								}
							}));
						}
						DataSourceManagerService.deleteRelatedImportSchedule(row);
					});

					ZHPromise.all(verifyDependencies).then(function() {
						if (existDependentDataSources) {
							MessageService.showMessageByCode('ERROR_MESSAGE_ON_DELETE_DATASOURCE');
						} else {
							widget.removeCheckedRows();

						}
					});
				}
			});
		}).catch(function(e){
			UtilitiesController.showMessage(e.error,"error");
		});
	};

	function checkForDataSourceDependentsJoin(widget) {
		var dependentDataSources = false;
		if (!widget.isNewRow() && widget.backupRow && !widget.backupRow.joinDataSource && widget.currentRow.joinDataSource) {
			return getDependentJoinDataSources(widget.currentRow._id);
		}
		return ZHPromise.all(dependentDataSources);
	}

	function getDependentJoinDataSources(dataSourceId){
		return JoinDataSourceService.getDependentJoinDataSources(dataSourceId).then(function(response) {
			var dataSourceDependencies = response.dataset.dependentDataSources.shift();
			return dataSourceDependencies ? dataSourceDependencies.dependentDataSources : false;
		});
	}

	function validAggregationFunctionWhenFact() {
		var ret = true;
		var columnsWidget = getColumnsWidget();
		var columnsCurrentRow = columnsWidget && columnsWidget.currentRow? columnsWidget.currentRow : {};
		var isFactColumn = columnsCurrentRow.isFact || columnsCurrentRow.type === 'fact';
		if (UtilitiesController.isColumnGenerated(columnsCurrentRow) && isFactColumn && !columnsCurrentRow.function) {
			ret = false;
		}
		if (!ret) {
			var errorMessage = ScreenService.i18n('Column {0} should have an aggregation function');
			errorMessage = errorMessage.replace("{0}", columnsCurrentRow.label);
			UtilitiesController.showMessage(errorMessage, 'E');
		}
		return ret;
	}

	self.setValuesAfterSave = function(dataSourceWidget, response) {
		if(response.dataset) {
			updateDataSourceOnQueryChange(dataSourceWidget, response);

			if (response.dataset.dataSource && response.dataset.dataSource.length > 0) {
				dataSourceWidget.currentRow._id = response.dataset.dataSource[0]._id;

				if(dataSourceWidget.currentRow.joinDataSource){
					JoinDataSourceService.updateImportDataSourceDependencies(dataSourceWidget);
				}
			}

			setKeyColumnsOnChangeDataSource(dataSourceWidget);
		}
		setFlagsOnSaveDataSource(dataSourceWidget);
		dataSourceWidget.view();
	};

	function updateDataSourceOnQueryChange(dataSourceWidget, response) {
		if (dataSourceWidget.currentRow._queryChanged || dataSourceWidget.currentRow._originDatasourcesChanged) {
			if (response.dataset.columns && response.dataset.columns.length > 0) {
				dataSourceWidget.currentRow.columns = response.dataset.columns[0].columns;
				self.setChildrenData(dataSourceWidget);

				if (response.dataset.relations && response.dataset.relations.length > 0) {
					dataSourceWidget.currentRow.relatedStructuresType = response.dataset.relations[0].relations;
					self.updateQuery(dataSourceWidget, response);
				}
			}
		}
	}

	self.updateQuery =function(dataSourceWidget,response){
		if (response.dataset.columns && response.dataset.columns.length > 0) {
			dataSourceWidget.currentRow.columns = response.dataset.columns[0].columns;
			self.setChildrenData(dataSourceWidget);
		}

		if (response.dataset.relations && response.dataset.relations.length > 0) {
			dataSourceWidget.currentRow.relatedStructuresType = response.dataset.relations[0].relations;
		}
	};

	function setKeyColumnsOnChangeDataSource() {
		var widget = templateManager.container.getWidget("importScheduling");
		if (widget.getField('keys').changed) {
			templateManager.container.getWidget('columns').dataSource.data = widget.currentRow.columns;
			delete widget.getField('keys').changed;
		}
	}

	function setFlagsOnSaveDataSource(dataSourceWidget){
		self.setDateFilterColumn(dataSourceWidget);
		dataSourceWidget.currentRow.changed = false;
		dataSourceWidget.currentRow.__is_new = false;
		dataSourceWidget.currentRow._queryChanged = false;
		dataSourceWidget.currentRow._originDatasourcesChanged = false;
		dataSourceWidget.currentRow.isScheduleSave = false;
	}

	self.onChangeQuery = function(row) {
		row._queryChanged = true;
	};

	self.onChangeOriginDataSource = function(row){
		row._originDatasourcesChanged = true;
	};

	self.setChildrenData = function(dataSourceWidget) {
		var parentCurrentRow = dataSourceWidget.currentRow;
 		var columnsWidget = getColumnsWidget();
		var dataVisualizationWidget = templateManager.container.getWidget('dataVisualization');
		var columnsDataSource = parentCurrentRow.columns.map(self.decodeColumnType);
		columnsWidget.dataSource.data = columnsDataSource;
		dataVisualizationWidget.dataSource.data = columnsDataSource;
        dataVisualizationDataSource = columnsDataSource;
 		self.setImportSchedulingData(dataSourceWidget);
	};

	self.setImportSchedulingData = function(dataSourceWidget) {
		var importScheduling = getImportSchedulingWidget();
		var persistTypeField = importScheduling.getField('persistType');
		var periodField = importScheduling.getField('period');
		var scheduleExpressionField = importScheduling.getField('scheduleExpression');
		var beforeImportScript = importScheduling.getField('beforeImportScript');
		var afterImportScript = importScheduling.getField('afterImportScript');
		var execAfterIfImportError = importScheduling.getField('execAfterIfImportError');
		var execAfterIfBeforeScriptError = importScheduling.getField('execAfterIfBeforeScriptError');
		var pageSizeField = importScheduling.getField('pageSize');
		var isActiveField = importScheduling.getField('isActive');

		persistTypeField.value(dataSourceWidget.currentRow.persistType);
		pageSizeField.value(dataSourceWidget.currentRow.pageSize || null);
		periodField.value(dataSourceWidget.currentRow.period || null);
		scheduleExpressionField.value(dataSourceWidget.currentRow.scheduleExpression || '');
		isActiveField.value(dataSourceWidget.currentRow.isScheduleActive || false);
		beforeImportScript.value(dataSourceWidget.currentRow.beforeImportScript || '');
		afterImportScript.value(dataSourceWidget.currentRow.afterImportScript || '');
		execAfterIfImportError.value(dataSourceWidget.currentRow.execAfterIfImportError || false);
		execAfterIfBeforeScriptError.value(dataSourceWidget.currentRow.execAfterIfBeforeScriptError || false);
	};

	self.decodeColumnType = function(column) {
		if(column.isFact) {
			column.type = "fact";
		} else if(column.isDimension) {
			column.type = "dimension";
		}
		return column;
	};

	self.setIsKeyColumns = function() {
		var dataSourceWidget = getDataSourceWidget();
		var field = dataSourceWidget.getField('keyColumns');
		var dateFilterDataSource = field.dataSource;
		var keyColumns = [];
		if (Util.isArray(dataSourceWidget.currentRow.columns)) {
			keyColumns = dataSourceWidget.currentRow.columns.map(function(column) {
				if (column.isKey) {
					return column.name;
				}
			});
		}
		dateFilterDataSource.data = dataSourceWidget.currentRow.columns;
		field.setValue(keyColumns);
	};

	self.setDateFilterColumn = function(widget) {
		var dataSourceWidget = getDataSourceWidget();
		var dataSourceColumns = dataSourceWidget.currentRow.columns;

		var dateColumns = [];
		if(Util.isArray(dataSourceColumns)) {
			dateColumns = dataSourceColumns.filter(function(column) {
				return column.isDate;
			});
		}

		var importSchedulingWidget = templateManager.container.getWidget("importScheduling");
		importSchedulingWidget.getField('dateFilterColumnLabel').dataSource.data = dateColumns;
		widget.currentRow.dateFilterColumnLabel = dataSourceWidget.currentRow.dateFilterColumnLabel;
		widget.currentRow.dateFilterColumn = dataSourceWidget.currentRow.dateFilterColumn;
	};

	self.columnsNewRow = function(columnsWidget) {
		columnsWidget.newRow();
		columnsWidget.currentRow.isGenerated = false;
		self.setExpressionOrFunction(columnsWidget);
	};

	function unselectColumnsAndErrorMessage(selectedRow, errorMessage) {
		ScreenService.alertNotification(errorMessage);
		selectedRow.__isSelected = false;
		UtilitiesController.showMessage(errorMessage);
	}

	self.validateSelectColumn = function(selectedRow){
		var dataSourceWidget = getDataSourceWidget();
		if (!dataSourceWidget.currentRow.joinDataSource && UtilitiesController.isColumnGenerated(selectedRow)) {
			unselectColumnsAndErrorMessage(selectedRow, "You can't select a generated column!");
		} else if (isColumnJoinKey(selectedRow)){
			unselectColumnsAndErrorMessage(selectedRow, "You can't select a join key column!");
		}
	};

	function clearCheckedRowsAfterSelectGeneratedColumn(columnsWidget, selectedRow) {
		if(UtilitiesController.isColumnGenerated(selectedRow)){
			removeOnlyGeneratedColumns(columnsWidget.dataSource.checkedRows);
			selectedRow.__isSelected = false;
		}
	}

	function removeOnlyGeneratedColumns(checkedRows){
		var generatedRow = checkedRows.find(function(row){
			return row.isGenerated == true;
		});
		UtilitiesController.removeItemArray(checkedRows,generatedRow);
	}

	self.validateSelectColumnToClearCheckedRows = function(columnsWidget, selectedRow){
		var dataSourceWidget = getDataSourceWidget();
		if (!dataSourceWidget.currentRow.joinDataSource && !!UtilitiesController.isColumnGenerated(selectedRow)) {
			clearCheckedRowsAfterSelectGeneratedColumn(columnsWidget, selectedRow);
		} else if (isColumnJoinKey(selectedRow)){
			clearCheckedRowsAfterSelectGeneratedColumn(columnsWidget, selectedRow);
		}
	};
	self.validateSelectAllColumns = function(columnsDataSource){
		var dataSourceWidget = getDataSourceWidget();

		columnsDataSource.data.forEach(function(row) {
			if ( (!dataSourceWidget.currentRow.joinDataSource && UtilitiesController.isColumnGenerated(row)) || isColumnJoinKey(row)) {
				row.__isSelected = false;
			}
		});
		ScreenService.alertNotification("Generated columns and join keys were not selected!");
	};
	self.validateSelectAllColumns = function(columnsDataSource){
		var dataSourceWidget = getDataSourceWidget();

		columnsDataSource.data.forEach(function(row) {
			if ( (!dataSourceWidget.currentRow.joinDataSource && UtilitiesController.isColumnGenerated(row)) || isColumnJoinKey(row)) {
				row.__isSelected = false;
			}
		});
		ScreenService.alertNotification("Generated columns and join keys were not selected!");
	};

	function isColumnJoinKey(row){
		var dataSourceWidget = getDataSourceWidget();
		var dataSourceInformations = dataSourceWidget.currentRow.dataSourcesInformations;

		for (var relations in dataSourceInformations) {
			var keysTranslations = dataSourceInformations[relations].keysTranslation;

			for (var key in keysTranslations) {
				if (row.name === key) {
					return true;
				}
			}
		}
	}

	self.blockFieldsEditionWhenKeyColumn = function(widget){
		var propertyFields = ['type', 'isKey'];

		var editableFields = [];
		var nonEditableFields = [];

		var column = widget.currentRow;
		var dataSource = widget.parent.currentRow;

		if (dataSource.joinDataSource) {
			nonEditableFields.push('name');

			if (column.isDimension && isColumnJoinKey(column)) {
				nonEditableFields.push(propertyFields);
			} else {
				editableFields.push(propertyFields);
			}
		}

		UtilitiesController.setFieldsReadOnly(widget, editableFields, false);
		UtilitiesController.setFieldsReadOnly(widget, nonEditableFields, true);
	};

	var datatypeDatasource = function(isFact){
		var data = [
			{
				"id": "A",
				"label": ScreenService.i18n("Automático")
			},
			{
				"id": "S",
				"label": ScreenService.i18n("String")
			},
			{
				"id": "N",
				"label": ScreenService.i18n("Número")
			}
		];
		if(!isFact){
			data.push(
				{
					"id": "D",
					"label": ScreenService.i18n("Data")
				}
			);
		}
		return data;
	};

	self.fillRelatedColumnData = function (columnsWidget) {
		var relatedColumnData = columnsWidget.dataSource.data.filter(function(obj){
			return obj.isFact;
		});
		columnsWidget.getField("datatype").dataSource.data = datatypeDatasource(columnsWidget.currentRow.isFact);
		columnsWidget.getField("relatedFactLabel").dataSource.data = relatedColumnData;
		columnsWidget.getField("secondRelatedFactLabel").dataSource.data = relatedColumnData;
		columnsWidget.getField("relatedFactMultiple").dataSource.data = Util.clone(relatedColumnData);
	};

	self.setExpressionOrFunction = function(columnsWidget) {
		UtilitiesController.clearChangedRows(columnsWidget);
		self.setFunctionVisibility(columnsWidget);

		setExpressionFieldParameters(columnsWidget);

		var fieldName = columnsWidget.getField("name");
		fieldName.readOnly = UtilitiesController.isColumnGenerated(columnsWidget.currentRow);
	};

	self.getFieldsVisible = function(columnsWidget, fields){
		fields.forEach(function(field){
			columnsWidget.getField(field).isVisible = true;
		});
	};

	self.getFieldsNotVisible = function(columnsWidget, fields){
		fields.forEach(function(field){
			columnsWidget.getField(field).isVisible = false;
		});
	};

	self.unsetFields = function(widget, fields){
		fields.forEach(function (field){
			delete widget.field;
		});
	};

	self.clearValuesColumns = function(columnsWidget, invisibleFields){
		invisibleFields.forEach(function(fieldName) {
		var field = columnsWidget.getField(fieldName);
			columnsWidget.currentRow[field.name] = "";
		});
	};

	self.setFunctionVisibility = function(columnsWidget) {
		var visibleFields = [];
		var invisibleFields = [];
		var fields = [];
		var column = columnsWidget.currentRow;
		var directionField = columnsWidget.getField('direction');
		var expressionField = getColumnsWidget().getField('expression');

		if(getColumnsWidget().parent.currentRow.hasDataMining && getColumnsWidget().currentRow.specialFunction == "case"){
			expressionField.placeholder = expressionField.placeholderCase;
		} else {
			expressionField.placeholder = expressionField.placeholderRegular;
		}

		if(!column.isFact && column.type === 'fact'){
			column.datatype = "A";
		}
		columnsWidget.getField("datatype").dataSource.data = datatypeDatasource(column.type === 'fact');

		if(column.type === 'fact') {
			if(UtilitiesController.isColumnGenerated(column)) {
				visibleFields = ["name", "label", "type", "mask", "direction", "function", "isFunctionAccumulative"];
				invisibleFields = ["isKey", "isFunctionAccumulative", "specialFunction", "relatedFactLabel", "expression", "validateExp"];
			} else {
				visibleFields = ["name", "label", "type", "mask", "direction", "function", "isFunctionAccumulative", "specialFunction", "relatedFactLabel", "expression", "validateExp"];
				invisibleFields = ["isKey"];
			}

			directionField.validations.required = {};
		} else {
			visibleFields = ["name", "label", "type", "direction", "isKey"];
			invisibleFields = ["function", "isFunctionAccumulative", "specialFunction", "relatedFactLabel", "expression", "validateExp", "mask"];

			if(directionField.validations.required) {
				delete directionField.validations.required;
			}
		}

		column.isFact = column.type === 'fact';
		column.isDimension = column.type === 'dimension';

		UtilitiesController.setFieldsVisibility(columnsWidget, visibleFields, true);
		UtilitiesController.setFieldsVisibility(columnsWidget, invisibleFields, false);
		self.clearValuesColumns(columnsWidget, invisibleFields);

		var isJoinDataSource = columnsWidget.parent.currentRow.joinDataSource;
		columnsWidget.getField("columnOriginDataSources").isVisible = isJoinDataSource;
		columnsWidget.getField("joinFunction").isVisible = (isJoinDataSource && column.isFact) ? true : false;

		setComplementFieldsVisibility(columnsWidget);

		fields.visibleFields = visibleFields;
		fields.invisibleFields = invisibleFields;

		return fields;

	};

	self.readExpression = function(widget) {
		var expression = '';
		var validExpressionMsg = "Expressão válida.";
		var invalidExpressionMsg = "Expressão inválida.";
		if(widget.parent.currentRow.hasDataMining) {
			try {
				if(widget.currentRow.specialFunction === 'case') {
					expression = readCaseExpression(widget);
				} else {
					expression = readRegularExpression(widget);
				}
				if(expression){
					UtilitiesController.successNotification(validExpressionMsg);
				}else if(widget.currentRow.expression){
					UtilitiesController.errorNotification(invalidExpressionMsg);
				}
				return expression;
			} catch (e) {
				UtilitiesController.showMessage(e, "error");
			}
		} else {
            try{
			    if(widget.currentRow.specialFunction === 'case') {
				    expression = readCaseExpressionWithoutDataMining(widget);
			    } else {
				    expression = readRegularExpression(widget);
			    }
                if(expression){
					UtilitiesController.successNotification(validExpressionMsg);
				}else if(widget.currentRow.expression){
					UtilitiesController.errorNotification(invalidExpressionMsg);
				}
                return expression;
            } catch (e) {
                UtilitiesController.showMessage(e, "error");
            }
		}
	};

	self.getExpression = function(widget) {
		return widget.currentRow.expression;
	};

	function readRegularExpression(widget) {
			var facts = widget.dataSource.data.filter(function(column){
				return column.isFact === true;
			}).map(function(column){
				return column.name;
			});

			var nonFacts = widget.dataSource.data.filter(function(column){
				return column.isFact === false;
			}).map(function(column){
				return column.name;
			});
			return ExpressionInterpreter.readExpression(widget.currentRow.expression, facts, nonFacts, widget.currentRow.name);
	}


	function readCaseExpression(widget) {
		var expression = widget.currentRow.expression;
		var requiredExpMsg = "O campo \"expressões\" é obrigatório para funções CASE.";
		var instructions = "Para expressões CASE, siga o modelo: CASE [condição] THEN [expressão] DEFAULT [expressão]\n\nEx: CASE $fatoRelacionado1 != 0 THEN (( $fatoRelacionado2 - $fatoRelacionado1 )*100 / $fatoRelacionado1 ) DEFAULT 100";

		if(expression) {
			var caseExp = formatCaseExp(expression);
			var regEx = new RegExp("(CASE.* [^ ].* [<>!=].* [^ ].* (THEN).* [^ ].*).* DEFAULT.* ([^ ].*)");
			var testExp = regEx.test(caseExp);

			if(testExp) {
				widget.currentRow.caseExp = caseExp;
				return expression;
			} else {
				UtilitiesController.showMessage(instructions);
			}
		} else {
			UtilitiesController.errorNotification(requiredExpMsg);
		}
		return null;
	}

	var readCaseExpressionWithoutDataMining = function (widget){
		var expression = widget.currentRow.expression;
		var requiredExpMsg = "O campo \"expressões\" é obrigatório para expressões CASE.";
		var caseSqlMsg = "Para criar uma expressão case sem mineração de dados, você deve consultar a sintaxe \"CASE\" de seu banco de dados.";

		if(widget.currentRow.specialFunction == 'case'){
			if(expression){
				if(checkExpressionString(expression.toUpperCase())){
					widget.currentRow.expression = expression;
				} else {
					UtilitiesController.showMessage(caseSqlMsg);
					expression = "";
				}
			} else {
				UtilitiesController.showMessage(requiredExpMsg);
				expression = "";
			}
		}
			return expression;
	};

	function checkExpressionString(expression){
		var arrayKeyWords = expression.split(" ");
		var arrayExpression = expression.split("");
		var checkExpression;
		var countChar = 0;
		if(arrayKeyWords[0] == "CASE"){
			for(var i = 0; i < arrayExpression.length; i++){
				if(arrayExpression[i] == '('){
					countChar++;
				} else if (arrayExpression[i] == ')'){
					countChar--;
					if(countChar < 0) {
						UtilitiesController.errorNotification("Não há \"(\" para fechar com \")\".");
						checkExpression = false;
					}
				}
			}
			if(countChar > 0){
				UtilitiesController.errorNotification("Existem mais \"(\" do que o necessário.");
				checkExpression = false;
			} else if (countChar < 0) {
				UtilitiesController.errorNotification("Existem mais \")\" do que o necessário.");
				checkExpression = false;
			} else if (countChar == 0 && checkExpression !== false) {
				checkExpression =  true;
				UtilitiesController.showMessage("CHECK_CASE_SYNTAX");
			}
		} else {
			checkExpression = false;
		}
		return checkExpression;
	}

	function formatCaseExp(expression) {
		var newExp = '';
		var words = expression.split(' ');

		words.forEach(function(word){
			if(word[0] !== '$'){
				word = word.toUpperCase();
			}

			word = word.replace('\n', ' ');
			newExp += word + ' ';
		});

		return newExp;
	}

	this.reloadColumnsData = function(columnsWidget){
		if (columnsWidget.mode === "list") {
			columnsWidget.dataSource.data = columnsWidget.parent.currentRow.columns;
			columnsWidget.reload();
		}
	};

	self.importColumns = function(widget) {
		DataSourceManagerService.syncColumns(widget.parent.currentRow).then(function(response) {
			UtilitiesController.showMessages(response.messages);

			var columns = response.dataset && response.dataset.columns ? response.dataset.columns : [];
			columns = columns.map(self.decodeColumnType);
			widget.dataSource.data = columns;
			widget.parent.currentRow.columns = columns;
			self.setDateFilterColumn(widget.parent);
		}, function(response){
			if(response.data){
				if(response.data.errorCode && response.data.errorCode === 1){
					MessageService.showMessageByCode('DATASOURCE_REPEATED_COLUMN_NAME',{"C": response.data.error.replace("Exception: ","")});
				}else{
					UtilitiesController.showMessage(response.data.error, 'E');
				}
			}
		});
	};

	self.removeCheckedColumns = function (widget) {
		if (widget && widget.dataSource) {
			var dataSource = widget.dataSource;
			var removedAny = false;
			dataSource.getCheckedRows().forEach(function (row) {
				var rowIndex =  dataSource.data.indexOf(row);
				if (~rowIndex) {
					dataSource.data.splice(rowIndex, 1);
					removedAny = true;
				}
			});

			if (removedAny) {
				self.removeCheckedRowsUsingSave(widget);
				widget.dataSource.checkedRows = [];
				widget.isRemoving = true;
				widget.list();
			}
		}
	};

	this.columnsWidgetCancel = function(widget){
		if(!widget.isNewRow()){
			widget.currentRow = widget.backupRow;
			widget.dataSource.data[widget.currentRow.__index] = widget.currentRow;
			widget.parent.currentRow.columns[widget.backupRow.__index] = widget.currentRow;
		}
		MasterDetailUtilities.masterDetailRollback(widget);
	};

	this.columnsWidgetOnSave = function(widget) {
		if(widget.isValid()){
			var errorOnWidget = validateSpecialFunctionRequiredFields(widget);
			widget.currentRow.isDate = widget.currentRow.datatype === 'D';
			if(errorOnWidget) {
				UtilitiesController.errorNotification(errorOnWidget);
			}  else if (widget.currentRow.isDimension){
				self.saveColumns(widget);
			} else {
				if(widget.parent.currentRow.hasDataMining) {
					if(widget.currentRow.expression !== ""){
						if(this.readExpression(widget) !== undefined || readRegularExpression(widget) !== undefined) {
							self.saveColumns(widget);
						} else {
							UtilitiesController.showMessage("COLUMN_CANNOT_SAVE");
						}
					} else {
						self.saveColumns(widget);
					}
				} else {
					if(readCaseExpressionWithoutDataMining(widget) !== undefined || readRegularExpression(widget) !== undefined) {
						self.saveColumns(widget);
					} else {
						UtilitiesController.showMessage("COLUMN_CANNOT_SAVE");
					}
				}
			}
		}
	};

	self.removeCheckedRowsUsingSave = function(widget) {
		self.saveDataSource(widget.parent,false).then(function() {
			delete widget.currentRow._altered;
				if(!!widget.isRemoving){
					widget.list();
					widget.currentRow = [];
				} else {
					widget.view();
				}

			self.updateIndicatorsOnChangeColumns(widget.parent.currentRow);
			widget.isRemoving = false;
		});
	};

	self.saveColumns = function(widget) {
		widget.parent.currentRow._columnsChanged = true;
		var fields = self.setFunctionVisibility(widget);

		widget.currentRow._altered = true;
		if(Array.isArray(widget.currentRow.color) &&
			widget.currentRow.color.length > 0 &&
			widget.currentRow.color[0]
		){
			widget.currentRow.color = [widget.currentRow.color[0]._originalInput];
		}
		if(!UtilitiesController.isColumnGenerated(widget.currentRow) && widget.currentRow.expression) {
			var expression = this.getExpression(widget);
			var beginningExpression = expression.substring(0, 4).toUpperCase();
			if(beginningExpression !== "CASE"){
                expression = readRegularExpression(widget);
			}
			widget.currentRow.functions = expression.functions;
			widget.currentRow.mathExpressions = expression.mathExpressions;
		}

		widget.parent.currentRow.columns = widget.dataSource.data.map(self.cleanColumnRow);

		widget.parent.currentRow.keys = widget.dataSource.data.filter(function(column) {
			return column.isKey;
		}).map(function(currentValue){
			return currentValue.name;
		});

		self.unsetFields(widget, fields.invisibleFields);

		self.saveDataSource(widget.parent,false).then(function() {
			delete widget.currentRow._altered;
				if(!!widget.isRemoving){
					widget.list();
					if(!!widget.currentRow.__is_new){
						widget.currentRow = [];
						ScreenService.closeSwipe();
					}
				} else {
					widget.view();
				}

			self.updateIndicatorsOnChangeColumns(widget.parent.currentRow);
			widget.isRemoving = false;
		});
	};

	function updateWidgetOnChangeColumns(indicator,newDataSourceColumns){
		WidgetService.findViewsByIndicator(indicator).then(function(widgets){
			widgets.forEach(function(widget){
				updateWidgetsColumns(widget,newDataSourceColumns);
			});

			DataSourceManagerService.persistWidgetsColumns(widgets);
		});
	}

	self.updateIndicatorsOnChangeColumns = function(row) {
		var relatedIndicators;
		var newDataSourceColumns = row.columns;

		DataSourceManagerService.getIndicatorsRelatedToDataSource(row._id).then(function(response){
			relatedIndicators = response.dataset.indicator;

			relatedIndicators.forEach(function(indicator){
				indicator.dataSourceColumns = newDataSourceColumns;
				updateWidgetOnChangeColumns(indicator._id,newDataSourceColumns);
			});
			DataSourceManagerService.persistIndicatorsColumns(relatedIndicators);
		});
	};

	self.cleanColumnRow = function (columnRow) {
		var cleanedRow = Util.clone(columnRow);
		cleanedRow = UtilitiesController.removeFrameworkProperties(cleanedRow);
		cleanedRow.isSortable  = !!cleanedRow.direction;
		cleanedRow.isFact      = (cleanedRow.type === 'fact');
		cleanedRow.isDimension = (cleanedRow.type === 'dimension');
		return cleanedRow;
	};

	self.openAndSetRelationsData = function(container) {
		ScreenService.openSwipe(container.getWidget('relations_tab')).then(function(){
			self.setRelationsData(container);
		});
	};

	self.setRelationsData = function(container) {
		var relationsData = container.getWidget('dataSource').currentRow.relatedStructuresType;
		var columnsDataSource = container.getWidget('columns').dataSource;
		var structuresData = container.getWidget('relateStructures').currentRow;
		var relationsWidget = container.getWidget('relations');
		var relations = [];

		if (relationsData && relationsData.length > 0) {
			var structureRelations = self.getStructureRelation(relationsData, structuresData._id);
			if(structureRelations) {
				relations = (structureRelations.relations || []);
			}
		}

		relationsWidget.currentRow.structureType = structuresData._id;
		relationsWidget.parentsDataSource.data = self.fieldKeysToObject(structuresData.fieldKey);
		relationsWidget.childsDataSource.data = columnsDataSource.data;
		relationsWidget.relationsDataSource.data = $.map(relations, function(value, index){return [value];});
		if(relationsWidget.relationsDataSource.data.length === 0) {
			try{
				relationsWidget.editActionClick();
			} catch(e){}
		}
	};

	self.cleanAllRelations = function(widget){
		var currentRowIndex = widget.parent.currentRow.__index;
		UtilitiesController.confirmMessageByCode("RESET_ALL_RELATIONS",{},{
            "type": 'CONFIRMATION',
            "success": function (){
                DataSourceManagerService.cleanAllRelations(widget.parent.currentRow._id).then(function(){
					widget.parent.reload([], false).then(function(){
						widget.parent.currentRow = widget.parent.dataSource.data[currentRowIndex];
						if(!widget.parent.currentRow.__index){
							widget.parent.currentRow.__index = currentRowIndex;
						}
						widget.reload();
					});
				});
            }
        });
	};

	self.cleanRelations = function(relationsWidget) {
		UtilitiesController.confirmMessageByCode("RESET_RELATION",{},{
            "type": 'CONFIRMATION',
            "success": function (){
				relationsWidget.relationsDataSource.data = [];
				self.saveRelations(relationsWidget);
            }
        });
	};

	self.closeRelations = function(relationsWidget) {
		ScreenService.closeSwipe();
		relationsWidget.parent.parent.activate();
	};

	self.fieldKeysToObject = function(fieldKeys) {
		return fieldKeys.map(function(element) {
			return {
				"name": element
			};
		});
	};

	self.getStructureRelation = function(relations, structureId) {
		return relations.filter(function(element) {
			return element.structureType === structureId;
		}).shift();
	};

	self.saveRelations = function(relationsWidget) {
		var dataSourceWidget = relationsWidget.container.getWidget('dataSource');
		var dataSourceCurrentRow = dataSourceWidget.currentRow;

		if (!dataSourceCurrentRow.relatedStructuresType) {
			dataSourceCurrentRow.relatedStructuresType = [];
		}

		var cleanedRelations = relationsWidget.relationsDataSource.data.map(function(row) {
			return UtilitiesController.removeFrameworkProperties(row);
		});

		dataSourceCurrentRow.relatedStructuresType = dataSourceCurrentRow.relatedStructuresType.map(function(relationObj){
			if(relationObj.structureType === relationsWidget.currentRow.structureType) {
				relationObj.relations = cleanedRelations;
			}
			return relationObj;
		});

		var structureRelations = self.getStructureRelation(dataSourceCurrentRow.relatedStructuresType, relationsWidget.currentRow.structureType);

		if(!structureRelations) {
			dataSourceCurrentRow.relatedStructuresType.push({
				"structureType": relationsWidget.currentRow.structureType,
				"relations": cleanedRelations
			});
		}

		self.saveDataSource(dataSourceWidget, false).then(function() {
			relationsWidget.list();
		}).catch(UtilitiesController.showMessageErrorOfBackEnd);
	};

	self.setImportHistoryDataSource = function () {
		if (!filterWidget.currentRow.period) {
			self.setImportHistoryDateFilter();
		}

		self.filterDataSourceImportHistory(filterWidget);
	};

	self.openImportFilterWidget = function () {
		self.factoryFilterWidget();
		ScreenService.openPopup(filterWidget);
	};

	self.factoryFilterWidget = function () {
		if (!filterWidget) {
			var importHistoryWidget = getImportHistoryWidget();
			filterWidget = DataSourceManagerFactory.factoryFilterWidget(importHistoryWidget.container);
			importHistoryWidget.widgets.push(filterWidget);
		}
	};

	self.setFilterWidgetLastActiveWidget = function (filterWidget) {
		var importHistoryWidget = getImportHistoryWidget();
		filterWidget.lastActiveWidget = importHistoryWidget;
	};

	self.setImportHistoryDateFilter = function () {
		var importPeriodField = filterWidget.getField('period');
		var periodValue = importPeriodField.value() || {};

		periodValue.start = moment().subtract(3, 'days').startOf('day').format("DD/MM/YYYY");
		periodValue.end = moment().endOf('day').format("DD/MM/YYYY");
		if (!importPeriodField.value()) {
			importPeriodField.value(periodValue);
		}
	};

	self.filterDataSourceImportHistory = function (filterWidget) {
		var dataSourceWidget = getDataSourceWidget();
		if (dataSourceWidget.currentRow._id && filterWidget.currentRow.period) {
			var dataSourceId = dataSourceWidget.currentRow._id;
			var startDate = filterWidget.currentRow.period.start;
			var endDate = filterWidget.currentRow.period.end;
			DataSourceManagerService.findByDataSourceIdAndPeriod(dataSourceId, startDate, endDate).then(function (importHistory) {
				var importHistoryWidget = getImportHistoryWidget();
				importHistoryWidget.dataSource.data = importHistory;
			});
		}
	};

	self.showLastLog = function () {
		var importHistoryWidget = getImportHistoryWidget();
		if (!logWidget) {
			logWidget = DataSourceManagerFactory.factoryLogWidget(importHistoryWidget.container);
		}
		logWidget.isVisible = true;
		var logs = importHistoryWidget.currentRow.logs;
		logWidget.newRow();
		logWidget.currentRow.logField = logs.length > 0? formatLogMessages(logs) : '';
		ScreenService.openPopup(logWidget);
	};

	this.formatLogMessages = formatLogMessages;

	function formatLogMessages(logs) {
		var type = ScreenService.i18n('Type');
		var message = ScreenService.i18n('Message');

		return logs.reduce(function (prev, current, curIndex) {
			if (curIndex === 1) {
				return formatLogMessage(type, message, prev) + '\n\n' + formatLogMessage(type, message, current);
			} else {
				return prev + '\n\n' + formatLogMessage(type, message, current);
			}
		});
	}

	function formatLogMessage(type, message, log) {
		return type + ': ' + log.type + '\n' + message + ': ' + log.message;
	}

	self.closeImportLog = function (logWidget) {
		logWidget.isVisible = false;
		getImportHistoryWidget().activate();
		ScreenService.closePopup(logWidget);
	};

	self.setStatusColumnValue = function(row, element) {
		element.children('span').remove();
		element.children('div.circle-status').remove();
		element.children('div.progress').remove();
		element.children('div.progress-bar').remove();

		var widget = templateManager.container.getActiveWidget().name;
		var status = widget === 'dataSource' ? row.importStatus : row.status;
		var subprocesses = Util.isUndefinedOrNull(row.importSubprocesses)? row.subprocesses : row.importSubprocesses;
		var hasError = Util.isUndefinedOrNull(row.importHasError)? row.hasError : row.importHasError;
		var totalImported = Util.isUndefinedOrNull(row.totalImported)? 0 : row.totalImported;

		if (hasError) {
			element.append("<div class='circle-status error'></div>");
		} else if (status === 's') {
			element.append("<span>Saving documents...</span>");
		} else if (status === 100) {
			element.append("<div class='circle-status success'></div>");
		} else if (status === 'c') {
			element.append("<div class='circle-status cancel'></div>");
		} else if ((status > 0 && status < 100) || (status >= 0 && status < 100 && subprocesses > 0)) {
			element.append("<div class='progress progress-striped import-progress active'> <div class='progress-bar import-progress-bar' role='progressbar' aria-valuenow='" + status + "' aria-valuemin='0' aria-valuemax='100' style='width: " + status + "%" + "'> </div> </div>");
		} else if(totalImported > 0) {
			element.append("<span>In process (" + totalImported +" linhas)...</span>");
		} else if(status === 0 && subprocesses === 0) {
			element.append("<span>Starting Processes...</span>");
		}
	};

	self.setDataSourceLastImportInfo = function (dataSourceWidget) {
		var dataSource = dataSourceWidget.dataSource.data.map(function (item) {
			return item._id;
		});
		DataSourceManagerService.getLastImportHistoriesFromDataSourceIds(dataSource).then(setDataSourceLastImportInfo(dataSourceWidget));
	};

	self.uncheckColumns = function (columnsWidget) {
		if (columnsWidget && columnsWidget.dataSource) {
			columnsWidget.dataSource.removeCheckedRows(columnsWidget.dataSource.getCheckedRows());
		}
	};

	self.setImportSchedulingView = function() {
		getImportSchedulingWidget().view();
	};

	function setDataSourceLastImportInfo(dataSourceWidget) {
		var dataSource = dataSourceWidget.dataSource.data;
		return function(importHistory) {
			dataSource.forEach(function (data) {
				importHistory.forEach(function (importData) {
					if (importData.dataSourceId === data._id) {
						data.status = importData.importStatus || importData.status;
						data.hasErrror = importData.importHasError || importData.hasError;
					}
				});
			});
			templateManager.updateTemplate();
		};
	}

	function getDataSourceWidget() {
		return templateManager.container.getWidget('dataSource');
	}

	function getImportHistoryWidget() {
		return templateManager.container.getWidget('importHistory');
	}

	function getColumnsWidget() {
		return templateManager.container.getWidget("columns");
	}

	function getImportSchedulingWidget() {
		return templateManager.container.getWidget("importScheduling");
	}

	self.formatDatasource = function(array){
		array.forEach(function(row){
			for(var field in row){
				if(row[field] && typeof row[field] === "object"){
					row[field] = row[field].date;
				}
			}
		});
	};

	self.buildGridDataVisualization = function(widget) {
		widget.fields = [];
		var fields = dataVisualizationDataSource;
		for(var i=0; i<fields.length; i++){
			var field = {
				name: fields[i].name,
				label: fields[i].label,
				isVisible: true,
				showOnList: true
			};

            if(fields[i].isDate){
                field.displayFormat = "DD/MM/YYYY";
                field.dateFormat = "DD/MM/YYYY";
                field["datetime-preset"] = "date";
                field.template = "field/calendar.html";
            }

			metaDataFactory.factoryField(field, widget, true);
		}
	};

	this.prepareImportHistoryRow = function(importHistory){
       if(importHistory.elapsedTime){
        	importHistory.elapsedTime = convertNumberToTimeFormat(importHistory.elapsedTime);
        }

        if(importHistory.importStatus === 'canceled') {
        	importHistory.status = 'c';
        } else if(importHistory.importStatus === 'saving') {
        	importHistory.status = 's';
        } else if(importHistory.subprocesses && importHistory.imported && importHistory.subprocesses > 0){
        	importHistory.status = Math.round(100 * importHistory.imported / importHistory.subprocesses, 0);
        } else if (importHistory.importStatus === 'done' && importHistory.endDate && !Util.isEmptyObject(importHistory.endDate)){
        	importHistory.status = 100;
        } else {
        	importHistory.status = 0;
        }
	};

	function convertNumberToTimeFormat(number) {
		var sec_num = parseInt(number, 10);
		var hours = Math.floor(sec_num / 3600);
		var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
		var seconds = sec_num - (hours * 3600) - (minutes * 60);

		hours = hours < 10 ? "0" + hours : hours;
		minutes = minutes < 10 ? "0" + minutes : minutes;
		seconds = seconds < 10 ? "0" + seconds : seconds;

		return hours + ':' + minutes + ':' + seconds;
	}

	this.buildNotificationError = function(creatorInformations, dataSourceName, message){
		var row = {};
		row.name = creatorInformations.NAME;
		row.email = creatorInformations.EMAIL;
		row.dataSourceName = dataSourceName;
		row.message = message;
		DataSourceManagerService.sendNotificationError(row);
	};

	function updateWidgetsColumns(widget, newDataSourceColumns){
    	if (typeof widget.columns !== 'undefined'){
    		for(var i = 0; i < newDataSourceColumns.length; i++){
    			if (typeof widget.columns[i] !== 'undefined'){
        			widget.columns[i] = newDataSourceColumns[i];
    			}else
    				widget.columns.push(newDataSourceColumns[i]);
    		}
    		removeColumnsSettings(widget,newDataSourceColumns);
    	}
	}

	function removeColumnsSettings(widget,newDataSourceColumns){
		if (widget.columns.length > newDataSourceColumns.length){
	    	for(var i = 0; i < widget.columns.length; i++){
	    		if (typeof newDataSourceColumns[i] === "undefined"){
	    			widget.columns.splice(i, 1);
	    		}
			}
		}
	}

	self.setVisibilityApi = function () {
		var apiWidget = getApiWidget();
		var widget = getDataSourceWidget();
		if(widget.currentRow.hasDataMining  &&
        !widget.currentRow.joinDataSource &&
        widget.currentRow.finality !== CODE_FINALITY_SEND_MAIL){
			apiWidget.isVisible = true;
		}else{
			apiWidget.isVisible = false;
		}
	};

	var setVisibleEditFromDataVisualization = function () {
		getActionEditFromDataVisualization().isVisible = getLocalVar('clienteNRORG') === '5';
	};

	var setVisibleTabSendEmail = function (widget) {
	    widget.widgets.forEach(function (subWidgets) {
            if(subWidgets.isVisibleSendEmail &&
                widget.currentRow.finality === CODE_FINALITY_SEND_MAIL){
                subWidgets.isVisible = subWidgets.isVisibleSendEmail === 'Y';
            }
        });

    };

	function setVisibleIsTeknisa(widget) {
		getFieldIsTeknisaFormDataSource().isVisible = !!getLocalVar('isTeknisa');
		var arrayIsVisible = [
			getFieldQueryFormDataSource(),
			getActionEditFromDataSource(),
			getActionSaveFromDataSource(),
			getActionSaveFromColumns(),
			getActionEditFromColumns(),
			getActionImportFromColumns(),
			getActionDeleteFromColumns(),
			getActionAddFromColumns(),
			getActionCleanRelationsAllFromRelations(),
			getActionEditFromImportScheduling(),
			getActionSaveFromImportScheduling(),
		];
		var fieldQuery = getFieldQueryFormDataSource();
		fieldQuery.readOnly = false;
		arrayIsVisible.forEach(function (element) {
			element.isVisible = true;
		});
		if (!getLocalVar('isTeknisa') && widget.currentRow.IS_TEKNISA === 'Y') {
			fieldQuery.readOnly = true;
			arrayIsVisible.forEach(function (element) {
				element.isVisible = false;
			});
		}
	}

	this.dataSourceManagerAfterMoveRow = function(){
        var widget = getDataSourceWidget();
		setVisibleIsTeknisa(widget);
		widget.widgets.forEach(function (subWidgets) {
            subWidgets.isVisible = true;

        });
        UtilitiesController.clearChangedRows(widget);
		MasterDetailUtilities.afterMoveRow(widget, 1);
        widget.activate();
    	JoinDataSourceService.configWidgetWhenJoinDataSource(widget.currentRow.joinDataSource, 'dataSource');
        setVisibleTabSendEmail(widget);
		self.setVisibilityApi();
        var dataSourceId = widget.currentRow._id;
        setLocalVar('DATA_SOURCE_ID',dataSourceId);
        if(!widget.currentRow.__is_new) {
			self.setChildrenData(widget);
			self.setImportSchedulingView();
			notifyInactiveDataSource(widget);
        }
        setFieldsExcel(getWidgetDataSource());
        setFieldsFinality(getWidgetDataSource());
		setVisibleEditFromDataVisualization();
	};

	function notifyInactiveDataSource(widget) {
		var inactiveDataSourceMsg = "This datasource is set to use datamining but the import schedule is not active";
		if(widget.currentRow.hasDataMining && !widget.currentRow.isScheduleActive){
			ScreenService.alertNotification(inactiveDataSourceMsg);
		}
	}

    this.dataSourceManagerBeforeMoveRow = function(){
        var widget = getDataSourceWidget();
        UtilitiesController.deleteChangedFlag(widget);
	};

	this.specialFunctionFieldOnChange = function(widget) {
		setComplementFieldsVisibility(widget);

		if(widget.currentRow.specialFunction !== 'mod') {
			widget.currentRow.secondRelatedFactLabel = null;
		}
		setExpressionFieldParameters(widget);

	};

	var setExpressionFieldParameters = function(widget){
		var expressionField = widget.getField('expression');
		if(widget.currentRow.specialFunction !== 'case') {
			widget.currentRow.caseExp = null;
			expressionField.validations = null;
			expressionField.placeholder = expressionField.placeholderRegular;
		} else  {
			expressionField.validations = {"required":{}};
			expressionField.placeholder = expressionField.placeholderCase;
		}
	};

	var getPeriodFieldOnImportHistoryWidget = function() {
    	return filterWidget.getField('period');
	};

	this.importHistoryActionEventImportFilter = function () {
    	this.openImportFilterWidget();
    	PeriodsUtilities.setIntervalsOnPeriodField(getPeriodFieldOnImportHistoryWidget());
	};

	function setComplementFieldsVisibility(widget) {
		widget.getField('relatedFactLabel').isVisible = widget.currentRow.specialFunction != 'case';
		widget.getField('relatedFactMultiple').isVisible = widget.currentRow.specialFunction === 'case';
		widget.getField('secondRelatedFactLabel').isVisible = widget.currentRow.specialFunction === 'mod';
		widget.getField('modSecondValue').isVisible = widget.currentRow.specialFunction === 'mod';
		widget.getField('potentialExpoent').isVisible = widget.currentRow.specialFunction === 'power';
	}

	function validateSpecialFunctionRequiredFields(widget) {
		var specialFunction = widget.currentRow.specialFunction;

		if(specialFunction) {
			var functionsThatRequireRelatedFact = ['abs', 'sign', 'ceil', 'floor', 'sqrt', 'mod', 'power'];

			if(functionsThatRequireRelatedFact.indexOf(specialFunction) > -1 && !widget.currentRow.relatedFact) {
				return ScreenService.i18n('Por favor, preencha o fato relacionado para a função ') + specialFunction;
			}

			if(specialFunction === 'mod' && !widget.currentRow.secondRelatedFactLabel && !widget.currentRow.modSecondValue) {
				return ScreenService.i18n('Por favor, preencha o segundo fato relacionado para a função MOD');
			}

			if(specialFunction === 'power' && !widget.currentRow.potentialExpoent) {
				return ScreenService.i18n("Por favor, preencha o expoente");
			}
		}
	}

	this.dataVisualizationActionEventEdit = function () {
		var widget = getWidgetDataEdit();
		self.buildGridDataEdit(widget);
		ScreenService.openSwipe(widget);
	};

	self.buildGridDataEdit = function (widget) {
		widget.fields = [];
		var fieldNames = [];
		var fields = dataVisualizationDataSource;
		for (var i = 0; i < fields.length; i++) {
			if (fields[i].isGenerated) {
				fieldNames.push(fields[i].name);
				var field = {
					name: fields[i].name,
					label: fields[i].label,
					isVisible: true,
					class: 6,
					template: "field/text-edit.html",
					showOnForm: true
				};

				if (fields[i].isDate) {
					field.mask = {
						"type": "number"
					};
				}

				metaDataFactory.factoryField(field, widget, true);
				var fieldType = metaDataSelectNativeEditData();
				fieldType.name = fields[i].name + '___TYPE';
				if (fields[i].isDate) {
					fieldType.dataSource = metaDataSelectNativeDataSoure();
					fieldType.defaultValue = '+D';
				}
				metaDataFactory.factoryField(fieldType, widget, true);
			}
		}
		widget.dataSource.data = [];
		widget.newRow();
		widget.currentRow.__META_DATA = {};
		widget.currentRow.__META_DATA.FIELD_NAMES = fieldNames;
		widget.currentRow.__META_DATA.DATA_SOURCE_ID = getLocalVar('DATA_SOURCE_ID');
	};

	self.dataEditActionEventSave = function() {
		var widget = getWidgetDataEdit();
		widget.saveRow().then(function () {
			ScreenService.closeSwipe();
			getWidgetDataVisualization().reload();
		});
	};

	self.dataEditActionEventBack = function () {
		var widget = getWidgetDataEdit();
		widget.list();
		ScreenService.closeSwipe();
	};

	var metaDataSelectNativeDataSoure = function () {
		return {
			"data": [
				{
					"name": "+ Dias",
					"value": "+D"
				},
				{
					"name": "- Dias",
					"value": "-D"
				},
				{
					"name": "+ Meses",
					"value": "+M"
				},
				{
					"name": "- Meses",
					"value": "-M"
				},
				{
					"name": "+ Anos",
					"value": "+A"
				},
				{
					"name": "- Anos",
					"value": "-A"
				}
			]
		};
	};

	var metaDataSelectNativeEditData = function () {
		return {
			"name": "type",
			"label": "Padrão incremental",
			"class": 6,
			"isVisible": true,
			"showOnForm": true,
			"showOnView": true,
			"showOnList": true,
			"template": "field/select-native.html",
			"defaultOption": 0,
			"defaultValue": "NUMERIC",
			"descriptionField": "name",
			"valueField": "value",
			"dataSource": {
				"data": [
					{
						"name": "Numérico",
						"value": "NUMERIC"
					},
					{
						"name": "Alfabética",
						"value": "ALPHABETIC"
					}
				]
			}
		};
	};

	this.setScheduleStatus = function() {
		var widget = getDataSourceWidget();
		if(!widget.currentRow.hasDataMining) {
			widget.currentRow.isScheduleActive = false;
			widget.currentRow.isScheduleSave = true;
		}
	};

    var getDimensions = function(){
        getColumnsWidget().dataSource.data.filter(function(item){return item.isDimension;});
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('DataSourceManagerController', DataSourceManagerController);
});


// FILE: assets/js/controllers/DefaultRelatedStructuresController.js
function DefaultRelatedStructuresController(ScreenService, DefaultRelatedStructuresService, UtilitiesController) {

	var self = this;
	var currentRow;

	self.openAndSetRelationsData = function(relateStructuresWidget) {
		ScreenService.openSwipe(relateStructuresWidget.container.getWidget('relations_tab'));
		self.setRelationsData(relateStructuresWidget);
	};

	self.setRelationsData = function(relateStructuresWidget) {
		var structuresData = relateStructuresWidget.currentRow;
		var relationsWidget = relateStructuresWidget.container.getWidget('relations');
		relationsWidget.parentsDataSource.data = self.fieldKeysToObject(structuresData.fieldKey);
		DefaultRelatedStructuresService.findByStructureTypeId(relateStructuresWidget.currentRow._id).then(function(response) {
			if(response.dataset.defaultRelations.length !== 0) {
				currentRow = response.dataset.defaultRelations[0];
			}
			else {
				currentRow = {
					"structureType": structuresData._id,
					"relations": []
				};
			}
			relationsWidget.relationsDataSource.data = currentRow.relations;
		});
	};

	self.saveRelations = function(relationsWidget) {
		currentRow.relations = relationsWidget.relationsDataSource.data;
		DefaultRelatedStructuresService.saveRow(currentRow).then(function(response){
			if(response.notifications.length > 0) {
				var notification = response.notifications[0];
				UtilitiesController.notificationMessage(notification.message, notification.type);
			}
			currentRow._id  = response.dataset.defaultRelations[0]._id;
		});
	};

	self.fieldKeysToObject = function(fieldKeys) {
		return fieldKeys.map(function(element) {
			return {
				"name": element
			};
		});
	};

	self.closeRelations = function(relationsWidget) {
		ScreenService.closeSwipe();
		relationsWidget.parent.parent.activate();
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('DefaultRelatedStructuresController', DefaultRelatedStructuresController);
});

// FILE: assets/js/controllers/DrillDownController.js
function DrillDownController(localStorageEngine, ScreenService, templateManager, ChartViewController, GenericDashboardService,
ViewService, UtilitiesController, $location, $window, StructureService, GenericDashboardController, GenericDashboardFactory, $rootScope) {
	var fieldOnClickCrosstab = null;
	function setLastActiveDashboard(view) {
		var containerName = view.container.name;
		if (containerName == 'mainDashboard') {
			var tabbedWidget = templateManager.container.getWidget('dashboard_tabbed');
			if (tabbedWidget && tabbedWidget.template !== GenericDashboardFactory.ACCORDION_TEMPLATE) {
				GenericDashboardService.setLastActiveDashboard(tabbedWidget.currentWidget.name);
			}
		}
	}
	this.fieldFieldOnClickCrosstab = function(args){
		fieldOnClickCrosstab = args.owner.field;
	};

	var setStructureNext = function(structureId, structureLabel, indicatorNames, name, search) {
		if (structureId && structureLabel) {
			var drillStructureId = indicatorNames[name][ViewService.DRILL_STRUCTURE_ID];
			var drillStructureName = indicatorNames[name][ViewService.DRILL_STRUCTURE_NAME];
			if (!drillStructureId) {
				drillStructureId = [];
				drillStructureName = [];
			}
			drillStructureId[search[ViewService.VIEW_LEVEL]] = structureId;
			drillStructureName[search[ViewService.VIEW_LEVEL]] = structureLabel;

			indicatorNames[name][ViewService.DRILL_STRUCTURE_ID] = drillStructureId ;
			indicatorNames[name][ViewService.DRILL_STRUCTURE_NAME] = drillStructureName;
		}
	};

	this.openNextIndicatorView = function(view, row, onClick) {
		if(!onClick && view.actionOnClick){
			onClick = view.actionOnClick;
		}
		if(view.chartType == "gauge"){
			var data = view.dataSource.data[0];
			row[view.chartXAxisField] = data[view.chartXAxisField];
		}
		var structureId = templateManager.container.getWidget('filterWidget').currentRow.structure;
		var structureLabel = templateManager.container.getWidget('filterWidget').currentRow.structure_label;
		if(fieldOnClickCrosstab && view.isCrossTab){
			if(view.parameters.dimensionY.indexOf(fieldOnClickCrosstab.name) == -1){
				row[view.parameters.dimensionX] = fieldOnClickCrosstab.label;
			}
			fieldOnClickCrosstab = null;
		}

		if(row.__index){
			delete row.__index;
		}
		setLastActiveDashboard(view);
		if(onClick == "drillDownPre"){
			localStorageEngine.setLocalVar('previousConditions', view.parameters.previousConditions);
		}else if(onClick == "drillDownPos"){
			localStorageEngine.setLocalVar('postConditions', view.parameters.postConditions);
		}
		if (view.parameters && view.hasDrillDown !== false) {
			var period =  "";
			var search;
			if (getLocalVar('indicatorNames')[getLocalVar('indicatorName')] === undefined){
				search = $location.$$search;
			}
			else{
				search = getLocalVar('indicatorNames')[getLocalVar('indicatorName')];
			}
			var chartXAxisField = view.chartXAxisField;
			search[ViewService.VIEW_LEVEL] = isNaN(+search[ViewService.VIEW_LEVEL])? 0 : + search[ViewService.VIEW_LEVEL] + 1;
			
			search.fA = view.getParent().fullAccessProfiles || UtilitiesController.getUrlParamAtIndex("fA");
			if (view.chartType !== "grid" && !view.isCrossTab && view.chartType !== 'periodsList' && d3 && d3.event) {
				str = $(d3.event.target).attr('field-name') ? $(d3.event.target).attr('field-name') : "";
				if (!str) {
					{
						str = !!view.currentRow[chartXAxisField] ? view.currentRow[chartXAxisField] : "";
					}
				}
				period = str.indexOf('/') > -1 ? str : "";
				var color = $(d3.event.target).attr('fill');
				search.serieColors = [color];
			}
			var filterPeriod = templateManager.container.getWidget('filterWidget').currentRow.period;
			if(!period && filterPeriod && filterPeriod.start && filterPeriod.end){
				period = filterPeriod.start + "_" + filterPeriod.end;
			}
			setBaseDrillDownParameters(search, view, row, period);
			var isStructureDrillDown = UtilitiesController.getUrlParamAtIndex(ViewService.IS_STRUCTURE_DRILL, search[ViewService.VIEW_LEVEL]);
			setStructureDrillDownParameters(search, view);

			var dashboardFilters = templateManager.container.getWidget('filterWidget').dashboardFilters;
			if(dashboardFilters){
				search[ViewService.FILTERS] = dashboardFilters;
			}

			var filterRow = templateManager.container.getWidget('filterWidget').currentRow;

			if (isStructureDrillDown && view.drillDownType !== 'widget') {
				var clickedRow = view.dataSource.data[row.__index];
				var structureName = clickedRow[view[view.dimensionProperty]];
				StructureService.getStructDrillInfoByName(structureName, view.parameters.indicatorId).then(function(){
					openNextIndicatorViewWhenIsStructureDrillDown(search);
				});
			} else {
				setParametersWhenIsNotStructureDrillDown(search, view).then(function(){
					if (view.drillDownType === 'widget' || view.drillDownType === undefined) {
						$rootScope.$evalAsync(
							function($rootScope) {
								var name = setDrillDownName($location.search(search).$$search);
								var indicatorNames = getLocalVar('indicatorNames');
								indicatorNames[name]=$location.search(search).$$search;
								setStructureNext(structureId, structureLabel, indicatorNames, name, search);
								localStorageEngine.setLocalVar('indicatorNames', indicatorNames);
								var windowName = templateManager.container.preview ? "viewPreview" : ViewService.INDICATOR_VIEW_WINDOW;
								$location.path('/product#' + windowName).search('Name=' + name).hash('');
						   }
						);
					} else if (view.drillDownType === 'view') {
						return ViewService.findGeneratedMenu(view._id).then(function(response) {
							var menu = response.dataset.view[0];
							localStorageEngine.setLocalVar(ViewService.MENU_ID, menu._id);
							var name = setDrillDownName($location.search(search).$$search);
							var indicatorNames = getLocalVar('indicatorNames');
							indicatorNames[name]=$location.search(search).$$search;
							localStorageEngine.setLocalVar('indicatorNames', indicatorNames);
							localStorageEngine.setLocalVar('indicatorName', name);
							$location.$$search.Name = name;
							delete templateManager.containers["product#" + ViewService.DASHBORAD_WINDOW];
							$location.path('/product#' + ViewService.DASHBORAD_WINDOW).search('Name=' + name).hash('');
						});
					}
				});
			}

		}
	};

	function setDrillDownName(search){
		var name = '';
		if(search.indicatorView === undefined) {
			name = search.sId + '|' + search.menuId;
		}
		else if(search.menuId != undefined){
			name = search.indicatorView + '|' + search.menuId;
		}
		else if(search.indicatorView.length === 1) {
			name = search.sId + '|' + search.indicatorView;
		}
		else{
			for(var i =0; i<search.indicatorView.length - 1;i++){
				name = name + search.indicatorView[i] + '|';
			}
			name = name + search.indicatorView[search.indicatorView.length - 1];
		}
		return name;
	}

	function openNextIndicatorViewWhenIsStructureDrillDown (search) {
		return function (response) {
			var dataset = response.dataset;
			if (dataset && dataset.structureInfo && dataset.structureInfo.childStructure) {
				var childStructure = dataset.structureInfo.childStructure;
				var structure = dataset.structureInfo.structure;
				var drillDownColumn = dataset.structureInfo.drillDownColumn || '';
				UtilitiesController.setSearchParameterOrPush(search, ViewService.STRUCTURE_ID, structure? structure._id : '');
				UtilitiesController.setSearchParameterOrPush(search, ViewService.STRUCTURE_TYPE_ID, childStructure._id);
				UtilitiesController.setSearchParameterOrPush(search, ViewService.DRILL_PROPERTY, drillDownColumn);
				UtilitiesController.setSearchParameterOrPush(search, ViewService.STRUCTURE_NAME, structure? structure.name : '');
				return $location.path('/product#' + ViewService.INDICATOR_VIEW_WINDOW).search(search);
			}
		};
	}

	function setStructureDrillDownParameters(search, view) {
		var paramValue = !view.drillDownView ? '1' : '';
		UtilitiesController.setSearchParameterOrPush(search, ViewService.GROUP_BY_STRUCTURE, paramValue);
		UtilitiesController.setSearchParameterOrPush(search, ViewService.IS_STRUCTURE_DRILL, paramValue);
	}

	function verifyPropertyDimensionProperty(view, search, row) {
		var drillDownProperty = getDimensionDrillDownProperty(view);
		if(Util.isArray(view[drillDownProperty]) && view[drillDownProperty].length > 1){
			view[drillDownProperty].forEach(function(dimensionProperty){
				if(row[dimensionProperty] !== undefined ){
					UtilitiesController.setSearchParameterOrPush(search, ViewService.DRILL_VALUE, row[dimensionProperty]);
				}
			});
		} else {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.DRILL_VALUE, row[view[drillDownProperty]]);
		}
	}

	var getDimensionDrillDownProperty = function(view){
		if(Util.isArray(view.drillDownDimensions) && view.drillDownDimensions.length > 0) {
			return 'drillDownDimensions';
		}else{
			return view.dimensionProperty;
		}
	};

	function verifyPropertyPreviousConditions(previousConditions, search){
		if(previousConditions && (previousConditions.length > 0 || !Util.isEmptyObject(previousConditions))) {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.PREVIOUS_CONDITIONS, previousConditions);
		} else {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.PREVIOUS_CONDITIONS, "");
		}
	}

	function verifyPropertyPostConditions(postConditions, search){
		if(postConditions && (postConditions.length > 0 || !Util.isEmptyObject(postConditions))) {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.POST_CONDITIONS, postConditions);
		} else {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.POST_CONDITIONS, "");
		}
	}

	function verifyPropertyDrillDownType(view, search){
		if (view.hasDrillDown !== false){
			if(view.drillDownType === 'widget' || view.drillDownType === undefined){
				UtilitiesController.setSearchParameterOrPush(search, ViewService.INDICATOR_VIEW_ID, view.drillDownView || view._id);
			} else if(view.drillDownType === 'view') {
				UtilitiesController.setSearchParameterOrPush(search, ViewService.MENU_ID, view.drillDownDashboardField);
			}
		} else {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.INDICATOR_VIEW_ID, view._id);
		}
	}

	function setBaseDrillDownParameters(search, view, row, period) {
		verifyPropertyDimensionProperty(view, search, row);
		var periodSplit = period.split("_");
		var periodSplitLength = periodSplit.length;
		view.parameters.period = view.parameters.period == null ? 0 : view.parameters.period;
		if(periodSplit[periodSplitLength-1] && periodSplit[periodSplitLength-2]){
			search[ViewService.PERIOD_START] = periodSplit[periodSplitLength - 2];
			search[ViewService.PERIOD_END] = periodSplit[periodSplitLength - 1];
		} else 
		if(view.parameters.period.length >= 1 && !verifyAllPeriod(view.parameters.period)){
			var start  = view.parameters.period.map(function(period){
				return period.start;
			});
			var end = view.parameters.period.map(function(period){
				return period.end;
			});
			
		  search[ViewService.PERIOD_START] = start.length === 1 ? start[0] : start; 
		  search[ViewService.PERIOD_END] = end.length === 1 ? end[0] : end; 
		} else {
			search[ViewService.PERIOD_START] = view.parameters.period && view.parameters.period.start ? view.parameters.period.start : null;
			search[ViewService.PERIOD_END] = view.parameters.period && view.parameters.period.end ? view.parameters.period.end : null;
		}
		verifyPropertyPreviousConditions(view.previousConditions, search);
		verifyPropertyPostConditions(view.postConditions, search);
		verifyPropertyDrillDownType(view, search);
	}

	function verifyAllPeriod(period){
		var isAllPeriod = false;
		period.forEach(function(value){
			if(value == null){
				isAllPeriod = true;
			}
		});
		return isAllPeriod;
	}
	function setParametersWhenIsNotStructureDrillDown (search, view) {
		var dimensionProperties = view[getDimensionDrillDownProperty(view)];
		if(!Util.isArray(search[ViewService.DRILL_VALUE]) && Util.isArray(dimensionProperties)){
			dimensionProperties = dimensionProperties[0];
		}
		if(Util.isArray(dimensionProperties)){
			dimensionProperties.forEach(function(dimensionProperty){
				UtilitiesController.setSearchParameterOrPush(search, ViewService.DRILL_PROPERTY, dimensionProperty);
			});
		} else {
			UtilitiesController.setSearchParameterOrPush(search, ViewService.DRILL_PROPERTY, dimensionProperties);
		}
		var structureId = view.parameters.structureId;
		var structureName = view.parameters.structureName;
		return StructureService.getStructureByUser(search.fA).then(function(response){
			if(response && response.dataset && response.dataset.getStructureByUser){
				var topStructure = response.dataset.getStructureByUser.find(function(structure){
					return structure.topLevel;
				});
				if(topStructure){
					structureId = topStructure._id;
					structureName = topStructure.name;
				}
			}
			setSearchParameters(search, structureId, structureName);
		}).catch(function(){
			setSearchParameters(search, structureId, structureName);
		});	
	}

	var setSearchParameters = function(search, structureId, structureName){
		UtilitiesController.setSearchParameterOrPush(search, ViewService.STRUCTURE_ID, structureId);
		UtilitiesController.setSearchParameterOrPush(search, ViewService.STRUCTURE_NAME, structureName);
		UtilitiesController.setSearchParameterOrPush(search, ViewService.PREVIOUS_CONDITIONS, "", true);
		UtilitiesController.setSearchParameterOrPush(search, ViewService.POST_CONDITIONS, "", true);
		UtilitiesController.setSearchParameterOrPush(search, ViewService.STRUCTURE_TYPE_ID, '');
	};

	this.openPrevIndicatorView = function() {
		ChartViewController.removeChartWidgets();
		if(widget.getField("drillDownFilters").dataSource.data.length > 0){
			ChartViewController.setIsReturningFromDrillDown(true);
		}
		localStorageEngine.setLocalVar('returningFromBackButton', true);
		if(getLocalVar("filterWidget") && getLocalVar("filterWidget")[getLocalVar('indicatorName')]){
			var filterWidget = getLocalVar("filterWidget");
			delete filterWidget[getLocalVar('indicatorName')];
			setLocalVar("filterWidget",filterWidget);
		}
		return $window.history.back();
	};

	this.setNotReturningFromBackButton = function(){
		localStorageEngine.setLocalVar('returningFromBackButton', false);
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('DrillDownController', DrillDownController);
});

// FILE: assets/js/controllers/GoalNotificationsController.js
function GoalNotificationsController(templateManager, ScreenService, GoalNotificationsService, ChartViewService,
GenericDashboardService, UtilitiesController) {

    var self = this;

    var getGoalsNotificationsWidget = function(){
        return templateManager.container.getWidget('goalNotifications');
    };

    this.setFieldsVisibility = function(widget) {
        if(widget.currentRow.notifyCondition === 'periodically'){
            widget.getField('scheduleExp').isVisible = true;
        } else {
            widget.getField('scheduleExp').isVisible = false;
            delete widget.currentRow.scheduleExp;
        }
    };

    this.addNewNotification = function(widget) {
        widget.newRow();
        widget.currentRow.goalId = widget.parent.currentRow._id;
        widget.currentRow.dataSourceId = widget.parent.currentRow.dataSourceId;
    };

    this.saveNotification = function(widget) {
        delete widget.backupRow;
        widget.currentRow.dataSourceId = widget.parent.currentRow.dataSourceId;
        widget.saveRow().then(function(response){
            widget.view();
        });
    };

    this.editNotification = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    this.rollBack = function(widget) {
        if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
    };

    this.goalsNotificationsWidgetBeforeMoveRow = function(){
        UtilitiesController.deleteChangedFlag(getGoalsNotificationsWidget());
    };

    this.goalsNotificationsWidgetAfterMoveRow = function(){
        self.setFieldsVisibility(getGoalsNotificationsWidget());
    };

    this.goalsNotificationsActionEventBackList = function(){
        ScreenService.closeSwipe();
    };

    this.goalsNotificationsActionEventRemove = function(){
        var widget = getGoalsNotificationsWidget();
        widget.removeCheckedRows().then(widget.reload);
    };

    this.setReceiversMails = function(widget) {
        var mails = widget.currentRow.receivers;
        mails = mails.split(';');
        widget.currentRow.receiversMails = mails;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GoalNotificationsController', GoalNotificationsController);
});

// FILE: assets/js/controllers/ImportDefaultController.js
function ImportDefaultController(templateManager, IndicatorService, ScreenService, ImportDefaultService, ViewService, $rootScope, $location,
                                 UtilitiesController, localStorageEngine, GenericDashboardController, ConnectionService, ConnectionController) {

    var self = this;
    var defaultImportDatabase = 5;

    var getImportWidget = function () {
        return templateManager.container.getWidget('import');
    };

    var getImportTreeWidget = function () {
        return templateManager.container.getWidget('importTree');
    };

    var getImportTabbedWidget = function () {
        return templateManager.container.getWidget('importTabbed');
    };

    var getImportTabbedMenuWidget = function () {
        return templateManager.container.getWidget('import_menu');
    };

    var getImportTabbedDashboardWidget = function () {
        return templateManager.container.getWidget('import_dashboard');
    };

    var getImportTabbedIndicatorWidget = function () {
        return templateManager.container.getWidget('import_indicator');
    };

    var getImportTabbedIndicatorViewsWidget = function () {
        return templateManager.container.getWidget('import_indicator_views');
    };

    var getAccessProfilesWidget = function () {
        return templateManager.container.getWidget('setAccessProfiles');
    };

    var getCreateConnectionWidget = function () {
        return templateManager.container.getWidget('createConnection');
    };

    var getFieldAccessProfilesFromAccessProfilesWidget = function () {
        return getAccessProfilesWidget().getField('accessProfiles');
    };

    var getFieldDbUrlFromCreateConnectionWidget = function(){
        return getCreateConnectionWidget().getField("dbUrl");
    };

    var getFieldServiceFromCreateConnectionWidget = function(){
        return getCreateConnectionWidget().getField("service");
    };

    var importSelected = function () {
        var importWidget = getImportTreeWidget();
        var rows = importWidget.getCheckedRows();
        var accessProfiles = getFieldAccessProfilesFromAccessProfilesWidget().value();
        rows.forEach(function (row) {
            row.__rightActions = [];
        });
        ImportDefaultService.importRows(rows, accessProfiles).then(function (response) {
            importWidget.reload();
            ViewService.updateMenu();
            if (response && response.notifications && response.notifications.length > 0) {
                response.notifications.forEach(function (notification) {
                    UtilitiesController.notificationMessage(notification.message, notification.type);
                });
            }
        });
    };


    this.loadImportMenusAndDashboards = function () {
        IndicatorService.setBaseConnectionToImport(defaultImportDatabase).then(function () {
            getImportTreeWidget().reload();
        });
    };

    this.colorRowIfAlreadyImported = function (element, row) {
        if (row && row.alreadyImported) {
            element.css("color", "#0000ff");
        }
    };

    this.loadViews = function () {
        var indicatorWidget = getImportTabbedIndicatorWidget();
        var indicatorViewsWidget = getImportTabbedIndicatorViewsWidget();
        indicatorViewsWidget.dataSourceFilter = [
            {
                "name": "indicator",
                "operator": "=",
                "value": indicatorWidget.currentRow._id
            }
        ];
        IndicatorService.setBaseConnectionToImport(defaultImportDatabase).then(function () {
            indicatorViewsWidget.reload();
        });
    };

    this.dashboardPreview = function(){
        var id = getImportTabbedDashboardWidget().currentRow._id;
        openDashboardPreview(id);
    };

    var openDashboardPreview = function(name){
        UtilitiesController.openWindow("product#dashboardPreview?Name=" + name,true);
    };

    this.buildDashboardPreview = function(){
        var id = $location.$$hash.substring($location.$$hash.indexOf("Name=") + 5);
        GenericDashboardController.buildPreviewDashboards(id).then(function(){
            var tabbedWidget = templateManager.container;
            GenericDashboardController.activeDashboardSearch(true);

            var firstWidget = tabbedWidget.widgets[0].widgets[0];
            tabbedWidget.currentWidget = firstWidget;
            firstWidget.activate();
            templateManager.updateTemplate();
        });
    };

    this.importWidgetAfterSelectRow = function (selectedRow) {
        if (selectedRow.type && selectedRow.type == "I") {
            var widget = getImportTreeWidget();
            var parent = widget.dataSource.data.filter(function (row) {
                return row.primaryTreeId == selectedRow.parentTreeId;
            });
            if (parent && parent[0]) {
                widget.dataSource.addCheckedRows(parent[0]);
            }
        }
    };

    this.dashboardOrViewPreview = function(){
        var widget = getImportTreeWidget();
        var row = widget.currentRow;
        if(row && row.type){
            if(row.type == "D"){
                openDashboardPreview(row._id);
            }else if(row.type == "I"){
                widget.computeDescriptionOrder();
                self.importWidgetAfterMoveRow().then(function(){
                    getImportTabbedIndicatorViewsWidget().activate();
                });
            }
        }
    };

    this.viewPreview = function(){
        var id = getImportTabbedIndicatorViewsWidget().currentRow._id;
        openViewPreview(id);
    };

    var openViewPreview = function(id){
        localStorageEngine.setLocalVar(ViewService.INDICATOR_VIEW_ID, id);
        var defaultParameters = ViewService.mapDefaultParameters();
        var blankParameters = defaultParameters.reduce(function(prev, cur) {
            return prev + (cur ? '=' + '&' + cur : '');
        }) + '=';
        var search = urlConstants.VIEW_LEVEL + '=0&' + constants.INDICATOR_VIEW_ID + '=' + id;
        search = search + '&' + blankParameters;

        var indicatorNames = getLocalVar("indicatorNames");
        if(indicatorNames === undefined){
            indicatorNames = {};
        }
        indicatorNames[id] = $location.search(search).$$search;
        localStorageEngine.setLocalVar("indicatorNames",indicatorNames);
        if(getLocalVar("filterWidget")){
            getLocalVar("filterWidget")[getLocalVar('indicatorName')] = undefined;
        }

        UtilitiesController.openWindow("product#viewPreview?Name=" + id,true);
    };

    this.importWidgetAfterMoveRow = function () {
        var importTreeWidget = getImportTreeWidget();
        if (importTreeWidget.currentRow.type) {
            var type = importTreeWidget.currentRow.type;
            var activateWidget = null;
            var dashboardWidget = getImportTabbedDashboardWidget();
            var menuWidget = getImportTabbedMenuWidget();
            var indicatorWidget = getImportTabbedIndicatorWidget();
            var indicatorViewsWidget = getImportTabbedIndicatorViewsWidget();
            var visibleWidgets = {
                "D": [dashboardWidget.name],
                "M": [menuWidget.name],
                "I": [indicatorWidget.name, indicatorViewsWidget.name]
            };
            var allWidgets = [dashboardWidget, menuWidget, indicatorWidget, indicatorViewsWidget];
            allWidgets.forEach(function (widget) {
                widget.isVisible = visibleWidgets[type].includes(widget.name);
                if (widget.template == "widget/form-view.html" && widget.isVisible) {
                    widget.currentRow = importTreeWidget.currentRow;
                    widget.view();
                    activateWidget = widget;
                }
            });
            return ScreenService.openSwipe(getImportTabbedWidget()).then(function () {
                if (activateWidget) {
                    activateWidget.activate();
                }
            });
        }
    };

    var openPopupAccessProfiles = function(){
        var rows = getImportTreeWidget().getCheckedRows();
        var openAccessProfilesPopup = rows.some(function (row) {
            return row.type && (row.type == "D" || row.type == "M");
        });
        if (openAccessProfilesPopup) {
            ScreenService.openPopup(getAccessProfilesWidget());
        } else {
            importSelected();
        }
    };

    var openPopupCreateConnection = function(){
        var dbUrlField = getFieldDbUrlFromCreateConnectionWidget();
        dbUrlField.value(dbUrlField.defaultValue);
        getFieldServiceFromCreateConnectionWidget().value(true);
        ScreenService.openPopup(getCreateConnectionWidget());
    };

    var getDefaultConnectionParameters = function(){
        return {
            "oracleWalletUse": "N",
            "driverClass": "oracle.jdbc.driver.OracleDriver",
            "name": "MAIN CONNECTION",
            "language":"pt_br",
            "languageLabel": "Portuguese (BR)",
            "database": "Oracle",
            "main": true,
            "dialect": "org.hibernate.dialect.OracleDialect"
        };
    };

    this.testConnection = function(){
        var widget = getCreateConnectionWidget();
        if(widget.isValid()){
            widget.currentRow = Object.assign(widget.currentRow, getDefaultConnectionParameters());
            ConnectionController.testConnection(widget);
        }
    };

    this.createConnectionWidgetActionEventSave = function(){
        var widget = getCreateConnectionWidget();
        if(widget.isValid()){
            ScreenService.closePopup(widget);
            widget.currentRow = Object.assign(widget.currentRow, getDefaultConnectionParameters());
            ConnectionController.saveConnection(widget).then(function(response){
                openPopupAccessProfiles();
            });
        }
    };

    this.importTreeWidgetActionEventImport = function () {
        ConnectionService.checkCreateNewConnection().then(function(response){
            if(response.dataset.dataset.hasConnection || response.dataset.dataset.idOrgSaas){
                openPopupAccessProfiles();
            }else{
                openPopupCreateConnection();
            }
        });
    };

    this.accessProfilesWidgetActionEventImport = function () {
        var widget = getAccessProfilesWidget();
        if (widget.isValid()) {
            ScreenService.closePopup(widget);
            importSelected();
        }
    };

    this.accessProfilesWidgetActionEventCancel = function () {
        ScreenService.closePopup(getAccessProfilesWidget());
    };

    this.createConnectionWidgetActionEventCancel = function () {
        ScreenService.closePopup(getCreateConnectionWidget());
    };

    this.importViewWidgetActionEventCancel = function () {
        ScreenService.closeSwipe();
        getImportTreeWidget().activate();
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register('ImportDefaultController', ImportDefaultController);
});

// FILE: assets/js/controllers/ImportQueueController.js
function ImportQueueController(templateManager, ImportQueueService, ScreenService) {
    
    var self = this;
    
    function getQueueWidget() {
        return templateManager.container.getWidget('manualImportQueue');
    }

    this.loadQueue = function() {
        var queueWidget = getQueueWidget();
        var dataSourceId = queueWidget.parent.currentRow._id;
        ImportQueueService.getQueueByDataSourceId(dataSourceId).then(function(response) {
            var queue = response.dataset.manualImportQueue;
            queue.forEach(function(item) {
                item.period = formatPeriod(item.period);
            });
            queueWidget.dataSource.data = queue;
        });
    };

    function formatPeriod(itemPeriod) {
        var period = itemPeriod;
        if(period && period.start && period.end) {
            period = period.start + ' - ' + period.end;
        }
        return period;
    }

    this.removeSchedule = function() {
        var queueWidget = getQueueWidget();
        var queueItem = queueWidget.currentRow;
        ImportQueueService.removeItemFromQueue(queueItem).then(function(response){
            UtilitiesController.successNotification('The item was successfully removed from queue!');
            self.loadQueue();
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ImportQueueController', ImportQueueController);
});


// FILE: assets/js/controllers/ImportSchedulingController.js
function ImportSchedulingController(templateManager, PeriodsManagerService, MasterDetailUtilities,
MessageService, ScreenService, UtilitiesController, DataSourceManagerController, DataSourceManagerFactory,
DataSourceManagerService, localStorageEngine, ImportQueueService, PeriodsUtilities) {

    var self = this;
    var importNowWidget;
    var DATASOURCE_KEYCONFIRMATION = "The Data Source keys must order the data in a unique way. Would you like to proceed?";
    var CODE_FINALITY_SEND_MAIL = DataSourceManagerService.CODE_FINALITY_SEND_MAIL;

    var getImportNowWidget = function () {
      return  templateManager.container.getWidget('importNow');
    };

    var getImportNowParentWidget = function () {
      return  templateManager.container.getWidget('importNow').parent;
    };

    var getWidgetImportScheduling= function(){
        return templateManager.container.getWidget("importScheduling");
    };

    function getImportSchedulingWidget() {
        return templateManager.container.getWidget('importScheduling');
    }

    function getDataSourceWidget() {
		return templateManager.container.getWidget('dataSource');
	}

    this.setValuesFromParent = function() {
        var widget = getImportSchedulingWidget();

        if(typeof widget.currentRow.period !== "object") {
			var periodName = widget.currentRow.period ? widget.currentRow.period : 'All period';
			widget.currentRow.period = {"period": { "name": periodName }};
		}

        var period = null;
        if(!widget.currentRow.period){
            period = widget.currentRow.period;
        }

		PeriodsManagerService.setPeriodFieldValue(widget, 'period').then(function(){
			self.setPersistTypeDatasource();
			setDataSourceKeys(widget);
			setDateFilterColumn(widget);
			setImportSchedulingData(widget);

            if(period){
                widget.currentRow.period = period;
            }
		});
    };

    this.setPersistTypeDatasource = function(){
		var widgetDataSource = templateManager.container.getWidget('dataSource');
		var widgetImportScheduling = templateManager.container.getWidget('importScheduling');
		var persistTypeData = widgetImportScheduling.getField("persistType").dataSource.data;
		var checkData = false;

		persistTypeData.forEach(function(data){
			if(data.name == "Drop By Period"){
				checkData = true;
			}
		});

		if (widgetDataSource.currentRow.dateFilterColumn === null && checkData === true){
			persistTypeData.dataSource.pop();
		} else if (widgetDataSource.currentRow.dateFilterColumn !== null && checkData === false){
			persistTypeData.push(
				{
					"name": "Drop By Period",
					"value": "dropByPeriod"
				}
			);
		}
	};

	function setDataSourceKeys(widget) {
		var dataSourceWidget = getDataSourceWidget();
		var dataSourceColumns = dataSourceWidget.currentRow.columns;

		widget.getField('keys').dataSource.data = (dataSourceColumns || []);

		var keyColumns = [];
		if(Util.isArray(dataSourceColumns)) {
			keyColumns = dataSourceColumns.filter(function(column) {
				return column.isKey;
			}).map(function(column) {
				return column.name;
			});
		}

		widget.currentRow.keys = keyColumns;
	}

	function setDateFilterColumn(widget) {
		var dataSourceWidget = getDataSourceWidget();
		var dataSourceColumns = dataSourceWidget.currentRow.columns;

		var dateColumns = [];
		if(Util.isArray(dataSourceColumns)) {
			dateColumns = dataSourceColumns.filter(function(column) {
				return column.isDate;
			});
		}

		var importSchedulingWidget = templateManager.container.getWidget("importScheduling");
		importSchedulingWidget.getField('dateFilterColumnLabel').dataSource.data = dateColumns;
		widget.currentRow.dateFilterColumnLabel = dataSourceWidget.currentRow.dateFilterColumnLabel;
		widget.currentRow.dateFilterColumn = dataSourceWidget.currentRow.dateFilterColumn;
	}

	function setImportSchedulingData(widget) {
	    var dataSourceRow = getDataSourceWidget().currentRow;
		var persistTypeField = widget.getField('persistType');
		var periodField = widget.getField('period');
		var scheduleExpressionField = widget.getField('scheduleExpression');
		var beforeImportScript = widget.getField('beforeImportScript');
		var afterImportScript = widget.getField('afterImportScript');
		var execAfterIfImportError = widget.getField('execAfterIfImportError');
		var execAfterIfBeforeScriptError = widget.getField('execAfterIfBeforeScriptError');
		var pageSizeField = widget.getField('pageSize');
		var isActiveField = widget.getField('isActive');

		persistTypeField.value(dataSourceRow.persistType);
		pageSizeField.value(dataSourceRow.pageSize || null);
		periodField.value(widget.currentRow.period || dataSourceRow.period || null);
		scheduleExpressionField.value(dataSourceRow.scheduleExpression || '');
		isActiveField.value(dataSourceRow.isScheduleActive || false);
		beforeImportScript.value(dataSourceRow.beforeImportScript || '');
		afterImportScript.value(dataSourceRow.afterImportScript || '');
		execAfterIfImportError.value(dataSourceRow.execAfterIfImportError || false);
		execAfterIfBeforeScriptError.value(dataSourceRow.execAfterIfBeforeScriptError || false);
	}

	this.editImportScheduling = function(widget){
		if(getDataSourceWidget().currentRow.connection){
			MasterDetailUtilities.formEdit(widget);
		} else {
			MessageService.showMessageByCode('NO_CONNECTION');
		}
	};

    this.importSchedulingFieldOnChangePersistType = function(args){
        var field = args.owner.owner;
        var widget = field.getParent();
        if(field.value() === "dropByPeriod"){
            widget.getField("dateFilterColumnLabel").validations = {required:{}};
            widget.getField("period").validations = {required:{}};
        }
        else{
            widget.getField("dateFilterColumnLabel").validations = null;
            widget.getField("period").validations = null;
        }
    };

    this.importNowFieldOnChangePersistType = function(args){
        var field = args.owner.owner;
        var widget = field.getParent();
        if(field.value() === "dropByPeriod"){
            widget.getField("period").validations = {required:{}};
        }
        else{
            widget.getField("period").validations = null;
        }
    };

	this.saveImportScheduling = function(importSchedulingWidget) {
        var widget = getImportNowParentWidget();
		getImportNowParentWidget().currentRow.isEdited = true;


        var dataSourceWidget = getDataSourceWidget();
        if(dataSourceWidget.currentRow.finality === CODE_FINALITY_SEND_MAIL){
            importSchedulingWidget.currentRow.keys = ['ID'];
            importSchedulingWidget.currentRow.dateFilterColumnLabel = 'ID';
            importSchedulingWidget.currentRow.persistType = 'drop';
        }
		if (validateImportScheduleWidget(importSchedulingWidget)) {
            if(dataSourceWidget.currentRow.finality !== CODE_FINALITY_SEND_MAIL &&
                JSON.stringify(widget.backupRow.keys) != JSON.stringify(widget.currentRow.keys)){
                UtilitiesController.confirmMessage(DATASOURCE_KEYCONFIRMATION, "alert", function(){
                    importSchedulingSaveProcess(importSchedulingWidget);
                });
            }else{
                importSchedulingSaveProcess(importSchedulingWidget);
            }
		}
	};

    var importSchedulingSaveProcess = function(importSchedulingWidget) {
        if(importSchedulingWidget.currentRow.period == undefined){
            importSchedulingWidget.currentRow.period = null;
        }
        var currentRow = importSchedulingWidget.currentRow;
        var dataSourceCurrentRow = importSchedulingWidget.parent.currentRow;
        dataSourceCurrentRow.period = formatPeriodToSchedule(currentRow.period);

        var schedulingProperties = ['persistType', 'scheduleExpression', 'beforeImportScript',
            'afterImportScript', 'execAfterIfImportError', 'execAfterIfBeforeScriptError',
            'dateFilterColumn', 'dateFilterColumnLabel', 'keys', 'pageSize'];

        schedulingProperties.forEach(function (property) {
            dataSourceCurrentRow[property] = currentRow[property];
        });

        dataSourceCurrentRow.type = 'scheduledBI';
        dataSourceCurrentRow.isScheduleActive = currentRow.isActive;
        dataSourceCurrentRow.isScheduleSave = true;
        dataSourceCurrentRow.isEdited = currentRow.isEdited;

        PeriodsManagerService.setPeriodFieldLabel(importSchedulingWidget, 'period');
        setChangedColumns(importSchedulingWidget);
        DataSourceManagerController.saveDataSource(importSchedulingWidget.parent).then(function(){
            importSchedulingWidget.redraw();
        });

    };

	function validateImportScheduleWidget(importSchedulingWidget) {
		var isScheduleActive = importSchedulingWidget.currentRow.isActive;
		if(importSchedulingWidget.isValid()) {
			if(isScheduleActive && !importSchedulingWidget.currentRow.scheduleExpression) {
				MessageService.showMessageByCode('REQUIRED_SCHEDULE_EXP');
			} else {
				return true;
			}
		} else {
			MessageService.showMessageByCode('REQUIRED_FIELDS');
		}
	}

	function formatPeriodToSchedule(period){
		var CUSTOM_PERIOD_IDENTIFIER = '@';
		var PERIOD_CODE_MARKS = "'";

		if(period && period.period && period.period.name){
			var periodCode = period.period.name;
			period = periodCode.charAt(0) === CUSTOM_PERIOD_IDENTIFIER ? periodCode.split(PERIOD_CODE_MARKS)[1] : periodCode;
		}

		return period;
	}

	function setChangedColumns(importSchedulingWidget){
		var widgetKeys = importSchedulingWidget.getField('keys');

		if(widgetKeys.changed){
			setKeysColumns(getDataSourceWidget().currentRow);
		}
	}

	function setKeysColumns(row) {
		if(row.columns && row.keys) {
			row.columns = row.columns.map(function(column) {
				column.isKey = (row.keys.indexOf(column.name) !== -1);
				return column;
			});
		}
	}

	this.openImportNowPopup = function (importSchedulingWidget) {
		if (!importNowWidget) {
			importNowWidget = DataSourceManagerFactory.factoryImportNowWidget(importSchedulingWidget.container);
		}
		importNowWidget.isVisible = true;
		importNowWidget.newRow();
		importNowWidget.currentRow = Util.clone(importSchedulingWidget.currentRow);
		importNowWidget.currentRow.period = Util.clone(importSchedulingWidget.currentRow.period);
		delete importNowWidget.currentRow.period.period;
		ScreenService.openPopup(importNowWidget);
	};

    this.importNowActionEvent = function(importWidget){
        if(importWidget.getField("period").validations && importWidget.getField("period").validations.required){
            if(importWidget.getField("period").value()){
                this.checkImportExistence(importWidget);
            }else{
                UtilitiesController.showMessage("MISSING_PERIOD");
            }
        }
        else{
            this.checkImportExistence(importWidget);
        }
    };

	this.checkImportExistence = function (importWidget){
		var importSchedulingWidget = getImportSchedulingWidget();
		if (importSchedulingWidget.isValid()) {
			var scheduleItem = buildScheduleToSave(importWidget, importSchedulingWidget);
			DataSourceManagerService.findImportsInProgress(scheduleItem.dataSource).then(function(importsInProcess){
				var processes = importsInProcess.dataset;
				if(processes.totalImportsInProcess[0] > 0) {
				    showSchedulingOptions(scheduleItem, processes);
				} else {
                    self.saveImportSchedule(scheduleItem);
				}
			});
		}
	};

	function buildScheduleToSave(importNowWidget, importSchedulingWidget) {
		var dataSourceId = importSchedulingWidget.parent.currentRow._id;
		var currentRow = importNowWidget.currentRow;
		//The schedule expression will be empty when making an import now operation
		return {
			dataSource: dataSourceId,
			period: formatPeriodToSchedule(currentRow.period),
			persistType: currentRow.persistType,
			scheduleExpression: '',
			beforeImportScript: currentRow.beforeImportScript,
			afterImportScript: currentRow.afterImportScript,
			execAfterIfImportError: currentRow.execAfterIfImportError,
			execAfterIfBeforeScriptError: currentRow.execAfterIfBeforeScriptError,
			status: 'queue',
			type: 'manualBI'
		};
	}

	function showSchedulingOptions(scheduleItem, processes) {
	    var messageText = 'There are one or more import in progress. What would you like to do?';
	    var dialogType = 'CONFIRMATION';
	    var options = [
	        {
	            label: ScreenService.i18n("Cancel"),
	            code: cancelScheduling(scheduleItem)
	        },
	        {
	            label: ScreenService.i18n("Replace import processes"),
	            code: replaceImportProcesses(scheduleItem, processes)
	        },
	        {
	            label: ScreenService.i18n("Add to queue"),
	            code: addScheduleToQueue(scheduleItem)
	        }
	    ];

	    ScreenService.showCustomDialog(messageText, dialogType, null, options);
	}

	var replaceImportProcesses = function(scheduleItem, processes){
	    return function() {
    		if(processes.totalImportsBlocked[0] > 0){
    			MessageService.showMessageByCode('IMPORT_IN_PROCESS');
    			ScreenService.closePopup();
    		} else {
    			DataSourceManagerService.cancelImportation(scheduleItem.dataSource).then(function(){
    				self.saveImportSchedule(scheduleItem);
    			});
    		}
	    };
	};

	function treatScheduleNowResponse(response) {
		response = response || {};
		var status = 0;
		var errorMessage = "There was an error making the import";
		var successMessage = "Importation successfully scheduled! You can follow up the process at the tab 'Import History' in a few moments";
		if (response.notifications && response.notifications[0]) {
			var notification = response.notifications[0];
			if (notification.type === 'success') {
				status = 1;
			} else {
				errorMessage = notification.message;
			}
		} else if (response.messages && response.messages[0]) {
			var message = response.messages[0];
			if (message.type === 'E') {
				errorMessage = message.message;
			} else {
				status = 1;
			}
		}

		if(status){
			ScreenService.closePopup();
			templateManager.container.getWidget('importScheduling');
			UtilitiesController.successNotification(successMessage);
		} else {
			ScreenService.alertNotification(errorMessage, 'E', 'x');
		}
	}

	var cancelScheduling = function(scheduleItem){
	    return function() {
    		DataSourceManagerService.updateStatusSchedule(scheduleItem.dataSource);
    		MessageService.alertNotificationByCode('CANCEL_IMPORTATION_SCHEDULING');
    		ScreenService.closePopup();
	    };
	};

	var addScheduleToQueue = function(scheduleItem) {
	    return function() {
	        scheduleItem.userResponsible = localStorageEngine.getLocalVar('username');
	        ImportQueueService.addItemToQueue(scheduleItem);
    		ScreenService.closePopup();
	    };
	};

	self.saveImportSchedule = function(scheduleItem) {
			DataSourceManagerService.saveImportSchedule(scheduleItem).then(function(response) {
			treatScheduleNowResponse(response);
		});
	};


	this.pageSizeAlert = function(field) {
		var pageSize = field.value();
		var notification;

		if(pageSize < 500) {
			notification = "Too short page size may cause a timeout at the server";
		} else if (pageSize > 5000) {
			notification = "Too large page size may cause a forced shut down of the server";
		}

		if(notification) {
			ScreenService.alertNotification(notification, 10000);
		}
	};

	this.importSchedulingActionEventImport = function() {
		if(!!getImportNowParentWidget().parent.currentRow.isEdited){
			getImportNowWidget().currentRow = Util.clone(getImportNowParentWidget().currentRow);
			ScreenService.openPopup(getImportNowWidget());
			PeriodsUtilities.setIntervalsOnPeriodField(getPeriodFieldOnImportSchedulingyTab());
            if(getPeriodFieldOnImportSchedulingyTab().value().period && getPeriodFieldOnImportSchedulingyTab().value().period.name && !getPeriodFieldOnImportSchedulingyTab().value().period.name.includes('@')){
                getPeriodFieldOnImportSchedulingyTab().value().period.name = PeriodsUtilities.getCustomPeriodCode(getPeriodFieldOnImportSchedulingyTab().value().period.name);
            }
		} else {
			UtilitiesController.showMessage("IMPORT_SCHEDULE_EMPTY_FIELDS");
		}
	};

	var getPeriodFieldOnImportSchedulingyTab = function () {
		return getImportNowWidget().getField('period');
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ImportSchedulingController', ImportSchedulingController);
});


// FILE: assets/js/controllers/JoinDataSourceController.js
function JoinDataSourceController(templateManager, UtilitiesController, DataSourceManagerService, ScreenService,
DataSourceManagerController, JoinDataSourceService) {

	var JOIN_SETTINGS_WIDGET_NAME = 'joinSettings',
		COLUMNS_RELATIONS_TAB_WIDGET = 'columnsRelations_tab',
		COLUMNS_RELATIONS_WIDGET = 'columsRelations',
		DATASOURCE_WIDGET = 'dataSource',
		KEY_COLUMNS_PROPERTY = 'isKey',
		FACT_COLUMNS_PROPERTY = 'isFact',
		DIMENSION_COLUMNS_PROPERTY = 'isDimension',
		ORIGIN_KEYS_FIELD = 'originKeys',
		ORIGIN_FACTS_FIELD = 'originFacts';

	this.configWidgetWhenJoinDataSource = function(isJoinDataSource, widgetName) {
        JoinDataSourceService.configWidgetWhenJoinDataSource(isJoinDataSource, widgetName);
	};

	this.setJoinSettingsData = function(widget){
		widget.dataSource.data = [];
		var parentWidget = widget.parent;
		var originDataSourcesIds = parentWidget.currentRow.originDataSources;

		originDataSourcesIds.forEach(function(originDataSourceId){
			parentWidget.dataSource.data.forEach(function(dataSource){
				if(dataSource._id === originDataSourceId){
					var originKeys = filterColumns(dataSource.columns, DIMENSION_COLUMNS_PROPERTY);
					var originFacts = filterColumns(dataSource.columns, FACT_COLUMNS_PROPERTY);
					var data = buildJoinSettingsData(dataSource, originKeys, originFacts, parentWidget.currentRow);
					widget.dataSource.data.push(data);
				}
			});
		});
	};

	function buildJoinSettingsData(dataSource, originKeys, originFacts, joinDataSource){
		return {
			originDataSourceId: dataSource._id,
			originDataSourceName: dataSource.name,
			originKeys: getColumnNames(originKeys),
			originFacts: getColumnNames(originFacts),
			joinDataSourceId: joinDataSource._id,
			joinColumns: joinDataSource.columns
		};
	}

	function getColumnNames(columns) {
		var columnNames = [];
		columns.forEach(function(column){
			columnNames.push(column.name);
		});
		return columnNames;
	}

	function filterColumns(columns, filter){
		var filteredColumns = [];
		columns.forEach(function(column){
			if(column.isGenerated && column[filter]){
				filteredColumns.push(column);
			}
		});
		return filteredColumns;
	}

	this.setOriginColumnsData = function(widget){
		widget.parent.dataSource.data.forEach(function(dataSource) {
			if (dataSource._id === widget.currentRow.originDataSourceId) {
				var originKeys = filterColumns(dataSource.columns, DIMENSION_COLUMNS_PROPERTY);
				var originFacts = filterColumns(dataSource.columns, FACT_COLUMNS_PROPERTY);

				widget.getField(ORIGIN_KEYS_FIELD).dataSource.data = originKeys;
				widget.getField(ORIGIN_FACTS_FIELD).dataSource.data = originFacts;
			}
		});
	};

	this.openAndSetRelationsWidget = function(relationType){
		ScreenService.openSwipe(templateManager.container.getWidget(COLUMNS_RELATIONS_TAB_WIDGET)).then(function(){
			setRelationsData(relationType);
		});
	};

	function setRelationsData(relationType){
		var fieldName = relationType === 'keys' ? ORIGIN_KEYS_FIELD : ORIGIN_FACTS_FIELD,
			columnProperty = relationType === 'keys' ? DIMENSION_COLUMNS_PROPERTY : FACT_COLUMNS_PROPERTY,
			joinSettingsWidget = templateManager.container.getWidget(JOIN_SETTINGS_WIDGET_NAME),
			originDataSourceId = joinSettingsWidget.currentRow.originDataSourceId,
			joinDataSourceWidget = joinSettingsWidget.parent;

		var relations = getColumnsTranslationsFromDataSourceInformations(joinDataSourceWidget, relationType, originDataSourceId);
		relations = getColumnsRelationsFromTranslations(relations);

		var relationsWidget = templateManager.container.getWidget(COLUMNS_RELATIONS_WIDGET);
		relationsWidget.relationType = relationType;
		relationsWidget.parentsDataSource.data = joinSettingsWidget.getField(fieldName).dataSource.data;
		relationsWidget.childsDataSource.data = filterColumns(joinDataSourceWidget.currentRow.columns, columnProperty);
		relationsWidget.relationsDataSource.data = relations;

		if(relationsWidget.relationsDataSource.data.length === 0) {
			try{
				relationsWidget.editActionClick();
			} catch(e){}
		}
	}

	function getColumnsTranslationsFromDataSourceInformations(joinDataSourceWidget, relationType, originDataSourceId){
		var translations = [];
		var dataSourcesInformations = joinDataSourceWidget.currentRow.dataSourcesInformations;
		if (dataSourcesInformations && dataSourcesInformations[originDataSourceId]) {
			dataSourcesInformations = dataSourcesInformations[originDataSourceId];
			var columnsTranslations = relationType === 'keys' ? dataSourcesInformations.keysTranslation : dataSourcesInformations.factsTranslation;

			for (var translation in columnsTranslations) {
				var columnTranslation = {
					parent: translation,
					children: columnsTranslations[translation]
				};
				translations.push(columnTranslation);
			}
		}

		return translations;
	}

	function getColumnsRelationsFromTranslations(translations){
		var columnsRelations = [];
		translations.forEach(function(translation){
			translation.children.forEach(function(child){
				var relation = {
					originColumn: translation.parent,
					joinColumn: child
				};
				columnsRelations.push(relation);
			});
		});

		return columnsRelations;
	}

	function formatRelationToSave(relationsData){
		var translations = {};
		relationsData.forEach(function(relation) {
			var originColumn = relation.originColumn;
			var translation = translations[originColumn] || [];
			translation.push(relation.joinColumn);
			translations[originColumn] = translation;
		});

		return translations;
	}

	this.saveRelations = function(relationsWidget){
		var translationType = relationsWidget.relationType === 'keys' ? 'keysTranslation' : 'factsTranslation';
		var joinDataSourceWidget = templateManager.container.getWidget(DATASOURCE_WIDGET);
		var relationsData = relationsWidget.relationsDataSource.data;
		var joinSettingsWidget = templateManager.container.getWidget(JOIN_SETTINGS_WIDGET_NAME);
		var originDataSourceId = joinSettingsWidget.currentRow.originDataSourceId;

		if(!joinDataSourceWidget.currentRow.dataSourcesInformations){
			joinDataSourceWidget.currentRow.dataSourcesInformations = {};
		}
		if(!joinDataSourceWidget.currentRow.dataSourcesInformations[originDataSourceId]){
			joinDataSourceWidget.currentRow.dataSourcesInformations[originDataSourceId] = {};
		}

		joinDataSourceWidget.currentRow.dataSourcesInformations[originDataSourceId][translationType] = formatRelationToSave(relationsData);

		DataSourceManagerController.saveDataSource(joinDataSourceWidget).then(function(){
			relationsWidget.list();
			relationsWidget.activate();
		});
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('JoinDataSourceController', JoinDataSourceController);
});

// FILE: assets/js/controllers/LoginDecorator.js
function LoginDecorator() {
    var INIT_CONTAINERS = ['userLogin', 'logoff', 'init', 'signUp'];
    this.apply = function(containerName, container) {
        if (INIT_CONTAINERS.indexOf(container.name) === -1) {
            container.events = container.events || [];
            container.events.push({
                'name': 'ContainerBeforeinit',
                'code': 'ZmartLoginController.validateLoginOnEnterProject(true);'
            });
        }
        else if (container.name === 'userLogin') {
            var loginWidget = container.widgets[0];
            loginWidget.fields.push(forgotPasswordField);
        }
        return container;
    };

    var forgotPasswordField = {
        "name": "FORGOTPASSWORD",
        "label": "Forgot password?",
        "class": "col-xs-offset-1",
        "placeholder": null,
        "readOnly": false,
        "isVisible": true,
        "isVisibleForm": true,
        "isVisibleGrid": false,
        "icon": "question icon-black",
        "template": "field/label-with-icon.html",
        "events": [{
            "id": "292088786982201526348",
            "name": "FieldOnClick",
            "code": "ZmartLoginController.openForgotPassword(args.owner.owner.widget);"
        }]
    };
}

Configuration(function(ContainerDecoratorProvider, ContextRegister) {
    ContainerDecoratorProvider.registerProvider(new LoginDecorator());
});

// FILE: assets/js/controllers/MenuConfiguration.js
function MenuConfigurationController(ScreenService, templateManager, ApplicationContext,
                                     UtilitiesController, ViewService, MessageService) {

    var self = this;

    this.containerBeforeInitMenuConfiguration = function () {
        ApplicationContext.engine.initSessionAcl();
    };

    var getMenuConfigurationWidget = function () {
        return templateManager.container.getWidget('menuConfiguration');
    };

    var getFieldAccessProfilesFromMenuConfigurationWidget = function () {
        return getMenuConfigurationWidget().getField('accessProfiles');
    };

    var getDashboardConfigurationWidget = function () {
        return templateManager.container.getWidget('dashboardConfiguration');
    };
    
    this.menuContainerAfterinit = function(){
        getMenuConfigurationWidget().sourceUrl = UtilitiesController.getUrlZmartNext("menuConfiguration");
        Util.doAsync(ScreenService.hideLoader, 2000);
    };


    this.menuConfigurationWidgetBeforeMoveRow = function () {
        UtilitiesController.deleteChangedFlag(getMenuConfigurationWidget());
    };

    this.menuConfigurationWidgetAfterMoveRow = function () {
        var widget = getMenuConfigurationWidget();
        UtilitiesController.clearChangedRows(widget);
        var parentMenuId = widget.currentRow.parent;

        if (parentMenuId) {
            ViewService.findView(parentMenuId).then(
                function (response) {
                    widget.currentRow.parentName = response.dataset.view[0].name;
                }
            );
        }
        setParentMenuFilter(widget);
    };

    var setParentMenuFilter = function (widget) {
        var menu_id = widget.currentRow._id;
        var parentMenuField = widget.getField('parentName');
        parentMenuField.dataSourceFilter = [
            {
                "name": "type",
                "operator": "=",
                "value": "M"
            }
        ];
        if (menu_id) {
            parentMenuField.dataSourceFilter.push({
                "name": "_id",
                "operator": "NOT_IN",
                "value": [menu_id]
            });
        }
        parentMenuField.reload();
    };


    this.menuConfigurationActionEventAdd = function () {
        var menuConfigurationWidget = getMenuConfigurationWidget();
        UtilitiesController.changeWidgetVisibility(menuConfigurationWidget.widgets);
        menuConfigurationWidget.newRow();
    };

    this.menuConfigurationActionEventDelete = function () {
        var menuConfigurationWidget = getMenuConfigurationWidget();
        return menuConfigurationWidget.removeCheckedRows().then(function () {
            ViewService.updateMenu();
        });
    };

    this.menuConfigurationActionEventEdit = function () {
        var menuConfigurationWidget = getMenuConfigurationWidget();
        menuConfigurationWidget.backupRow = Util.clone(menuConfigurationWidget.currentRow);
        UtilitiesController.changeWidgetVisibility(menuConfigurationWidget.widgets, false);
        menuConfigurationWidget.edit();
    };


    this.menuConfigurationActionEventCancel = function () {
        var menuConfigurationWidget = getMenuConfigurationWidget();
        if (menuConfigurationWidget.currentRow.__is_new) {
            menuConfigurationWidget.dataSource.data.pop();
            ScreenService.closeSwipe();
            menuConfigurationWidget.list();
        } else {
            Util.extend(menuConfigurationWidget.currentRow, menuConfigurationWidget.backupRow);
            menuConfigurationWidget.view();
        }
        UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getFieldAccessProfilesFromMenuConfigurationWidget());
        UtilitiesController.changeWidgetVisibility(menuConfigurationWidget.widgets, true);
    };

    this.menuConfigurationActionEventSave = function () {
        var menuConfigurationWidget = getMenuConfigurationWidget();
        return save(menuConfigurationWidget, function () {
            ViewService.updateMenu();
        });
    };

    this.menuConfigurationActionEventBack = function () {
        var widget = getMenuConfigurationWidget();
        UtilitiesController.masterDetailBackToList(widget).then(function () {
            UtilitiesController.deleteChangedFlag(widget);
        });
    };

    var save = function (widget, callback) {
        if (widget.isValid()) {
            var parentNameBkp = widget.currentRow.parentName;
            widget.currentRow.parentName = undefined;

            UtilitiesController.validatecolorField(widget, 'headerColor');
            UtilitiesController.validatecolorField(widget, 'headerGradient');

            return widget.saveRow().then(
                function (response) {
                    var dataSourceName = widget.dataSource.name.substr(1);
                    widget.currentRow._id = response.dataset[dataSourceName][0]._id;
                    widget.currentRow.changed = false;
                    widget.currentRow.__is_new = false;
                    widget.currentRow.parentName = parentNameBkp;
                    widget.view();
                    UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getFieldAccessProfilesFromMenuConfigurationWidget());
                    widget.getField('parentName').reload();
                    UtilitiesController.changeWidgetVisibility(widget.widgets, true);
                    if (callback) {
                        callback();
                    }
                },
                function (response) {
                    widget.currentRow.parentName = parentNameBkp;
                }
            );
        } else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
        }
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register('MenuConfigurationController', MenuConfigurationController);
});

// FILE: assets/js/controllers/PeriodObservationController.js
function PeriodObservationController(ScreenService, templateManager, restEngine, UtilitiesController) {

    var getWidgetObservations = function(){
        return templateManager.container.getWidget("observations");
    };
    
    this.periodObservationContainerAfterinit = function(){
        getWidgetObservations().sourceUrl = UtilitiesController.getUrlZmartNext("observacao-de-periodo");
        Util.doAsync(ScreenService.hideLoader, 2000);
    };

    this.observationActionEventAdd = function(){
        getWidgetObservations().newRow();
    };

    this.observationActionEventEdit = function(){
        getWidgetObservations().edit();
    };

    this.observationActionEventBack = function(){
        UtilitiesController.cancelEditRow(getWidgetObservations());
    };

    this.observationActionEventSave = function(){
        var widget = getWidgetObservations();
        if(widget.isValid()){
            widget.saveRow().then(function(){
                ScreenService.closeSwipe();
                widget.reload();
            });
        }
    };

    this.observationActionEventDelete = function(){
        var widget = getWidgetObservations();
        restEngine.post({
            'service': '/periodObservation/delete',
            'typeRequest': 'POST',
            'params': {
                'requestType': 'Row',
                'row': widget.getCheckedRows()
            }
        }).then(function() {
            widget.reload();
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('PeriodObservationController', PeriodObservationController);
});


// FILE: assets/js/controllers/PeriodsController.js
function PeriodsController(ScreenService, MasterDetailUtilities, templateManager, MainDashboardController,
PeriodsUtilities, MessageService, UtilitiesController) {

    var self = this;

    var getPeriodsWidget = function(){
        return templateManager.container.getWidget('periods');
    };

    this.periodsContainerAfterinit = function() {
        getPeriodsWidget().sourceUrl = UtilitiesController.getUrlZmartNext('periods');
        Util.doAsync(ScreenService.hideLoader, 2000);
    };

    //widget interactions
    this.back = function(widget){
        ScreenService.closeSwipe();
        widget.activate();
        widget.list();
    };

    this.backOrCancel = function(widget) {
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            widget.list();
            widget.currentRow = {};
            ScreenService.goBack();
        } else {
            widget.view();
            for (var i in widget.backupRow) {
                widget.currentRow[i] = widget.backupRow[i];
            }
        }
    };

    this.savePeriod = function(widget) {
        widget.currentRow.custom = true;
        if (widget.isValid()) {
            delete(widget.currentRow.Erro); 
            widget.saveRow().then(function() {
                MainDashboardController.forceDashboardReload();
                ScreenService.closeSwipe();
                widget.reload();
                widget.list();
            }, function (response) {
                UtilitiesController.showMessage(response.data.error);
            });
        }
    };

    this.editPeriod = function(widget) {
        var currentRow = widget.currentRow;

        if(currentRow.base){
            MessageService.alertNotificationByCode('NON_EDITABLE_PERIOD_MESSAGE');
        } else {
            MasterDetailUtilities.masterDetailEdit(widget);
        }
    };

	this.testPeriodCode = function(args){
        var widget = args.owner.widget;
	    var periodCode = widget.currentRow.periodsCode;
	    var startDateExpression = widget.currentRow.START_DATE_EXPRESSION;
	    var endDateExpression = widget.currentRow.END_DATE_EXPRESSION;
        var testMessage = '';
	    if(startDateExpression || endDateExpression || !periodCode){
            PeriodsUtilities.getDateByExpressions([startDateExpression,endDateExpression]).then(function (dates) {
                testMessage = ScreenService.i18n("From") + " " + dates[0] + " " + ScreenService.i18n("to") + " " + dates[1];
                UtilitiesController.showMessage(testMessage);
            }).catch(UtilitiesController.showMessageErrorOfBackEnd);
        }else{
            periodCode = PeriodsUtilities.conditionPeriod(periodCode,false);
            var interval = PeriodsUtilities.decodePeriod(periodCode);
            testMessage = ScreenService.i18n("From") + " " + interval.start + " " + ScreenService.i18n("to") + " " + interval.end;
            UtilitiesController.showMessage(testMessage);
        }
    };
    
    this.i18nBasePeriods = function(){
        var widget = getPeriodsWidget();
        widget.dataSource.data.forEach(function(period){
            if(period.base){
                period.name = ScreenService.i18n(period.name);
                if(period.labelOnView){
                    period.labelOnView = ScreenService.i18n(period.labelOnView);
                }
            }
        });
    };

	self.validateSelectRow = function(selectedRow){
	    if (selectedRow.base) {
			unselectRowsAndErrorMessage(selectedRow);
		}
	};

	self.validateSelectAllRows = function(selectedPeriods){
	    selectedPeriods.data.forEach(function(row) {
			if (row.base) {
				row.__isSelected = false;
			}
		});

		MessageService.alertNotificationByCode('NON_DELETABLE_PERIOD_MESSAGE');
	};

    function unselectRowsAndErrorMessage(selectedRow) {
		MessageService.alertNotificationByCode('NON_SELECTABLE_PERIOD_MESSAGE');
		selectedRow.__isSelected = false;
	}

}

Configuration(function(ContextRegister) {
    ContextRegister.register('PeriodsController', PeriodsController);
});

// FILE: assets/js/controllers/PeriodsListController.js
function PeriodsListController($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query,
							   CrosstabService, $filter, maskEngine, ZhUtil, $window, ApplicationContext) {
	// BEGIN: Initialize
 	var self = this;
	self.periods = {};
	var moment = $window.moment;
	var allowMultiSelect = null;
	$scope.filtered = [];

	this.__defineGetter__('widget', function() {
		return $scope.widget || $scope._currentPopup;
	});
	// END: Initialize

	// BEGIN: Utilities
	var CROSSTAB_TEMPLATE = 'zh-crosstab#widget/crosstab.html';
	var TREE_TEMPLATE = 'tree/grid.html';
	var TREE_LIST_TEMPLATE = 'tree/list.html';
	var GANTT_TEMPLATE = 'tree/gantt.html';

	var _isTemplate = function(template) {
		return self.widget.template && self.widget.template.indexOf(template) !== -1;
	};

	this.isTree = function() {
		return _isTemplate(TREE_TEMPLATE) || _isTemplate(TREE_LIST_TEMPLATE);
	};

	this.isGantt = function() {
		return _isTemplate(GANTT_TEMPLATE);
	};

	this.isCrosstab = function() {
		return _isTemplate(CROSSTAB_TEMPLATE);
	};
	// END: Utilities

	// BEGIN: Search
	$scope.searchList = this.widget.searchList;

	this.search = function(item) {
		self.widget.searchList = $scope.searchList;
		item.__notSelectable = false;
		if ($scope.searchList) {
			var returnValue = false;
			$.each(item, function(key, value) {
				var field = $.grep(self.widget.fields, function(field) {
					return field.name === key;
				})[0];
				if(field && field.translate){
					value = ScreenService.i18n(value);
				}
				returnValue = (!Util.isUndefinedOrNull(field) && field.isVisible && field.showOnList && !field.skipSearch &&
					!Util.isUndefinedOrNull(value) && value.toString().indexOfLatin($scope.searchList) !== -1);
				return !returnValue;
			});
			item.__notSelectable = !returnValue;
			return returnValue;
		} else {
			return true;
		}
	};

	function getLikeAllFilter(widget){
		var filter = [];

		if (!Util.isEmptyOrBlank($scope.searchList)) {
			var columns = widget.fields
			.filter(function(field){
				return field.isVisible && field.aclIsVisible && field.showOnList && !field.skipSearch;
			})
			.map(function(field){
				return field.name;
			})
			.join('|');
			filter = Query.build().where(columns).likeAll("%" + $scope.searchList + "%").where();
		}

		return filter;
	}

	var _searchWithPaginationReload = Util.buildDebounceMethod(function(widget, callback) {
		var deferred = ZHPromise.defer();
		if(Util.isArray(self.widget.fields) && self.widget.fields.length) {
			widget.reload(getLikeAllFilter(self.widget)).then(function(data) {
				if (callback) callback();
				deferred.resolve(data);
			}, function() {
				deferred.reject();
			});
		} else {
			deferred.reject();
		}
		return deferred;
	}, 1000);

	var _resizeGrid = Util.buildDebounceMethod(function() {
		self.widget.resizeGridColumns();
	}, 1000);

	if (!self.isGantt() && !self.isCrosstab()) {
		$scope.$watch('searchList', function(current, old) {
			if (self.widget.searchWithPagination && !Util.isUndefined(current)) {
				_searchWithPaginationReload(self.widget, function() {
					updateGroupedData();					
				});
			}
			if (!Util.isUndefined(current) && current != old && self.widget.resizeGridColumns) { // new grid
				_resizeGrid();
			}
		});
	}
	// END: Search

	// BEGIN: Toggle
	function deselectAll(widget) {
		var event = widget.unSelectAll();
		if (!event || !event.isDefaultPrevented()) {
			widget.dataSource.clearCheckedRows();
			if (!$scope._currentPopup) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}
		}
	}

	function selectAll(widget) {
		widget.dataSource.data.filter(self.search).forEach(function(row) {
			row.__isSelected = true;
			widget.dataSource.addCheckedRows(row);
		});
		if (!$scope._currentPopup) {
			$rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
		}
	}

	$rootScope.$on("ui.deselectAll", function(event, widget) {
		deselectAll(widget);
	});

	this.onLongTap = function(row) {
		if (row.__group) return;

		if (this.widget.allowSelectOnLongTap) {
			this.toggle(row);
		}
	};


	this.toggle = function(row) {
		var widget = self.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}

		if (!event || !event.isDefaultPrevented()) {

			var index = getIndex(row, $scope);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}

			if (self.isTree()) {
				self.handleDependentRows(row, widget);
			}

			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}

		}
	};

	$scope.widget.toggle = self.toggle;

	this.toggleAll = function() {
		if (this.widget.hasCheckedRows()) {
			this.widget.unSelectAll();
		} else {
			this.widget.selectAll();
		}
		this.widget.updateCheckedRows();
	};

	this.widget.toggle = this.toggle;
	// END: Toggle

	// BEGIN: Others functions
	$scope.openRow = function(row, widget) {
		if (row.__group) return;

		if (widget.dataSource.getCheckedRows().length || allowMultiSelect === widget || widget.allowMultiSelect) {
			self.toggle(row);
		} else {
			widget.setCurrentRow(row);
		}
		widget.dataSource.updateCheckedRows();
	};

	$scope.openRowInnerList = function(row, widget) {
		if (typeof(widget.currentRow) !== "undefined" &&
			widget.currentRow.__index === widget.dataSource.data.indexOf(row)) {
			delete widget.currentRow;
		} else {
			$scope.openRow(row, widget);
		}
	};

	function getIndex(row, scope) {
		return self.widget.dataSource.checkedRows.indexOf(row);
	}

	$scope.scrollTo = function(evt) {
		$timeout(function() {
			var item = $(evt.target).closest('li');
			var itemTop = item.position().top;
			var itemBottom = itemTop + item[0].clientHeight;
			var scroller = item.closest("[zh-scroller]");
			var top = 0;
			if (itemTop < 0 || itemBottom > scroller.height()) {
				if (itemTop < 0) {
					top = scroller.scrollTop() + itemTop - 10;
				} else {
					top = scroller.scrollTop() + itemBottom - scroller.height() + 10;
				}
				scroller.animate({
					scrollTop: top
				}, 500);
			}
		}, 100, true);
	};

	$rootScope.$on("ui.allowMultiSelect", function(event, widget) {
		if ($scope._currentPopup) {
			return;
		}
		allowMultiSelect = widget;
	});

	$rootScope.$on("ui.disableMultiSelect", function() {
		if ($scope._currentPopup) {
			return;
		}
		allowMultiSelect = null;
	});
	// END: Others functions

	// BEGIN: Conditional actions control

	var hasConditionalActions = false;
	var leftActions = [];
	var rightActionsGrid = [];
	var rightActionsList = [];
	var rightActions = [];
	var defaultActionsValue = function(){
		$scope.visibleLeftActions = leftActions.length;
		$scope.visibleRightActions = rightActions.length;
		$scope.leftActions = leftActions;
		$scope.rightActions = rightActions;
	};
	var buildGridActions = function(){
		hasConditionalActions = false;
		leftActions = $scope.widget.getActionsByVisibility('grid-left');
		rightActionsGrid = $scope.widget.getActionsByVisibility('grid-right');
		rightActionsList = $scope.widget.getActionsByVisibility('list-right');
		rightActions = rightActionsGrid.concat(rightActionsList);		
	
		$scope.widget.actions.forEach(function(item) {
			if (item.condition) {
				hasConditionalActions = true;
			}
		});

		if(!hasConditionalActions){
			defaultActionsValue();	
		}
	};  
	$scope.buildGridActions = buildGridActions;
	buildGridActions();
    
	$scope.$watch("widget.dataSource.data", function(data) {

		if(hasConditionalActions){
			
			$scope.visibleLeftActions = 0;
			$scope.visibleRightActions = 0;

			if ($scope.widget && $scope.widget.dataSource && $scope.widget.dataSource.data) {
				$scope.widget.dataSource.data.forEach(function(row) {
					ApplicationContext.registerInstance('row', row || {});
					row.__rightActions = rightActions.filter(function(action) {
						return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
					});

					row.__leftActions = leftActions.filter(function(action) {
						return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
					});
					$scope.visibleRightActions = Math.max($scope.visibleRightActions, row.__rightActions.length);
					$scope.visibleLeftActions = Math.max($scope.visibleLeftActions, row.__leftActions.length);
					ApplicationContext.removeInstance('row');
				});
			}	
		}
	});


	// END: Conditional actions control

	// BEGIN: Order by
	this.changeOrder = function(column) {
		if (!!column.disableOrdering) return;

		if (self.widget.order == column.name && self.widget.reverse) {
			self.widget.order = undefined;
			self.widget.reverse = false;
		} else {
			self.widget.reverse = (self.widget.order == column.name) ? !self.widget.reverse : false;
			self.widget.order = column.name;
		}
		updateGroupedData();
	};

	var orderByColumns;
	var orderByFieldFlow = {
		"datetime.html": getDatetimeProxyToOrderBy,
		"calendar.html": getDateProxyToOrderBy,
		"select-month.html": getMonthProxyToOrderBy
	};

	function getDateTimeFormat(field) {
		var format;
		var dateFormat = field['datetime-dateFormat'] || Util.DATE_FORMAT;
		var timeFormat = field['datetime-timeFormat'] || Util.TIME_FORMAT;
		switch (field['datetime-preset']) {
			case 'date':
			case 'month':
				format = dateFormat;
				break;
			case 'time':
				format = timeFormat;
				break;
			case 'datetime':
			case 'datetime-local':
				format = dateFormat + ' ' + timeFormat;
				break;
			default:
				format = dateFormat + ' ' + timeFormat;
				break;
		}
		return format;
	}

	function getDatetimeProxyToOrderBy(columnName, field) {
		var format = getDateTimeFormat(field);
		return getProxyToOrderBy(columnName, format);
	}

	function getDateProxyToOrderBy(columnName, field) {
		return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
	}

	function getMonthProxyToOrderBy(columnName, field) {
		return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
	}

	function getProxyToOrderBy(columnName, format) {
		var proxyProperty = getProxyProperty(columnName);
		buildDateProxyColumn(proxyProperty, columnName, format);
		return proxyProperty;
	}

	function getProxyProperty(columnName) {
		return "__proxy_" + columnName;
	}

	function buildDateProxyColumn(proxyProperty, sourceColumn, format) {
		if ($scope.widget.dataSource && Util.isArray($scope.widget.dataSource.data)) {
			$scope.widget.dataSource.data.forEach(function(item){
				item[proxyProperty] = moment(item[sourceColumn], format)._d;
			});
		}
	}

	function addColumnToOrder(columnName, reverse) {
		var field = $scope.widget.fields.filter(function(field){
			return field.name === columnName;
		}).shift();

		if (field) {
			var template = field.getTemplateSimpleName();
			if (orderByFieldFlow[template]) {
				columnName = orderByFieldFlow[template](columnName, field);
			}
		}

		orderByColumns.push((reverse ? '-' : '') + columnName);
	}

	function getOrderByColumns() {
		orderByColumns = [];

		if ($scope.widget.groups) {
			$scope.widget.groups.forEach(function(group, index) {
				addColumnToOrder(group.name);
			});
		}

		if ($scope.widget.order && !Util.isFunction($scope.widget.order)) {
			Util.parseToArray($scope.widget.order).forEach(function(collum){
				addColumnToOrder(collum, $scope.widget.reverse);
			});
		}
	}
	// END: Order by

	// BEGIN: Group and summarization
	$scope.fieldOrder = function(field) {
		return (field.fixed ? '0' : '1') + ('000'+field.sequenceInList).substr(-4);
	};

	$scope.widget.groupedData = [];

	// initialize widget.groups
	updateWidgetGroups();
	$scope.$on("updateWidgetGroups", function(){
		updateWidgetGroups();
	});

	function updateWidgetGroups() {
		$scope.widget.groups = $scope.widget.fields.filter(function(item) {
			if (item.grouped) item.isVisible = false;
			return item.aclIsVisible && item.grouped;
		}).sort(function(a, b) { return a.sequenceInList - b.sequenceInList; });
	}

	var oldItemsPerPage = $scope.widget.itemsPerPage;
	var oldHasPagination = $scope.widget.hasPagination;

	$scope.$watchCollection('widget.groups', function(newValue, oldValue) {
		if (newValue != oldValue) {
			if (newValue && newValue.length >= 1) {
				$scope.widget.itemsPerPage = 0;
				$scope.widget.hasPagination = false;
			} else {
				$scope.widget.itemsPerPage = oldItemsPerPage;
				$scope.widget.hasPagination = oldHasPagination;
			}
			$scope.widget._gridResize = true; // forces the grid element to recalculate column widths
			$scope.widget.reload();
		}
	});

	$scope.$watchCollection('widget.dataSource.data', function() {
		updateGroupedData();
		self.periods = {};
		updatePeriods();
	});

	var groups = [];
	var summaryFields = [];
	var summary = {};

	function resetFooterVariables(group) {
		group.footer.count = 0;
		summaryFields.forEach(function(field) {
			group.footer[field.name] = {};
			group.footer[field.name].sum = 0;
			group.footer[field.name].count = 0;
		});
	}

	function addGroupHeader(group, index) {
		var groupHeaderRow = {
			__group: true,
			__groupHeader: true,
			__groupIndex: index,
			__groupLabel: ScreenService.i18n(group.label),
			__groupValue: !group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil),
			__groupOpened: true,
			__groupHeaders: []
		};
		// add header for outer groups to the added row
		for (var i = 0; i < index; i++) {
			groupHeaderRow.__groupHeaders.push(groups[i].lastGroupHeaderRow);
		}
		group.lastGroupHeaderRow = groupHeaderRow;
		$scope.widget.groupedData.push(groupHeaderRow);
	}

	function summaryData(group, row, isGroup) {
		var field, value, lastValue, _group;
		for (var i = 0; i < summaryFields.length; i++) {
			field = summaryFields[i];
			if (isGroup) {
				lastValue = ' (' + (!group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil)) + ')';
				_group = group.footer;
			} else {
				lastValue = ' (' + ScreenService.i18n('TOTAL') + ')';
				_group = group;
			}

			if (field.expression == 'COUNT') {
				row[field.name] = ScreenService.i18n('Count') + lastValue + ': ' + _group[field.name].count;
			} else if (field.expression == 'SUM') {
				value = _group[field.name].sum;
				row[field.name] = ScreenService.i18n('Sum') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
			} else if (field.expression == 'AVG') {
				value = _group[field.name].sum / _group[field.name].count;
				row[field.name] = ScreenService.i18n('Average') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
			}
		}
	}

	function addGroupFooters(groupIndex) {
		if (summaryFields.length === 0) return;

		for (var i = groups.length - 1; i >= groupIndex; i--) {
			if (groups[i].footer.count > 0) {
				var groupFooterRow = {
					__group: true,
					__groupFooter: true,
					__groupIndex: i,
					__groupHeaders: []
				};

				// add header for outer groups
				for (var g = 0; g < i; g++) {
					groupFooterRow.__groupHeaders.push(groups[g].lastGroupHeaderRow);
				}

				summaryData(groups[i], groupFooterRow, true);

				$scope.widget.groupedData.push(groupFooterRow);
			}
		}
	}

	function addSummary() {
		if (summaryFields.length === 0 || groups.length === 0) return;

		var summaryRow = {
			__group: true,
			__groupFooter: true,
			__groupSummary: true
		};

		summaryData(summary, summaryRow);

		$scope.widget.groupedData.push(summaryRow);
	}

	function calcSummary(row) {
		groups.forEach(function(group) {
			group.footer.count += 1;
			if (summaryFields.length > 0) {
				summaryFields.forEach(function(field) {
					if (!Util.isUndefined(row[field.name])) {
						group.footer[field.name].count += 1;
					}
					if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
						group.footer[field.name].sum += parseFloat(row[field.name]);
					}
				});
			}
		});

		summaryFields.forEach(function(field) {
			if (Util.isUndefined(summary[field.name])) {
				summary[field.name] = {
					count: 0,
					sum: 0
				};
			}

			if (!Util.isUndefined(row[field.name])) {
				summary[field.name].count += 1;
			}
			if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
				summary[field.name].sum += parseFloat(row[field.name]);
			}
		});
	}

	function initSummaryFields() {
		summaryFields = $scope.widget.fields.filter(function(field) {
			return !!field.expression && !field.grouped;
		});

		// reset summary variable
		summary = {};
	}

	function initGroups() {
		groups = [];
		$scope.widget.groups.forEach(function(group, index) {
			var newGroup = {
				field: group,
				name: group.name,
				label: group.label,
				lastValue: undefined,
				footer: {}
			};
			resetFooterVariables(newGroup);

			groups.push(newGroup);
		});
	}

	function getOrderedDataByColumns(dataSet, orderByColumns) {
		return orderByColumns.length ? $filter('orderBy')(dataSet, orderByColumns) : dataSet;
	}

	function updateGroupedData() {
		if (Util.isUndefined($scope.widget.dataSource) || Util.isEmptyOrBlank($scope.widget.dataSource.data)) {
			$scope.widget.groupedData = [];
		} else {
			getOrderByColumns();

			if (Util.isUndefined($scope.widget.groups) || $scope.widget.groups.length === 0) {
				$scope.widget.groupedData = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
			} else {
				$scope.widget.groupedData = [];

				initSummaryFields();
				initGroups();

				var dataAux = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
				dataAux.forEach(function(row, index) {
					var groupBreak = false;
					groups.forEach(function(group, index) {
						if (group.lastValue != row[group.name] || groupBreak) {
							if (!groupBreak) addGroupFooters(index);

							group.lastValue = row[group.name];
							resetFooterVariables(group);

							addGroupHeader(group, index);
							groupBreak = true;
						}
					});

					row.__groupHeaders = groups.map(function(group) { return group.lastGroupHeaderRow; });
					$scope.widget.groupedData.push(row);

					calcSummary(row);
				});

				if ($scope.widget.groupedData.length > 0) {
					addGroupFooters(0);
					addSummary();
				}
			}
		}

		if (self._updateVisibleRows) {
			self._updateVisibleRows();
		}
	}

	this.toggleGroup = function(row) {
		row.__groupOpened = !row.__groupOpened;

		if (self._updateVisibleRows) {
			self._updateVisibleRows();
		}
	};

	this.getClosedStatus = function(row) {
		var status = false;
		if (row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function(group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};
	
	function updatePeriods() {
		$scope.widget.fields.forEach(function (field) {
			$scope.widget.fieldGroups.forEach(function (group) {
				$scope.widget.groupedData.forEach(function (row) {
					self.getPeriodsFromField(field, row, group);
				});
			});
		});
		$scope.widget.groupedData.sort(function (firstRow, secondRow) {
			if (firstRow.__isTotalRow) {
				return 1;
			} else if(secondRow.__isTotalRow) {
				return -1;
			} else {
				return secondRow.__periodPercentRawValue - firstRow.__periodPercentRawValue;
			}
		});
	}
	
	this.getPeriodsFromField = function (field, row, group) {
		var filteredKeys = Object.keys(row).filter(function (key) {
			return ~key.indexOf(field.name);
		});
		filteredKeys = filteredKeys.map(function (key) {
			var keySplit = key.split('_');
			var toDate = keySplit.slice(-1).shift();
			var fromDate = keySplit.slice(-2, -1).shift();
			toDate = moment(toDate, 'L');
			fromDate = moment(fromDate, 'L');
			return {
				name: keySplit[0],
				untouchedName: key,
				from: fromDate,
				to: toDate
			};
		});
		
		filteredKeys.forEach(function (key) {
			setPeriodLabelAndOrder(key);
		});
		
		filteredKeys.sort(function (firstValue, secondValue) {
			return firstValue.order - secondValue.order;
		});
		
		var periods = [];
		
		for (var i = 0; i < filteredKeys.length; i++) {
			var key = filteredKeys[i];
			var rawPercent;

			var period = getFieldCopy(field);
			
			period.name = key.untouchedName;
			period.label = key.label;
			
			var lastPeriod;
			var lastPeriodValue;
			var thisPeriodValue;
			var percent;
			
			if (i === 0) {
				row[period.name + "_showBadge"] = false;
			} else if (i === 1) {
				lastPeriod = periods[0];
				lastPeriodValue = row[lastPeriod.name];
				thisPeriodValue = row[period.name];
				percent = Math.floor(lastPeriodValue) === 0? 100 : (((lastPeriodValue * 100) / thisPeriodValue) - 100);
				rawPercent = percent;
				
				row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
				row[period.name + "_isLowerThanBaseValue"] = percent < 0;
				row[period.name + "_equalsBaseValue"] = percent === 0;
				row[period.name + "_showBadge"] = true;
				
				percent = Math.abs(percent);
				
				percent = percent.toFixed(1);
				row[period.name + "_percent"] = percent;
			} else {
				lastPeriod = periods[0];
				lastPeriodValue = row[lastPeriod.name];
				thisPeriodValue = row[period.name];
				percent = Math.floor(lastPeriodValue) === 0? 100 : (((lastPeriodValue * 100) / thisPeriodValue) - 100);
				rawPercent = percent;

				row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
				row[period.name + "_isLowerThanBaseValue"] = percent < 0;
				row[period.name + "_equalsBaseValue"] = percent === 0;
				row[period.name + "_showBadge"] = true;
				
				percent = Math.abs(percent);
				
				percent = percent.toFixed(1);
				row[period.name + "_percent"] = percent;
			}
			
			periods.push(period);
			if (row[group.name] === "Total") {
				row.__isTotalRow = true;
			}

			if (i === filteredKeys.length -1) {
				row.__periodPercentRawValue = rawPercent;
			}
		}
		
		self.periods[row[group.name]] = periods;
		return periods;
	};
	
	function setPeriodLabelAndOrder(key) {
		var thisMonth = moment();
		var thisMonthMonth = thisMonth.month();
		var thisMonthYear = thisMonth.year();
		
		var lastMonth = thisMonth.subtract(1, 'month');
		var lastMonthMonth = lastMonth.month();
		var lastMonthYear = lastMonth.year();
		
		
		if (key.from.year() === thisMonthYear && key.from.month() === thisMonthMonth && 
			key.to.year() === thisMonthYear && key.to.month() === thisMonthMonth) {
			key.label = 'Mês atual';
			key.order = 0;
		} else if (key.from.year() === lastMonthYear && key.from.month() === lastMonthMonth &&
			key.to.year() === lastMonthYear && key.to.month() === lastMonthMonth) {
			key.label = 'Mês anterior';
			key.order = 1;
		} else {
			key.label = 'Ano anterior';
			key.order = 2;
		}
	}
	
	function getFieldCopy(field) {
		return {
			name: field.name,
			label: field.label,
			aclIsVisible: field.aclIsVisible,
			aclReadOnly: field.aclReadOnly,
			align: field.align,
			class: field.class,
			direction: field.direction,
			events: field.events,
			function: field.function,
			idGenerated: field.idGenerated,
			isDate: field.isDate,
			isFact: field.isFact,
			isGenerated: field.isGenerated,
			isKey: field.isKey,
			isSortable: field.isSortable,
			isVisible: field.isVisible,
			mask: field.mask,
			readOnly: field.readOnly,
			readOnlyOnEdit: field.readOnlyOnEdit,
			sequenceInList: field.sequenceInList,
			showOnForm: field.showOnForm,
			showOnList: field.showOnList,
			showOnView: field.showOnView,
			showSeparatorButton: field.showSeparatorButton,
			showSignalButton: field.showSignalButton,
			translate: field.translate
		};
	}
}

// FILE: assets/js/controllers/QueryRunnerController.js
function QueryRunnerController(templateManager, QueryRunnerService, metaDataFactory, ScreenService,
MessageService,UtilitiesController) {

    var self = this;
    var getQueryRunner = function() {
        return templateManager.container.getWidget('queryRunner');
    };

    this.queryRunnerContainerAfterinit = function() {
        getQueryRunner().sourceUrl = UtilitiesController.getUrlZmartNext("queryRunner");
    };


    var getWidgetQueryRunner = function() {
        return getContainer().getWidget('queryRunner');
    };

    var getWidgetResults = function() {
        return  getContainer().getWidget('results');
    };

    var getWidgetImportHistory = function() {
        return getContainer().getWidget('importHistory');
    };

    var getFieldQueryFromQueryRunnerWidget = function(){
        return getWidgetQueryRunner().getField('query');
    };

    var getFieldStatusFromQueryRunnerWidget = function(){
        return getWidgetQueryRunner().getField('status');
    };

    var getFieldConnectionNameFromQueryRunnerWidget = function(){
        return getWidgetQueryRunner().getField('connectionName');
    };

    this.importHistoryActionViewResult = function (args) {
        var resultWidget = getWidgetResults();
        var widget = getWidgetQueryRunner();
        var historyRow = args.row;
        resultWidget.modeExecuteQueyZmart = 'normal';

        resultWidget.fields = [];
        resultWidget.dataSource.data = [];
        var keys = [];
        historyRow.queryKeys.forEach(function (row) {
            self.createField(resultWidget, row);
            keys.push(row);
        });
        resultWidget.redraw();
        widget.currentRow.keys = keys;
        resultWidget.dataSourceFilter[0].value = historyRow.process;
        resultWidget.reload();
        widget.currentRow.query = historyRow.query;

        widget.currentRow.connectionId = historyRow.connectionId;
        UtilitiesController.setValueInSelectField(getFieldConnectionNameFromQueryRunnerWidget(),historyRow.connectionId);
        ScreenService.closeSwipe();
    };

    this.importHistoryActionSetQuery = function (args) {
        var resultWidget = getWidgetResults();
        var widget = getWidgetQueryRunner();
        var historyRow = args.row;
        resultWidget.modeExecuteQueyZmart = 'normal';

        widget.currentRow.query = historyRow.query;
        widget.currentRow.connectionId = historyRow.connectionId;
        UtilitiesController.setValueInSelectField(getFieldConnectionNameFromQueryRunnerWidget(),historyRow.connectionId);
        ScreenService.closeSwipe();
        UtilitiesController.confirmMessageByCode("EXECUTE_QUERY", {},{
                "type": 'CONFIRMATION',
                "success": function (){
                    self.executeQueryMining(widget);
                },
                "fail": function(){

                }
            });
    };

    this.queryRunnerFieldOnClickStatusHistory = function (args) {
        var widget = getWidgetImportHistory();
        widget.formTemplate = widget.template;
        widget.reload();

        ScreenService.openSwipe(widget);
    };

    this.modeSwitch = function () {
        var resultWidget = templateManager.container.getWidget('results');
        if(resultWidget.modeExecuteQueyZmart === 'fast'){
            resultWidget.modeExecuteQueyZmart = 'normal';
            ScreenService.successNotification('MODE_QUERY_RUNNER_NORMAL');
        } else {
            resultWidget.modeExecuteQueyZmart = 'fast';
            ScreenService.successNotification('MODE_QUERY_RUNNER_FAST');
        }
    };

    this.execute = function(widget){
        var resultWidget = templateManager.container.getWidget('results');
        if(resultWidget.modeExecuteQueyZmart !== 'fast'){
            self.executeQueryMining(widget);
        }else{
            self.executeQuery(widget);
        }
    };

    this.executeQueryMining = function (widget) {
        var resultWidget = templateManager.container.getWidget('results');
        var query = widget.currentRow.query;
        var driverClass = widget.currentRow.driverClass;

        if (widget.isValid()) {
            if (validateQueryType(query,driverClass)) {
                QueryRunnerService.dropCollectionQueryRunner()
               .then(function () {
                    return QueryRunnerService.getColumnsMetaData(widget.currentRow);
                }).then(function (response) {
                    resultWidget.fields = [];
                    resultWidget.dataSource.data = [];
                    var keys = [];
                    response.dataset.getColumnsMetaData.forEach(function (row) {
                        self.createField(resultWidget, row.name);
                        keys.push(row.name);
                    });
                    resultWidget.redraw();
                    widget.currentRow.keys = keys;
                    return QueryRunnerService.runQueryMining(widget.currentRow);
                }).then(function (response) {
                    var process = response.dataset.queryResult[0].PROCESS;
                    resultWidget.dataSourceFilter[0].value = process;
                    widget.currentRow.process = process;
                }).catch(UtilitiesController.showMessageErrorOfBackEnd);

            } else {
                MessageService.alertNotificationByCode('ALLOWED_ONLY_QUERIES_SELECT');
            }
        } else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
        }
    };

    this.executeQuery = function(widget) {

        var resultWidget = templateManager.container.getWidget('results');
        var query = widget.currentRow.query;
        var driverClass = widget.currentRow.driverClass;

        this.clearResults(resultWidget);

        if (widget.isValid()) {
            if (validateQueryType(query,driverClass)) {
                QueryRunnerService.runQuery(widget.currentRow).then(function(response) {
                    var queryResults = response.dataset.queryResult.filter(function(result){
                        return !Util.isArray(result);
                    });
                    var allKeys = [];
                    queryResults.forEach(function(result){
                        Object.keys(result).forEach(function(key){
                            if(!allKeys.includes(key)){
                                allKeys.push(key);
                            }
                        });
                    });
                    for (var object in allKeys) {
                        self.createField(resultWidget, allKeys[object]);
                    }
                    queryResults.forEach(function(row) {
                        resultWidget.dataSource.data = queryResults;
                        resultWidget.redraw();
                    });
                },UtilitiesController.showMessageErrorOfBackEnd);
            } else {
                MessageService.alertNotificationByCode('ALLOWED_ONLY_QUERIES_SELECT');
            }
        }
        else {
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
        }
        resultWidget.isVisible = true;
    };

    function validateQueryType(query,driverClass){
        var queryType = query.split(' ')[0];
        queryType = queryType.toUpperCase();

        if (queryType === 'SELECT' || driverClass == 'Spreadsheet'){
            return true;
        }
    }
    var getContainer = function(){
        return templateManager.container;
    };

    self.queryRunnerFieldOnChangeConnectionName = function(args){
        var widgetQueryRunner = getWidgetQueryRunner();
        var fieldQuery = getFieldQueryFromQueryRunnerWidget();
        if(widgetQueryRunner.currentRow.driverClass == "Spreadsheet"){
            fieldQuery.label = fieldQuery.labelSpreadsheet;
        }else{
            fieldQuery.label = fieldQuery.labelOrigen;
        }
    };

    this.createField = function(widget, name) {
        var field = {
            'name': ScreenService.i18n(name),
            'label': name,
            'showOnForm': true,
            'showOnView': true,
            'showOnList': true,
            'readOnly': true,
            'template': 'field/text-edit.html'
        };

        metaDataFactory.factoryField(field, widget, true);
    };

    this.clearResults = function(widget) {
        widget.fields = [];
        widget.dataSource.data = [];
        widget.isVisible = false;
        templateManager.container.getWidget('queryRunner').activate();
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('QueryRunnerController', QueryRunnerController);
});


// FILE: assets/js/controllers/ReleaseNotes.js
function ReleaseNotesController(templateManager, localStorageEngine, ReleaseNotesService,
ScreenService, UtilitiesController, MessageService, MasterDetailUtilities) {

    var self = this;
    var WIDGET_NAME = 'releaseNotes';

    function getReleaseNotesWidget() {
        return templateManager.container.getWidget(self.WIDGET_NAME);
    }

    this.widgetOnEnter = function() {
        validateZmartAdminUser();
    };

    function validateZmartAdminUser() {
        var loggedUser = localStorageEngine.getLocalVar('login');
        ReleaseNotesService.getZmartAdminUsers(loggedUser).then(function(response){
            var zmartAdminUsers = response.dataset.zmartAdminUsers;
            var isUserZmartAdmin = zmartAdminUsers.filter(function(adminUser){
                return adminUser.userMail === loggedUser;
            }).length > 0;

            if(!isUserZmartAdmin) {
                ScreenService.openWindow('mainDashboard');
            }
        });
    }

    this.newRow = function() {
        var widget = getReleaseNotesWidget();
        widget.newRow();
    };

    this.deleteRows = function() {
        var widget = getReleaseNotesWidget();
        widget.removeCheckedRows();
    };

    this.cancelEdit = function() {
        var widget = getReleaseNotesWidget();
        UtilitiesController.cancelEditRow(widget);
    };

    this.goBack = function() {
        var widget = getReleaseNotesWidget();
        UtilitiesController.goBackWidget(widget);
    };

    this.saveRow = function() {
        var widget = getReleaseNotesWidget();
        if(validateDate(widget.currentRow.releaseDate)) {
            prepareWidgetToSave(widget);
            UtilitiesController.saveRow(widget);
        } else {
			MessageService.showMessageByCode("REQUIRED_FIELDS");
        }
    };

    function prepareWidgetToSave(widget) {
        var releaseDate = '<div><span style="font-size: 14px;"><font face="Arial">' + widget.currentRow.releaseDate;
        var releaseVersion = ' - Versão ' + widget.currentRow.version + '</font></span><br></div><br>';
        var releaseAuthor = '<br><span style="font-size: 14px;"><font face="Arial">Author: ' + widget.currentRow.createdBy +'<br></font></span><br></div>';
        var releaseNotes = releaseDate + releaseVersion + widget.currentRow.releaseNotes + releaseAuthor;
        widget.currentRow.log = releaseNotes;
    }

    this.editRow = function() {
        var widget = getReleaseNotesWidget();
        MasterDetailUtilities.masterDetailEdit(widget);
    };

    function validateDate(date) {
        var re = /^\s*(3[01]|[12][0-9]|0?[1-9])\/(1[012]|0?[1-9])\/((?:19|20)\d{2})\s*$/;
        return date.match(re);
    }

    this.loadReleaseNotes = function() {
        var whatsNewWidget = templateManager.container.getWidget('whatsNew');
        var whatsNew = "";
        ReleaseNotesService.getReleaseNotes().then(function(response){
            var releaseNotes = response.dataset.releaseNotes;
            releaseNotes.forEach(function(releaseNote){
                whatsNew = whatsNew + releaseNote.log + "<hr><br>";
            });

            whatsNewWidget.currentRow.releaseNotes = whatsNew;
        });
        whatsNewWidget.view();
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ReleaseNotesController', ReleaseNotesController);
});

// FILE: assets/js/controllers/SignUpController.js
function SignUpController(ScreenService, UtilitiesController, SignUpService, MessageService) {

	var USER_LOGIN_WIDGET = "identityLibraries#login";

	this.cancelSignUp = function(widget){
		widget.currentRow = {};
		ScreenService.openWindow(USER_LOGIN_WIDGET);
	};

	this.registerNewUser = function(widget){
		var row = widget.currentRow;
		if(widget.isValid()){
            if(UtilitiesController.validateEmailFormat(row.mail)){
                if(row.password === row.passwordConfirmation){
                	var firstName = row.userName.split(" ")[0];
                	var lastName = row.userName.split(' ').slice(1).join(' ');

                	var userInfo = {
                		'ORGANIZATION_NAME' : row.organizationName,
                		'NAME': row.userName,
                		'FIRSTNAME': firstName,
                		'LASTNAME': lastName,
                		'EMAIL': row.mail,
                		'PASSWORD': row.password,
                		'IS_ACTIVE': 'Y',
                		'SAAS': row.saasConnection
                	};

                	SignUpService.registerNewUser(userInfo);
                } else {
                    MessageService.errorNotificationByCode('PASSWORDS_DOESNT_MATCH');
                }
            } else {
                MessageService.alertNotificationByCode('ENTER_VALID_EMAIL');
            }
        } else{
            MessageService.alertNotificationByCode('REQUIRED_FIELDS');
        }
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('SignUpController', SignUpController);
});

// FILE: assets/js/controllers/StructureController.js
function StructureController(ScreenService, ApplicationContext, templateManager, StructureService, UtilitiesController, MessageService, restEngine) {

	var self = this;
	var fieldsNormalEdit = ['name', 'structureParentName', 'structureChildName', 'typeName'];
	var fieldsAddTooMany = ['structureParentName', 'structureChilds', 'typeName'];
	var fieldsToUnsetValue = ['typeParentName', 'typeChildName', 'parentName'];

	// Flags to control reload
	var needLoad = {
		"structure_types": true,
		"hierarchy_types": true,
		"structure": true,
		"hierarchy": true
	};

	var getWidgetStructureTypes = function(){
		return templateManager.container.getWidget("structure_types");
	};

	var getWidgetHierarchyTypes = function(){
		return templateManager.container.getWidget("hierarchy_types");
	};

	var getFieldTypeParentNameFromHierarchyTypesWidget = function(){
		return getWidgetHierarchyTypes().getField("typeParentName");
	};
	
	var getFieldTypeChildNameFromHierarchyTypesWidget = function(){
		return getWidgetHierarchyTypes().getField("typeChildName");
	};
	
	var getFieldParentNameFromHierarchyTypesWidget = function(){
		return getWidgetHierarchyTypes().getField("parentName");
	};

    this.containerBeforeInitStructure = function() {
        ApplicationContext.engine.initSessionAcl();
    };

    this.hierarchy_typesWidgetAfterMoveRow = function (args){
        args.owner.activate();
        this.setChildFilter(args.owner);
        this.swipeCloserAfterMoveRow();
        this.loadFilterForHierarchySelects(args.owner);
    };
    self.structureTypesWidgetActionEventDeleteAll = function () {
		var messageConfig = {
			"type": 'CONFIRMATION',
			"success": function (){
				restEngine.post("/DeleteAllStructure",
					{
						"requestType": "FilterData",
						"filter": []
					}
				).then(function () {
					getWidgetStructureTypes().reload();
					getWidgetHierarchyTypes().reload();
				});
			}
		};
		UtilitiesController.confirmMessageByCode("TYPE_STRUCTURES_DELETE_ALL", null, messageConfig);
	};

	this.structureWidgetAfterMoveRow = function(){
		setLocalVar('structureId',getStructureWidget().currentRow._id);
	};

    this.structure_typesWidgetAfterMoveRow = function(args){
		args.owner.activate();
		setLocalVar('structureTypeId',getWidgetStructureTypes().currentRow._id);
        this.setChildFilter(args.owner);
        this.fillFieldKey(args.owner);
        this.loadFieldKeysFromQuery(args.owner);
		this.swipeCloserAfterMoveRow();
		this.loadFilterForHierarchySelects(args.owner);
    };

    var getStructureWidget = function(){
        return templateManager.container.getWidget('structure');
    };

    var getUserResponsibleFieldFromStructureWidget = function(){
        return getStructureWidget().getField('usersResponsible');
    };

    var getAccessProfilesFieldFromStructureWidget = function(){
        return getStructureWidget().getField('accessProfiles');
    };

    //General
	var changeWidgetVisibility = function(widgets, isVisible) {
		widgets.forEach(function(widget) {
			if(!widget.keepVisibility){
				widget.isVisible = isVisible;
			}
		});
	};

	var setFieldVisibility = function(fieldsNames, widget, visibility) {
		fieldsNames.forEach(
			function(element) {
				widget.getField(element).showOnForm = visibility;
			}
		);
	};

	self.loadOnEnterOnce = function(widget) {
		if (needLoad[widget.name]) {
			widget.reload();
			needLoad[widget.name] = false;
		}
	};

    self.structureWidgetAfterLoadDataSource = function() {
		var fieldOperator = getUserResponsibleFieldFromStructureWidget();
        UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(fieldOperator);
        var fieldAccessProfiles = getAccessProfilesFieldFromStructureWidget();
        UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(fieldAccessProfiles);
	};

	this.removePaginationFromSelectWidget = function(widget){
		widget.searchWithPagination = widget.hasPagination = false;
	};

	var reloadHierarchyFields = function(){
		getFieldTypeParentNameFromHierarchyTypesWidget().reload();
		getFieldTypeChildNameFromHierarchyTypesWidget().reload();
		getFieldParentNameFromHierarchyTypesWidget().reload();
	};

	//Structure Type and Hierarchy Type Widgets
	self.newType = function(widget) {
		changeWidgetVisibility(widget.widgets, false);
		reloadHierarchyFields();
		widget.newRow();
	};

	self.editType = function(widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		changeWidgetVisibility(widget.widgets, false);
		reloadHierarchyFields();
		widget.edit();
	};

	self.cancelType = function(widget) {
		if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
			widget.list();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
		changeWidgetVisibility(widget.widgets, true);
	};

	self.backType = function(widget) {
		ScreenService.closeSwipe();
		widget.list();
	};

	self.saveHierarchType = function(widget) {
		if(widget.currentRow.typeChildName !== widget.currentRow.typeParentName){
			if (widget.isValid()) {
				var bkpFieldValues = [];
				fieldsToUnsetValue.forEach(
					function(element) {
						bkpFieldValues.push({
							"name": element,
							"value": widget.currentRow[element]
						});
						delete widget.currentRow[element];
					}
				);
				if (!widget.currentRow.parent) {
					widget.currentRow.parent = null;
				}
				widget.saveRow().then(
					function(response) {
						var dataSourceName = widget.dataSource.name.substr(1);
						widget.currentRow._id = response.dataset[dataSourceName][0]._id;
						widget.currentRow.changed = false;
						widget.currentRow.__is_new = false;
						widget.currentRow.structureTypeChild =
							UtilitiesController.findByMongoId(widget.getField('typeChildName').dataSource.data, widget.currentRow.typeChild);
						widget.currentRow.structureTypeParent =
							UtilitiesController.findByMongoId(widget.getField('typeParentName').dataSource.data, widget.currentRow.typeParent);
						bkpFieldValues.forEach(
							function(element) {
								widget.currentRow[element.name] = element.value;
							}
						);
						self.setChildFilter(widget);
						self.loadFilterForHierarchySelects(widget);
						widget.view();
						changeWidgetVisibility(widget.widgets, true);
					},
					function(response) {
						bkpFieldValues.forEach(
							function(element) {
								widget.currentRow[element.name] = element.value;
							}
						);
					}
				);
			}
		} else {
			UtilitiesController.showMessage("PARENT_AND_CHILD_DIFFERENT", "error");
		}
	};

	self.saveStructureType = function(widget) {
		if (widget.isValid()) {
			var connectionNameBkp = widget.currentRow.connectionName;
			widget.currentRow.connectionName = undefined;
			widget.saveRow().then(
				function(response) {
					var dataSourceName = widget.dataSource.name.substr(1);
					widget.currentRow._id = response.dataset[dataSourceName][0]._id;
					widget.currentRow.changed = false;
					widget.currentRow.__is_new = false;
					widget.currentRow.connectionName = connectionNameBkp;
					self.setChildFilter(widget);
					widget.view();
					changeWidgetVisibility(widget.widgets, true);
				},
				function(response) {
					widget.currentRow.connectionName = connectionNameBkp;
				}
			);

		}
	};

	self.deleteType = function(widget) {
		widget.removeCheckedRows();
	};

    this.structureWidgetOnEnter = function(){
        self.loadData(getStructureWidget());
    };

	self.setChildFilter = function(widget) {
		UtilitiesController.clearChangedRows(widget);
		widget.widgets[0].dataSourceFilter[0].value = widget.currentRow._id;
		needLoad[widget.widgets[0].name] = true;
	};

	self.loadFilterForHierarchySelects = function(widget) {
		UtilitiesController.clearChangedRows(widget);
		if (!widget.currentRow.__is_new) {
			widget.widgets[0].getField('structureParentName').dataSourceFilter[0].value = widget.currentRow.typeParent;
			widget.widgets[0].getField('structureParentName').reload();
			widget.widgets[0].getField('structureChildName').dataSourceFilter[0].value = widget.currentRow.typeChild;
			widget.widgets[0].getField('structureChildName').reload();
			widget.widgets[0].getField('structureChilds').dataSourceFilter[0].value = widget.currentRow.typeChild;
			widget.widgets[0].getField('structureChilds').reload();
		}
	};

	self.loadFieldKeysFromQuery = function(widget) {
		var data = [];
		if (widget.currentRow.query) {
			StructureService.getFieldKeysFromQuery(widget.currentRow).then(
				function(response) {
					UtilitiesController.showMessages(response.messages);
					if (response.dataset && response.dataset.columns) {
						var columns = response.dataset.columns;
						widget.getField('fieldKey').dataSource.data = Util.clone(columns);
						widget.getField('descriptionField').dataSource.data = Util.clone(columns);
					}
				},
				function(response) {
					MessageService.errorNotificationByCode('ERROR_ON_LOAD_FIELDS_FROM_QUERY');
					widget.getField('fieldKey').dataSource.data = data;
					widget.getField('descriptionField').dataSource.data = data;
				}
			);
		} else if (!widget.currentRow.__is_new) {
			MessageService.alertNotificationByCode('REQUIRED_FIELDS');
			widget.getField('fieldKey').dataSource.data = data;
			widget.getField('descriptionField').dataSource.data = data;
		}
	};

	self.fillFieldKey = function(widget) {
		UtilitiesController.clearChangedRows(widget);
		widget.getField('fieldKey').dataSource.data = [];
		var fieldsKey = widget.currentRow.fieldKey;
		if (fieldsKey) {
			fieldsKey.forEach(
				function(element) {
					widget.getField('fieldKey').dataSource.data.push({
						"name": element
					});
				}
			);
		}
	};

	//Structure and Hierarchy Widgets
	self.newRow = function(widget) {
		widget.newRow();
		widget.currentRow.typeName = widget.parent.currentRow.name;
		widget.currentRow.type = widget.parent.currentRow._id;
		changeWidgetVisibility(widget.widgets, false);
	};

	self.editRow = function(widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		changeWidgetVisibility(widget.widgets, false);
		widget.edit();
	};

	self.editStructure = function(widget) {
		widget.backupRow = Util.clone(widget.currentRow);
		changeWidgetVisibility(widget.widgets, false);
		widget.edit();
		if (widget.currentRow.generated) {
			widget.getField("name").readOnly = true;
			widget.getField("externalIdString").readOnly = true;
		}
	};

	self.backView = function(widget) {
		ScreenService.closeSwipe();
		widget.list();
		widget.activate();
        UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getAccessProfilesFieldFromStructureWidget());
        UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getUserResponsibleFieldFromStructureWidget());
	};

	self.deleteRows = function(widget) {
		widget.removeCheckedRows();
	};

	self.loadData = function(widget) {
		if (needLoad[widget.name]) {
			widget.reload().then(
				function() {
					widget.dataSource.data.forEach(
						function(element) {
							element.typeName = widget.parent.currentRow.name;
						}
					);
				}
			);
			needLoad[widget.name] = false;
		}
	};

	//Structure Widget

	self.cancelStructure = function(widget) {
		if (widget.currentRow.__is_new) {
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
			widget.list();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
		widget.getField("name").readOnly = false;
		widget.getField("externalIdString").readOnly = false;
        UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getAccessProfilesFieldFromStructureWidget());
		UtilitiesController.bugPaginationOnSelectMultiplyOnModeList(getUserResponsibleFieldFromStructureWidget());
		widget.reload();
		changeWidgetVisibility(widget.widgets, true);
	};

	self.saveStructure = function(widget) {
		if (widget.isValid()) {
			if(widget.isNewRow()){
				widget.currentRow.externalId = widget.currentRow.externalIdString.replace(/\s/g, "");
			}
			widget.currentRow.generated = true;
            UtilitiesController.bugPaginationOnSelectMultiplyOnModeView(widget.getField("usersResponsible"));
			widget.saveRow().then(function() {
				changeWidgetVisibility(widget.widgets, true);
				widget.currentRow.changed = false;
				ScreenService.closeSwipe();
				widget.reload();
			});
		}
	};

	self.loadStructuresFromQuery = function(widget) {
		StructureService.getStructuresFromQuery(widget.parent.currentRow, widget.dataSource.data).then(
			function(response) {
				UtilitiesController.showMessages(response.messages);
				if (response.dataset && response.dataset.structure) {
					widget.widgets[0].dataSource.data = response.dataset.structure;
					widget.widgets[0].isVisible = true;
					ScreenService.openPopup(widget.widgets[0]);
				}
			}
		);
	};

	self.loadSeveralStructuresFromQuery = function(args){
		var widget = getWidgetStructureTypes();
		var structMessage = "";
		StructureService.getManyStructuresFromQuery(widget.dataSource.data).then(function(response){
			var structure = response.dataset.data;
			if(structure.length == 0){
				MessageService.showMessageByCode('STRUCTURE_ADD_NONE');
			}else{
				structure.forEach(function(str){
					structMessage += str.typeName+" - "+str.name+"<br>";
				});
				var messageParameters = {
					"n": structure.length,
					"list": structMessage
				};
				var messageConfig = {
					"type": 'CONFIRMATION',
					"success": function (){
						StructureService.saveTooManyStructures(structure).then(function(){
							MessageService.successNotificationByCode("STRUCTURE_ADD_SUCCESS");
						});
					}
				};
				UtilitiesController.confirmMessageByCode("STRUCTURES_FOUND", messageParameters, messageConfig);
			}
		}).catch(function(error){
			UtilitiesController.showMessage(error.data.error, "error");
		});
	};

	self.saveAddStructure = function(widget) {
		var checkedRows = widget.getCheckedRows();
		checkedRows = checkedRows.map(
			function(element) {
				element.generated = true;
				element.changed = true;
				element.__is_new = true;
				return element;
			}
		);
		StructureService.saveTooManyStructures(checkedRows).then(
			function(response) {
				widget.dataSource.data = [];
				ScreenService.closePopup(true);
				needLoad[widget.parent.name] = true;
				self.loadData(widget.parent);
				var message = response.notifications[0].message;
				var type = response.notifications[0].type;
				UtilitiesController.notificationMessage(message, type);
				widget.parent.list();
				widget.parent.activate();
			}
		).catch(function(response){
			if(response.error){
				MessageService.showMessageByCode(response.error, null, {type: 'error'});
				ScreenService.closePopup(true);
				ScreenService.closeSwipe(0);
				widget.parent.parent.reload();
			} else {
				UtilitiesController.showMessage(error.data.error, "error");
			}
		});
	};

	//Hierarchy Widget
	self.addTooMany = function(widget) {
		setFieldVisibility(fieldsNormalEdit, widget, false);
		setFieldVisibility(fieldsAddTooMany, widget, true);
		widget.newRow();
		widget.currentRow.typeName = widget.parent.currentRow.name;
		widget.currentRow.type = widget.parent.currentRow._id;
	};

	self.cancelHierarchy = function(widget) {
		if (widget.currentRow.__is_new) {
			setFieldVisibility(fieldsAddTooMany, widget, false);
			setFieldVisibility(fieldsNormalEdit, widget, true);
			widget.dataSource.data.pop();
			ScreenService.closeSwipe();
			widget.list();
			widget.activate();
		} else {
			Util.extend(widget.currentRow, widget.backupRow);
			widget.view();
		}
	};

	self.saveHierarchy = function(widget) {
		if (!widget.currentRow.structureChilds) {
			saveOne(widget);
		} else {
			saveTooMany(widget);
		}
	};
    

	self.relateFieldKeys = function(widget) {
		var relationWidget = widget.container.getWidget("relations");
		var parentWidget = widget.parent;
		relationWidget.relationsDataSource.data = [];
		relationWidget.parentsDataSource.data =
			UtilitiesController.transformArrayInData(parentWidget.currentRow.structureTypeChild.fieldKey);
		relationWidget.childsDataSource.data =
			UtilitiesController.transformArrayInData(parentWidget.currentRow.structureTypeParent.fieldKey);
		ScreenService.openSwipe(relationWidget.parent).then(function() {
			relationWidget.activate();
		});

	};

	var swapRelations = function(relations){
		relations.forEach(function(relation){
			var oldParentKey = relation.parentKey;
			relation.parentKey = relation.childKey;
			relation.childKey = oldParentKey;
		});
		return relations;
	};

	var checkInvalidRelations = function(relations){
		var uniqueChilds = [];
		for(var i=0;i<relations.length;i++){
			if(uniqueChilds.includes(relations[i].childKey)){
				return true;
			} else {
				uniqueChilds.push(relations[i].childKey);
			}
		}
		return false;
	};

	self.generateHierarchies = function(widget) {
		var relations = swapRelations(widget.relationsDataSource.data);
		if(checkInvalidRelations(relations)){
			ScreenService.showMessageByCode("CANNOT_MATCH_STRUCTURES", "W");
 			ScreenService.closeSwipe();
			widget.parent.parent.activate();
            widget.parent.parent.reload();
			return;
		}
        widget.parent.parent.dataSource.selectAll();
        widget.parent.parent.removeCheckedRows().then(function(){
            var currentRow = widget.container.getWidget('hierarchy_types').currentRow;
            var copyRow = {
                "_id": currentRow._id,
                "name": currentRow.name,
                "parent": currentRow.parent,
                "structureTypeChild": currentRow.structureTypeChild,
                "structureTypeParent": currentRow.structureTypeParent,
                "typeChild": currentRow.typeChild,
                "typeChildName": currentRow.typeChildName,
                "typeParent": currentRow.typeParent,
                "typeParentName": currentRow.typeParentName
			};
            StructureService.generateHierarchies(copyRow, relations)
                .then(function(response) {
                    if (response.messages) {
                        UtilitiesController.showMessage(response.messages[0].message, response.messages[0].type);
                    } else if (response.notifications) {
                        ScreenService.closeSwipe();
                        widget.parent.parent.activate();
                        widget.parent.parent.reload();
                        UtilitiesController.notificationMessage(response.notifications[0].message, response.notifications[0].type);
                    }
                    widget.reload();
                });
        });
	};

	var saveOne = function(widget) {
		if (widget.isValid()) {
			widget.currentRow.structureChilds = undefined;

			var typeNameBkp = widget.currentRow.typeName;
			widget.currentRow.typeName = undefined;
			var structureChildNameBkp = widget.currentRow.structureChildName;
			widget.currentRow.structureChildName = undefined;
			var structureParentNameBkp = widget.currentRow.structureParentName;
			widget.currentRow.structureParentName = undefined;

			widget.saveRow().then(
				function(response) {
					var dataSourceName = widget.dataSource.name.substr(1);
					widget.currentRow._id = response.dataset[dataSourceName][0]._id;
					widget.currentRow.changed = false;
					widget.currentRow.__is_new = false;
					widget.currentRow.typeName = typeNameBkp;
					widget.currentRow.structureChildName = structureChildNameBkp;
					widget.currentRow.structureParentName = structureParentNameBkp;
					widget.view();
				},
				function(response) {
					widget.currentRow.typeName = typeNameBkp;
					widget.currentRow.structureChildName = structureChildNameBkp;
					widget.currentRow.structureParentName = structureParentNameBkp;
				}
			);
		}
	};

	var saveTooMany = function(widget) {
		if (widget.isValid()) {
			var rowBkp = Util.clone(widget.currentRow);
			var structureChildsBkp = widget.currentRow.structureChilds;
			var indexedStructuresById = getStructuresIndexedById(widget.getField('structureChilds').dataSource.data);
			var parentName = widget.currentRow.structureParentName;
			var structureParent = widget.currentRow.structureParent;
			var type = widget.currentRow.type;
			widget.currentRow.changed = false;
			structureChildsBkp.forEach(
				function(element) {
					widget.newRow();
					widget.currentRow.name = parentName + " - " + getStrucutreName(indexedStructuresById, element);
					widget.currentRow.structureParent = structureParent;
					widget.currentRow.structureChild = element;
					widget.currentRow.type = type;
					widget.currentRow.structureParentName = undefined;
					widget.currentRow.structureChildName = undefined;
					widget.currentRow.structureChilds = undefined;
					widget.currentRow.typeName = undefined;
					widget.currentRow.changed = true;
				}
			);
			widget.saveRow().then(
				function() {
					ScreenService.closeSwipe();
					setFieldVisibility(fieldsAddTooMany, widget, false);
					setFieldVisibility(fieldsNormalEdit, widget, true);
					widget.list();
					self.loadData(widget);
					widget.activate();
				},

				function() {
					widget.reload().then(
						Util.extend(widget.currentRow, rowBkp)
					);
				}
			);
		}
	};

	var getStrucutreName = function(indexedStructuresById, id) {
		return indexedStructuresById[id].name;
	};

	var getStructuresIndexedById = function(strucutresData) {
		var indexedStructuresById = {};
		strucutresData.forEach(
			function(element) {
				indexedStructuresById[element._id] = element;
			}
		);
		return indexedStructuresById;
	};

    this.swipeCloserAfterMoveRow = function(){
        ScreenService.closeSwipe(1);
    };

}

Configuration(function(ContextRegister) {
	ContextRegister.register('StructureController', StructureController);
});

// FILE: assets/js/controllers/StyleController.js
function StyleController(ScreenService, UtilitiesController, StyleService, MessageService, templateManager) {

    var self = this;

    var getWidgetStyle = function() {
        return templateManager.container.getWidget('style');
    };

    this.styleContainerAfterinit = function() {
        getWidgetStyle().sourceUrl = UtilitiesController.getUrlZmartNext("estiloCSS");
        Util.doAsync(ScreenService.hideLoader, 2000);
    };

    this.backOrCancel = function(widget) {
        if (widget.isNewRow()) {
            widget.dataSource.data.pop();
            widget.list();
            widget.currentRow = {};
            ScreenService.goBack();
        }
        else {
            widget.view();
            for (var i in widget.backupRow) {
                widget.currentRow[i] = widget.backupRow[i];
            }
        }
    };
    

    this.deleteCustomStyle = function(widget){
        var checkedRows = widget.getCheckedRows();
        checkedRows.forEach(function(row){
            StyleService.removeAppliedCustomStyles(row.formatedName);
        });
        widget.removeCheckedRows();
    };

    this.saveStyle = function(widget) {
		if (widget.isValid()) {
			if (!widget.isNewRow()) {
			    var originalName = widget.backupRow.formatedName;
			    StyleService.removeAppliedCustomStyles(originalName);
			}

			var styleName = widget.currentRow.name.replace( /\s/g, '_' );
			widget.currentRow.formatedName = styleName;
			widget.saveRow().then(function () {
			    setStyleUpdatesToRelatedGrids(widget.currentRow);
				ScreenService.closeSwipe();
				widget.reload();
				widget.list();
			}, function (response) {
				UtilitiesController.showMessage(response.data.error);
			});
		} else {
		    MessageService.alertNotificationByCode('REQUIRED_FIELDS');
		}
    };

    this.inferCssClasses = function(args) {
        var cssCode = args.row.cssCode;
        var classes = cssCode.match(/(\w+)?(\s*>\s*)?(#\w+)?\s*(\.\w+)?\s*{/gm);

        classes = classes.map(function (classe) {
            return classe.trim().replace('.', '').replace('{', '');
        });

        args.row.cssClass = classes.join("");
    };

    function setStyleUpdatesToRelatedGrids(style){
        var relatedGrids;

        StyleService.getGridsRelatedToAlteredStyle(style._id).then(function(response){
            relatedGrids = response.dataset.gridStyle;

            relatedGrids.forEach(function(grid){
                grid.cssClass = style.cssClass;
                grid.styleName = style.name;
            });

            StyleService.persistStyleUpdatesToGrids(relatedGrids);
        });
    }

    this.validateSelectRow = function(selectedRow){
	    if (selectedRow.blocked) {
			unselectRowsAndErrorMessage(selectedRow);
		}
	};

	this.validateSelectAllRows = function(selectedPeriods){
	    selectedPeriods.data.forEach(function(row) {
			if (row.blocked) {
				row.__isSelected = false;
			}
		});

		MessageService.alertNotificationByCode('BLOCKED_STYLES_CANT_BE_SELECTED');
	};

    function unselectRowsAndErrorMessage(selectedRow) {
		MessageService.alertNotificationByCode('BLOCKED_STYLES_CANT_BE_SELECTED');
		selectedRow.__isSelected = false;
	}

	this.setActionsVisibility = function(widget){
	    UtilitiesController.clearChangedRows(widget);
	    widget.getAction('edit').isVisible = !widget.currentRow.blocked;
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('StyleController', StyleController);
});


// FILE: assets/js/controllers/WebSocketClient.js
var WebSocketClient = function(ZhSocketFactory, AutoUpdateService, ChartViewService,
GenericDashboardService, templateManager, DataSourceManagerController, restEngine,QueryRunnerService) {

    var pendingMessages = [];

    var NAMESPACE = KpiConfig.wsNamespace;
    var webSocket = null;

    var socketClient = function() {
        this.finishImportation = function(eventData) {
            var dataSourceId = eventData.dataSource;

            if(eventData.updateIndicators) {
                AutoUpdateService.setDataSourceLastUpdateTime(dataSourceId);
                AutoUpdateService.updateIndicators();
                AutoUpdateService.updateQueryRunner(dataSourceId);
            }
        };

        this.updateImportProcess = function(eventData) {
            var index = -1;
            var widget = templateManager.container.getActiveWidget();
            var importHistory = eventData.process;
            DataSourceManagerController.prepareImportHistoryRow(importHistory);
            if(widget.name === 'importHistory' && widget.parent.currentRow._id === eventData.process.dataSourceId){
                index = widget.dataSource.data.findIndex(function(data){
                    return data.process === importHistory.process;
                });

                if(index !== -1){
                    widget.dataSource.data[index] = importHistory;
                } else {
                    widget.dataSource.data.push(importHistory);
                }

                widget.reload();
            } else if (widget.name === 'dataSource'){
                index = widget.dataSource.data.findIndex(function(data){
                    return data._id === importHistory.dataSourceId;
                });

                if(widget.dataSource.data[index]){
					widget.dataSource.data[index].importStatus = importHistory.status ? importHistory.status : 0;
					widget.dataSource.data[index].subprocesses = importHistory.subprocesses ? importHistory.subprocesses : 0;
					widget.dataSource.data[index].totalImported = importHistory.totalImported ? importHistory.totalImported : 0;
				}
            } else if (widget.container.name === 'queryRunner'){
                var widgetQueryRunner = widget.container.getWidget('queryRunner');
                widgetQueryRunner.currentRow.importProcess = importHistory;
                var fieldStatus = widgetQueryRunner.getField('status');
                if(fieldStatus.widget.currentRow.process === importHistory.process){
                    QueryRunnerService.setStatusColumnValue(fieldStatus, importHistory);
                }
            }else if(widget.container.name === "chartView"){
                if(widget.container.widgets && widget.container.widgets[1]){
                    var status = ChartViewService.getImportStatusText(importHistory);
                    widget.container.widgets[1].noMatchText = status;
                    widget.container.widgets[1].chartMessageNoData  = status;
                    if(widget.container.widgets[1].updateChart){
                        widget.container.widgets[1].updateChart();
                    }
                }
            }else if(widget.container.name === "mainDashboard" || widget.container.name === "genericDashboard"){
                var view = widget.widgets.find(function(widget){
                    return widget.dataSourceFilter[0].value === importHistory.process;
                });
                var statusText = ChartViewService.getImportStatusText(importHistory);
                view.noMatchText = statusText;
                view.chartMessageNoData  = statusText;
                if(view.updateChart){
                    view.updateChart();
                }
            }
        };

        this.getCreatorDatasource = function(eventData){
            var id = eventData.idCreatorDataSource;
            var dataSourceName = eventData.dataSourceName;
            var message = eventData.message;
            getDataFromDatasourceCreator(id).then(function(data){
                var creatorInformations = data.dataset.user[0];
                DataSourceManagerController.buildNotificationError(creatorInformations, dataSourceName, message);
            });
        };
    };

	var getDataFromDatasourceCreator = function(ID){
       return restEngine.post("IDENTITY#/zi_user",
            {
                "requestType" : "FilterData",
                "filter": [
					{
						"name": "ID",
						"operator": "=",
						"value": ID
					}
                ]
            }
        );
    };

    var createMessage = function(event, namespace, room, data) {
        data.key = KpiConfig.wsKey;
        return {
            'namespace': namespace,
            'room': room,
            'event': event,
            'data': data
        };
    };

    var sendMessage = function(event, namespace, room, data) {
        data = data || {};
        var message = JSON.stringify(createMessage(event, namespace, room, data));
        if (webSocket.isOpen()) {
            webSocket.send(message);
        } else {
            pendingMessages.push(message);
        }
    };

    this.connect = function(organizationId) {
        webSocket = ZhSocketFactory.create(socketClient, KpiConfig.wsServer);
        webSocket.addListener({
            'onOpen': function() {
                sendMessage('createNamespace', NAMESPACE);
                sendMessage('createRoom', NAMESPACE, organizationId);

                sendMessage('on', NAMESPACE, organizationId, {
                    'event': 'finishImportation'
                });

                sendMessage('on', NAMESPACE, organizationId, {
                    'event': 'updateImportProcess'
                });

                sendMessage('on', NAMESPACE, organizationId, {
                    'event': 'getCreatorDatasource'
                });

                if(pendingMessages.length > 0) {
                    pendingMessages.forEach(webSocket.send);
                    pendingMessages = [];
                }
            }
        });

    };

    this.onError = function(listener) {
        webSocket.addListener({
            'onError': listener
        });
    };

    this.removeListener = function(listener) {
        webSocket.removeListener(listener);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('WebSocketClient', WebSocketClient);
});


// FILE: assets/js/controllers/dashboardConfiguration.js
function dashboardConfigurationController(ScreenService, templateManager, ApplicationContext,
                                     UtilitiesController, ViewService, MessageService) {

    var getDashboardConfigurationWidget = function(){
        return templateManager.container.getWidget('dashboardConfiguration');
    };

    var getActionimportFromDashboardConfiguration = function(){
        return getDashboardConfigurationWidget().getAction('import');
    };

    this.containerAfterinitDashboardConfiguration = function() {
        getActionimportFromDashboardConfiguration().isVisible = getLocalVar('isTeknisa');
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('dashboardConfigurationController', dashboardConfigurationController);
});

// FILE: assets/js/controllers/logImportation.js
function logImportation(UtilitiesController, ScreenService, templateManager, zhFilterPreferences, restEngine, MessageService,
                        DataSourceManagerController, MasterDetailUtilities) {

    var self = this;

    var getFieldQueryLogImportation = function(){
        return getWidgetLogImportation().getField("QUERY");
    };
    var getWidgetLogImportation = function () {
        return templateManager.container.getWidget('logImportation');
    };

    var getFieldQueryFailImportation = function(){
        return getWidgetFailImportation().getField("query");
    };
    var getWidgetFailImportation = function () {
        return templateManager.container.getWidget('failImportation');
    };
    var getWidgetWalletClient = function () {
        return templateManager.container.getWidget('walletClient');
    };
    var getWidgetErrorNotification = function () {
        return templateManager.container.getWidget('ERROR_NOTIFICATION');
    };
    var getWidgetAllClientMongo = function () {
        return templateManager.container.getWidget('ALL_CLIENT_MONGO');
    };

    this.allClientMongoWidgetActionBack = function() {
        var widget = getWidgetAllClientMongo();
        UtilitiesController.masterDetailBackToList(widget);
    };

    this.allClientMongoWidgetActionEdit = function() {
        var widget = getWidgetAllClientMongo();
        UtilitiesController.masterDetailEdit(widget);
    };

    this.allClientMongoWidgetActionCancel = function() {
        var widget = getWidgetAllClientMongo();
        UtilitiesController.masterDetailRollback(widget);
    };

    this.allClientMongoWidgetAfterMoveRow = function(args) {
        setLocalVar('allClientMongoNrorg',args.row.NRORG);
    };

    this.allClientMongoWidgetActionSave = function() {
        var widget = getWidgetAllClientMongo();
        if (widget.isValid()) {
            widget.currentRow.changed = true;
            widget.saveRow().then(function(response) {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;
                widget.view();
            });
        }
    };

    this.allClientMongoEventActionRebuildCollectors = function() {
        var messageConfig = {
            "type": 'CONFIRMATION',
            "success": function () {
                restEngine.post({
                        service: "product#/rebuildCollectors",
                        typeRequest: "POST",
                        params: {
                            "requestType": "FilterData",
                            "filter": []
                        },
                        timeout: 36000000
                    }
                ).then(function () {
                    getWidgetAllClientMongo().reload();
                });
            }
        };
        UtilitiesController.confirmMessageByCode("REBUILD_COLLECTORS", null, messageConfig);
    };

    this.logImportationContainerAfterinit = function () {
        loginPromise.then(function () {
            var widget = getWidgetLogImportation();
            zhFilterPreferences.startFiltering(widget);
        });
    };

    //Widget interactions
    this.newRow = function(widget){
        widget.newRow();
    };

    this.deleteRows = function(widget){
        widget.removeCheckedRows();
    };

    this.cancelEdit = function (widget) {
        UtilitiesController.cancelEditRow(widget);
    };

    this.goBack = function(widget){
        UtilitiesController.goBackWidget(widget);
    };
    this.edit = function(widget){
        MasterDetailUtilities.masterDetailEdit(widget);
    };

   this.saveWalletClient = function (widget) {
        return widget.saveRow().then(function () {
            UtilitiesController.goBackWidget(widget);
            widget.reload();
        });
    };

    self.logImportationFieldOnClickButtonCopyQuery= function(){
        var text = getFieldQueryLogImportation().value();
        UtilitiesController.copiedToClipboard(text,true);
    };

    self.failImportationFieldOnClickButtonCopyQuery= function(){
        var text = getFieldQueryFailImportation().value();
        UtilitiesController.copiedToClipboard(text,true);
    };

    this.logImportationActionEventReload = function () {
        var messageConfig = {
            "type": 'CONFIRMATION',
            "success": function () {
                restEngine.post({
                        service: "product#/importLogImportation",
                        typeRequest: "POST",
                        params: {
                            "requestType": "FilterData",
                            "filter": []
                        },
                        timeout: 36000000
                    }
                ).then(function () {
                    getWidgetLogImportation().reload();
                });
            }
        };
        UtilitiesController.confirmMessageByCode("GENERATE_LOG_IMPORTATION", null, messageConfig);
    };

    self.setStatusColumnValue = function(row, element) {

        element.children('div.circle-status').remove();

        var status = row.importStatus ;

        if (status === 'failed') {
            element.children('span')[0].style.display = 'none';
            element.append("<div class='circle-status error'></div>");
        }else if (status === 'done') {
            element.children('span')[0].style.display = 'none';
            element.append("<div class='circle-status success'></div>");
        } else{
            element.children('span')[0].style.display = '';
        }
    };
    this.logImportationWidgetBeforeMoveRow = function (args) {
        Util.doAsync(function () {
            var importFailImportation = getWidgetFailImportation();
            if(importFailImportation.currentRow.logs && importFailImportation.currentRow.logs[0] && importFailImportation.currentRow.logs[0].logs){
                var logs =  importFailImportation.currentRow.logs[0].logs;
                importFailImportation.currentRow.logField = logs.length > 0? DataSourceManagerController.formatLogMessages(logs) : '';
                templateManager.updateTemplate();
            }
        },500);

    };

    this.failImportationActionEventReload = function () {
        var messageConfig = {
            "type": 'CONFIRMATION',
            "success": function () {
                restEngine.post({
                        service: "product#/notificationImportationFail",
                        typeRequest: "POST",
                        params: {
                            "requestType": "FilterData",
                            "filter": []
                        },
                        timeout: 36000000
                    }
                ).then(function (response) {
                    getWidgetFailImportation().reload();
                    if(response &&
                        response.messages &&
                        response.messages[0] &&
                        response.messages[0].message){
                            var message = response.messages[0].message;
                            UtilitiesController.showMessage(message);
                    }
                }).catch(UtilitiesController.showErrorMessage);
            }
        };
        UtilitiesController.confirmMessageByCode("GENERATE_LOG_IMPORTATION", null, messageConfig);
    };
    this.logImportationWidgetOnEnter = function () {
        var widget = getWidgetFailImportation();
        if(!widget.first){
            zhFilterPreferences.startFiltering(widget);
            widget.first = true;
        }
    };
    this.walletClientWidgetOnEnter = function () {
        var widget = getWidgetWalletClient();
        if(!widget.first){
            zhFilterPreferences.startFiltering(widget);
            widget.first = true;
        }
    };

    this.allClientMongoWidgetOnEnter = function () {
        var widget = getWidgetAllClientMongo();
        if(!widget.first){
            zhFilterPreferences.startFiltering(widget);
            widget.first = true;
        }
    };

    this.failImportationActionEventErrorNotification = function () {
        var widget = getWidgetErrorNotification();
        ScreenService.openPopup(widget).then(function () {
            widget.reload().then(function () {
                widget.moveToFirst();
                widget.edit();
            });
        });
    };

    this.errorNotificationActionEventBack = function () {
        ScreenService.closePopup();
        getWidgetFailImportation().activate();
    };

    this.errorNotificationActionEventSave = function () {
        var widget = getWidgetErrorNotification();
        widget.saveRow().then(function () {
            self.errorNotificationActionEventBack();
        }).catch(function (response) {
            UtilitiesController.showErrorMessageTrue(response);
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('logImportationController', logImportation);
});


// FILE: assets/js/config/events.js
function ZmartEvents(eventEngine) {

	eventEngine.registerEvent("ContainerOnRefreshIndicator", "onRefreshIndicator");

	eventEngine.registerEvent("ContainerOnRefreshDashboard", "onRefreshDashboard");
	

}

Configuration(function(ContextRegister) {
	ContextRegister.register("ZmartEvents", ZmartEvents);
});

// FILE: assets/js/config/routeSubscriber.js
function ZmartRouteSubscriber(eventAggregator, ViewService, ChartViewController, GenericDashboardController, localStorageEngine) {

    _buildIndicatorView = Util.buildDebounceMethod(function() {
    	ChartViewController.returnBuild();
    }, 1000);

    _buildIndicatorViewPreview = Util.buildDebounceMethod(function() {
    	ChartViewController.returnBuildPreview();
    }, 2000);
    
	eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function (openEvent) {
	    if(localStorageEngine.getLocalVar('isLogged')){
            if (openEvent.container) {
                var isDashboard = openEvent.container.name === ViewService.DASHBORAD_WINDOW;
                var isChartView = openEvent.container.name === ViewService.INDICATOR_VIEW_WINDOW;
                var isPreview = openEvent.container.preview;
                if(isDashboard){
                    GenericDashboardController.activateFirstOrLast();   
                } else if(isPreview){
                    _buildIndicatorViewPreview();
                } else if(isChartView){
                    _buildIndicatorView();
                }
            }
	    }
    });
}

Configuration(function(ContextRegister) {
	ContextRegister.register("ZmartRouteSubscriber", ZmartRouteSubscriber);
});